#include "matrix.h"

void print_mat(const std::vector<int>& M, int rows, int cols);

std::vector<int> getA();
std::vector<int> getB();
std::vector<int> getC();

#define TEST(TEST_NAME) std::cout << "____ Test " << TEST_NAME << " ____\n";
#define CASE_OK(CASE_NAME) std::cout << "OK Case " << CASE_NAME << "\n";
#define CASE_ERR(CASE_NAME) std::cout << "ERR Case " << CASE_NAME << "\n";

int main()
{
    // ====================================================
    // test 1
    // ====================================================
    {
        TEST(1);
        const int N = 2;
        const int M = 2;
        const int P = 2;
        const std::vector<int> A = {
            1, 2,
            3, 4
        };
        const std::vector<int> B = {
            -5,  7,
             6, -8
        };
        const std::vector<int> C_sol = {
            7, -9,
            9, -11
        };
        // ------------------------------------------------
        const std::vector<int> C_test1 = matmul1(A, B, N, M, P);
        if(C_sol == C_test1) {CASE_OK("matmul1");}
        else
        {
            CASE_ERR("matmul1");
            std::cout << "Expected:\n";
            print_mat(C_sol, N, P);
            std::cout << "Got:\n";
            print_mat(C_test1, N, P);
        }
        // ------------------------------------------------
        const std::vector<int> C_test2 = matmul2(A, B, N, M, P);
        if(C_sol == C_test2) {CASE_OK("matmul2");}
        else
        {
            CASE_ERR("matmul2");
            std::cout << "Expected:\n";
            print_mat(C_sol, N, P);
            std::cout << "Got:\n";
            print_mat(C_test2, N, P);
        }
    }
    // ====================================================
    // test 2
    // ====================================================
    {
        TEST(2);
        const int N = T;
        const int M = T;
        const int P = T;
        const std::vector<int> A = { // identity
            1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
            0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
            0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
            0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
            0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
            0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
            0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
            0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0,
            0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0,
            0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0,
            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0,
            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0,
            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0,
            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0,
            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0,
            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
        };
        const std::vector<int> B = {
            0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5,
            6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1,
            2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7,
            8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3,
            4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
            0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5,
            6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1,
            2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7,
            8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3,
            4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
            0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5,
            6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1,
            2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7,
            8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3,
            4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
            0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5
        };
        const std::vector<int> C_sol = B;
        // ------------------------------------------------
        const std::vector<int> C_test1 = matmul1(A, B, N, M, P);
        if(C_sol == C_test1) {CASE_OK("matmul1");}
        else
        {
            CASE_ERR("matmul1");
            std::cout << "Expected:\n";
            print_mat(C_sol, N, P);
            std::cout << "Got:\n";
            print_mat(C_test1, N, P);
        }
        // ------------------------------------------------
        const std::vector<int> C_test2 = matmul2(A, B, N, M, P);
        if(C_sol == C_test2) {CASE_OK("matmul2");}
        else
        {
            CASE_ERR("matmul2");
            std::cout << "Expected:\n";
            print_mat(C_sol, N, P);
            std::cout << "Got:\n";
            print_mat(C_test2, N, P);
        }
        // ------------------------------------------------
        const std::vector<int> C_test3 = matmul3(A, B, N, M, P);
        if(C_sol == C_test3) {CASE_OK("matmul3");}
        else
        {
            CASE_ERR("matmul3");
            std::cout << "Expected:\n";
            print_mat(C_sol, N, P);
            std::cout << "Got:\n";
            print_mat(C_test3, N, P);
        }
    }
    // ====================================================
    // test 3
    // ====================================================
    {
        TEST(3);
        const int N = 24 * T;
        const int M =  9 * T;
        const int P =  8 * T;

        const std::vector<int> A = getA();
        const std::vector<int> B = getB();
        const std::vector<int> C_sol = getC();
        // ------------------------------------------------
        const std::vector<int> C_test1 = matmul1(A, B, N, M, P);
        if(C_sol == C_test1) {CASE_OK("matmul1");}
        else
        {
            CASE_ERR("matmul1");
            std::cout << "Expected:\n";
            print_mat(C_sol, N, P);
            std::cout << "Got:\n";
            print_mat(C_test1, N, P);
        }
        // ------------------------------------------------
        const std::vector<int> C_test2 = matmul2(A, B, N, M, P);
        if(C_sol == C_test2) {CASE_OK("matmul2");}
        else
        {
            CASE_ERR("matmul2");
            std::cout << "Expected:\n";
            print_mat(C_sol, N, P);
            std::cout << "Got:\n";
            print_mat(C_test2, N, P);
        }
        // ------------------------------------------------
        const std::vector<int> C_test3 = matmul3(A, B, N, M, P);
        if(C_sol == C_test3) {CASE_OK("matmul3");}
        else
        {
            CASE_ERR("matmul3");
            std::cout << "Expected:\n";
            print_mat(C_sol, N, P);
            std::cout << "Got:\n";
            print_mat(C_test3, N, P);
        }
    }

    return 0;
}

///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////

void print_mat(const std::vector<int>& M, int rows, int cols)
{
    if(M.size() == 0) {
        std::cout << "empty matrix" << std::endl;
    } else if(M.size() != rows*cols) {
        std::cout << "Wrong size: " << M.size() << " != " << rows << "*" << cols << std::endl;
    } else {
        for(int i = 0; i < rows; ++i) {
            std::cout << "  ";
            for(int j = 0; j < cols; ++j) {
                std::cout << M[i*cols+j] << " ";
            }
            std::cout << std::endl;
        }
    }
}

std::vector<int> getA() {
    return {
    83,-14,77,15,93,35,86,-8,-51,-79,62,-73,-10,-41,63,26,40,-74,72,36,-89,68,67,-71,82,30,-38,23,-33,35,29,-98,-78,-42,-31,67,93,-44,-89,-58,-71,73,-79,19,84,37,98,24,15,70,-87,26,-9,80,56,-27,-38,70,96,-19,5,25,-16,27,36,5,-54,29,13,-43,24,-5,82,45,-86,67,-66,64,-57,50,-13,-92,-24,78,88,84,-97,-49,54,99,32,-40,-24,68,39,-88,-74,86,-6,39,95,70,-66,78,-33,-99,-3,2,17,-8,-48,56,1,-20,-14,-59,-35,-11,-56,-81,-60,29,-69,17,-3,71,-19,-25,9,27,67,-44,-3,53,86,65,6,-17,-81,-76,28,-29,32,-71,
    3,-81,-30,68,8,15,40,49,96,23,-82,-55,-54,-49,21,55,79,-12,64,-72,-59,50,93,0,-66,64,24,14,87,-44,43,-9,-73,65,-41,36,-68,51,-63,-72,-25,-93,-26,21,-42,95,-71,-63,-65,93,-82,-72,43,-89,28,29,76,-96,-57,63,-87,38,-94,-60,4,-82,28,-12,69,17,17,96,24,43,-30,83,-10,-1,72,25,-56,90,5,39,54,86,-31,-18,42,-36,97,7,55,-96,48,-89,-78,-72,-1,43,46,68,40,-78,11,-90,-95,-99,-39,30,-22,5,20,36,-56,-74,22,-35,8,-84,-18,-42,24,-63,-38,-76,-100,-64,-48,-1,79,50,-32,-29,73,31,-19,30,33,-6,-40,63,99,81,
    -1,96,59,73,-87,-32,90,-5,26,-34,-16,40,-10,-16,76,42,36,7,-55,56,79,-82,-13,-88,48,72,-41,-91,36,-90,42,87,-94,1,13,72,21,-45,-81,99,21,4,39,-89,40,-33,5,-72,27,50,84,-42,20,-76,-78,-31,96,-19,-70,-16,-8,72,-28,-50,-75,85,-78,-1,-60,-58,-2,13,-2,90,24,90,-91,81,-81,36,32,55,94,-96,79,69,-27,76,-50,-45,-40,42,79,-16,93,-95,-79,67,4,-87,61,54,26,-41,44,-98,-98,6,84,-79,-58,-32,28,89,-28,8,58,-2,-64,-92,53,-52,3,33,33,-52,-10,54,67,46,68,29,0,-54,88,97,-51,90,3,-67,63,-3,-47,-8,
    -14,25,52,96,75,88,57,29,-64,-40,-86,21,-40,4,-72,-73,-50,48,56,2,94,-3,-1,-57,-61,-98,-72,-97,0,-19,-53,38,59,51,35,34,39,-8,-85,27,4,-71,-51,64,-15,-71,43,35,77,0,-62,71,49,-11,67,88,-8,95,43,44,-71,90,-18,40,41,69,-74,-68,-39,-58,60,17,-77,61,-19,9,90,-75,96,67,-23,-66,-10,-74,24,-43,-86,68,-95,58,12,86,0,46,26,94,16,52,78,29,46,-10,-53,70,-49,-20,-69,93,-43,-73,12,-14,-86,55,12,-10,-88,-21,-90,69,89,-26,55,-59,-80,-67,87,-12,38,66,70,-16,-44,-83,-94,-40,-51,-63,-95,-41,-83,18,45,83,
    -27,-42,-27,-63,-11,-17,-93,-22,57,14,-29,29,0,-41,-82,-62,-75,88,-26,-67,57,-19,-7,58,-30,-1,-83,-61,69,63,-78,94,73,-53,-69,-38,-18,90,-8,91,-43,15,21,57,74,-9,47,51,-69,-79,37,40,-46,-70,-2,25,-19,16,16,-98,-69,39,96,-96,38,80,-82,-79,70,-38,-88,79,77,-15,36,4,76,-17,-93,59,-43,44,-1,11,27,50,-64,60,-82,5,63,-51,-56,11,-95,34,-9,75,55,-86,89,68,93,18,-95,-18,-78,82,17,-70,-7,74,26,93,-14,-47,43,-26,-86,13,79,77,62,75,-12,19,10,32,-6,-83,46,-65,37,-9,53,43,73,28,25,-9,10,-82,-83,-64,
    63,-45,-10,-42,30,4,71,-39,-67,-15,89,-27,-96,-49,-95,-50,-32,3,-15,-94,95,-61,49,20,67,-74,63,-23,96,81,-35,60,-64,55,70,18,-89,42,32,96,79,21,-30,84,72,-73,-66,-60,-17,-28,98,-70,-37,47,50,-70,73,14,-41,22,-53,24,-18,-65,-68,-96,54,-57,-2,-14,-60,-22,59,-38,-38,-17,-59,-52,23,24,-28,22,54,35,-79,-43,65,47,71,76,-31,-82,1,3,-47,33,7,59,28,-94,97,20,-16,8,34,-2,91,76,-2,15,-48,71,89,59,6,-90,-84,-76,9,39,-100,78,9,-47,-19,14,38,89,-74,-33,47,-77,87,-69,32,22,-19,-25,-50,79,90,-46,50,31,
    -87,-43,-6,81,-19,-97,20,-67,82,81,-13,-85,-4,-75,-96,22,-8,51,-3,-68,34,81,6,-85,-43,8,95,99,62,-3,-17,76,54,77,9,87,32,82,-79,-34,63,-40,-18,-89,-15,-14,-15,30,90,83,14,-24,16,-80,-8,25,28,-61,25,-10,36,60,-82,-57,37,28,82,-79,10,55,-12,-75,-85,70,-63,-47,-92,22,83,50,-43,-3,-73,26,-31,-29,-49,49,10,28,-61,98,-12,-90,93,77,-10,76,99,52,31,-13,77,-1,-43,66,52,-83,-59,35,68,-2,84,95,76,-95,-34,-72,54,28,8,93,-22,97,55,-28,74,-55,-100,25,97,-17,-88,27,82,-79,-7,34,-61,34,-79,59,85,57,
    54,61,62,72,41,16,-48,-50,-38,-18,99,-83,54,73,-85,6,-49,64,90,-37,-9,72,-63,37,-41,-72,-29,-20,87,-44,90,41,70,52,-35,11,-31,17,61,-17,51,12,0,6,-62,-33,64,-11,-68,54,4,-25,-21,41,-88,38,69,-64,70,56,-56,60,-51,-86,-35,14,26,-14,83,-61,69,35,52,-79,93,90,-11,9,-69,-27,-36,35,48,-5,-23,13,-67,98,49,55,55,93,-32,56,60,33,-77,-14,71,58,-23,40,-55,81,61,90,23,-50,0,54,-25,64,-58,-76,-41,19,-11,-56,-31,-62,51,76,83,-81,33,43,4,56,81,-25,-34,-89,67,12,92,29,2,-32,31,-98,74,-93,18,-84,
    83,77,-13,-28,73,57,-38,25,33,97,-4,-82,41,53,26,-26,80,93,-15,48,5,-70,-71,59,-2,60,62,-76,19,-20,-59,2,-90,-20,26,83,89,40,8,23,38,57,-7,-69,10,-80,5,-10,-87,91,-62,-30,-79,-33,-71,-29,80,43,-5,-1,-76,-12,54,86,69,-68,69,10,73,-70,-67,-37,87,-21,-6,-51,-1,51,39,-36,42,30,-14,15,49,15,86,-19,-89,-66,33,87,22,-13,73,-57,19,42,54,-56,24,39,-41,-37,-82,53,12,-31,5,-96,33,99,34,-81,15,-65,-13,-47,-31,-50,-13,-98,-63,62,-11,-90,-95,60,4,11,57,-71,-97,-84,-8,-79,22,-95,-57,79,61,28,78,-53,
    0,-55,-18,87,-1,51,89,86,53,-74,48,94,-64,6,7,-8,-83,-36,21,-80,-20,66,94,54,23,-63,33,-16,17,12,31,-83,9,65,56,-92,-31,45,95,-78,71,-5,69,59,-99,-24,-48,71,40,25,43,72,-9,-11,27,66,-22,-88,-50,96,24,33,-87,86,-1,70,94,68,15,41,42,-61,-63,-37,98,90,39,-98,13,-69,28,-43,-96,71,46,-17,38,-75,-5,40,-79,72,-74,-66,58,25,56,52,-55,-28,-54,39,11,-17,-97,-39,-75,42,-84,39,74,44,96,30,-33,-6,-87,57,19,-40,-50,-8,-68,-24,-21,90,53,-65,-5,98,7,41,37,70,76,40,84,-99,-17,-100,-8,-91,-4,-60,
    39,63,35,4,-27,-94,64,23,-49,49,51,30,39,4,65,86,2,-75,-21,91,47,-93,-16,31,61,19,31,-47,28,-73,-6,-81,43,81,23,68,87,-61,-57,-62,88,94,20,-20,-2,-14,-82,52,-37,-2,-5,-90,5,79,42,-34,-2,-75,-28,78,-47,66,97,48,47,-28,16,-14,12,59,-23,0,53,97,32,-97,-65,51,7,-2,-51,54,-39,6,34,-97,25,84,-72,97,63,-67,15,60,81,-86,85,-3,0,-3,-92,29,49,13,-21,-66,16,14,85,75,65,86,30,-74,-8,-84,-71,69,52,-91,66,15,95,-67,-72,76,-53,-87,-74,0,-38,86,-71,63,0,8,97,-84,75,82,44,40,20,-74,
    18,65,94,-1,34,46,8,-47,-38,3,-14,-58,-68,34,-93,-90,-66,-31,96,-85,84,-4,24,82,-35,-49,16,61,43,-63,87,-39,-98,81,60,-12,-21,-80,41,-7,24,-72,35,8,62,42,-30,96,63,-34,-89,0,15,-13,-66,32,-10,50,-7,-67,39,-20,-6,-7,-87,-46,82,-56,-25,23,38,51,51,-27,-89,-35,68,-19,-87,-17,-1,77,35,-86,-36,69,46,7,-28,91,40,-89,-77,87,-43,-64,93,-61,-19,-80,14,71,71,-82,96,34,-17,64,67,-3,-100,-33,-26,35,-67,-10,-43,32,49,29,23,-10,-8,47,-71,49,35,-78,40,68,-57,-45,39,66,-75,36,-47,-40,-48,-80,57,-96,-13,83,
    40,21,-74,97,-95,27,-22,28,69,70,-73,98,20,63,-79,-40,-17,-84,-33,-77,82,-8,11,-65,53,63,-92,-38,68,-5,98,-40,68,76,-91,73,3,-13,-46,-27,-43,-19,-77,29,96,-56,42,-20,-88,-91,-45,47,-46,66,34,59,-19,42,73,-99,-62,71,-87,58,-1,-78,-16,55,-39,-10,80,-29,23,-97,-100,20,-100,42,52,-88,4,-93,59,58,25,-6,17,58,-64,-10,60,-74,-86,73,37,-35,48,-79,-80,-91,-37,-100,-68,86,-96,-67,-42,56,27,-90,68,-69,69,28,41,-54,74,-42,-95,-90,1,65,89,-33,-10,26,32,-62,-1,5,-100,62,-43,-68,48,-39,-83,-93,69,-3,69,38,28,39,
    -82,-30,85,92,-20,42,54,33,7,43,-100,50,-79,-15,88,20,90,40,-66,-53,25,83,-39,-6,42,-70,91,-37,20,20,54,38,42,-8,30,-78,-66,-15,8,-6,80,8,-56,-98,45,84,22,-13,25,57,35,-98,-8,48,-4,86,30,-60,-51,-49,12,56,89,-46,-52,-28,-72,-18,57,-12,76,37,97,-80,39,-6,-95,-86,82,-18,23,-31,-64,-85,-83,-16,53,-1,24,54,50,-64,10,-8,-58,58,64,-77,-59,21,11,69,-90,-40,90,2,55,47,16,-11,81,-61,58,-83,-94,75,1,11,-26,78,-35,77,66,76,-31,61,34,-67,36,27,6,99,-3,16,-40,-61,70,67,86,-14,-92,67,-23,66,
    36,-65,-7,-63,46,19,67,-88,-4,34,40,17,-5,-26,50,-69,-98,-92,-70,-49,25,-58,-10,-53,61,-24,34,69,95,63,35,31,51,80,20,-3,0,-12,-39,96,74,-99,14,-31,-72,-84,-48,82,-75,-18,-67,2,-23,23,49,90,51,-65,60,46,99,-53,-71,2,-72,49,99,28,89,-87,-24,63,66,90,84,94,59,-64,76,84,-29,-91,-62,0,-16,39,-10,-65,75,-50,81,26,-50,62,-72,-22,-36,79,58,-47,44,-66,69,11,77,5,-43,36,42,-66,-28,65,95,10,-35,-68,-51,7,-33,76,58,-99,2,60,63,-18,-62,-73,-86,96,-67,58,-70,54,-31,-93,-41,27,-5,-99,13,67,18,-40,
    -71,35,-8,-69,-57,12,-93,53,13,-38,13,-72,-4,51,-44,10,-1,-59,-31,-19,95,-10,-11,-46,69,36,-92,-18,4,26,-5,33,-86,87,64,-43,-49,24,-90,64,38,23,93,-66,-74,-99,45,25,94,66,-94,89,-44,-53,95,-74,84,-97,-40,-60,-18,55,73,-52,-5,-10,5,46,66,-33,-37,4,90,-92,-10,68,61,-65,-7,55,-99,-49,44,-90,-49,-9,88,-65,-53,-52,-25,-19,56,-100,29,-49,-58,-66,49,60,1,64,-36,-57,72,55,-89,33,42,-44,-60,-4,-41,36,-42,10,-72,-54,-3,-25,-6,24,56,-50,25,-15,-47,19,-29,55,-20,24,19,-4,67,-56,-97,30,29,46,86,70,94,-55,
    -42,-48,8,-14,-2,-95,13,44,30,-79,47,-93,-42,-100,-22,29,7,58,-47,79,-93,72,75,-90,2,4,8,-60,-74,2,-14,-15,6,46,23,-96,51,-63,1,-67,-90,-52,92,-31,-100,-29,50,8,81,56,-13,88,28,-38,51,83,18,-41,-77,45,14,-39,82,72,-93,-43,29,11,-6,-70,44,-43,-22,37,26,-22,60,-72,86,41,84,-75,-18,-35,-61,-67,-52,58,-56,-77,-45,10,-15,89,-17,-56,-54,-88,55,-7,-58,0,50,20,89,-72,50,49,-44,37,-58,-59,14,24,-42,54,9,-42,-36,54,-19,19,-84,18,-40,-1,63,6,-37,18,99,5,70,1,77,11,29,28,-88,86,-83,-45,79,-17,
    79,89,-63,-11,-53,-47,95,-20,24,-89,99,84,11,14,-57,26,84,-58,-68,7,-4,61,-82,-75,-59,-69,63,-90,-14,-6,-6,-83,-65,-17,-42,-18,37,-47,15,-87,-83,14,98,-20,80,-59,-94,64,-65,-62,-77,31,52,-58,9,45,-75,-76,56,-37,-29,-98,-20,6,-15,-9,89,74,44,-44,88,61,70,38,-59,2,79,-52,-82,66,-62,-58,-50,42,-16,59,-12,-39,35,96,24,-94,98,56,13,-65,-53,-46,62,44,10,2,-43,-68,-60,-49,-14,71,-1,4,-11,-11,46,39,84,82,-50,-28,-57,86,-32,19,92,-82,-24,-43,-47,-25,-37,-85,-81,25,-31,-23,9,9,-20,95,-68,31,0,22,-80,-54,
    -87,56,-71,-36,-72,-76,-50,-52,44,94,-82,72,-96,-28,-53,-33,-61,-81,93,9,-52,-46,70,28,-50,55,-41,50,-71,-69,48,-58,88,77,6,68,54,8,69,-2,3,87,-30,7,-89,-31,26,-97,-60,-29,-36,-12,26,86,-84,76,41,-73,-22,22,-41,-74,65,99,-96,-77,67,58,32,-12,-92,87,-24,-70,-6,-61,-49,72,-58,-8,-56,6,80,22,-7,49,-2,86,28,76,9,87,-98,-74,38,58,-51,-42,-32,81,46,-72,20,74,-42,-86,-86,-38,39,-44,54,35,15,34,-43,60,-65,7,46,64,-17,-45,3,-63,-67,-58,48,-17,0,68,-84,-54,49,37,-79,59,3,87,-79,-58,-57,75,77,10,
    -38,86,-30,-3,-7,-83,-39,28,-76,17,66,-42,59,-34,-7,-41,86,9,-43,35,-54,-70,-5,2,-83,16,-4,-87,-8,-26,23,6,-40,94,-97,6,63,-83,-66,39,34,52,49,-7,-82,-58,4,-95,4,13,-8,2,44,87,-96,61,-44,1,26,48,27,-50,-46,39,96,-91,-55,11,-74,-20,-98,12,-68,52,57,-97,46,-39,60,-50,27,-48,53,-29,-60,57,84,-52,-90,11,48,37,13,54,-23,-39,63,-26,24,42,54,26,-46,39,-22,64,42,-75,77,2,-73,4,54,80,75,46,-10,60,94,0,-77,42,90,-12,-4,-81,49,-88,-7,-75,54,-52,51,60,87,82,24,29,59,2,83,86,-94,37,
    67,34,-64,-91,-54,-70,9,-31,25,-49,57,73,-30,-42,85,64,-17,39,64,86,-48,3,68,76,-68,27,-70,67,-34,-63,4,-67,71,-8,94,-83,-25,-45,-62,-100,7,47,73,77,57,59,-7,92,-50,-43,78,2,60,47,-69,44,-74,-39,11,-8,-2,68,-23,-79,-40,-29,90,-13,-21,80,-13,86,-21,-87,63,-64,24,-91,-72,74,-34,59,-71,-73,-42,-40,71,-16,-27,-65,29,24,-97,58,45,-85,82,-12,3,-39,-32,42,-53,0,55,62,36,-69,71,17,58,-62,-24,87,-83,34,99,40,70,-28,27,-49,96,-18,-90,-6,-2,92,82,53,-47,-98,-53,-48,-46,55,-86,91,86,38,-40,44,28,-12,
    -17,45,-78,-66,-63,44,7,65,96,-45,-53,-94,49,-3,50,-17,-98,-97,-62,-50,-45,-8,-95,-79,-65,91,11,-5,-12,-61,83,23,36,57,58,26,2,-35,-9,-50,20,-10,-92,-78,88,58,-43,90,61,-5,-60,68,40,97,41,75,41,-47,-77,-71,-56,58,52,33,16,10,-41,70,27,50,72,-100,-60,80,74,80,38,-69,71,-1,-21,63,-81,-81,61,12,46,-46,17,69,-17,62,28,87,-5,96,-50,54,66,29,-44,38,29,-4,18,-45,-71,-92,-13,-100,7,18,15,-22,37,-72,90,-65,-18,8,-95,17,-78,-15,-43,-31,-19,7,-77,99,-64,-21,-11,18,27,-93,73,56,-85,-88,8,-26,30,76,
    4,-81,56,-54,-45,-61,6,-88,8,28,-3,65,97,30,-28,72,29,-39,-97,-82,31,-17,77,-44,91,44,-31,-48,70,51,-72,-74,23,84,-28,30,75,79,-6,-16,-41,91,-51,-91,-27,-26,81,-98,35,37,72,-34,-80,1,-26,-89,-55,43,-85,67,-53,-57,93,22,80,-83,-96,7,96,-2,91,-92,-11,93,-83,62,-33,50,16,54,-13,-60,72,-41,41,-54,-77,38,42,-62,5,89,-18,-50,-37,-86,-33,-33,21,-84,65,-35,-76,-94,-42,-59,20,25,91,88,-21,-69,-72,-49,-10,69,-3,13,-41,91,52,16,32,86,66,95,-100,85,-86,-27,-99,79,38,77,37,96,-82,-43,-27,62,-55,4,93,-75,
    7,-65,46,-95,49,-95,48,-47,-27,-19,-61,-61,-24,91,77,-9,-36,30,-78,55,8,60,51,26,69,77,-60,-85,-19,-67,92,89,-31,39,46,70,-4,-6,-77,70,75,-86,61,-96,-95,38,-5,-79,-31,-31,76,77,-71,-20,-45,99,-43,-4,66,38,-19,58,-21,2,-51,-75,-76,46,-28,-53,16,-53,-39,-23,-49,-82,-84,-2,-60,-63,68,-84,66,49,-4,-79,48,5,-31,-34,96,51,77,-25,-95,-74,-47,-70,72,25,-71,-12,72,-57,18,76,-39,-14,-74,1,23,94,-30,-11,-4,18,-38,96,-76,32,-37,-80,-65,-60,-53,92,-82,0,-78,43,-75,-96,83,50,-53,1,-22,8,87,4,62,-38,-49,-68,
    51,47,2,66,43,26,-50,58,-2,-63,50,-2,-71,69,98,-96,12,76,-92,-53,-22,-45,49,8,63,36,64,-23,51,-85,9,-46,14,-88,-80,-90,-10,-30,68,41,7,-29,-61,-11,-60,89,-7,4,17,-47,51,47,8,-48,-45,-77,41,72,1,-8,39,62,46,-46,26,19,-84,69,89,84,10,-51,-45,1,-10,-53,-58,83,3,12,88,-93,-89,-4,-89,-81,-29,52,91,24,96,30,-13,43,36,65,-38,52,34,-97,-11,96,-96,-4,-3,-6,-4,-8,-71,99,56,-83,58,67,-35,22,86,37,-26,77,-39,-29,8,0,66,96,-34,80,49,-100,35,-62,97,40,34,-54,86,-70,-10,-84,82,-54,85,-8,
    -86,-49,-86,52,40,-59,30,53,-36,90,-46,-70,86,-28,62,-13,-28,-3,25,-79,89,-40,-80,76,-58,10,92,-76,9,29,-31,-25,80,-17,-73,20,24,9,-26,88,-1,-20,70,-62,52,32,25,76,82,-49,98,71,-37,70,47,-43,-68,-9,-66,41,-79,3,-84,53,38,96,-74,-37,5,-48,-97,57,32,26,95,-64,-42,20,-88,92,23,62,64,38,32,-37,48,65,7,82,-42,28,-63,-73,-67,75,-77,-41,90,28,11,46,-15,95,-28,-68,-69,-18,5,96,-25,-72,58,-61,19,-57,-46,-33,60,61,-99,70,-59,-62,97,-25,-87,-80,-14,56,1,98,2,38,45,74,23,29,-44,-72,25,31,-92,-65,
    22,-21,-70,77,46,90,-10,47,13,32,37,10,-93,51,-17,-7,7,36,-57,-91,-74,89,35,49,18,91,77,95,23,38,-70,97,-83,-87,74,16,55,-35,-85,-32,49,53,79,56,-44,-86,-99,15,50,97,-24,76,38,-89,26,-92,-98,-45,-97,-23,93,85,-25,-89,98,-99,27,6,-82,42,74,67,-53,53,23,3,67,-23,-82,-31,-74,-6,-54,64,5,24,72,-40,79,27,37,25,-36,64,36,-85,66,-85,21,84,9,-5,52,57,-99,-73,-40,20,-44,-69,90,-18,25,88,-54,83,12,70,-57,-9,-51,32,-84,-34,-3,4,-19,-37,-29,54,99,81,-99,51,-10,-98,31,2,-25,87,33,-35,-30,-89,
    -95,-32,46,17,91,89,60,92,-27,29,-42,70,-67,-9,85,-43,97,37,90,99,-12,-20,53,71,34,-72,59,-80,-55,81,83,-50,-99,29,19,92,-30,-20,37,43,-91,47,-34,94,-61,-97,-97,88,40,93,87,81,-75,41,52,12,21,-37,32,67,-56,15,69,-54,-56,-11,90,14,21,27,-91,30,-25,-25,-24,-34,-69,-20,-46,71,-75,-58,4,-97,35,-43,15,-44,-28,-53,75,17,14,45,63,58,86,53,-76,7,-19,-67,89,8,-39,-83,74,-8,-51,-72,-85,-25,-78,-28,-70,57,-19,45,66,53,92,41,-30,-42,-62,85,-84,-76,39,92,-17,72,77,-28,-68,-62,-58,-94,-18,-9,86,-50,18,-91,
    22,48,18,55,-55,-16,8,89,78,79,-53,-84,64,-85,93,-45,-93,-24,79,85,1,-89,75,-5,69,-90,86,56,-88,-43,17,-14,57,-65,-59,-97,-28,49,92,-50,-20,-60,-82,97,-45,-89,-96,-85,-60,84,-100,-7,47,27,88,17,37,74,-27,1,83,90,87,41,77,-72,-4,-99,30,-12,-97,-90,-20,-78,59,36,85,64,3,77,0,-45,-30,-53,82,-42,64,-28,85,-63,73,68,79,61,-39,9,89,57,10,19,-54,-34,-18,-22,-12,-59,66,-75,57,-31,-97,-43,24,-75,5,7,-16,-31,31,-31,-41,4,89,90,-35,-49,-49,-93,-92,62,-22,-94,-72,60,-15,-32,54,-49,-55,11,21,-100,69,-55,
    -74,-74,4,-38,-5,-65,31,6,40,20,-51,57,23,0,16,32,-86,95,-10,-58,7,75,-38,-87,79,-40,-23,0,-40,-54,-3,-62,-76,54,0,-29,89,-69,78,-19,4,79,91,-73,-21,7,11,-6,-46,-98,-12,14,29,3,-21,8,63,56,60,75,54,-90,-86,78,64,14,-50,53,-2,80,87,54,59,78,33,-62,-63,-55,-16,-56,99,73,58,28,-72,-63,89,91,46,49,18,0,59,32,-21,23,99,29,-71,-3,9,-84,-97,-80,46,36,-42,83,33,-5,-73,32,-80,37,-87,-52,27,-98,91,-27,3,9,25,-37,-58,-96,38,93,-15,19,-58,46,-13,-55,66,-67,81,77,69,15,-28,48,99,-8,
    -62,-88,40,65,-86,31,90,-30,92,15,33,86,-28,-77,79,57,-5,-79,4,82,-82,-78,68,52,99,-11,-81,-29,89,-82,15,-73,83,-45,44,49,38,34,-81,-17,-98,4,-31,-26,28,-51,83,-77,-78,-61,57,93,-38,25,45,-87,66,64,-63,56,34,52,83,17,60,-20,-33,-50,66,-14,33,68,91,-97,94,-29,-96,78,-54,78,-83,55,71,-21,-67,-32,-7,99,84,-18,55,19,-66,-9,88,94,71,-93,97,-11,-6,30,-42,-63,-15,-96,-40,-11,82,-42,-80,-100,-87,91,-69,-54,-88,-24,-54,96,58,53,-33,45,-56,-44,-9,67,-37,-12,-91,9,-81,-81,-2,-44,-77,58,-2,-94,-84,18,58,82,
    -39,41,-72,-27,-82,-74,-30,28,80,37,-27,76,45,65,-4,-91,-47,5,-30,-76,24,69,-67,47,27,31,5,-4,-51,-37,-22,10,57,-42,36,75,-15,58,3,-35,-53,29,-59,93,-6,37,54,-1,94,-76,-24,18,45,9,-82,-75,92,23,21,-59,39,51,3,-4,-91,91,-77,-6,49,-22,11,97,59,53,90,-47,-58,-4,5,89,-28,33,7,-82,94,-23,-57,86,53,16,79,-8,-33,-18,40,-72,-26,15,23,75,45,-66,72,5,39,14,10,34,-90,-85,23,-65,0,82,-47,-6,-40,-4,-68,-87,-88,-89,-43,-69,-6,49,-41,20,-84,34,-5,61,21,-80,-34,-88,-14,77,46,-51,44,-79,-16,-55,
    56,37,-9,-84,85,24,81,49,-13,90,80,81,-9,91,-47,7,26,1,-32,-53,-27,-13,-89,59,-36,10,-92,60,31,-56,57,-13,33,-51,-45,18,25,88,-33,-88,78,99,46,-31,91,51,28,-31,-96,97,-32,-23,-16,-21,89,0,89,49,-40,-27,94,-82,12,27,-81,20,-54,-4,8,-35,-40,-61,-83,-94,-40,-92,10,89,-71,14,38,-3,92,74,28,81,-26,18,30,-14,43,-24,-44,-45,-96,-73,27,-98,-25,-64,-81,36,-73,-64,-6,-13,-56,-96,-24,-27,19,66,22,-37,40,-49,96,66,21,-22,-95,-36,55,-39,-29,11,-59,99,13,16,87,32,-96,-86,21,-1,1,65,-45,30,91,26,-4,-35,
    89,-63,16,37,-97,89,-84,-92,-95,23,-78,-23,86,-85,-72,-49,-17,-85,83,-12,-71,-44,39,-18,-78,94,-88,65,21,9,82,-38,46,-49,-100,1,-60,68,-38,-54,-57,-64,-25,-71,-97,3,-20,-14,18,-85,26,-1,-28,-35,81,-54,-40,94,63,81,-45,45,-57,53,96,95,-46,89,63,-32,-65,58,-44,10,87,-41,65,-33,46,-17,-17,-28,34,7,38,15,53,50,9,16,-17,64,-39,-22,-31,-90,-26,24,99,89,44,-66,48,-99,-4,35,-40,-39,55,58,-4,90,-69,-70,-3,21,-3,-50,-77,59,18,6,-25,31,84,45,41,-90,-31,40,-100,65,26,0,66,74,-65,-73,87,42,37,-17,32,68,
    -35,-71,-59,63,-69,-36,22,49,70,49,-19,7,-6,-78,17,15,15,17,-19,93,-83,-53,68,-95,-74,55,47,64,91,-68,-16,-44,-39,-22,71,-55,42,-55,94,-35,95,-73,72,-11,-98,-11,-95,17,59,-14,10,28,-15,-70,-15,-36,86,85,80,77,-83,-36,85,-70,42,9,-25,-63,54,-78,2,1,-51,-74,91,-49,-33,48,20,-74,86,83,7,-29,-87,92,-13,51,29,-33,-20,46,32,-34,77,-74,27,-96,-37,-19,-74,17,35,-72,43,-22,-69,-89,-74,4,89,-88,-13,-4,-65,-48,-59,-25,-96,70,42,-16,17,-26,2,-54,-47,-71,-50,16,-37,-71,86,98,-43,29,-24,-12,40,-46,92,30,-82,-69,
    78,-95,-16,-81,-20,-60,42,-77,-24,-41,-51,31,-43,2,60,-41,71,75,-12,57,73,45,-62,-99,86,31,-45,30,-87,-27,62,91,-21,98,63,59,90,5,82,-34,16,84,97,73,-14,10,84,-91,-15,-27,18,-89,70,57,-88,8,-12,-32,-61,1,-7,53,-56,24,51,59,-16,-7,-36,-82,59,32,-98,9,-43,-59,-81,-58,-98,-44,15,21,-33,-63,-70,-68,46,18,-48,85,71,45,-10,67,-30,-59,27,6,-14,43,-24,-55,-24,-21,6,-15,-28,-23,27,-26,-14,94,47,53,-68,77,37,30,-53,89,15,70,-13,-95,38,9,98,-83,-85,-16,12,-9,-19,40,22,40,-74,46,-83,5,-79,-45,0,68,
    -91,-16,-2,-2,-86,45,-12,-19,16,27,-62,-94,-64,88,75,-97,-76,-13,-54,-43,-72,69,97,-46,-33,67,11,-12,-78,-89,-91,83,-53,7,-18,-87,-96,22,-6,20,-99,-16,-22,-63,-28,-47,92,-4,41,-62,54,21,59,51,27,-73,-30,-62,-33,-7,-98,-72,-72,-51,35,10,-85,-8,84,9,64,85,46,-57,74,-82,-52,66,15,41,57,-79,-38,-32,24,41,-5,95,80,15,-60,82,43,-32,83,-69,31,98,23,-33,-40,-61,-47,-94,-18,-73,-76,31,46,-9,-76,3,-36,-13,23,89,-72,71,36,60,-14,76,42,81,96,-22,64,-21,-24,-61,47,36,-21,52,-6,61,31,19,-56,-23,62,-31,32,-73,
    8,-44,68,-12,-73,4,-51,-35,-68,43,-54,80,73,-89,-40,-50,-50,59,38,29,11,-67,-57,94,4,-61,-76,-82,-40,56,97,68,64,65,-91,43,-31,-42,60,1,53,59,-66,27,22,-54,29,24,-95,-81,-46,-32,-96,-51,-38,-40,-60,-14,-21,1,95,-24,-31,11,-58,-70,-93,-37,-60,67,17,94,-22,51,-27,-100,97,-98,25,-98,73,31,-78,78,80,-64,-62,-80,23,17,21,-30,46,43,33,40,25,40,-97,66,-40,20,12,38,-77,85,39,-80,39,-84,74,12,47,96,-10,27,85,-19,-1,-40,50,73,30,-4,-32,-85,-64,93,56,92,11,68,-36,-77,-42,-12,-92,49,8,-1,65,-65,64,12,
    83,-46,91,68,-13,-57,-72,-62,-84,10,34,-64,26,23,81,-66,15,-7,-98,79,16,-40,-33,-23,62,28,76,27,63,40,-60,-2,47,-69,-33,-66,-26,-5,-76,-58,58,59,-70,-16,-66,-88,18,49,-43,-28,-20,73,-16,0,-98,-54,80,31,74,-5,-77,66,93,22,97,-40,57,-76,8,81,-82,-34,-8,-51,2,26,-87,72,-73,-30,-56,-40,95,28,60,50,27,40,-19,-47,87,4,-81,80,-73,68,93,36,44,-47,-31,63,-29,14,64,73,92,29,45,-80,51,41,-20,-2,-79,92,-52,48,-16,29,-99,-29,86,72,-97,65,41,96,-47,-15,-51,-26,-100,-28,88,64,-55,81,-7,42,53,96,-17,85,
    -5,-43,77,43,-95,-39,-75,59,-16,63,31,-13,28,-76,-64,33,10,-63,-93,10,10,96,-73,7,29,-28,-98,-66,-31,-63,-81,64,-6,48,7,-48,61,32,-89,-55,-5,-6,-16,75,19,20,8,-71,58,-32,-9,-80,-36,18,27,45,91,29,79,-88,67,50,76,-39,-2,-17,-35,-41,68,-24,4,-85,23,40,43,-58,-39,-97,-77,-81,-29,-34,-61,-13,85,18,-68,28,-52,63,40,67,13,-32,80,63,3,-54,-78,-29,-26,-22,-61,97,-81,82,-61,-68,37,-86,-49,-91,-19,-58,-4,-82,12,-19,-2,12,-56,90,-21,10,-42,-88,-75,13,-90,-52,85,-16,-22,76,-18,-51,10,-27,-19,-53,-60,-16,-44,-27,
    -22,53,-9,91,86,41,3,82,31,35,-56,41,47,-30,-46,57,-30,91,93,48,19,-25,98,-71,1,-21,-23,-7,16,-15,66,94,90,-43,85,76,50,-11,59,-67,76,-97,26,23,-27,32,-68,43,24,-75,-56,-57,-47,94,-75,6,-75,-46,99,41,39,65,36,30,74,21,-42,24,62,69,9,38,73,-65,-87,98,67,-55,-6,-57,-77,-10,-13,-72,-16,-36,34,9,18,-15,-49,-43,-98,87,39,76,-40,98,-48,-77,19,61,61,-8,48,75,-57,15,72,-11,59,47,79,98,75,-85,-38,-39,24,32,46,27,89,48,-34,-19,-24,27,-69,-20,50,50,93,-37,95,41,90,90,-91,15,-21,68,-38,-90,
    18,90,-75,80,-49,1,12,98,81,-47,-2,-53,-14,27,74,17,-93,-24,-80,-99,-8,-85,-6,-18,-95,3,49,-64,-77,-88,98,41,-98,75,-27,-95,-72,37,-97,-91,91,-46,-43,-23,81,83,47,40,-40,67,-7,52,34,40,86,-9,-57,36,27,-33,0,25,60,-46,-48,-66,-41,-20,71,-85,-10,14,-79,-1,44,54,-18,-9,-6,42,-90,40,-54,44,-20,85,-65,-25,-27,14,42,73,91,-45,-73,-57,-11,-62,23,-40,-47,65,27,-74,-36,-29,80,47,62,-25,-59,72,67,-60,-32,-53,-75,3,22,-2,69,-83,-77,60,-28,-98,55,13,-60,-22,25,-54,96,-48,-28,-40,23,-47,-93,-63,-20,-99,61,-53,
    -59,-71,-54,-34,-16,20,16,-47,-11,39,65,13,41,20,-74,81,51,4,79,47,56,-48,-93,-68,-43,67,-31,-11,68,-69,36,-91,-40,34,-73,-3,-46,43,50,44,-18,-32,-43,-77,88,-64,56,-9,40,36,-62,-4,40,98,28,97,65,50,-62,33,81,26,-6,93,-40,21,-10,-34,-36,93,-90,-2,-39,-80,21,-99,-92,-71,-7,48,65,31,96,-43,29,-23,-94,-54,-73,-56,31,60,70,-75,53,-70,-2,-56,-51,14,89,-89,-88,-50,-69,-15,-97,-61,15,-4,39,-68,80,36,-58,-39,-87,-52,8,92,-7,91,52,-85,-83,5,98,-33,1,-53,34,-10,-42,46,-8,-58,84,-4,-67,-49,44,73,-65,24,
    9,-23,86,-26,26,-54,-34,71,89,-82,86,58,23,36,-74,77,83,60,-33,94,-42,12,-64,-58,-92,-31,-55,52,-58,-19,-71,3,58,67,77,-64,65,95,-93,54,-87,46,13,-11,-18,39,-34,-82,-49,-15,-88,9,49,-100,4,57,-31,49,-38,-36,-70,-9,19,41,-90,-51,77,75,-56,37,29,-90,-65,-58,-1,-31,33,17,-13,84,54,-49,-54,4,-49,50,-87,-27,-49,-25,37,-66,18,-44,75,28,5,4,3,2,-59,85,12,-24,-73,-89,-2,13,-20,85,-3,86,89,-5,-10,-60,-55,-44,13,-3,-17,-98,31,-98,-41,-42,-18,16,62,38,-82,-96,23,30,-68,2,-7,-18,-85,25,-32,-35,-36,-43,
    60,6,-51,-94,62,15,-45,-54,17,38,-100,76,96,-18,-55,10,-80,-85,-34,43,-2,-49,-54,43,33,-39,-79,-99,-22,-15,-90,-61,91,-88,97,54,27,-48,-100,-56,42,-100,73,-10,-66,70,-100,-45,85,-33,50,-17,18,96,79,-49,-90,-100,-95,-12,-15,67,-21,-24,79,-24,82,-42,28,34,3,-78,-14,-72,12,21,98,13,76,35,80,-22,71,-2,-25,50,1,-63,50,6,-23,87,-74,57,-85,-95,-67,98,-84,14,32,19,36,-81,-1,-51,-8,-51,62,-80,84,-6,-2,55,44,25,-43,-55,62,-93,4,92,46,30,49,62,-13,34,60,3,-52,-56,74,37,-85,-27,38,7,22,-48,27,-41,46,-74,
    -86,-58,51,-76,-13,-34,83,43,-42,30,-27,59,-8,13,-7,-96,-84,-6,-52,91,-69,-84,-84,69,23,-9,73,-97,50,19,81,16,13,-16,40,-100,50,24,-4,60,-46,21,19,-2,-66,65,2,-49,-41,2,94,-58,18,-90,63,94,-47,36,97,-97,-93,78,20,20,-38,-88,-28,65,36,68,25,42,-58,97,-60,-24,62,-6,-21,73,97,-75,67,-33,88,30,-39,-59,-82,-90,-55,-75,40,17,-3,-97,29,69,20,-34,-10,97,-92,32,46,-99,60,-40,-5,-60,-67,-56,-83,-100,12,-95,-18,25,-53,0,36,-56,-23,-24,61,74,-69,-10,44,3,8,86,1,-31,18,99,-30,-22,-40,65,-30,-55,-90,-12,
    -54,22,-55,-20,99,-8,81,-13,-12,-90,16,-51,85,47,-8,-19,-97,52,-33,-44,-79,37,-45,-9,-85,15,-43,-62,61,-33,-22,59,41,23,-61,-8,68,72,80,-44,35,-4,-42,20,95,2,-99,98,-46,-80,-46,-24,57,-38,67,72,77,76,62,-10,95,-60,-99,-12,16,93,81,-16,65,61,-8,0,-43,2,72,-96,-96,25,-97,-41,45,-91,-13,2,-29,-46,-73,-99,83,41,-57,78,82,-55,-33,-50,-62,-52,34,55,61,78,-92,70,-19,80,74,-63,6,29,-4,-97,39,35,-42,62,90,-15,15,25,-74,59,3,60,-96,22,-90,-58,70,-4,97,-17,-73,-95,-47,8,38,-72,45,96,9,94,99,-52,
    29,-43,63,-29,-6,-22,96,-27,89,-48,-67,93,74,96,87,-3,-56,37,80,-29,-6,-66,79,-68,-86,-23,-72,23,-77,80,24,4,89,-13,76,-16,17,24,57,7,76,42,-48,3,-10,-60,-100,35,77,80,-94,-29,66,38,-44,80,67,-64,-44,90,68,80,-6,58,-81,-78,94,-12,47,3,95,-25,97,-100,78,-12,92,30,-77,69,-37,81,92,-71,19,-100,-90,-14,89,-34,-72,57,98,23,15,-83,-3,-39,5,96,64,-47,-28,62,-95,-98,-50,97,-15,-75,18,48,-94,-38,77,-22,-37,39,16,-48,57,45,61,55,-80,-23,24,17,-10,82,14,-45,87,38,-83,92,-8,-81,41,77,96,-41,25,-98,
    -27,-45,-68,36,94,49,40,4,-6,2,-41,14,31,36,83,-79,-30,-3,-72,-43,87,97,-99,-20,68,-6,-43,64,53,35,19,-74,90,51,15,36,-48,-93,-60,46,-39,-48,-88,44,88,96,18,-90,-55,46,-81,-67,-4,-80,-35,-36,-34,22,-71,-81,-91,48,97,51,-49,-36,-12,4,-28,28,-98,-15,32,-85,-70,-28,63,48,-18,60,-54,1,45,42,-27,-90,-93,39,-15,-64,10,-6,36,-40,-54,39,24,-14,-57,48,66,98,-66,-1,-87,-84,71,28,64,-94,-12,-90,7,-66,5,33,-56,12,-28,-19,0,35,76,88,95,74,27,-29,-40,23,-80,-74,-27,-94,25,-14,-78,-51,66,38,-45,-94,0,14,
    92,-95,-53,89,-31,-28,-30,21,-93,-54,9,54,-80,89,-75,-68,-36,97,-41,37,55,84,75,29,-67,-7,-33,-60,-1,20,55,-8,25,-98,-67,47,-74,3,-32,-67,2,30,39,-26,19,17,7,35,14,66,-28,-78,2,99,51,-12,-8,71,28,91,-9,-65,-65,68,-62,-32,15,-36,72,36,-50,-74,-34,41,0,37,-42,-93,-28,25,25,44,47,-72,43,-50,16,-65,-79,96,78,12,-68,-86,-19,-78,-66,48,-62,58,-16,-60,84,2,82,-15,-61,92,-56,11,17,70,-93,-84,98,2,-33,-34,-11,-60,-86,-32,53,-54,-66,86,-80,-32,-14,-41,-73,23,-49,63,-75,-67,-52,-35,-74,-7,76,-5,-37,84,
    -36,-87,-62,31,-21,28,71,-55,96,-24,-56,-70,-86,-36,-50,-47,-25,-71,76,27,93,53,12,93,-82,-10,-14,47,86,1,-17,-50,14,21,33,45,-51,-44,43,-55,-15,87,-73,-49,-49,-70,4,-21,-89,-68,58,-96,86,-30,50,-96,61,-64,3,-1,-10,86,1,56,8,86,-46,9,94,97,7,-21,84,-14,-17,87,16,39,-82,28,72,76,-16,58,-53,-66,14,60,-77,70,-89,-35,56,-36,-79,-84,-98,-73,26,96,76,85,-72,12,-29,11,0,40,-98,-30,-80,74,-53,4,-16,-54,-9,51,-94,-34,21,69,31,-71,-67,4,-54,-65,32,24,83,8,9,-89,-79,32,-26,-27,-28,-23,43,-8,-97,42,
    97,-12,88,-60,91,94,6,12,15,89,41,48,-7,39,35,77,-85,19,-14,-24,-18,-41,-91,57,-16,-19,86,-73,26,89,22,-25,-23,10,15,-32,-43,73,32,72,62,-74,-27,-93,17,-92,85,33,-21,-77,61,14,-18,70,-29,66,-96,-91,45,30,98,-33,-95,-72,78,-28,-52,87,-55,81,59,59,59,84,66,-24,45,51,-91,-76,74,71,38,-44,-7,61,74,-51,70,-28,79,21,-9,36,49,-79,-92,97,-92,-95,30,68,64,-11,52,31,66,-3,-18,27,-26,-91,-50,64,-83,-4,-22,92,45,-100,-84,25,21,7,-87,-78,29,74,-80,37,79,2,-43,-4,92,62,-73,58,-41,-39,-63,-15,-30,-12,
    -98,-60,84,-20,-68,29,-20,-100,-94,54,7,72,76,-64,46,48,74,-23,-49,-69,73,95,93,-100,5,-95,-86,42,-58,-64,82,-56,-24,18,24,-40,-100,57,-40,-94,11,68,30,-61,56,-24,88,30,54,91,14,27,38,-41,-20,-57,-84,-54,85,-41,-18,20,3,11,-62,80,71,-62,-11,84,97,-100,4,27,91,-40,-96,31,91,10,-26,-43,89,-88,16,-31,-45,-15,15,93,44,50,13,99,-39,3,31,-68,42,-80,68,-9,72,-28,-82,16,33,74,-53,-24,-64,22,33,-74,34,-99,-5,42,-14,63,-13,82,13,-100,34,74,-45,-83,58,49,-62,27,-60,62,51,-41,-22,-16,85,-22,60,-78,-100,-55,
    -52,-14,-53,-5,80,-15,-42,-33,-80,23,-81,6,-3,-25,23,-92,-76,13,-13,-35,-24,-62,-24,6,23,-87,84,-65,-65,36,-19,-65,75,-20,31,55,17,41,-77,37,-35,94,95,62,-31,19,22,-54,-68,-91,-37,-40,-52,39,67,23,52,3,58,40,-60,-61,75,67,19,6,22,-63,-52,-3,26,-35,-56,22,-73,13,93,2,11,-75,11,74,-62,11,13,57,-66,-82,-88,93,10,-96,84,-15,71,-44,44,-54,-7,44,43,71,-91,39,-55,88,-47,-62,-10,64,-84,54,-9,-46,-35,-96,63,0,74,75,-55,-16,80,-71,22,51,-15,-34,97,30,62,-7,-46,71,-68,-1,11,-63,38,-98,54,6,8,-55,
    12,73,-99,-25,-75,28,-98,-30,64,82,0,-14,-14,37,4,-65,-32,-34,-20,22,89,-35,73,1,-46,-37,-45,-92,69,63,-95,-19,-12,59,8,14,-61,-37,84,-97,-55,-64,-10,-17,74,46,-29,94,-87,51,-32,54,68,93,-45,-77,-43,-38,83,78,77,41,-88,-34,0,20,80,39,83,16,-6,81,5,84,-84,-21,31,-13,-75,-4,91,-55,-50,59,-62,-42,82,-5,-28,66,-26,-50,7,-14,-32,59,58,-100,-50,-6,16,44,-73,73,-19,-57,-48,64,31,-71,-88,22,-26,-38,-19,13,-28,-84,-92,45,34,-66,47,93,-28,-85,52,-69,-85,54,-23,-17,-2,4,-43,31,47,-39,-5,-70,-9,-41,4,17,
    22,38,-70,46,54,91,-57,-60,77,90,-67,-50,-95,37,-67,72,-9,10,-92,41,66,-83,-75,-35,-70,-28,-4,-27,32,-100,91,-94,-10,73,52,96,64,96,-64,-6,86,-31,44,44,-94,29,-32,-51,-61,-72,43,-95,-55,68,-30,-72,92,18,1,24,-29,-8,30,61,-82,-17,58,34,-69,94,-72,-83,-84,72,13,74,-47,34,76,-8,62,-29,-51,60,91,-28,88,83,42,-11,8,-87,34,-62,27,4,-27,-63,-62,4,83,-33,-26,51,91,39,26,-55,-27,-46,-11,-64,25,-9,96,16,63,36,-1,-43,77,-93,-77,11,98,50,-85,-29,39,-94,28,22,-27,54,-74,-84,-7,52,-39,67,-94,3,-97,31,
    94,51,-53,9,-61,-2,66,-84,6,89,-20,-44,-9,-53,-21,82,-47,-93,-43,-22,61,-17,-5,-45,87,8,74,93,-89,29,-24,-95,-20,-25,-34,-29,73,-67,-13,-69,74,19,-13,-82,67,67,52,72,74,-91,-49,-12,-8,-2,-5,-21,-94,-31,-76,-82,98,-48,75,-70,27,-58,1,1,-73,40,-16,53,-40,72,23,79,39,-24,51,-35,-15,54,5,30,-48,-100,61,-41,-31,38,-71,-33,90,56,-51,18,50,50,71,77,-9,-45,-17,3,79,-94,-18,-30,-18,-67,-12,20,88,93,-98,92,46,63,-49,67,1,-68,-65,-56,89,-64,62,39,87,-15,-31,-70,-8,52,33,-28,10,-85,-6,45,-100,82,-83,40,
    28,-81,33,-26,34,36,-59,88,69,-72,-68,10,-35,46,-99,-96,-69,-30,-66,-25,74,19,47,-15,34,94,82,-14,76,99,-73,4,-82,12,30,4,-100,72,-8,69,52,-24,-69,-83,-78,-67,-79,-95,-45,-93,-19,-70,26,80,-33,-88,74,-51,-1,51,-100,-22,-93,-82,42,38,-26,42,-38,67,-36,-86,-57,-5,84,-34,-20,57,-77,36,65,4,18,-57,-15,85,-44,-89,86,7,14,86,37,22,56,-21,-88,-70,-79,-26,49,-63,-60,93,-15,-76,-89,65,82,86,-47,-1,-9,71,42,28,8,98,39,-54,-43,-46,84,94,28,-8,-27,40,23,-53,66,-76,-64,-94,-83,21,-69,80,-61,65,67,-8,-36,-90,
    -84,-94,-10,-24,57,-71,-25,66,-65,-41,61,15,52,86,-45,-73,85,73,-49,-78,80,-79,43,63,53,34,28,-28,79,92,-66,95,50,24,-77,-41,6,-2,-74,41,10,87,-43,-86,25,-36,-59,11,-62,-56,33,-82,-35,-72,33,71,63,-39,-57,-6,-95,-22,-11,7,-98,-36,67,-40,15,45,-98,77,-16,-89,91,-91,-25,84,-28,-35,-72,-95,-65,-54,86,68,-83,1,-71,-88,95,86,-58,-64,-6,97,-48,-87,-43,67,58,-41,44,-58,-78,35,-97,-2,-29,-24,15,0,-67,51,46,-81,19,15,-28,1,-21,-33,-13,22,-45,33,19,8,46,76,-25,4,88,72,-2,-90,-41,-46,-40,31,30,76,-69,63,
    27,29,35,98,96,-93,-1,-25,-73,87,97,-18,20,-32,90,-33,97,-82,23,85,90,74,47,-51,-72,60,32,10,-12,15,-75,-85,96,60,65,92,20,-35,-80,47,-48,-83,81,24,-62,24,43,-65,42,-81,-80,-16,93,19,-15,73,-21,-82,-17,67,33,8,34,30,-79,0,22,41,65,-6,-60,69,-36,73,-7,2,-3,89,37,91,-92,9,-25,-99,80,61,26,-40,79,-91,79,-36,-31,14,94,-10,66,-31,83,-17,63,75,-48,-73,49,-3,-19,98,-14,-82,90,-6,-21,-35,-53,-40,78,73,20,57,82,-49,-78,-96,65,68,-6,-17,-63,-70,66,53,-95,70,32,6,-32,-86,5,6,84,-5,-99,-36,
    12,-100,24,-9,74,96,-100,8,-53,74,12,65,43,59,-52,32,41,67,-63,-2,37,70,-95,57,84,10,-36,68,57,-83,84,-31,-83,8,12,43,-44,13,-48,-44,-13,16,-79,-70,-73,-79,15,20,88,52,-81,-22,22,-76,-65,-42,-14,-1,-73,-57,16,-89,-36,-14,-28,77,-71,28,-10,-67,36,77,2,57,60,-19,31,-25,-98,-29,-73,-79,49,50,97,85,60,-17,-64,87,26,5,51,90,91,75,19,-80,-45,-91,-94,92,39,8,1,-1,89,-68,74,91,-44,53,64,-95,-45,61,42,-84,-56,31,55,22,-64,6,-35,-73,-67,84,99,-11,46,57,-19,-15,-35,82,84,-93,67,10,98,-77,-85,15,
    80,-29,-24,-25,-61,-27,-94,94,95,-58,-47,-88,-79,-14,49,72,27,-5,30,-92,80,47,43,16,54,10,78,5,-15,93,-80,-35,-84,-52,-8,-93,21,-2,54,-31,-8,-93,-19,-35,-55,-70,-62,73,25,20,81,-43,-81,-24,73,74,38,3,-69,-77,-51,-97,-59,-83,-49,33,-75,73,-68,79,-58,-24,38,23,42,83,6,32,8,-69,-96,-58,-59,-77,18,-86,49,-43,-30,32,-20,19,35,-27,-12,87,-93,-35,60,91,-56,54,-33,34,-23,61,18,35,-55,-74,67,-51,-80,-92,-75,39,74,74,96,96,-41,-72,-33,94,-46,56,81,-39,-79,-7,-96,-82,-53,-77,52,-23,85,-78,-88,-70,-99,79,-68,-79,
    39,-43,60,-34,83,8,-38,-58,-11,30,-63,43,-14,70,56,-41,-36,12,-23,11,35,-70,88,-28,-96,-99,-45,-95,32,-13,-73,-76,44,39,-10,-21,-52,52,-26,-11,-18,11,84,20,-67,40,80,-3,-48,-91,61,39,91,49,-88,96,2,-33,-99,-65,54,80,-89,-50,20,1,-19,20,-47,55,-91,-12,-82,93,60,-48,-15,-8,49,-11,-46,-38,-72,-55,-88,-60,-59,-34,7,95,-47,-87,75,-36,-85,-53,65,-3,19,71,-48,28,-89,-29,-27,71,23,10,-84,-76,99,-30,-13,80,15,-49,-80,9,17,-20,-44,23,-55,83,87,-39,-69,53,58,50,-24,-90,-69,-13,-67,-44,-90,8,19,-74,33,-30,-52,72,
    -50,16,-77,-77,-75,-8,-45,-19,-85,0,-36,55,13,-53,-40,-77,50,36,86,33,75,-81,89,85,28,8,64,-87,31,64,85,81,-20,60,-96,5,-96,-89,38,-80,12,55,27,77,-98,87,-99,52,23,87,-15,-2,-94,-73,-65,-14,-13,51,-1,-82,16,-64,52,96,96,8,-46,-99,20,92,73,84,-1,0,61,2,-61,62,-94,62,49,-56,-88,8,-77,-53,94,-38,99,-6,-67,-33,30,85,-37,-21,-7,-31,-68,-87,-86,57,97,13,-91,11,67,-52,-27,74,-90,75,70,74,-17,45,-79,-71,-93,72,-25,-60,-61,-94,77,-45,-63,-29,24,-31,-64,38,78,86,-96,87,-3,71,-65,22,-3,97,-3,-33,
    71,32,64,44,62,24,-31,89,64,8,95,42,15,84,65,92,-95,-99,-18,83,87,-14,70,-64,-42,57,59,55,54,-92,75,-23,-7,39,-78,-45,-85,-9,96,80,51,92,-26,-81,28,-61,-89,34,92,-55,17,80,32,-60,68,90,97,27,-3,4,36,-28,33,-71,64,7,-64,79,98,-68,11,2,76,37,-27,5,76,-64,91,21,-19,8,-47,-87,48,-79,55,46,49,-95,-98,-63,-71,35,-82,93,-57,-94,25,93,90,88,-53,67,26,20,-76,-46,56,-85,27,90,-25,80,55,-76,-98,11,22,51,16,-24,40,45,-89,58,-9,-94,16,68,-100,6,8,99,-75,86,20,-51,93,-72,-84,20,70,-56,
    1,26,-32,3,89,42,-94,-43,70,-2,54,81,-92,-55,40,24,65,40,-70,-74,39,-44,-88,-89,-43,-95,92,-74,-22,62,-30,79,-60,-10,-66,-71,-68,-60,-14,-98,38,-59,35,-2,-62,-25,-78,56,67,-48,82,-41,-40,46,-30,-30,52,-86,-4,-70,29,66,61,69,-44,95,99,-60,87,-63,94,-23,-22,29,75,-31,57,97,-75,24,-99,-93,83,-38,5,-94,32,-91,72,80,91,1,46,52,-29,54,47,-78,94,34,-41,88,-89,38,-31,38,59,-74,35,84,3,37,43,-14,51,-52,-56,-65,-42,-83,-85,-51,18,61,54,41,15,-47,63,-39,88,-77,-51,51,13,18,-10,-28,-3,77,-92,-100,14,51,
    -62,-83,51,-17,-48,61,-48,-33,-89,70,80,17,-36,-53,70,-73,8,10,-98,9,-38,-33,80,4,39,-23,-67,-1,-71,0,-50,-33,17,54,2,22,15,54,41,-22,-23,22,95,41,21,66,20,82,-24,23,-57,-10,90,-77,46,-18,-48,80,81,-19,-20,84,1,97,38,-97,-29,-47,10,-87,32,87,-13,27,28,60,-7,-52,42,22,23,86,-88,14,61,59,48,-86,39,29,-53,71,13,48,20,-49,4,92,-43,-86,-95,89,-99,-8,-32,-19,-48,14,-19,47,-64,-95,-15,-100,-29,-54,59,-81,-40,-50,48,-40,-79,-86,-92,42,-83,12,-14,-26,-22,-9,-85,-21,35,-64,12,39,50,94,-62,38,99,23,
    38,-78,70,-50,-59,82,-48,41,-58,-74,7,-49,20,25,15,-94,99,94,-3,67,-75,84,-97,38,-25,53,32,14,-57,-17,-63,33,5,-93,35,-2,90,88,91,-68,-86,-1,-65,-14,-76,3,-8,75,-3,41,42,-26,25,-3,-88,-100,-98,44,14,-55,79,4,31,-64,11,-34,34,-47,-46,26,-62,-80,77,-75,6,-99,28,50,28,77,-9,-77,-48,68,20,-36,69,23,61,-65,20,40,-9,-49,77,55,18,-37,-92,24,41,98,-3,18,-76,55,-29,52,6,-100,30,49,-77,82,18,-5,98,-61,70,59,-74,-9,-48,18,-6,81,-27,64,-56,-67,-11,86,-68,38,-44,-92,-7,28,12,51,-20,-58,1,-45,
    76,-81,50,27,-90,21,-14,-64,64,38,54,-90,19,79,75,16,65,-84,-46,97,54,10,57,99,90,-31,51,-30,-36,-48,25,92,-77,-72,19,-67,1,-94,-79,-83,-4,28,-73,68,7,54,-16,-28,70,-10,-79,-24,-100,78,-72,91,-100,79,-87,-36,83,-9,-92,58,19,28,43,-80,-14,64,89,82,-8,-84,2,-100,71,38,-76,-7,-72,98,22,-71,76,50,72,28,-19,37,44,-36,-72,53,74,-1,-19,17,-29,67,-19,-40,1,74,-23,56,26,-100,-6,2,-55,23,0,19,52,-71,-31,-24,57,2,-87,2,66,94,-45,-8,-7,-12,-91,65,-93,-9,77,-40,-83,-46,-84,43,-94,11,-55,-96,-66,-2,
    23,38,27,93,66,-16,-53,31,38,-86,25,45,-42,71,-67,-32,-12,92,59,-35,-47,76,-28,-79,-29,-70,32,-32,-66,18,66,58,-92,-7,3,74,30,50,6,-32,16,83,-86,75,54,99,-57,-58,92,54,-40,97,-18,32,-82,5,62,3,73,97,-79,-8,7,-70,37,-90,56,-33,12,-38,36,-71,-54,2,56,52,-47,99,47,-55,5,-93,94,87,-9,65,-8,-47,-32,-83,-98,-59,9,9,-77,-53,-29,80,-34,36,94,54,65,-60,-44,-27,45,10,-76,-8,7,29,-49,-46,-32,42,-81,60,95,-13,29,50,28,91,-89,4,90,-65,-16,-44,71,-22,-89,-12,-29,-81,61,-84,-19,85,-40,89,-34,11,
    -57,34,-47,-86,-54,-100,1,75,50,81,18,62,-15,8,49,69,17,20,-100,28,60,-29,-1,21,-61,81,-42,-1,22,24,-90,17,-42,-37,31,4,-37,84,-69,66,65,50,-20,-97,-42,-71,24,75,49,24,-45,-39,47,7,82,86,-12,92,-15,62,16,-5,-69,26,10,14,-70,74,-2,-39,-8,-85,-89,72,-82,-78,-99,-5,49,2,-81,-95,63,67,-36,-3,53,4,-11,91,-34,-95,-62,-51,31,49,63,13,-25,-87,26,67,-72,-10,39,99,12,-8,-54,-87,94,-35,70,9,84,-66,-94,38,38,-5,-19,56,-48,19,-95,35,-32,68,-100,-5,33,26,-86,14,68,53,-35,-20,97,11,-54,91,76,-84,
    0,61,-49,-42,-49,41,5,-16,-2,9,-97,3,-56,-76,-76,96,19,57,-25,34,-77,-57,-61,-12,-24,-63,99,22,80,28,-10,-67,-59,41,-9,-8,-65,97,76,33,6,31,88,3,55,-88,-1,-73,22,26,13,45,70,52,-66,-54,41,85,20,22,13,-90,55,54,4,98,-54,39,95,74,-28,-46,6,-40,57,-87,-75,-92,-60,-1,35,53,-56,-43,-42,-70,3,99,-84,-77,-27,29,85,80,-16,-11,79,-18,-72,26,-91,-48,80,-33,-35,-11,-20,-10,-50,-27,89,37,78,85,94,-64,-84,-51,88,-68,-76,-87,61,-91,46,97,-1,25,-68,-21,-97,-59,32,84,8,97,-75,40,-61,-25,65,80,-88,44,
    65,58,32,81,-93,-28,65,-69,-14,-21,93,-68,-24,-8,9,8,71,-36,1,-45,-100,-91,-96,-74,2,-57,-47,-33,-77,-82,63,41,76,-4,22,84,20,-60,67,-94,19,60,90,47,-96,-49,-92,28,16,9,83,16,-29,88,-6,73,83,48,92,59,66,56,0,42,-96,74,-22,24,-86,98,-17,33,58,-75,-67,63,-23,-59,-57,45,50,-74,-87,-79,66,8,46,2,-92,-9,-39,74,-53,61,-32,-97,35,-1,27,50,-3,-90,-65,-93,88,-32,22,-35,-39,65,62,-88,44,75,85,-38,-65,-16,64,-57,75,-75,69,74,86,90,77,-26,-11,4,76,38,-33,63,45,-45,84,-80,-28,-55,85,86,-43,-19,
    13,95,44,49,-21,8,44,-46,-14,-34,80,-76,-44,57,-2,45,-87,-74,-65,-68,90,80,-61,74,52,-37,-81,38,-51,-71,71,63,76,15,-36,-45,76,60,-39,-38,-74,-59,-62,-18,50,-11,79,63,15,-86,96,-43,47,-65,-69,-1,-1,-97,89,0,84,61,-85,12,-72,-69,67,56,92,80,70,-82,73,-91,-47,23,-2,84,-14,-87,99,-66,-77,-2,-78,-94,49,-27,-91,-9,73,45,-96,41,-43,-68,72,76,41,64,8,11,35,-19,20,40,-96,-82,24,-57,84,-25,-71,-93,73,-49,-87,-25,-24,75,66,-98,-28,22,43,-18,6,15,-90,-53,-68,19,59,19,52,-21,-41,9,-50,83,4,-66,-41,33,
    93,84,37,58,-89,-87,-15,29,-85,58,-49,58,92,58,-74,-98,-95,-42,-27,64,77,-74,96,88,35,-54,71,91,-68,82,24,-75,19,-39,35,30,-73,-79,-88,-58,31,-37,53,23,-79,79,-23,79,89,51,43,18,29,39,6,-84,37,-71,-93,69,-88,-17,46,-17,-3,-18,65,24,-45,77,18,38,-59,-29,-39,-86,-98,-10,93,43,93,-11,-39,22,80,67,-10,18,97,-3,-61,61,33,86,-56,-70,-80,61,54,-73,39,-28,65,32,-4,78,46,-50,-80,40,94,-86,81,-93,-12,61,75,79,-69,24,-72,-29,-63,61,-91,-19,43,81,-58,-3,-92,33,22,25,-35,-82,3,-88,-80,-25,4,-86,-11,-63,
    -78,30,98,49,9,-70,-75,-11,1,-38,-97,-38,-5,46,-57,-11,96,-49,-77,18,76,-60,-12,-69,-96,-40,6,-40,-73,48,-3,-99,-22,-4,50,39,-22,27,80,31,89,83,93,-64,-18,-64,25,-70,-13,0,48,-85,41,88,98,-3,-52,-44,58,-25,56,-45,-72,-66,3,78,25,81,-43,6,12,98,-59,5,-66,23,41,12,53,80,64,-47,47,-43,93,-3,55,94,6,13,-79,-38,20,-50,-51,-76,80,74,-95,-62,-20,18,-64,22,75,-77,97,17,35,3,-51,-1,8,49,57,2,-54,12,-4,52,77,-31,-85,-3,19,-36,21,0,90,-21,38,-29,97,26,45,24,49,42,93,-16,97,-57,36,6,
    92,-7,-92,90,57,-44,43,34,25,10,-69,97,26,5,-3,16,84,87,39,33,-35,-16,-43,67,79,-49,-49,76,-54,39,-66,-62,84,94,-72,41,-98,23,-73,-20,-67,-41,-23,11,16,26,-72,0,-35,19,85,30,-96,-58,-3,35,-55,1,-37,-9,92,-50,-19,77,-56,-38,18,-53,85,46,27,71,57,-96,-66,-75,-18,-86,-23,47,34,-38,-71,-10,-44,-21,-75,2,80,-60,45,72,-10,79,1,35,-59,-80,34,78,18,-87,49,-25,69,36,-48,-49,50,29,98,-16,91,79,74,99,-42,51,1,90,44,99,63,-66,78,-36,21,71,-64,-93,49,54,-80,-49,81,89,87,-67,-60,37,14,90,-78,5,
    70,48,5,80,-48,58,71,96,9,-14,82,-61,2,-44,-90,39,-37,60,-55,-16,63,79,-27,50,64,-34,-13,-21,-92,-39,36,78,62,93,11,-86,4,-18,62,-87,68,44,-95,-30,-100,-85,61,-84,27,-93,0,-10,38,25,92,2,43,-68,33,52,-7,-30,-18,-93,-85,93,73,19,-73,35,-15,95,-68,-10,-34,84,-43,79,-100,37,86,-48,-73,-24,30,-80,79,-27,-48,-36,77,-3,34,-88,-43,2,57,-70,-79,85,18,-42,80,2,0,98,-14,58,78,-13,-5,16,-9,-26,93,-79,-54,24,-53,-50,88,76,0,-25,88,57,77,46,39,50,31,-91,-91,-37,11,9,-38,98,-81,-60,-63,-34,8,28,
    -7,1,2,39,77,49,90,66,25,-58,-59,-34,51,-82,-88,-10,68,-5,-48,29,58,63,91,-80,13,-90,-88,50,-23,-27,-69,70,26,33,-39,4,-66,-97,-30,11,-55,-37,77,96,-67,41,-61,53,88,-9,-17,47,-46,-26,19,68,-64,-16,-30,-35,57,1,87,-17,86,-51,87,72,-48,-91,-64,-50,-76,65,-54,-91,7,-63,63,-53,28,98,94,35,-76,14,3,-40,98,25,26,7,79,13,90,17,14,30,-10,67,-9,26,69,16,-9,-85,-23,50,-47,-60,98,33,-62,44,-32,-86,-90,23,-25,-92,-51,53,-33,-72,-34,-42,-3,33,40,87,0,-69,-35,-79,-1,9,-64,77,-41,-59,69,-91,27,60,
    54,95,74,64,19,-51,25,20,-98,92,0,-79,-98,97,54,42,37,-94,26,-98,79,77,11,-85,54,23,-91,24,-16,36,-16,90,-17,-90,55,54,-40,-68,-26,-86,76,26,87,31,24,-7,-27,-39,-49,51,-85,-70,29,27,98,-17,-98,59,59,-14,-53,95,-23,30,-94,84,-15,-82,68,11,-68,44,-62,72,-25,62,-35,1,75,-31,-48,90,-1,81,-31,-51,-83,-77,-92,28,10,-45,24,39,86,-18,23,-77,0,-57,-14,-16,87,24,-44,-85,-62,-26,-32,-87,43,20,56,94,-46,77,-56,-77,-99,-48,51,-37,-40,-73,-98,98,9,-23,-79,-91,20,7,-6,-41,84,2,-26,-78,76,42,88,-29,-85,-4,
    -34,-79,-75,-38,-56,-74,14,95,-11,26,75,-57,24,-16,20,-3,-54,92,57,-8,-48,41,94,-22,15,23,73,-45,94,88,51,-88,61,77,-26,-95,55,41,52,97,-81,27,-60,44,64,13,-59,-90,5,98,2,9,-9,-4,88,-93,71,61,-38,66,1,66,30,-38,-5,57,-33,-50,-50,71,99,-31,-1,92,65,-37,5,-93,-27,62,57,27,-28,-51,-77,-88,-44,95,73,-30,13,-26,-64,95,36,31,4,55,34,54,26,33,76,77,25,41,92,82,48,-83,-3,6,44,21,7,20,33,63,-33,6,-67,80,32,-78,27,-32,5,-68,75,-61,38,1,25,14,-69,-50,-92,-77,85,56,-59,-18,-86,85,
    -97,21,5,36,-64,24,94,-78,-44,26,-56,84,46,49,68,21,41,6,74,-34,73,-95,-32,81,-71,-95,-11,-78,-13,-44,59,-58,29,-83,30,66,41,24,40,98,-98,84,82,48,-15,50,-79,78,-92,96,-56,81,53,-35,14,82,-30,-96,56,10,-40,16,4,89,33,-65,7,-26,59,-53,-28,14,-69,-94,-38,-31,8,-64,-53,17,-68,-56,-50,37,-39,65,-28,-17,69,-72,45,81,44,-50,70,-23,-15,-22,4,96,-23,76,10,-39,35,25,30,-57,61,29,12,45,25,63,-18,38,-20,6,-78,-51,35,67,82,-21,69,4,9,-46,82,-87,-97,-88,41,13,-27,-24,90,-97,20,3,-16,-68,-100,62,
    47,-17,0,-73,89,22,-72,-24,-58,-90,56,-89,15,-83,-82,-51,-70,-79,-39,71,-14,34,-52,-23,-11,20,80,74,52,81,36,0,-84,-64,27,57,11,56,86,5,18,42,-32,-67,11,-14,-17,41,7,-4,-36,46,-69,12,-77,20,-68,55,-54,37,-12,34,-63,-44,23,16,66,-66,-76,52,-9,43,-6,59,76,-95,-54,-89,-2,5,8,62,-49,-9,-73,-74,-89,-89,34,-90,48,-78,44,37,-69,-33,-46,97,-47,-70,49,44,73,95,4,2,52,2,-35,-50,-93,73,64,-89,-84,91,-63,80,55,23,90,3,-2,34,93,29,54,99,26,59,-71,-25,4,55,-78,60,-43,74,-38,-26,76,-31,-100,40,
    -68,-84,-16,-78,96,-61,45,86,-6,-57,-27,-61,72,79,-62,-50,38,-32,-23,-58,75,-1,54,84,-27,-84,-42,-51,-62,-42,42,-30,-25,78,-56,-77,69,90,-90,-37,85,-65,-45,10,-86,-7,60,4,13,-62,-1,-12,37,-47,-28,63,-30,-17,64,-40,41,6,-18,68,-64,27,92,-95,-31,54,-79,-46,89,-24,16,55,69,-23,-41,-65,-33,58,23,56,-88,48,19,-66,-69,-64,-54,72,-6,28,-7,31,-93,-15,36,-24,-61,9,-17,-72,85,99,35,7,-72,94,42,-5,53,-83,-96,17,65,-25,3,48,11,49,-27,-94,-23,66,37,-15,-97,-75,13,-58,35,96,-78,-28,48,57,31,28,51,-27,-76,-44,
    -57,80,25,60,-45,-72,9,67,77,34,73,-45,0,-38,-8,3,-13,57,-3,74,54,19,-53,-46,-72,78,82,31,-96,58,40,99,38,-35,59,46,-6,-80,13,-29,54,-62,-22,-94,-48,70,-39,39,28,58,-34,-66,-71,-35,-60,-91,-57,-78,41,99,33,-19,50,-77,46,10,69,-60,-18,-18,-36,89,-28,42,95,24,-35,-91,-84,45,67,-18,31,49,47,71,58,42,45,-49,-6,-22,32,-56,2,-69,6,-29,-77,41,6,-13,30,30,-70,-23,-45,-5,-14,-77,92,-94,5,23,7,-96,-6,-35,46,91,-31,-60,-30,-99,-63,24,84,43,95,-92,-16,-47,-5,66,84,-23,-56,91,24,-18,-86,16,-60,-29,
    -61,47,-25,-15,65,-79,-23,-66,-86,-1,87,-49,-77,72,46,70,32,-69,-76,-73,49,60,5,93,51,81,-24,17,-2,-32,88,-11,-84,-37,75,81,-64,-96,67,-50,3,54,-47,26,-22,52,-4,-90,35,-28,-62,-16,84,-5,-22,35,-72,6,-96,26,74,92,-84,42,-93,43,-25,-56,-1,-58,-54,-98,97,0,-20,-73,-48,28,-62,87,-99,-72,-77,85,23,1,-27,3,59,-71,-70,-14,-78,98,-72,-71,-7,4,-75,92,-2,-28,46,-5,24,-74,23,28,54,13,-33,7,41,-10,-55,-84,92,-30,19,3,-1,1,89,21,-1,70,3,44,-74,28,36,24,52,-18,72,-24,-40,-53,-96,-85,60,-29,-26,1,
    62,19,-31,6,89,88,-91,-59,-10,51,62,41,-27,-83,-14,-1,-2,-78,75,-50,-43,47,27,17,46,31,-16,6,-97,11,59,17,-70,-72,-77,72,69,84,-87,-41,-13,27,-100,60,-3,-62,-89,95,-87,87,-55,-30,-14,72,39,-67,-44,76,-9,-89,-13,-49,28,-31,-69,51,-59,-100,-13,-94,-89,75,-14,64,-13,83,-98,-1,78,-85,-62,-77,-63,-24,48,77,9,-96,53,53,-85,-60,56,-5,9,87,98,3,40,-15,-39,3,-40,47,-33,0,30,-30,99,8,-63,-63,-16,-25,13,-68,-48,75,88,57,-20,55,49,-64,-50,-90,75,-52,-35,-33,-15,27,71,-2,-26,-62,98,-95,-40,49,-35,-2,38,49,
    73,-97,33,77,78,21,-14,-42,-72,-65,-54,-70,97,-78,-70,-37,-11,16,90,-88,-86,-36,-49,64,-79,-37,13,-13,61,-49,88,-14,-46,22,-85,85,95,1,95,76,-12,-58,-94,-14,-84,-63,49,-43,53,39,-30,19,55,-27,-65,29,36,48,68,-50,51,56,-12,-95,-70,-44,42,78,9,38,54,98,32,60,-16,0,97,85,57,2,-24,79,73,83,-48,-92,12,41,56,-20,91,-41,89,-69,-83,-81,-13,59,97,97,-51,3,95,33,-36,-69,-67,-87,68,43,16,96,-26,-11,79,-21,-50,-56,-80,58,24,63,-82,-35,94,-65,-15,-66,46,-66,-69,96,38,78,-19,54,61,-33,67,-19,62,-65,-23,-64,
    -23,-44,15,27,0,35,-63,77,-2,-45,-58,-55,-58,-21,-21,89,-34,-38,37,56,-60,-82,10,53,-15,-71,-66,-53,-83,11,-64,-6,19,51,73,72,-61,-90,49,37,-82,43,34,-40,75,-87,-51,-59,-25,38,97,-33,57,-41,-80,94,-12,-46,94,5,17,-70,51,-63,33,-24,-91,72,-61,10,-38,-43,-95,96,-31,80,62,-29,73,89,-91,22,-91,66,81,-71,13,-30,-64,7,27,5,-11,-69,42,-78,7,3,47,46,65,-91,55,71,57,-75,3,-29,-52,-23,-39,-43,51,-78,76,33,-49,-11,55,-61,48,-18,-3,89,-35,91,63,73,-5,10,-29,-88,-29,79,35,-71,56,-61,0,-96,-32,13,-39,-81,
    35,-63,4,39,-22,-41,30,78,-6,-73,67,59,-81,31,-16,-34,93,56,-22,-35,-13,-86,-6,-57,-95,46,99,-27,-88,-88,-56,47,-50,49,-62,-20,60,69,-41,-46,48,-74,-86,67,9,-2,-67,3,-94,-36,68,-7,-22,-86,-12,-17,60,-13,8,72,0,4,-28,2,-95,10,34,66,-69,93,-80,80,72,86,47,-19,-15,33,84,-57,97,52,37,-73,18,77,-38,-21,-83,22,-97,-31,-74,-73,71,84,-10,5,50,21,-49,70,-47,-77,57,53,4,94,-62,-59,-63,87,45,26,-86,-36,4,-72,-5,21,50,-50,42,28,-22,13,12,20,70,-86,93,73,85,-53,96,-6,-100,-47,-12,-62,-54,-23,-75,91,
    -96,91,-45,-40,-81,2,33,21,53,-25,-99,83,40,-34,3,-38,-68,-4,-64,69,43,84,-37,-5,89,3,-15,-65,33,-38,-73,37,-47,-66,49,-76,37,-18,-3,-10,9,-49,-27,-51,-31,76,11,1,-76,-1,-29,68,-16,34,-85,-75,90,1,-39,23,63,-12,-88,69,22,61,45,-89,95,-57,1,4,-54,26,5,15,2,68,-84,-21,-80,-13,-53,4,74,-86,-19,16,15,42,-61,-69,82,-97,-100,-43,16,45,-32,11,40,70,67,86,-52,-76,1,3,92,-30,82,-88,9,-71,-32,-17,-57,50,51,11,-56,42,42,-73,45,-6,-16,-39,-61,52,24,32,-26,-9,18,23,15,72,-74,-40,-58,-92,-76,51,
    89,-55,87,84,95,38,-53,91,-19,89,-82,78,-65,-46,-60,-73,-93,64,59,-67,-92,-71,56,23,-99,34,35,-57,-6,60,47,-17,-95,34,-80,52,-28,-33,43,5,-91,14,-16,-56,68,-24,-29,27,-60,82,61,-52,-88,-31,24,-87,56,-41,-91,-50,71,56,-14,76,-10,6,-20,14,25,76,-80,34,-10,-44,31,-42,32,54,38,24,37,-1,73,49,20,-51,62,-24,8,71,27,-20,-21,65,-92,69,-77,-11,-16,48,-35,-44,-65,7,-36,66,17,96,20,-45,20,-43,54,-55,58,-25,-6,-27,51,55,96,-70,-65,-24,95,95,-55,-82,-64,-19,19,-47,-11,54,60,53,-28,-22,1,92,33,-78,-98,-60,
    -81,12,-33,14,85,70,-31,82,-99,-44,-90,48,-49,-45,19,-60,-11,-62,-7,78,44,-46,-16,-84,32,85,60,-83,59,14,-43,79,-73,-24,-55,64,-53,-34,-2,-52,22,8,48,25,16,-33,65,-95,-43,-41,35,53,-35,71,69,49,57,-18,66,-32,48,76,99,27,-48,-56,-8,-1,10,42,-1,84,-49,-100,-38,19,-81,79,-24,-23,90,-89,-70,-45,83,52,-96,-8,86,23,60,34,-1,60,-38,3,56,6,-97,-81,-52,-46,55,-49,-46,69,70,-26,49,-54,3,-9,-90,33,47,45,-63,3,-63,75,-74,49,10,-23,-39,24,81,70,-70,36,89,-22,90,44,30,-55,14,-48,-29,-85,-1,74,6,-39,
    -41,-95,-94,-3,9,95,-28,-13,-4,34,17,58,-42,-2,28,-12,86,-31,19,76,65,-99,73,-69,-47,-56,-54,4,70,5,65,-70,10,71,-21,71,18,3,-41,-85,-62,76,73,-4,-74,-47,37,-88,-26,-44,40,-61,-91,-34,-29,-38,-38,-31,19,-67,74,-16,-37,-15,-92,-6,-44,26,97,-33,41,-65,95,-34,-16,-79,-29,-79,85,45,-71,-22,37,-62,44,-92,-48,58,-23,71,91,-48,-44,-94,-11,-84,0,45,42,-2,-87,-64,-15,60,-46,69,82,-22,42,19,-77,71,97,60,-91,-7,20,-38,-96,98,85,47,-98,-59,54,-9,-43,-94,88,52,-44,53,40,-6,14,94,-37,-4,-28,-94,67,-52,29,17,
    60,39,-90,-67,53,-86,-69,38,62,85,32,-32,-24,41,-74,-36,-55,-65,-82,85,29,32,32,44,80,56,2,-1,-96,-68,-84,17,23,-73,50,-72,93,33,18,7,-30,-98,27,46,44,6,62,89,41,32,27,-78,-84,-41,18,48,-85,-79,-52,72,-95,-36,-59,-20,-57,43,-40,89,-72,78,-4,-2,81,76,96,-23,-18,58,-82,-77,-57,45,97,-89,56,-33,60,24,-60,60,-52,-55,-76,-11,-23,-80,-68,37,-91,-88,16,-43,10,-51,-67,6,-22,15,-36,-4,-10,-41,94,-61,-29,2,7,83,-74,47,43,-26,45,67,15,-78,39,-1,60,0,11,-72,58,-79,-71,43,27,7,-41,44,55,1,55,1,
    41,-74,56,-100,-91,82,47,52,-91,-8,-28,-24,-33,63,76,79,-36,-61,-93,-78,-87,88,65,-8,-53,76,-64,54,-70,44,-44,23,70,-88,23,32,46,-30,-64,7,15,-40,84,82,-76,12,13,-12,3,20,-38,16,-40,79,-91,-41,8,97,-87,38,41,-79,-39,-36,85,84,96,-16,-94,84,91,-27,-55,27,-93,-79,-61,20,9,-5,92,23,-89,-96,2,-80,-37,-90,70,-23,0,11,-2,61,75,-64,-3,23,20,-44,-40,11,29,5,91,-11,26,82,-91,87,-23,54,-38,-11,58,64,-39,-78,27,-69,-49,-73,43,49,41,-30,37,-62,46,-43,-6,-94,21,76,-37,12,-35,41,-6,-74,-20,72,32,42,
    -87,-9,-42,-26,-35,37,6,16,65,-99,-83,-42,-29,55,-52,69,-88,43,75,33,-29,90,97,-12,-17,-8,14,63,-84,99,-43,-71,-10,16,3,55,-47,-91,-29,70,-90,-12,80,-66,95,-71,-97,-92,72,31,-7,95,-27,-57,-17,57,35,49,-28,3,48,30,32,90,98,87,45,-97,97,68,26,59,-91,6,45,-96,35,49,64,-41,-68,-90,-46,5,-47,-11,-86,40,91,-13,-57,-9,69,27,-18,19,14,79,22,11,48,-52,-77,9,7,68,13,-58,-31,-70,-46,1,-60,-40,59,-55,50,25,85,-7,-88,80,84,33,7,-34,-48,-79,98,27,-15,-2,27,-40,-93,-66,-72,-28,-71,-50,2,-17,3,-6,
    43,-38,39,-55,88,24,38,52,4,23,-62,63,41,-58,85,39,21,-78,-63,-51,-18,-56,-65,-38,-31,-84,12,71,-1,-84,-34,95,-70,57,92,70,-18,31,-25,38,-94,-35,-46,-1,7,-61,-61,81,-39,-72,82,-5,-75,-83,57,-6,-66,-78,-83,-15,90,-65,-68,72,93,77,43,-73,8,70,17,-34,-65,71,65,-6,62,56,-25,75,37,-43,-30,62,-73,80,-92,-87,-98,-75,-50,92,-87,-65,64,58,-88,-41,85,-28,-71,-46,-10,-84,-74,-93,-37,88,64,-10,16,-47,0,86,67,27,66,75,-8,20,52,-6,-36,17,29,-71,-25,-7,-60,-88,65,-78,-33,-93,-62,-7,-85,-47,-67,31,44,49,84,-56,
    -12,51,23,-94,26,67,27,-70,-39,91,48,43,72,-77,-64,-87,-64,54,-65,-45,61,-75,0,76,-21,33,7,75,-65,-57,-81,-25,94,-6,-19,-28,-39,60,-45,22,52,-97,-83,-24,78,-46,-59,66,60,-24,-79,21,-46,21,-50,-15,-93,57,-40,-58,-99,-69,-31,47,25,50,-80,38,11,-73,-88,-85,-70,-70,-9,60,-64,33,27,96,61,48,17,67,-30,67,-96,-23,-23,-36,-29,-70,-53,40,77,72,-58,49,-38,53,76,-25,20,58,5,64,19,41,-51,-54,-11,62,94,-94,-18,-84,26,-14,-7,-97,51,16,-67,98,-44,62,23,-1,12,37,-96,-60,-88,-23,99,-31,93,18,62,-58,64,-49,56,10,
    -90,-62,27,36,25,72,39,-72,-11,24,-74,-3,-14,1,48,50,91,5,91,-97,82,90,-75,75,-40,87,69,-76,-61,25,-14,-51,16,-87,37,-59,38,-72,-31,-73,52,-53,-24,90,1,-75,41,-8,30,-16,-5,-36,-74,20,91,-14,-40,60,-38,-49,-63,48,-48,53,14,89,94,-48,-83,-85,-69,21,-37,7,-37,16,-16,-96,-92,-86,-60,-45,-22,-34,28,-31,52,88,81,-86,91,-81,15,-57,24,-71,-68,-29,-67,-99,-14,-36,-26,1,23,37,-83,-92,-6,77,74,34,-67,-47,53,61,-26,5,1,-92,-28,-8,79,87,35,3,-84,19,74,-99,-28,-87,-83,46,-34,40,-65,-64,0,29,-87,-25,-84,-54,
    -20,69,59,6,-26,-40,14,-2,52,-7,-15,-61,49,-47,58,-25,-46,82,-60,-29,28,7,64,64,43,-36,-55,8,-9,-39,7,23,82,-34,-70,-91,-21,-4,7,83,42,93,-77,-9,98,33,66,-47,-32,-93,24,96,-34,-60,12,61,5,10,-31,48,23,28,-28,6,95,-46,-33,-74,2,74,-91,-56,19,84,35,18,-30,-46,23,38,-87,-53,-14,-21,-60,51,92,-3,-39,-39,-55,36,90,-31,-58,37,23,-39,15,-74,-64,24,-78,7,-39,10,-75,-17,64,48,-79,29,48,-41,-92,88,-90,-100,-15,-77,-87,-18,60,55,52,54,-56,-73,-84,59,-95,-96,-64,-72,11,-51,38,-11,32,-46,89,53,83,37,
    -88,43,-23,75,-5,14,98,8,49,10,-84,1,-35,-88,28,-67,72,34,-63,60,14,0,-91,-48,89,41,6,79,-54,-11,-32,-90,-68,-2,85,27,-88,36,87,61,-54,55,-86,-37,68,-5,-52,-8,81,-15,-48,-5,38,-87,47,27,54,-95,6,-100,-6,27,62,-22,-75,48,57,89,-16,-4,3,-18,-48,17,-2,72,-88,46,-36,45,84,-32,-60,22,-19,87,-99,87,44,-40,39,-62,87,1,-32,-88,-99,77,53,-15,-26,-44,20,-22,26,70,-50,-10,16,-34,-64,-48,-14,-24,74,67,-84,76,6,60,-64,-55,51,75,-2,-81,39,0,97,-8,37,-77,1,-43,1,-73,-73,3,17,-4,69,53,48,-45,
    82,-25,74,98,-49,-20,10,-61,-23,-39,-86,75,-67,53,-25,82,97,-87,57,-2,-78,-42,25,-98,13,95,-2,-66,0,98,-59,-18,-27,67,-68,24,-53,43,15,-24,56,29,-97,41,-66,-69,23,-68,-56,-20,30,-82,-10,-92,-80,-97,-97,-30,89,3,-31,-70,38,42,97,70,-81,96,-35,34,-76,-78,-84,-72,63,-50,-41,-61,-66,-45,-29,17,-75,62,25,-54,17,-20,-32,-41,83,-11,-11,-27,-68,-61,-56,-97,-65,9,37,-40,-17,5,-60,99,56,-1,90,42,6,-39,-41,31,75,36,29,-55,68,98,4,52,-13,93,77,-29,-68,-79,-26,-80,-17,64,-68,18,-31,-28,17,77,23,7,20,-71,21,31,
    60,-4,-80,-10,41,88,-60,-55,-8,-21,91,-30,3,75,-57,29,95,26,-7,79,-55,63,-49,14,-8,74,74,64,-97,-5,-4,16,91,-32,58,-15,-44,50,-18,1,81,-27,71,-16,-99,-86,-86,48,-7,-41,28,38,74,79,-96,-33,6,78,-69,-39,-27,-21,77,-83,47,-13,2,56,-63,84,-43,71,-90,80,-45,-89,94,-79,59,-61,-19,-13,29,-93,-81,34,-26,-23,12,-42,38,38,37,-32,55,-63,55,-91,45,45,93,-98,-32,55,82,23,66,28,-3,-22,68,30,17,-3,37,36,83,64,-87,-4,-78,4,86,11,72,41,0,-21,50,-55,76,-5,99,-56,-97,81,20,21,-38,-83,99,82,-1,17,
    -69,-64,-95,-85,52,-81,-37,26,-25,-51,-62,-53,42,-10,26,-56,36,-45,-9,35,-1,-6,-31,71,-84,-17,40,-85,-35,-61,-16,-4,-72,-10,-37,80,-39,26,7,36,27,-3,35,-31,39,-87,13,-25,-32,-95,63,20,-49,84,-9,-33,-33,-68,-65,-16,-29,19,32,-1,61,96,-68,-78,-26,39,10,-98,88,-55,-77,27,-41,-11,55,27,-6,-30,-53,45,-46,-9,65,-27,23,-100,9,46,-29,-59,-54,-67,89,-70,-45,-36,69,18,-82,9,63,-59,-64,-78,-18,43,2,76,-87,1,-26,-81,92,91,-8,-33,91,-99,-86,-38,95,-88,-53,84,-58,-45,-100,63,73,-82,-28,36,-88,-40,11,-6,-96,65,-77,-83,
    -82,-51,-63,-89,-60,-19,-70,83,-17,-56,97,-22,-44,-55,-86,-2,0,-85,-39,-75,85,-15,13,49,46,-76,96,50,41,-81,67,60,-80,56,23,-40,-62,53,-57,-27,98,-60,3,-94,-63,17,-43,37,-16,-30,62,-30,56,-24,71,2,-48,67,-96,94,38,71,6,58,-20,-71,70,18,34,65,43,32,-94,-54,-9,95,15,48,33,-100,-82,-53,-78,74,-25,93,-72,28,61,-68,-26,-1,56,32,-90,-64,61,80,6,-5,-2,-51,80,-44,-53,-77,-49,62,71,84,62,41,84,84,-32,-41,78,-4,39,-9,29,65,42,-63,-3,4,-27,-90,-63,31,-42,35,32,90,91,79,-87,-58,41,84,79,4,-23,63,
    -60,45,74,70,-58,-34,-39,23,-69,56,60,29,-40,85,-9,-3,16,49,-68,-52,39,-77,-73,52,-82,20,-12,97,-76,66,12,-83,11,-14,87,5,52,-99,-72,36,57,40,-35,-83,25,56,-33,93,-42,-1,93,97,-25,-80,2,-7,-7,90,-10,-31,8,-98,-14,-80,-60,-26,25,45,-73,6,-19,84,-54,-2,53,72,-94,-80,17,-36,20,11,-86,95,-17,16,-12,76,-42,30,-54,67,-16,32,39,24,58,-84,-79,85,22,-46,21,21,4,-25,-7,11,-5,-90,27,67,-27,41,14,57,-91,54,33,-32,-64,79,87,20,-36,26,97,-78,-6,70,-92,69,-75,-19,-10,29,56,83,92,-96,45,20,23,-81,
    -39,90,28,-77,-4,-39,-9,33,93,-70,-47,-43,56,-98,-21,-98,73,-61,71,50,21,-39,-21,-71,96,-28,-67,42,44,9,13,-95,-1,-59,80,95,-46,-29,-20,-53,1,34,4,-91,-12,36,-88,-39,75,83,-89,-52,97,-57,-22,93,-33,-37,-13,-89,72,52,68,-77,45,49,71,-100,72,-49,-53,-26,37,52,-65,78,-12,99,-61,-85,35,-97,-36,32,98,-6,25,-35,9,65,-24,-66,17,96,-43,-37,45,80,-37,18,84,-38,44,21,66,31,-1,-94,31,-9,-78,-34,-54,-62,50,-56,-16,-73,-39,93,92,-11,-73,-38,-15,-63,25,-69,-83,40,1,-99,-98,-55,-25,69,76,26,75,-41,69,49,-23,-85,
    39,-73,-89,-77,-93,-28,-83,51,61,96,-87,-53,-67,-10,-70,-97,-70,31,-96,85,76,-69,6,4,58,-67,64,27,83,93,-5,-26,-79,6,-2,80,31,-33,31,-8,-37,-3,-9,-3,87,21,0,70,-48,-44,7,80,-60,-87,-63,50,-54,-99,-71,81,94,24,-44,67,83,6,-1,14,73,-69,-94,-64,28,-2,-15,-33,71,37,89,-76,94,-4,56,-66,9,93,36,8,46,65,-59,-7,-10,-51,-88,-27,55,12,87,-20,43,45,-31,-77,95,54,-58,67,-8,-68,43,-62,28,-1,-76,90,-55,60,-50,91,25,-9,-64,67,-7,-51,-60,-100,61,79,-19,-44,-23,50,-69,-28,-44,-27,-61,48,-43,82,86,86,
    -66,-90,28,-21,22,78,-78,-52,-31,-41,-85,14,-40,-92,15,-79,87,96,-23,-84,-2,-40,-11,54,33,-20,-45,43,-85,-59,29,-51,-96,-43,-20,26,87,-46,26,-44,-35,42,71,25,50,-62,46,-11,-66,75,6,-16,87,47,-62,-27,-73,-55,-84,-58,-61,-3,-57,95,54,-25,21,-7,30,-100,1,95,42,-28,-27,-56,-38,71,-15,-4,-1,43,32,-14,-10,23,59,-30,68,75,12,-93,-28,8,-98,78,83,-24,-29,13,-24,73,-39,70,97,34,66,-40,-43,-97,8,56,-1,93,-57,41,16,54,-89,-64,30,-24,-56,54,84,98,-15,67,-74,56,-67,-46,-19,-54,76,-69,32,-6,-57,89,-2,51,46,97,
    -4,41,38,-88,-5,2,-51,77,78,-55,32,14,95,-83,-19,74,25,-34,-72,-93,12,57,90,-56,-49,-67,-66,49,36,32,-2,-67,-27,-63,97,68,-9,-54,46,-31,-57,-70,83,91,-53,-84,65,72,-65,93,31,47,50,21,92,54,6,78,3,-57,10,-98,-24,83,91,-27,51,-66,-28,-51,3,-33,-21,-62,-42,-22,-94,-77,-97,-59,69,34,41,-29,-92,33,-75,14,11,29,9,-79,-17,85,56,-74,-89,-41,-40,-17,9,15,50,40,5,61,-81,11,84,-78,5,53,8,-54,-75,16,31,-50,83,94,31,-8,-85,66,-70,23,92,-59,82,-48,-24,-9,19,78,-68,-76,39,-49,-12,-24,-75,93,-71,33,
    91,-94,-50,22,-91,85,16,40,-71,-17,-41,-41,-94,-49,-48,40,-44,-20,-68,75,-89,-84,52,50,-81,-60,-74,-56,85,-92,77,76,66,79,-50,75,16,18,-32,46,-99,27,57,59,78,-90,-1,86,-58,-17,62,-47,-49,-34,56,-78,6,82,-34,-9,42,96,-81,9,27,-31,-16,-56,87,52,42,40,-69,-1,-49,-90,-39,2,96,4,86,10,57,89,76,13,-88,-66,-52,-70,-23,90,78,-4,-49,-42,65,-12,2,4,-60,-56,96,-28,-5,-53,-66,57,50,82,-39,-12,-55,70,-71,73,-64,-59,60,-16,-76,-63,-74,54,34,-22,-88,-49,66,-34,56,58,-38,4,82,58,52,68,-33,-46,3,28,42,48,
    -50,71,-27,-14,65,-67,70,41,-29,49,-5,57,-73,-40,-92,45,26,16,-45,89,-79,90,-1,25,58,66,79,-39,-54,21,61,-4,-56,87,83,-91,-80,5,2,-57,-46,50,-100,-67,-90,-91,78,36,25,86,-23,-2,76,76,-77,34,-6,-98,48,40,-25,9,-63,20,-4,-80,-19,-31,-23,84,12,32,34,13,65,-4,-26,-4,-68,51,-18,-38,-50,-42,90,-75,44,-15,-72,-8,-75,-45,54,-38,75,-98,-66,-91,-77,12,45,36,96,-69,-99,13,27,-73,-91,11,78,-57,73,80,1,64,-94,98,-51,86,-10,-74,-59,96,-11,69,51,-25,78,74,39,23,-38,35,54,-85,-99,-67,-58,-90,44,-79,-46,18,
    -99,7,-66,-93,5,-65,45,48,-39,87,-4,2,56,-53,78,-14,-78,-31,-91,-64,57,15,-48,58,48,-6,68,-56,15,74,14,-31,-18,48,-72,39,83,74,-13,97,13,84,-1,-79,83,-71,7,57,-1,16,94,-92,83,-54,66,-17,-8,-14,27,60,-39,-58,-71,-5,90,-91,-66,26,83,-26,23,-52,-90,74,-31,93,-96,76,51,3,-56,-3,-37,-73,-5,-71,-38,87,15,90,47,28,-68,28,75,74,38,62,-48,-27,-64,27,22,-54,2,91,91,6,20,42,61,64,91,-76,44,-14,-95,-94,-26,-28,96,73,-99,80,-46,28,7,44,-10,59,69,26,-13,-9,24,41,-65,-32,-53,7,-90,60,-29,54,
    36,67,-60,41,74,-34,-87,22,92,66,3,-54,-5,-90,42,85,21,11,-36,60,55,40,1,42,-92,48,-51,71,-40,-28,25,96,-60,17,37,66,84,3,-12,76,-31,43,74,16,5,16,-46,-73,-73,-30,87,82,62,-59,24,71,89,25,-6,-98,98,71,98,90,-12,-12,56,-76,43,96,-100,64,40,26,-19,-55,-58,-13,-76,-30,57,64,-96,19,-95,-19,-58,46,-94,-12,48,-96,-41,-1,46,0,39,2,-24,-18,-1,-23,98,91,55,-69,-64,98,18,-87,-80,-25,77,-76,95,82,5,89,-20,-36,-22,-71,-80,37,-20,67,89,-81,-31,-34,-47,20,-5,-97,11,-50,-65,0,-100,53,13,-80,-71,-10,
    97,76,-76,2,17,4,18,95,-15,39,-15,17,6,-26,-12,75,-8,-7,-52,87,-3,59,90,32,-41,90,-63,72,63,-34,-86,60,94,90,-86,12,-53,33,-41,32,-28,44,-98,78,71,-10,5,-37,84,5,-97,-19,-35,93,65,-24,35,-98,-51,-2,-79,-85,-90,67,-94,77,-21,-95,10,39,-11,34,-65,-9,-36,-94,82,21,-30,66,-73,-27,99,-8,-82,64,68,5,66,69,56,39,37,66,7,-5,43,86,0,-95,-23,89,39,13,-19,-97,-29,-37,25,41,81,52,66,80,96,84,44,16,-10,-38,-14,-54,54,-77,64,-39,18,8,99,70,13,-23,59,-95,42,-60,-92,-87,-45,-15,7,-64,89,-27,
    68,85,-90,12,2,0,27,40,-2,-19,15,-38,42,85,22,-7,55,88,22,14,93,-84,7,53,-70,62,-61,-11,51,80,62,19,18,72,84,72,-76,-37,12,-26,44,-73,-11,-62,64,-89,-17,19,99,-94,85,44,-78,-8,-2,-96,-93,-11,-7,58,-79,-92,-71,39,32,-87,11,9,-24,75,83,72,54,-28,62,18,-64,-54,-63,87,-48,-77,32,-74,-33,82,-69,-26,-77,24,84,-56,-16,-86,84,-83,-73,-53,26,-44,-77,9,-72,77,34,43,-4,70,41,-15,9,45,8,93,71,28,75,-98,-98,50,-21,87,95,63,-99,31,32,80,78,-42,88,-99,68,-83,31,-46,12,-73,76,-47,12,-15,98,73,
    -21,21,1,-94,-24,-45,57,55,94,4,-30,47,-65,-97,28,-87,-39,16,67,81,-15,-2,87,49,-23,63,-98,41,-99,52,-34,-68,-74,67,-62,2,75,47,-91,69,-49,-69,17,-62,-66,-55,-96,-52,13,71,81,51,-79,69,0,-50,-16,-45,91,-15,59,-42,17,85,77,8,-61,52,-93,-52,22,-41,-20,91,-51,-34,-64,-47,-34,1,76,-52,-96,97,69,5,-53,-95,12,91,-57,-29,-51,60,-91,78,-80,-52,31,-72,-51,-95,-61,29,96,-12,47,84,42,-86,-63,18,-86,-58,-32,35,47,-33,40,-41,-42,83,-18,-41,-4,-57,-62,16,-8,21,-4,41,26,35,-78,-26,-76,69,-42,18,35,-5,-64,49,
    37,-44,-16,-64,24,77,-53,82,60,-18,94,-92,25,-68,25,69,5,73,-38,-69,9,-16,-43,-15,-94,15,-97,-7,62,-61,-57,-48,-52,-21,-60,72,-44,-12,6,69,22,0,77,-53,-16,-46,-83,89,-72,-21,72,-11,16,-71,74,-26,-4,77,67,-41,68,62,-37,-84,94,3,88,-98,43,-53,71,-35,-1,1,13,84,7,-18,25,35,13,-2,76,81,-21,50,-45,76,-21,75,87,-52,-11,-50,-84,-17,5,57,-14,-51,-96,-91,14,3,-38,79,-61,70,-39,65,-43,75,-85,-66,56,-6,36,64,-78,-84,-9,-91,-36,80,-89,-68,-84,-31,89,-46,18,93,-85,-68,-51,-22,-88,-12,-52,-75,-95,-43,52,-80,
    -9,9,-33,-72,-75,-11,96,16,-49,-88,-52,-86,-56,-36,-17,-66,18,53,79,34,86,-72,-88,50,17,-88,27,22,-31,-20,-5,13,41,62,-7,66,-97,-11,-66,-94,-47,-18,73,-3,47,-44,-17,-83,-90,-37,51,48,-9,15,-50,60,-21,29,-65,1,-91,30,66,50,-56,-41,-32,99,0,2,-94,53,37,79,-98,-16,87,-14,-47,49,1,57,-51,44,-76,-49,5,4,81,-8,57,90,-78,23,-7,-82,82,13,-83,-66,-84,-77,39,5,54,41,41,-6,79,94,95,-68,51,-55,77,76,-52,-66,-68,-71,26,89,72,0,12,-83,18,-54,30,35,-20,-2,59,-81,-97,-35,12,44,59,-56,39,7,-24,-58,
    52,-95,70,52,-61,-98,82,17,91,6,-83,-45,23,35,1,-47,-29,81,52,82,52,7,-53,-83,-48,59,-39,43,-34,-11,37,70,-53,8,22,-62,10,-44,56,-46,62,73,9,-63,-39,-89,-57,32,44,47,-34,-3,-46,-35,14,-42,-76,27,-99,-58,68,39,64,-85,-53,-61,-94,9,-53,62,63,62,-13,-27,99,48,-16,-58,-68,80,89,-2,-23,96,64,43,-46,40,-78,-92,35,91,-1,-49,58,-2,-10,64,-93,-10,-22,71,-48,18,96,-97,-34,-68,98,-49,-88,39,49,-10,35,-35,85,42,-94,60,50,-7,3,-51,44,-39,47,87,78,6,-23,-92,-23,-19,26,-75,-64,45,-43,-66,96,-30,74,-3,
    12,-39,63,-51,-97,69,-91,53,-86,12,54,58,26,-47,-3,4,60,74,-36,89,-93,91,-85,-56,-12,-28,-22,-16,-6,-96,81,-94,-34,44,-44,-79,-35,-83,27,-21,30,-19,90,-44,35,-13,12,-5,-86,76,-16,-79,19,-49,-83,-93,24,-52,-9,-82,-48,-75,77,-30,21,85,-56,87,-98,71,-82,-16,-96,-92,40,39,-52,-48,86,-38,81,-77,-65,-48,-26,-47,-40,50,1,3,69,5,-20,-2,28,-98,-65,72,89,-63,43,59,22,47,20,-86,-61,68,-81,77,82,52,-100,69,-96,-73,74,-36,77,-73,20,-2,33,-100,96,-39,-98,83,85,43,-79,-20,-45,-5,79,75,-39,-82,-5,-20,48,-23,84,0,
    98,89,27,73,-95,5,-100,25,-97,-15,-22,52,-2,-20,35,-17,76,8,63,-69,55,43,-94,69,13,-47,-99,-87,-18,86,14,-20,75,-59,-47,80,-2,-94,6,-46,-9,84,-94,90,-84,-7,-27,92,54,89,75,-39,-16,33,-70,49,86,-68,63,-32,-82,-23,49,45,-30,54,25,69,60,83,-77,-48,19,81,-58,-12,-74,-33,-68,80,-44,-92,94,40,41,76,-10,-20,-92,-95,48,78,-18,-51,-25,52,-96,-99,-27,16,36,-4,68,-44,-23,62,-4,-44,82,28,88,-62,36,82,31,-70,59,73,10,19,30,-90,-50,-88,-88,25,16,68,-22,42,-16,-85,-62,-95,-77,-32,19,19,24,1,-53,12,-8,36,
    -53,75,18,58,0,80,-23,-70,90,-21,94,-98,-95,-38,22,-17,-96,-93,-50,-57,-88,-27,-37,-17,44,39,85,92,-49,-23,-20,-50,-96,98,-92,4,78,-14,34,20,17,28,-25,22,90,97,58,95,-96,60,90,68,-66,-95,-48,78,44,89,22,47,-34,54,98,-30,52,-94,74,30,-56,-40,-97,62,88,-22,-64,30,-25,46,-23,-68,7,-81,-100,-7,-76,-96,-77,20,93,46,-32,-89,-48,-82,-19,-95,-76,-93,-13,-79,-81,-10,35,59,68,23,-58,96,-30,19,-72,29,-9,80,-78,-33,85,-55,88,78,43,-92,90,96,-74,-77,-99,-98,83,-60,-77,54,31,10,14,-49,-66,56,-53,-44,-73,-25,85,18,
    -44,-93,86,93,-96,26,71,48,-14,13,44,12,-11,-3,-34,-76,-11,-58,78,-80,52,-8,-28,38,-52,-29,94,-72,47,79,46,55,38,-16,48,43,62,-29,-9,-52,37,87,12,-74,-64,79,-50,25,-79,-72,-2,73,-27,22,12,-27,93,-42,1,40,-62,-100,-53,76,-16,47,19,99,-81,62,-53,8,49,12,34,37,-9,36,-37,12,16,13,-63,89,-65,1,-85,-72,60,16,21,98,16,68,26,53,-32,98,52,-13,60,51,47,-38,-37,-19,99,6,69,-86,-82,-63,27,-92,-73,62,9,94,43,-31,-38,-36,-81,31,84,46,-16,-48,-4,88,-9,-44,-9,-62,18,7,-29,-30,-87,-60,-16,84,-22,64,
    92,-43,-22,1,-49,-79,23,-35,-15,42,96,-30,-60,-20,22,36,-80,66,45,12,-96,-37,71,-24,-15,84,-32,70,68,-2,-14,12,55,64,-86,58,86,-11,-76,-29,83,-80,-7,-76,-47,68,12,-75,34,-43,-11,38,-27,-40,66,58,97,-65,-20,65,33,66,-22,-59,-17,-56,51,-31,33,27,40,-32,48,86,-8,53,54,57,78,-12,66,-80,78,91,-20,45,2,-23,32,34,95,-83,-47,-75,-90,36,-79,62,-95,6,-11,-3,74,89,-17,-81,-6,-63,76,-75,-23,-6,-55,-44,-14,25,53,88,55,-63,22,50,-94,27,75,-83,-37,96,31,-32,-46,72,66,-72,14,-99,-53,-92,91,-77,-67,-32,18,-22,
    -24,56,56,81,-4,11,70,70,-87,77,-2,40,46,61,-12,-71,-70,42,-99,48,70,15,49,-82,24,-8,93,9,61,-37,40,89,-81,-4,71,15,59,41,-14,24,-30,84,-36,68,-3,52,97,27,-6,51,-73,-84,-34,29,-66,42,-79,80,-48,34,-57,44,24,-85,-60,-5,30,51,88,68,-25,-89,4,-9,31,-98,-57,29,-19,-11,-20,9,-43,-2,-62,-56,-59,-89,-76,-55,46,-33,89,-30,-18,81,17,65,-16,-43,33,11,-32,-62,-98,-100,92,-55,-19,-27,-14,13,82,43,-89,72,87,-96,84,63,-51,-18,-69,90,52,-35,23,-79,82,-93,-22,-32,18,-1,-42,-28,-49,-50,17,32,23,55,-55,6,
    -49,-92,-70,-62,-87,-86,-98,-86,-52,85,57,-100,-98,-20,-79,37,-60,-48,-95,10,51,-37,83,-98,13,-48,-14,-64,60,-17,-6,-89,-9,25,-99,-44,-9,55,-77,40,-8,80,-8,-5,-88,-34,84,-96,18,89,15,-31,-96,-50,-77,17,-46,9,5,-86,44,52,-75,87,-71,79,96,-28,34,-81,64,-73,51,-43,-26,63,23,-42,20,-59,-53,87,-38,-97,-63,-63,20,-9,-2,77,-42,-58,-19,-65,81,-38,-86,77,35,49,96,-1,76,-1,56,-98,-85,-69,-40,35,-76,-41,22,-62,-38,11,-25,34,54,73,-37,-88,67,45,-52,49,-93,-38,26,42,63,-25,94,-9,26,50,-7,41,34,53,-24,58,-88,50,
    97,-26,-39,-76,-92,-84,98,-76,-72,17,69,-72,-82,28,-57,45,-29,-94,72,-83,-2,-2,67,43,-60,-47,-3,-32,-88,-39,71,61,-12,-68,85,-52,-100,35,-28,81,-47,93,-91,-29,22,-96,68,45,-89,-8,62,-39,43,-19,-96,35,35,-47,-97,47,-33,74,-40,7,59,-55,55,59,81,80,-8,86,73,-46,-91,-5,-42,30,-60,-79,-78,-98,-18,65,-16,-61,0,-81,-56,-44,18,-37,82,78,70,-7,75,26,-95,8,6,-3,-6,-21,51,-44,27,62,86,67,83,60,22,-82,-22,58,-43,78,29,-47,86,99,17,21,77,-13,66,4,65,71,-87,-77,-79,-41,-45,24,15,82,-14,-47,1,-30,-86,75,
    -12,-8,33,-3,-78,-38,2,61,-39,-81,-66,90,-41,-100,-5,-76,24,-40,-52,45,19,3,-79,87,-63,8,-60,38,30,-94,14,70,-50,99,-81,-27,-86,21,86,-25,-7,-80,-82,52,-80,-35,28,44,25,28,-59,-56,83,-37,83,72,23,-76,11,-95,82,77,27,-67,28,46,6,42,-81,-8,70,-36,-36,88,68,-16,-47,-51,81,78,77,74,74,13,-63,58,-15,12,34,48,-31,16,-23,96,49,-94,-6,-93,48,14,-49,18,30,15,-42,-1,52,11,48,33,-59,-23,-41,-84,90,-51,26,-24,61,-40,-76,-69,76,2,-73,-22,8,-26,-15,-92,40,37,-21,-30,-96,37,21,56,-99,-31,-59,-58,47,1,
    -90,-11,50,36,65,-37,96,42,-6,25,-56,-26,-97,4,48,-60,-36,-12,29,43,58,34,-19,-20,-10,-66,1,-16,-24,-100,85,-13,-10,-13,75,-93,50,24,49,-55,-51,45,-29,-96,-99,-81,-56,-34,-41,74,-91,69,-40,-58,-99,2,-24,3,86,5,-45,71,44,-3,-42,19,5,61,-57,-94,-94,-56,52,-23,-52,53,-52,-55,71,7,-81,81,-72,-69,75,-70,33,-96,-15,20,9,40,-57,-47,-62,-46,24,-5,-85,68,1,73,64,-47,-98,-87,59,-50,-42,30,9,-71,-37,-11,-40,-61,19,93,95,-96,-35,4,-3,-39,9,35,15,-67,82,-18,53,-17,-45,18,-11,-91,-17,-52,-89,41,30,20,22,46,
    61,82,85,-19,27,32,-63,45,-64,34,-94,-3,-79,-27,30,55,55,-64,-61,62,6,28,71,89,28,82,-18,-42,-46,-96,-96,-85,38,41,-52,-35,-27,-14,62,-39,72,68,58,46,93,-59,1,48,-23,-8,-38,83,-80,-67,24,48,-85,-94,59,-79,-90,63,-63,48,-43,-15,65,30,23,28,44,-52,-52,2,46,94,-5,-53,42,72,40,57,7,12,90,-69,-39,58,-63,-80,79,99,35,68,47,92,-94,-35,-25,-19,45,71,29,45,-27,-25,39,69,-25,-18,-7,67,-61,53,-69,81,84,-8,39,74,-36,-29,73,52,91,21,-4,97,86,71,-21,-17,42,60,-72,68,88,68,-11,-85,-98,34,34,93,
    -13,65,74,-28,-90,-34,98,-26,-11,71,78,-20,-8,-25,-70,-70,98,-39,13,-7,-79,42,-87,61,-38,2,76,-36,88,-38,-43,-24,-72,-69,0,-62,-51,-2,-36,38,-31,-57,71,14,18,-99,-4,16,-38,62,61,-65,4,74,49,18,-24,-23,82,-83,40,-61,-7,68,-78,93,-94,-76,-57,-30,62,12,-35,-15,78,83,-14,-25,-48,0,-63,-87,-12,-7,88,-63,-89,-84,14,45,-67,-94,84,-22,74,58,-77,32,82,-34,55,-3,31,-80,82,-39,56,-31,36,-92,21,25,73,-91,-82,13,98,81,30,13,26,15,-29,-38,46,-54,-79,69,30,-97,88,-15,0,19,6,-17,80,-86,4,-31,-78,-75,94,47,
    -65,65,-39,85,46,43,-2,-75,-90,70,-13,56,68,8,26,-2,-36,14,36,16,85,94,99,17,-92,-45,38,82,-19,33,29,-32,-50,90,-47,48,-15,4,73,-4,-74,-39,-48,46,21,30,96,85,-56,32,-46,81,26,-47,-49,34,-39,89,-32,-6,-26,98,14,24,40,19,-27,-74,-77,-54,-26,-99,-41,26,47,33,-91,44,-30,-47,-72,24,87,55,30,-62,-59,91,-73,10,-15,2,-40,99,78,-100,18,-49,78,-6,-50,-48,-5,-39,31,95,94,40,-9,-35,45,19,41,-16,-74,-29,22,-32,-38,2,30,-1,-96,90,-2,82,90,69,86,21,-85,88,25,-38,-51,-44,-43,-56,48,0,-39,-54,72,-98,
    30,98,-74,-95,-82,-12,7,48,88,-37,38,-62,97,-19,59,83,54,74,71,-21,-63,73,-12,-54,69,-64,47,82,-66,19,84,65,-31,-90,70,-12,-49,-71,-64,91,-8,27,81,-11,-92,41,-75,62,-33,48,93,-44,-27,-19,3,-58,-30,-98,-76,4,-27,61,-79,42,23,91,-70,74,72,-33,17,-36,94,-1,6,54,92,-17,-32,-89,83,-87,-32,-43,-5,71,-1,65,73,-24,-79,-54,-63,-57,40,60,86,71,35,59,90,-96,-77,84,3,-19,-10,-53,16,10,-41,0,23,27,57,70,-50,-92,35,75,-16,57,73,21,-100,13,-66,86,-16,21,-3,-26,25,21,-90,81,54,-48,-72,-29,62,-61,23,-62,
    66,-20,-92,-84,-60,-4,91,-23,-95,64,98,57,-22,-68,43,14,5,93,89,-69,14,51,-88,-80,-96,92,-57,18,-68,66,56,50,98,17,67,-61,-87,10,16,-82,-73,-34,75,-43,-49,-30,23,56,-37,12,39,29,-84,51,2,-80,44,45,38,28,12,-53,-22,10,64,97,1,29,8,-83,47,-13,-64,-26,-4,-61,44,-81,95,60,-16,35,-59,0,-62,43,20,34,-11,-90,-38,-99,-91,41,-37,73,38,65,2,98,34,-99,-63,70,-25,33,-91,72,5,57,32,89,-56,73,-59,-18,-31,61,-83,58,23,-69,11,33,24,74,-42,-85,39,13,65,-26,14,-97,96,42,-12,58,-34,93,15,50,34,59,
    -25,75,93,44,-12,62,2,64,94,-87,-51,18,88,7,33,-21,72,-1,53,-61,-46,2,81,-6,-40,99,-60,75,49,74,-14,-76,2,-21,69,42,-58,-77,6,88,-63,55,-42,77,-37,92,-44,87,-57,62,-74,-51,-36,59,43,24,58,83,-49,7,58,-11,32,-88,68,-47,54,62,-24,-39,2,-35,-84,61,-58,-69,5,-49,71,48,-87,97,97,-71,57,40,53,15,-24,56,-77,-66,-55,7,98,65,12,52,-20,88,65,34,-94,82,-5,-52,65,-100,-1,36,0,-36,-14,-3,-7,-57,90,98,10,-34,54,33,-48,51,92,-50,-31,-44,54,1,97,20,-65,-97,-46,31,-49,71,-17,3,-40,84,67,46,
    -19,-39,89,-77,-41,-1,-59,66,85,-7,-31,29,-5,38,86,-50,-61,83,70,-25,86,-76,58,89,95,-59,-8,-45,-75,-40,53,-41,-27,42,-66,32,94,-24,-50,31,69,20,-88,-35,-42,50,-33,-50,33,-63,-23,-29,-87,-65,-87,60,-72,5,68,-94,17,21,-35,42,-84,-49,-73,-90,27,-23,-7,-51,97,5,14,-92,-92,33,-90,-7,22,-13,65,35,-26,30,-5,2,87,-37,-40,5,37,25,99,53,77,-74,15,-44,4,-92,5,-47,-35,71,-87,-27,4,-77,19,-74,10,-64,13,36,66,-91,-61,53,-76,-1,-90,61,77,-90,14,6,-64,-71,14,40,89,-80,46,-93,91,59,-68,-52,83,51,-26,-55,
    87,88,82,53,49,73,-41,25,-76,-31,-13,53,-21,53,-41,-32,35,26,60,-24,46,6,-17,-11,-34,-32,37,-99,71,64,-54,59,4,-72,-36,-95,-99,-25,30,-74,45,-83,79,76,23,91,-56,-42,-31,57,-14,-85,-37,70,-44,-19,90,46,-18,61,10,-19,-28,66,9,37,71,-37,64,1,-59,9,-29,72,-62,46,15,-66,-96,84,91,-10,51,-45,-88,-92,-12,54,54,71,16,-84,-96,88,82,13,-23,53,28,-58,-94,69,3,-71,-6,41,75,-91,76,31,94,19,74,-3,74,38,5,-37,-7,-89,86,-91,-73,90,49,61,-45,79,-34,84,73,-75,5,-72,-46,99,22,-70,61,-50,61,7,69,35,
    -96,-4,-26,10,11,19,-27,-51,-20,-47,-61,-71,-86,-54,60,-19,82,-15,6,-60,-34,12,-61,88,42,-100,-62,-44,-41,59,91,-36,-45,-83,26,18,36,99,67,-84,-48,58,98,-81,-95,10,-48,39,48,10,79,-86,-78,-81,-98,65,71,92,-79,-69,3,64,-53,59,34,73,-23,-78,72,97,91,77,55,41,96,60,-97,-52,-48,51,58,-69,65,32,-98,19,-3,26,-37,-30,9,67,87,56,-22,-27,29,55,-5,-47,4,-62,82,-40,79,30,72,35,30,-76,-14,-12,-44,-96,-79,-42,23,-30,36,39,-7,-55,6,-20,-99,84,5,-18,-9,-100,-12,96,91,-30,-44,22,-99,-72,-43,31,-95,44,72,-39,
    0,45,-29,-77,67,-92,14,-40,-95,-80,-8,-93,-96,97,-59,-52,50,29,-56,41,52,-48,-37,5,-68,-27,-64,-63,-83,60,-50,17,-43,22,-8,-75,-18,-93,-15,-13,79,30,-54,36,27,-12,84,-71,69,80,70,21,-68,86,-74,-36,-41,-85,2,-72,27,52,-3,85,-74,41,62,8,48,-53,48,-20,29,94,-84,-43,-66,-100,-62,56,80,61,-71,64,47,-92,-72,58,-77,30,38,2,-65,-65,-61,-39,76,-99,22,-23,-99,-30,57,30,-84,73,39,-97,25,78,59,-43,39,40,-79,-62,48,-99,96,23,-16,34,-74,19,21,-35,32,-3,-33,-46,74,-32,-24,-17,50,-55,-92,-10,-52,-15,-32,59,42,59,
    -1,15,97,-52,-31,45,23,53,-69,49,-76,52,67,56,-51,86,63,76,-94,-61,-41,56,-16,-80,-54,-16,5,-34,95,-100,-75,-53,-33,74,-53,36,19,70,-59,50,20,-35,2,-61,-78,4,-75,37,-20,-69,76,-9,87,-87,11,86,-51,-83,-96,-55,-83,-70,-56,-64,56,-9,73,-24,-39,-86,-74,33,-20,-19,24,-46,85,49,91,-83,32,19,-40,-28,-16,72,-42,34,41,-86,31,10,96,75,46,53,-34,-29,-71,-21,-14,-93,65,18,-12,-11,24,-75,-61,-33,-58,23,86,-45,95,-29,-73,-95,-95,20,-80,-12,-70,-84,-37,28,69,-19,-100,98,12,-62,58,-23,56,-54,67,-68,-76,58,-1,-82,-19,-15,
    -27,-71,8,0,-66,-35,72,-46,53,-98,-77,-84,-17,92,-51,-17,43,62,21,1,-61,-71,-1,58,-39,23,-32,60,42,2,97,-85,31,6,-32,65,71,-60,72,25,95,95,-7,-22,87,-5,13,-18,-43,-66,83,-52,-37,-17,59,-24,-94,-73,-64,0,29,-67,16,-40,-9,84,-74,-37,76,50,40,71,45,-67,-51,-16,28,62,67,37,48,2,86,63,37,45,-61,-4,24,-73,96,-46,-87,12,14,4,-52,-8,19,-75,-58,-41,96,39,-7,98,-76,73,12,43,11,-39,-55,-51,76,83,46,68,-21,-30,95,-73,24,60,40,91,-35,40,-65,84,65,-22,-56,62,69,-11,12,93,-86,-76,-12,25,-63,34,
    26,-86,-31,-28,82,0,43,29,27,-33,-10,-81,-90,7,60,-54,-57,25,-24,-61,-61,-55,28,51,91,-5,28,-21,-80,65,-87,47,31,-66,71,13,34,14,95,-86,34,-15,-67,44,44,93,42,87,19,-82,-73,10,-36,7,-38,-93,-98,42,-14,75,7,-48,-26,-61,-14,-55,4,73,12,-49,87,-54,36,-80,42,-68,66,-63,-80,-63,-45,99,47,-29,6,61,-22,-39,3,17,-64,63,69,-38,-98,-93,7,-42,32,71,10,19,69,98,-8,12,-69,-42,1,-97,-53,56,-98,94,80,-40,8,-90,-27,-89,27,-39,74,-52,23,28,-92,83,39,40,54,-51,-88,24,99,4,88,82,14,-11,85,61,97,39,
    7,77,52,-85,88,-75,79,-33,-13,5,16,62,86,24,45,25,-84,-48,26,-72,76,25,32,16,8,98,-95,-55,11,-98,85,-81,32,-11,86,20,14,17,39,53,-77,-93,-32,-91,31,65,-14,48,-83,-88,28,45,37,-39,-39,97,-89,-82,-57,23,-79,80,-6,5,-31,-68,77,35,50,16,-59,25,24,9,-14,-93,-26,72,7,-56,84,-64,89,-27,-51,-97,-29,-40,-79,-34,35,94,98,81,-1,-81,-86,-24,6,16,93,47,93,-31,-92,-21,-24,-65,-49,84,-69,87,72,-80,-40,-79,-77,-17,-67,97,1,69,91,-1,50,-9,18,16,67,-75,84,12,24,77,81,-15,-44,-42,20,-41,94,51,-54,66,
    23,-41,39,-1,-6,72,-4,96,-7,-61,-53,96,30,66,12,-50,43,-3,14,-81,-74,-4,4,-17,6,76,-6,-48,-21,-7,70,55,4,9,-94,98,-19,2,94,27,-7,-58,23,-76,60,-65,-74,-45,-16,40,-74,11,88,-17,46,-54,-89,-60,-2,-9,-15,-32,98,89,77,-96,88,-89,-42,34,38,51,28,-39,27,-12,-52,53,-5,-15,-54,-26,96,86,57,42,-67,68,34,31,-89,-80,52,-91,-91,29,65,49,-8,-77,-16,82,27,-88,95,-46,-47,-56,60,0,-19,58,74,-23,44,31,71,77,0,-43,-39,-37,-23,-87,-27,-61,-6,90,88,39,-86,-76,21,41,-63,17,47,-58,13,7,42,94,-83,17,
    -77,62,-100,46,-9,-48,-97,52,16,-67,65,41,-28,-88,31,-88,51,-3,-63,-76,90,-26,-7,38,-32,-94,97,-90,52,-85,-21,27,29,-20,-27,-80,-16,-23,73,-100,10,-10,-7,34,-46,-75,46,-95,22,83,-70,13,-91,-25,-97,-23,-18,-100,-60,86,67,19,14,-4,51,39,17,-64,-84,-58,88,-22,-16,-18,-88,39,7,59,-4,81,-6,-22,-54,4,54,-99,33,88,-98,73,74,21,-55,-60,-82,96,80,-13,-68,48,29,73,79,65,-45,91,4,14,2,53,-53,49,-69,-54,-95,85,-53,38,25,-51,-36,-48,23,9,-8,41,5,-76,80,-10,25,-39,-37,4,-74,-30,-53,-17,84,2,-64,83,-49,-81,
    -71,-44,57,-23,46,-18,-22,-90,34,-99,19,-73,-6,-75,-97,-26,-33,28,35,-18,84,14,52,84,97,-12,-14,85,-77,-11,-96,-47,45,61,-18,-57,-56,60,-46,78,14,25,57,60,2,13,-65,69,93,22,-49,78,-64,55,14,85,95,-100,70,-81,-11,75,-28,86,-12,54,-71,32,-34,-65,-37,32,61,72,-7,-37,-63,-20,-67,-69,54,-64,61,43,44,-25,28,39,-25,-49,58,16,78,82,2,-34,36,-17,51,55,19,66,-61,-20,-10,-16,43,28,16,76,11,-29,65,-76,66,9,-1,-6,-52,26,-3,-93,-58,-73,89,-4,-54,-22,79,49,85,-2,-33,76,78,57,-39,22,-63,77,-50,48,-100,67,
    72,66,-24,-77,-87,-23,49,10,-64,43,-10,77,91,36,-93,23,-15,92,-79,52,21,52,-39,82,-74,-1,11,-72,99,12,96,24,-70,-28,47,-57,1,-51,6,37,44,96,15,36,32,22,-41,69,-33,-68,-27,88,84,34,22,62,-67,-67,91,85,-3,87,9,28,-89,-44,71,-35,5,-23,-98,-50,-75,17,38,57,-60,49,-22,-41,33,51,-53,18,-14,69,-68,-29,-98,-77,-44,-48,62,65,80,26,-78,51,91,-21,81,93,-19,-94,-37,71,16,-45,20,46,14,-46,-2,61,24,36,-18,8,-93,-64,-16,-84,-12,-54,81,-80,24,55,24,-33,87,5,-87,-32,-37,-24,92,31,-17,-36,-22,97,18,28,
    10,94,-36,44,3,-29,80,87,-13,-79,85,-79,-59,-38,28,65,29,15,-78,-6,36,-14,70,28,69,-95,92,-53,-98,-37,27,-36,-91,91,60,-36,-85,41,-49,-46,62,89,-25,55,-97,-96,21,32,-29,43,-73,-93,81,49,35,51,55,-20,50,9,-5,-22,26,4,-79,-14,69,-64,79,72,-57,-7,61,18,-51,16,-26,70,-51,-2,-35,-72,5,-53,29,93,98,-64,-27,0,-2,68,78,76,-76,52,-86,93,88,94,66,-69,-13,-21,2,-12,96,28,-90,97,-74,-24,77,-16,-25,-42,-23,-75,-53,2,25,45,-78,-44,-27,-54,-92,-13,92,48,-67,58,32,21,89,-66,61,37,14,72,-14,41,-100,-37,
    -75,-25,-26,-46,0,21,56,77,-82,30,-67,-9,76,93,78,68,42,64,-22,-26,-15,-80,-40,98,57,74,22,-56,-33,22,-41,-56,97,33,-2,-51,-94,-94,-73,-76,36,-88,15,-35,58,-54,85,0,-90,-84,26,-53,-64,86,97,93,12,20,89,80,-58,-51,76,92,34,-73,93,41,85,72,17,22,85,85,39,-5,-69,24,95,93,40,21,92,28,-41,89,22,71,-39,-37,3,4,-36,-20,-52,99,-41,-59,-8,-56,-86,-91,18,51,94,-91,-54,77,34,93,22,-74,-34,14,7,25,-44,81,-52,17,-56,52,73,9,84,73,-40,95,-85,52,-61,81,61,58,-16,8,67,-18,-15,-47,-73,-92,-68,-7,
    -26,39,70,-18,20,-82,52,16,22,-23,-75,6,-49,-15,1,-82,89,-7,51,51,-97,-65,-89,22,-31,-4,28,-52,-44,12,41,-17,-49,11,65,23,-19,17,91,-96,95,-83,-38,-54,-46,16,-84,-56,-91,67,47,64,54,58,-14,-77,-94,-86,-29,-85,26,64,-2,29,-73,-37,-96,8,33,96,-36,-20,65,-21,26,71,95,-6,-85,56,-39,14,-28,-85,-76,58,90,83,25,-39,98,-97,77,-52,-15,-96,11,-59,-36,96,89,-19,-24,54,60,54,-22,7,-52,-55,-85,9,60,-13,24,36,97,14,19,22,27,-31,-22,4,17,15,-40,81,56,-23,29,46,58,-94,-48,-30,60,30,77,-91,76,92,-30,-12,
    -69,47,-76,-72,-39,-4,3,41,65,81,97,-65,-52,-42,68,-96,35,-3,2,-55,3,55,15,-84,-63,-8,77,-87,-64,99,-47,-33,46,-22,-53,60,26,50,1,-57,-17,-2,-22,31,8,98,36,95,-4,90,92,51,45,7,19,35,51,-4,0,39,-4,54,58,-58,84,-94,54,10,56,-45,53,92,6,84,23,66,82,-89,14,-70,-98,6,82,-1,-86,-47,-66,17,50,35,-43,98,-59,-85,92,-75,-79,47,-13,30,-98,-60,22,-40,76,-55,-73,11,-91,-7,41,-89,99,-25,10,-87,-19,-3,-17,-69,-68,40,81,-27,7,73,-50,-19,20,37,-89,-25,29,-15,-13,-42,-18,-34,69,-9,59,-38,2,59,
    -10,65,24,71,-38,-93,54,-54,-1,35,19,-93,8,21,-12,81,-42,-1,-92,39,36,-5,97,-82,62,66,10,-27,81,64,-68,-77,29,9,94,43,16,0,89,-84,-65,-40,75,-5,81,-37,-72,91,14,-64,31,-50,-16,28,-32,98,-53,30,-29,80,-5,-44,-97,76,65,49,20,-67,-51,9,-51,-16,22,76,31,3,-61,60,-53,5,48,78,55,-68,-42,76,82,-43,-42,-46,-63,53,-38,-60,-18,-73,89,54,-40,-62,-85,-38,-26,37,-62,58,93,-70,-30,-60,35,-82,70,-57,-97,-72,71,85,-14,-71,91,23,35,-47,16,-83,-68,5,71,-7,-4,86,7,22,-24,45,-20,-31,75,50,9,-37,21,-21,
    6,24,-41,-23,-39,45,-42,-95,-79,93,-90,37,62,-57,94,33,-12,-58,-28,95,65,48,40,-55,69,-32,48,-22,-17,-31,9,41,93,20,70,-94,-82,-72,-89,-61,-78,-78,28,84,-35,-26,-30,53,17,-6,0,-18,94,92,-21,63,-88,-73,93,-5,-4,-46,-64,-59,74,-94,-52,92,87,-41,83,-91,-67,11,45,-50,86,67,55,-45,-39,-45,37,-45,-52,-84,70,-40,-56,63,56,-8,-83,44,34,44,3,82,36,-10,93,-28,-49,79,83,96,-71,21,16,37,76,29,92,-35,37,-60,-66,7,-47,-70,-77,61,22,92,5,-44,36,-92,90,-75,-50,36,-3,1,-33,-68,-50,-52,6,66,85,-66,-5,78,
    0,-68,70,34,-8,-77,64,15,84,-62,7,-58,-53,96,-98,89,-79,-47,-23,18,54,44,50,56,93,-44,22,30,-9,70,-40,43,54,-69,-71,98,-94,45,13,43,83,73,85,-70,69,87,-80,90,92,-51,-40,47,94,-90,-45,39,-81,30,-31,-38,0,30,57,-94,13,86,-95,19,31,-30,-38,-34,-57,99,-3,-88,87,-31,54,79,18,-86,-22,-36,-75,-14,-97,96,-84,-75,-90,-32,7,67,-26,-80,5,-69,-9,-64,2,6,-46,-3,5,3,10,-56,72,-36,76,-57,-21,-94,7,-44,-8,-37,-96,8,-60,-86,28,47,33,55,19,-62,86,-90,-74,-60,-32,-20,38,74,84,-100,70,8,-84,-2,3,-53,
    5,-89,3,49,-74,-41,-90,-34,-75,38,65,58,-7,-16,48,-68,-54,74,-76,15,-93,62,-59,43,14,63,-97,31,-38,7,78,19,-30,-66,-80,-4,-7,-70,14,-29,21,-21,-71,66,15,78,50,61,4,75,-24,-37,-11,-31,-94,-96,-67,-90,-13,-53,17,-83,66,39,-49,38,-65,-3,69,1,68,-58,80,49,60,-5,-21,11,-92,84,-62,-63,99,27,6,58,-17,-9,68,70,-10,-63,88,-44,76,91,47,63,-12,-32,-36,-92,10,96,58,-78,-57,89,33,-97,73,71,-60,25,-49,-1,-17,34,42,-97,5,-67,92,-55,-59,20,-64,88,-17,-23,-44,-1,-15,18,-53,95,41,-10,85,-26,-7,10,98,-14,
    -65,-99,85,18,35,27,21,92,12,13,89,54,-67,-74,42,-32,3,99,19,40,17,66,-12,-90,-44,-27,-15,2,35,-65,88,71,88,25,-59,-25,-96,15,-32,17,80,-91,-29,66,-65,65,86,-10,64,6,-69,-66,-28,-29,-56,-19,44,-19,83,-69,-32,23,-98,-92,-100,96,-16,-96,-37,4,21,-57,13,44,-91,1,10,96,91,-74,2,74,-88,26,-55,-91,-93,41,-58,42,73,-37,-83,27,-29,-83,23,55,-78,86,-41,-5,82,73,92,91,-26,2,39,-83,-20,93,92,-55,-28,89,-46,-69,31,-52,74,4,11,43,-69,83,61,7,38,35,-55,50,-70,-73,-25,74,71,1,-72,-38,18,9,-92,-38,
    6,-20,-48,60,11,83,-92,-63,-61,-28,81,-78,-45,42,29,45,-71,27,-53,-41,6,22,86,-71,-77,-86,-8,-6,23,-48,56,-71,32,-92,41,95,43,2,-67,82,74,-86,-95,81,8,-14,-22,37,-87,26,48,20,-100,34,49,76,49,93,-30,-76,-55,-74,-94,29,-65,-1,25,78,-99,58,13,27,24,18,-40,84,-96,-9,-27,-82,69,73,-10,69,-40,91,45,-91,37,-85,85,82,94,91,12,81,-57,89,11,-56,-1,-76,24,75,42,84,59,47,-73,84,17,96,57,59,66,-83,50,-37,78,-13,-69,16,-78,-75,-41,-14,-94,2,-25,17,47,74,42,-77,49,-64,59,-40,35,87,-56,-48,83,53,
    63,1,71,-34,65,1,5,96,-83,27,73,-23,65,79,79,-60,-52,-22,66,42,1,-33,79,-39,27,14,-52,23,19,83,-71,-18,-15,-100,48,-50,-47,-46,98,-29,33,71,0,-1,50,-21,-9,50,58,-42,-7,-89,25,-28,-28,-95,86,-28,-20,5,-44,9,88,41,-39,-12,43,-33,42,41,38,-72,12,-62,27,-86,-31,-82,64,79,-24,-43,91,54,-19,-85,11,-32,-12,43,25,-4,-95,-35,89,-82,54,32,-15,-52,-75,-77,-24,-11,-87,55,-45,-17,74,-81,-38,-98,-71,-47,8,10,-79,71,30,9,15,-44,57,-28,-27,46,-10,-73,-70,76,-72,-93,99,-44,-4,65,-88,51,-100,38,22,-38,-8,3,
    -32,-99,-86,-11,24,-4,50,91,4,-41,-37,78,57,54,57,87,-18,85,46,-67,94,94,98,-94,-3,98,-4,-81,-87,88,23,-67,41,-11,-26,-34,85,-76,-91,-10,-65,-27,-80,92,79,77,31,-39,15,-23,-6,9,-29,-55,-33,68,-57,63,39,-92,3,-86,-59,97,-45,15,-85,-59,-9,-76,-17,26,-51,-45,18,-72,-68,-51,-59,47,-22,-12,-92,1,33,-25,-79,28,90,61,37,-54,27,78,-57,83,-54,-42,76,37,-66,-89,-36,-16,-34,34,-36,-2,84,-42,98,-86,46,-94,16,-69,-66,37,59,-76,-50,-4,-78,30,-73,-35,13,-27,-25,-11,-90,-38,0,-74,-2,66,61,62,-84,-3,72,-86,11,-30,
    -79,79,1,-45,-31,-39,-69,19,9,-46,-51,-12,-29,14,61,47,-97,24,-91,-45,50,7,73,-37,-79,90,12,46,-96,76,-84,77,55,18,-68,-76,31,-84,-4,92,-30,-3,-19,41,-88,94,-60,67,18,-99,75,-79,-40,48,-64,-18,38,49,80,43,25,96,20,-68,-34,5,-91,49,21,-43,42,43,54,75,-64,18,69,-71,-62,-60,-70,-87,-87,-9,61,-51,25,0,50,5,95,-73,-47,-33,-88,20,24,-79,-31,97,-22,63,40,84,38,29,-45,-40,58,-7,0,40,-42,-35,-69,-81,-34,8,-29,69,65,-82,96,-81,-62,60,39,62,33,60,60,-89,24,-48,48,-38,81,-97,-26,39,-52,26,-20,6,
    91,-37,-23,-42,24,1,79,-11,-29,27,8,9,-12,-53,-28,-79,-92,-68,85,-16,-16,33,46,-82,-12,73,9,-12,-1,41,46,-9,-95,75,1,-19,-72,-20,70,0,-93,-21,61,-53,-22,-67,-31,-62,-83,-46,22,-46,39,-79,-76,79,94,33,-81,-55,-25,65,36,-68,-8,89,13,21,-79,-17,73,81,14,34,-72,45,68,-3,35,37,3,-42,43,94,-69,67,25,77,1,44,-78,76,-39,11,60,6,-48,-27,-73,-26,8,-100,-45,75,34,-17,20,-46,33,-45,44,88,-35,-13,35,-52,-93,-40,77,8,57,52,-64,70,-37,96,-24,15,-79,-45,41,-19,-45,96,-44,42,-68,76,48,17,84,-8,-43,-99,
    80,92,-50,39,-95,79,-53,14,31,-65,-64,46,83,13,-38,-96,20,55,-63,28,52,-6,-30,36,-78,-82,-95,58,11,-38,60,-57,-45,62,82,-88,41,81,26,-27,-84,14,-81,99,79,-67,55,0,89,92,-72,-7,-62,-2,81,-39,-32,86,-81,31,-52,79,26,-45,-7,8,-81,-65,-59,-3,-40,-91,-88,-69,60,91,-83,15,-57,-94,-40,-29,-49,-2,21,84,11,90,70,31,-27,70,-38,-100,78,8,-40,97,43,2,-5,55,11,7,86,-28,-50,-97,39,-6,61,51,65,64,-98,-61,48,13,81,-30,96,-46,93,-89,-46,71,19,15,20,-86,69,-85,21,-68,-26,-41,-44,25,-37,48,71,76,-1,36,
    -7,-47,27,93,19,8,-84,-33,15,-91,-22,69,32,49,-64,-48,15,57,-32,88,90,-6,-52,-54,-81,-37,94,90,91,-54,-21,-16,-49,58,-70,70,-33,-54,-62,82,-93,-32,3,39,70,40,43,-63,-51,-89,-74,-61,-42,-26,-62,-23,37,-68,-80,28,-70,-49,65,-18,-91,95,-48,-72,93,-58,62,-100,11,-34,-9,-67,58,-66,-30,-93,98,-4,-53,-44,70,-15,-15,-41,-31,5,40,-48,8,-43,34,18,-48,38,-2,-55,33,-39,97,-56,27,-12,-23,-63,22,47,-56,20,96,-57,76,18,-20,-38,30,2,19,-30,54,-72,27,-60,-2,31,30,48,28,-37,-91,-23,-41,-12,65,-12,25,-61,88,22,12,84,
    17,-12,2,50,2,-68,-48,74,-46,58,54,33,50,-96,-84,-68,52,44,-4,62,-79,55,-50,38,96,28,78,84,-50,-58,68,19,-18,-78,-31,85,-93,-27,11,-87,83,-35,47,33,-79,63,66,73,-40,-86,35,-67,21,38,24,17,-34,-46,1,-32,-4,-79,39,78,96,-91,15,-97,-66,-74,-84,-82,-57,15,3,-36,-69,-79,-62,-9,-65,25,-24,9,-85,-100,-74,-19,-46,-20,1,2,-47,93,-19,-51,-98,96,-96,88,75,73,-94,70,-12,-38,-65,19,-17,-75,62,-29,50,-61,80,18,91,58,99,98,90,-47,0,-56,-54,81,-55,-100,30,-50,40,5,-77,-1,-25,11,61,-38,83,96,-13,-3,-33,90,
    -12,99,8,-20,10,-41,-22,0,-36,30,-4,-38,12,42,-86,94,92,55,-1,67,-46,26,-22,-85,89,-87,63,-72,11,-69,70,99,82,-70,-69,-8,-58,9,45,6,92,-59,21,-44,35,35,-50,27,-58,1,-6,-4,-73,25,-37,68,-10,-73,49,-99,-90,-29,-99,44,2,-68,-63,44,-6,34,-98,-62,27,23,-6,-37,-89,44,90,-47,-55,-63,2,-76,14,65,93,-96,92,-6,-94,54,65,59,-1,-33,43,-12,63,89,-78,66,-73,49,-59,21,-88,-96,-35,55,-42,-38,44,60,-13,-42,-23,-20,-38,-78,26,-80,76,-9,31,-25,11,-25,15,-26,-84,37,92,-56,-61,86,17,-97,90,35,58,-100,97,-46,
    12,-16,12,90,16,27,-88,-58,47,40,-14,-21,68,97,6,-17,-77,22,73,68,18,-88,54,-64,67,-4,23,-22,-3,20,32,61,-95,-55,51,-27,24,15,16,-29,-44,54,2,76,51,60,-41,26,-17,84,-6,53,48,-100,41,16,-3,-36,94,46,85,26,7,42,23,-89,-85,-53,26,-17,-29,34,-63,25,10,40,-14,22,-33,21,6,61,-26,-45,62,68,71,-89,-68,65,57,69,-57,-36,11,67,-73,-21,66,54,62,89,-60,-100,15,51,-8,-99,-27,59,22,31,21,48,86,-65,-32,57,-54,-47,-26,55,-78,18,71,-14,-63,99,17,55,5,-21,45,45,-69,-40,-52,24,-87,21,-65,-13,-47,8,
    87,91,43,-44,49,41,9,75,-52,-17,-55,-80,69,-66,-81,-62,90,24,18,-13,-79,1,-1,-30,77,12,43,-87,99,96,21,38,40,-83,94,-59,10,-45,-84,59,39,-38,79,-40,-52,50,-1,90,-74,69,-23,-53,-78,76,17,52,-60,-39,65,91,-91,-62,30,49,-93,-24,90,-30,32,59,-71,-29,73,60,-17,-79,-38,82,-88,-12,3,-11,-13,26,-82,-95,-70,-42,-82,95,-98,-73,85,32,29,45,-92,-29,-85,40,-70,96,63,-97,8,-53,77,70,81,89,-90,-63,-70,-3,15,-100,-46,45,-89,-28,-8,-87,0,-71,45,81,74,5,-48,41,98,-17,37,61,-62,-55,60,-85,67,-6,56,77,31,39,
    -73,-54,39,-19,43,50,6,-65,63,-94,-36,-40,39,91,66,91,32,64,-74,-78,-23,-83,-81,90,-68,39,-16,41,-32,67,-20,-5,65,-81,29,8,22,-65,43,-63,93,59,-2,32,2,-84,-25,-65,32,2,-91,-91,19,-72,51,3,67,-65,-56,-64,-98,-24,-17,19,-52,-88,-73,70,-53,22,-93,-60,-66,-43,24,36,-27,-100,23,57,-46,32,19,73,13,-30,-72,-68,-42,-75,-32,-88,1,4,32,49,16,59,-29,16,-18,-21,8,-32,-12,-67,-96,14,85,28,-29,-61,-88,90,64,25,-39,92,10,-29,-83,30,-17,-29,-66,-85,72,3,75,-56,-81,-91,-25,-21,-23,63,12,-67,-23,-51,13,49,-12,26,
    -9,-48,-97,4,97,13,75,66,-56,-41,89,-22,-26,62,81,-99,-94,52,62,-19,-68,91,-56,-4,-75,22,-2,38,-77,86,-84,14,91,-80,-81,40,-15,46,6,29,-95,-4,60,-68,-90,-7,85,-84,98,48,-51,82,-9,-7,-70,16,67,28,-93,42,15,75,-91,-42,47,-72,98,-67,26,-44,14,32,4,-74,16,14,-28,1,-18,-30,-51,31,-48,41,77,-18,-91,-4,-89,68,-61,-22,44,-52,-12,-57,-72,-14,-72,54,-6,43,-62,99,21,54,-35,93,-44,48,-37,5,-69,15,98,-92,50,60,5,13,-20,-4,43,76,44,-69,-80,24,-31,-52,-22,63,43,17,14,-35,-29,32,-42,27,-68,22,85,-85,
    89,35,-76,-9,-53,-19,-96,28,29,-1,4,-75,30,-24,49,51,25,79,-33,-80,-4,33,-15,20,-35,44,99,-51,-82,36,-35,59,-76,-59,-49,-29,-26,-93,99,-97,7,-44,-72,-63,32,77,89,-91,8,-44,-70,-95,89,-33,-23,-93,63,-72,-44,81,65,-27,41,89,14,92,-40,88,99,-88,91,-42,-80,71,-4,4,0,-63,14,-91,-55,-4,66,86,15,-57,45,-21,-29,-98,-88,88,-73,53,77,42,-3,90,-18,-51,-98,-74,7,74,97,55,-22,-2,44,44,-41,89,-60,-75,28,8,-80,-27,87,-9,-73,99,-68,-45,5,-91,49,-98,51,-69,-97,5,-43,-37,-21,-93,-82,58,-43,15,2,16,4,-5,
    41,84,-97,61,-42,42,-96,-15,-7,-64,-8,98,98,-59,53,49,-27,56,-45,-18,-81,-14,-11,90,-56,46,5,-1,-38,61,94,55,46,49,16,56,-9,-27,-59,84,-39,-14,35,-41,27,-12,61,-100,96,68,-17,68,-46,24,10,-49,-29,-85,50,33,28,44,-59,-74,-55,9,82,36,82,76,-28,96,-38,-93,-93,89,-53,68,42,-56,-12,-75,-88,43,49,22,94,72,-11,44,-42,69,-60,-1,96,-15,-92,78,73,43,54,97,39,-84,-95,46,-94,-48,-33,48,-52,55,25,-88,-2,-74,-14,-56,99,75,40,-91,45,80,8,-7,-83,16,-29,42,-89,-74,40,50,94,-3,-51,52,49,16,-48,50,-29,29,
    62,22,56,-51,66,7,76,7,16,-27,-61,-76,66,-91,-8,-62,51,-44,-84,-9,-94,62,88,-93,15,-10,23,67,92,95,-51,54,-31,-95,55,-65,-36,32,-6,80,57,34,56,24,-5,-100,14,-54,-92,30,-10,67,-8,-22,74,59,20,-2,-21,12,-55,28,-81,14,-15,74,1,49,-42,96,-19,-84,-18,-63,40,77,89,-46,75,98,36,-35,65,-72,-4,39,40,-84,89,-81,-19,86,47,0,0,-68,-74,54,81,-15,50,-86,1,84,-97,93,13,92,-53,-12,42,35,54,-41,63,-98,51,-45,-82,40,-26,-49,-21,73,51,79,-95,78,-15,38,-85,87,4,16,71,7,9,-64,0,8,25,94,-57,-69,
    -46,58,-67,-95,-86,-97,-3,-60,-45,76,-86,58,8,71,-12,93,-38,3,33,66,71,-44,-26,-20,-7,26,40,70,20,83,-99,26,-6,86,31,8,41,81,0,48,9,14,7,-83,-62,95,63,-48,-49,-4,-82,-78,52,-56,-45,97,70,95,67,-57,31,20,69,-75,6,-47,-15,-100,-66,85,48,43,-48,-45,13,-58,-97,-24,-6,-94,24,-36,-72,-72,9,-17,-74,-69,-69,45,-26,62,-34,96,39,24,-51,-76,24,-65,61,-27,78,-87,-20,-57,-93,-65,-29,1,-59,95,18,-30,76,-73,5,54,-90,36,-1,-63,-50,65,-67,89,-58,-66,65,66,-31,27,91,99,-8,-28,95,0,-41,-34,53,1,-38,-29,
    23,90,-50,80,44,-39,-31,-57,-2,19,61,-17,-39,-97,-31,-74,69,-10,53,13,-59,-54,85,-64,-2,96,3,-49,-3,17,23,-80,7,73,53,-49,86,-78,-54,-64,41,-93,19,-98,-90,40,81,32,-70,34,-55,-28,32,82,8,-70,78,-89,-18,-24,28,57,-52,-65,-18,-99,-62,-31,-77,-15,5,17,-8,77,-29,-45,17,-48,87,48,39,-68,-80,-29,66,28,-46,44,-8,36,72,-80,-55,73,-92,27,26,98,96,2,-17,2,-81,28,-21,-58,83,48,-53,22,-4,86,6,16,-91,-28,97,-37,68,-11,51,-7,-39,96,-34,-79,24,92,-80,-28,-6,-45,26,-35,35,57,8,18,6,55,92,-98,-7,-2,
    -29,2,22,-80,18,-9,-39,-31,84,22,-34,-98,-4,42,94,-32,66,-59,23,-7,6,-41,-98,66,29,8,-27,22,-37,66,72,-14,-79,-5,6,39,38,67,-92,22,-59,-74,-24,37,20,-30,-95,-13,-37,-19,32,70,-8,34,-12,-79,-57,62,-5,-42,-20,-80,44,1,-33,-50,92,-95,69,53,-21,-90,79,55,48,-48,-75,53,39,-59,-66,-29,63,-22,-95,-49,0,-100,65,47,-90,-54,19,54,-1,86,56,-56,91,25,97,-30,36,28,-75,-64,80,-97,89,19,-56,76,42,59,-46,0,-90,6,-48,28,-94,-37,26,25,69,77,12,-22,-79,-45,55,70,26,-9,-1,3,27,-21,6,-31,51,2,97,-55,
    -39,-97,-3,-76,-38,50,52,-32,65,-22,45,-14,-45,-91,64,-71,-35,20,99,91,63,-50,-6,-57,-18,53,-36,-67,-93,-87,-70,-79,-84,28,97,-70,-70,49,-50,95,79,96,-67,86,-95,98,-85,-30,-30,67,-87,-67,-83,-40,-72,-1,65,-8,-16,24,5,15,45,-26,95,42,4,-75,-57,55,72,74,3,-95,61,-40,55,28,31,77,-5,-4,-89,13,8,91,64,-27,-64,-51,98,41,16,-5,67,11,-62,-76,-12,-67,31,-88,-92,-66,69,21,-6,25,-51,77,2,45,-74,-35,-90,34,-91,74,-92,-55,-77,58,38,39,-47,6,-98,-57,-18,-58,-23,-87,-46,-63,-53,76,58,-7,-47,-93,-77,7,4,-51,
    73,14,83,34,-11,43,79,-36,1,17,56,7,75,10,50,-43,53,79,-30,59,-84,-31,35,26,15,88,34,38,-4,90,39,-79,5,22,55,46,-82,34,62,-29,-49,18,78,79,-71,-19,-64,34,-40,7,-7,-71,28,-71,55,-57,69,-59,33,-35,-68,-28,-62,-11,47,93,87,65,27,49,88,31,-32,67,-38,-51,-52,-2,83,-40,57,-72,-59,-14,-91,97,-19,79,90,-85,96,74,-13,-65,-85,-66,-72,-98,51,8,52,40,91,-80,7,-47,-79,7,3,56,19,-39,84,61,-1,94,10,80,25,-48,95,21,-73,-17,8,94,-31,-11,97,-79,49,49,61,-8,21,-80,45,42,-21,-100,-50,98,-39,-66,
    59,60,80,-79,-7,5,-26,-12,79,-47,23,-61,-53,93,28,-56,-86,-23,45,27,69,-82,-1,-34,12,78,-33,62,76,-20,97,-12,93,29,-39,-14,-65,35,26,66,40,-50,-95,88,95,34,84,61,63,-18,-12,-15,0,87,51,-87,17,70,27,-7,-49,-24,33,96,-94,95,-66,93,-18,60,59,-77,62,16,63,57,-98,47,18,18,81,58,-97,-18,-3,54,-53,-86,-23,74,-40,-20,-49,45,-24,-91,40,10,-98,75,-78,13,98,85,81,61,42,84,-40,-87,-46,-58,23,-43,-24,-79,63,-25,87,-60,-51,-53,20,-48,93,96,-87,-15,-42,-85,-40,-67,-20,-90,-82,-38,23,12,98,84,-23,-48,78,-99,
    -39,-94,-78,-76,-19,61,-35,-18,-91,85,35,54,34,0,91,44,16,-96,77,48,-86,47,62,-10,-40,12,-74,37,64,-96,-10,-75,-90,-36,-98,-57,-74,-33,25,87,-48,-88,41,38,13,32,-17,81,-12,-88,29,3,60,-8,93,72,-96,-81,-39,21,75,52,-2,37,-84,0,80,-58,19,5,-71,24,18,22,-38,83,-93,-3,64,-5,10,-7,50,-78,37,-5,94,-6,14,-45,67,-59,59,-35,30,76,18,10,70,89,68,52,13,-62,-74,76,-79,-67,-75,85,-19,35,-70,-69,9,68,27,3,-38,93,11,-71,87,-30,-54,17,98,16,-72,21,6,48,-75,71,38,-49,47,59,-63,73,-4,18,-40,26,
    1,70,-6,-72,-75,8,74,36,-11,61,59,-12,-22,57,56,-42,-70,-86,58,-45,86,-4,-41,-15,-45,-4,-90,3,66,-77,-70,-33,93,-24,-52,70,37,-78,7,-74,35,-82,-34,65,75,-77,-76,58,89,34,-87,75,-69,-28,-87,-62,20,-77,42,-14,-54,72,6,91,-100,54,14,37,76,-79,64,11,-61,82,-72,66,57,-48,-24,47,39,-58,-26,70,-34,-13,-92,87,-37,2,-75,-39,-74,31,-95,-73,-15,19,64,-87,-8,-20,-24,-17,-37,5,-99,-80,9,-22,67,-52,20,-58,-82,86,-19,79,25,-4,-19,51,10,8,-18,15,35,20,-66,-49,33,-22,32,-90,61,95,15,62,-85,-76,92,35,-27,12,
    29,-57,-49,62,-78,76,-41,-44,27,69,16,62,-16,-97,-18,-30,-46,-85,0,86,77,-39,33,-8,75,49,-31,68,-64,-6,-68,65,37,-17,27,12,-40,38,20,39,7,-64,1,43,-61,35,65,-55,3,-35,84,-20,-22,17,25,-46,-82,94,-78,-94,-12,-46,71,77,-10,-49,-11,50,89,-91,-59,-3,-55,-5,92,-64,-70,58,-66,33,75,-82,66,-46,-13,-9,8,-42,-63,-18,-36,25,-12,-64,2,78,-61,44,-20,-72,53,22,-23,51,-83,-30,39,99,80,73,-15,55,43,-49,61,83,-6,-31,-59,31,3,5,-92,44,-7,-38,22,-16,-94,55,13,12,-23,90,-37,-54,-88,54,97,92,28,-18,0,-77,
    -15,-39,-94,31,-17,99,-86,86,-43,-78,30,50,-15,-95,-65,-57,60,0,55,89,-58,70,-65,-45,25,84,-53,-95,-33,-1,-72,4,61,87,-64,-56,86,-50,82,-57,-75,13,46,-38,-30,33,-43,-70,-67,-87,71,-25,35,58,-18,-40,42,-18,-83,61,81,98,-34,42,85,-46,86,23,-44,69,67,-19,-66,-35,-5,-44,98,53,-14,83,18,-91,-90,-47,67,-7,-34,9,75,-17,-29,56,-67,-11,-49,-82,43,89,-6,99,10,-87,-67,-4,-22,-20,52,-72,-67,-62,63,51,99,73,57,66,-34,-77,-24,93,-42,-1,2,-8,88,53,-38,-17,-6,56,-66,-95,69,-81,1,99,0,-46,-73,33,-56,90,-63,-56,
    16,94,-90,34,-83,-62,28,-73,-11,82,19,-71,-13,34,12,81,-10,-1,86,-88,18,-60,63,-82,-54,43,4,-10,33,41,-14,-99,35,-51,36,4,-13,-84,31,29,50,-97,58,-63,37,-29,-82,79,70,57,43,-60,49,7,-89,95,-50,15,-15,35,-92,24,37,43,73,-75,99,-88,93,30,41,43,-15,0,80,-78,71,50,54,-7,-41,-3,-15,-92,-96,-4,-97,-46,63,-59,90,71,65,-21,14,38,4,-35,50,-51,96,-8,92,-67,44,24,-44,-33,-26,-90,60,-14,-41,97,-6,-36,-6,-50,-30,-43,-9,-40,81,-92,-9,-5,-2,47,13,-100,96,-39,-56,88,-6,88,12,50,-45,39,12,67,-75,72,
    65,-29,88,-89,-27,-42,-80,64,-29,-99,24,62,-51,-78,10,62,75,6,-77,-81,-53,17,-92,11,-80,-85,-50,-16,83,-73,-44,0,-49,96,-37,24,-45,-17,-59,26,-15,65,-60,34,40,50,-52,-33,-91,-29,-14,-44,40,-54,67,12,-38,70,97,97,49,-95,-51,-100,-98,12,25,-43,95,18,35,-68,-65,75,18,75,-22,66,-58,-13,89,-19,-5,-18,27,-38,94,41,84,91,-10,34,49,39,34,51,-49,-89,60,99,29,-5,-69,65,-78,50,-8,0,-32,-13,87,-42,68,-18,-8,-53,-3,-14,41,-67,-70,-69,67,79,71,-46,30,74,-83,-58,73,47,37,-95,-36,59,7,8,-88,75,95,51,85,15,
    86,-23,15,-17,16,-44,-32,46,-13,-64,-23,-90,-58,59,85,-41,-99,58,58,-10,15,-78,-99,-78,31,-87,50,-22,65,35,46,51,65,-39,86,-67,-31,-46,31,8,-58,-92,19,-16,67,4,44,68,-86,2,-90,30,-23,-89,52,60,-75,2,-62,42,90,36,-7,55,49,-69,40,-82,-15,-29,-21,-20,-21,98,64,98,-46,-40,18,-32,63,28,50,-60,39,3,-100,64,57,90,6,47,-21,51,-46,-72,82,94,99,-80,-35,-70,0,44,-72,-36,-6,-66,25,-88,-46,-60,40,-95,-68,-20,8,84,96,-83,26,-45,65,-95,-94,71,86,41,66,37,61,-69,67,-87,28,47,-71,-78,81,6,-65,-65,46,27,
    -60,78,-93,0,14,-96,-30,-59,-89,-65,-54,69,-94,84,10,24,21,-77,8,-12,88,-64,-65,-82,-90,68,24,97,56,-77,25,48,-99,-16,-51,16,-60,19,9,-49,6,7,-79,-36,-8,-17,89,-87,7,-3,-46,-5,-15,-59,-35,-53,62,90,97,18,-87,-26,-34,66,58,-33,-66,51,38,43,-98,-56,-49,75,-39,-5,-41,-98,-40,18,-1,-34,65,36,8,-69,35,-30,-79,-16,88,86,58,-94,4,-31,-26,39,-80,-88,-66,-78,-43,-63,50,-30,32,-91,-28,93,27,23,-41,-8,-89,19,23,46,41,-4,83,-19,-66,41,-12,39,62,62,-70,82,74,-36,57,83,2,7,-47,-66,16,-23,79,95,-100,-61,
    -13,-89,10,63,10,-48,-89,-7,33,-54,86,73,37,49,35,67,-17,-38,-69,-60,97,-15,-1,-49,20,-85,-20,-49,62,-19,-58,-50,-56,53,65,54,57,76,99,-58,74,86,-84,11,-13,3,-70,70,65,-38,63,63,47,-38,66,-81,-70,-54,-29,92,-21,13,94,24,-82,59,-70,75,-12,-70,-82,-86,68,86,-22,-45,-11,8,77,-93,22,40,70,-78,-97,-12,-59,-67,-14,-36,-75,66,30,72,-10,48,31,20,-24,71,2,-54,86,-30,-68,-36,-23,-27,24,-93,-20,-53,47,-98,-31,2,-10,-38,-13,77,-21,-35,43,-91,-63,85,9,20,-43,37,-8,60,83,-22,82,67,-6,-88,-59,70,-81,21,-83,18,
    24,-62,73,-86,-47,-40,43,32,25,-62,-7,14,-77,54,-65,-67,-56,27,93,-73,57,27,47,-97,39,88,73,10,-39,43,-71,85,33,-98,52,86,-86,-53,-30,-8,-14,15,-94,-39,-78,-59,-54,-34,20,-61,-55,-23,19,92,-20,-42,32,-94,69,94,49,50,31,-18,52,-65,-79,18,-17,-57,-90,21,59,-83,34,33,-42,81,99,31,72,96,-92,-9,41,-59,2,-27,99,-29,-81,48,73,3,82,-23,38,55,95,73,99,-94,-6,10,-77,81,95,-67,62,46,64,34,42,25,78,-17,18,-20,9,17,3,-20,-83,-24,83,51,53,-26,7,0,-53,-42,6,-6,20,-19,-25,15,15,89,-39,-69,-25,-97,
    8,-47,-61,-74,33,0,-5,88,80,12,-84,16,-84,69,90,23,-78,89,33,28,-17,-47,-90,10,68,-23,-1,81,-40,75,36,69,80,75,47,-34,-73,95,-46,60,-41,-29,-24,-25,92,-82,50,-86,-93,-17,-5,-57,36,5,5,-44,34,-95,-63,94,32,74,15,-36,1,-37,30,29,-42,37,-59,-31,-92,17,45,52,35,-5,67,-6,79,-86,37,-33,-29,-57,-24,-95,-100,-35,51,-16,91,-33,48,-7,-18,-69,74,-8,68,-85,61,28,-16,6,-20,-81,54,99,-35,-15,-87,-97,52,-16,98,80,-59,50,-54,93,-66,-63,-40,34,-18,-6,17,-44,86,-15,-77,47,-87,7,-94,-54,-22,60,-55,44,97,-89,
    -1,49,-53,-51,-18,-11,99,28,-18,-67,65,-6,19,-52,40,-63,-44,26,-26,-68,-75,-60,-9,83,-14,70,-5,-17,-34,-8,-54,17,94,-6,66,-24,83,65,-96,-83,50,21,-37,-79,69,-97,-42,78,-19,-15,-90,-42,25,1,-58,-89,-77,37,-6,-59,82,41,58,76,87,-24,-96,22,93,-92,-9,-5,29,6,17,-49,9,27,81,42,-88,43,-100,37,-56,94,0,-80,-16,95,-87,-34,-12,72,94,-73,-100,98,49,94,-94,-60,-11,-13,46,58,-10,-93,-14,-77,1,-50,66,1,40,63,-52,-60,35,32,87,48,98,-73,-28,92,-46,-27,42,-97,-81,48,95,-40,87,-7,-81,-70,0,-43,-47,-99,-41,72,
    -45,99,35,3,-60,-30,87,-73,-82,37,-45,-9,-19,-39,16,-77,-35,35,23,-88,95,-38,6,66,-8,58,-77,-2,12,-17,-30,19,-18,57,74,-26,79,61,2,49,-2,9,92,79,70,8,54,87,95,-23,-100,91,39,-42,57,-68,-84,33,-18,28,68,4,99,-50,-39,73,-23,92,34,79,41,84,-60,34,15,-90,94,-31,98,90,46,50,33,-14,-92,-58,-30,-76,27,52,5,-5,-44,4,-2,-31,-70,75,-39,64,54,54,1,94,-60,-84,56,-65,-14,54,77,-68,-44,-38,-30,-36,4,40,-59,-68,-56,-2,-21,-48,54,77,-27,84,52,-14,1,-42,-59,-98,-96,-67,70,61,-32,-44,-33,97,-59,24,
    -41,-37,40,16,56,81,0,52,-21,79,-95,86,57,-70,70,-39,17,-77,20,10,-75,24,43,48,-15,64,-96,5,61,97,-71,73,-39,69,-59,69,3,-7,21,34,-28,26,20,81,9,-57,95,78,-34,15,-12,-56,-61,83,92,-23,47,48,34,-39,-2,63,86,11,-16,-73,-20,39,20,-99,-26,44,-20,46,-22,89,89,-27,67,-92,88,-93,-48,79,90,96,-92,-10,44,42,51,-58,-43,37,5,-6,16,85,-67,88,39,7,32,-81,-46,-90,60,95,-65,-21,-97,-77,-14,-93,55,-72,-97,15,18,0,-90,-79,-6,-33,58,-100,61,-74,85,95,-86,24,-46,-1,-5,-40,-39,-45,-44,-3,-14,-89,-28,72,
    -81,79,53,-78,-5,-29,-26,5,-7,69,24,-97,69,-14,82,54,-67,96,-69,-61,-53,26,0,9,34,8,-94,-80,19,-70,45,-10,-90,-2,-35,5,21,39,62,66,8,-62,22,-23,24,4,84,9,0,-85,49,0,-7,-51,-91,-73,57,67,-100,-72,97,-3,19,-93,47,84,-36,-32,23,78,87,-68,-83,9,9,-7,-87,-7,-97,-35,-40,4,65,54,5,26,33,-86,-7,-15,-58,-9,82,13,-50,-19,-3,-33,2,73,-55,-11,-95,-86,98,-34,8,-89,12,-37,-72,72,19,94,-22,-76,20,64,-62,66,-51,-20,-91,84,94,-41,-83,-57,-22,19,-84,24,-92,73,-10,58,-60,50,21,4,-87,50,-72,32,
    -4,7,8,68,-77,46,34,-76,79,43,-92,25,55,-74,68,-67,-3,37,9,6,10,-100,-36,2,50,86,-94,16,-12,-65,-100,84,-6,9,52,-31,-45,-61,-7,-66,34,-46,11,-11,-20,-68,-25,29,69,84,-65,-69,84,-48,34,-13,90,-8,-97,-22,-21,55,-86,25,-36,66,94,20,57,-12,6,-8,42,-82,33,74,-50,-92,3,71,93,-9,-98,29,43,-12,68,-15,33,23,63,-88,-21,77,-10,43,-5,-64,-37,-95,24,22,-3,-82,40,-70,44,42,91,-52,13,84,-9,-33,-35,86,56,-66,-29,-59,9,-66,-95,88,63,-5,32,-90,32,47,-85,-92,-31,12,-73,61,-5,71,-45,-62,71,20,74,-38,
    88,-61,-100,-4,-75,-29,-11,35,-43,-6,23,-80,90,-93,-69,-26,7,98,-18,28,11,-39,-58,58,85,-3,48,56,70,22,71,10,61,71,58,39,95,-53,-26,-48,41,49,-27,-17,9,-44,57,-84,-46,92,44,17,-95,38,-73,90,-64,75,-1,58,-3,-78,-32,-89,45,-74,50,40,-27,76,93,66,25,-82,-50,86,-74,59,-98,-20,3,-1,50,9,-63,77,-49,-75,5,50,35,54,24,-97,65,-30,-71,-33,-90,-46,-57,55,-79,21,73,-77,7,99,82,10,-68,-14,61,34,47,50,11,98,28,68,49,-37,23,-75,-33,-12,-5,-52,8,-42,-45,51,-87,-72,-28,39,-49,80,-10,-67,-58,-26,-29,55,
    -92,-82,5,-28,-83,-67,-60,18,-51,63,43,-32,-96,91,16,-36,-51,23,-85,-86,51,-60,5,-98,-80,48,88,-86,-78,-41,69,83,-70,-74,-45,99,-88,-5,17,-39,-89,-88,29,67,55,97,-69,56,21,98,23,-28,-62,-72,-73,-90,28,-85,-76,51,-74,45,34,-44,72,41,-45,84,-64,24,97,99,-11,-22,18,96,-25,1,53,48,0,-24,73,-10,-44,52,1,85,67,25,-12,93,-77,74,-98,95,15,-91,31,3,34,-20,-45,75,58,25,71,85,-73,-24,-14,-21,52,-89,69,61,63,-30,-2,30,48,38,-25,71,-36,-71,18,79,-61,-99,-66,25,-19,-59,-48,-61,67,-77,76,94,0,-38,-27,-96,
    -27,94,17,-64,-83,67,18,65,5,46,-12,69,75,58,0,-86,59,-13,-9,-60,28,43,-69,-5,67,59,-59,-81,-78,-34,75,-53,-39,-55,-64,-22,-88,6,95,-30,-48,-17,-61,-72,-59,-8,94,52,79,-14,44,7,-71,27,-97,48,-14,96,19,-40,-37,-53,-40,24,-8,-4,-98,-44,-98,97,26,-93,-68,18,-65,-75,-90,81,77,89,-81,21,-52,-99,-52,-97,49,-14,-100,-79,47,-37,-32,-93,39,12,-45,41,20,-43,-10,99,-36,-26,-31,-49,-1,-21,85,-72,-80,-96,1,-80,-95,49,-76,7,87,76,28,-14,-61,48,-7,78,-88,48,-29,-16,58,13,-65,74,87,4,78,-62,35,15,-34,7,19,67,
    28,-23,-32,4,84,-45,-20,-36,42,19,64,87,-51,28,-12,20,64,98,85,-100,24,72,4,2,-90,-60,-83,-72,-1,-11,95,-73,66,-37,31,2,-30,-89,18,64,-18,-66,52,-17,62,-60,55,-22,90,40,-22,14,-36,35,69,-26,27,38,-46,-74,-73,-51,-46,45,-36,37,99,35,1,17,-49,83,51,-97,-81,-35,95,74,44,85,15,74,-48,79,9,-79,-94,-64,59,-40,-85,39,-38,-31,-16,26,58,-64,-87,-41,53,-35,95,-43,-80,-86,-26,68,88,70,5,-45,45,57,87,-94,78,45,95,38,57,-90,-71,-81,-69,-35,-2,89,1,-37,-51,-93,-20,-56,-84,1,-90,-10,-79,50,-87,26,-94,10,
    -16,45,16,-38,-10,-37,52,-53,-27,81,-81,-44,99,-31,-54,0,32,-53,7,-87,-57,23,-34,-47,66,-61,-97,79,65,61,-11,-51,6,57,-36,48,-79,-84,48,-54,50,67,-45,1,88,-47,-99,20,0,-39,-15,43,36,3,48,-46,42,3,-67,8,-83,-26,9,-25,84,73,-76,57,42,24,55,-56,-57,-90,45,-69,-37,-2,51,63,59,89,-42,-4,-56,58,50,87,-86,36,-53,-69,62,56,6,46,-18,-18,3,-76,-94,-89,-32,-51,-79,65,-20,-15,-37,84,-100,-77,25,-89,-29,69,21,73,8,35,-39,55,66,76,-36,-75,22,46,7,78,22,14,89,42,15,62,7,96,-53,22,32,-100,-3,-43,
    63,20,-74,-16,-54,35,72,7,-58,-10,-17,6,15,58,-96,75,-12,-22,-59,-71,-80,56,91,-21,4,91,-98,-64,91,-1,-7,6,-80,-28,42,-34,-93,14,25,-51,57,61,8,-28,-29,-36,99,59,43,92,88,15,49,79,95,-95,22,49,-58,13,-100,-13,71,-80,59,14,-62,66,80,16,68,37,-23,-72,62,48,-56,61,-41,87,54,-53,-45,-45,78,-98,-40,1,51,-46,66,51,42,-10,24,1,-96,14,-80,84,-70,40,-26,-41,68,-64,-93,64,49,-82,4,-45,-83,59,-38,48,61,23,-51,12,29,67,-85,-29,-43,39,25,-87,-46,-3,50,-64,-63,-76,-4,57,-88,-45,21,61,-26,-75,69,-9,
    84,-69,-61,-3,6,-60,61,-64,60,77,-41,-83,-32,84,-69,22,-19,33,11,18,57,-93,-73,21,-38,49,-18,-12,26,-97,32,11,-65,71,60,41,64,22,29,24,-1,89,-7,67,25,-24,-58,-93,-91,53,-23,18,12,-43,-61,74,-94,-26,-85,32,-23,47,95,-36,-30,56,58,-66,-70,-13,10,29,-72,-96,-52,54,-20,-58,13,-58,47,42,60,-41,99,-100,86,-95,-26,-99,-10,51,48,-63,68,70,45,26,-95,75,-87,-33,-44,-58,-29,5,48,4,-1,-39,46,-53,-97,-42,6,-97,-42,-56,60,32,-55,-98,-64,45,-60,-96,16,37,-18,-27,13,-5,-8,69,-11,16,-74,89,-80,26,-50,18,25,-94,
    76,83,-91,87,-72,21,-29,-75,-24,7,23,-32,-37,91,-95,-3,-84,18,93,8,-60,-66,-76,-82,-76,96,-4,-74,14,-79,-68,-57,-95,93,-70,85,-33,1,-90,43,61,-15,-89,76,-24,68,-26,-8,-13,-81,-99,-21,-95,77,-51,81,74,46,-92,-60,67,92,-17,24,38,-35,-39,-95,67,-76,0,-20,9,-37,-44,86,-69,-18,-70,-30,1,83,1,-41,61,51,-60,35,-3,-100,75,-84,45,-89,41,-17,-24,54,-60,-57,78,-8,-25,-12,55,-16,-74,38,66,-44,-91,-80,92,-38,-21,53,-87,-29,-12,10,-76,-85,-21,69,-74,-80,-96,3,26,96,-2,5,-12,26,45,95,-90,71,-15,28,79,-54,48,71,
    9,79,-24,22,-97,64,85,-73,32,-36,96,10,-64,52,-87,62,48,-36,-81,88,90,64,83,52,35,-80,-20,-33,-33,81,90,76,60,-33,-50,15,83,-65,42,15,-49,-10,-74,-13,-58,91,-98,90,-93,-79,78,97,86,13,1,-27,34,82,-8,53,63,-17,-19,-25,50,31,91,33,19,33,-51,70,-76,-73,-90,-82,-82,-88,-91,-74,85,39,75,23,-95,-23,97,-9,-89,-11,-56,74,72,25,-99,22,-92,92,-92,27,-22,-43,-50,-46,-64,-40,-28,-94,-76,-67,-68,-91,73,60,-15,30,89,-18,-79,-100,23,-83,74,-52,-6,75,22,2,20,30,-70,98,39,-68,-96,75,-8,-24,82,-32,10,66,-71,-65,
    -22,14,65,-33,48,86,-33,-76,-97,93,-28,-51,21,94,51,-59,-23,33,91,68,65,95,44,-91,-77,-22,-71,33,96,-41,20,-25,25,-15,42,-26,-77,62,-2,78,55,22,-73,-24,-32,79,-31,45,12,-88,-34,-70,59,-90,-61,83,-60,-31,-32,88,-20,89,-37,-95,-74,58,-69,-50,20,-19,-72,-73,-97,-44,4,-76,87,25,21,-49,38,87,-19,-51,49,-27,-68,89,-6,-47,78,74,94,93,31,-80,51,15,-30,-77,96,-49,-49,52,-41,7,76,-54,-68,-51,-51,22,-63,31,24,86,56,8,28,-50,61,-42,76,55,-49,7,28,55,-78,50,78,71,-99,-71,23,-88,-12,-49,-90,-27,-48,-40,47,-11,
    43,-29,-72,-1,80,56,49,41,-86,25,-51,-83,84,77,72,-93,-21,3,30,-19,-16,-95,-55,73,-44,-44,-54,8,16,93,50,-41,-35,78,10,-55,-14,-41,38,-48,36,39,-31,20,-32,94,-21,48,-3,61,81,81,66,-74,6,74,-18,52,-65,50,-2,85,61,63,15,-29,60,-99,-18,-50,-47,-82,42,22,-9,-90,-32,22,58,65,-16,39,-1,-50,18,-95,-75,-100,10,-40,3,-92,97,-36,23,12,36,-65,13,-82,-63,66,89,79,-60,-68,90,-39,6,0,-74,90,-60,-23,93,10,-17,70,10,45,-70,65,53,79,-70,-72,-9,66,-37,56,36,0,22,-23,-20,14,-39,-78,75,68,22,54,10,14,
    83,3,-76,-34,-75,-13,63,-93,-48,16,86,-18,44,-23,-100,59,-67,-11,-40,7,18,-8,74,80,14,1,48,-64,-93,-90,3,91,14,27,-91,39,-86,73,47,-81,-59,-15,1,86,63,-46,-3,48,-5,9,-92,-87,-99,82,93,-33,-17,-7,4,-57,4,-93,-66,70,-14,-57,-39,-47,-84,-92,-28,-90,-6,25,48,-91,79,-55,57,-26,55,65,-60,8,-1,-67,76,35,27,-68,78,83,91,-88,53,-23,-93,-86,30,76,75,54,86,-79,-20,34,-70,-89,79,39,-62,-14,-43,-22,95,8,11,71,-57,90,-97,73,73,94,85,-22,-77,45,-55,6,21,-80,-40,-41,-59,40,93,-77,52,24,14,90,-89,23,
    -32,58,84,31,-19,-73,22,-16,-99,-53,-70,38,-22,-95,-65,23,11,56,43,72,-85,36,-36,60,11,-84,-15,77,58,-52,-47,-74,58,-63,10,-61,-84,84,-25,69,83,-43,-40,-39,-38,-5,36,74,-96,-69,-2,-81,67,62,32,-70,31,69,60,-11,69,-87,-32,27,-98,78,66,-82,14,-59,88,-3,-2,-52,11,12,43,-53,-62,47,-21,36,19,98,-49,-97,81,82,72,41,-77,-7,6,-9,-80,8,-79,38,26,-13,31,-34,85,81,14,-52,-7,-90,-5,32,9,26,-80,80,25,-77,35,6,-95,7,99,-71,-100,5,-28,72,65,-54,10,-9,33,93,58,-30,74,-76,18,-80,86,-86,4,-52,-8,24,
    -72,-83,48,64,-25,5,-77,-26,-14,76,31,11,48,-4,57,-89,40,-58,56,-50,13,31,-74,83,51,-87,49,-45,-39,-58,31,-59,-41,-69,-95,-13,37,81,13,-77,57,-55,34,-43,-7,-57,-80,85,-62,77,-65,51,-40,-38,-14,-89,-73,-64,-82,-60,-22,49,-19,-11,33,-61,76,-30,-80,-58,-55,29,-13,32,38,32,27,11,18,65,-12,5,-32,0,67,55,11,46,91,-19,86,-79,82,-32,10,15,-41,-61,37,-21,-19,83,60,20,-33,-2,-96,94,61,22,60,-51,-72,-72,1,47,83,-88,-54,26,-55,-68,47,-72,-48,-90,-5,11,-99,-67,42,-66,68,2,6,-65,53,10,81,14,85,41,16,65,
    22,-83,-36,-43,82,10,36,79,-5,-65,-93,47,97,-45,-89,98,-60,53,84,-92,-92,-10,95,13,53,76,-73,38,-30,-5,3,44,13,19,-99,-53,-70,-11,-74,-23,25,86,76,22,-59,87,21,-67,-7,-43,-59,1,48,88,-86,1,64,93,91,86,-11,-54,30,54,65,-16,53,95,25,79,-28,-50,17,-51,73,10,88,-54,43,-19,3,36,34,-97,24,0,-44,-59,-6,-53,-73,35,-55,10,-59,-89,46,94,58,-29,25,-69,-26,95,-68,-53,5,72,-55,-99,-94,48,37,40,4,-38,93,12,3,-61,60,-18,74,57,44,67,68,-58,13,27,-34,90,10,40,85,42,39,-57,-34,84,-56,-28,-16,81,
    -87,88,-57,58,1,-2,97,-87,33,23,70,-71,-10,-9,-28,55,-82,38,45,28,-70,-17,-78,69,26,-12,53,70,13,89,3,-22,78,-1,-64,-69,49,-67,96,-18,56,18,12,98,-91,36,5,79,-74,-50,7,-44,-15,-19,77,63,22,30,33,-65,19,89,13,-51,-12,49,-68,-63,82,-72,-28,-10,-1,36,-60,-40,-28,45,-60,-2,47,47,6,-67,81,-17,96,3,65,-18,38,-64,-29,-49,-14,11,-100,-30,0,34,-49,72,-76,50,8,-84,-90,32,61,-50,-70,8,50,-12,-7,31,71,-10,-66,36,24,72,25,47,75,63,-42,75,-67,58,9,-16,-17,-15,86,43,1,97,-24,-86,99,6,22,1,
    47,-84,-68,18,-42,-82,-93,82,42,-16,29,17,-53,39,-8,-20,-51,53,17,32,90,3,-24,-57,-48,4,-43,-96,62,80,5,9,48,38,80,58,8,87,40,-49,-77,-79,20,-30,60,65,50,-91,18,-81,-6,-39,23,22,4,27,-74,14,31,88,-54,-63,-50,94,27,-70,52,87,69,-56,-62,92,-35,59,-38,77,24,64,38,94,84,-68,-93,-41,6,12,86,-16,-74,-30,-27,-28,7,-25,-82,86,5,22,73,-26,66,64,-34,83,-25,-20,60,51,-56,-2,-3,80,-17,5,91,89,69,78,-26,47,-100,-1,71,7,74,89,-55,-69,-37,-82,5,-71,34,23,64,-91,55,24,60,51,-78,-42,32,5,
    63,23,-53,32,-47,-27,-21,-47,72,-98,12,-2,91,57,29,-94,28,-66,87,-38,9,51,-28,-36,-25,84,-85,-51,94,-1,7,57,-77,54,-59,-72,-73,-28,-66,51,-26,46,-51,-83,-44,30,-77,84,16,-90,98,-75,61,-78,-59,88,59,-44,90,53,8,-3,11,-17,-97,4,11,82,29,45,-15,-97,-56,34,73,0,-84,96,-64,32,59,86,57,-28,61,50,-39,20,-94,-49,-27,-34,0,84,49,-45,-59,-87,-63,70,58,22,25,54,-92,-2,-94,-76,-53,94,56,-94,81,-35,-22,42,-85,-9,-38,73,-6,-13,-60,94,-28,41,-51,-35,54,-62,-65,-35,12,60,-81,-28,-41,78,-4,6,72,-96,-36,5,
    -79,-6,99,88,86,-87,62,-20,1,54,-73,25,-5,-72,90,-50,-33,-75,15,31,37,-14,-96,48,-36,-48,-94,89,-43,-30,-6,-70,65,-6,-81,3,7,33,83,60,-61,62,-15,-14,91,-25,-12,-90,-48,-97,41,90,-58,45,-10,6,98,-3,-53,-93,19,42,89,-64,88,60,39,-53,-7,-25,-92,-16,-63,-7,71,80,21,-41,90,73,-85,32,15,-43,-71,58,15,79,-45,63,38,26,-43,28,63,-55,-12,2,-8,34,29,-48,-82,67,46,41,-1,-81,-99,-10,92,68,74,-40,77,55,-82,-8,35,-75,-93,-27,51,-36,53,14,-39,94,69,54,-20,-2,58,-2,-83,4,92,17,-25,45,59,-80,13,33,
    -20,-10,-12,-50,34,75,-25,42,-99,78,58,6,-55,-80,0,66,26,80,-84,-16,-69,-66,41,-77,3,-84,68,-38,-64,-67,47,-32,23,87,18,-43,63,45,51,16,-24,10,-78,21,-18,23,-13,-92,-45,3,-56,-14,-11,85,-39,92,54,-71,-94,42,-86,53,-89,37,-7,-19,47,-44,79,-2,24,-45,60,46,28,-58,-79,67,-98,-23,-78,-53,63,12,84,-75,-44,38,-94,-85,-67,-79,68,96,10,-39,-23,-91,-31,-92,8,93,-85,-80,-8,43,-37,13,-38,-83,90,-15,-36,6,97,-99,-17,-95,-9,-59,-80,24,-86,41,20,25,54,50,34,-76,58,94,69,74,67,-87,-31,82,27,32,99,-31,-83,-84,
    27,66,-83,-90,71,8,-96,92,-67,-82,-15,53,-5,-61,-45,82,-85,-86,28,37,40,-5,50,61,-23,-71,-7,-71,-49,-38,-55,78,-72,-38,41,52,22,45,-4,55,15,81,-39,-89,-80,16,45,-64,-18,-27,73,-78,21,-25,84,98,5,-71,79,56,-8,-76,34,-28,-62,-73,-76,61,72,20,-32,-12,-47,29,-49,-26,98,96,-38,-20,-79,-65,-45,42,10,-61,-7,67,-80,-28,23,12,-3,-90,37,-13,-63,61,-52,-90,-66,17,-50,-13,98,-47,13,96,-99,75,-71,-78,62,-16,65,-75,-25,-42,92,-53,-18,16,12,31,-22,-51,-81,15,-90,-33,-23,96,36,-21,-16,35,32,97,-17,33,73,-36,8,35,
    0,-27,60,27,-17,5,75,65,73,-13,-51,-49,36,-32,66,-2,35,-56,-5,-28,-25,79,59,-92,28,94,-7,-47,-41,-99,41,59,-26,-47,-13,9,58,-38,-73,31,1,-24,82,-63,44,1,35,-69,-3,-70,-45,72,61,-86,80,-58,-91,-74,-5,-32,-73,-64,79,-46,-10,-34,15,-52,-20,42,32,81,70,-34,70,-34,67,6,-2,-36,-12,53,-63,-98,20,69,-56,-19,95,39,1,-77,28,-20,-71,-82,47,44,-82,79,-13,50,13,-91,-83,-17,-24,36,41,26,-99,-70,-69,90,84,51,11,-72,84,7,19,-15,-18,-53,66,-37,-35,-35,-93,-64,-56,-54,38,57,-44,7,93,-16,-56,34,-90,97,16,93,
    39,0,45,-50,28,-19,9,-100,67,43,-53,33,-94,-35,50,-86,1,94,-88,91,4,-32,-1,-3,52,43,83,14,-8,0,8,-69,-100,-95,-67,-19,-14,43,-19,-95,-62,-20,38,-55,45,-12,-89,98,-65,-77,90,-9,-56,-11,40,48,84,23,-37,-72,-77,23,-41,76,-20,-56,-43,18,87,-10,24,-74,70,-38,-77,16,3,-66,-34,-10,-91,-44,81,-47,-3,21,-98,-67,96,-83,-39,72,40,-28,0,-80,17,57,38,-44,99,-38,-66,69,-23,-91,-63,32,95,4,-78,-95,12,-97,-90,62,-24,-88,47,-76,-71,61,48,21,85,-52,-7,2,57,84,11,56,98,-3,26,-73,7,63,59,2,-81,81,7,-16,
    36,-30,-54,64,-18,93,41,-36,6,-11,37,-8,38,-69,46,-5,67,-43,-48,65,55,30,93,-86,-55,52,16,-35,-14,76,1,-78,-54,47,39,-20,-8,-20,-4,99,69,-66,-9,-41,-83,-63,7,84,-53,59,49,54,-59,42,68,-14,-53,-64,3,33,64,-96,7,-90,51,-54,-57,96,-22,39,47,-100,73,90,-89,90,79,18,74,-74,-71,-24,32,-30,70,-48,57,-83,-11,-40,2,-47,-35,-90,-84,68,-92,-41,64,-13,50,-37,-61,24,-47,-50,-34,-67,69,93,-89,98,-31,44,-31,91,96,78,-91,-63,-62,11,-9,-45,-27,-41,-76,-18,-30,40,21,-80,-96,60,-4,-43,62,63,-58,31,8,-46,-18,29,
    50,3,20,46,-19,-19,36,-29,45,-73,27,-82,-62,51,-48,8,-57,73,-20,-53,-15,77,-43,-52,-8,-49,31,-100,5,-87,-19,-93,16,-99,54,49,-65,-10,-79,80,-31,48,98,-41,-49,-49,67,-6,76,99,-6,-86,76,3,-86,-32,54,45,-80,-88,11,1,-81,79,-45,25,29,-10,67,50,-30,88,-50,-80,-53,1,-29,66,47,-100,66,-59,-86,42,-4,-72,63,-49,25,83,-85,-64,-63,-66,68,-56,60,-3,34,-21,99,56,-32,-99,76,15,2,-100,-66,-99,-100,-100,95,-86,-6,-9,94,-91,-6,19,93,9,56,82,44,24,-74,-44,73,12,35,24,-32,55,25,-56,-29,79,96,-43,80,96,-43,75,
    62,3,-81,56,13,13,-24,-42,-77,84,-60,19,-40,-82,75,85,30,-38,9,98,-82,86,94,-59,17,43,-2,-3,39,7,-75,-46,62,-56,-90,-25,57,-62,-15,80,74,-23,-49,86,-5,78,71,77,-59,-68,27,-89,70,-26,-96,-13,-83,-46,85,8,13,62,62,-25,-94,25,3,15,15,40,-4,-10,-82,-1,-24,-35,-22,-100,-57,-29,-68,70,-18,-97,96,-62,-10,-87,-8,-73,22,57,89,-16,-68,47,61,-13,-37,-23,-72,11,67,98,-90,95,-37,-12,95,-42,59,80,81,93,83,77,83,-75,91,-25,-95,13,-68,46,49,-83,-6,-37,-44,9,40,-16,72,-41,-18,82,-46,-2,-77,-50,56,34,82,37,
    -20,17,15,-37,42,6,-9,-1,71,-25,-54,-28,-56,-8,35,1,53,-25,85,25,34,-80,-41,41,18,-18,43,-26,69,25,64,-51,-58,-21,-36,36,37,55,-64,-40,83,-66,32,-73,-22,-80,28,-69,-53,-34,56,34,-14,-85,-25,56,-50,70,82,-81,-5,46,-80,37,-75,-64,-75,14,92,-39,74,27,-53,7,54,25,-21,-65,56,26,1,-36,-88,39,-68,-61,-53,-18,-91,-71,-47,4,76,-75,93,53,-39,-81,-80,-95,-20,94,32,-20,-47,-61,-95,32,-26,14,-89,75,78,-77,14,-90,-85,61,-56,-76,42,-3,29,-30,-26,22,76,36,-7,96,-7,26,90,-74,-94,96,-35,-89,28,39,25,-9,-34,-44,
    -33,32,-34,-18,-55,11,6,87,60,87,-42,-65,-38,86,23,55,82,-84,-19,24,94,-61,20,-41,-49,-99,50,28,-56,-84,84,11,0,-97,45,45,-34,-96,33,-74,91,43,-87,-95,29,36,13,-37,5,46,87,-1,86,-40,11,89,-87,-39,-31,-43,-70,-94,21,-18,-91,-34,-72,-25,-30,13,-47,-86,-44,-33,71,37,55,-16,52,-40,-69,-9,60,-83,51,-29,58,64,-16,-73,74,-34,-67,-5,49,-58,61,29,-31,84,-58,-25,50,-50,42,21,39,-51,-94,-9,10,89,82,-78,-94,-14,-55,16,-98,-19,43,76,48,77,-77,-51,71,85,78,93,21,72,-32,-29,-78,62,92,61,-89,50,-96,21,39,38,
    -5,-3,76,40,-87,31,-78,9,7,22,-62,31,23,61,-32,-47,54,89,-75,74,-88,99,36,56,60,48,-41,16,-79,50,-46,-31,-52,83,-91,-87,14,83,-26,73,5,12,-44,-72,-74,24,-19,80,-35,-42,7,-71,-43,-57,86,-83,43,97,-15,17,47,-8,86,95,-25,47,61,89,31,87,62,88,0,-81,-83,26,-5,-50,-42,13,9,17,42,66,61,-20,36,-44,-23,-79,25,-75,13,-37,72,40,-89,-15,29,42,-27,-56,30,25,-37,-1,-97,10,2,-87,-77,-89,31,66,29,-56,98,-35,0,-24,-61,78,1,4,-59,-75,-55,-48,11,26,46,84,70,-71,9,85,28,12,-4,-70,25,19,93,-92,
    -63,23,-48,-64,40,53,12,79,31,65,-16,72,90,-19,77,-47,-93,23,-63,78,52,-2,63,-67,62,-41,-85,-60,31,61,-52,68,-16,1,56,-76,-94,-80,-96,-11,85,-60,-39,28,21,90,81,28,14,71,6,-82,-79,-78,-97,84,-67,-81,-76,64,80,-28,33,-84,25,-59,-60,31,-38,96,-80,99,88,-66,27,9,-76,-39,90,-10,84,-52,9,-95,-30,-36,89,4,-17,-87,-32,-85,-62,53,83,63,95,76,-53,-91,72,-33,-92,61,1,36,-78,26,49,-88,-84,33,-39,-23,38,31,42,-20,87,77,-7,-44,45,-17,61,28,99,8,4,-54,17,-71,13,26,42,67,-86,64,-55,15,77,-39,-52,90,
    -9,38,21,85,-82,9,14,-36,17,-41,-1,-70,40,-2,39,-56,44,-44,-27,-90,82,-85,77,48,32,-78,-37,9,-65,-37,-49,-22,-98,24,-37,-80,85,30,-64,2,89,36,33,29,-66,-28,26,31,-20,51,41,-85,19,-30,63,-49,-56,-73,12,-21,-10,63,10,-56,-13,25,-35,-27,-45,-47,27,97,89,-40,26,-76,-16,-96,7,65,56,-52,32,-25,-30,-5,26,14,-26,-62,45,17,53,-93,61,-60,33,78,-35,88,-68,93,85,73,-95,-36,97,-10,-32,4,-93,-76,4,39,-49,26,86,77,92,-87,67,-10,30,20,-3,-57,13,-70,74,78,71,58,71,8,31,77,24,-19,-81,93,37,78,69,-58,
    -31,-79,20,-45,50,13,-32,18,-97,50,90,0,94,3,-17,20,-66,-94,-22,57,14,-91,34,-61,-10,5,84,80,-17,5,-78,52,26,-6,60,-71,59,80,-1,62,31,-11,-85,-23,45,-50,97,31,-44,-73,-12,70,-64,75,61,-21,-20,45,-41,16,-49,-67,-32,29,-21,80,10,-61,61,-91,-99,-8,-49,68,69,48,-82,18,-21,-26,45,67,97,33,-6,58,64,75,-44,75,-9,-41,60,11,88,-60,-8,-1,-69,-47,-40,-68,-3,11,53,-82,-41,71,36,-10,98,-67,-42,95,18,4,53,-65,-21,61,-90,22,-80,23,-66,-91,63,-22,-40,94,-17,20,78,80,84,-17,98,-57,-45,-14,34,-95,-29,44,
    -100,89,-52,5,-76,-20,-33,87,-98,-13,10,88,-52,-75,-34,-40,71,1,33,49,-19,17,85,-21,12,92,17,-54,97,88,42,49,30,43,-46,54,75,-27,-7,77,13,-97,18,13,-72,36,74,99,-62,-41,1,19,76,-14,51,-12,-22,-80,-13,-73,-91,-71,76,39,24,82,-55,-1,-92,39,-71,21,42,47,86,23,-65,12,-26,-27,71,75,45,47,-87,-52,-64,-57,-32,-25,-30,-71,-44,98,20,81,81,66,-68,89,-43,-39,-38,51,60,-100,-26,96,13,49,-79,-16,-24,18,84,-58,18,72,-15,39,-53,8,68,3,6,-11,-64,-61,-93,-31,80,-36,-18,-58,-85,-57,95,-58,91,8,91,64,44,-81,
    -65,-20,-39,53,-48,-1,-8,-49,59,-87,7,-83,2,43,-43,-39,64,-63,25,-53,-68,-8,-10,-73,34,-67,-13,-23,97,83,-3,84,-36,10,38,16,61,82,20,-28,95,-73,-10,-51,70,47,10,-13,36,-65,34,68,-20,76,47,-86,-91,86,-8,58,70,41,43,-66,-97,-67,2,17,-85,-78,89,63,-51,-21,-88,72,78,23,-41,15,10,45,-65,90,21,35,57,-18,21,-99,92,-57,42,35,-23,-3,20,32,-86,-12,6,56,-49,56,87,-37,-20,66,86,-9,-67,-51,-64,-32,-61,9,-97,48,91,-23,49,83,20,-9,-29,50,41,43,-66,7,-69,40,-37,-18,-52,-97,-2,28,69,-16,71,-98,-15,59,
    -78,-23,-32,78,-75,-41,-45,75,95,27,-82,-82,-71,59,61,-37,-81,93,-96,-66,-73,-96,-63,25,-15,-94,62,-44,60,-53,-84,83,24,-16,61,2,-4,68,-23,-9,-5,-5,-39,-23,-93,-78,40,78,67,-4,-88,95,1,-50,20,86,-92,-18,94,69,-18,-90,4,58,-53,-35,-40,43,85,37,86,-20,85,-53,9,44,-79,-98,22,89,-2,-14,84,51,-12,-44,-11,-3,-9,-64,-82,73,-54,-26,31,-55,-9,44,40,-24,-19,-22,8,-82,-23,-82,14,-1,-80,-64,-12,-30,-25,-76,74,63,-20,-37,-88,71,-49,82,-4,-50,-44,80,-5,47,24,-64,-77,57,-34,-16,28,44,2,-58,-57,-26,-69,-17,44,6,
    7,18,-79,87,34,-66,11,85,-84,59,87,-75,39,-17,24,-85,-29,0,25,37,84,-47,-19,-62,-5,-24,12,26,59,-44,84,-34,27,-94,-94,-87,92,17,50,-92,-72,38,85,68,-79,10,35,-56,-90,-40,81,46,13,-85,84,61,-9,-4,-13,-97,4,-76,-31,-69,-70,-25,-56,-78,44,47,82,73,-15,68,-7,-42,-70,-72,2,92,-59,-65,38,54,-98,22,15,-6,-82,55,-3,22,79,18,6,61,46,2,83,-10,-51,65,-85,34,-15,-40,92,-33,-11,-54,-41,-18,-18,97,-64,-16,19,4,30,-11,-89,79,64,90,-50,-30,51,96,72,-14,-62,74,-97,-46,8,-59,14,-47,8,55,-1,68,89,33,
    17,-74,70,89,-18,0,-22,-7,-68,-58,-65,-18,-36,38,-22,89,76,-32,63,79,-26,-77,-80,41,-24,29,-4,28,-51,-14,-87,66,-36,83,55,-2,-16,86,43,-32,80,30,50,-55,-32,80,86,96,-52,49,75,23,72,48,-36,-99,-71,12,-19,-22,50,-6,96,66,-22,4,-36,14,-10,59,34,-30,-11,-16,-33,9,-84,-47,5,-36,54,-67,39,-21,81,55,-68,-90,-32,13,-60,70,59,88,37,89,-8,53,-45,34,13,89,57,-98,-75,-76,12,-59,-70,69,-42,36,-98,49,-85,35,5,99,-3,73,-36,37,43,76,-22,-68,65,70,86,73,5,-1,14,-86,1,40,90,65,33,72,-13,43,9,-59,
    -55,-24,77,50,-24,-26,-25,-8,64,70,-32,-58,-97,86,64,89,59,-79,-60,-27,-65,-7,65,-74,-41,-49,98,-2,46,-41,39,91,36,16,-7,64,43,68,-44,-41,39,25,-47,-6,-37,-83,-65,-78,91,-25,-53,78,68,65,56,79,-84,55,-23,14,66,69,-42,-46,-63,51,-82,80,-28,-73,91,63,4,-56,-43,67,-86,-8,89,-95,67,88,83,-13,53,40,-33,21,-53,-4,-64,65,-35,-54,-80,3,97,-62,-17,-79,-35,-73,84,21,-29,93,-12,-15,-15,-71,-58,-96,-82,78,-56,23,-30,11,-55,17,-93,-67,-66,-27,-21,-46,76,-72,-7,-89,-50,10,-62,86,84,-38,80,-28,-1,-83,-46,42,-78,-76,
    20,-34,47,90,29,92,-41,36,77,-7,61,-92,0,-11,-63,45,-47,-61,-93,-57,-75,91,5,-95,-84,-95,75,-30,-1,49,-6,-81,67,93,61,-4,38,72,84,15,-83,-2,24,17,87,13,-38,-60,52,-30,-16,29,-87,41,-13,-71,-2,-38,-1,97,-89,45,68,78,39,-19,-74,-23,-47,-90,92,-77,8,-32,40,-4,81,55,-12,33,77,24,-85,90,-82,2,-80,16,64,-29,14,75,-83,-66,5,56,16,31,85,21,41,77,44,-98,-54,-63,-50,79,44,38,-35,21,-85,-20,-37,-67,82,-17,49,46,7,-85,-27,24,-50,30,-68,18,-39,-83,-61,54,46,84,8,44,-79,58,24,65,-51,89,-62,-36,
    -31,1,-51,-97,37,98,1,-56,66,26,20,68,-44,52,-14,-31,21,25,75,-33,9,-16,64,-18,-58,-12,99,-57,-71,89,7,-2,-9,8,-47,80,-41,54,76,-75,-20,-4,93,-12,-100,-21,57,-27,56,32,40,-82,-84,4,0,-89,44,-48,-46,73,-59,14,-77,-16,-78,-24,-36,-19,82,-60,6,-38,-12,-49,50,-12,-18,-41,61,-61,92,-46,-43,60,10,9,71,-45,61,-74,80,55,40,-96,-61,62,-20,4,-56,-37,-4,2,-23,85,54,80,-75,88,-61,39,-73,-69,93,-64,44,-45,-2,15,10,-41,-7,-9,-86,-67,-53,-94,48,-21,-38,44,-6,58,98,72,-5,4,4,21,-7,43,-40,-28,27,-95,
    9,-29,-88,-93,86,23,-82,-20,66,85,-35,65,91,65,-56,-95,-39,39,63,-40,-37,-41,64,67,32,-91,10,44,-18,89,1,43,-40,13,2,99,88,-28,-69,6,-43,-52,-29,-100,-86,16,-95,-25,-93,21,-13,-30,80,-48,89,-36,-39,-1,-92,-5,-11,9,-62,1,74,-8,0,63,65,-17,-31,74,32,41,75,46,-43,32,73,-84,-47,-39,-14,85,-35,-25,-99,-22,26,-91,26,67,-30,16,-31,-55,-91,21,-92,74,-95,-23,48,37,70,-25,-65,79,60,60,95,65,73,-67,51,-62,-40,4,-31,87,14,95,6,36,11,27,81,20,-99,41,-54,-94,71,47,-5,41,74,-18,21,34,-58,16,0,68,
    2,-97,-94,62,7,-73,49,73,-78,-92,10,34,35,-57,-94,36,85,-95,-6,8,52,41,-51,26,-77,22,-87,-82,91,13,86,-7,16,44,-45,75,72,57,49,94,65,-41,-20,-48,-46,-61,89,91,-56,-65,-1,96,77,49,-26,-100,-77,-13,70,-86,52,56,-41,-32,1,15,96,-27,-76,45,-81,41,56,-48,93,62,-9,34,54,35,70,-47,-17,47,54,57,-1,78,97,-30,44,49,-74,-96,70,-21,-29,66,4,-53,-37,-24,88,71,28,81,33,-81,-32,-13,54,-62,-7,89,85,-53,46,-16,77,-5,54,22,-55,33,-22,-85,-88,1,33,-31,48,96,45,-12,67,-27,-79,-48,-8,-11,40,98,27,-67,
    87,-36,-68,-15,49,-90,81,-45,-16,-74,40,14,93,53,67,26,74,15,22,19,-97,-59,44,76,-7,-64,-34,85,86,93,70,73,-90,-97,-90,59,65,91,66,-99,69,7,15,62,-88,-18,-12,86,49,62,-43,4,55,-99,-20,-99,89,46,86,75,-8,9,0,2,-36,10,-87,-71,-46,31,82,-77,-62,-3,38,-98,31,78,40,-68,41,97,36,-4,98,68,-3,39,-33,-64,14,11,45,66,65,-91,-23,78,90,-17,-39,72,-42,0,-79,96,-46,4,75,95,36,-32,92,-76,16,43,44,66,-18,-89,-98,49,-26,-1,15,-61,-40,44,69,-50,-73,-69,74,-14,-17,95,-18,37,-49,-91,32,87,-23,-23,
    63,46,72,7,12,-94,-29,-34,55,45,65,-77,85,77,67,54,79,95,37,53,-67,-80,0,15,58,51,-23,-58,-10,54,-29,-47,0,-57,60,64,-50,-69,-70,57,77,47,80,14,24,48,-80,3,-5,58,-44,28,78,8,-57,-12,-41,-28,-17,49,79,54,54,-69,50,15,96,52,-2,-74,9,-25,74,-10,41,-2,-62,62,54,-15,20,62,-87,-50,-29,-92,-9,30,-19,26,-68,-40,-68,86,-9,82,-47,-13,34,52,-34,96,79,92,-14,21,-58,-24,-65,96,61,7,59,74,57,-70,82,-100,12,-37,-22,-4,-25,11,-65,67,-55,-12,-94,32,92,-76,28,24,-84,-86,97,-41,-58,-68,-45,-97,39,66,
    29,-52,48,-37,-51,-87,-21,27,9,54,90,44,-27,-64,85,-20,68,77,4,-52,-47,-27,-38,-50,32,-44,-18,39,-89,-79,-42,-8,-78,-94,55,-77,-81,-66,2,29,41,-7,-75,-86,-19,62,-54,-51,-8,3,-3,45,76,11,96,-40,19,-70,51,30,-48,-91,-78,-74,-84,77,-51,87,64,51,-32,-43,-56,-6,-29,25,-92,-30,26,0,25,75,-54,-47,-14,94,13,57,-76,-36,-13,-72,-74,-39,-46,-58,91,3,81,55,-45,-50,-88,-49,-4,-65,77,4,5,-45,-95,82,-69,-97,-65,-31,97,48,-73,73,65,14,-98,43,76,-44,85,67,12,18,-26,19,68,-62,70,-36,-27,47,21,31,-97,-22,13,86,
    -19,49,-45,-70,-51,-18,55,66,49,57,9,-23,66,-54,-56,-70,65,-30,49,-15,8,19,2,33,-81,-77,-84,74,1,30,60,34,-69,-85,16,-68,-2,-29,51,99,-71,-88,-24,47,-41,-28,77,76,42,26,-87,-98,-55,15,35,-84,-10,52,90,91,-66,50,-23,-83,66,93,-99,-84,-35,52,-85,-54,17,-9,93,28,63,22,-44,-43,-52,-31,-41,-55,37,46,-38,-73,-50,-48,71,-16,55,-52,-47,21,-58,-45,-11,-41,59,-44,-43,-24,47,-50,-96,-38,24,-88,19,24,-18,30,69,-81,-24,83,-54,27,88,-83,63,43,-82,-83,-84,-88,-76,-43,-29,35,13,28,64,-88,30,20,-74,-94,33,-3,30,-85,
    -73,-49,86,-45,-65,-68,82,75,-98,-2,-30,72,-33,-14,84,-9,43,-93,78,-92,-13,42,-80,69,-37,-54,75,96,43,5,63,70,56,49,77,43,33,12,-30,-13,-90,40,-41,-23,-74,-57,20,21,-98,-50,29,-11,93,1,-42,-44,47,-67,-96,-10,-10,67,-88,99,16,42,42,-99,-46,-87,-11,-36,5,48,-7,32,44,65,-95,46,67,-13,88,60,88,-54,68,36,-68,72,78,22,-9,43,21,-93,-15,-84,61,-9,81,-50,7,-14,98,52,70,-6,-31,76,-59,-64,-37,81,97,-97,-73,17,39,-89,-10,-30,34,33,65,-93,41,-98,-77,-46,-7,-96,4,52,-9,2,4,-39,49,73,89,42,-91,-96,
    23,58,-92,50,-24,-1,14,18,21,-52,-49,-14,-45,44,-60,-69,98,33,-65,2,-15,-22,57,41,-8,6,14,-19,-52,-24,-62,71,-66,-54,-27,-38,-3,87,80,-81,87,-68,5,95,76,-2,-74,27,83,-87,-71,-79,-56,86,62,36,-56,-71,69,-8,5,-93,-85,-9,-95,-59,54,3,-72,-14,-26,68,18,31,63,-53,-71,89,74,-87,54,55,-66,98,-6,96,-14,38,77,-44,83,-18,-85,50,26,73,91,-20,-72,-80,66,2,88,-63,-67,3,-64,-37,-8,62,-72,98,17,-38,97,-89,-90,-17,2,88,-9,-63,-78,-41,87,48,84,79,-72,-88,51,47,-34,39,36,-1,94,72,14,-14,34,42,-16,-97,
    56,-67,67,67,-31,-31,55,60,6,77,71,-7,26,55,-76,6,19,75,-95,85,-34,41,-15,-40,13,51,-2,-53,-6,35,-49,-50,68,70,69,-63,-61,-24,50,-3,-94,21,42,32,-23,67,90,96,-6,96,34,61,37,71,-79,-97,22,-28,-98,68,-93,-95,71,75,75,40,-87,66,-83,63,-37,-77,-64,-94,7,13,25,-51,-38,-81,97,96,80,-13,19,54,-10,-59,-74,-8,-90,-67,-50,81,60,-75,-27,25,92,-10,-60,-45,-35,-23,-87,24,90,-10,74,-96,10,71,52,-58,-42,-29,48,48,-35,26,93,-25,11,-57,-92,-28,-80,-67,49,-88,-24,90,-80,41,-33,85,-34,-91,-24,-60,-86,38,63,18,
    32,-78,42,-19,22,-93,59,15,-66,-77,10,-58,-53,31,-73,96,-5,3,86,15,-55,-47,1,-37,-85,29,-45,81,-33,-82,-1,99,-60,-59,-68,15,-52,44,-70,-18,67,-7,-24,-86,-76,4,-90,19,59,49,87,56,54,-12,-81,69,69,-26,50,-64,-55,-50,87,-63,-57,72,52,-56,16,35,26,-17,-72,55,-3,-96,-89,-41,-25,70,-92,-38,27,15,2,-2,-16,-29,73,-65,-41,70,-63,-53,-93,-68,-81,60,-24,35,-5,55,70,-25,-38,-81,-21,-27,-22,6,95,-61,69,-26,54,-77,73,90,-5,-2,77,6,-32,66,53,-25,99,24,87,-73,-41,-18,34,81,9,96,-100,88,-79,-69,-5,69,-78,-84,
    -57,28,-61,68,18,-14,-34,-4,93,-14,-86,-2,14,65,-77,1,93,-66,36,27,-84,-55,-24,68,86,-3,99,-67,-82,-79,-51,-38,1,-60,82,-80,-21,-51,-32,-76,-13,-18,22,-99,-52,-3,3,93,32,91,-28,0,-64,48,-32,74,98,-80,-93,-84,93,-92,-70,-5,-51,-87,-33,-20,-86,-13,4,1,69,78,3,69,-72,58,14,12,1,87,-88,-11,35,-68,-36,-15,-48,-29,2,98,-20,32,-55,-19,-3,12,13,11,99,69,-87,20,47,-32,-58,27,78,56,-61,-21,-5,-49,68,83,-64,-68,-32,-60,-44,22,-62,-12,-45,-17,69,52,47,-66,-84,46,-97,-71,-33,2,-3,-39,-70,-73,-83,-31,6,-35,
    -27,-74,-52,9,-89,16,49,-33,-61,40,55,-54,-77,-24,50,71,-38,66,-31,-35,95,-12,67,-56,49,97,-77,19,-33,29,84,40,8,-68,-99,19,0,50,38,39,-10,-55,37,-34,-27,-12,89,35,-46,-42,-100,-98,47,19,98,-52,-31,-78,67,36,3,3,-72,-89,-13,-71,-18,88,31,72,-21,74,-83,-83,-60,-58,-43,-71,77,-37,-61,29,-35,38,-99,-84,87,-30,-62,6,58,41,-90,38,5,-3,19,39,-63,-50,12,17,-76,-19,-14,-84,24,43,-55,-47,-94,-63,35,-28,75,-12,-12,62,58,-22,-31,16,71,-69,-94,-24,28,25,68,66,75,-20,-65,52,13,21,68,89,-36,66,-57,-30,-97,30,
    -6,78,-82,34,-7,28,-88,62,96,36,45,-98,64,-75,27,32,43,-46,64,78,-94,-22,-1,27,-33,15,-7,-38,38,-52,44,84,78,14,19,-29,-58,-17,85,-62,-81,30,92,84,56,19,-32,-1,26,-67,-22,32,11,29,-89,30,-55,-44,93,35,4,-11,19,-65,-96,-62,6,98,-26,-8,-63,-55,-78,81,81,30,-47,-50,-70,79,-17,-40,-37,46,89,-25,-72,86,-17,73,21,40,-37,-59,75,-33,-69,-19,-35,-43,25,-46,-45,48,-64,-64,-22,-59,-62,-40,-80,-27,20,-65,-81,10,10,-52,-52,46,-79,70,-14,-64,63,-39,3,46,-6,21,56,-80,75,-89,20,63,-53,98,56,-14,-89,76,59,31,
    64,31,93,-74,31,-58,72,4,64,-42,41,27,-29,-56,-75,66,17,81,86,-55,92,6,60,-60,-44,17,-22,-33,-55,89,51,-91,20,44,-64,-49,86,-92,56,50,-33,-3,-71,-10,-7,-93,-44,-89,88,94,-44,81,0,-84,73,9,33,-97,28,31,92,-21,40,-35,-76,76,16,62,85,-76,-35,-96,-27,-54,-6,67,-47,3,30,-58,-3,38,75,50,54,0,-41,-60,-45,87,71,-53,-81,-37,12,-57,-60,81,57,-75,-95,-78,81,-69,-31,-73,50,22,30,-20,16,-72,-82,-57,78,24,43,89,16,98,28,-61,98,47,3,10,42,43,-57,0,20,-99,22,-47,-68,43,-20,82,-82,-89,-86,34,91,84,
    30,21,8,-27,10,77,-76,-62,68,-78,86,-29,-16,28,66,28,80,-62,-19,-45,-9,-35,98,72,-53,68,35,-87,-45,26,97,85,-53,-43,-42,9,-66,-66,47,-97,-44,-15,-74,41,66,45,-79,46,83,-46,-99,-73,19,52,51,66,20,-14,79,75,64,28,12,63,85,23,-28,72,57,71,27,-86,-91,5,7,75,-50,-20,-27,86,34,-25,-87,-47,79,64,71,-49,-98,2,79,66,-70,-9,29,67,-86,-99,39,72,24,-34,38,33,72,97,8,74,-23,34,60,63,-91,25,-84,40,-11,87,91,-9,-59,70,-43,-29,-86,38,90,-72,39,30,52,-84,48,-10,49,-28,-61,-90,-53,16,44,59,-21,5,
    -15,-53,-55,26,86,88,-82,-73,59,-73,-50,73,66,-59,53,-43,23,6,-75,-29,48,75,-4,88,85,-5,-44,29,-46,36,-66,-9,35,-69,-82,22,-81,88,1,78,67,4,3,-15,45,-43,-5,-80,15,20,-57,-37,-5,39,-97,-68,-66,-40,61,-59,48,47,-16,-17,78,-46,57,98,-58,-89,28,62,15,84,-53,-88,-59,-6,-68,8,-85,-25,23,-38,-85,27,-5,-99,87,-92,94,35,8,-21,70,-14,33,28,-64,28,39,65,-58,-94,49,-11,-82,42,36,-50,-50,51,25,73,65,-8,-48,-88,-54,-61,-79,-60,26,29,-29,-3,67,-95,77,56,-15,-32,21,27,-26,-78,68,-56,16,4,-6,18,-93,71,
    -57,73,-84,-4,85,-38,-13,58,-46,-86,-61,26,-37,59,-17,92,15,20,-88,88,-53,-62,-90,67,-18,-22,-76,28,96,-69,-1,91,56,-33,-13,-6,29,75,-48,36,41,-56,-38,56,-97,-3,48,18,-83,12,6,-84,50,-32,83,84,-2,-93,12,-6,91,63,-15,47,-69,25,-59,60,-48,-54,-52,93,-10,10,1,-55,-93,-99,15,76,13,73,92,-85,-59,76,99,-9,35,-37,85,26,-74,-78,-74,-43,-1,-81,70,51,-35,-82,-4,7,-19,97,52,-60,98,-33,-83,-37,-60,-91,-22,-67,37,-71,24,-75,-8,-39,3,19,-64,29,28,35,49,-2,-13,66,-31,35,-26,50,33,-74,90,-17,-54,59,-1,-14,
    21,77,-28,-90,59,48,-65,-97,-90,-9,22,98,-80,-97,33,21,-47,72,-12,22,-40,14,-76,93,40,67,-72,86,26,27,-75,99,-43,-51,-90,-84,97,97,-81,59,-12,-6,57,-39,-51,-57,82,2,67,-78,77,-73,36,-99,72,-71,68,53,67,-53,-20,44,-54,-11,93,-44,-43,43,-46,-71,2,-6,-25,-88,55,24,-93,38,-74,26,12,-97,54,-51,57,-22,-70,25,-69,97,24,-36,42,71,53,35,79,63,-70,85,92,33,-20,-33,-3,-65,91,-44,25,69,82,90,25,-12,-61,-18,67,-31,59,50,-82,-16,-86,60,-93,20,-52,86,-17,-22,24,-25,63,56,-6,-88,43,-63,-32,-31,-94,-49,-89,-17,
    -9,-98,65,-90,-29,25,61,-59,61,27,-98,68,-53,-50,6,-18,-20,-70,9,-56,38,-97,-44,-18,-60,-23,3,-53,28,14,-18,71,16,-52,82,39,25,43,80,-62,-30,82,-94,-30,84,-36,52,-35,-53,-38,-39,85,-83,17,-33,-42,94,70,-43,-26,-16,39,-54,-100,39,80,39,-36,23,20,2,45,-46,-40,67,-61,25,20,-44,24,34,17,9,51,86,29,61,33,-1,-82,-93,36,10,-95,-12,-51,85,-72,14,-40,0,68,58,6,-71,25,-3,6,97,53,82,-17,-78,-9,87,-91,72,48,-6,-76,67,1,60,-23,-41,-52,26,96,-72,92,-91,-20,13,19,87,-6,-56,-16,-100,94,90,34,-23,64,
    -75,-84,73,50,65,-33,74,-16,21,34,13,32,34,91,28,63,36,-11,95,-51,-92,82,43,-95,-81,-5,99,-91,-71,-72,73,-94,-55,-1,56,62,18,30,-54,91,-36,11,23,51,2,-96,14,-62,-7,9,-13,-46,-56,82,-41,-37,-23,-90,-76,6,-62,49,13,-65,-52,-31,49,19,52,95,10,-84,-42,86,67,61,90,-67,51,-65,-5,-61,-59,-9,73,0,54,-49,10,78,57,1,-21,-78,-12,28,-8,-62,-53,96,85,9,-88,-56,-5,32,57,-63,17,8,-75,64,-1,-34,-45,-27,19,-91,24,-71,39,-67,-18,-81,-44,71,-1,0,61,-2,-4,46,59,-40,42,-93,44,-1,-56,-38,-92,-79,78,-41,
    -60,-66,32,59,95,-92,40,-13,-58,-77,-42,-50,46,57,50,7,-93,-54,-95,66,-42,48,73,-97,-53,70,-83,-93,43,95,-33,83,-71,51,42,-75,12,-65,12,54,-90,70,-96,56,-21,6,-85,-14,52,-80,-96,-90,68,78,65,68,0,82,27,-57,78,46,-73,-93,-2,21,-68,62,56,-4,16,18,-82,-28,-26,-3,78,-59,35,-18,-38,40,-8,82,-30,-90,2,70,92,82,-87,70,28,92,78,78,-34,62,40,74,-89,-92,-7,-71,-20,19,-21,-42,61,66,40,75,-42,85,9,28,95,-88,-50,87,94,-84,10,-26,-92,-12,-47,-26,-98,45,-51,-87,54,-6,95,-66,-87,-26,-55,26,92,85,1,-49,
    22,-89,79,17,75,82,-43,21,98,67,95,58,-93,-100,85,-91,46,86,75,-48,-20,70,-14,-55,96,31,72,88,-31,-27,-61,91,-64,71,-39,-89,53,-30,32,3,-63,-20,-39,96,-68,-54,-95,30,84,80,82,-36,2,-31,10,-50,52,-18,-61,73,7,-70,65,44,-99,78,55,6,-52,-60,-91,-15,-28,-77,-19,4,-79,-62,87,-94,71,69,-78,-75,90,-16,-24,-5,66,15,-32,-74,45,-67,70,-1,-89,25,57,-89,17,19,-4,41,42,29,98,15,20,85,-27,-9,54,-4,-32,-3,80,44,92,-1,-41,-40,25,57,-54,-5,-92,-91,72,65,-79,-58,-64,-31,83,78,51,81,-54,-77,66,19,14,-27,
    -85,34,22,48,-21,14,99,-10,26,24,99,24,71,-93,-14,-5,25,7,37,13,28,21,-56,-69,-46,-10,-46,73,-91,20,-54,77,-45,68,-23,34,-66,-24,76,12,52,76,-63,23,35,23,-82,-88,82,8,26,-38,81,70,-54,-65,-88,0,-92,-79,-79,-94,50,-72,26,-73,14,60,-45,-10,-75,-93,-82,-38,-18,54,37,1,18,-29,61,-56,-15,42,66,31,29,78,-68,38,52,-95,44,-46,85,-77,-66,-1,83,-11,41,-92,-51,60,-78,31,-34,11,84,84,34,45,81,-80,-61,-1,3,69,-22,-13,7,82,-8,-97,88,-71,-74,-78,-20,62,-36,-78,22,13,34,-3,-56,-100,60,81,36,-5,26,17,
    -33,66,69,70,-13,-53,-42,46,81,2,49,69,-16,28,44,64,-58,-92,86,64,-27,-28,-87,17,-76,74,-50,61,-79,29,-70,-12,-53,99,-90,34,98,-80,-20,31,-25,81,1,59,9,97,23,51,-95,-38,-32,-22,34,-19,47,59,-93,98,-28,-72,27,-46,-32,74,-46,-21,-40,-96,-49,40,36,26,21,89,-63,83,86,61,-14,43,75,54,-79,-39,88,20,72,95,-30,-56,-76,49,-49,-8,-25,5,23,35,-39,75,-25,49,-47,49,38,-57,32,-24,-96,-82,-81,31,25,-60,92,13,13,17,-40,83,13,-16,33,64,29,-92,-79,-48,-4,-17,79,71,-68,85,20,23,-72,-96,99,84,-77,-81,-33,48,
    -89,-89,-87,-24,28,-27,-40,-58,10,45,-94,-61,5,-72,-57,-99,-37,75,25,-5,12,97,70,40,-98,70,-24,-75,41,43,25,52,54,-10,-71,-65,15,-59,-23,-75,38,-17,-84,95,-37,-40,97,26,87,-26,74,-1,-29,44,91,-27,-34,-81,-50,-93,62,27,12,-32,-83,-7,3,-15,-14,80,10,-76,-84,27,-81,-69,39,68,58,-22,-58,-16,77,-34,-72,20,39,-5,39,90,-46,-47,-31,66,21,-61,-89,77,24,-3,9,-66,-27,25,13,-55,57,-96,-87,-33,-18,-92,51,-41,-26,31,31,65,-74,-30,7,-67,23,77,51,97,-84,63,74,40,-88,-17,26,-14,61,-60,31,18,96,-4,37,-21,-44,-12,
    -10,-18,-81,-78,-52,98,-8,55,-69,68,-16,34,65,0,-3,-9,92,-38,74,19,48,35,11,-69,-95,7,27,42,86,36,-70,-23,-82,-98,-49,-34,-100,-5,74,83,-37,-42,17,80,59,67,-29,3,-71,98,-78,29,33,-15,60,-9,45,-61,-67,31,75,-84,60,94,-82,63,-88,70,-41,86,5,-26,-55,-78,-45,56,41,-22,-41,22,-24,-18,-49,62,19,-89,-47,-36,-97,-62,48,78,-94,-40,24,76,-76,37,46,-65,23,-49,9,68,-74,16,24,67,95,36,-58,-77,70,-7,85,-11,-43,90,54,-40,81,-46,90,87,14,15,64,-10,-96,10,25,27,14,-65,-52,40,51,24,-41,-2,-40,1,22,-70,
    47,-41,-28,-96,-50,78,-84,-69,-68,-94,70,-2,-27,34,89,-23,97,14,57,11,49,57,-97,-47,-19,-86,51,-6,-32,-27,24,-85,33,96,71,35,74,87,18,58,45,-12,-43,71,-77,-54,-100,-80,60,9,-17,-38,-34,-62,15,48,-96,-82,94,-28,-8,18,-61,77,-33,-90,12,93,49,-70,52,-53,18,61,70,-7,-93,70,-35,19,-20,48,81,-2,38,-52,98,43,-33,92,67,11,-37,-93,-12,30,69,52,-25,-29,34,79,-30,-48,40,-60,-2,-1,62,63,-81,94,-36,52,-7,-98,53,-9,-3,72,-64,-35,-17,-1,-76,23,-71,45,-25,-44,-84,-91,-64,-62,-87,28,-22,11,-72,93,-73,99,87,-9,
    3,-68,-7,-44,-24,43,-72,12,-92,-37,-89,84,38,92,29,65,-52,46,26,-64,84,40,65,-85,-97,45,60,-70,44,47,21,99,32,67,-44,-92,10,-64,20,-30,-48,-17,-46,90,-25,83,56,-25,-19,-18,12,-82,74,-71,85,78,74,45,60,-30,-56,-66,-31,76,53,-23,36,-85,14,-92,-15,66,-9,-9,56,18,-74,12,94,-40,47,6,-70,21,35,67,-49,-39,12,-88,31,8,98,-100,-63,51,-22,73,66,44,34,-97,-38,-75,-54,-82,-4,-76,-17,-10,-64,82,96,-34,3,-17,-67,7,44,-3,-29,-73,-94,-31,79,-57,72,-91,68,-10,53,2,-7,-85,-20,-9,86,76,-33,-79,-34,4,-97,-86,
    70,6,-51,56,-87,45,-47,-16,72,-41,5,51,-46,-23,13,-77,67,-82,-23,-88,-66,57,-45,72,-15,23,93,3,-73,96,-31,49,-46,18,5,-80,-85,59,-44,87,-82,62,-9,-75,-61,56,-52,59,74,-23,71,60,87,79,32,-28,2,-23,76,29,-27,97,-22,-20,-32,84,0,-17,-5,8,-29,-35,-30,14,-10,-38,-30,-10,73,-56,68,44,57,55,-25,-11,-73,77,67,55,-42,92,5,-11,-28,73,-75,24,-44,20,-67,-21,85,-45,45,-24,17,67,66,-10,64,34,87,21,-59,-38,-90,-79,92,77,28,-50,22,-67,-9,94,-94,16,-29,-85,-64,4,-6,74,59,92,-50,-71,59,68,-29,23,-45,-42,
    -56,-4,-27,-45,-31,-83,84,98,-33,-42,-69,11,-47,90,-73,-24,57,-36,80,-49,-62,-9,-57,-60,20,55,60,44,-22,-85,54,23,-36,79,30,33,96,-34,31,-84,25,15,27,30,-43,54,-94,-86,70,38,-83,60,-71,-39,52,2,68,13,98,-54,28,52,21,92,32,-49,26,28,-30,-91,44,47,24,-29,-23,81,78,-65,95,48,73,-87,-39,54,-74,-87,-92,94,26,-94,92,-45,59,14,99,-9,17,-23,71,87,-13,16,-14,63,39,63,45,-31,50,-8,-82,23,-43,-21,-70,-65,-8,-62,81,71,-55,74,-74,56,-60,77,99,9,-45,70,-3,-6,-62,35,-43,78,-1,54,-53,-51,99,-83,25,-44,
    -52,55,-56,-7,45,-75,-36,42,51,-58,-2,43,-81,-51,-47,-74,-80,2,20,-42,-63,30,88,-12,-16,88,-10,-17,57,-85,-8,-94,22,36,-1,-81,13,15,62,-35,57,-88,60,28,-38,-87,7,-66,15,-73,-56,53,9,85,93,94,73,-17,-71,30,-2,21,-12,72,-91,39,92,23,-94,54,40,63,18,0,-56,80,14,51,66,81,30,-89,-14,-60,96,-20,86,21,63,-85,-97,-86,-11,-8,38,-2,83,30,-79,90,-64,61,5,55,-86,1,-13,28,-48,-46,-39,35,-83,48,27,65,80,13,-14,-5,-20,-11,9,69,33,-52,-80,-31,30,-59,-41,67,55,16,-26,69,-82,61,-51,-78,67,10,57,36,
    -90,-64,1,42,1,87,-62,82,-71,47,3,62,47,23,83,-22,17,-58,97,-28,11,-29,93,29,84,-58,3,4,4,-87,-60,-33,-51,42,-91,51,-19,-53,-15,10,47,88,-75,-6,-36,-92,24,81,-97,-27,-95,14,44,98,95,-19,92,-50,85,-4,-37,-75,63,13,19,73,-84,-99,72,1,11,19,-59,-12,66,-43,-51,-58,90,-48,16,95,18,-88,45,-35,-7,37,15,-22,86,79,-44,49,-56,75,-26,-40,76,-53,13,-60,66,6,-20,-16,16,29,27,6,81,-57,-46,-49,-45,99,16,1,-11,-68,31,-25,63,87,76,-93,15,-49,-81,-57,-50,-16,-17,68,90,16,-47,6,-55,32,65,79,75,-81,
    82,82,70,-1,-65,-41,-17,67,-14,-54,54,-37,-95,-79,-34,-76,-35,-32,8,0,-64,-50,-84,-59,9,-86,73,-26,93,0,-7,27,-17,-85,-74,-30,-25,-39,-63,61,7,44,76,-36,65,94,-60,-70,-38,0,31,-49,51,-1,-8,-88,13,-82,-14,-42,18,31,86,53,46,-88,-76,73,-74,-87,-13,33,57,-37,50,-25,-90,90,-43,-28,-9,88,-25,94,-12,20,-94,53,38,-56,-88,-92,75,98,62,-27,62,38,-53,88,51,34,-26,-39,97,-76,36,59,66,93,-68,-43,34,-41,-49,74,79,-91,27,17,-47,-9,26,-72,-11,-60,-46,-96,78,1,92,29,-65,-82,42,84,94,-22,44,61,-76,28,70,10,
    87,74,-16,67,83,63,-64,-63,-45,14,-83,-4,54,-29,-48,32,24,97,-86,59,-85,8,44,62,87,-60,23,63,68,-7,-27,7,-81,9,-26,-97,-76,-37,92,-69,77,61,-20,32,85,32,-84,9,29,82,-79,-3,-9,17,-41,-70,57,82,93,-23,-73,-82,84,-53,-21,-89,-98,3,-26,46,35,-49,-41,67,35,-56,-1,52,6,-71,34,-21,26,-75,96,37,-93,5,19,-100,34,-2,70,18,-3,-51,-19,-49,5,55,97,92,59,-43,59,-6,-47,-42,98,11,-61,33,90,17,10,38,-46,18,-57,25,-30,77,-76,-59,-52,-27,-58,29,25,47,-63,22,39,96,31,-2,-58,37,9,-59,-52,48,26,91,
    -82,88,29,-28,-42,25,50,29,2,26,22,2,99,64,-68,-24,12,-31,51,-49,-83,-18,50,-89,71,-89,-48,-80,59,30,63,77,19,-8,-98,77,-83,4,-42,-28,-70,80,74,81,45,58,-42,-43,-73,-91,-40,96,-57,62,-92,67,73,-88,39,-15,43,2,-86,-86,94,68,91,64,-28,-50,88,-46,82,62,-64,27,21,-54,36,0,-93,97,-51,-98,11,-43,69,-15,-79,8,70,64,-90,36,30,57,5,22,-27,77,24,-39,-68,6,-77,-80,86,-4,-34,22,97,-75,-29,98,-73,83,-45,97,-80,-72,-43,42,45,-32,-22,-25,77,83,97,-50,13,-27,11,45,80,-14,17,18,83,83,-60,32,60,-36,
    30,87,99,37,36,-81,17,94,13,-38,-86,91,38,43,27,-13,93,-60,61,-44,37,-7,-6,-46,-89,-71,89,-97,61,1,-33,-57,-12,-34,32,77,-63,-50,23,50,12,37,-6,-50,-68,73,-10,77,65,-49,-67,2,44,-21,8,7,9,-51,10,-78,50,78,-34,90,96,50,67,-66,52,-10,36,-83,79,82,-81,-89,55,9,88,20,60,73,74,-44,-47,-66,63,62,-65,-26,84,85,-96,-98,76,-100,5,-5,86,-43,38,23,-26,17,-43,-6,-19,-87,-97,-79,-15,16,95,-88,72,-100,-54,36,62,-18,62,-2,-81,18,-47,47,18,58,-57,-43,67,33,32,94,-50,89,-12,31,-98,-57,5,-60,59,0,
    -48,32,0,-50,-80,-86,32,34,-36,4,-48,-31,-49,22,79,-54,79,47,79,11,41,82,1,81,-35,-45,-76,70,-5,36,-78,-1,-80,22,50,-60,-12,-66,74,5,38,78,74,42,0,6,40,-20,-47,20,91,46,-46,-56,-21,19,-48,-45,-10,47,91,12,-53,-89,87,49,-97,-73,83,29,-16,-26,7,-89,-84,60,17,56,-60,22,-72,83,-80,-18,80,-1,-98,-68,54,-56,79,-2,8,78,-39,-5,-21,-35,-25,-37,46,59,-11,-94,22,5,66,-61,-87,-94,61,-58,41,33,-24,21,-68,-70,53,39,-26,-15,89,83,-85,-50,-70,-53,-33,-43,-38,-86,69,51,72,-9,-92,38,31,-79,44,-56,-85,-63,
    78,-56,59,62,-26,-36,-47,1,1,-58,-16,-31,-55,-34,16,-36,-24,78,-22,-55,29,-50,36,37,40,-81,10,-64,-36,-22,-26,94,22,85,8,96,-51,14,-3,3,8,33,72,53,52,-12,-82,-72,-34,-52,-75,47,-49,61,-64,91,33,46,-72,49,-76,-46,-5,46,-61,-97,-5,-60,17,44,43,78,-22,-33,31,82,55,1,-90,73,2,87,20,53,48,8,-4,-67,-45,-24,82,-69,30,-23,30,69,-67,-75,62,2,69,5,-20,99,73,64,81,-20,-83,43,-46,19,30,26,-76,-69,-65,-27,-36,-58,49,99,-27,-20,-24,55,-99,9,80,63,-88,2,-79,-56,-47,46,-92,-65,-74,-74,30,-68,-3,-39,
    59,22,-56,46,95,-40,40,-4,-41,-87,76,36,69,30,97,1,-55,61,55,-34,58,-91,-88,66,-56,-61,44,26,-29,-58,87,82,16,83,-20,-37,-56,-80,59,3,86,-12,91,7,70,41,-92,-85,2,16,-18,-40,25,-54,79,21,-15,23,-53,9,17,87,-57,-67,70,24,-52,-86,-4,-92,-30,-18,-52,-39,89,-82,-98,50,-67,-43,-34,-33,-31,43,14,-52,-36,51,24,-37,-88,-59,-50,-44,-73,73,32,-73,39,-72,87,-91,-37,-65,-77,-96,-95,77,54,-61,-66,72,-42,4,15,72,4,79,24,-72,95,-12,-78,-55,44,-99,-82,-72,-72,10,-91,-84,19,-76,-97,42,-72,-91,-28,83,-100,6,55,-42,
    62,-77,-69,-33,-98,7,-53,97,95,-79,95,-8,-78,65,-28,-97,-25,-19,71,47,-43,74,-59,-14,35,13,-31,35,72,76,94,34,99,-75,53,54,-16,53,3,-69,74,-50,23,49,68,48,4,-57,81,-25,42,-61,-99,84,25,-11,-3,46,-76,-31,22,-82,56,-26,95,9,80,-69,-38,-65,-37,89,-14,86,-10,-46,-14,94,-51,-32,-79,92,7,74,76,-68,-37,-75,78,-12,-53,-48,-42,-97,26,-94,-36,-42,-63,-21,-6,0,-80,32,-61,10,38,25,4,87,93,-23,31,0,-49,7,32,-33,-15,-38,55,32,15,65,-13,93,71,51,-48,-91,82,-2,-39,-98,-70,0,12,68,78,68,55,71,-55,-13,
    -76,-51,46,8,16,83,71,-77,-33,-14,88,54,-69,12,-42,-17,-27,-60,81,34,-57,63,87,7,31,65,-24,-61,-12,-27,-22,12,22,76,-79,90,60,92,-87,-73,-70,54,34,-87,-82,92,-3,-9,84,-70,77,-73,46,64,-13,-71,-19,63,-32,70,88,-2,-18,11,-73,3,1,39,-53,-33,18,-71,73,-48,-57,91,96,-8,-18,81,-26,-41,-92,-80,76,-53,-50,-43,62,-30,-73,51,21,-38,-86,48,65,15,-13,65,34,-95,94,7,-90,-63,-2,-94,-19,32,87,56,44,48,76,20,95,78,29,58,-99,9,9,22,71,23,-78,-12,90,9,-47,-23,-34,-100,84,-24,89,-65,-65,-29,67,-78,-73,11,
    22,55,83,18,-14,-35,-72,-13,74,-11,61,45,64,83,85,54,-56,-61,83,10,91,20,39,80,55,74,-97,-26,48,-70,86,-29,38,21,41,76,86,69,-85,60,-42,76,-43,-78,59,-57,28,-97,-66,-36,-35,-75,84,56,57,39,30,61,-87,-69,43,51,-46,-67,-27,95,61,-89,64,76,24,74,52,81,48,-37,76,-72,-82,62,-8,-64,39,-24,92,-3,-85,-25,-90,81,6,53,32,60,39,57,55,0,21,-29,-23,-55,97,81,-74,45,97,55,73,-85,17,-34,-49,-43,42,96,6,10,-29,16,-57,77,-79,27,-11,12,-15,96,-87,58,-33,-58,-97,-84,-77,81,61,72,36,-14,40,-46,52,43,
    -37,-53,39,21,9,-90,89,52,87,-90,-21,-72,23,16,-76,-12,-26,-57,30,-23,-41,5,-41,-80,-22,47,-41,70,53,63,13,-84,-90,-47,-11,-29,-37,-22,-77,-97,-11,-45,-69,64,71,-92,-96,98,-49,34,-73,11,91,-62,-17,-79,86,42,91,39,6,-43,8,-32,10,97,40,25,-24,15,-20,17,70,-36,-67,-6,-28,-63,-56,-25,23,-29,86,-34,10,-78,-12,-52,16,-69,39,-78,-12,99,-57,50,49,35,-72,-75,-50,8,-6,73,24,-21,-81,48,16,-37,-76,91,34,-90,-43,96,84,-3,96,1,29,-12,23,69,-13,18,20,-64,-47,48,-87,-44,-92,-41,81,33,38,-100,-67,-94,-37,-43,97,49,
    -32,-93,-2,-48,4,-54,53,85,34,-71,55,-78,-1,-25,-90,5,75,-76,13,35,-17,-6,-80,74,-6,-46,80,9,11,30,-90,31,-11,-40,36,93,7,-59,79,-59,-78,-14,-85,22,13,-74,79,-12,-98,-8,75,-63,38,96,-89,84,-98,44,-7,13,-26,-45,-55,63,16,33,8,75,74,-61,68,97,25,84,-29,-62,62,-98,-22,64,-6,-46,-99,-68,-98,65,68,-96,9,-87,-31,83,-32,-34,-2,36,99,-94,11,74,-54,-20,23,23,-36,46,62,78,48,92,42,94,98,-5,-74,-100,-40,94,56,69,59,-74,4,-73,-8,-98,64,-56,61,-73,70,59,7,-7,-18,23,-61,96,1,39,89,95,33,39,
    -57,11,-8,3,-43,-52,-75,-84,-74,-19,-57,-29,-16,59,15,97,87,-63,56,-54,30,90,70,21,39,-77,-88,28,71,45,-81,-86,-44,11,-31,-35,12,-6,-19,38,76,-24,61,-88,-64,-72,61,75,-35,-31,-79,47,-41,43,-80,98,67,-68,-22,-10,77,-2,4,85,61,73,-50,-27,-80,31,64,48,-92,-23,12,96,6,-27,71,23,42,44,-29,53,-12,-57,4,-93,-24,34,-3,-95,84,53,91,46,26,-59,71,-2,25,35,-54,-15,-87,58,-19,-29,83,-96,94,-23,0,-83,-69,40,-39,-13,-1,-11,-79,48,-6,58,1,-15,-44,80,-21,-73,-22,-96,63,-23,-59,28,87,-26,99,71,-70,45,-52,30,
    15,-69,-77,76,-30,22,-35,-8,-29,59,-98,24,97,-42,4,-24,-15,35,-68,-100,64,-27,28,51,47,-21,-26,77,-75,-25,-41,40,6,-18,-32,77,57,-15,-79,-72,96,-77,4,93,-67,-91,-31,18,-4,-47,-29,-40,26,99,-89,-75,-69,-62,-98,-92,13,-86,0,-81,-4,20,48,-47,-95,69,-67,-99,44,38,47,77,-1,68,-52,47,-78,19,-93,48,-30,70,74,-47,-92,28,-39,21,42,13,-7,91,-67,-59,96,-62,63,82,92,-41,-28,39,-63,71,-41,-63,18,-19,8,-23,-70,30,-53,56,84,-92,-16,97,29,79,11,-78,70,44,-36,18,35,79,0,-21,-62,72,70,-73,-5,-71,-36,65,11,72,
    -58,41,-45,-58,-51,-61,-50,-15,-64,79,-36,47,54,86,-56,70,-43,79,49,57,-42,87,-18,-72,67,77,-43,-69,-5,68,56,37,-39,-89,31,-38,-98,81,48,-62,13,64,38,-81,3,82,89,60,13,-10,17,71,-71,51,51,96,-19,60,80,76,-19,36,-35,-6,-1,97,57,1,30,5,-9,95,-79,-71,-86,24,-89,55,-16,24,-55,54,-53,-73,-43,98,-77,38,11,-97,14,92,-9,80,-14,42,29,-57,95,-41,0,-13,-45,22,16,-79,98,-20,77,-65,56,74,41,56,-99,98,6,77,37,-83,32,-97,-91,-76,83,-4,18,-36,-9,14,24,92,53,31,-34,-31,4,-84,1,-19,-49,-90,-92,44,
    66,61,43,-28,38,-68,-58,-29,87,-49,-53,23,-1,17,87,43,-69,11,-13,84,94,5,-94,-1,21,-41,32,25,-31,40,-31,87,-46,64,-40,92,48,-46,-85,36,-43,-38,-41,57,80,-54,-48,63,10,39,48,-44,-56,6,55,17,65,-60,-58,87,32,-36,74,86,-72,86,79,29,-60,46,-35,-2,-91,-24,-93,41,-26,-41,-44,36,50,-96,93,46,-38,0,-85,80,40,-42,67,-75,-26,41,-89,-46,28,42,83,20,-11,-100,70,-50,-24,77,91,3,88,-53,-61,-10,-96,84,-64,-82,-15,-96,98,77,-38,65,-98,-12,-41,66,42,87,8,26,-41,97,-22,-70,99,7,59,42,-90,48,90,-99,90,46,
    86,79,16,-77,83,15,52,97,-68,7,-15,-9,-27,-73,-70,33,5,-10,-17,84,20,-18,-57,-21,-75,-47,79,67,-94,70,-35,92,49,81,67,-16,-4,-28,-19,29,79,66,72,-48,45,-97,37,-49,-7,-80,87,65,3,-70,96,-20,-65,28,-53,-59,-2,-36,86,99,-55,53,-65,-6,-75,16,75,56,-18,47,60,-73,50,-2,-70,95,18,69,60,73,51,9,-47,86,-63,-48,-72,-13,16,66,-14,14,-29,21,-92,-51,89,83,-95,-29,-18,66,-50,-67,-84,-67,-72,-14,54,41,-40,6,-50,-35,-8,-13,-30,-28,74,86,-62,-88,0,10,-15,-40,59,-26,-57,16,-3,-22,-66,-1,11,-50,-16,-9,37,-61,
    -68,49,45,-66,-86,89,21,-16,-38,-53,-77,-48,-41,-25,14,-4,36,-75,22,31,42,-81,-91,76,-29,20,-73,55,64,16,46,48,-35,-9,83,-69,-67,4,16,-5,4,-9,-1,63,66,-86,-88,2,-61,34,-66,33,6,-57,10,77,16,-11,-16,-20,57,-69,80,-78,-26,63,-47,7,-80,21,54,24,-88,-46,-61,31,-80,-49,-67,-89,86,-81,45,-8,-85,-93,-79,31,-52,5,63,-95,-12,43,79,63,-41,-68,-30,-21,6,77,55,-30,-69,94,-99,-97,98,87,-86,84,-94,11,-72,-79,-30,-99,4,18,6,-81,-25,47,15,-46,10,74,-61,32,-95,45,9,60,67,-8,6,69,-5,4,8,-38,-60,14,
    25,-80,-12,96,-79,44,-34,-20,16,42,-73,-69,48,37,57,87,-79,14,-16,31,-74,-48,75,32,73,-77,-63,-19,-15,-23,47,-90,-2,87,58,19,32,-75,51,-100,19,78,83,-33,15,40,-45,-11,-94,-9,-80,-68,-5,47,64,-80,70,-99,-47,-45,-69,-99,-82,-19,40,76,-100,-76,53,52,-76,-28,-18,-93,40,50,-1,47,-61,5,90,-41,37,-14,58,-46,6,29,-93,60,84,-62,13,54,19,-47,-17,-28,-70,-64,76,-46,9,-42,-38,-99,60,13,48,99,-81,38,-90,8,76,69,14,-17,-2,22,95,-66,12,8,41,84,13,-76,-92,43,-40,84,50,21,94,-88,22,55,25,-78,6,-4,13,17,
    57,-11,-14,-29,72,36,93,19,70,-42,79,11,-58,-7,-13,-50,88,48,86,38,-79,32,2,-4,-61,80,18,46,-24,-69,-37,-67,21,1,-43,-55,-63,-98,65,59,-40,44,-77,-46,89,10,-44,30,-90,-58,-32,-68,27,71,-20,66,3,98,-36,79,82,-21,65,-45,32,74,0,21,76,-83,-19,-63,14,4,-57,-45,66,0,85,77,-6,-46,61,73,77,-59,40,-20,91,56,-89,73,-64,28,-72,20,2,-19,-58,-69,-2,23,-80,64,79,-37,-80,45,15,-95,-26,-90,-89,-65,83,40,-72,-25,72,-80,84,84,93,-80,-36,74,40,19,-45,82,50,5,-43,22,-78,88,85,-58,34,1,-53,60,-37,-89,
    96,-54,51,-76,-26,24,-56,-42,-40,90,-70,-24,16,22,95,71,-43,-3,-72,14,-81,-50,3,57,-8,89,-42,-8,49,-27,-45,-3,19,-94,-26,45,30,18,-97,42,-40,85,-81,76,8,-86,-1,65,-36,-20,-69,-65,30,34,92,75,-25,-98,-81,-75,-25,-26,-26,47,-68,48,-8,15,-81,-52,-43,-21,-67,-24,-92,41,-57,-41,-42,7,39,-10,42,-30,76,-13,97,-48,-11,-84,-71,65,-58,3,12,-26,-48,-96,89,-29,-96,-53,2,-62,-25,10,31,18,22,90,77,-39,-68,-28,31,-92,59,28,-40,-52,-4,-59,65,38,45,-23,65,49,-66,54,72,90,53,-26,-72,81,37,12,-49,-41,-46,81,20,-14,
    -47,-96,-6,64,-16,7,64,81,0,30,-29,-55,59,-64,46,45,43,18,36,-52,45,64,81,-18,-72,-67,-7,-18,-86,65,68,-81,69,-85,83,-94,-26,47,39,74,-71,-90,72,89,99,18,-14,-6,89,-78,-6,34,39,-24,68,67,9,-39,-98,23,78,70,42,0,37,-23,6,-89,-76,-55,38,-94,-93,10,95,-94,80,-67,-48,69,-44,47,3,95,-77,-77,14,32,-16,16,-93,-85,-13,49,15,76,-74,-79,40,-98,-34,-22,-92,-27,88,55,-68,68,89,-16,-10,97,-69,45,92,-46,69,6,38,-95,-77,45,-80,-38,-54,35,90,-28,56,30,-73,74,-92,87,-100,48,-5,-68,-31,-64,-32,59,-67,0,
    4,-75,6,-75,-17,-55,-69,-42,90,-49,-28,-63,39,-37,-39,-53,45,-60,-78,54,-72,74,-46,-25,-94,23,11,-74,-66,44,26,-9,21,85,16,56,-70,47,-85,72,51,-61,-91,-10,-46,-77,37,-100,-37,11,6,-57,-15,60,18,-57,-64,81,-30,-30,25,48,13,98,-15,-70,55,15,29,22,-12,-68,13,-51,22,68,-28,-88,68,-12,-25,-74,-17,13,86,-98,8,74,83,30,97,-39,-21,10,-89,64,-8,18,-68,74,-60,-28,-94,6,21,-19,-26,46,-55,94,86,20,-80,-31,-67,58,-77,42,33,59,-28,30,72,3,-60,83,-80,-15,2,-96,-41,-6,-24,-35,-100,49,-2,-26,-5,43,-80,-67,-36,-60,
    3,-51,99,-22,43,-16,-63,68,-86,-91,-77,-94,93,95,-9,-53,99,2,41,-25,20,-6,77,-82,68,-28,62,41,-42,78,81,61,79,32,-61,-25,-84,-71,-57,-70,-62,-34,89,83,-38,-68,82,-87,35,24,-59,-45,-82,-82,-75,38,-58,39,-21,0,17,-87,-87,-51,45,-95,24,14,-66,67,-4,-76,85,-15,-92,99,-30,42,-87,-95,-34,-94,-88,-64,-76,89,27,-34,29,58,67,-54,71,80,-53,-31,85,71,35,71,-10,-69,96,-24,-31,56,-73,39,-2,92,44,17,98,8,5,-78,-3,-68,-11,-74,91,-92,-75,14,40,-28,35,26,96,-30,-51,-14,2,97,14,71,53,-6,-38,-96,-14,58,-27,-63,
    -34,78,-89,15,63,-48,-6,6,-40,19,-80,-99,43,8,-21,39,78,-20,78,-68,-22,-8,55,83,86,-83,39,-75,75,-88,-86,93,91,-75,9,6,-70,-97,-88,-10,-26,-68,-57,69,40,-26,-39,-29,55,-61,3,-67,-17,-41,68,-78,-72,8,99,-96,20,-87,97,-37,90,-42,69,-80,-87,81,-37,-61,66,6,-91,-42,81,-78,29,-12,-87,85,-27,-4,96,-59,-30,-76,49,-31,80,-78,-18,78,-15,-75,-64,-93,-55,-98,88,8,-59,6,-85,-50,65,48,-76,-54,88,-63,-69,-39,-14,79,2,56,-96,-96,-74,36,-74,-40,14,11,-15,3,18,-17,5,-41,91,46,65,58,49,-18,-42,73,29,-54,63,12,
    -41,-51,-56,62,-95,-100,18,83,36,44,96,-97,-93,-67,6,-22,16,63,37,8,-39,-46,-82,62,37,-23,-12,-82,-25,-49,-18,35,0,26,-51,-43,26,67,93,-85,11,89,-82,-30,-78,24,48,-9,-61,-63,51,-48,-56,69,-85,81,98,3,51,-26,54,85,-91,-94,-36,-42,-37,42,77,-92,57,-60,97,75,10,72,-49,-89,-37,-10,-52,66,-5,-8,35,10,-75,-14,-87,28,60,19,14,21,-23,30,31,-8,-28,8,1,-70,0,-50,-95,-90,-78,-91,73,-63,-49,74,-97,46,-82,-9,-44,-56,77,-79,24,37,-8,90,-90,69,20,41,-86,93,-51,15,-77,49,65,80,11,40,89,-63,77,-59,-89,33,
    39,-71,-76,96,-75,-99,69,50,-10,-86,40,0,35,-87,-59,49,-94,-58,-36,81,43,82,13,-94,22,3,-57,-49,-4,-46,84,87,-16,60,35,9,61,5,-89,-97,19,4,-45,-46,17,-52,-96,-25,-58,-80,-92,85,54,21,44,-72,76,39,-20,-76,94,-84,-88,-70,77,47,-9,-10,-48,3,-6,23,59,1,30,-24,50,-14,3,92,-42,11,30,-87,-68,-74,-59,-91,65,-27,-67,59,-10,97,41,-81,45,-67,9,49,-12,55,-27,-53,-43,55,-25,59,-59,78,3,-1,-11,-67,-36,-27,-89,-42,-18,77,31,-32,36,73,-35,-70,92,-90,-85,-46,12,3,61,37,2,70,-8,77,29,33,7,85,84,96,
    -82,-51,-31,-70,7,4,59,90,72,-53,64,89,-23,8,52,-8,14,-36,47,-24,-99,-51,98,-7,78,28,78,-15,13,62,-67,-17,63,3,-35,-30,59,-76,-39,31,-28,77,-28,1,-63,-76,94,4,-60,41,32,-59,43,30,-14,21,-90,-84,59,23,79,92,59,-6,-53,76,65,-94,53,78,-11,77,7,62,-22,44,86,-76,-52,79,18,32,-80,-39,-37,-41,34,-27,-25,-55,49,-94,-62,-40,1,-15,-64,-82,-8,41,96,81,18,3,-5,49,99,-18,73,-100,-87,-9,32,-67,4,95,-56,91,-79,20,88,70,-22,-74,-70,31,-36,-34,49,8,60,-3,-59,-22,52,37,-73,52,71,53,4,84,96,88,
    69,-47,84,-86,96,-95,-14,84,27,64,63,57,-4,-73,75,97,35,35,-5,76,66,-53,13,93,51,-64,-2,-45,-80,-5,-56,90,0,80,-96,-4,37,-58,-20,-36,-94,-57,73,-46,-78,48,-48,57,-16,-1,-14,-50,46,51,95,-2,88,-54,53,60,41,49,50,-59,29,6,-11,-82,48,69,-18,7,65,-45,61,87,56,65,45,92,-36,83,-58,63,34,89,-39,-26,-65,66,-65,28,-32,85,21,49,-56,-90,68,92,80,-98,-1,45,-90,-39,-16,66,78,-71,58,95,64,52,58,-1,-7,71,73,29,37,60,-91,57,-2,31,7,42,41,-25,34,-27,-23,-66,70,-13,-53,-45,-95,77,-64,63,72,-99,
    -33,30,-48,61,1,77,-58,91,38,-49,48,-64,34,-93,78,-72,-18,12,1,12,-2,-28,51,-3,-21,57,-25,15,-28,-53,16,40,30,68,1,83,-2,43,26,-64,46,-73,-28,-19,-66,-98,9,-31,-34,62,81,65,86,-16,-38,-35,-59,-11,33,14,-11,-99,-46,-81,22,7,54,-80,-98,-19,-44,48,-40,-20,81,-54,-18,-58,15,48,57,-52,-35,43,33,-20,61,74,69,-6,-60,-42,-5,-6,-71,-83,-99,84,89,-97,-83,-55,4,-71,-23,-63,-25,59,-20,43,-40,89,91,25,32,-24,-95,-7,-97,75,39,-57,85,87,38,-85,56,-9,51,46,-53,68,91,3,49,-31,40,-76,80,72,67,-60,61,11,
    18,-54,87,23,-9,90,-50,-69,86,88,70,24,-97,26,67,6,-28,66,-74,-84,-31,75,-63,62,99,-83,-14,19,-90,-100,82,-20,-54,69,55,-11,12,-94,20,-2,46,-58,74,49,69,-7,-45,-59,-40,-19,-91,-19,-92,-54,95,-93,-84,-66,-22,78,-66,-40,-42,-20,82,-87,69,-6,71,-58,44,17,-16,18,-82,-95,63,-27,-53,75,6,-92,-91,14,-45,-96,-26,-29,90,4,-99,-76,65,-41,-44,-1,-28,78,93,-56,72,-11,61,-92,59,80,-86,22,-95,-87,-2,-36,-79,-41,30,-72,-85,56,-1,-94,-39,-48,-70,-74,11,-13,25,84,-35,-30,80,89,11,-7,97,-30,-75,-89,92,83,-76,-58,-53,-54,
    1,29,-26,17,-14,26,23,47,78,5,25,-58,44,-98,78,-91,-76,58,50,35,-97,48,-95,-71,59,49,-88,84,44,-89,82,97,-8,-92,-34,78,-14,41,-23,-83,-53,54,-41,91,8,-63,53,32,47,3,-81,50,-49,76,31,-89,-22,-5,47,-78,-42,-19,71,51,-11,-62,-19,28,79,11,45,78,-35,-96,70,74,93,-25,-42,40,30,-70,-58,82,-94,74,45,-64,-31,-56,-42,28,-23,82,-21,66,-80,12,-54,51,-77,91,30,-11,47,-48,-85,40,27,-75,-68,-91,-45,-25,43,-86,-51,-60,-50,-30,-16,-39,98,61,-57,-71,-20,-85,42,26,66,65,-30,-4,-94,-83,0,73,10,79,99,42,89,-94,
    -83,32,72,-82,73,75,-11,-91,-64,-61,23,-69,-31,-97,-54,-89,-19,64,76,51,-87,83,69,13,56,-21,-7,7,73,-66,-34,43,66,-62,13,-9,65,-98,1,-99,94,-76,84,-37,79,82,-26,-88,47,-98,64,60,85,-15,-27,-6,16,18,53,-11,52,71,84,-29,-90,-2,62,27,-48,-85,-19,-54,91,-35,9,22,-52,35,35,95,38,-49,7,-25,36,32,21,-96,-49,-25,-55,-97,-54,-70,-74,-92,-20,89,36,32,56,-83,31,48,-66,-60,-30,-66,28,-95,81,-34,-92,-12,-7,44,-79,-85,48,-28,-58,46,-73,-60,76,-94,-99,-44,95,-11,40,51,6,-29,-49,-8,-36,22,27,92,-21,8,-42,-12,
    97,51,84,70,18,-15,94,60,-69,73,-99,59,79,-98,67,74,-9,7,-22,49,79,29,93,-57,-97,20,87,-65,81,-55,-25,30,48,-89,0,-33,-4,-6,79,-21,-33,80,-62,99,34,-95,25,77,-35,-97,-22,96,-15,-28,-61,-60,92,-74,-73,25,-77,-46,-45,23,-34,55,42,14,-99,22,94,-79,-46,84,-80,89,42,45,18,-93,1,97,55,38,-31,94,78,-87,72,58,39,47,-88,94,-78,-22,2,65,93,-45,-13,39,-24,-7,-25,-52,-66,-83,46,53,76,99,50,31,37,71,-23,-33,84,-99,-75,-25,0,-62,-78,23,-32,-24,-12,-87,31,27,52,60,-80,-72,60,-45,97,6,60,74,5,10,
    -43,94,-67,35,-38,69,-12,-61,-55,89,-71,-81,12,-2,-5,52,11,-74,-21,-84,38,-1,96,-1,6,93,57,-34,19,-85,28,77,-91,61,-36,-77,-17,4,-37,-20,-7,-8,-1,57,42,46,9,-94,24,88,-78,-37,-60,-82,62,46,63,-29,65,-65,-14,-55,64,-4,59,-72,-29,-58,32,34,22,78,79,73,35,21,19,-55,27,-57,85,1,-42,-75,-29,72,24,-65,-56,-59,-30,30,-14,-66,78,-55,-86,-50,-13,-2,36,61,-24,15,34,64,89,-95,-91,-32,-99,94,70,-41,72,-59,-68,-4,-72,-24,37,-50,58,-77,-64,37,-79,-98,39,8,-47,-25,-78,-19,43,8,-55,-16,14,-94,52,-33,-99,22,
    26,25,16,10,-79,-4,86,10,-1,-3,-15,35,86,6,-10,25,67,43,52,41,-76,-53,-51,-78,31,15,-72,-16,34,81,58,-39,6,-74,71,-21,23,10,89,-78,-93,-73,9,93,-15,99,-30,-96,-6,74,-3,19,-78,47,-7,53,-86,21,89,49,3,0,62,-91,26,-15,-11,-51,95,-70,-77,54,-91,33,-1,-5,-16,69,-49,79,44,49,-50,18,-52,43,-77,-86,-36,-87,15,19,-35,-23,29,91,63,70,-59,10,-100,-84,-35,62,49,64,-91,86,-14,-40,17,82,61,67,0,-91,-38,23,76,-22,-12,-9,98,53,21,-21,-3,-64,-51,38,46,-99,6,63,63,-92,28,24,94,66,85,11,48,46,
    -70,-52,8,-56,23,-16,22,64,27,-28,-31,-52,51,66,-16,-100,-44,-69,54,63,46,-31,71,-74,94,65,92,31,-72,40,-71,-90,40,37,-46,-84,73,76,80,1,-51,-51,49,52,-84,86,5,-28,-83,-41,87,63,-20,58,42,-26,-25,34,-95,3,-73,-65,13,67,24,19,83,-2,-4,15,51,-3,65,-48,-51,33,-62,54,57,-93,65,45,71,-54,3,13,-28,31,99,-70,86,-74,-83,-48,46,41,71,-19,91,-81,97,42,16,14,95,18,-53,85,-76,56,93,42,1,16,40,57,81,-88,-12,80,94,-26,-93,-89,26,5,5,-2,86,-4,17,83,91,86,-3,38,4,-4,23,-20,-47,16,-78,6,
    -68,62,-37,-87,27,51,46,21,78,53,33,-44,-90,-62,54,96,-14,-76,80,-23,62,-71,-85,-82,26,-9,-2,31,-93,-27,-63,92,-65,1,-95,62,-96,51,-64,34,-44,69,91,-34,-93,-3,-37,-55,-27,95,23,-65,-76,-10,-47,2,81,4,-67,89,77,-29,33,64,24,38,27,-20,-58,63,-85,-50,-16,-94,-31,-57,-45,-16,-12,29,-21,-89,64,-45,-46,-82,58,-65,-26,91,76,-49,-86,9,-85,38,-52,-6,71,-58,-91,86,-8,-7,44,13,-12,99,97,77,28,-24,40,-7,32,94,-37,-58,-70,-11,85,58,40,-100,-32,7,-10,-32,-98,-39,10,-89,99,54,-43,43,-32,-3,43,-83,-26,-77,-6,67,
    -32,78,61,-17,72,43,72,57,2,12,9,22,72,-100,90,74,-87,-48,37,-87,58,46,56,78,-56,-49,96,-30,-25,-58,-63,-5,72,51,79,44,-54,3,1,0,68,-89,-78,40,63,-88,14,76,-84,-97,89,75,50,-2,53,46,49,49,-84,76,91,-46,-76,63,57,-97,-41,-97,-42,13,4,-74,24,-22,66,-61,-57,-68,-85,-41,-64,57,-66,38,-45,88,-16,56,-11,0,-15,-67,6,9,48,-37,-36,-92,-81,22,21,-25,49,97,53,-33,88,96,0,-97,-44,-12,12,-58,78,67,82,-38,-24,-28,14,61,-43,73,-30,-95,88,34,65,-93,-44,-62,-18,57,87,-64,-75,75,84,25,31,92,65,-57,
    -65,43,63,-83,57,-61,-59,-29,-100,-2,96,-78,-44,85,56,-79,92,64,-88,-25,22,99,-37,-1,75,-53,76,6,-60,41,49,-73,-16,12,96,93,3,-62,-36,3,-12,-39,-23,-4,-54,85,-30,90,50,-18,-83,-76,-19,-20,23,-44,-20,-1,14,-28,92,-36,-1,-72,28,-5,-79,-68,-15,-63,-13,26,-2,17,-78,-4,2,-56,-61,-48,26,-44,-72,-92,-11,51,-84,21,-98,31,93,94,47,-56,74,-73,-9,95,-89,77,-67,-49,-45,-17,68,-23,32,-30,-26,71,-25,-100,79,3,60,-32,-93,29,89,-91,-88,34,56,11,78,30,38,-30,-22,2,99,-37,53,-46,46,-27,-17,-70,43,57,53,-82,58,33,
    -26,-30,-99,-19,99,43,-58,-89,-23,98,22,-44,-19,-87,26,11,15,-23,74,-32,31,-28,41,-86,3,-64,24,-44,-93,-18,41,-19,4,43,14,4,-14,56,15,15,7,90,71,40,-97,-51,-49,18,26,77,38,-43,-99,-21,-76,4,-33,48,13,-26,82,-46,7,-14,49,-79,42,87,78,-42,3,-63,-52,-26,77,-97,-24,80,21,-98,9,-41,-88,10,-10,88,67,57,-12,-20,-68,-30,-14,-9,8,88,13,-49,75,-57,-39,-70,-20,-39,-43,-91,-36,33,41,85,-13,-50,96,-1,60,-14,-61,27,95,27,59,79,49,98,-29,-42,86,84,-39,13,79,22,44,-89,35,-99,-80,99,34,61,36,-79,-89,-16,
    -27,23,70,12,-49,17,-60,62,-3,41,60,68,99,-2,4,-40,-88,-17,34,8,-6,-31,9,-34,-80,95,-73,56,68,90,-60,93,13,62,-42,16,-20,-50,-21,77,91,91,-3,91,-10,-99,-97,-46,-16,-10,62,30,59,-77,96,80,70,75,-12,38,65,29,-16,-70,-57,42,47,23,92,78,-48,83,21,49,74,-89,50,30,-83,86,-80,31,-32,31,6,-36,11,-24,-61,-100,67,56,-19,-49,39,24,45,-14,-100,37,-84,-48,20,37,54,-53,1,-44,29,-30,-57,49,-98,11,-20,-40,28,44,-11,19,44,-44,-72,77,-41,67,-47,-96,-95,-47,-7,-79,-42,-87,58,64,-88,-89,-80,41,-18,-37,-10,-64,
    27,23,-4,-93,-33,85,-22,63,93,6,-8,-96,-27,45,-92,-22,51,1,-49,61,-33,62,25,-21,73,45,73,-93,61,-37,-57,-60,38,-8,-53,-95,-71,25,20,-77,84,-88,-73,-43,58,88,-12,9,-59,39,-30,-40,-47,47,-8,-73,-8,-35,86,-47,-20,-70,-55,-81,-26,-8,-76,3,70,45,26,54,9,6,63,-33,-54,-49,28,-13,43,50,0,96,-3,92,75,90,-43,62,95,37,44,-59,56,70,-15,33,-27,-45,-70,52,61,39,-42,25,59,-44,28,-13,-5,-29,90,95,-80,-13,87,47,-71,96,9,77,34,-95,-82,-58,75,-45,75,-99,-37,57,53,-76,-3,63,1,-44,-81,-70,-5,14,-47,-15,
    -38,-27,25,-99,-79,54,98,-18,31,32,-60,1,74,67,57,2,68,72,-41,21,96,8,-64,-50,16,-45,-20,64,22,-15,-51,36,59,26,37,32,-19,35,66,64,-81,-94,-34,46,26,75,-52,-6,47,-41,68,-5,20,56,45,-64,64,77,-100,-62,-37,-98,74,74,28,11,6,61,-53,-76,26,-82,83,92,64,9,19,64,55,-34,-76,-25,13,44,-68,-41,32,96,88,-15,-66,-97,-61,-92,77,67,-29,-65,-19,-30,-40,-93,-11,95,51,-47,4,-30,-82,-41,-12,94,35,-99,-10,19,-88,74,15,-99,-41,1,-96,-2,9,34,-82,80,69,-1,-49,81,-42,-8,76,-91,45,-68,-69,15,-56,19,-39,31,
    20,3,-50,-15,-22,17,-14,37,-82,-58,88,27,76,58,-41,-2,-91,-38,-69,-33,54,60,-72,-48,92,-41,19,-64,30,81,19,-98,-16,69,-13,14,-14,25,-48,4,20,92,83,-4,50,95,46,59,-43,30,78,64,-58,-94,68,-66,-83,-13,-77,-1,-80,42,1,-43,-36,41,71,50,-34,75,-93,86,67,90,-65,-31,85,33,-72,95,-85,-94,11,-43,64,-69,-56,81,-30,-81,-20,-9,13,34,0,77,-25,-77,-20,-7,99,-13,-68,18,-71,-81,88,67,52,68,62,68,27,-75,77,-9,-44,-79,-75,26,-60,-43,69,54,91,-79,31,-82,-55,-89,-36,96,-50,-4,14,-20,15,-46,-53,-81,-77,61,39,2,
    38,17,93,94,38,70,-28,31,-72,94,-15,71,-85,68,42,-40,32,-94,-44,82,2,23,-86,69,77,-39,88,52,74,28,-46,12,97,-100,-42,35,22,31,-34,2,-23,3,-26,-8,-28,-84,5,-96,74,13,38,-24,-64,53,97,66,66,85,-30,-7,-35,-23,-95,-86,-71,-84,2,51,-1,20,-46,76,-76,-20,-32,-4,-52,25,-48,-78,-61,90,-50,-73,-5,-53,93,-38,84,-84,55,-98,-7,12,-84,22,28,-30,-75,-73,91,-21,55,-85,11,-76,-37,59,49,15,33,40,-43,83,68,53,82,61,67,19,29,-26,-27,-78,86,-59,44,-33,12,70,-6,3,-99,-98,18,-35,-74,33,-76,27,-52,10,-32,-43,
    -55,-64,62,80,49,29,-1,-21,-97,24,-47,42,65,50,-91,-23,-28,-45,32,-75,-43,-50,-10,-65,83,-33,63,83,77,83,93,-78,71,55,-98,-28,-63,-47,3,-60,77,57,82,-5,7,43,24,79,99,-43,-96,-44,-41,-53,-8,95,14,-93,78,-9,42,23,-35,13,-21,-80,85,-32,-75,-11,8,55,98,-57,-50,5,38,74,-64,37,-69,92,46,-57,-61,90,-62,5,49,68,96,-9,-8,-86,-96,23,-66,89,91,-41,-70,-49,-86,-72,-54,-84,-15,85,91,-27,-26,74,-34,-80,69,-43,62,-93,63,11,76,11,-98,-80,25,-94,43,11,48,-14,71,30,37,37,59,84,54,-56,21,-3,18,95,-77,36,
    68,93,45,30,-100,-40,-58,28,24,-56,48,-51,-97,-9,61,51,29,-16,-19,-33,-79,92,-97,27,89,24,-76,-41,-29,-52,47,-61,-59,-8,70,93,5,64,22,-71,-92,-30,-22,63,14,91,-34,-57,-25,0,-38,-51,-8,-83,76,-19,-7,-99,-8,65,-99,-61,56,94,-16,26,87,89,-10,-39,-82,-49,-16,-52,-86,50,-60,-19,93,-33,81,-92,-32,-75,-75,-55,-41,19,98,51,-64,99,91,92,45,-25,19,-16,-36,61,46,-66,64,-70,-18,79,80,-26,-88,25,-6,45,-15,62,70,11,59,-71,82,57,81,-82,8,24,-90,-95,99,-19,-10,-85,95,-64,-51,59,-82,-17,-10,50,58,-46,-25,-48,99,61,
    66,22,-76,26,3,-94,-65,-64,-76,-4,60,-66,1,11,-32,43,26,-37,79,27,-78,49,-89,-35,-1,69,19,27,73,19,-60,-9,-59,-36,-83,-4,-78,-95,33,98,-47,-7,84,6,-95,-48,-50,-17,15,-71,-89,-10,31,-78,55,-70,43,-26,9,-32,45,49,59,38,65,29,-65,39,-14,68,37,39,-39,22,-55,-82,74,-5,2,-58,-23,-35,32,-92,-13,-61,90,82,-35,0,-50,-89,-99,-91,49,19,-10,36,-42,76,4,96,15,18,-30,13,-12,-56,-92,90,-62,-63,-45,22,97,-6,61,88,-24,-73,-60,-22,90,93,-60,-9,12,30,-72,71,7,-16,19,74,54,89,-13,-57,85,-52,85,76,-15,93,
    -2,-65,-13,-88,-77,16,91,-85,94,81,-92,-66,-76,-27,65,-48,96,-76,-11,-33,98,-57,-44,-14,-14,93,-14,-76,21,71,-83,-80,-94,-44,84,81,72,75,48,19,8,9,5,32,82,-78,37,78,-54,-74,97,97,-31,-47,-17,8,98,69,-16,20,-8,1,-8,-49,57,-24,84,-18,-97,-15,-99,11,94,-42,-57,28,-19,32,-42,27,58,-45,24,80,-40,59,-60,-90,-20,24,30,25,-75,74,28,34,-98,-36,-84,-95,49,69,16,43,-72,12,-77,9,-56,-19,88,55,36,-35,35,48,76,75,59,-91,-49,-59,34,-24,-84,-38,62,-82,-22,31,76,-72,0,-8,23,-20,56,47,41,53,80,30,8,17,
    47,95,-83,23,70,76,32,73,-82,-82,-51,-66,-68,63,-96,11,-6,80,39,95,-75,-38,-73,33,61,69,-14,42,-1,46,-89,98,41,-72,21,-37,-95,-94,-64,-77,-76,-63,-91,-91,-99,65,20,-5,46,-89,-58,23,-75,-30,-44,87,-61,43,29,-10,-11,-8,-60,83,20,61,-54,-23,19,-17,-48,-4,20,-39,5,-27,79,77,69,25,88,63,-100,-87,-67,-44,52,-76,51,-67,-34,-59,25,6,-76,98,-80,-78,-25,39,5,28,87,78,41,-56,-49,20,-79,72,97,61,36,97,27,21,-94,-69,46,57,-35,64,50,42,-29,26,40,-9,49,-84,82,-94,-4,22,84,-63,66,88,10,40,60,-41,-47,-4,
    -91,80,-82,-85,64,16,72,-71,-68,23,71,3,-99,12,46,50,-20,-19,57,28,55,-7,65,21,-19,-73,-87,-58,-13,-33,-10,-4,99,-40,-37,63,76,-65,44,61,10,-32,-36,12,32,63,-86,-88,96,23,40,51,-83,-43,-76,-2,-63,-62,40,-24,-43,83,24,-92,-57,87,24,72,74,-32,33,-63,36,49,1,-32,12,15,-68,8,-61,24,-41,-92,34,-16,6,23,-26,99,99,31,82,23,-9,77,-38,-85,49,-64,-64,-18,-75,24,-68,26,93,-4,94,77,57,85,-46,68,93,40,-96,-49,-37,-22,-98,-86,61,84,-11,-47,62,51,-80,-37,39,-44,46,65,81,30,-9,26,-74,-63,-45,83,-78,-39,
    52,-85,-99,56,-81,-36,87,-79,-70,0,58,19,5,-28,-30,26,35,-90,34,-67,27,15,63,70,93,42,-92,-99,-23,-70,-38,81,98,-36,-10,69,-20,-23,90,11,77,48,-70,-65,-80,-47,61,8,-85,-5,41,42,63,5,12,8,99,-28,9,-24,-45,72,-42,-47,-64,0,74,-32,77,64,79,6,13,-90,41,-15,-37,-46,93,-70,2,35,72,-83,92,-64,25,91,9,-65,-81,64,59,-71,69,47,29,43,-85,-42,7,95,17,72,57,-90,-90,72,-83,-97,-98,19,-10,74,36,-66,-90,-87,77,-29,-100,97,-65,59,78,-96,6,8,-1,22,18,-93,17,-65,31,-74,98,41,98,-85,45,-100,-14,87,
    26,-78,-78,-12,87,99,60,88,48,47,47,27,52,-94,87,-49,28,5,-90,97,-7,42,75,-57,-65,73,-42,32,25,-4,20,-97,18,-6,-9,-43,-7,-97,-55,-58,51,45,21,-45,51,8,6,-69,65,-31,80,-42,63,55,-47,98,28,-37,31,-95,59,-97,-92,-23,-3,-49,-13,90,-93,32,84,58,-23,5,-87,80,-87,-29,-89,79,-8,43,89,-45,-2,-57,-94,78,-42,37,83,-82,-8,43,-53,89,47,34,31,54,-33,16,-36,-4,73,-71,-71,87,0,92,18,45,36,-41,52,-66,-46,58,-87,13,-53,48,31,39,-56,78,-20,91,13,-88,-3,32,-20,61,80,-47,90,61,92,-58,54,-38,87,-10,
    22,40,-24,28,-2,-11,-59,-2,90,24,-63,-66,-45,18,-75,20,-18,22,-48,62,-65,84,-85,-75,46,-92,-81,-48,-78,-93,42,44,-1,70,-27,49,-88,-34,-53,-98,43,-63,88,98,7,-87,18,89,-13,22,51,22,6,18,-1,-96,-22,18,-44,1,-23,50,-55,76,21,-30,26,-15,-11,25,-13,-68,62,-25,-70,-31,-60,48,-90,-21,-78,61,53,28,-20,-48,-15,10,22,-7,-89,-100,-56,-91,28,17,-69,-94,-98,20,32,-59,52,-54,-32,34,-32,8,-66,-22,87,-44,-60,-8,37,-28,44,-78,82,-82,15,46,-82,11,55,47,80,86,53,82,7,-63,75,-89,-16,-57,-2,52,3,32,-70,-58,41,-78,
    34,-22,-54,-70,-48,81,-51,19,27,-33,83,-18,-86,63,-80,20,-2,-21,57,-27,-9,-59,-31,41,45,72,-75,28,-85,66,50,49,96,97,80,-100,30,-71,20,-43,-52,-45,39,-85,-82,59,35,16,-61,-8,42,-18,-14,-89,75,83,83,-100,11,50,67,-38,0,15,-89,-68,16,41,-39,88,50,9,-57,-11,24,-39,-100,-89,30,91,-44,-76,-75,42,-65,-100,-23,70,-99,89,21,20,-97,-79,-65,-86,-95,51,7,-34,39,-43,27,34,-2,-96,96,-50,67,-22,94,-77,2,71,17,-11,72,95,-41,25,84,80,97,87,53,-68,53,58,36,-88,76,-73,-31,-44,62,19,-40,10,21,79,88,-33,-97,-58,
    -61,20,31,63,-33,90,-12,-49,23,-15,90,-72,-31,43,87,-95,55,15,85,76,71,-1,-53,83,-91,69,63,49,88,18,91,-73,-62,22,90,6,-36,-22,9,87,15,0,-84,85,95,55,42,51,-78,-21,-21,46,78,27,29,87,-52,-56,36,-64,62,-21,-36,53,-99,-94,-89,-34,-63,-28,-95,52,72,73,-11,68,28,84,-29,3,-37,2,-51,94,-71,30,81,-71,75,-30,-82,-11,49,-66,94,3,-60,-95,21,-23,-22,26,82,-98,0,-77,22,80,-93,45,-17,-77,-52,32,-83,-71,-85,50,-41,90,72,-23,-69,-26,11,-74,77,51,83,-2,81,-39,76,63,-84,-24,38,-10,-91,46,-64,-8,21,-16,
    77,-10,13,92,-60,24,34,65,-99,65,-61,12,43,-84,-84,27,-34,49,40,-58,-36,56,-29,-98,-53,-20,0,-65,24,-79,19,1,11,84,-55,4,9,79,-31,-38,-3,8,75,-60,-24,43,67,42,-8,8,36,8,-84,-93,10,-85,-13,-89,-50,-88,-68,21,-35,-4,6,11,-100,67,-58,-31,-71,39,29,56,32,-95,-49,-1,-1,-5,59,-65,-97,28,95,14,43,-18,25,46,-54,9,-33,-36,-95,25,27,57,-8,69,-74,-26,-39,7,-70,93,-36,-18,-56,63,29,56,99,33,-16,94,-53,79,-72,72,25,-73,-19,45,-57,-61,-30,70,96,15,-9,75,41,4,-18,71,-51,47,-95,-6,-38,35,-50,-87,
    -80,-14,59,-33,-35,40,91,43,19,-76,-12,62,-37,10,-16,-40,77,75,87,18,-68,-31,-58,-19,68,99,75,83,-14,77,96,6,-85,56,25,-67,-4,16,76,-85,41,-84,29,-96,-22,-87,-84,-92,40,-97,26,72,-75,20,-46,45,-28,81,28,58,11,25,-83,26,-67,42,11,-19,-41,-13,-52,52,-45,77,56,-14,42,25,-6,-18,-20,-80,-45,5,-7,-39,-97,65,94,31,-25,5,-92,-56,84,93,87,-5,-26,98,35,-26,50,90,-49,-42,-24,93,83,70,-24,-36,43,-17,21,88,-4,-24,5,-10,-92,32,96,68,77,80,14,16,-73,88,14,62,-37,-36,53,-34,-26,81,-88,-42,4,-60,-26,99,
    23,47,39,-81,-76,-56,-39,84,76,-43,4,53,89,-82,-31,-83,59,35,79,74,51,-16,-60,-74,-82,-48,-64,-26,-56,10,-27,67,-91,64,38,-67,-92,-100,69,36,-91,-26,-58,51,-56,-37,68,-97,99,-1,77,50,36,70,28,54,74,64,80,-81,-74,-47,38,-64,-83,-71,-79,77,81,43,13,90,69,55,-7,-87,-81,-39,69,70,13,-2,20,-51,-32,-51,55,95,65,35,66,92,40,4,-72,9,33,-99,-62,-34,44,51,-43,13,-41,50,-21,-70,64,-52,-100,-71,46,-28,-70,67,-27,85,62,-61,-28,28,-69,64,84,-89,-75,-30,-88,-37,36,57,66,45,-78,77,-4,1,-93,-88,49,-93,93,-52,
    -20,-77,-85,53,60,77,44,-68,-43,75,48,93,86,73,-37,51,88,-100,-40,6,45,-18,-16,93,84,-9,57,-15,-49,50,33,-17,73,48,88,33,-23,33,17,-14,8,-83,-20,-53,42,-5,98,30,-5,-42,-63,93,-8,-27,86,-72,16,-4,14,67,-54,99,-50,72,0,91,-43,29,24,75,-84,84,92,-52,-69,-13,43,81,-83,91,91,-94,84,-16,-69,22,-36,48,-82,78,15,-83,30,18,89,-70,9,-54,11,85,73,27,69,18,27,53,-95,-29,-14,-78,-86,-22,81,98,14,-88,20,78,12,-9,9,-72,8,-61,-2,-51,-79,59,-5,32,44,-31,-88,13,39,39,-82,44,62,5,18,76,35,-1,
    26,-51,-36,99,79,76,-10,-12,56,50,-21,-46,99,0,-87,-54,85,9,-33,97,75,-94,-12,93,50,-97,-50,-31,31,85,20,-42,86,36,-43,66,13,-1,-94,-79,-99,-14,-72,-100,38,-59,-54,23,51,-34,20,-22,-28,61,-29,-77,64,-26,-56,-5,-89,64,53,50,1,62,-32,66,-87,-26,87,-86,60,-85,66,-49,-91,65,74,12,-69,-5,90,3,-44,-87,-22,72,-13,22,-33,-1,-13,73,-99,40,35,-31,6,1,43,-7,15,-44,-39,-18,7,-30,99,-19,82,-70,28,24,33,36,37,-88,8,-75,34,76,-24,73,1,-23,13,-64,46,-81,-11,-59,-35,57,-51,26,91,56,96,90,90,30,-80,18,
    -46,53,-93,43,17,-33,68,-96,95,96,77,-4,-75,-57,33,23,-38,74,65,-21,31,-86,-95,22,23,53,12,13,-65,32,83,-59,-62,42,85,55,-90,5,59,-95,2,-11,2,-21,32,87,-97,46,61,20,-74,-7,34,83,-85,-91,37,-20,-26,24,-88,-42,66,-50,52,51,58,62,-44,17,68,10,58,-78,90,-10,-91,45,-63,22,-35,-37,-85,51,-54,83,61,35,-37,-65,-40,-73,-55,78,-70,98,29,88,60,37,-43,-20,-100,-84,-46,90,6,-37,-13,-5,38,52,10,53,3,9,-12,-36,-56,3,-48,-44,-17,-3,-66,13,-5,15,1,-92,-95,10,-60,-95,26,-5,47,85,10,-66,-68,-52,86,43,
    54,-59,-48,-58,6,-4,-2,58,5,81,-93,-9,-54,3,-93,-1,-37,-88,-91,3,69,-12,98,-32,-27,9,2,5,57,-60,-100,11,-19,-48,6,39,1,56,-51,-94,-11,-43,-3,-13,12,-44,86,75,-80,95,-70,41,-65,81,-91,-40,-58,11,66,51,3,18,-85,37,71,-27,-72,-28,-71,-22,-70,-30,-13,-21,57,51,-12,43,-22,8,-10,8,-98,26,-59,-89,86,-17,-25,4,-65,78,-77,-50,-33,46,23,-4,70,4,74,52,74,-87,-69,31,64,-29,-74,-58,80,-32,2,82,94,44,45,33,79,-80,89,-86,99,64,-84,-34,10,39,14,-20,-57,-60,-68,69,-47,-84,-48,69,-13,-22,63,-81,-1,-34,
    53,-55,62,-1,78,41,19,20,8,-30,-16,24,-11,47,-36,-97,79,-41,96,64,81,-51,80,-67,-81,-81,-36,34,91,63,-100,44,-92,14,95,39,-44,-33,-41,16,-11,-5,-60,78,-58,4,34,-78,16,-18,38,-3,31,-30,-18,2,-11,-2,-63,-20,-39,89,77,-78,-44,24,61,-36,91,72,80,-19,-33,-80,-89,-38,-23,-3,-64,93,79,74,-58,-37,-56,-24,65,-15,75,54,66,88,-4,-5,-90,52,-81,23,-84,-89,-53,96,-56,-33,-84,-45,-19,-7,5,17,-62,-16,-9,-68,47,-13,-39,-35,72,-64,-81,-10,76,15,85,39,-81,57,-86,-65,20,-38,-69,64,-19,-100,71,62,-55,-24,-21,-64,61,22,
    -80,-40,-91,-19,25,33,17,-3,-76,-54,64,61,85,84,-30,99,-81,90,-87,-97,6,-6,55,-22,-44,-100,6,-13,88,-81,-91,-91,-20,70,-10,57,4,60,-46,-20,-94,-81,93,43,-97,-36,42,74,-46,56,29,13,-50,-16,91,-41,-63,-51,46,-75,-31,56,-66,1,-22,-23,-42,82,89,-35,-86,95,-64,-92,-10,91,-28,-68,65,-22,40,-5,91,91,31,34,-50,68,84,48,94,-95,4,80,-42,83,-43,-32,17,-54,33,32,93,21,-8,-65,12,64,-32,-70,94,-92,-23,86,51,-92,-28,53,77,-92,2,23,-87,58,3,-29,-7,13,40,63,11,-75,-5,-43,47,39,-8,-89,-45,60,-59,-51,21,18,
    87,-76,27,-40,30,-44,-32,84,79,-18,-6,-18,5,88,95,97,51,-41,75,98,16,22,37,-92,33,92,21,27,93,94,-55,81,-82,24,-59,0,80,-39,-64,59,-5,31,94,-99,19,41,50,-78,-100,25,72,16,99,-39,-23,-15,-95,98,-88,98,92,-43,79,62,82,72,-37,14,-66,-1,-74,29,-70,-80,-18,1,13,-67,-25,-34,10,-53,82,10,8,-41,95,13,-91,-93,-36,-99,-84,95,64,50,20,79,-35,54,-70,-57,-65,-39,-37,-30,14,28,-45,42,94,65,89,29,-73,-50,40,74,15,50,81,79,3,50,27,-33,0,-53,98,17,53,-71,60,88,-58,75,-90,8,4,-35,-50,-50,83,92,
    79,-90,-58,20,85,9,-78,18,-11,25,-32,68,45,21,-85,43,-62,68,72,99,8,-86,26,19,-25,82,84,25,-67,19,17,64,30,59,-64,-33,-79,-42,37,10,84,-94,-70,-19,-21,-55,-76,17,-87,49,-32,21,15,95,92,90,29,29,16,62,-52,-67,27,30,45,15,97,66,74,35,-72,10,-7,-42,91,72,-45,-85,-59,-32,16,10,41,-68,-43,34,-78,38,-85,38,1,-37,24,80,94,-79,-5,43,87,-79,-70,67,-17,-25,-75,-26,99,-20,-58,-59,0,58,3,93,90,12,-21,65,-50,-6,3,3,10,79,-17,-44,-100,31,51,87,4,82,6,88,-91,-17,-86,-91,15,-44,-50,-85,67,5,
    61,9,-83,-60,-26,-33,-13,30,71,97,9,-94,-47,10,-11,-96,-51,94,38,56,34,48,91,48,9,7,57,59,-26,24,-84,-65,85,-67,-72,-40,0,15,90,-77,12,-49,82,65,61,-29,21,63,-83,-40,-29,3,60,-38,-48,-31,69,61,80,-56,-63,96,-69,-78,-71,-41,-66,-19,74,76,-43,-14,-72,-61,-97,41,-38,-23,4,-68,-11,27,35,-51,90,39,-30,11,0,-50,7,-11,98,39,12,-73,98,-2,-40,25,-25,-31,63,3,8,19,96,71,96,1,-97,-15,-72,-10,34,70,-70,56,-18,-18,-94,89,72,-96,-20,-64,83,-69,34,95,56,61,-35,-29,-36,25,90,-87,-52,86,14,-97,-77,-6,
    -6,9,-83,24,-35,-1,58,-77,-60,30,-73,-27,18,62,4,-47,10,-88,-86,27,-17,-69,-48,-74,-56,1,-88,10,4,-64,56,50,45,73,-74,-89,-76,85,-66,-83,-33,-86,-58,86,28,46,-61,90,10,-95,17,45,88,70,71,84,-77,84,-54,-21,72,55,-70,-31,28,-44,-68,5,93,-81,-26,-39,-67,16,-53,13,-86,-62,4,24,95,73,-79,84,-5,93,20,18,29,67,50,1,22,80,70,-98,88,-97,-41,82,-78,-15,-57,-93,53,-58,-28,67,-20,28,-57,-73,2,-35,-89,49,-42,-16,-80,87,-49,70,-60,25,2,62,-21,-10,17,39,-76,39,76,-81,-2,30,-39,23,-3,-7,-49,-7,21,5,
    10,-16,-93,68,20,-73,7,71,97,-1,-52,-1,-39,28,41,79,-81,66,-30,95,37,21,25,99,-56,75,-56,-53,-32,17,5,78,54,12,-2,-26,-9,57,98,-60,-44,-54,39,-31,26,-68,-52,-3,50,19,-7,-12,92,70,-61,-64,45,-17,-65,65,53,-8,-5,7,56,45,81,-53,-46,79,-13,10,78,78,80,-44,-89,80,54,13,-49,-53,-99,-57,69,-60,-69,15,-24,-81,-68,81,11,-20,-12,-32,-75,-79,-33,-20,-99,55,42,31,-15,-26,87,-52,-45,41,62,-42,40,15,2,10,8,-15,77,84,4,-91,17,68,-11,-95,-64,-33,-74,-45,-1,79,-90,-59,10,-52,16,-50,-4,23,43,-42,81,-16,
    74,-65,94,-18,21,71,-82,77,32,35,45,74,-8,33,93,-30,89,-8,-50,51,33,12,99,-99,14,-52,-24,-42,6,-42,42,-68,-55,-12,-34,18,-89,-64,-4,-5,23,93,-31,67,-21,-38,-62,-32,6,-12,71,-60,52,23,93,-33,71,-30,25,-71,28,-33,-86,25,7,-20,-56,18,-31,-8,-87,92,-15,-65,60,64,-51,50,84,56,-62,8,96,90,-69,41,-43,54,-89,82,83,-9,-99,97,-83,-40,-70,-39,78,-1,5,44,-57,90,79,55,7,-72,5,-57,84,-5,3,32,-14,-14,-26,-5,-60,-15,-70,76,29,83,73,46,-56,55,59,74,-46,-36,18,-50,-94,-3,57,-35,26,-37,9,-38,10,64,
    95,48,-49,-79,-56,-9,-42,-74,-81,87,-91,93,33,-95,48,-56,80,55,-40,-2,57,-33,48,14,32,-74,29,93,-12,-60,58,-65,88,-91,-44,84,-48,-33,-90,-28,-46,72,-83,40,29,17,84,9,-28,-55,60,81,-36,8,-4,96,34,-75,42,-78,-35,0,-90,-94,-39,-34,90,-35,-15,53,-11,40,-23,6,-68,-94,76,-84,16,-100,-87,-24,82,-71,36,-22,78,-30,-45,20,-56,-27,72,-46,-21,-67,-27,-79,-50,58,74,-8,-2,-49,50,30,-42,26,-1,26,27,-36,54,-39,-6,-10,39,-28,60,46,44,-96,-81,16,-41,-2,1,-16,-28,51,-58,-54,-57,-7,-50,94,75,-40,-28,-74,86,51,-9,-8,
    -88,37,82,51,61,-6,-2,5,-2,-31,73,9,68,-26,45,-60,77,88,-62,73,-67,-60,19,-92,0,91,-65,-62,43,-22,30,55,-85,-36,-41,28,58,9,85,-91,78,58,-30,-2,-68,-84,-10,61,56,29,34,89,21,-95,97,-78,49,-68,-88,-56,-38,43,51,-23,59,-90,57,18,-29,42,27,-50,0,49,0,-16,65,91,-54,21,-28,32,10,93,-10,-40,67,-61,-56,80,-17,7,-25,-66,84,-66,97,42,4,-32,-64,83,-30,37,33,71,-79,-2,14,-81,72,86,4,-66,31,94,-6,-1,85,39,-21,-80,-54,6,-94,82,40,-97,-24,-55,-76,13,-72,-54,-98,61,69,75,-88,-17,95,36,-79,-49,
    70,53,97,-83,-96,82,56,-17,-98,54,89,-92,-12,-19,64,65,26,40,-70,7,86,-68,-80,8,59,-68,43,54,20,65,57,-57,-30,54,-40,-26,-12,68,9,-58,-26,-50,-97,62,31,67,-21,-90,-41,9,-31,97,-7,-11,5,-47,-26,-51,59,94,66,-31,-63,88,-77,-51,-86,-36,-83,23,6,-9,25,9,-46,-92,-72,-15,70,-13,95,-61,-15,-60,29,90,-7,55,91,-95,1,9,-26,-9,-3,-51,40,-37,13,58,38,-80,-99,-37,-19,-93,-76,10,-7,94,97,-60,-66,-66,-20,15,77,26,-78,20,-17,23,-70,57,-86,-21,58,7,-5,-28,-83,-67,-56,-82,-3,25,-74,21,87,-29,-33,37,11,-47,
    71,43,20,0,-79,42,-79,4,-82,-97,-87,84,-66,72,-9,29,96,8,63,-60,27,-88,17,5,-15,-43,28,52,94,91,58,17,34,-22,70,8,73,91,-88,91,46,78,75,80,2,-33,10,-2,27,-75,-10,-46,-63,59,11,22,-84,-61,-74,62,-70,84,-68,17,-85,-98,-75,88,45,89,79,-9,67,6,-77,-31,25,33,19,-47,10,-91,-41,47,21,71,-79,-11,62,-100,-48,93,36,-64,-38,51,90,-13,-9,35,-24,-78,78,96,29,-99,-83,-46,-13,37,-41,97,-2,19,-3,-81,-10,-30,9,-48,-30,13,97,59,1,-41,10,-9,-2,54,78,27,76,56,-25,5,9,-8,12,96,81,23,46,-68,
    -58,-57,-49,-16,-35,12,-11,36,77,-62,-53,-22,-50,9,21,48,-37,-1,-25,-60,-93,50,-3,17,95,61,13,76,-15,-89,-92,-21,-46,-88,16,20,-24,-95,-92,-94,-57,-45,-64,-55,16,58,94,80,-91,-79,-80,-83,24,-31,34,19,-69,-53,-53,-32,-89,-92,-1,-35,-80,-85,37,-52,-80,-3,-46,-84,4,-9,-39,-79,-99,-93,-99,-90,-71,73,-73,53,94,61,-76,77,61,-29,-55,72,-21,-3,89,-49,12,-21,-48,33,76,6,1,-67,49,14,-46,2,22,-93,13,51,32,-8,-44,-22,-94,-20,-44,19,51,-47,-57,83,50,-68,86,-37,-37,-62,48,92,-3,-99,-75,98,15,-69,1,-63,90,66,40,-26,
    -90,48,52,-84,-72,60,35,32,-86,30,-33,16,63,-47,31,78,-56,-21,70,41,-68,95,39,48,78,92,37,68,10,-70,-58,21,78,47,37,59,7,-75,-57,-27,55,-38,42,-30,15,-27,-51,59,5,71,-48,37,19,44,-63,-3,36,75,-82,-53,57,-40,20,-13,59,-43,-54,-81,-66,-59,-8,42,3,-66,-88,71,8,-87,-18,65,85,35,2,4,-69,40,-99,67,-33,71,66,76,84,-62,-37,43,-52,-38,62,-18,3,-93,-76,-93,-59,89,30,1,54,-88,-82,39,47,21,95,30,-87,49,-2,-68,20,-84,-92,4,-45,-29,-52,-45,33,62,37,89,69,14,48,63,-45,-22,-84,-91,-10,-65,49,90,
    8,-4,20,21,-55,70,5,66,39,13,-78,46,36,-78,-99,-30,85,-10,-89,6,-96,59,-31,-41,-63,-14,-31,-21,21,70,-79,-71,-82,-6,2,-36,-84,-93,82,55,-28,-96,1,-92,-73,-46,-70,64,45,-59,-30,1,-100,40,61,89,78,82,-79,-49,4,-58,-68,22,-12,34,38,5,-7,20,-40,65,-23,14,73,-44,20,56,-80,-35,97,-10,67,50,82,-20,-9,60,-86,12,-89,-30,7,-57,92,95,29,-17,0,-78,55,13,39,-68,79,-35,-60,-1,-79,-40,17,70,3,-16,20,-15,16,-88,98,30,-24,61,-100,83,57,44,31,-14,-73,83,-39,83,-4,52,67,-73,-83,-92,27,90,-80,-56,61,23,
    -20,-67,61,96,-55,11,-22,-26,-76,-22,-43,81,74,88,-80,-98,24,-19,37,-80,85,4,-100,55,-36,-21,45,-63,-25,-42,12,55,-8,-75,3,-11,36,81,63,61,-89,73,-6,85,13,14,39,-63,47,-24,-90,33,33,-90,-12,97,-11,85,86,64,-56,-1,-29,-12,24,74,77,-39,7,-7,74,18,-34,-32,-45,79,35,95,-83,-18,-29,-73,-85,-96,-63,-45,54,-22,-59,40,-58,-63,91,13,25,16,-61,-46,29,46,47,-45,-84,-87,23,-29,45,-42,-34,62,-7,-10,41,-40,-6,30,16,-100,-92,9,-7,-98,-2,-16,-33,-77,52,-42,-71,-19,-96,77,36,72,42,-88,-57,-13,-78,-38,-99,15,52,-6,
    76,98,-76,-56,99,84,5,-56,38,-97,-20,-95,78,-67,-85,-93,14,-81,84,-97,91,27,-85,87,66,-63,-51,68,5,-99,-86,-19,99,-61,-23,50,75,82,46,14,37,-73,-29,15,-40,-13,22,26,-42,-41,-19,-50,38,-4,89,-44,-14,-62,76,43,91,91,-76,42,-70,-47,-7,-95,87,-61,-29,24,-82,43,-61,-70,82,13,9,-60,24,90,-58,-86,-61,-69,-29,-23,-79,-53,-80,-88,-62,96,55,-80,-51,-100,78,-64,91,-51,12,-90,44,51,92,26,-84,1,19,-7,-8,61,7,31,45,78,60,66,-74,80,31,-84,76,-14,-63,77,-14,67,-87,-71,-84,-23,-61,13,-72,-68,91,96,33,10,-11,77,
    -76,97,-40,69,27,-80,-13,53,52,-82,22,28,4,59,5,42,26,-30,72,-6,47,63,59,27,47,51,-24,-67,-87,65,-90,89,14,-77,10,-58,95,98,47,48,68,-31,-24,-75,80,34,67,-42,56,-9,5,-44,-93,-84,35,54,19,-89,87,33,29,50,-26,95,73,85,89,20,35,37,-32,3,58,45,28,-9,-69,-52,1,39,39,58,95,46,75,31,-47,-6,-6,92,79,-77,42,-46,-81,-33,91,-92,88,26,-3,-44,-71,-92,53,10,-49,36,58,52,-24,-51,-37,-29,48,-62,54,-99,32,-51,45,-36,-28,-12,18,-57,7,9,-96,-5,87,-47,4,68,-39,-43,-22,-36,94,88,69,-30,90,-68,
    -7,38,-30,-52,-9,54,-3,-64,-82,21,-24,36,65,84,97,21,-69,36,74,35,5,88,93,35,-48,39,-24,73,61,-34,-95,-46,56,-25,-46,99,-70,51,87,-100,-27,-36,-11,90,-100,-14,11,-69,75,-63,67,-68,-75,12,19,-70,-97,95,-97,64,-87,-91,70,21,-64,-75,20,-82,28,8,-81,53,24,8,43,-24,46,-94,7,21,-56,-74,5,-79,38,-75,-49,-7,-80,-93,-91,86,-84,80,7,-96,57,-72,75,-15,-12,94,-61,64,-46,34,-60,52,41,-1,-26,85,26,79,-94,-36,56,-90,10,77,-83,19,63,85,-1,-30,41,-92,-50,16,-6,-10,10,-15,-46,16,-81,-54,-31,12,46,-5,-51,-28,
    -74,-92,-12,83,-82,98,60,87,-30,-25,72,69,-3,-35,30,0,82,-76,90,-56,-39,97,61,-68,95,82,45,-7,-23,94,65,3,-98,-94,-14,-80,56,98,59,-74,-75,-17,48,-25,49,-22,75,31,-46,17,75,15,-34,-12,47,-86,-30,-8,7,47,-61,-75,-97,-59,-69,-59,-86,87,-60,73,66,-83,-43,14,-8,-94,44,-81,-11,98,-11,-36,13,7,5,-88,21,75,5,-71,23,44,-46,-22,37,-63,19,51,76,11,-23,42,29,34,-92,73,-8,52,45,33,2,-66,49,-85,-7,-46,-72,-85,-70,-15,96,5,81,-98,83,18,91,2,-30,67,-86,47,-38,-5,-67,-78,-80,-23,75,65,-90,-23,51,59,
    -55,97,-86,-75,-88,96,10,60,1,-9,62,-64,-91,5,-10,-69,-28,-44,30,34,51,15,9,24,92,-64,-11,54,-35,93,-34,10,90,-68,-13,54,-72,97,14,-19,-60,-72,17,-98,33,-93,-67,57,-36,16,-56,67,-69,53,-9,76,89,33,30,-46,-22,48,17,-32,80,-96,74,60,54,-60,-59,94,-32,10,96,53,-30,82,-38,-14,-50,6,-47,-19,-89,-3,-43,-100,-70,40,-93,8,-60,24,28,73,-20,2,33,-66,42,27,81,62,89,29,-33,-41,-37,29,45,13,-64,51,95,-53,-52,-96,-100,-22,-4,-93,38,37,-17,-34,10,63,68,95,50,-38,22,31,-76,-36,-88,-9,23,-24,-80,-79,89,-92,
    72,36,-92,72,41,-92,-98,-63,67,40,26,-50,58,-64,65,-22,-68,-33,40,6,50,64,70,63,-93,-54,-61,-73,-81,-72,88,43,65,96,15,-42,-96,17,-53,23,-91,74,25,67,-90,-10,-55,94,10,85,-47,-40,1,75,-77,8,73,-86,-13,92,95,-25,35,12,-77,-98,70,79,19,-83,2,28,91,-73,-53,54,-30,-8,0,80,-71,5,92,30,-19,68,90,-46,82,-70,99,77,-43,34,-11,-19,-11,-89,-40,-92,-71,15,-11,72,42,36,26,64,81,79,-4,-90,-16,-11,-7,65,-43,-65,72,-61,-35,71,69,-25,-43,-42,56,46,-30,68,7,-49,-17,48,-77,78,-64,2,42,-83,-67,91,80,17,
    -20,-75,-65,37,-88,59,28,-70,-18,-3,5,-9,8,-39,90,30,81,-3,81,-83,97,56,95,-67,-90,-11,-97,-57,-68,35,13,12,60,0,-99,24,-41,30,54,41,79,-41,84,-13,-28,74,69,-46,-29,2,-77,-80,-89,70,-46,-79,-89,-43,-83,-56,92,-18,56,4,82,58,-72,-59,-60,83,34,71,94,18,-41,-33,-7,80,-27,16,-17,-4,37,-6,-34,43,-33,-23,-100,-64,21,44,18,-70,-100,0,40,-72,-7,80,63,-73,51,58,-2,62,77,43,-5,-50,-89,78,-2,48,-76,64,-9,-57,93,-57,-20,-33,39,98,-3,39,-49,89,20,44,69,-17,24,-28,93,-78,87,22,17,-18,24,28,-88,-78,
    -71,-64,38,-28,-69,-16,16,11,51,-93,-38,-100,47,13,89,-33,-43,10,2,33,82,-52,55,21,70,-76,-97,-5,53,-85,17,82,-97,-92,-46,-65,-8,22,-2,95,30,60,95,-23,-27,84,-4,-17,-54,98,-84,-72,-54,24,2,69,48,5,-36,1,-27,33,-65,-72,41,-58,-85,85,64,14,80,-54,-26,75,-25,0,11,71,83,57,22,51,38,20,-25,-60,-11,76,-3,5,-23,22,-61,-35,51,32,7,66,18,-29,-20,-50,-30,7,78,45,-93,-11,17,42,99,91,-7,-11,11,21,29,-99,-3,78,6,26,1,-3,91,-48,-70,-50,70,0,-26,-49,2,44,10,-20,-11,-31,22,-94,63,21,97,-92,
    -90,-39,29,-9,-38,78,-31,20,5,22,70,-52,74,52,-1,45,-96,73,48,6,17,-90,87,58,-21,9,17,94,82,-34,-98,44,27,32,-65,41,10,-44,-86,67,79,84,-84,5,36,-33,-50,40,92,50,-54,-39,60,85,-81,91,-54,36,85,-72,-97,88,72,-18,-28,59,-76,34,68,90,-46,-1,74,-30,-96,-38,-11,-93,-98,-19,9,-100,42,-30,86,61,-87,-68,-50,51,-87,-95,-9,37,-13,63,-51,-37,-51,-83,-95,3,-32,79,25,-28,-59,-86,-21,95,-5,41,48,89,-89,34,51,76,18,53,27,-17,58,-82,-79,97,33,-78,-87,83,-61,-82,-62,7,50,64,31,43,30,63,39,-74,4,-13,
    67,-33,73,18,-57,-57,-77,23,27,81,-7,0,31,-73,22,44,62,13,62,-100,72,-36,16,-45,-92,-53,-82,47,-75,22,86,92,41,59,-37,37,54,-14,-40,-19,-32,53,33,99,32,-93,-5,-6,20,-91,-5,-56,-26,-89,0,-18,-90,18,81,-65,93,67,-72,-14,-22,-9,-77,-68,77,-17,-34,97,89,99,48,21,59,-57,68,-69,-95,-37,-24,-21,-74,28,61,89,98,42,-76,91,-39,-48,-70,39,-5,-47,71,-27,-11,89,-30,-22,41,71,51,0,66,19,31,71,34,59,-50,13,-13,63,2,-14,5,26,-71,66,31,-41,-43,-74,13,-71,-1,-98,-82,22,-68,11,-55,83,-89,-89,-45,43,83,89,
    2,-67,-46,42,97,56,-72,-46,-65,-43,73,66,17,-70,92,82,-89,-56,36,82,18,68,-7,63,3,-43,26,10,-100,9,0,-46,-5,54,96,-56,-37,76,-50,-2,-66,-77,-84,3,-94,60,-63,69,-96,73,51,-26,-7,97,89,48,-46,16,-41,-94,-75,11,-88,20,-83,61,16,-20,37,67,30,23,90,46,78,48,7,-85,18,-37,40,-79,38,85,70,-21,34,76,95,93,82,73,-44,47,-55,-27,8,62,54,97,29,-64,21,-29,83,-49,-80,-58,-81,-10,5,11,11,95,-3,-66,-25,-69,-90,-78,76,45,95,-68,-8,-59,-43,52,-97,63,49,32,-100,-78,55,35,-74,75,77,-55,-35,-66,56,29,
    82,-95,63,-91,-64,25,-69,64,-30,-73,48,14,-80,-94,-34,-77,69,68,7,21,42,-38,-44,68,90,-67,65,-93,-80,22,36,-98,27,-49,-89,16,77,-58,-68,99,-79,33,14,-59,-9,32,-84,-88,-48,-25,34,95,-10,90,63,-20,76,-19,-13,48,-97,76,50,-18,-73,13,50,56,7,35,8,29,-80,74,22,11,6,-9,-25,59,66,-91,6,-44,52,-79,-12,-20,2,76,-72,-43,4,30,-8,31,-57,94,-60,50,29,0,-69,1,-26,54,-88,32,97,-60,-9,15,-51,49,24,53,71,-88,85,-27,40,-87,31,-56,43,-77,28,38,69,20,-11,-1,-80,20,52,-6,-74,17,-74,75,57,-30,-9,-94,
    71,67,12,42,79,-3,68,-28,-37,-1,16,58,-26,96,-3,-57,16,38,-6,-12,-42,-53,82,-63,16,61,-88,-27,83,-45,-69,54,-78,43,-51,54,93,-83,-74,-44,-32,-6,-86,42,-9,-37,37,59,-99,84,-52,-88,83,82,-51,-1,-5,-87,72,-22,-79,-97,85,-57,-1,-66,49,-8,-97,27,0,-29,-78,14,65,-35,78,-46,76,31,-62,-76,43,-79,59,92,-28,-46,58,-56,33,79,-100,-30,74,-49,-44,-24,43,-41,-97,-57,-18,77,9,-53,94,-61,1,71,71,-8,47,14,-35,6,-41,38,61,-83,34,-6,48,-14,16,-26,37,72,50,-20,83,54,-25,-35,31,85,12,26,76,65,-51,47,-43,96,
    -38,-25,3,21,-87,-36,-10,-1,10,-62,86,-74,-36,23,50,-85,56,33,69,83,50,-48,20,-86,30,-3,79,31,96,89,28,-42,-36,-69,31,-71,-53,73,-20,57,63,-34,35,-72,42,-63,-57,-50,70,64,85,-28,68,6,-14,-49,55,-83,82,51,-94,-38,62,-78,-7,-55,-97,-8,-81,-16,-99,82,50,36,-90,-56,73,-95,-54,-5,69,-68,67,-10,38,-47,41,-55,-29,23,96,-71,38,10,-96,-69,56,-93,-24,75,-9,-23,57,94,-86,20,90,39,25,-63,87,-5,-31,54,85,59,60,-22,-96,83,53,52,64,-9,-37,68,-25,-81,28,51,94,71,-72,3,17,94,-77,8,34,49,-3,-27,96,18,
    -73,33,-23,39,-89,-67,-26,64,37,91,-92,-100,11,-17,71,-61,-66,17,-89,-38,-79,-20,-91,96,40,-5,-3,-63,68,-55,55,47,78,84,-13,41,69,13,-42,7,-44,-34,-41,68,49,-69,-93,35,48,70,49,21,-97,58,18,43,-47,67,81,-79,-87,-12,69,43,-27,-92,-15,-58,-27,43,1,30,61,61,-2,10,92,-43,97,-8,-72,46,-86,-69,5,-16,74,58,-97,7,32,-84,96,1,60,-79,-39,-55,-85,34,40,17,-36,1,-22,62,63,-78,-80,60,14,-100,6,-20,83,63,16,9,-26,20,69,-94,36,65,-41,-52,38,20,-55,53,-94,85,-78,-29,-62,-48,85,53,-26,57,-35,41,57,24,
    -79,-8,-13,38,-98,61,-90,23,19,46,-12,78,47,78,-50,92,-17,-43,30,6,-20,68,10,-35,-26,85,-77,39,26,32,-37,-1,-75,51,89,79,-36,99,2,84,46,42,14,-7,-28,17,-63,55,74,67,-87,-94,36,24,-29,-90,-39,-6,1,87,79,65,-62,-96,16,-72,35,32,-73,89,-32,25,31,35,-82,-97,-48,-92,10,78,75,24,84,-37,-52,-45,-75,9,2,27,-52,-19,-8,-14,-63,60,14,24,-56,42,13,13,-33,-56,-52,-62,99,-48,98,9,-70,73,85,66,-63,85,-79,-38,-54,-25,41,-6,8,-67,33,-3,45,99,-79,90,-7,34,3,13,30,3,3,-19,55,1,91,85,-26,28,
    51,-37,14,24,26,60,52,-33,-93,-40,-47,-8,58,98,-9,79,88,37,-34,-57,-50,96,46,5,-22,-47,-94,-79,90,-20,1,-7,-4,-85,70,-78,28,22,-59,-13,34,-54,79,-8,-55,-78,-76,85,11,-58,81,61,-62,-21,-34,-32,33,-76,-59,-25,5,43,69,-99,10,39,75,-62,-87,68,-23,47,-85,-92,-60,-88,-69,-84,49,42,-42,-18,-44,-52,62,-26,-31,-53,-1,-38,-26,56,57,-57,9,-32,34,-64,-42,47,-96,36,-5,71,44,-13,83,27,3,-15,-78,13,67,-22,13,-19,4,82,-20,-97,-55,55,59,54,98,-32,-78,-15,4,-67,-68,-39,21,27,-68,-83,-86,68,-3,-31,-47,19,82,72,
    49,-4,-94,-47,-70,-14,-43,-73,-59,68,82,92,-63,56,-23,-7,89,9,54,-38,89,39,-20,55,7,77,-75,12,-52,59,-64,97,-93,-58,-50,-62,29,-41,-35,22,-72,99,-34,-35,56,43,10,-3,-95,-35,60,46,-96,92,1,-37,21,-22,27,69,-62,63,66,-55,-94,68,-65,-13,-72,-47,61,-44,-48,-72,-27,60,23,83,-42,-20,-100,70,-74,56,14,28,71,-65,-94,-2,56,96,-38,-26,-58,-80,-58,-23,-41,-78,30,-80,-22,35,-100,3,-5,24,-61,5,-96,-61,-73,83,48,-7,11,19,28,-31,-82,-64,-34,-68,10,-40,-48,5,37,-37,27,-80,-17,58,7,-64,-87,54,12,-48,-88,-32,-8,-61,
    -49,92,-15,14,11,-87,84,29,-98,-98,61,12,-38,65,69,51,-72,-3,23,-36,-93,-70,-48,-80,37,64,-27,49,-68,17,40,-64,9,-75,50,-80,91,-14,2,93,-12,-37,-43,2,81,-73,-94,-91,24,29,-75,31,12,-23,51,-51,-59,76,98,-74,-7,38,-38,-98,16,64,-77,7,-49,25,0,-9,40,57,46,21,36,52,-17,-40,-19,8,43,45,38,-53,94,79,-77,44,57,17,-65,-81,19,51,36,-6,-42,39,71,10,82,-36,-81,28,85,56,-68,-80,68,14,-19,12,-89,-81,11,58,50,34,2,8,-97,-11,-21,-25,92,-85,69,-98,54,-7,-36,-11,57,-16,-83,-6,-60,-50,-33,60,16,48,
    -76,-21,67,35,-63,-31,-30,-8,77,-75,81,-43,0,26,-76,-78,28,31,15,93,-80,-76,-23,-63,18,-31,-61,37,81,7,-15,-94,87,-96,41,-76,-26,63,16,-97,89,-50,-40,-11,76,-63,11,4,68,78,-51,88,-98,-22,-75,-27,-1,-83,-90,-19,24,-52,-13,11,-48,-72,-12,-22,44,-96,-18,33,-46,-6,-78,82,31,-14,-61,-1,-84,-12,-13,71,67,-35,96,-34,-66,58,-1,58,-94,86,-78,-89,67,10,-11,11,-34,23,-56,-27,70,-82,-45,1,-44,-6,53,-27,83,-60,96,2,-43,92,20,-9,50,-80,-98,57,58,-76,68,-23,-14,-91,88,52,-15,84,-75,-45,55,-19,8,63,75,-39,-64,10,
    -46,84,-88,11,76,-15,55,27,5,57,-64,-85,33,56,-7,-81,17,-67,23,-98,70,49,-91,25,82,18,-12,9,79,-23,-80,-67,-39,84,-3,90,-31,-96,17,26,13,53,42,-54,9,87,-83,78,-80,-8,81,90,-59,90,-33,75,8,56,-15,-60,85,5,-75,98,-59,-26,88,-37,-22,-43,89,43,-90,-17,41,-29,-30,58,-50,-9,-97,83,-67,-56,25,1,-80,86,-91,5,-74,46,62,-97,44,55,-22,-15,-82,8,42,60,-48,5,-57,45,28,14,-44,30,57,-41,13,42,3,-61,-57,-25,-75,-96,32,-97,50,-54,-94,-53,-98,-64,32,72,45,-26,32,49,79,76,-6,-40,-58,50,90,51,-91,4,
    -7,-35,-5,37,-8,20,41,-75,-25,-56,-77,-19,-9,77,-30,-77,50,-85,-51,-18,16,81,-90,62,-59,-96,-35,-69,55,-74,-13,-51,-57,82,38,36,-46,-69,13,29,-25,36,63,18,14,-67,-7,-84,-100,43,-50,-32,-24,-39,-70,17,-83,-53,0,73,-26,88,-78,17,22,12,-95,77,43,18,-42,71,-93,-79,-11,-27,-94,35,-11,-94,-70,-9,-26,6,4,-43,75,22,4,-25,-5,78,-85,-31,-52,38,81,-47,-33,-24,24,25,99,31,47,-11,-96,53,76,-55,12,-94,36,38,12,-59,95,-13,15,0,14,-90,-70,-18,-69,-22,72,64,-16,-61,-60,60,64,92,-9,-37,-19,-53,69,-91,-56,-67,-85,32,
    71,27,73,67,66,-12,-81,-68,-50,-51,14,-19,-20,-62,97,16,-23,90,-24,94,82,67,-91,-85,66,30,-76,-90,63,-61,42,-13,66,-32,-46,-16,8,-27,16,11,-26,83,44,6,73,42,-78,-49,32,-2,97,14,-83,58,29,35,-11,5,-3,-96,44,92,-9,-38,-40,45,-2,20,70,66,-69,97,49,76,3,23,-30,26,26,-98,76,-25,-32,46,-67,49,-19,22,-46,31,27,50,23,70,-88,35,16,10,-45,-62,-24,-61,-65,78,-85,-9,-47,-63,69,31,-61,-3,-94,-41,-57,91,-92,-23,14,-86,-92,93,64,-17,63,28,-82,-69,90,-27,-78,-34,12,-91,96,-21,0,-99,16,-31,-16,7,67,-10,
    66,62,-66,26,-61,0,40,99,-7,-44,-18,-92,84,0,92,74,26,-86,93,-62,-77,41,-30,-24,43,38,45,-73,-54,-36,-30,64,-73,-44,-9,-82,56,83,70,-51,-60,-96,-91,76,5,-99,3,83,67,-4,73,43,89,-57,-81,-16,82,64,12,80,81,34,44,60,90,87,78,46,71,0,47,63,5,8,-9,62,-38,94,45,-71,-58,18,72,-68,62,91,16,-4,8,80,-24,-59,-34,-80,-99,-92,60,31,6,31,-68,-95,-54,37,14,37,-1,76,-16,96,57,26,14,30,10,-72,-27,79,24,33,59,-100,74,78,73,75,86,-15,-41,-7,-84,-9,-2,-86,80,64,3,-69,-8,39,-73,-50,-82,-7,
    32,28,22,57,7,-54,-9,-81,99,-35,97,24,93,35,-91,-96,80,77,-5,-21,43,-73,95,98,-90,-12,38,89,38,56,-18,-78,-16,56,79,44,-97,-30,15,54,-12,-36,-70,-67,99,91,-63,80,20,-16,-89,-37,-37,-94,-39,-27,-54,99,-86,36,55,-4,58,92,-47,38,-12,-92,60,-97,14,-100,-33,44,-67,-34,-13,-78,98,-93,-42,-91,-78,21,68,-17,46,66,-65,60,3,42,-43,-39,34,-38,51,74,-30,-36,77,36,-36,-56,-68,-50,-37,19,-76,-39,-22,-17,-77,0,56,91,83,3,57,70,-37,-88,13,-28,-26,99,34,77,74,56,93,3,-56,-90,48,-24,-88,-89,95,-64,-76,-27,-29,-53,
    25,-72,90,-39,31,48,-69,46,60,-4,-81,86,-52,5,16,74,-86,9,-23,-42,19,-75,-13,-17,88,-66,20,-87,8,91,-88,85,-81,-97,-54,2,51,-22,-51,-37,26,-80,-98,74,-23,-30,48,-9,79,-74,2,-49,-97,89,-14,92,-77,-94,57,-17,50,69,-31,21,72,15,-76,-25,45,-75,-9,-76,-55,-55,98,22,15,-1,66,-54,25,-32,-51,-20,-91,36,-76,-68,-6,81,-32,-56,3,-11,66,-73,-96,90,-45,2,67,46,26,64,43,-24,-14,-42,75,-96,-44,-48,-76,6,33,-67,-58,57,-82,-12,-9,-62,33,46,27,-1,73,31,-59,-72,85,-92,26,-37,72,-31,40,-90,-21,-33,-85,36,-28,91,
    -58,-95,77,-64,14,95,24,-95,-67,-43,3,60,8,-23,-57,49,5,81,9,84,-56,-19,-47,36,-56,33,56,-89,-31,-72,-98,-37,85,79,-1,-1,26,75,-43,11,85,60,-77,-7,89,-33,95,-53,-100,4,-69,96,38,-64,-15,82,69,-59,93,90,-79,47,-95,-94,-21,-44,-43,-95,-68,14,-31,69,-73,-8,-38,-84,11,-43,-37,11,14,-54,8,-48,-17,93,86,-96,86,79,95,-93,-22,52,65,57,-91,-78,15,-59,-11,84,-90,-32,-24,-76,-16,40,-66,0,-49,48,-2,11,52,81,-44,38,86,-58,-31,33,-51,-1,-15,66,-43,-6,89,24,-13,-22,8,-51,46,36,-74,82,-72,-88,-66,-20,12,33,
    43,-36,14,-100,54,-48,-58,-77,85,-56,22,-77,-90,31,-31,51,-45,9,-71,15,10,27,-96,36,-38,-68,48,-4,-36,-40,-71,-92,-24,-4,-92,-70,48,-98,-95,-14,-54,-20,-39,-91,63,30,60,71,91,42,-62,2,-31,-58,90,-17,-73,39,80,-57,51,61,-97,80,-43,63,-90,58,66,68,96,64,-52,-43,25,63,39,-62,34,31,80,73,85,-99,67,75,-15,46,-34,-83,90,70,78,45,50,88,9,12,-2,-73,-20,-6,43,-20,3,-31,-56,94,7,78,-23,-61,3,-38,-60,-29,90,-23,-83,-44,-6,59,-74,-27,5,28,-87,-86,-59,11,93,-27,57,36,54,-40,57,50,-94,-36,-20,-16,3,84,
    -2,-4,-93,-12,73,-76,-3,-32,84,-25,-7,-59,-96,6,-93,-55,-83,-100,-30,26,36,-76,38,-54,26,44,10,59,-72,66,-5,-21,-38,2,67,-65,-22,-84,-45,-86,-8,48,-45,-4,-94,-38,-7,75,-38,63,1,51,40,-61,97,-34,84,-41,77,64,77,-28,-57,-61,74,63,-73,-47,79,-66,-33,-29,83,-25,19,89,37,-88,65,52,76,-82,-45,16,-42,-96,34,94,-85,12,58,93,36,54,84,11,-31,-89,16,-52,98,35,-28,81,-90,91,22,0,-96,-61,-48,32,-90,-41,-100,-32,-37,-14,-38,-22,98,72,-29,35,-22,-44,98,47,-81,14,48,-83,-99,72,50,-88,63,-75,12,19,-84,16,-97,-74,
    27,-97,-54,90,-10,-40,-80,-60,-67,-56,27,11,-48,25,-89,-77,91,59,93,93,31,43,57,-54,20,-79,18,-11,-11,21,-33,-84,-23,14,-94,67,74,78,59,59,-78,-13,-77,-26,64,-66,-50,56,45,-57,1,-72,38,-42,-74,-41,-69,44,0,20,-34,67,36,43,-19,-6,-38,-92,-28,-79,19,-5,8,42,21,-27,28,71,81,-75,-86,-18,-47,53,-8,-68,64,23,76,-36,95,94,83,31,37,-83,-75,51,-75,-3,73,-4,44,-67,-61,-82,-42,19,41,-61,45,56,-27,50,-39,17,82,77,-60,11,93,-65,5,76,18,-57,93,43,46,70,93,19,-33,37,53,-42,7,63,77,-51,-97,-26,57,-72,
    77,-82,46,11,47,86,-78,40,74,28,16,-56,23,-38,88,-31,-68,81,89,-49,-30,-6,9,78,-43,-13,-73,-88,61,84,-59,-10,54,87,-98,-47,25,-76,93,-49,52,-39,-4,-25,-25,-16,-3,8,-83,-14,59,-61,32,-31,-83,41,8,96,6,-79,32,47,12,-62,-14,14,-9,-89,38,-64,-85,-57,-50,11,-30,25,47,67,85,16,-95,-3,55,-11,18,25,-69,78,-27,-11,99,58,-64,-37,48,-26,77,92,-63,-32,-72,-48,-37,-70,15,33,8,-38,1,-7,78,-42,42,-14,48,-40,-89,31,-62,-64,-80,90,94,-44,-47,43,-18,83,35,19,-97,15,72,-34,-2,-13,99,-94,2,-48,-1,-68,11,-58,
    18,-89,-46,-19,42,-55,-30,62,-65,-36,70,-12,59,-48,-77,-54,71,-74,14,-5,-8,-88,35,-56,-82,89,-52,-31,-79,11,11,-8,22,18,25,-36,63,95,-74,98,-88,48,-62,23,-100,-38,70,24,-60,-64,71,33,-52,-42,29,-34,-53,77,35,-31,-11,99,13,-89,-31,-62,-24,-68,-66,-46,82,-54,-97,-80,69,55,-66,-9,-69,-25,27,-97,60,27,-87,-11,93,-39,-34,-19,-18,55,-68,95,19,1,-67,47,-15,-81,-99,-33,-83,56,-61,39,-36,-74,-70,-5,1,10,-50,-87,-63,-36,2,83,-23,68,-84,11,-24,-100,6,95,-47,-9,42,38,-37,-5,57,-20,-96,96,-81,-80,-78,-98,15,-25,12,18,
    -12,1,34,42,-16,63,-37,-48,-26,39,-48,32,-14,5,-77,80,95,-62,-73,52,19,-69,1,-10,-49,-25,-56,19,51,8,89,91,-90,-25,34,-54,-62,97,-49,12,-12,3,-56,74,61,-81,6,56,-42,33,-39,29,17,62,-81,20,-63,-84,91,40,24,80,32,-14,-45,-82,33,-55,67,84,57,-45,-61,53,81,-100,25,-13,9,83,-80,22,-36,37,-16,-65,-42,-27,-49,-51,-86,76,82,-2,-38,89,16,47,-65,-17,-17,44,-10,23,-2,-29,-25,-77,58,36,58,78,-42,-78,-32,42,-91,-22,68,-39,-21,82,89,-39,-68,-97,-49,0,51,-62,83,-66,82,-75,9,-68,-4,85,-45,-46,21,65,84,80,
    39,-48,-26,49,-18,94,62,62,28,51,-77,60,54,-74,-40,-43,-84,95,-8,99,-80,53,-69,16,38,-61,22,-88,56,7,44,-4,11,70,97,94,65,59,56,-7,10,31,-94,-84,10,-34,-26,26,-38,18,25,-18,-29,9,51,62,48,-27,74,4,80,-30,52,-56,-60,49,-62,57,8,94,-97,70,-23,-91,87,87,-25,-87,-34,-11,31,91,24,54,-48,-25,16,52,0,-58,-43,-67,12,61,-23,5,11,15,-86,71,-39,-83,42,38,-74,-19,-22,-46,-54,44,-5,77,-13,-81,-17,40,-54,-100,-8,47,94,1,80,7,-37,-91,64,26,24,78,-3,85,48,91,-25,74,-76,53,80,-30,-3,-24,99,85,
    -53,-17,-23,-6,-65,69,-59,-71,-77,73,-12,-14,82,-96,-88,-42,83,-39,95,-17,-47,-30,-91,-71,-76,90,0,-27,-82,-49,-42,-35,-14,-13,59,21,-43,52,3,-68,-23,43,18,-89,48,-18,-31,-17,-5,-36,66,48,35,75,30,11,17,-70,84,35,-67,95,53,20,-18,-36,93,39,-83,-52,23,-54,92,-59,-42,-8,-25,27,27,71,92,-55,-29,-21,72,-99,90,-10,83,74,77,-83,69,-18,-11,-48,47,-66,43,16,-17,-33,62,-73,-40,-80,-29,-12,48,98,-41,-8,95,30,71,67,-16,61,-91,-33,35,39,-64,-43,-79,25,-39,68,60,56,-16,95,75,-53,-78,-64,-33,-7,24,67,-57,35,-41,-62,
    -83,-70,57,1,43,67,21,-21,6,57,-12,-21,-65,49,-52,47,-43,-16,42,-67,-17,16,-31,3,61,45,-30,-96,-20,-18,94,-51,12,51,3,-44,18,-76,-13,76,33,27,-44,68,-24,-44,15,33,40,-43,18,76,25,87,-21,-14,-16,49,-10,16,-69,-64,66,-4,88,-31,4,-42,-7,91,87,78,18,95,47,46,-49,-38,-69,43,-28,50,19,97,-11,50,-16,74,-48,26,-58,-17,63,-92,31,3,-71,-65,13,22,78,0,1,-52,-5,-100,46,46,-86,78,-58,-62,-20,13,88,21,-36,-76,-53,16,50,-10,51,-35,-2,-17,20,28,-30,-66,-98,-51,34,-45,-51,-70,-93,96,28,-78,26,22,-40,58};
}

std::vector<int> getB()
{
    return {
    -64,-100,-21,0,-76,27,-84,27,-83,19,92,-33,-98,13,47,-75,47,50,-74,-67,57,-25,-37,65,-77,44,39,49,-34,51,7,2,52,39,54,76,-34,22,3,-65,-58,-52,2,96,13,-98,73,-88,52,99,-55,-39,-73,-39,78,-50,-95,17,-100,-77,-31,-41,-22,73,98,-68,49,-36,55,-95,-49,49,-95,6,45,18,8,19,30,-88,-30,27,-27,-3,88,-96,48,45,21,0,21,42,59,99,15,58,-69,-83,74,38,-78,-74,-61,79,32,37,-3,-60,-92,79,-96,-22,6,-23,76,-53,-67,-24,44,55,76,-35,-51,-13,-36,17,-55,-52,
    34,72,38,8,98,78,-13,-18,15,36,-26,23,15,-22,-99,73,7,29,-80,41,57,65,-52,33,82,-51,-79,-53,66,-82,-53,0,90,85,60,40,63,-53,74,-22,83,-100,-99,-50,-70,55,-76,38,-64,-56,-69,94,61,-69,27,-4,-20,0,-5,-53,-29,42,-1,-87,27,-40,54,-57,-41,-20,21,-57,-19,-25,-7,11,-70,17,-99,-82,14,84,-88,-73,-85,-8,23,-4,92,70,-5,-85,12,94,-71,92,-94,35,-65,-34,-85,56,9,-52,83,54,12,65,-76,13,84,-10,-2,96,-31,-35,-12,45,13,-67,15,-92,-52,-72,55,-71,-80,61,
    64,-93,79,32,63,-60,80,-1,95,-8,-36,71,58,-52,-39,-92,-55,30,-27,-15,-25,87,18,43,47,19,71,-46,48,43,68,-35,50,47,97,-35,88,29,64,35,74,-71,-42,32,-71,19,40,-74,-99,-87,12,-71,52,82,72,-48,-47,-5,6,54,-62,-26,-81,40,74,68,-95,62,-3,70,-51,-29,51,7,55,80,78,-5,-93,31,-39,-29,60,65,-47,-16,-83,7,79,24,-87,-83,50,-68,9,24,52,15,-62,-99,-15,-61,-75,88,46,80,68,-24,-72,75,-92,-11,-54,68,-94,-48,-95,-76,11,84,0,24,54,-50,8,-37,27,-40,
    78,17,-39,-85,57,-14,-97,-45,-81,24,32,-53,-49,40,-12,-50,-40,-6,2,-35,-30,65,-50,70,-59,56,73,49,-81,0,61,50,-83,74,65,26,-87,21,82,-68,97,14,-21,-52,-94,19,-2,-34,-87,52,-16,-16,-31,-14,-94,10,-58,79,-41,-87,-21,72,63,-51,47,-19,75,60,54,57,92,51,-77,-77,99,-71,-6,-50,-52,7,54,32,43,-76,-82,50,34,-88,-19,46,-75,13,18,41,62,-83,-78,-11,77,76,99,21,27,-78,-4,-22,-96,90,-20,-48,-50,-65,36,45,-41,6,-53,93,-30,29,-9,-5,-58,62,-64,56,79,-90,
    -55,9,86,-56,82,-35,-33,-21,-4,-77,-79,76,-25,-29,-89,63,69,-30,-79,-84,-84,-9,45,7,86,39,-31,75,-5,-99,85,41,10,72,37,-8,89,-96,23,-15,79,45,-38,-46,68,-27,69,37,-4,90,54,12,-19,-49,71,-32,91,93,95,-62,94,80,31,-44,4,-31,48,-6,-75,-76,79,-95,69,93,-89,37,-33,81,-25,63,23,81,-73,-95,84,98,-75,27,91,-80,66,85,-100,-51,-59,-43,18,42,51,-4,66,-18,-47,-13,-24,-36,-76,43,97,-49,-42,21,84,-15,26,69,-65,3,-4,-73,23,-86,64,-25,-36,58,32,34,
    0,-65,-70,-82,18,-17,5,94,0,-19,-11,-3,-15,-1,-82,-79,84,-4,90,71,99,-61,98,-26,-47,15,2,17,-27,-66,-48,25,-78,-66,-5,40,18,52,34,70,-67,75,-33,-30,-26,-62,-8,-90,-14,-66,81,-14,-27,32,12,27,99,-86,96,-76,-99,-100,1,-77,-65,96,15,5,48,-99,-25,33,76,42,56,-98,32,-52,64,-81,-18,45,-43,8,-71,69,87,80,36,83,-96,-11,84,5,-36,71,-47,79,-24,-99,80,3,-13,8,97,-57,-38,30,-57,-74,49,-23,23,-94,85,53,27,72,33,-85,8,-10,4,-8,95,68,-37,-51,
    -1,-9,-98,-69,94,-11,-9,91,-16,53,21,27,79,-78,57,3,80,42,-92,8,-33,-7,23,75,83,80,-33,79,48,-18,80,-100,25,82,-69,19,-76,-25,-38,-40,-72,36,88,60,58,45,-37,39,-61,-77,99,6,16,-26,-19,-48,-46,0,-69,55,34,63,55,-41,45,38,30,21,-87,93,82,-58,-19,22,54,-61,-33,69,30,-42,92,29,17,-40,-96,98,12,-42,-49,-5,65,37,-42,20,97,-96,-89,-21,-23,-76,-76,-41,-82,5,81,72,-3,-100,41,-73,-41,-15,9,28,46,-35,26,-42,-25,29,6,-59,-33,64,61,-84,20,24,
    -5,98,49,-28,-43,67,77,91,-8,-26,91,-67,-46,-98,19,63,30,-83,80,-91,-73,-45,-10,33,-4,57,-2,-90,25,-82,34,73,68,-17,-55,78,-97,74,69,-5,-99,12,-20,-93,15,51,70,97,20,2,58,48,57,-51,-67,-46,58,-17,-84,-16,2,50,-91,-78,86,-46,-100,41,80,69,88,81,-18,-80,88,49,72,10,-2,-56,-88,-43,92,22,-42,78,76,68,-39,44,-48,15,-6,-87,38,-68,-81,38,73,-100,-40,13,33,-6,34,-26,95,6,84,-7,50,49,2,-5,-29,12,-27,99,-19,34,43,-15,-50,89,-1,40,74,18,
    -22,47,18,-10,-87,-96,36,47,-22,31,-95,-86,77,7,15,-21,-98,38,92,-73,37,-27,14,-68,58,16,74,-91,-44,48,28,-14,-53,98,77,-40,-98,13,-93,-68,97,-88,-53,74,20,14,-95,-26,-47,97,-46,42,-78,68,75,33,-16,1,42,-8,-51,-78,78,-52,-79,7,9,75,73,-32,-92,70,-67,-93,44,5,21,49,31,74,-1,85,17,-27,53,-56,58,89,45,1,-19,46,-25,-88,94,-4,19,-45,-76,-8,24,84,-86,9,43,58,-86,-84,-40,45,43,11,-17,-40,36,-64,-44,-5,78,53,48,-89,-1,-77,-77,-55,-28,-5,
    1,-4,87,77,-68,-98,-14,75,12,-48,91,24,97,34,-65,-20,46,24,-31,54,-81,99,7,67,-90,58,42,86,-96,-86,-19,-43,-38,-80,-66,-6,-78,20,-79,-13,24,65,-89,21,-49,99,54,50,23,75,4,42,74,64,-39,36,22,-97,-26,-22,-82,55,35,-68,76,69,-21,50,-59,0,-63,65,-83,-99,-61,-31,-100,45,-81,-25,-28,-25,-83,98,-61,-70,34,14,-67,61,44,-97,16,-20,-64,-56,-99,15,95,95,67,84,12,37,85,3,-94,37,-100,77,-88,-28,-96,81,-30,96,-89,57,-38,97,18,-94,-100,86,-62,88,-69,92,
    -45,78,87,75,14,-49,12,-48,55,18,89,55,-53,54,80,-49,-13,-50,99,-49,-93,-39,-52,-23,-32,-100,-36,6,41,-53,-2,96,-75,-63,71,-9,-11,-65,43,96,5,-15,3,52,-61,-17,56,-22,34,55,29,-7,-83,77,71,-63,30,87,-5,-29,-66,94,-81,-89,83,91,2,-76,78,-54,-80,-16,-17,-24,-64,-26,59,44,52,-55,-48,-18,-9,-31,-89,-38,-42,41,-99,53,64,-65,47,84,98,-17,75,0,7,5,-2,-20,89,81,56,-74,-45,-33,22,60,13,74,94,-96,95,-95,-82,-47,-1,-81,-93,-37,-94,6,-53,56,89,74,
    -92,-3,-20,7,29,-31,-60,-15,-53,-52,52,70,-92,-83,-4,-98,-79,-8,59,91,45,-42,62,52,22,20,-41,69,-24,-100,-4,-15,-51,76,44,78,-55,36,-85,-55,84,20,-85,44,37,-37,-2,11,55,-42,2,1,-32,-83,-95,90,37,-84,12,66,-83,-92,-97,-82,36,-1,97,33,-65,64,78,-80,-16,45,16,-78,-91,-85,33,64,-27,87,-83,41,56,-77,-16,-54,91,48,-88,-92,8,67,79,-56,-34,76,29,1,40,-40,-27,-75,-95,90,99,66,-95,-16,-17,-70,-29,0,23,-72,-25,59,-26,-33,7,-62,-73,-85,57,-94,-89,-77,
    34,-7,-24,-73,53,50,-96,58,40,-97,-23,-3,-13,60,-21,10,-88,-98,-10,-60,62,64,-41,-79,54,-62,89,11,97,52,34,31,-55,63,10,98,13,14,9,-95,-31,86,-46,8,-2,33,-81,-38,-13,9,2,-99,-74,61,-77,80,-100,-88,-56,97,16,30,-20,62,-7,91,12,58,-43,-79,63,27,59,-83,87,-91,2,-94,-28,90,-32,74,91,-6,-12,66,-74,-60,78,-30,-63,-53,53,-31,-91,-54,-40,-27,-95,-30,-53,20,49,58,38,36,68,92,-5,92,82,63,18,26,-91,-94,-8,-65,-54,-77,-42,-65,-30,-89,5,-69,-91,17,
    4,66,39,3,-13,-12,-38,77,-75,82,69,20,74,4,-17,-8,-70,-56,-49,22,-21,-3,45,-11,-15,-85,-48,42,-54,-38,-41,3,-72,99,-94,15,-61,-80,-56,-36,-98,-86,36,76,18,71,-79,48,-85,-28,-78,-53,69,68,36,-94,35,89,48,82,-97,60,-63,-69,11,95,-1,50,-84,43,67,-82,9,3,47,79,-25,68,27,-58,-60,50,-11,61,-30,-74,68,-95,67,-32,39,70,-72,76,53,39,24,-96,-58,40,0,-91,10,-91,12,-43,89,39,-75,-32,82,-83,70,-29,31,-60,-3,-1,-54,-84,19,37,86,48,14,-8,-61,90,
    -4,-19,30,96,-10,-60,58,55,-50,-1,-6,-73,19,-24,97,-10,48,28,-18,97,79,-20,-86,-2,-82,52,98,-16,-56,-62,-26,-7,-29,-96,-59,62,96,51,17,98,-50,63,-74,22,-60,-77,-88,40,51,-54,37,30,27,3,80,45,-92,79,-71,-96,69,-45,-51,-60,-41,-9,54,-93,94,-77,-94,97,87,84,19,79,-93,-17,19,10,29,-92,92,-44,-36,72,53,-28,3,34,-72,-28,89,-22,-35,-52,69,-81,-92,15,95,-86,12,82,50,-69,61,57,14,-68,-81,96,-60,-89,-48,-44,35,-94,28,-61,40,57,-37,30,87,28,30,56,
    0,-10,-77,-5,-44,36,-23,-94,19,90,15,-66,74,34,-70,66,-3,-66,-77,85,92,3,76,-15,-88,-61,67,99,68,-3,7,-80,40,31,15,96,-33,44,-97,86,-14,18,72,-40,-95,54,-74,2,41,-99,-61,33,5,-85,-82,17,-45,85,-31,75,-65,76,95,75,7,10,71,26,-94,26,-35,-8,-55,89,52,-98,-4,30,4,-63,-16,44,70,89,59,41,-42,66,-22,27,-7,-35,-44,-12,-60,15,50,64,-58,56,-58,-41,-100,-13,-52,52,-11,44,-65,46,81,19,-58,4,-40,53,97,18,20,-25,98,65,-7,-94,-94,33,21,56,
    -51,63,-35,-8,74,-35,31,-77,70,73,19,57,-29,-47,-24,13,57,36,-34,6,-94,38,33,56,4,-74,62,-38,12,-16,70,13,99,-65,-95,74,-47,-11,49,75,-38,-32,32,33,21,-92,-54,-70,-4,-36,-12,-46,55,-78,-89,-89,-100,73,-27,12,-91,-57,-74,61,-69,83,-13,-64,-28,88,-89,34,-44,-5,-81,30,-45,-83,12,51,-66,53,-95,89,75,68,52,27,-6,-75,-60,-45,-80,-82,-84,-97,53,-45,-61,-74,-57,-98,12,-48,-3,32,34,4,1,98,-45,35,51,-87,24,-22,33,-72,-94,-73,-47,98,-17,-74,68,-49,-71,21,
    7,-31,47,2,-77,-88,6,21,-4,-60,-75,97,-61,-67,33,42,98,-91,-79,31,-62,79,11,-57,77,46,-31,45,97,-49,-82,-44,20,18,11,43,30,-83,-84,-74,-90,94,-77,-99,79,8,43,29,18,64,-88,8,-5,23,51,-28,-79,73,69,71,-76,88,-73,-4,6,-10,91,88,-40,-92,-86,-30,54,89,-29,-15,-2,-86,-86,68,31,-74,-24,-74,2,-73,-49,23,52,72,-6,76,12,-26,-28,70,64,-36,58,-76,-76,24,-54,78,14,17,-37,64,-16,-23,32,67,-45,-92,45,57,87,48,-67,40,21,27,-32,85,-99,41,56,18,
    -95,-34,-6,81,91,41,59,5,-90,74,-79,-54,-97,53,-87,-42,13,11,68,-48,-41,-99,44,32,-20,-87,18,-66,54,-26,52,11,-8,-54,-8,-17,39,-97,40,50,77,61,96,32,-34,62,42,79,-27,10,-68,-16,-37,28,-83,44,41,-13,78,-53,13,-18,10,-95,-20,-98,-11,-80,57,-71,22,34,43,-30,-82,-91,-68,61,41,-43,-29,25,42,-13,-47,11,-69,95,98,61,-58,11,-57,53,16,23,7,57,-5,-35,87,-83,51,-18,-12,22,-9,-28,-17,84,-18,6,9,-76,93,63,35,76,58,-15,37,52,96,32,-43,-36,-44,17,
    -78,3,34,61,21,-63,-57,61,59,-14,85,-58,-29,-33,49,80,-57,-6,43,78,-29,-47,-85,60,58,-89,-55,67,-24,1,-16,-2,56,70,-41,-23,-92,-46,90,67,40,76,62,-89,95,11,44,-61,57,-61,69,80,-55,85,41,-97,96,86,22,-76,-61,59,-26,95,29,33,-75,37,39,67,-43,-20,-5,-81,-57,-9,-18,39,-18,-61,79,-49,-80,-76,-64,13,-73,85,-1,1,61,-10,-40,-64,-63,90,21,-38,-21,-39,82,36,41,-23,55,36,20,-63,-24,-98,-71,-45,-46,-51,-69,-58,62,-90,79,-87,11,41,-45,72,29,-8,62,-50,
    7,-59,11,-11,30,-96,18,-15,41,-61,-25,69,-59,4,76,-53,5,-93,-58,-81,-83,-79,84,28,14,-61,0,95,-17,-86,-54,90,-92,9,-69,38,14,50,75,-93,41,-98,28,82,-42,4,82,63,11,-76,34,-20,97,18,-92,-36,57,-39,-41,-59,-73,-43,83,87,67,67,25,-67,17,-47,92,-90,-45,20,44,-34,76,26,81,-61,50,16,19,0,-66,-21,64,44,-8,75,37,20,-67,20,-41,-100,-13,37,85,56,90,29,18,97,-51,-37,-37,77,89,-55,-84,-8,13,35,92,47,-86,8,-57,-41,-17,80,31,16,-47,90,-32,-8,
    27,-95,-99,17,34,19,-33,35,82,30,12,24,27,28,68,-60,-85,12,40,82,-80,83,-59,-45,16,72,24,69,62,92,-39,-10,98,-38,59,-16,34,-74,-80,16,9,32,92,88,-39,60,81,-24,-28,21,-42,-56,-44,-49,0,72,-77,-76,41,38,68,55,-72,18,69,39,-97,3,18,-77,72,79,55,64,67,68,-23,48,97,49,-79,-93,46,-22,-41,-54,-50,-66,-78,92,72,90,-1,-100,-39,-32,-8,-36,72,-90,-61,-4,89,94,-88,8,15,-11,-43,12,91,-70,19,37,8,30,35,59,65,57,3,37,-1,54,-10,60,-78,82,
    76,46,44,-85,-58,33,62,-45,93,-23,44,-50,89,-13,-67,-40,-76,41,91,11,-48,8,-80,7,-55,20,-39,35,-68,-16,-31,61,-70,-87,76,-27,-2,38,-20,-8,67,-76,-6,8,12,27,69,-12,-79,12,-100,-27,72,72,81,-83,-8,-58,5,-23,26,26,-62,9,-8,66,34,90,-43,-86,-66,-76,90,-19,33,-46,-92,-46,43,-71,18,-5,-45,-10,-33,-64,-41,12,-70,64,41,9,-9,31,-82,83,-3,52,25,54,66,12,31,8,93,-36,63,-99,-30,6,83,88,53,90,-22,72,-74,37,-16,-44,-46,77,65,-3,8,35,-68,-94,
    -13,57,12,-95,-31,-57,14,-38,-41,-23,-84,29,35,51,17,-60,41,95,-88,19,-15,-51,27,39,-74,-55,-64,35,80,-32,41,-80,77,-95,77,99,-51,-9,-87,8,20,81,-62,-93,-16,55,99,77,3,-88,-4,40,-87,76,-69,-61,-79,-81,26,53,-61,-81,73,-84,-75,51,67,-26,94,81,34,-33,14,72,-74,99,-20,-74,76,35,-62,-75,-73,-97,-99,-42,-58,-78,-23,21,27,-32,40,1,-16,65,-96,-96,91,50,37,26,-83,-49,-50,-56,-50,-18,-78,-73,69,-88,-96,48,-85,-95,-94,-91,79,-65,30,6,3,-29,59,40,88,63,
    -4,80,14,-67,-42,-17,-16,-40,27,35,-5,49,14,-84,61,18,65,28,-25,23,38,-46,59,-32,12,-86,39,-28,6,-20,87,-98,-40,1,-65,-30,85,-28,-18,-36,59,77,66,-27,94,-21,43,11,-92,70,-66,46,76,45,-86,-12,60,6,60,-82,86,48,73,98,-99,60,-80,38,-68,2,-97,91,-20,69,-84,-74,-52,59,37,-44,-19,71,54,-43,17,21,46,-71,-73,-42,99,65,58,72,15,60,33,35,-2,65,-63,-47,9,-31,74,25,-5,-77,-63,84,-69,18,56,38,76,-75,-41,-26,-94,-62,32,57,-97,-57,30,-30,-97,-85,
    -95,-47,32,94,-41,-59,-36,-67,19,11,-92,56,-4,92,-74,-96,30,-98,81,89,-24,39,-73,61,-4,82,-96,78,-48,59,45,-91,64,-22,-97,-77,-29,-81,-91,90,31,-83,-2,79,-91,25,83,39,-21,16,-20,8,-45,7,-31,3,-59,-75,-18,-7,84,79,-46,0,-91,-42,24,-19,-71,-15,-77,60,2,-26,-9,-36,51,-26,-97,-70,42,-16,38,97,43,59,1,85,84,35,-70,68,-34,-63,-79,-24,-53,-3,9,-24,82,-16,89,-16,58,-20,48,-39,7,4,-8,-51,40,-70,-53,-65,90,0,72,26,87,55,-53,-95,92,-32,-19,-61,
    65,42,15,47,-73,-44,83,37,37,-16,99,96,40,-57,-55,-20,-75,-56,-33,67,-4,-60,-6,-65,95,41,-59,39,-91,-26,78,26,-83,-55,-27,96,2,-92,33,91,-8,-16,87,-68,27,84,-36,53,-71,32,20,25,24,-86,61,-29,7,54,-90,-32,-72,88,94,-55,85,-33,-59,-61,-24,27,-70,20,-89,69,53,-9,54,69,-56,35,-47,16,-40,77,-17,73,-52,90,27,-90,-89,8,50,-95,53,36,-27,47,75,1,-26,58,-79,-63,-21,26,28,-67,-4,24,68,1,93,81,-69,-24,6,-69,-82,-66,-58,-71,42,92,-65,-53,28,-40,
    46,-44,13,-80,-86,-14,-90,-7,-87,38,-21,-39,-85,99,62,-92,80,45,-64,-13,77,-94,-27,-81,-64,67,-89,-77,66,-8,-65,13,48,48,-15,14,-66,-5,-93,99,86,-62,-40,-99,-62,75,61,-82,20,97,-43,-3,-97,30,-32,91,-51,-20,66,-84,24,-99,29,-28,1,-86,86,36,10,45,35,-4,84,48,-51,-78,-25,62,92,47,11,-98,-55,14,32,-35,-42,82,45,-76,98,-79,78,-21,-7,-21,-7,-69,-85,-97,29,3,-49,13,-97,-48,-13,-70,-86,-69,77,25,-67,74,92,18,-60,-50,0,37,-74,-50,59,-96,29,-96,-64,-78,
    -64,-97,78,65,-42,81,-70,-39,-66,-31,-9,-100,0,-79,-22,-14,95,-30,-96,-65,-28,-44,25,-50,6,-16,-93,87,-12,-57,61,-24,-54,-9,93,5,-27,-77,18,-41,-56,62,-89,45,83,-11,-69,78,59,87,66,83,-57,-9,-66,49,27,-59,-12,-85,-64,-51,-56,-18,-7,-63,39,66,-87,-42,77,-43,72,40,54,55,-70,37,-15,41,24,-97,25,67,-6,11,-32,-79,4,56,89,40,58,85,74,-49,-78,14,69,87,24,-2,-55,-4,-62,51,-49,-32,-11,-12,62,65,-8,-13,33,86,50,53,60,-46,10,1,46,-80,-14,20,-29,8,
    -66,-60,-52,58,38,-55,-46,28,96,-43,97,-63,46,59,-97,-62,98,36,76,48,-11,36,54,51,-63,0,71,-25,-80,94,36,-93,86,84,65,24,-71,72,-47,-75,-71,-98,-37,75,13,-82,-87,11,6,90,-89,47,-22,17,-1,-32,-83,-78,43,89,-83,-21,96,-97,-37,14,-20,-56,-14,-15,-78,15,-61,-63,-9,52,-45,-44,-85,61,98,-74,8,-23,-5,59,-3,-36,82,-60,-47,51,20,-98,54,35,-32,86,32,54,-77,54,-31,-38,91,12,-86,-54,21,-71,59,-29,-93,-33,48,2,-73,-55,18,61,-62,72,12,10,26,-82,97,94,
    57,29,0,80,-17,21,-57,-26,-66,-91,-28,7,-61,-69,78,98,-49,-21,-47,-70,-24,71,91,14,-5,-45,76,73,-75,74,67,34,3,-81,15,39,41,10,65,27,19,-62,-66,10,-79,64,-39,-28,-57,14,2,-28,-63,-55,86,33,-48,15,6,-70,-11,-74,64,44,-55,-69,-17,86,41,-51,13,-87,-61,-1,23,-40,-36,84,-15,-41,-50,39,31,-12,85,-30,73,89,85,-21,-81,-26,-43,84,70,-45,67,-46,-59,9,-45,7,22,-6,58,97,6,-78,34,91,-66,84,-17,65,-76,-32,-65,97,-91,72,-71,-71,98,-14,65,69,-7,32,
    75,35,93,-70,94,15,-24,52,-35,34,27,99,-22,61,35,61,-22,12,81,-34,9,90,-62,-10,71,-63,29,-12,58,-78,73,33,9,66,15,3,-66,43,-92,-1,-23,35,-50,7,-52,85,-32,26,-3,-99,92,59,44,83,-51,15,-80,30,-96,30,53,29,-37,-38,-5,30,18,-19,-27,26,80,2,-39,82,-90,-39,68,-70,87,-83,84,32,28,28,15,78,-57,-13,-40,99,-83,-87,28,-68,-24,76,62,46,-43,-65,-28,90,-11,-15,72,-1,46,-8,-18,-15,10,-34,-83,90,94,32,20,89,-81,81,89,-64,94,-31,-32,-78,-55,82,
    68,-45,-83,92,-55,-41,-23,-83,10,-77,-38,92,9,24,-90,-22,14,56,-89,-65,46,82,-84,-13,71,62,56,91,85,54,74,5,-91,43,-2,-94,-98,27,75,13,3,-63,-43,-88,61,-32,-58,-24,-76,-47,11,22,88,-21,-91,59,-59,18,2,78,-76,-24,-64,-67,-80,-14,91,74,-87,66,-13,16,-96,-3,-20,65,65,23,-59,41,28,4,-36,16,83,-27,-73,77,43,30,7,67,58,43,-100,30,-71,43,5,-57,10,44,11,14,-59,92,-21,58,15,-27,0,95,77,64,-36,13,89,-57,-58,33,25,49,0,-16,-7,-47,-86,22,
    96,-29,17,6,68,-71,-80,61,73,-48,20,-60,-23,-80,-65,-94,36,-1,19,25,-5,61,-90,-80,-37,11,4,56,64,-29,-70,12,-6,0,-81,-38,-19,91,-76,-46,95,-4,-6,-28,16,29,-21,-96,81,50,29,-24,-36,40,-52,27,-97,53,-65,19,76,-83,-69,-78,17,-98,-15,98,46,-39,-48,-7,57,46,18,-75,-72,49,-71,-91,-1,10,-63,15,-98,-15,94,57,90,-71,-24,18,99,-40,41,16,-86,78,15,60,91,19,-94,48,-82,24,25,-54,-27,6,-93,24,-84,96,92,71,-67,86,28,-76,68,57,-6,67,-83,-65,-17,-69,
    -87,50,44,-96,-78,50,4,-60,-74,81,-62,-1,-13,97,75,-44,93,67,79,-74,6,7,-98,-26,-36,-3,-59,-19,32,-24,-35,-2,79,-91,-46,53,11,11,93,37,-56,83,88,32,80,63,40,25,-17,19,51,89,78,54,-37,95,-97,-44,-72,-13,-16,-7,85,-37,-46,92,-84,65,-45,61,2,-1,-4,-10,-69,-24,-94,23,53,-11,94,5,-22,73,-89,-7,68,-86,49,96,-47,-67,-10,-61,-51,-4,83,-83,-38,-62,31,64,37,-73,-93,21,-44,65,-56,-91,-46,-61,-34,-16,64,-23,77,32,-57,-22,-20,-3,-37,22,-12,12,-81,-29,
    30,-19,9,-87,97,98,-60,56,19,-4,21,16,-42,27,7,24,-89,-29,54,40,55,97,-82,-65,-54,-18,10,-14,94,29,57,-24,62,-82,-11,11,-83,-18,-80,-12,30,41,-96,88,-79,-37,-35,-68,34,-81,73,-59,68,43,-71,-85,-75,39,1,72,-80,-89,-100,34,81,-10,-55,50,24,-83,-61,-46,11,95,95,32,-41,-40,64,45,-21,-11,87,-53,-67,-84,-86,-90,55,-32,34,27,-21,35,-39,-40,77,-42,-89,1,-24,-98,7,87,97,2,19,-92,62,-65,54,93,-23,41,93,-38,57,-93,-28,-36,-25,-93,91,6,-6,-48,19,-29,
    10,30,24,38,-16,-69,25,-19,34,96,-58,-52,-68,96,-58,-39,37,-65,23,-6,94,47,58,-78,6,49,-20,-100,53,99,-29,-85,-19,95,54,65,79,-21,99,13,-72,-59,13,-40,-63,55,21,26,42,96,-80,-11,43,30,-37,-50,-69,43,-50,36,43,-26,51,76,-79,5,42,-100,37,-7,13,65,-14,79,77,23,34,-50,-51,-71,-54,-79,-30,41,3,33,91,34,-72,94,-30,23,-32,73,0,-11,-21,94,-58,68,-13,55,-15,25,34,-86,-52,-79,-36,-51,-98,-38,-78,-76,-97,25,9,47,-89,37,41,-67,-39,61,-94,13,-98,-63,
    7,-56,-95,46,52,-10,-29,-62,4,-29,-89,20,-28,65,34,-54,89,-10,71,-2,-63,34,88,30,67,-99,-9,73,14,-55,-89,73,-10,68,19,-58,-41,42,-20,15,65,-8,88,89,-43,22,-65,99,-88,-94,-3,-99,40,-63,-17,-41,-62,74,84,4,-80,95,-23,-38,16,48,4,27,-58,36,-58,-93,-72,82,96,-62,-43,83,-63,-79,89,-14,-77,-19,24,6,-8,14,-67,-23,-81,-95,-76,-4,-33,40,-55,23,-33,-13,-41,-38,-53,-12,-56,95,-22,1,79,15,23,20,-99,-2,54,-23,-96,-54,92,89,23,-89,94,48,-41,-87,-12,4,
    36,8,44,96,70,43,-64,-86,38,-34,-32,17,81,91,-62,34,-11,44,64,-55,90,8,87,-34,19,81,-86,30,47,-46,87,-65,62,31,-69,-16,-26,-81,-1,64,-15,67,-18,-82,10,-28,5,51,-84,-31,48,58,77,35,-76,-52,-31,90,78,-84,-55,17,-49,59,-52,-65,-56,-26,-94,-5,91,-8,-86,-75,-38,24,-3,67,-73,-35,88,75,75,17,11,-100,65,80,90,-4,48,87,-87,99,-1,-86,-66,43,-60,-59,90,-69,-15,-96,-44,-1,-20,5,-81,-41,22,-93,-66,-2,25,-3,-2,-58,-71,-60,-10,77,-72,-44,77,27,-78,63,
    -78,-38,56,64,-54,-59,20,2,-7,52,60,12,-89,-18,19,97,32,96,-5,82,91,24,-77,-67,2,-97,-11,-69,-18,-37,94,4,26,51,-80,24,44,40,78,-63,-8,38,49,-45,73,-79,-48,-43,69,-1,-60,12,76,-85,46,-70,-82,87,-39,0,-49,-93,56,29,58,76,-47,-45,-32,83,-8,12,22,94,67,-53,67,71,4,-12,-77,-4,53,99,11,-49,-71,29,-62,-58,81,-59,-51,37,22,60,-35,27,-85,-15,11,-41,97,85,-95,16,-68,72,-12,-12,-87,11,85,66,-38,-4,69,-9,78,59,-15,59,-99,34,-51,-25,-6,14,
    -97,9,-100,-34,21,49,-97,26,-34,-65,-49,6,-25,-36,69,-88,82,-69,9,51,-26,-13,10,59,98,-37,-7,-53,-9,40,62,-54,-99,14,12,22,-85,15,-51,-67,2,-48,39,29,-32,8,42,-50,39,-49,53,65,-10,-85,24,-12,-69,70,-12,22,-38,-98,68,-37,-32,-68,38,-17,-1,39,17,-99,91,-44,-18,-41,17,-76,-39,-92,-73,66,74,-31,81,-50,10,64,-80,98,-14,-18,-100,-94,-2,-32,-62,-64,51,37,75,-80,-10,18,-23,25,29,46,49,90,54,29,8,28,98,-59,79,8,-94,51,6,-56,86,6,-49,-16,26,-59,
    -28,-22,31,99,-50,73,17,27,-2,-54,-75,-52,88,80,29,-4,-40,27,37,91,-64,43,-5,-6,88,-19,53,91,65,-21,-68,89,9,63,88,60,37,-43,-61,-65,-45,-35,-65,-57,97,64,-9,-43,-56,80,-99,-20,-24,-4,26,-36,77,-21,-93,-6,59,-61,-17,-32,55,-77,-72,-56,-68,-32,-69,-13,33,-81,-18,30,83,25,39,79,5,40,-89,81,88,38,97,65,-83,-96,-89,76,96,-54,97,51,-79,-75,47,-47,-55,78,40,30,97,74,-40,33,-1,52,12,-95,-56,24,38,-67,-38,36,50,-69,-8,14,60,-12,60,57,91,82,
    34,38,-65,80,17,28,10,-34,-46,23,99,54,-73,-36,11,-29,40,-99,-44,54,37,59,85,-70,-27,45,-30,-67,-46,-38,67,89,52,55,69,-79,-65,-69,-12,-11,6,-61,95,33,3,-42,-95,-57,-40,-87,97,49,72,35,31,97,-20,-98,83,35,-84,50,-24,68,-43,97,90,-8,-72,-70,82,35,-31,29,-80,25,88,-75,68,-100,91,18,49,15,-47,-67,13,33,-65,-4,-80,-97,98,-52,23,-44,-55,13,0,-74,-5,-18,13,65,64,33,-10,4,11,-90,4,2,28,-95,69,81,-62,-18,-33,-75,30,-61,-72,29,-12,52,37,33,
    -83,-63,11,-35,-28,-76,-70,88,58,-28,-8,-79,-66,96,-25,63,53,-56,-4,44,27,15,69,-91,55,50,-10,95,-46,-73,-20,-29,-83,92,36,-59,-84,18,-71,26,42,21,-1,77,-31,74,-8,-78,-81,88,18,-2,-44,-12,-41,-89,90,50,-94,-56,29,-14,-33,46,-70,56,87,-53,-74,68,25,69,-11,77,-2,58,51,90,33,22,30,-49,72,86,-9,-68,49,-67,82,7,-23,-37,94,45,-90,-76,1,49,-77,-21,18,1,-52,59,-22,-2,-30,81,-12,-97,4,71,-94,-72,9,-50,-40,-89,-17,94,18,13,-42,-36,-42,-80,-59,11,
    21,-84,90,91,17,91,51,47,-11,-79,-19,30,76,37,-47,34,65,62,36,-74,25,-80,72,-56,33,82,-40,43,54,1,-94,-72,18,48,-81,87,39,70,-13,-71,43,68,11,19,5,64,-94,-78,-22,42,-100,-96,62,73,-100,-53,7,-88,42,-38,14,48,-10,84,-3,-39,23,88,84,62,-83,27,-18,-20,99,39,96,-95,62,-25,47,62,-69,-38,-13,83,9,95,95,-96,-91,-39,52,-49,97,-99,12,21,90,-4,-65,59,76,18,-60,75,-43,88,80,-81,-37,27,34,-54,89,-79,81,-49,-84,77,-93,-75,90,11,-72,88,13,41,
    -39,-45,89,-4,-86,65,66,-94,40,76,95,-28,47,-90,-100,-19,-91,41,3,90,44,-29,19,51,-51,10,-37,-23,-50,28,70,11,83,12,59,49,-23,-22,56,70,-46,3,-58,53,-35,94,-65,-26,-12,-10,-35,-68,61,36,36,62,-54,51,-60,-52,-69,62,59,-86,-26,-29,63,52,1,-29,-26,7,26,16,-40,92,-37,-53,-34,51,37,-17,35,-49,-80,-29,-87,-82,74,-95,-33,-95,68,78,71,-58,-51,-13,46,2,10,-80,-91,-63,-11,-30,81,52,-31,-53,3,59,-17,-62,-90,3,-38,75,-27,88,81,40,46,1,19,17,43,20,
    -96,-10,-77,15,10,84,4,99,-94,-15,3,-24,-16,-94,-65,19,97,97,74,-89,24,-52,99,-43,88,45,58,-41,-85,2,32,71,44,7,-62,-94,-9,42,58,-2,79,-39,26,-84,-80,13,35,-83,10,10,80,86,10,31,-56,-2,-71,54,10,-56,-44,-58,-85,52,49,-94,59,92,48,17,-10,-20,30,-84,-4,50,-19,83,19,91,-7,-1,30,-97,-69,26,-46,-40,-20,16,-44,-11,10,-77,-59,-89,-71,52,-97,30,-79,-54,-90,52,-86,-42,2,-4,-59,-78,-61,-13,-27,69,-58,4,-5,-4,16,28,64,24,-83,-26,48,-42,-15,77,
    63,-59,-41,84,-13,-79,36,1,-21,-61,97,-27,-87,89,60,-14,58,-98,43,-94,-49,-41,34,-85,84,51,-58,-16,61,27,-87,24,68,-27,-39,-93,-6,97,9,26,88,-42,99,-99,-53,11,-60,58,-35,83,64,16,94,-50,-16,30,53,26,-86,-34,-95,-72,43,26,-47,-96,33,-1,-99,94,-75,42,-95,76,-5,-48,-13,35,62,53,18,-22,-79,-35,28,5,95,-67,-69,-90,0,89,90,-57,15,-57,-53,-100,-6,-100,95,20,42,-100,-4,-62,-96,36,73,19,-11,-56,97,10,-39,-22,68,-44,11,51,-82,-89,40,-92,-46,-45,-97,-47,
    -92,-2,-46,-97,70,96,55,-82,86,11,54,12,-70,-57,8,80,6,69,-42,-26,-75,21,-75,96,33,18,-96,39,25,60,93,33,-90,99,36,80,47,-57,98,34,55,53,98,37,48,6,-31,-94,-73,-21,-68,-48,-99,-90,-52,-14,28,-95,-75,-47,17,70,87,27,21,-25,-41,-31,19,-43,55,-74,62,53,15,-37,-89,85,-31,-62,-36,-46,-58,-83,-36,-9,3,44,-52,-19,97,65,-97,36,-56,25,-88,3,46,-17,12,1,-39,-25,-46,76,-10,-83,13,11,-45,30,65,-3,47,81,40,-97,25,88,-16,23,5,-61,-89,1,64,-77,
    -96,10,6,-31,63,67,-4,69,-4,86,86,-91,-3,-59,-9,-85,39,-61,96,-21,-58,-26,-80,-22,97,-23,17,-92,79,-66,84,35,-4,-58,56,-40,62,-48,-71,-42,-62,-84,-81,-12,-91,-37,-45,48,2,-49,80,-4,-23,52,26,-74,-71,-57,-65,60,-71,71,-52,26,-87,-96,38,27,-43,67,37,-53,35,57,-13,45,-80,42,-55,-26,46,-75,22,-77,77,-52,-50,59,-57,-63,19,-27,-92,19,51,73,24,-11,-47,33,-92,90,80,-4,-1,-32,-59,19,62,-62,45,8,-84,-33,-16,93,-33,34,4,11,23,-76,36,-17,43,-13,-44,-81,
    28,9,52,-12,0,33,84,99,53,77,-29,15,-84,68,76,-68,88,-40,77,-45,46,-18,18,-31,-42,-46,52,53,-7,60,73,73,-30,-23,-38,22,62,98,-27,15,76,-4,83,44,-83,11,76,-95,71,5,-88,-31,87,-17,38,97,37,-10,51,83,-50,-24,8,-28,5,70,46,-32,69,72,35,-3,-80,-30,-59,-63,81,69,94,4,-26,59,73,-86,-58,-37,-89,31,53,14,14,56,42,-77,80,-52,45,27,16,-34,-49,-49,63,-29,-26,-96,-39,-45,73,-93,12,-100,66,-15,-86,-92,1,77,-8,-46,-8,-94,62,86,81,43,-66,-21,
    -30,2,45,-27,54,9,44,80,-35,57,-65,91,65,47,43,-17,85,57,-56,-62,-14,36,-8,78,-6,-45,65,-72,98,51,7,-80,-46,-48,-7,60,13,89,40,31,47,75,-26,-36,-25,-83,47,-88,26,43,-50,-88,-21,-6,43,74,1,-40,54,-49,-89,-39,-29,-35,-35,64,-75,31,6,65,-86,-47,-7,-12,-31,20,-43,68,-16,35,12,-14,-1,91,80,-58,17,-66,2,-29,-15,-34,-16,57,-17,50,-27,-39,-67,79,-74,-53,84,-29,-13,-95,43,44,74,-73,-69,-14,-35,30,-71,98,25,47,-68,79,70,17,-55,-45,-74,29,-95,0,
    90,90,-69,68,-63,68,92,24,25,35,-80,-1,15,-49,37,32,81,67,30,6,14,62,-14,-64,-68,-17,-9,-42,64,-52,10,54,-62,-6,-25,-25,-86,-81,51,39,6,71,91,73,74,28,6,56,-5,-64,-86,61,51,-48,98,35,-12,-59,45,-48,-58,-44,59,-20,2,-66,8,-32,-95,-89,-93,11,83,98,-15,57,-21,91,-35,74,79,-68,36,-18,-64,-14,-83,24,27,15,77,69,23,36,-98,-75,22,-38,-7,27,-27,-48,90,-44,-97,-25,66,-18,-82,-69,-92,50,-85,-4,84,-48,82,54,76,-38,-31,53,83,92,-59,85,69,15,
    99,-38,94,-27,-34,85,29,-79,-40,-53,-45,-69,-69,-36,81,-54,12,65,-50,-5,-29,-73,9,40,32,-8,32,26,-70,1,-59,-71,15,-64,2,34,-79,84,55,-67,83,63,-36,-86,79,97,-87,-9,15,-37,38,86,-58,-53,27,75,-8,-89,-47,22,13,-6,3,80,-18,58,14,3,-6,-78,-11,-23,37,5,-56,16,3,-43,-93,70,-28,46,8,15,45,-65,-58,-63,99,-5,11,-36,41,-85,-56,-76,25,11,-21,-81,-15,68,-52,-78,-26,-8,38,29,1,-3,-1,74,-57,-93,-59,89,95,-17,78,46,30,-10,-90,-29,-43,-94,-5,82,
    -31,75,53,54,-5,53,28,69,98,-82,-50,-1,16,1,-75,11,9,-82,-48,-44,-47,-69,-98,83,21,64,-45,78,70,2,12,-60,-71,-83,46,25,70,-25,-6,68,-7,-3,-80,61,98,97,-27,-41,-84,25,15,-31,8,-31,-95,81,85,-40,59,56,62,-77,48,92,-60,-6,-31,63,21,15,31,67,-88,3,28,-89,1,53,-30,-31,-69,38,-10,39,59,-5,21,45,55,-68,-47,-30,-92,53,-86,-100,99,-65,63,21,50,-53,-60,63,2,20,26,-97,-26,96,-76,-43,-14,15,48,-54,62,-79,43,-30,-94,48,-8,-86,1,6,66,-48,
    41,30,73,-57,29,65,-94,-69,-14,32,87,12,-19,63,69,67,-22,-31,65,93,-9,-40,-37,-3,60,55,-37,-39,-87,-71,14,6,11,-61,49,92,57,56,-76,-57,-60,63,55,21,26,-24,-59,57,45,58,50,88,-81,-87,-63,31,68,0,93,33,-18,-41,-61,93,-50,88,38,-93,-4,62,-98,37,-23,57,-90,55,85,-97,12,-17,62,-86,-29,33,79,9,-36,-1,-91,-91,-68,-57,-80,-29,89,-77,-40,27,-70,56,-59,-67,-55,18,42,-92,-27,-20,-89,38,63,25,52,-66,-90,-16,43,-73,83,5,-64,-84,48,-91,-61,-11,-68,-49,
    -84,-86,60,-43,99,-95,27,42,65,53,-78,77,-9,85,-46,95,71,-35,-21,-33,-56,-85,72,-68,83,-28,-59,-78,62,-75,-26,30,-60,-66,40,-61,91,-33,33,57,72,7,-14,-37,-8,40,-89,-84,57,-58,-17,53,-43,-45,86,-60,-21,79,-85,93,-95,-11,-24,97,-25,-84,-64,-34,-65,22,-77,-92,-71,-39,-77,-26,54,-66,-10,11,-23,73,-83,-14,80,-97,27,-89,82,-6,57,87,83,-67,36,-90,-51,73,-24,-64,-5,-48,-56,-24,13,-80,50,-81,-46,-60,31,-17,65,0,70,-3,3,-51,9,-63,43,-34,77,-22,-1,13,-12,0,
    -14,16,-12,33,20,-15,62,86,5,12,-95,59,5,-12,95,-30,88,65,68,-9,66,-71,29,-39,-5,6,-9,46,-81,31,98,-42,-53,-14,43,20,71,5,6,-72,70,11,88,-25,-100,35,-3,88,0,-83,-68,-82,-54,13,-21,-7,71,70,91,42,-47,41,52,0,-72,-4,-80,51,53,-22,80,-25,90,20,50,90,-45,-100,30,7,69,14,-23,68,-73,56,-87,98,-22,-95,-7,-17,46,-55,83,26,-7,56,-70,99,-66,62,-26,76,-66,77,18,-59,77,-51,48,-2,15,-75,-34,43,33,-20,-7,63,-63,86,-54,83,84,81,-38,-23,
    -11,-56,-24,-24,-94,51,-48,92,28,71,-67,-43,72,33,55,-13,10,74,82,95,-46,-24,58,-9,-86,56,-74,98,-11,-60,-24,31,-16,4,-93,-58,-45,-89,-66,35,34,-81,44,-42,52,-48,46,-38,-74,-20,-43,-68,8,67,23,23,-25,1,-27,17,42,1,-52,78,-94,-93,73,-87,-82,59,49,53,79,-55,-89,31,-3,9,46,-25,90,-97,7,-50,23,82,73,98,-16,99,15,78,0,15,56,6,22,29,-28,41,-59,73,-54,-80,-82,-91,3,-32,-29,-51,43,-87,-95,-49,-37,-20,-67,89,-22,-31,88,46,-53,-12,-39,-96,47,36,
    -15,-81,-71,26,92,-73,-2,62,-64,-98,-70,7,-97,74,-28,60,77,36,-60,-90,25,-29,-68,13,-83,-21,53,-70,-65,-48,18,-27,-29,47,51,-85,74,-50,78,63,-96,60,-78,-93,34,-5,68,11,-69,60,-26,8,-69,6,73,-100,37,26,83,25,31,1,98,54,-99,49,70,27,51,0,-10,55,-40,13,63,47,60,-17,-90,91,95,-16,-49,-73,42,-76,79,-68,2,62,57,-67,-84,55,40,69,-44,10,-4,-92,62,87,63,74,-48,-22,21,-88,13,32,55,9,68,-94,-12,-37,82,-33,-5,-16,82,-48,-30,50,59,-90,19,-85,
    -28,-85,-77,86,54,39,60,-94,-83,-66,70,-17,18,25,92,-14,-17,-20,49,-35,-1,96,2,-19,-52,24,-69,59,-14,50,75,10,66,50,-4,-28,-59,-92,-21,-41,94,-51,42,12,27,-14,51,-90,66,0,-72,-35,-3,-18,-1,-3,58,30,-43,-56,-67,84,54,51,-14,-98,-77,28,62,-46,39,57,4,-67,-31,83,19,-80,45,-63,-27,25,-46,22,-93,53,71,65,-64,80,61,-31,64,-33,-80,-49,-79,-57,79,84,-2,-30,41,54,3,62,-11,-78,83,-66,-41,-92,60,13,30,19,-81,-47,-63,-45,-66,98,24,98,-82,-4,-99,-61,
    -9,32,-77,41,-98,16,-53,5,-21,36,-21,-86,23,38,-78,-17,-96,4,54,-77,57,91,-70,91,42,6,42,60,-98,43,99,-7,-24,-77,87,-70,39,-66,36,70,71,-85,84,-6,6,58,29,-38,-38,-17,-63,72,-73,-33,15,69,73,-43,29,75,-99,28,-80,-71,3,59,-89,95,94,47,65,17,15,2,63,73,-40,-56,-65,-25,-21,-28,99,6,-9,14,-25,-36,72,56,91,25,-63,63,6,40,23,17,-13,17,-83,-47,86,32,7,49,-43,19,93,-8,94,-28,16,93,79,-93,-40,-94,-29,-68,63,-86,9,-100,77,-85,-8,0,
    -16,80,69,1,85,55,85,-8,-44,-58,-89,-51,-14,58,-26,-98,3,-95,61,63,-89,84,47,26,98,-44,-22,28,-77,23,80,8,3,50,61,-60,-43,47,-16,14,41,-53,15,-72,57,41,82,-39,46,-4,-76,10,-20,24,88,-69,80,-33,59,56,90,39,-84,-55,-59,77,-63,-49,-24,21,65,-82,20,-68,98,-22,74,32,39,-28,-72,15,82,-39,-61,71,-8,72,-62,-97,-20,80,42,-4,77,36,-27,66,87,50,-13,4,20,59,36,-30,-63,-38,-98,28,35,83,-56,17,96,-65,-12,-12,59,-22,-9,-61,-90,85,35,39,21,61,
    5,-40,-37,44,64,83,4,1,-47,41,15,-93,-30,-50,42,-34,20,38,-47,60,78,-87,91,-79,-96,53,-93,40,-7,-20,-47,50,41,16,-5,5,-1,51,58,4,-56,-74,-37,-34,-24,-94,32,48,-4,38,9,-25,51,-48,-4,55,-95,-45,-53,-50,-12,0,-99,81,-84,-52,-62,-33,99,49,23,-57,75,39,10,-97,-3,94,52,93,32,-87,-80,-65,-35,17,43,-78,24,90,-27,64,-57,-26,45,-89,-26,36,-69,-27,-15,6,-32,-40,-55,30,-37,42,-23,67,-12,61,32,8,97,49,77,40,72,-46,-18,97,-82,25,-77,16,-11,-3,
    -48,20,22,37,-74,90,49,-76,-27,64,18,2,32,-94,-37,-84,-33,-40,66,96,52,-10,-50,87,-13,-31,-36,10,37,53,-93,89,-75,29,-22,-48,-29,-73,-24,-4,91,46,98,-25,53,-38,-8,72,-26,-90,68,79,0,-81,66,87,-60,82,-3,29,36,-44,18,61,37,96,65,60,75,93,-43,66,40,-93,94,-55,-79,-14,-83,-4,-4,37,-73,96,8,45,35,48,27,84,-23,-85,-60,47,77,29,-5,-6,-11,-30,-12,46,89,80,6,83,-75,27,-31,94,75,17,31,-98,-35,-60,-1,-100,40,-73,36,70,-58,76,-31,71,-43,65,
    -82,46,87,6,-55,76,-14,51,59,-37,-22,80,-43,-46,-51,88,-92,14,80,8,-34,-27,35,54,-57,29,-18,12,-47,39,-71,-29,-14,-83,29,-17,93,-85,-66,5,-70,-36,-63,-13,18,-13,-73,-21,53,-92,87,72,-19,-26,26,-24,55,-91,40,-92,0,70,-69,86,87,60,-79,32,27,-45,89,57,20,-73,-56,90,-34,-76,-31,71,84,-92,-5,-35,-18,74,-7,-62,35,-67,98,-65,-97,-70,74,42,42,95,-73,-30,-49,-84,79,23,95,76,-35,-87,-100,-13,85,84,-5,-68,-99,78,-94,-6,-84,41,-21,66,77,-17,96,51,77,39,
    46,-96,61,49,-79,-8,-76,68,-32,-10,82,-80,29,19,56,-76,51,57,-98,58,-97,70,-1,-65,37,-24,18,-15,79,-53,76,-22,-48,37,-73,-75,82,-96,93,-50,-54,27,-77,75,-54,79,99,50,-11,54,8,-56,24,59,-21,-87,88,97,-1,67,-55,-73,-55,-51,17,-75,-26,-1,-71,-33,1,-73,47,24,-98,93,4,-99,43,93,55,-97,-63,-68,63,-31,97,51,-82,-52,-30,-85,-24,16,-36,93,-7,38,-8,-26,58,93,1,5,18,3,-50,74,-44,46,19,64,49,8,48,12,77,45,-85,96,94,-14,-89,-30,54,-24,-85,-53,
    66,7,-27,-24,-48,74,81,22,-71,84,-52,-15,30,67,-51,-21,-24,49,44,-95,-5,11,-99,-11,97,65,11,51,-59,-22,50,59,85,-77,-64,89,49,69,12,78,53,60,-84,-17,-20,-83,15,8,67,-89,13,-38,22,-33,3,20,-68,66,-77,-75,-56,26,84,81,1,-80,70,-97,90,-66,81,-5,47,49,79,79,19,-54,-13,-14,-43,-100,0,79,-33,55,-49,-49,-27,-73,-24,-31,53,-39,-50,-94,-67,72,-91,75,59,43,-29,-42,-8,-98,-63,-89,-100,-24,-51,-43,-24,1,88,-4,-92,-60,-1,-19,-33,76,50,-80,89,52,-74,-78,
    25,88,98,-64,31,-79,-6,-77,75,31,87,75,-93,88,84,-65,-58,72,-17,-50,-88,83,-16,-21,11,86,-49,0,39,-70,22,-84,-82,-28,-48,1,-55,-2,24,-80,81,-37,47,40,-96,31,27,-54,4,-89,-52,-32,46,-16,0,-91,-29,51,9,62,33,83,30,3,-92,34,-96,-47,84,-19,-74,65,-4,73,5,0,5,-68,-2,61,95,47,-71,-7,-69,29,2,-46,-67,63,68,18,47,-50,-78,55,36,78,60,20,-89,86,-15,8,12,42,-92,-83,75,7,30,22,-94,59,-84,89,-59,-30,96,-26,-66,16,92,33,67,66,-60,3,
    -3,-100,-76,8,39,-39,68,-49,-96,77,-80,31,-64,50,-47,94,-39,-79,83,2,-56,31,28,-70,-52,-27,63,-85,91,55,-30,-12,7,-54,49,-54,8,17,-51,-36,-54,-31,47,-66,-81,-100,-72,-67,74,-88,35,-82,43,-84,-52,43,-11,63,58,-68,-30,81,21,77,-73,-78,76,-13,39,-75,51,38,-5,-2,72,-34,-49,-99,-1,77,65,87,47,-40,55,47,-96,-56,10,14,-24,32,-5,49,9,23,23,-15,62,-85,11,14,53,58,64,25,76,67,-22,-24,44,95,-85,-9,-44,70,90,-88,66,0,-22,94,-68,74,44,-6,-51,-33,
    31,-89,-18,94,-23,87,52,-58,-35,29,61,43,57,-42,-9,72,1,-1,-6,-8,11,-40,44,89,-94,29,15,2,-77,64,70,54,28,-96,49,-95,44,-47,99,-91,34,61,4,-9,-81,95,-85,-28,-6,9,64,57,21,9,-1,-20,90,-86,82,-87,31,4,19,-41,-39,-80,-84,-95,-74,-84,-86,60,77,-30,-48,48,-34,-81,-80,12,29,37,-30,-98,98,21,-18,88,-13,-83,-99,-82,21,-28,-71,82,-7,-54,87,19,-86,53,-69,91,-76,-17,-9,-58,3,-37,54,84,-100,-76,38,98,-3,73,38,85,90,-9,55,-37,64,37,-54,-91,
    83,85,-20,49,39,11,40,15,47,83,57,-50,-54,11,86,99,-12,24,49,85,-51,-12,70,-61,79,-22,3,-5,-85,1,4,50,-14,36,-1,77,-52,91,-8,95,74,-51,97,72,13,83,23,-99,-41,-75,38,9,-35,-39,0,96,39,55,-8,54,-44,48,-44,-5,-15,55,-28,33,46,65,-20,-28,66,-23,96,-21,-40,-28,-20,19,-3,-81,80,62,-68,-19,58,71,-64,-98,-23,-55,51,33,40,-64,-12,-88,21,86,29,-99,-90,-4,-22,-94,75,-10,-70,-92,-91,27,79,90,-59,-89,-77,52,34,-89,6,-89,-44,-43,44,96,-55,84,
    61,66,22,-10,19,32,38,97,-10,14,-13,21,-26,-51,-100,-47,91,94,-84,66,46,50,77,-48,61,-66,-38,-43,82,7,93,-5,-74,15,38,45,99,-24,-5,-11,42,34,62,-84,35,63,-79,-22,57,-63,-56,-45,87,-78,-41,0,-92,21,9,42,81,2,-62,-93,69,76,4,68,-96,99,-42,47,34,-80,15,-79,35,37,0,-56,26,44,-1,-34,18,59,18,26,-68,-72,-79,-87,-18,-41,72,-96,87,-23,72,43,-72,82,-10,14,55,58,36,-10,-53,-64,-13,25,80,86,91,-1,-3,-38,77,-18,-58,98,-53,24,-91,-80,-20,96,
    -51,-47,40,-23,87,82,44,42,-8,-20,85,39,16,-28,-83,-52,-90,-92,-1,8,-78,-23,90,-36,-73,-63,41,-63,-91,-79,85,-90,-26,-23,-12,-38,-88,84,56,4,-36,-7,96,32,65,-87,80,76,73,-68,-64,96,-39,78,-40,-12,67,53,-23,29,75,-85,39,-51,-8,79,-37,-44,63,72,13,-73,-35,-39,59,83,-74,-8,11,51,24,47,47,85,-23,60,-75,96,-35,3,25,-60,18,-35,-58,62,96,-43,71,60,-71,-16,39,-53,-3,-49,-70,23,43,41,-26,-81,-60,74,56,-31,-14,81,-35,51,-64,91,92,6,-92,-66,-31,-96,
    -9,92,64,-27,-24,4,-28,25,55,2,-52,-50,95,22,-79,87,-52,77,-44,34,58,21,-62,95,-36,82,1,-28,68,22,-23,-89,14,93,-16,42,-51,56,-33,56,-90,-33,-94,-95,42,-73,44,90,56,-100,-23,15,22,15,-38,86,-51,63,-89,-83,-62,-12,-72,52,-67,13,-53,-17,-79,-34,-61,84,34,98,89,-24,77,34,-34,34,-14,-5,-51,60,62,-37,-1,-89,-74,10,-72,-84,50,-43,21,83,22,68,-34,-5,86,6,-21,20,-44,-79,96,-67,7,-85,67,93,-38,68,54,-75,-69,-47,88,-90,15,-83,-74,-35,-74,-1,-52,0,
    -33,-33,47,-46,25,-73,26,81,-52,-25,-86,7,-10,34,0,52,2,-94,29,-14,-89,18,-52,26,-13,-74,43,-35,26,92,17,45,-89,-36,99,36,-9,-22,17,-9,53,-17,98,95,-83,-49,-1,-28,-91,-71,10,-79,-1,58,-1,86,84,-57,-97,10,87,20,-44,98,84,-93,-14,-72,-15,-97,19,90,-62,18,85,-44,21,-15,-20,30,14,90,3,-87,48,-97,51,32,98,6,95,37,-74,-49,-13,62,-90,73,90,-52,28,10,-62,66,80,-76,-78,1,9,-46,83,75,-56,-13,40,92,-10,-57,-23,-60,49,-28,77,27,-25,-84,-11,-15,
    89,-20,-15,17,42,24,-65,22,0,-90,75,-91,-36,10,84,9,97,-24,1,-61,19,78,-69,-32,-98,-92,95,-23,-76,-16,15,65,16,-100,-18,-42,-24,-30,32,76,-20,-41,37,44,70,-27,-47,19,49,55,59,-32,-15,90,36,-12,51,-17,17,27,20,84,-7,-64,37,27,-5,-87,97,79,42,-23,-61,-21,-78,61,53,-25,-20,2,82,91,71,-80,82,-41,8,-15,-5,77,-88,-85,62,-43,-49,-1,85,-2,12,34,-22,-94,64,-31,-14,38,82,91,65,-38,-7,48,-46,-84,20,88,-72,-72,-27,23,57,-63,38,19,-5,89,-82,-68,
    -60,-17,18,-30,-59,-18,39,27,-80,21,18,86,35,64,-14,41,80,-94,29,60,86,54,-17,43,92,-79,-37,-61,63,-67,-77,-97,-32,41,-75,10,-76,64,89,-56,-63,8,82,24,-76,-32,-34,56,-74,47,17,-88,2,-100,56,-54,74,71,-15,37,56,8,-8,-75,-51,17,-13,-75,-67,-24,-78,-78,36,-96,46,60,-75,64,17,-49,12,-66,-36,-86,86,72,-40,60,-5,-3,-51,-49,-95,-7,-24,-94,62,63,-68,47,92,-46,69,28,-90,-32,-11,-65,84,58,-61,-4,44,-45,-38,30,-73,-26,-57,22,71,-56,25,28,-10,54,35,-48,
    -31,19,-100,-39,25,21,90,35,41,-69,23,26,89,62,74,33,69,-63,-85,96,11,-42,-30,-65,-45,95,63,45,49,-50,49,-81,21,49,-68,-54,-77,-78,34,16,-95,-43,94,94,71,69,-21,40,-94,-5,-12,-31,5,10,4,60,-43,-80,-43,59,-30,-93,30,44,8,62,90,31,85,24,-100,42,33,94,37,-44,63,-84,48,21,-37,88,43,69,-2,-53,-19,56,-81,39,-33,-58,46,97,86,-46,59,76,38,-4,-47,38,-61,38,32,28,47,-4,44,-5,69,60,-64,12,29,34,12,-90,-58,31,1,9,73,-53,6,11,2,-82,
    -60,92,14,-7,30,5,83,-86,-67,-18,-38,78,30,-68,38,66,44,-81,0,-44,81,-5,40,-17,-96,-35,-18,-37,77,36,-19,-83,28,47,-38,-42,-47,97,-75,-14,-20,-13,-84,-90,19,6,76,-84,25,-72,-76,-41,23,64,42,-20,82,76,43,59,13,76,76,-59,-25,90,-48,28,87,-23,67,-33,64,83,-23,-16,-10,-95,-48,67,-66,-24,78,9,93,-28,89,75,-51,-16,34,62,-88,-38,-97,-12,-48,-93,68,-61,-16,35,-41,-99,19,-12,-63,61,-6,-11,-20,-20,65,-41,89,58,31,31,33,80,-85,-81,42,-72,-19,-2,-32,33,
    5,-64,25,42,72,36,43,-57,-76,80,56,-30,-31,-64,50,86,-53,-8,45,79,-77,-70,11,90,-50,-46,70,-17,4,-62,69,61,-73,-6,3,99,-70,98,94,-94,78,50,-23,99,38,79,86,86,-29,-17,65,-6,13,-24,-63,15,-18,-93,99,38,98,68,0,-75,-86,55,76,96,54,70,-98,-16,-28,-21,-16,-90,-41,-78,96,-18,5,61,29,70,90,66,-14,-76,73,-63,63,71,57,-37,48,71,-82,24,67,24,-6,21,-91,18,53,-7,29,64,15,25,-54,72,-61,75,42,81,41,80,-95,-33,69,68,90,26,83,39,-51,54,
    -37,-84,-70,10,-10,-61,-72,-57,-16,9,-93,51,-65,-47,23,-74,81,-82,-93,-26,50,64,41,20,-15,-16,-54,-80,75,-4,26,90,-36,57,0,54,96,-19,49,-19,42,56,84,29,-90,8,55,43,-22,14,-83,-72,-69,11,48,16,95,47,36,70,-57,-37,12,-41,-80,-87,66,68,-54,15,-99,40,-28,86,-30,-66,-54,77,77,24,-8,46,52,23,-43,-47,91,-96,-100,-21,26,95,-6,-61,-46,-34,-96,-80,-65,-98,-12,88,42,60,74,-88,46,72,90,-25,-4,-66,-79,-99,9,-21,-46,0,-65,-46,31,62,-51,-74,-99,-45,-8,57,
    -72,-21,59,16,-32,1,-24,94,14,74,19,-44,-51,-85,-10,-30,68,51,1,-78,-49,37,-24,34,-1,-23,60,-48,33,-95,-91,13,-64,20,-71,4,-79,-43,-49,-13,-69,70,-5,-20,37,85,50,6,36,-48,-20,39,41,9,-26,92,86,86,96,-29,43,57,84,80,-23,-87,-64,-2,-78,-61,38,-47,-91,-67,33,-1,71,-64,57,-41,-60,-63,99,81,46,-75,25,-15,63,21,8,7,-22,-55,-61,55,-90,-25,-95,-67,15,-5,-14,-24,-19,72,27,-48,-92,-16,11,-100,-26,-38,33,-80,-61,-42,-43,-97,31,18,-38,-91,63,1,16,73,
    76,-27,-94,-57,21,-55,20,-98,-83,-53,-46,77,-16,17,29,58,80,-38,30,-81,-28,88,74,-97,6,-64,64,-79,-11,-68,94,-34,-43,53,-39,78,98,81,80,67,81,86,-4,-35,4,-75,-77,-16,-61,53,-45,-37,-7,-70,-34,51,-82,82,72,8,-86,19,26,-29,-28,87,-98,-78,21,34,41,2,21,-63,19,-75,-86,42,-39,-95,-53,16,-32,-7,-2,-14,-56,17,-32,69,-75,-66,-12,3,-43,60,-10,-41,34,63,94,-25,-35,-33,-36,36,-56,-70,-22,5,-65,-22,73,-45,71,-28,41,-85,41,-91,84,18,95,-76,-79,-48,84,-37,
    -36,-30,-73,-90,-3,-56,-23,61,81,21,91,-89,78,78,-11,51,-67,-40,75,26,-72,16,-13,-88,86,-18,-63,-93,-13,73,-77,51,96,-50,61,-7,-54,90,-93,79,11,50,91,-59,29,-20,44,14,-7,20,93,21,88,-20,-15,75,15,-26,34,-98,-100,57,53,96,59,-34,-59,58,-44,-52,37,67,99,28,60,-20,61,4,94,-46,76,87,27,17,-80,-88,-56,35,-61,78,-11,-61,-12,-58,87,99,8,-72,57,16,-71,-5,-65,-72,-77,95,8,84,-1,-46,90,-24,94,17,93,-86,-18,89,1,21,67,90,12,-93,84,-1,-93,-56,
    -21,64,60,8,-41,95,-12,-65,42,96,71,-59,-97,62,-31,97,-21,-86,63,61,-97,-84,-66,23,-94,46,30,42,97,-11,38,-71,-46,-2,-11,-35,-55,78,-48,87,26,-76,80,29,86,-98,-22,-83,68,-59,31,24,-43,17,-53,-85,-36,29,9,-87,-81,-53,-58,-27,45,32,-10,42,-38,43,-19,88,67,-38,-30,5,-36,48,-26,84,42,57,8,51,-25,55,19,-9,-15,-72,4,-44,-24,47,-19,73,-69,71,16,-7,-34,97,-67,85,-89,-45,-58,-73,56,-31,-88,-2,-74,20,-99,-47,-72,20,-4,-35,1,-99,21,77,0,-98,2,-17,
    25,-82,76,44,68,61,29,31,-83,72,59,73,-59,23,-77,-81,-57,-76,-75,-77,-3,21,-12,98,22,61,27,74,15,-71,-43,93,-100,85,37,68,47,-34,-1,16,90,-90,-59,83,33,-36,55,29,-60,80,52,-11,1,93,-13,76,54,66,2,-30,96,60,-37,96,-3,-100,16,96,-82,67,12,61,78,5,-4,-37,69,-49,92,62,83,45,-49,-63,38,-9,-87,-56,-43,15,14,-47,-73,29,-99,-23,-19,17,-27,-48,-63,38,-87,-85,-57,-39,-70,65,13,-77,27,-52,20,30,-15,-90,-79,-50,-46,-69,18,21,-64,45,-98,-62,-78,-16,
    7,-52,36,44,86,1,11,-71,62,42,46,-73,17,-27,-24,-63,56,61,99,77,64,-47,60,-18,26,97,79,29,87,-98,65,-6,-98,1,-61,88,54,2,69,16,-56,16,-4,61,41,24,50,-3,-63,49,-73,-99,55,87,-17,-67,36,-37,62,23,-83,27,-82,-81,80,-91,59,34,11,28,-97,56,96,-49,-31,38,75,72,-65,-88,21,-86,-86,-72,-98,-51,-38,-10,12,76,-86,-19,-44,-68,0,-64,93,-41,23,4,40,-22,-88,36,29,-66,-74,4,-94,-38,-32,-21,-72,-18,8,-70,32,22,-27,96,-2,-13,-22,6,-29,30,-5,-36,
    42,-82,20,-18,-52,-15,70,29,19,97,-67,25,-89,-47,-96,-61,36,-36,-78,-80,38,-5,-84,89,82,-54,-5,-95,77,90,-79,19,60,41,53,-92,-74,23,37,-3,20,22,-78,-17,76,79,23,-36,95,45,-64,34,-60,-96,23,74,-49,70,31,80,61,52,-49,21,-7,-96,-18,-28,27,71,69,-100,-54,44,-17,74,-25,58,90,-78,3,-74,56,-5,-70,-69,-79,-67,-98,52,65,15,4,-84,88,50,-80,-30,74,0,94,-5,-48,-60,-61,-13,66,-34,-54,56,-59,49,82,97,97,64,81,-82,50,35,71,15,-50,-73,32,-62,77,4,
    -39,51,-96,-93,-53,-44,99,-62,44,65,-43,90,21,-2,-9,-45,-53,-60,71,-72,-41,21,15,82,89,65,-91,21,56,87,-75,-83,-10,82,-76,-11,-62,75,28,34,40,37,-24,-87,-65,20,-32,-18,60,91,-37,-81,13,78,53,2,-4,63,75,-48,2,52,-79,92,34,97,-18,25,72,62,-41,-36,-1,36,-23,34,-92,-3,68,68,-12,83,40,-99,14,-7,-45,-90,-44,82,-86,58,35,87,3,-79,36,85,46,8,47,-94,24,98,-6,-99,-16,-46,-2,-48,-78,38,88,62,40,2,-92,47,-36,-36,30,-70,75,-83,-83,-22,-62,53,
    -85,37,-87,14,-5,-11,12,41,-10,96,95,40,-100,-83,-22,88,32,70,-58,40,-30,6,56,52,-12,-17,69,-43,61,59,-90,76,-4,75,-10,91,-36,-98,32,6,50,27,98,3,-3,-23,-57,81,99,-14,-27,-31,44,81,-79,-67,-35,42,-10,-74,-46,-47,55,2,80,-55,-6,97,-100,78,-45,2,-42,-46,-95,7,-17,-51,-12,-18,-13,13,4,-69,-6,-23,-84,59,72,-41,86,-74,-36,-7,28,-56,-10,74,93,-10,53,-51,45,63,-45,50,-30,38,51,10,72,90,-77,-24,-78,17,6,90,-23,-22,-51,-85,-44,-35,-40,84,10,50,
    59,-45,93,-36,4,38,79,59,-60,-99,-51,44,11,-78,34,-14,-50,8,-97,56,-1,-68,86,0,99,-58,66,-41,79,-72,62,38,-17,55,54,40,-7,-15,-49,-15,-14,1,-71,49,-25,64,-65,-23,-28,90,86,71,75,72,-76,26,67,90,38,46,70,0,-64,-47,-45,42,45,0,-73,97,85,-35,-50,-85,-86,-23,79,-51,54,-97,91,40,27,66,-35,51,-55,-68,-7,83,78,-37,-17,66,-32,-10,8,-86,90,35,-89,27,-100,-87,42,-86,-10,-27,-85,-56,-71,58,37,56,25,54,7,22,86,52,5,16,-85,40,-18,-17,82,90,
    -51,72,-23,-88,-49,-71,-75,46,-57,15,71,-90,12,-100,68,-99,56,-55,55,-85,67,-7,67,-76,-91,34,64,-57,-82,-2,85,-33,-30,-38,-68,-26,-9,-43,-80,-14,25,91,-52,89,92,68,90,0,66,-3,16,-15,90,-65,10,51,70,-74,46,40,25,31,59,47,45,91,-79,88,-99,93,-74,26,85,-26,15,29,94,-43,-71,60,54,-3,-54,-4,-15,8,99,-93,34,45,47,-89,-24,-42,59,73,-98,32,61,-97,26,87,29,-37,-87,96,44,59,-95,25,20,11,75,18,-93,-40,-74,-94,-33,12,51,66,24,79,76,-65,52,78,
    19,13,81,97,-48,-38,-88,-83,-42,56,76,-37,-18,-4,74,-43,66,33,-31,92,39,-12,-43,-58,-94,33,-79,82,68,25,61,-61,-62,42,-63,-58,-95,-99,-41,-85,58,-64,-69,92,-16,57,-99,-49,-9,-30,95,82,10,-48,-75,16,85,98,-50,5,24,-89,-3,14,6,-14,9,-37,-13,68,-70,-55,56,-39,89,-59,-81,90,-56,10,-88,-61,-56,-26,-8,-31,90,-71,-80,93,35,44,-96,84,-90,-38,-30,19,25,57,-60,8,55,96,69,44,-63,88,-13,-19,-50,-49,-27,-5,26,65,16,-32,46,36,-39,-67,-68,-82,-83,95,-68,-13,
    14,10,-55,54,-82,52,51,87,-52,40,28,35,74,30,39,-53,77,-35,64,-6,-15,-38,82,-1,-4,-85,-31,-35,-90,1,53,-24,-89,50,83,-71,2,34,-83,50,-74,-3,-62,52,-73,77,51,-95,94,-33,-49,-21,30,-67,30,-74,-100,99,-9,62,1,96,-61,12,46,-78,42,-100,8,11,3,-14,60,41,-61,87,18,90,44,-36,-42,47,-5,88,81,-74,66,81,77,-91,44,-22,58,35,43,-44,57,-63,-43,-83,0,60,3,-40,-47,42,99,23,-15,-4,87,-5,-57,-18,-17,-76,-40,-99,58,-62,62,-46,-32,20,41,63,77,-50,
    -100,86,-33,0,46,22,12,51,-35,12,-26,2,60,-39,97,-97,-5,-68,80,8,-67,-10,-2,47,-4,66,-32,-63,30,97,-13,-18,83,6,83,-19,-72,-53,-68,-55,11,-42,47,-29,-29,44,-25,66,28,-93,-74,13,-3,24,-39,45,-57,-19,82,73,-22,-79,-45,-87,27,90,-6,55,-62,-22,-99,1,36,0,73,59,97,-100,-23,25,-93,4,-61,-44,-72,-48,-99,-29,33,35,96,63,56,-96,76,-65,46,-78,42,36,0,-5,-62,88,96,-89,47,45,-89,-76,-30,70,-20,-39,78,9,13,79,32,98,-34,29,61,74,-15,37,-91,-69,
    59,51,68,-89,-53,-94,99,95,-83,-2,40,80,-25,62,2,55,76,-20,-84,-59,-41,1,-60,-23,-70,-99,-49,-33,-9,-40,-2,-50,63,-34,-86,62,-76,65,57,93,-36,-51,25,-9,64,-73,98,40,7,-85,81,18,16,-79,95,-2,75,-54,65,-34,-42,-85,-32,-78,-66,-18,84,-42,-52,-6,-96,12,43,29,55,7,57,53,99,-36,68,81,83,-64,-98,30,34,29,77,-49,95,-65,-33,-36,-91,-47,46,94,-37,94,-12,-33,-42,83,97,-87,-57,6,19,-58,22,-13,75,5,-24,30,88,62,-41,65,-86,-45,52,-67,19,62,-14,-83,
    8,49,64,48,69,-78,31,66,88,26,-76,7,21,46,46,-52,-96,-78,78,92,85,-62,57,51,-55,-39,84,16,-77,-78,33,31,23,-3,-21,-8,-28,63,-90,-40,-59,-66,67,14,81,-35,63,85,-12,41,29,-75,31,-62,76,76,-1,12,-8,-25,-14,-74,58,-91,-25,90,-46,47,5,-84,-93,46,-97,26,-87,36,92,76,-27,-68,-31,-98,-43,-99,-60,-67,77,-9,-3,-78,66,83,-52,25,44,-25,15,98,-77,-80,67,82,18,70,9,31,-42,-47,-41,31,-15,29,33,42,-18,25,27,-89,-84,-76,-67,35,-93,-67,-88,51,9,27,
    2,32,-1,-31,14,69,91,75,-47,-51,-72,-36,32,13,45,-83,-45,-73,42,82,91,10,-94,-76,-55,65,-42,-43,17,67,36,-81,-49,87,-60,65,9,83,41,14,-68,21,78,64,-65,-24,81,-58,3,-25,77,-6,85,35,-29,83,1,29,92,-82,-52,-19,89,-1,-80,81,-36,29,64,57,95,48,-69,74,12,-34,-50,-55,-40,5,-28,-63,52,-91,73,75,92,-26,4,37,-56,52,-82,-67,-97,90,66,-81,-28,30,29,-33,-22,60,93,42,78,95,39,-62,-47,-89,-24,57,-80,-51,32,65,-25,88,2,19,-8,-28,4,-5,-38,-30,
    14,-14,52,-57,54,-70,-97,47,24,81,-5,-37,72,48,-26,-52,5,-6,-51,-63,-89,-24,77,13,95,-31,37,-49,64,52,-27,-22,-62,-75,-26,44,-93,-71,-56,31,63,39,94,35,-13,-80,35,44,-34,36,33,-22,-88,10,43,59,79,-19,62,43,85,-65,73,-77,-88,99,20,-81,-71,64,2,44,3,-52,-21,42,-32,66,-62,34,2,71,64,-34,-19,8,-75,-88,41,39,55,26,74,-20,1,86,-68,-79,57,-39,85,-89,57,-60,-41,88,34,27,54,72,-87,-92,43,30,-26,76,-62,-49,88,79,90,-5,-43,16,28,58,-46,60,
    32,-89,73,17,-78,30,10,-67,18,-56,12,-28,-83,25,-68,12,-93,6,-11,-55,57,-71,76,-1,25,-67,-85,-47,44,-31,-87,-24,-68,86,45,-94,-32,-45,-61,86,-48,51,-90,-79,-72,-58,-15,88,0,74,-67,-91,-96,62,8,-19,47,-25,34,-9,-4,-1,19,28,-63,-35,-14,5,-28,-23,-57,-24,80,-47,-3,-92,-53,83,48,47,9,82,56,-35,-4,-84,-2,-57,-9,-68,-13,87,-17,-94,-33,20,-29,5,77,-4,82,-80,72,62,-75,-78,22,-28,57,-29,71,-82,-95,-21,-16,1,-5,34,44,38,19,83,-23,-98,90,44,-25,-87,
    -51,-48,9,83,-27,-66,45,-50,-44,-32,23,-87,-9,-54,-69,-4,26,-33,97,73,-98,-7,12,21,-23,89,-25,-81,-14,50,-68,-13,-97,42,-29,-72,76,68,-22,84,88,-99,97,-21,-100,80,75,26,-52,24,-1,-98,70,63,-25,99,53,50,-82,91,1,-50,-22,4,44,1,32,-28,-30,-90,-44,-42,64,-95,-10,16,86,-83,-58,86,42,-7,40,-36,-43,-85,-37,62,65,-67,53,-34,-65,83,22,80,-15,-94,-96,7,69,-87,65,-15,70,-45,-99,8,-27,95,-6,67,-60,86,83,-51,-99,-54,63,67,-69,-32,85,-34,-48,-40,98,-11,
    -82,55,96,87,68,61,-76,-10,69,-75,99,94,72,-55,61,-87,-68,-4,-86,-67,42,78,52,25,-54,-10,91,-2,2,42,39,20,-3,35,-40,-83,-51,-16,7,-82,-38,-42,64,-66,4,25,99,-12,-79,-86,21,15,44,-74,92,90,-32,35,-59,70,-23,80,-58,26,-32,2,43,17,39,-97,-13,-99,-39,-49,87,17,28,39,-95,1,5,-21,68,-51,-43,60,-61,25,-53,-68,-53,25,-87,-11,-49,-67,44,47,50,-17,2,89,-64,15,-60,-77,85,20,14,90,-79,-81,-79,41,-32,-70,-47,-40,7,0,-8,54,-75,-43,96,-71,-10,40,
    28,92,75,82,81,-89,-3,73,86,-18,-7,-47,-75,-34,-28,98,-93,41,-71,-40,53,36,61,-55,-57,38,-45,-61,19,45,31,99,38,-42,81,-29,-31,-69,-55,-93,13,38,-40,-10,-95,-15,-11,64,78,-30,77,31,-94,-10,28,-51,80,83,-60,0,29,71,-49,-81,81,-67,-10,-98,-36,-13,-90,29,-74,-78,72,83,-41,-39,-1,-63,-17,-24,68,-11,66,-3,-9,47,-20,-17,99,61,-93,-50,32,88,-65,75,91,-49,-38,53,33,-60,75,5,75,87,18,-25,-76,-99,51,45,-58,18,-58,-15,-83,-26,69,-84,88,76,-34,-28,16,-46,
    -53,7,5,10,-40,-62,2,88,43,-22,75,-39,53,51,-86,4,-4,-43,74,-10,-6,43,17,-37,59,5,91,78,29,-40,-68,77,67,37,-61,80,28,41,20,-29,-81,-53,-15,24,98,-49,81,-5,8,7,37,55,-49,-46,-82,-90,11,62,40,-59,-78,72,70,41,-38,-91,-27,90,2,93,-87,22,-60,-50,-54,91,2,79,-14,62,-13,75,17,38,-70,-12,0,-7,-50,-59,34,-76,65,4,17,79,-35,91,21,68,84,35,-58,77,37,-60,68,-61,-80,-94,54,7,81,-29,-55,63,59,-3,-43,61,38,43,37,-44,-100,-45,-65,-35,
    98,57,-67,82,-56,-73,11,81,-32,-21,73,-12,-63,27,95,-81,98,92,34,10,-11,91,-29,80,-13,61,-64,-13,-84,-29,52,-86,80,38,48,-76,65,-40,58,85,-9,31,-27,29,-42,-80,0,-92,64,-66,18,-94,78,42,86,-35,3,74,4,-29,-55,-92,-63,78,46,37,-98,-36,49,60,-99,-59,-9,-25,22,1,-53,74,10,-88,-92,-72,70,38,22,8,-97,-75,-18,-41,-52,-21,-32,-15,-43,66,-77,-88,-70,72,72,84,65,-84,-41,-61,-31,-42,-87,79,-78,-78,60,92,12,-18,0,-32,-40,82,27,-40,-86,95,46,-29,62,21,
    -17,44,-55,-92,28,63,-76,39,-98,-7,98,-84,-75,72,90,37,65,2,71,-83,22,-69,52,-50,-8,66,-55,90,89,-93,-37,-27,52,8,-67,-68,-77,-91,72,-74,-46,22,94,-21,-6,36,-84,-89,90,88,-71,13,71,81,63,-85,-1,-92,57,-12,68,-80,13,-28,-19,-2,4,4,7,-72,-18,62,50,-72,-59,-55,64,-90,-44,55,50,-63,-32,-27,70,83,-11,-31,43,-54,10,-37,-81,75,35,0,-26,92,56,81,72,91,-5,23,-81,-11,20,36,-1,28,-57,-99,66,-37,-26,-12,-54,15,10,41,62,72,-95,33,47,-8,85,73,
    -16,-7,7,-43,-16,-98,32,-44,-9,-96,44,42,32,87,43,-50,-50,70,39,-52,-15,-99,89,99,73,46,32,-28,-61,17,98,-25,63,5,-16,-53,-41,-84,-45,51,-80,99,-55,5,38,89,55,40,59,-54,88,96,99,30,96,72,-24,28,97,15,98,95,43,61,0,-73,60,11,-56,68,-38,-84,19,8,-27,-42,-51,81,98,60,-73,39,8,79,-31,4,3,-3,85,0,65,35,-5,8,-52,47,-13,-92,-41,-17,28,-27,0,-52,33,73,6,34,54,-44,-54,34,95,55,-35,-84,-41,68,-34,-56,-31,83,31,16,-9,79,-84,78,
    40,-73,-38,-32,-48,14,16,86,-61,74,-28,46,-69,19,-68,78,-26,-3,47,-15,-35,65,-18,-14,48,-87,-97,91,93,71,69,33,98,83,53,-98,-51,-30,88,-11,96,-87,87,79,32,-81,10,58,16,-91,-57,33,74,-23,-80,22,-9,75,-35,-64,46,-66,21,-4,70,-26,-2,71,-4,-61,-40,-55,-48,99,-76,36,-82,34,-6,-14,-5,-11,-80,-31,67,-60,43,-90,-85,-92,-54,13,95,19,61,17,45,-89,-12,-58,-98,1,-61,-94,0,-37,42,-29,50,88,57,-55,-22,77,15,97,-83,-90,-93,84,71,-95,-51,66,-24,-90,-17,-79,
    -78,23,-85,-76,-76,-46,-69,-23,18,25,48,-80,14,5,-35,44,-17,32,-7,-48,-5,0,-63,-34,-43,-62,-68,33,-51,-33,6,-77,90,22,99,67,76,82,96,-54,8,44,-34,74,-99,-16,18,-16,-84,-89,-11,11,-37,-22,77,-28,16,61,57,65,-72,-37,40,71,85,40,-10,14,22,-14,60,82,82,-73,56,83,11,-74,-80,-21,89,9,43,-96,87,20,-24,-45,-18,-67,73,-38,-51,13,85,-66,-47,-25,48,28,13,-39,10,-5,-60,-81,-69,3,-55,-49,82,87,60,25,91,-1,98,-80,54,-68,-95,27,-6,-46,-59,-68,-59,-54,
    7,89,26,73,-98,89,-80,-58,-92,-49,45,-95,2,28,92,-86,-95,36,-35,-97,8,-80,87,13,-1,-18,20,-8,14,61,-9,73,2,-83,-2,5,-94,19,47,66,70,-55,72,-75,25,16,-61,30,-96,-43,-14,12,-23,-27,78,28,7,-2,-79,73,-89,12,-1,13,-19,97,70,-12,16,18,-46,-61,15,78,-36,-60,-53,-45,70,-49,12,-92,16,41,-66,-6,-30,41,44,-57,-33,55,55,66,-80,88,63,91,-24,-68,-39,-17,-29,76,-87,-13,-84,-40,42,38,-36,-93,47,80,48,33,-74,70,-26,22,-87,41,-23,20,-41,-51,9,-77,
    -60,37,-93,1,-80,-22,-71,86,65,97,-54,-41,36,10,-34,-65,-58,-85,68,-32,37,94,90,51,-12,-81,-29,47,-31,32,22,-39,-30,29,15,-58,59,44,-72,24,42,-25,36,-70,37,-98,-35,32,69,-67,52,7,-73,43,-90,15,14,-67,-85,35,66,37,97,-64,-33,12,-70,78,-92,-41,-45,2,-14,91,32,75,45,97,7,15,82,-40,74,62,55,84,-71,-79,-83,-56,57,35,82,54,23,-99,-82,54,31,78,65,86,-19,-49,77,65,-74,23,63,-14,90,-3,46,16,-89,-47,-48,-59,-26,-31,-15,-69,57,-81,37,-20,72,7,
    -66,4,-14,99,-10,19,-98,-80,-16,81,-5,99,-33,-63,-51,65,53,-40,-82,-95,1,-56,-74,39,-72,35,10,17,-84,-17,-75,-98,39,-37,54,81,34,56,-99,-82,-11,-52,70,56,-15,-81,-27,90,-69,-57,47,33,-12,74,-76,16,-39,34,-67,-23,69,-90,-68,8,-75,86,-10,59,-6,-57,30,84,-8,0,-8,29,-29,66,20,-46,-91,-33,87,-51,93,63,17,-45,-2,51,-16,-81,13,16,28,39,2,-82,-2,49,-87,-72,33,5,-20,-75,-65,3,43,7,58,5,26,-3,54,-28,-39,-28,-21,11,75,63,30,-60,-68,-90,79,-14,
    80,-70,-65,94,-42,68,-1,-9,46,-14,94,-11,45,4,46,-76,-98,1,-4,-85,25,27,-22,-48,-10,-92,-8,-26,-81,-28,61,51,2,96,45,12,17,97,55,-37,-65,2,4,81,-94,-49,57,60,4,-47,75,-71,32,-47,-19,74,-86,25,-51,85,49,-90,36,-97,58,-18,16,-25,-69,-77,-10,-34,25,95,99,32,98,56,-56,2,61,72,31,-7,-75,64,-32,91,-59,-31,28,91,-21,-35,94,89,-1,-38,-35,30,-14,7,96,63,2,48,-53,0,4,-8,54,-82,-36,-15,11,-59,-99,31,-67,-57,-100,61,86,-69,78,80,73,-71,
    95,-62,59,-19,-3,8,-56,-100,-44,-8,-48,12,36,-93,30,-100,-56,94,93,98,25,78,-59,-22,-8,-73,-39,-30,59,-66,0,54,-76,-89,87,22,19,84,-26,27,-72,-22,92,64,-15,22,16,-18,-32,9,-20,46,-12,-27,76,32,-48,-63,-98,-89,-28,-46,66,-52,-82,5,70,37,-11,96,-83,17,-25,-91,-19,12,-17,97,46,4,-41,-74,-50,-1,-1,-74,-69,3,-37,85,15,-13,-60,-67,-12,-42,38,-90,47,-72,-93,16,45,-18,-23,79,46,13,76,-7,-83,87,-29,-33,-14,23,-55,-31,-74,-40,-45,41,48,-53,74,-64,-43,-35,
    98,56,-7,-95,-75,90,-61,2,69,86,-85,46,-69,84,-15,2,-49,24,-75,-52,93,-48,9,-100,45,-43,-53,20,-55,-44,37,-57,65,-70,-99,90,-80,-60,-8,90,78,-40,-12,-91,-56,25,12,-52,-51,-11,-4,95,41,-95,95,-13,-86,95,59,-89,-49,-52,-93,-84,-22,-92,58,-50,-100,-49,92,79,-37,-68,40,7,10,-96,7,59,-6,56,54,87,61,2,-74,28,-3,37,-9,0,85,-2,-31,-85,-42,-73,-34,-41,-70,-90,-62,-7,95,78,53,5,83,-88,-36,-71,68,71,-84,-18,-75,-57,-90,-26,80,1,74,18,52,95,33,-90,
    75,99,-31,57,62,-41,-97,57,90,8,-86,25,20,-70,54,-59,-99,70,23,78,65,-15,-48,-2,38,79,-84,-10,26,49,-47,1,1,22,-89,-37,82,-86,72,24,22,86,-51,-58,-84,-97,-17,70,25,58,48,-9,-57,53,89,34,32,-95,-76,-42,6,-23,12,-93,52,23,-78,-14,-11,46,-90,-89,32,-41,-95,1,-38,-59,-77,87,99,23,-70,95,-24,-81,29,60,76,5,-81,-65,35,-17,-58,-13,58,-83,73,-53,63,35,-42,-52,94,-85,49,-92,-44,24,47,-92,-53,78,55,-24,49,-64,-64,-22,41,-93,13,-72,-10,7,15,-52,
    24,-60,-5,-12,27,5,36,-27,21,-63,-19,29,61,81,-11,-92,11,-56,-64,-88,-68,-27,-10,74,-20,55,-98,23,-37,-30,71,39,-38,-81,-73,90,-24,15,-37,49,52,-3,-69,13,-22,20,-26,41,17,10,53,49,35,96,-25,-84,51,-70,-9,66,-48,-38,6,14,-67,-15,-44,10,-99,-28,-89,53,69,-58,-81,99,63,-55,40,-68,7,-7,81,-57,-59,-91,-89,93,-61,2,59,-57,16,-83,9,2,3,66,64,-44,-62,75,-91,59,-82,80,58,33,77,-50,65,-63,-5,-2,-20,37,7,-57,30,-2,-3,-59,-7,-87,-89,-97,-33,14,
    69,-69,70,59,-93,31,18,77,-36,-24,10,93,-22,-73,-70,73,25,-38,10,85,5,92,-65,-98,86,29,68,97,-16,-65,11,-47,-81,-67,-88,-74,16,-18,-97,80,58,-35,74,88,-8,56,61,69,-81,-76,54,24,-84,42,-21,-98,-77,99,99,7,86,-38,60,-95,-53,24,-69,64,-94,86,-4,16,-49,-78,4,-5,79,-83,-35,98,-59,71,-26,-90,13,5,64,-12,4,16,95,-57,78,7,-52,78,-69,-68,-6,89,70,90,-43,-78,65,13,-31,-4,31,34,-54,24,-42,20,-14,23,78,-49,-88,-66,67,-93,-23,-3,15,26,-25,-2,
    58,69,-60,28,12,-3,2,-23,63,24,25,-54,-90,71,70,-32,-57,-43,92,-79,8,-96,-44,-73,63,33,24,-22,-89,52,29,21,21,69,-50,85,-82,4,-86,81,80,39,-73,91,10,50,-41,6,-93,-97,27,-33,59,35,-6,23,-79,70,53,-68,22,-18,54,96,-97,56,81,-78,-88,96,55,93,-13,35,84,50,-15,95,-44,-56,99,35,-37,58,-29,57,-19,-56,-21,87,-24,-98,-31,-70,98,-75,38,31,99,51,-21,54,44,67,-11,-20,17,26,-25,25,70,-74,-40,-67,85,-17,42,-82,27,-78,-95,56,76,-73,38,26,-48,-23,
    -43,-97,80,-11,57,24,-44,99,56,25,25,-69,-98,-4,-42,-86,-19,95,-2,-76,65,77,-54,23,33,74,2,-76,0,6,1,9,9,-19,98,-34,57,-94,-83,-35,-17,43,-4,-15,91,6,0,-28,-47,50,-52,-81,-21,-54,-6,65,-80,96,89,-28,2,-58,82,-89,-25,32,-71,84,39,-53,-51,-78,42,45,-40,-15,-96,12,9,-91,-86,58,-72,-7,-96,-26,-42,77,-30,99,-51,72,-59,-69,83,16,16,65,0,-45,-88,1,-71,6,-53,-59,91,-97,53,-48,-88,67,-90,93,-87,67,-33,-29,44,38,-77,93,-38,-84,77,98,33,-7,
    63,85,0,27,-13,-19,85,-14,23,76,-59,-24,80,-47,96,91,98,-91,58,-34,-68,-46,56,-45,-53,-82,-28,76,-84,57,-79,31,-6,21,-42,33,-97,-57,71,26,71,-88,54,52,-82,2,-5,68,11,-95,-66,-4,-41,42,3,-42,13,75,87,-19,-16,-92,-87,31,-18,-77,-84,-63,-81,88,63,42,52,-31,-54,22,24,-7,-9,-65,-2,25,-17,57,-80,87,68,33,14,55,-86,99,15,79,-18,97,55,-2,-66,74,-14,49,-32,91,71,15,13,-5,-92,-96,-18,-41,-18,66,68,2,53,36,87,-33,-57,53,-82,59,33,-48,8,-12,
    51,43,-86,89,-56,-18,80,-85,49,-6,-38,10,-50,45,69,32,11,37,-66,-84,-74,73,-65,-79,27,-94,32,-40,-42,-59,0,61,36,14,51,80,96,-17,48,-2,29,-90,-92,80,7,77,12,-82,-34,-1,86,-56,-28,-78,-34,99,80,98,-89,90,91,11,52,27,77,-45,-40,74,-10,60,-76,-80,70,-68,52,-22,61,-36,-52,79,15,-65,-76,40,-91,-58,-9,41,92,3,31,84,66,-65,-37,44,-10,23,-30,81,-17,-6,53,-94,-22,5,-64,91,21,-16,70,89,-29,46,29,-68,88,72,25,81,-73,-43,17,94,-56,80,-10,35,
    -44,12,-32,39,58,-79,-3,-64,-22,33,-73,99,70,97,88,-59,44,-31,-74,-16,-58,51,-83,-31,60,34,-85,-95,15,5,-8,23,-83,60,-86,75,33,12,63,63,97,90,62,67,40,3,61,-64,24,87,-28,66,-10,-10,88,-49,-76,-97,-92,-9,-39,0,-86,-70,-88,-71,58,-3,41,21,-40,38,-36,-78,-42,56,-23,-81,44,-98,58,-84,20,-52,6,8,-49,31,-36,-89,74,25,11,89,7,-25,70,-35,72,11,87,-68,-99,-97,-93,-41,59,-16,-70,3,-62,88,19,59,89,-74,-81,-60,9,-17,-48,-65,60,15,-76,-32,91,94,
    33,63,-43,-28,96,-89,-25,-97,-78,-66,-61,-47,-11,-22,93,61,89,82,39,-92,-77,-52,-8,-73,-65,-96,-6,-40,-28,-15,-94,58,-99,-84,-70,49,79,6,4,-99,-8,43,-94,34,73,-100,-5,62,82,-66,71,57,-66,-85,36,-31,-81,-69,-19,-56,-32,-12,-98,-31,-44,84,70,-65,-58,-26,88,35,70,95,-79,-5,95,16,-42,29,50,81,-61,36,48,75,-95,19,-94,-13,15,-73,27,17,-52,35,2,-81,70,44,-55,10,-69,-85,5,-48,11,-48,20,21,34,-78,-46,73,58,54,48,16,-27,7,-97,89,34,-18,-42,82,69,12,
    53,39,-91,99,-99,-60,66,7,-55,-71,11,17,2,-55,40,56,-82,-50,10,19,66,36,-74,21,25,12,55,35,-54,24,-52,52,-85,-43,51,-83,-51,-31,-24,-54,-49,87,16,53,85,-44,-38,55,6,72,-26,-75,-40,-48,46,37,16,2,-27,63,78,-27,-33,94,-18,70,-89,-17,-61,-13,30,-10,-26,-54,96,-89,-46,-42,67,12,82,-59,37,-57,-54,-64,80,62,-10,-95,-23,-32,-22,44,-38,12,14,-27,96,6,12,-22,48,39,-24,44,50,-18,54,17,94,37,11,-16,32,9,-28,-36,-29,62,-30,1,-70,0,-55,-55,65,12,
    -30,61,-82,83,-61,66,-26,-33,11,-76,49,-35,-6,43,54,57,79,-14,-34,-49,3,37,-87,25,-62,96,-75,-64,93,42,48,-37,3,66,98,-6,32,-28,61,95,-51,-38,-39,43,58,67,0,37,-46,66,89,57,-45,54,-66,-6,2,-41,30,-5,54,-70,11,-43,96,9,4,-20,34,-83,-24,83,-20,-11,26,-62,-44,78,75,-38,44,16,71,99,-77,-95,-55,25,-83,27,73,71,57,-64,80,5,-3,-16,86,-69,2,-38,66,82,3,-8,-80,59,-30,47,74,66,-84,45,66,-61,51,63,16,20,-9,-59,-9,-52,-23,-29,54,27,
    8,-8,58,-90,6,-23,44,-91,69,16,20,92,-85,-6,10,-69,-60,-72,-30,43,92,39,-85,-17,80,6,-17,-90,29,89,37,-63,-19,-53,99,87,24,-5,48,-54,63,-31,90,79,63,0,-90,55,29,-67,-50,-27,72,-35,56,52,23,91,62,-95,81,-49,94,-86,-1,46,54,-77,-59,2,-31,57,71,-41,36,-13,12,-2,94,-59,31,97,14,-45,-86,-78,60,-10,65,-26,-5,-2,26,-59,13,-75,87,-81,-100,81,21,-78,38,-55,-19,-74,-16,93,24,-70,86,-92,79,-48,-37,94,-74,75,-64,44,-50,-17,-58,76,24,7,53,-36,
    26,5,97,-52,27,-13,-55,-91,13,-19,54,-63,11,-7,97,-9,-3,-87,-63,24,88,-75,-32,-62,8,10,66,84,70,-29,-52,-4,77,97,-4,4,-16,41,-35,97,74,-80,87,-14,-35,84,29,62,97,18,-14,86,43,6,76,-49,69,43,87,39,66,-64,87,43,85,-16,-100,-30,77,-35,-33,52,-63,-94,90,2,91,-29,-83,40,89,-45,26,-16,62,-45,35,-17,50,-26,-78,16,10,-91,-88,-52,-55,-88,18,-77,-23,37,27,-33,44,17,21,35,40,38,-25,29,94,54,-87,8,-91,0,91,59,74,65,-73,-63,26,-61,-15,72,
    -49,55,47,-19,-8,-26,48,-64,43,-31,71,-17,60,99,-88,54,53,77,-86,14,-71,-95,-75,55,70,-48,44,-52,-8,81,-80,43,36,67,76,-71,-59,24,-35,84,46,89,-33,-42,88,-69,64,-7,60,78,59,89,83,84,-3,5,-64,-59,5,-20,-77,26,76,59,-7,52,40,-13,-71,-94,-77,75,47,-57,-67,-13,-26,49,32,87,27,-9,28,-38,-73,-75,19,-37,-33,-76,-4,-10,50,-28,-51,96,-76,90,-65,-47,48,-42,80,-5,1,65,34,28,14,-34,15,-59,57,-57,-45,36,-31,74,51,-12,99,-53,78,1,19,79,-3,-4,
    21,32,49,-79,43,-18,-32,-56,-53,-98,24,14,-32,91,55,77,-65,63,13,-44,37,-35,44,-12,-36,-26,90,84,-47,-61,-68,75,24,81,48,-33,-85,-83,-37,-37,-81,88,29,-12,79,-16,-83,66,-53,-17,74,37,0,-30,-75,64,44,67,0,98,-93,32,25,31,-34,-27,50,-19,-10,-87,-4,-38,53,-75,2,33,-38,19,-49,-39,-98,-74,50,-46,-4,76,-81,-59,43,-29,91,2,-96,-32,-67,-78,-7,83,-45,36,49,52,98,2,29,0,87,43,71,-61,-95,26,-83,7,80,13,-17,99,6,79,-29,-51,-19,-73,-31,15,-99,63,
    -50,-44,-49,51,-92,-51,-94,-10,1,93,-67,-28,84,90,98,-99,-2,79,-33,33,30,73,12,53,75,94,32,-56,-39,33,59,-37,90,-90,67,50,-89,73,-60,12,66,-74,85,51,68,35,4,-82,14,71,52,-55,-3,-84,50,-28,62,-17,-32,-77,-84,80,-61,-94,42,-94,57,54,31,49,-82,-51,75,-97,-100,-4,39,5,14,-95,-72,-82,-98,25,-65,53,49,97,-64,-82,73,-96,50,-88,-89,-8,70,20,-54,-47,-31,-35,2,97,-80,-45,-7,11,12,59,17,40,78,19,-34,-87,-76,-85,62,-88,85,35,-83,35,-1,80,-72,-79,
    0,-74,-26,21,-57,29,18,-36,84,63,27,-4,-25,-56,-64,-47,16,-46,-82,-8,-78,32,5,-93,-80,22,43,19,2,23,-7,54,1,19,-25,45,-52,46,61,84,-39,-60,-20,36,-63,-31,41,53,-25,11,97,-3,-56,2,5,-36,76,-100,35,-70,-25,-72,84,28,0,-40,-75,0,-94,86,-15,-33,-73,17,56,-36,38,-3,69,-86,-91,18,-37,-95,-79,68,-79,97,20,56,-72,47,37,64,-24,-63,-76,1,37,82,-60,74,-98,-81,-8,58,-65,-70,-93,56,96,68,-26,-40,73,-5,80,94,-55,1,3,-75,-100,92,89,-72,-71,66,
    30,18,48,22,-55,2,41,37,-40,76,19,-80,-16,16,88,58,76,62,-94,8,8,-97,61,63,-72,62,55,69,90,36,35,-28,-93,-64,94,-48,38,-13,89,51,-85,-40,71,-1,76,11,-42,4,-75,-84,-87,-14,-81,74,-51,99,-12,-95,68,-21,-7,-44,3,0,-8,-2,52,82,85,-7,-15,53,54,-44,-48,-18,20,-38,87,45,-22,52,-69,-51,-22,-19,-52,67,-62,-83,98,31,-27,1,-68,17,51,36,-1,-11,-70,85,-58,36,93,-6,-82,-35,9,57,-89,87,-39,-58,89,40,-25,-63,7,-35,-94,-43,97,-69,10,81,48,-38,
    17,-100,51,-1,37,45,87,-18,-61,58,48,0,15,11,-12,77,5,-71,-31,81,-34,28,-54,-75,85,95,-44,-5,28,57,9,98,9,-88,-51,98,57,-63,-20,49,95,80,-51,-38,-9,-11,-61,97,18,-40,30,37,88,28,62,25,-24,-82,73,-96,27,-18,-98,-64,-5,-48,-66,-48,-59,-33,-99,-64,99,-49,-2,-9,-8,-10,-60,-89,50,22,48,-9,-98,10,-84,-22,-20,89,-65,60,-76,89,96,19,-59,83,23,-18,-98,-23,-30,-99,-20,69,-56,72,-89,-64,83,13,10,31,-96,13,93,73,43,-74,14,78,86,38,68,34,-43,61,
    -31,81,96,-29,-42,-34,-75,38,87,-31,-38,98,-42,-54,12,68,29,-32,-67,23,-59,77,49,56,7,-13,-54,-73,-79,4,89,-9,-63,85,14,-53,-97,39,37,91,61,99,41,71,97,-95,39,27,-26,73,-98,-33,50,3,75,-91,90,-78,-63,63,-22,-74,-46,-33,63,21,-34,18,-88,-97,9,73,54,-97,44,4,-92,-64,-69,-66,-91,85,-46,11,-12,81,20,-70,55,57,93,-67,35,-100,-48,-50,-27,18,69,-15,-27,-70,-89,-72,-67,7,-16,94,43,15,-72,4,-48,-66,-85,-8,-84,-12,22,71,97,-33,-95,-15,-81,-91,35,-8,
    28,-44,30,-99,-13,41,81,-28,-52,-35,-34,44,32,-5,-52,84,-19,-84,-24,-3,4,50,-31,1,70,-74,86,41,-65,-26,-66,63,30,16,17,69,-43,-50,-58,-43,16,8,-99,-100,55,-98,37,-63,-82,65,34,22,16,55,75,-62,81,14,79,69,88,65,-16,-30,-19,53,-60,-10,-96,-66,48,-28,42,1,-28,-50,3,-91,-13,21,75,73,95,-57,29,71,-19,-38,-15,-40,83,-75,-74,-32,-53,-41,-79,-61,50,-75,-27,-50,-51,68,51,22,-82,-93,-17,-43,-20,-42,82,-24,-47,-37,99,34,26,-16,47,9,-39,25,29,-40,84,3,
    -48,34,-20,25,36,30,-55,-12,-48,-85,-53,35,-28,27,46,-45,55,51,18,54,-14,96,90,85,-42,-49,10,39,-36,-54,-58,16,33,23,93,-31,-47,91,9,-43,-94,56,44,-69,-64,-58,-62,91,94,56,46,-68,5,36,-83,15,40,-21,-46,-96,25,-51,-28,-42,72,-83,-20,77,-92,89,86,-85,46,30,98,34,73,-64,25,67,44,-29,99,-51,-92,68,16,0,-53,23,-44,24,72,28,35,-4,-3,-85,-27,6,-96,-89,-27,2,41,-29,-64,-34,-41,62,85,-97,85,84,-95,45,4,21,-55,3,-56,1,-72,-84,81,63,-36,-21,
    30,-11,85,-14,0,-90,89,-58,-67,77,-40,-8,39,-54,-5,-23,-70,-48,-78,-13,74,-32,90,70,69,-82,39,51,33,-97,82,-85,-7,19,2,93,29,-9,87,14,-32,-52,-94,60,-6,-47,-63,-24,-42,-89,15,-68,-21,6,54,-99,-24,-7,4,-90,-3,-14,-75,42,-95,-21,35,-14,22,-25,-100,91,-25,58,3,21,63,-8,97,-79,3,-35,-95,35,71,60,-12,-53,-95,92,-91,2,-70,87,-56,87,-34,32,-27,89,59,-75,32,-66,35,87,55,-50,-21,4,-76,34,69,81,-79,-8,41,9,40,47,-47,49,-99,-17,-12,98,-30,55,
    -18,43,-4,-59,20,-72,-73,-93,-85,34,-42,-54,-62,-18,32,60,-37,54,-48,57,-85,92,-44,-31,-6,-91,52,34,59,75,-11,93,18,37,86,91,65,-87,98,32,47,-92,30,38,-10,-85,-2,6,21,50,-85,36,-5,-77,-95,41,84,10,-25,44,-15,17,37,55,-46,24,-2,72,-11,-3,4,-63,5,87,27,48,-98,-75,-94,23,27,73,-41,74,48,-83,15,32,-21,43,-72,64,-88,66,-29,-34,-58,70,-62,31,-81,-5,20,-24,-18,99,76,-64,-76,34,59,4,7,70,-22,-45,87,46,40,-82,-59,68,34,-47,-14,-94,-29,28,
    28,10,-88,-1,57,-16,75,-61,-64,52,-25,-40,-62,86,16,46,56,47,53,-4,-55,-55,14,-62,-86,-99,-9,0,-93,62,81,-13,-76,45,86,81,81,13,72,-83,-35,99,-70,4,85,46,-98,-6,-55,7,90,-10,53,-44,28,19,-43,71,71,-84,34,52,-45,10,-51,-59,-8,-69,55,-36,-52,72,-36,-70,-72,-99,77,82,-5,74,-58,-15,-83,95,-6,-3,-34,3,-31,-63,20,55,42,75,-35,91,69,9,-78,-24,74,-77,-52,90,-95,-71,91,34,-89,-61,9,-95,-24,26,-100,70,-77,-82,-26,-56,-44,46,99,98,-27,-83,89,94,
    26,64,-30,-48,39,-29,94,44,0,38,-21,-37,77,40,-31,-47,-34,-79,76,-59,-60,-98,-14,-52,0,-15,-54,73,-46,87,68,81,3,90,-67,94,-39,-72,-9,13,66,70,29,-5,-38,50,48,28,71,24,21,63,-22,-93,-89,78,-55,-91,52,-1,97,-28,-20,-48,-38,-34,-53,-24,-54,38,89,-88,-40,-30,7,22,-80,-93,-98,92,-16,23,7,62,-17,19,41,28,-20,-55,79,-23,17,-88,-18,31,-22,29,59,24,-81,1,-12,-21,71,47,-99,92,6,55,36,90,-22,-57,53,61,14,-54,-59,95,-9,73,-76,60,85,-42,-9,-85,
    87,-97,-9,-94,4,31,-63,-25,-22,90,-81,36,45,55,27,76,51,-68,-11,-83,-22,31,64,21,4,41,-19,-59,99,24,-44,39,27,47,45,-69,-22,35,7,8,25,-22,-56,-77,-66,23,99,37,55,-60,54,85,71,19,-42,-73,-40,39,-80,-41,-36,-24,98,91,-77,-4,-77,-47,83,-18,61,-40,60,-94,-17,46,-19,-66,-17,89,-25,-62,74,98,9,-67,78,69,24,98,80,88,-25,31,80,-50,79,55,4,62,89,-35,-78,1,-77,-42,-100,-43,-8,-17,-54,67,-27,-80,66,34,-95,44,3,30,42,-64,-30,69,67,-98,-80,-2,
    9,-24,-40,-2,-7,-66,52,-31,-8,52,26,85,87,72,52,-39,44,-30,95,2,66,-49,84,-39,-61,-46,-70,58,-91,-98,-44,18,-22,16,69,24,2,21,93,95,-27,19,80,-40,-57,-16,-27,87,7,-79,-11,-27,-28,-27,-14,11,-20,17,-79,-11,-29,-23,-93,2,45,76,-74,-53,-51,-81,-58,-26,-10,74,35,33,-89,60,20,18,81,62,43,-95,-65,-70,-32,-33,-1,-11,56,70,66,64,-76,-37,-8,-50,11,-6,-79,5,68,11,-68,-45,96,-57,-84,-31,13,97,-69,-92,55,-82,-62,-77,-14,-11,13,-58,-88,-69,58,-64,-5,-97};
}

std::vector<int> getC() 
{
    return {
    -51290,-27293,7218,-22361,-4025,47251,60373,14556,25860,51268,-11905,-55712,16800,-16114,-61903,71497,-1225,-1727,68035,29526,-34518,-67948,36112,24567,-8087,13176,-14382,-13081,17952,101830,-34863,-29168,-13522,-18046,28425,-63920,19016,22887,-10741,18925,47767,46316,44619,-45505,-14043,-29044,-25219,-30309,6596,71687,-41974,-48567,-42259,19667,9057,-39253,12921,-13736,-25572,-38290,87632,35675,-4591,62601,27052,-22753,-76055,-40052,50894,-71158,-18670,42161,90048,-60337,17466,17512,-98355,56846,-2621,4472,-26148,-40993,32306,-65812,13165,74525,7702,31462,6936,-72646,32030,49747,-39496,34856,34327,-4407,22566,23994,13868,-919,-23353,-38727,17611,30896,14094,33744,-51682,-58670,44341,13345,-55662,16966,17004,-53097,-1653,30578,-39886,-12092,57838,0,17294,5144,47892,-50314,-10910,-32700,51271,16694,
    -24341,-64212,-30545,-65265,-10502,-23352,-24933,-1657,16252,67778,14714,-33216,48883,-40760,-108110,30457,-12470,-58630,-22781,14889,-69809,22564,20274,-6168,-22853,-7684,61478,-58161,-5504,-35261,-9403,25632,-34545,-39600,38278,-20866,1129,11521,-49278,63385,-9721,29746,26512,50905,-77915,12427,5333,30152,-23644,-2644,10124,-48778,-53399,-18955,-55263,8888,-28236,17194,-65709,47613,-30115,36050,15174,-20355,6124,-53415,2029,63483,8565,-33516,-16923,-10751,36965,48419,25728,9800,-61588,-31966,-82566,43449,15637,12357,56864,11962,16441,-24157,20150,11981,9979,3728,84,-37407,-31177,14197,6673,2079,-15122,-2618,37835,-11767,43704,-27982,-37203,-21647,-34490,-13187,6163,-47041,13851,40041,59208,95666,-72075,-25769,5546,10219,-20865,27895,80808,-1163,78937,-29694,72738,-46598,30732,-33876,-5092,-58127,
    -83511,-13995,82556,1994,64153,16182,5927,21418,39071,45715,5237,43396,-9807,-62669,-23295,43129,-56337,-36653,34474,49218,44014,15213,-8793,32434,26318,-77413,26919,-72581,-33040,-28660,52995,27077,-43001,49414,8364,27102,49767,70740,-12479,-34077,-28904,23765,9844,54617,-6926,-23154,7198,5932,64194,-55352,23046,24467,85427,-22761,1168,-6285,8373,7251,-90528,-30372,-13106,39269,-17322,-8102,102419,-50034,-29466,36875,-27633,-65279,-27809,-43273,8628,-19797,25128,-20156,39655,13609,-11862,-71624,-4229,37365,43210,16373,-63461,-2003,35235,4954,8632,-9991,2433,-2552,-20605,-43960,34648,24166,18349,-25308,-81397,-64148,-57752,-62128,22736,-12543,34900,838,43567,-5005,16142,-34474,-8575,-44189,-13537,53238,-54788,-60482,-71525,47353,-67411,-22329,48424,-15821,18514,-24077,36312,8217,44643,-12362,
    -21636,14539,-21281,-57391,83648,36318,13558,65899,74053,-85779,12742,-19664,1513,2522,30087,-10250,-11812,15807,52354,-45822,16874,-70719,12127,28485,-6996,50290,5517,-30560,-24391,-6218,11379,12743,-15253,42094,55169,-69294,-46325,-257,-10052,-5709,-39361,12787,27837,-13642,25767,66416,30381,6289,-13065,-14218,72452,29226,31120,11301,-60886,50834,69940,-651,33298,51354,-17458,-24170,46447,4866,54208,81022,14447,-31,58870,24621,40833,-42046,22773,65076,62286,17371,-37879,32716,-37717,586,8370,-11617,27666,-2025,-64209,-43515,13501,-9268,72788,91688,39985,81160,-57165,-37806,7657,-11611,37089,49707,1431,-21629,-31763,-27133,64621,77276,-4895,18489,3843,85150,57234,-79328,-44634,45617,-40663,-37089,32608,50120,-726,100831,16439,100615,-12297,-23915,47618,-2514,-18282,72761,-56426,-76692,
    -29733,42459,68805,72936,-128,-38581,5625,-36001,79328,-75251,-48063,49703,-46329,5268,3221,2190,-42191,-7117,53270,5725,18027,71794,56303,-9715,25089,-21499,-8415,-21547,-17639,29044,-32132,7783,-52331,73585,30971,-20729,4855,52000,-83172,-51725,20305,-17698,23438,-23707,29165,-54624,-28552,39651,896,-23902,-29499,-56694,33387,53635,-3267,58507,48429,-16822,34767,58898,-10882,-24357,-57516,-71816,25800,78323,-5173,47148,-37005,3679,-29746,-14051,-25560,8928,41543,-22665,58378,9674,1555,-29073,19038,-12855,26166,-6142,-38842,21868,-37843,9751,-12731,-8829,-58541,72570,-28570,-28975,25706,5709,30957,18454,26957,32434,-37013,-18929,21257,48165,30667,-5668,-67716,50518,31501,-56817,25461,-30639,-63376,55946,-33730,-63322,-52397,60044,28396,39269,-99631,-27309,21423,-52291,-38120,43154,16858,124895,
    -19155,-31901,-1604,-27673,-3847,-48434,18585,-23077,27384,-25836,-6721,50136,-70936,38,4116,-8509,104004,-24370,10879,-73848,-28004,-18813,20364,-36343,-7771,-52172,-30578,54192,3895,-52514,3200,-19541,-61948,67620,21621,203,36589,20596,20943,-75827,-30967,21303,-5078,65017,41165,29277,20490,33525,402,12999,-30285,-23030,31171,23796,-30493,-41244,27694,46268,8947,37623,35284,-38751,63613,18774,25176,18651,-32623,6061,-49343,-665,-4517,-53867,-35348,66870,16803,-66024,41616,83297,46239,-36257,-34568,39520,21343,-14541,-17895,-42876,17530,-52605,5954,6442,39075,82269,-14478,49482,67509,-19612,1840,45385,41495,63167,59979,23605,2261,41966,-22722,-6072,-59942,8788,-7304,-39846,31806,-31823,27415,11535,148787,-47894,17355,24033,-50063,-22122,-65602,-1834,22589,18435,-35186,-92825,36093,34512,
    19412,-25482,-59277,2575,16752,17596,-13251,-26698,-25822,-23664,-319,43526,83164,27256,66990,-45555,-10318,-66923,50499,-40564,-56878,47605,-36198,-47935,1012,23267,-10088,-30302,54098,22346,-15153,6282,-52128,25767,35722,-81952,-12976,-30892,-46993,-36022,-26592,-17616,70693,41401,-86483,27337,13884,16316,-32503,13523,-9243,-14473,-19017,6652,-41075,47061,-115580,-3020,-29344,83205,-51746,-28689,8549,-57168,-19827,26998,24848,34472,13997,-10678,12102,-35677,7194,43976,-17915,19681,-47062,9125,-57237,-15301,29046,-66266,34005,-31314,-12429,30016,-63437,34526,-68883,36503,78082,67370,-85093,-2385,36550,26410,51410,-11142,48958,17577,-3978,73261,31563,33453,-28470,-5982,-75442,-41888,10087,55543,2160,18308,-41235,72361,95138,40018,44366,33344,-25902,90970,-50009,-16033,99753,-11041,-47417,-50244,29466,-86188,
    45232,-12945,25774,-1249,139,-11101,-55065,41087,10501,30139,-30133,-40433,-70723,-19327,-24945,39136,31900,7555,-29386,-27057,-19145,59241,16146,-23346,56881,-21123,21197,-9457,89489,4754,6486,15405,-48509,9946,66061,-32172,48715,-80726,23827,-40990,17913,703,-65435,-19047,-77263,53115,-41194,-38588,247,-23027,15334,-12116,-30599,49512,29274,-41715,7526,122174,68900,4721,-25356,-10910,-65745,-10793,66773,-53599,-45436,-31992,-45426,-20542,-3771,11374,-4395,-46124,-29942,16806,54877,26536,-53963,-31043,-15155,-19757,7357,-50265,25000,-76832,-26795,-57455,40641,70973,-9694,42731,-2643,38347,25876,55654,-47163,-9591,-4535,4186,28551,3874,-41517,-2421,16931,-44916,-25694,11159,-61657,-37808,41615,5294,600,63532,-35810,54024,-34349,61877,44604,22225,70703,-99209,12340,24204,-27792,-7159,18926,111766,
    -34326,-2695,69934,-43892,63005,-25567,-18355,57616,-78785,13398,-11448,9403,-10865,29769,-28132,-42877,45690,-29471,-3295,77891,-2720,2247,19778,-30033,57634,-46674,-19442,-44345,47510,-75770,-3766,-1804,-23693,37314,-37850,32811,78879,-44981,-29307,-43205,-18271,8807,-17852,-47229,-578,-21989,15473,46398,39212,30517,18123,30357,87736,31900,574,-83419,-68409,69904,-56963,39736,-27549,44992,-3070,-7434,-56533,45367,2499,-14638,26327,2258,-62098,-28924,5450,12126,-40964,-27810,5209,-9726,-14477,-16139,-523,-12848,-51031,40437,75758,8363,63314,-15123,8788,-13216,-34222,-8305,-65182,3643,3097,11355,13188,17167,-71435,5360,8650,72936,-779,-27266,35796,-3190,-50243,37722,22220,21667,900,-35293,-5512,-24658,-2052,-25355,-36200,-7747,-8011,-55728,46515,-33487,-21720,66696,-6652,-9078,27665,63402,
    18903,-30723,-44897,-3569,-47282,25864,9428,-19864,-33822,-48505,26168,-14697,45768,25700,7405,7868,-21027,-40715,-51636,-60295,-63889,-57022,-21699,68053,-24919,72124,26013,-7362,-61153,7916,26654,17058,-39321,-75733,94594,-30326,-9685,55024,-49239,50056,-5440,-7546,-24779,-15781,27158,-34662,27261,41977,-104046,31567,95548,38963,-13735,4886,33718,49468,-9727,-62093,183,439,-46661,6225,-38140,4015,-31240,19478,30899,-31219,12783,-78025,58412,66327,-5462,32681,-6535,-39207,22568,29780,72522,-33883,35232,9184,26390,-63107,1940,-42274,27259,-44283,-24069,27092,47365,-16294,-4931,49145,44421,15594,-62781,44204,24194,-54245,45514,-19769,-36359,-7372,-10195,-1237,-74240,-23477,22207,-26531,58102,60133,38224,-16992,63126,11859,-30498,17968,41657,-66721,38474,-9391,49461,-13949,-26377,-16385,-6267,-77578,
    -22074,17370,-12802,27407,-46190,-28841,20284,3678,-47069,25488,12659,-40069,24164,-16124,-54914,-13653,-18070,-54029,-76948,19894,-13561,57384,5913,37936,21650,-38499,6869,61938,19596,-2905,30814,-38005,-30215,-8676,1102,19384,-2078,-527,36924,-23081,5981,-56400,4069,-28729,56924,30478,24312,-16715,-21315,-22032,-10630,16342,28463,-31020,-14430,-30863,38677,53295,-78204,74120,20425,19622,9850,33438,-10589,13351,4763,61938,8461,17913,-76641,-33497,30612,-6185,5926,-20248,-23345,-5059,-34299,-20192,-50909,-5731,-41374,-16444,3789,10893,59018,22285,-42807,-13352,9767,-27423,-24415,64663,-15657,-23866,34194,49037,5432,2607,18771,11738,9046,39880,27883,59333,-25189,14795,28722,67543,-23650,49931,20918,36808,-21210,-101189,39820,-61252,24604,-80834,-9034,-30918,-59934,-624,-79218,-28087,11754,-19065,
    32107,-66289,16113,46163,-27221,1866,18845,-21004,-7427,8468,93797,30872,-54597,10833,-13568,-7058,81220,32292,-49324,-42822,-46120,-9382,21293,-4381,-33037,31436,-92575,17034,7012,-23049,-3125,-7501,-6109,32516,-18967,-51429,38606,704,79681,26097,41295,-25855,40714,26679,-22297,74678,63740,-20610,-24918,22005,14389,-35306,10300,72284,-6784,-46909,38860,5929,38632,-65784,-28373,-37189,28642,15078,37126,-106686,-15099,38187,11767,-38372,-2111,-48261,19751,4424,-31935,162,-43393,64032,76210,12577,2963,22889,-870,-81322,-83658,-51444,46423,12403,3714,-77326,7038,4741,40803,55151,-24655,-27956,-36625,-21285,14399,23439,63426,419,-26037,43079,-31227,-55213,82,9968,-1012,-24411,-42608,6764,-21586,8250,32291,29824,-66053,34959,-34355,33018,-39694,-79608,40287,-43367,-64941,24078,-47568,-3905,
    75557,-36825,-49635,65396,-38477,7678,-40334,-13011,-47797,37841,-6941,-18970,46995,-7961,1121,23046,-12395,73927,-6605,-17645,17244,-38523,17937,298,-11271,36010,-5414,74904,-35229,25590,-21313,15200,-39266,-38893,-18550,40086,-14270,31523,30506,-106064,69769,-31363,-74160,54357,-54896,36404,-96859,-9221,-18799,51154,12266,-21358,4141,-57460,-96611,38856,-26225,-74886,-54525,-54593,2529,-55815,-91955,34416,47640,-92962,29857,19801,37675,-15357,-56299,-5980,-11295,-41755,49635,-74526,-53399,20837,34731,-55552,-55845,-7391,-76054,106509,27846,-46863,73101,57872,94586,32373,-98358,54891,11811,90253,7844,-45228,16347,66320,-32619,63109,-58115,-16521,-9065,16628,-30755,50495,-12345,20501,-6520,16016,9059,12870,-20920,30868,8307,-30009,-2761,79250,-18005,1344,21498,8290,-1417,-18251,-8240,5883,-58402,-42099,
    24810,73405,-35515,-1431,-31093,-16753,65329,-58697,-15583,26269,21225,16377,48505,-17445,12108,82,5929,19075,-36397,-10671,-52204,-45628,21727,77376,93336,95648,27120,80114,-44069,3102,9337,-6841,56344,-13983,-17715,90378,-47027,-13193,19459,27511,49681,-38363,3391,-28191,13897,78834,-57243,-13883,-10884,-39452,-224,43392,21027,-30320,-24242,-215,-34010,-46138,-11193,12145,-13982,-15087,-49669,3140,6103,-14705,2715,61388,-43857,82142,-47091,22827,15137,14257,9249,-4429,9025,-89291,5393,43130,-1659,20812,25410,-20131,-35746,47114,-12405,-10369,-12110,-59428,-30562,-23908,58074,-37872,26893,-48749,-15033,22622,91071,8035,-36187,-39210,-53996,36447,24375,2333,-12823,-12127,-34414,-5394,25819,61295,42914,92099,25976,-14094,28260,25370,24631,53077,-31909,-21363,8467,20924,-6191,31058,-13294,9965,
    390,-17972,22328,45057,31345,-49974,-42284,61937,9769,-54872,59203,6926,-17799,57586,33124,-25046,25724,39587,-68109,-26904,-672,-38799,-32602,-12435,-27193,29403,9142,-21443,-22808,-4792,37488,-48201,-30477,-1957,-28229,-57705,-11970,41607,-66571,-56512,-30093,-21419,-40389,25332,111576,-63286,-3740,-23847,65752,-940,105361,51164,-5464,18667,-48164,55770,-72872,10220,-87395,-42756,10532,-42998,37150,21647,-50160,11500,8386,17867,11868,20745,-6766,-5207,-46114,102127,-72776,-6944,9585,48309,11213,15840,8461,-59012,-18343,19519,42388,-2985,54558,-3161,-133224,-43908,79742,-64584,27391,-18607,19676,36482,20572,62875,41213,61056,46137,18117,-63401,25521,-2756,-56999,-39204,-66265,31078,7390,-29507,33619,29349,63395,42718,67051,-30972,-57057,-15273,894,-61206,-30393,56921,-69362,-53802,-14704,-59339,-71045,
    -20962,40713,-4226,-32497,17668,-4326,40845,38541,6465,28484,-9110,-41002,-6665,16267,-35233,-11717,44836,-21371,11973,-54644,-99241,50488,-38835,1490,65073,-81588,19057,-49396,19289,41649,-11748,59842,-21982,-10198,16112,-27261,89032,-6838,-8117,21738,30918,-75277,-11525,30116,7292,4651,23535,-62880,43748,-59272,37072,1120,46242,-154,-33227,33257,-9858,-11413,-23881,-17438,38126,24298,23893,67535,10027,-28694,-88790,10239,-11436,25562,51428,-37190,8985,224,-16231,-47394,50151,24319,-20884,-5650,-57902,32619,5375,72588,15420,-22562,103140,5928,-46743,-11299,-10245,19657,-1163,-1422,-32923,96031,-5162,-5902,-82405,20074,2593,-12241,47867,-40999,25618,31285,-3453,5745,-11940,16283,37745,-13517,11121,63137,34135,45588,-59341,-11893,33912,-28539,-6644,29304,5621,33576,32678,9222,-21880,29360,
    8526,42177,7771,-1885,68199,45487,-2304,-43591,56837,-29015,79128,-36984,29900,-10862,33832,74487,11220,4488,-26940,3414,-28577,21546,-65523,-55625,-10717,100374,46254,-20061,9748,32297,49893,28714,59946,-645,7665,-39390,-3339,-38390,72183,16873,-59877,5664,5242,-12758,30583,7704,53690,-6198,-19787,57648,24505,89799,11435,34281,10338,-21921,20087,-33055,12257,23346,35148,61267,2658,38510,-37862,-12370,-20231,-30750,27670,-6252,10037,-926,-3849,-14326,-16193,14287,1739,-5356,-69072,-2267,2854,-34031,41288,-11751,21873,26823,15961,34463,-58799,22572,-2990,36057,-43305,-18236,-8744,-16768,26043,31561,19373,15375,-56007,10115,-15756,13961,46958,-13073,12991,-42694,-24036,-7106,-22454,24052,-11092,-32320,12805,55563,71659,55521,29311,18511,10183,21330,-18260,3599,-13178,19965,-26757,-45228,
    70383,50254,-68902,36410,-17244,55287,-30843,-45571,13985,46984,75179,-5429,-15794,-18945,35982,-20078,-45593,-4586,10268,-11496,-30455,12910,34570,-52081,-12658,-7486,-38762,504,-27080,31000,-8104,-22438,32939,3383,-31466,55830,-18885,6125,-8164,-2782,-7961,-17236,19561,63898,-2896,-11873,-66310,-51405,24049,17983,-837,-49458,33617,-43253,-7647,19288,22476,26982,-4731,-22982,36314,24281,-3703,32357,71132,5214,-2557,-67370,-35322,80122,61687,1678,-20773,18166,32483,24107,-105,1948,-18485,53169,11630,21012,-54151,13683,34708,52235,98793,10309,-37128,-721,-88607,-27182,22987,9201,42898,-31372,8067,-66061,25948,1357,-103403,-29743,38610,-20937,-7250,61080,-36486,40531,4943,31998,1436,-42684,61814,-25498,-10122,-45007,-12719,-2064,-1193,-57297,-62767,7345,15423,-24301,-2049,35632,-29660,-611,
    39659,19516,31105,12853,-23658,59784,-31966,-12576,39058,35479,-71703,-18097,-11360,60010,-69613,-18121,-37038,46067,-21936,11127,-5303,14269,36674,-10631,70430,-1078,-24077,-39598,38179,-15349,-47009,-235,12555,39250,-2821,-37421,-102165,65904,-57299,59235,47815,-25756,-64525,-85001,-53010,38252,54697,32595,7181,30584,-38286,33326,-26473,29718,-59436,-12059,-36202,44601,13987,24534,31490,-49566,-20517,-44273,-22536,-60985,8861,-28940,-69822,-2343,39369,-61823,-21458,-21343,-12973,12455,-24753,44807,10441,26245,-16118,-7984,7925,20491,-13703,-92194,-2321,-2801,-6430,36831,8898,42505,62413,-24215,3358,-47643,-53277,18029,-28130,-1922,52612,-3829,-54429,-49645,-14362,-39731,-31841,-31360,-58740,23981,-40000,-21928,-48208,15219,-10960,9122,-30813,61095,43185,-9760,75161,-77816,-38885,-7040,46205,-28094,-44257,-34459,
    69960,48485,10622,32914,18567,50475,24187,-1081,-19054,27521,46970,-23997,-51026,-53654,12993,73979,2690,-18104,-33929,51385,-43593,37301,-63994,-43129,-66985,-59890,-11405,49371,-42001,-79475,21515,35413,-19091,53966,-43331,74067,-37453,-22910,46333,32669,-22887,-31474,-25233,24882,9782,-76985,-8590,55761,29107,-26735,64223,43080,-55321,17708,-1916,-17326,-23295,24873,-48674,-10086,60985,-94585,-56750,-4302,7447,-3960,23237,-26272,-21734,21192,-40071,10438,31437,-55251,-1291,14964,44961,-39969,-81194,11758,-44321,21484,-5666,40197,16882,52987,-60019,-29744,-44567,49636,14486,-40807,9252,29372,-32311,29871,-60529,5465,-29271,56705,-67097,26329,42457,-15709,38984,9816,15014,56054,18213,41214,-19831,2268,-42783,4977,-89144,-19205,15655,-4631,-25728,-54920,-1604,-29606,-58356,48617,-36541,-8582,38092,22330,
    62910,11002,12009,84094,-106555,5689,-4077,-18325,-35700,2384,6032,-33282,-69949,44422,36425,-15215,-5061,38239,-56244,25490,43105,8592,-54232,-13148,2190,2709,-44855,49356,40271,962,-41070,18346,74010,43093,26511,-2799,13818,6254,17155,73779,51089,-55852,6955,14422,-48807,25187,33334,-29307,51088,13173,-23036,11309,-3608,22580,8724,31706,35300,48757,-35769,-6352,-15075,-69016,3608,-42753,-1314,-55304,79992,12889,-3704,-9541,-50846,15327,18667,32164,-54307,37288,-3179,-73814,33981,56608,67239,-32837,-1256,-12153,-1455,-91474,56686,19409,21463,-15259,-36330,-13321,90270,9712,-40494,19980,4395,-84590,-48101,-16430,13519,-15437,34552,-59289,13872,-38353,15511,71648,-5806,-49233,46054,-18320,-844,-19455,-58666,-24809,35342,8719,17691,47095,-974,-91704,-95727,-43483,-7526,19745,-28867,-9618,
    10888,5592,26814,18350,14352,96274,43571,14359,-13038,785,-50333,-8247,101038,1976,-22493,-15176,-61191,-80580,50890,23078,20787,-26077,-70259,26992,2907,84961,-5091,-41661,36309,126893,-66554,-6775,-5384,-61229,-7612,-27191,-21256,76079,-15624,-5868,-14355,-12878,8985,-40043,-32285,5016,-71978,31971,31190,-5322,61957,31453,-14883,36417,-23708,123635,-6514,-76949,-97283,-60440,36340,-23948,8412,6784,-31793,-13370,-43385,2825,37750,-1509,-39826,24728,-21414,51184,38636,-2921,27081,13725,-20393,65712,19375,52657,40092,-15766,2320,-701,38159,-41663,17619,-18040,-21444,37316,29976,8888,6133,-6057,18716,-37178,-5662,-66604,-27270,38563,15716,23485,64990,-47039,583,9398,-4284,4253,43815,39595,-44166,-20062,37386,26093,65757,-11548,19250,37921,16066,-77939,78081,-48577,-9284,-20125,-47687,-54781,
    -6229,-60502,-20790,-54278,-316,-1329,42884,-37320,34806,10773,-65702,13323,-109,-22274,-37638,-64875,-15100,5403,-15509,1027,3086,18847,-21848,-18529,42848,-42987,23757,-19535,11153,68873,26700,-14643,-40290,-4079,9538,-5441,-7379,39700,25774,-64051,33700,10556,-11599,69778,10890,-14440,-366,-35378,-2055,13381,-121916,-38706,56835,-420,-1186,-37151,-44201,-50714,15416,28228,49389,-3335,29869,35222,-25352,18967,6523,23651,-3075,-3345,41687,-38751,29442,-45844,59930,-40694,-53378,-59716,-22293,-30996,-7701,68606,-60169,52015,4964,-12060,5008,52902,-6239,10053,-6877,3271,-13680,-11586,22978,6954,39690,28118,51473,81079,-89168,-7006,-88343,-5073,-59858,52210,-34759,-58010,5832,73943,48817,27138,-44344,-2909,39172,43243,2614,-23257,34699,-8360,-26495,72684,-14046,33127,42626,-2040,-21640,-48221,
    -4980,9816,-6040,-35868,-7250,-2377,-15027,22756,-18919,41856,30337,20978,6126,-82066,27319,-42131,-52182,36807,-45016,19298,13311,45580,-22507,74129,-8422,-27013,71623,21158,-74050,12985,83008,-19421,2376,-49098,-55150,40128,53871,24964,3578,-27060,-33450,57224,-27566,31102,21540,-43268,23283,-19901,-5675,-15465,38118,-8823,58081,45535,33226,21152,-58347,-15583,34836,21429,25077,31800,34428,95511,-68552,87279,-15092,46642,-26878,33493,25061,-20216,-12018,47642,-14463,6968,11841,-6601,51050,-90085,-4764,-57281,61524,-5470,15505,24764,-99279,-61192,-25475,47348,-16196,-26444,-68824,16877,-24017,44527,6624,-19023,-42166,63718,5833,-47524,-17815,6833,-12683,38061,47405,-52698,13422,22131,-24817,-66816,51974,16584,-90649,-14391,24132,26632,-33462,-3630,44506,13599,24342,15673,31141,-26585,22808,46500,
    24073,18620,-8290,24158,-47618,-15831,126989,-94330,-45100,33543,-21912,59529,-23228,-56599,23600,21792,-37042,-25810,17832,-18946,47519,-71411,-26678,5497,-7956,48911,-19375,-5755,-69292,-4550,13140,752,-54870,-6256,-4214,44366,42894,-8331,82272,-33588,5152,-10951,-79892,-28732,-48935,-3444,-11923,38661,-32510,-28772,-28712,-23206,37360,-36532,106856,-48604,-9454,103998,27371,-43435,1786,20647,-4494,-29883,-24158,28359,623,-65969,50136,-7148,8425,2079,23886,-34473,22037,48037,26278,-52866,-28472,-23589,-17072,-10308,-453,-42528,-6195,14953,-49585,23223,50840,11809,71225,51805,-3062,80207,50749,-20803,29987,-24763,61221,54752,43924,-60372,16516,19042,14494,-74699,-25643,-33525,-24978,-22500,31961,43163,35962,-2925,19542,21560,7049,4671,-26769,-3871,-62813,35740,17562,41267,26073,59944,43253,-18201,
    -21898,38574,32353,28499,29564,-11663,62191,-5181,14871,-38937,-37853,26249,-12681,-73878,-68782,60358,16750,-885,-39311,-15486,-19823,-35703,15950,27069,14091,31995,21225,-6753,-7940,44285,-32617,-8381,-55399,1728,-48604,93123,-21185,117679,-9233,-29576,9485,71543,-1723,-45784,-17835,-9206,-13044,70608,-22579,-20971,3389,-4754,24304,1414,-48441,63003,-11690,-9900,38707,5217,-37789,56332,4864,-14291,9906,67742,27229,26316,9553,57460,84025,55807,-29359,-20354,29084,-20111,30068,-1699,-55008,36747,-41467,-19185,12719,-37669,-3160,85363,-31020,-82615,-31235,-8598,-2490,-35320,-37164,-80250,-19284,-25647,-43262,-57045,33879,-12725,5510,-32154,1821,-18010,15083,-42274,-41335,73064,66138,-25087,-56821,-20639,80578,8670,-33816,10301,39666,-10184,12225,-19080,9478,10722,8323,23467,-89357,14553,6877,61449,
    18523,28779,-20311,-20702,-44293,24172,-13479,-33651,-83460,39243,41989,-13710,4786,69586,5756,-44053,-8203,-26832,-47181,-37446,-20675,22108,-43206,37596,-16924,-20105,6403,-27770,1700,69906,15860,32173,-14404,-22080,56079,14647,-87026,-75743,-73323,35432,10846,-9668,7176,-4606,-17119,-16404,15725,-4175,-18087,71263,29609,57012,-37285,-2203,8526,61563,31056,5293,-16867,7113,-28374,67493,-2928,-82500,-21715,-14740,67018,-61361,-19299,-3456,66460,19629,41853,49049,3767,6456,-54850,11311,32165,-37419,-9912,-11668,47431,62317,15150,18224,4752,45600,-91506,34601,-12977,-30055,-18695,16572,-90149,90891,14213,-55340,51503,-21409,-755,-13334,87975,60684,25290,607,-11824,19252,78398,-45536,6790,-137154,30773,31899,61684,-11178,-45355,24795,13032,-44634,20942,-50983,40029,-29670,-36483,61985,-17942,-66126,
    -29269,40357,75119,-29673,62523,-56511,24084,135404,71172,8559,37839,-27336,-30367,23857,4206,10686,12106,6526,-11070,4320,-71288,-31699,11575,-3298,43370,-13698,58609,24523,-5861,-41565,18638,-17620,31828,-66936,-46156,-15516,-32840,-26564,-90607,15601,-4779,28714,-58708,-58012,50776,41414,53652,-38864,-4198,12064,25563,8639,24302,-22423,36746,-102413,57358,43942,46474,18583,-32390,-9958,22574,-2394,-48521,4728,-50430,-20551,-23730,20988,-3641,-15511,-35354,9340,-27999,-16404,30664,24683,7483,33850,-45902,-44302,-54675,-14865,-33231,-8432,74760,52563,41056,-61053,9489,26930,-51176,-9751,7275,-13054,34959,81285,-19418,-57021,62131,57721,-4154,-6151,52945,-2432,34308,-12339,25708,-38348,-23582,-35609,-68,-33641,-69076,685,-6474,-19008,34480,-54564,-20857,-29451,-50741,37759,88087,36426,-8948,26631,
    25961,45772,32463,-33630,11515,-39267,22999,65837,-43278,-17467,-45904,-34839,24537,14555,-10291,73658,46416,-10811,-44516,-11467,12748,2986,-85391,15952,14626,-26889,28037,-24542,47434,-34055,-38129,-4629,-68686,1274,3050,-42701,-38776,-40365,8443,-23836,36178,-25819,-33078,-49265,67536,31699,57279,20781,-70274,25710,-9318,56907,3616,31565,8365,-7845,56311,16882,-12895,-19574,41367,52232,27878,13928,-62749,-3923,-35411,57772,43764,26395,18769,65056,-29635,-57494,-19263,-23392,41170,7735,-50008,-7188,28297,-97,-2893,-41016,39983,-25500,31602,30074,18030,72363,70493,-25985,-48548,56065,-11605,36689,-858,55291,6263,12047,47214,49409,-20408,-6068,33440,57210,6037,10435,-4390,27473,-7219,72846,-21086,15547,11985,15864,5730,-84041,25143,1894,8083,-57401,-13949,71934,-45855,-41,-5569,12632,
    49020,-86037,-15614,7759,-11574,-10681,81218,23069,-32753,87056,-35055,62663,8404,-36412,-7396,4433,6715,-19660,-1195,41666,34231,15100,84480,-25208,27323,46097,23464,-18522,34835,-21200,-7086,-5462,33708,57382,-25261,37403,62805,-40513,10036,-29701,22037,42287,-33385,-28534,43750,6923,35625,41025,63193,-5542,-545,-2860,44386,-13475,-21598,58853,-2413,45156,108722,-38168,98759,12904,55543,43656,-15605,-77127,-29715,45838,-41530,33217,-54869,-5177,60780,6485,1224,-23958,15707,-9778,31090,59141,31775,48481,-8631,54185,-41351,-9807,71096,-69417,-19542,39982,7453,40897,58415,-34949,62712,-59068,-31283,-7680,-47331,29545,28597,-8481,-32031,19894,15211,-27802,-31480,-37282,-101165,-26187,3581,-12612,31868,-13537,-26299,28384,70003,83476,-39250,-46328,22708,76501,-72866,67611,-10176,90135,-66260,12527,
    -39856,-10344,5017,6711,-5421,2545,120203,33572,-29941,48424,-88045,26981,39267,-3874,-18466,-69965,68995,36832,32188,-17169,78361,-10275,1465,45682,32390,-4039,-22315,-46944,52769,56779,5951,-63438,-55760,54142,17969,-9525,-26395,71543,40847,25931,65985,38133,-10449,-30240,-53710,74660,32920,-62875,47402,-73144,-86449,-37070,41921,10945,-45654,26517,-29408,73698,60802,2021,-12843,-29914,-41225,-25129,-36297,13763,-5471,61062,21523,57150,-19917,1309,15383,15443,74967,-7495,-22560,27006,26775,-12229,-3590,16137,-5575,-12497,-75354,-13869,49832,-21252,-15737,49722,-19500,-1368,37758,-17366,80232,-38520,89067,-42421,-56428,40722,16106,-50430,188,73243,12140,-19804,22025,-60108,-37888,-62538,-17771,34497,-24563,-34705,-8107,-57713,-5049,-20970,93495,-11572,-15662,-19963,-108055,-2918,63650,64554,-25247,-38572,
    -41917,18909,-14011,-262,-5205,15051,21565,9942,42037,-16733,-6513,45467,30134,34202,-18386,4261,-76429,86247,-2812,4972,38759,15089,82481,17322,43120,12005,11813,-24466,14717,5898,-37331,-24633,1626,4509,-9119,-9963,-81573,35407,-52845,-51921,-48260,16178,-19170,-31414,5235,42502,-3990,-34711,24126,-10359,-24792,-40428,24937,-7490,-8818,9080,1002,-21111,-49298,-13622,-51613,89027,-32509,-21996,6167,-26866,-14613,10638,-1243,6814,8580,-11121,-4731,28363,11101,20811,17400,66643,60424,43214,-33330,-37620,54788,4756,-70256,-24618,10439,-41569,-5507,-21986,-420,27353,36693,-15926,-21073,1175,-8525,-19930,-23945,-7363,5986,-39049,22539,1083,-68713,20808,63029,-5193,-4918,-57968,35829,-84992,22041,56776,-29433,-54931,-53393,57056,-11837,37365,11336,5331,-25958,-26492,13712,33011,-43770,76771,
    -46543,-6004,-15047,10213,26951,8862,-31121,-45252,-54806,-29874,-43467,56435,-23774,-4686,-9937,17926,58655,-22894,3163,7409,-44254,17406,2642,14999,55167,-10247,-30052,28623,45295,-53399,-67312,7488,27031,1756,7031,71993,14071,26987,-42788,-33592,10755,-6086,-74775,-3101,31,8109,-67224,12293,27933,32923,39469,-27953,3949,-4509,-45260,-38265,33371,-3352,-564,4337,-61007,51697,-3898,-22419,-14398,-60143,6590,27349,-39726,6853,36532,-84265,-1761,-50152,-49291,28181,17829,-23432,16465,-12010,-2013,-15291,-43151,21869,22260,-19502,13677,5271,29720,-23530,-30268,-69908,21297,-17345,-7144,-64231,-40644,-26981,-44008,20602,-8447,-67538,-9864,3008,-6613,28924,-17029,-2544,30642,7155,27686,-52569,4256,-51076,15217,-935,-31235,31618,-43932,347,-1852,-114366,76288,-17638,13387,1389,26097,43954,
    21667,-27922,-66598,-6853,-22438,20088,-30488,3704,27731,33109,-58833,-45242,-8438,59429,-30972,-45721,-42110,44798,6275,-19766,33775,-605,-27791,64174,-64471,10124,8722,32250,-30537,74531,4512,-1896,-45661,-21576,-2269,-3629,9839,31099,49865,128379,36855,-16497,-41897,33719,12172,-100700,51902,-35092,-30590,-29830,-6152,-63301,-60788,-8642,18942,84182,-25986,-75552,38715,-69239,-37506,-20393,-16588,81594,-60860,27182,3163,27307,100814,-20511,47336,62700,3551,-6585,17639,-51857,-52206,10912,-32541,-21125,52244,-32324,-34197,69282,14825,15350,-58175,-44162,-35346,54397,-60321,-35304,14121,76364,-43791,30253,6593,35789,-3838,15870,30052,-48856,-19834,-28027,-54947,31475,39069,-91577,-34969,65101,24931,488,-10780,27933,-60720,-21208,-39209,-54362,82724,-4106,-4734,54093,-15716,-58641,80246,2420,-23100,-38387,
    -54555,47853,31294,20357,-40291,2381,12700,-15399,19865,36531,23679,-66222,-30356,-2129,-571,-6607,63142,-2725,-18973,2082,-40391,-71845,-28519,43986,-49979,38235,-744,-24760,-30804,-10867,51239,-8394,-104525,24078,-20626,23719,-23268,-6851,60717,42228,-33767,16585,-25714,-2147,43114,45653,27169,-35571,-58658,12591,3661,11128,-52575,-25656,21878,3149,-11590,794,10330,19232,-32358,36095,-25907,34473,-73816,-20113,60376,10123,-19605,62448,9423,36651,-5543,-18561,-19652,-62996,-3137,28627,-9159,-50119,22671,-3740,-38366,74763,-4900,33288,21051,41313,-28694,21837,-7151,-8926,-2786,9813,35828,32380,-17082,70080,-34966,-14220,53822,45848,-37795,19226,-37762,-50257,9926,-96542,-32427,15830,16373,21388,-57110,-59917,-37818,32295,46862,10783,7171,-13548,-12221,-17000,-28169,54066,3984,22431,53525,-24113,
    2069,8141,-48746,73180,-33482,26945,-82026,9031,29177,49485,-4895,22554,-62247,-35908,59723,3132,-9761,40317,19401,-10314,44420,6407,60428,14129,-10709,32763,-15166,60397,-56056,16289,-28899,36806,40250,33930,-58277,-10550,-20326,-49873,-9832,-4672,5303,6118,377,45524,-2820,-19245,-11624,12912,5625,-8051,54310,56546,-13138,47563,-52670,3390,-37564,18666,-33357,-32789,30570,-68461,-54712,68052,20806,5721,-12,17545,-54706,-35028,8797,3189,18622,77707,-53769,42427,-46912,50044,1927,-15500,56828,8253,34259,29379,-30685,4189,9169,-15972,19068,41972,10019,36103,31420,1077,-4355,-71751,22109,16041,-41720,-35347,-72475,-39369,12809,84817,25003,22152,-16656,-21416,-18408,-22670,-36535,-28304,-81997,-38596,55315,-30861,15212,23020,-61213,-19904,24292,-7934,-343,-34896,7970,26300,-28376,1682,
    107775,47773,17262,-45395,-38128,16692,123833,14564,30224,58861,-58495,59402,60029,24372,22694,57047,3914,47459,46476,29344,-4673,-24365,-54193,3022,-1290,6810,7849,-30062,-36681,-23094,-34940,6659,-53589,-28636,-87142,-4409,-18840,88704,-27911,45830,2847,20680,-22923,36777,70953,71692,21180,-27808,-51654,-37868,34052,33300,77859,-32787,45679,20433,-46766,-16000,56933,-36061,67701,47409,-46500,-58634,-7361,20168,6815,32134,-7140,12285,-4512,85001,70010,-22926,-28792,-7251,39199,60894,4981,-33768,8648,-4090,-29141,41883,-47658,30360,-11095,-29478,-59345,22642,30941,-17380,50003,-49174,-14297,-3865,78897,-41202,3048,-40239,8965,20643,-68974,-16329,-25181,-38366,58804,-23392,-32022,-52067,-21648,16219,6022,76653,-44708,25517,33203,4398,79444,-12201,-7030,16451,14728,31888,24181,47979,26872,61189,
    68815,-44397,-79695,57861,73196,-53338,-105323,-8453,6232,-65473,7836,6106,-19415,16721,60817,-8090,-20589,28050,-6875,-24490,18623,53259,77055,-23669,-9512,-52181,44909,-15325,-43834,55695,9105,-13104,91923,-2006,47066,-31968,-53788,75398,-21101,-45156,75800,-2250,824,-44801,-11977,-30833,-10235,-25,72559,3342,36453,-24016,14433,1126,-42954,20846,-42645,23272,-37175,1429,-30704,-102345,-529,-49557,17170,-7735,2280,1452,59572,17201,-58585,-27784,-10045,-49099,-7872,-7070,34762,-29544,-3491,-9339,-36780,36147,17035,37943,-44317,-31086,2082,4765,33393,34680,-89313,15910,131090,26978,28261,46439,10838,-35524,-16309,38667,35084,-23153,20734,15680,-9921,40344,68678,65085,18047,44569,-69415,-82109,-17563,104349,-23890,-7185,-98565,5607,-32475,15458,24501,22521,-4617,17805,37832,21095,14453,-7061,
    -25338,17685,-9062,30014,-58685,-85475,20020,41738,-34950,22063,-7426,31243,-28814,96533,46765,-43902,30406,-39822,-2167,41295,-57999,26546,-26589,17363,58664,-19637,-28277,6451,4362,34385,-11232,10734,-49040,47232,-15871,3454,-23928,-23693,61174,35464,45450,-43624,69101,-54939,4968,-51554,5126,4259,22907,12409,699,5146,51227,-10135,31180,-4384,11576,85546,-45624,-42557,7486,-42731,-73902,-2515,24361,-46816,-29660,-17502,-97926,7461,-76030,-8051,6574,-84248,50035,7951,2129,-17490,-17854,37719,27628,-63315,10262,4292,-17411,-5668,-5331,-37587,-2104,-10658,56917,65783,32315,3984,6882,64264,4337,4867,-22945,53861,5947,-17748,28786,29904,21927,-9438,-50987,82857,-24735,14954,42142,-34332,53789,13368,-65578,-69058,-16063,-41029,-33320,13925,1207,-17644,-68089,36358,-21625,-23801,43382,-6356,
    17985,16819,18393,-39574,23084,-3107,27668,15527,76523,15321,3672,-2499,12095,-8330,10143,3914,-40564,35547,63266,46048,-73049,26176,30479,22565,12395,41252,-14887,24701,-2659,39438,-3426,4301,2655,-18465,56076,-38354,-13517,-4568,-38450,-20704,44483,285,21978,-835,-39965,44446,-20990,-61079,58752,-52116,-52883,-14931,-20833,51639,-86673,-30495,-10268,-15067,-51524,24243,-2163,42268,-93969,46106,48475,28839,-35105,-19380,29787,78415,18826,44930,3503,-31199,-34035,43752,45230,-38927,31012,-91360,-62613,11232,-31107,23412,-47108,38534,-13623,-24231,-28634,63150,-2601,-40627,49274,-43488,9751,-10334,48393,24749,39420,41689,-71117,67428,-4968,26669,13421,-6683,8535,-9811,-42944,15512,-41481,-15149,-70719,9890,42620,-17728,-18518,35833,42552,43188,13041,-33776,-20544,71186,-46669,16294,3403,-2485,
    40355,-22734,-2952,52085,-15587,42021,-38529,-49970,-15202,-28237,-9357,-1488,-65873,11520,-20547,-39026,-56864,-26000,43799,-8134,-36241,126473,-37708,18034,14592,14523,-14982,45890,-68738,37791,18397,83294,8109,31672,81515,-23450,-58481,-31800,-24258,-31467,53396,-40920,39731,-8542,-1491,17973,-7428,11620,-67341,-35439,57399,2454,35483,-27596,-42189,83365,-12054,20880,-55748,3256,26030,37037,-7606,-14444,10330,-42137,43215,68470,-44445,-25532,34432,-58878,13413,-12488,37650,-26155,-19180,-13913,-56858,-67853,-1004,6877,25340,-73762,-86999,22708,-6952,24181,27987,88721,5624,32594,-44999,12009,-1125,-34998,7185,35997,-14928,-24015,-21974,44752,-8877,-18084,52440,21629,-77966,33229,-33544,9084,94006,60706,-36215,3360,26316,55106,31427,25361,-64426,-1449,-21647,-5011,-25456,23656,-37249,-31603,-62118,-28982,
    -15239,31929,-7454,-76568,-19867,67675,-6011,42171,-33660,-47465,11592,-14291,-1197,-39715,-9477,5720,27344,-52084,20348,-64422,-27325,12077,-28730,31579,4516,87423,18380,-30774,-3239,974,-15291,-32232,-26708,-2724,-34211,-9419,-4076,86637,-34649,-9933,22286,-50102,8657,-5441,-36741,61470,-14109,-27643,-28006,-4633,166,11193,19088,57642,14243,8415,-1248,1132,-53311,-16830,12293,10727,81320,-49699,-24704,-18779,14222,57627,75185,-16281,27338,6444,-20917,10603,27213,-17264,4672,22889,-20705,-51824,-32240,35989,51405,-52786,-3833,59694,41832,12951,-532,8794,-57164,-50627,1604,22430,70912,-39651,-17630,-30206,42283,66245,13196,45748,-60338,-43207,-44067,-66850,-9648,-2814,84808,18383,-16335,38425,-41569,5966,-31501,-33664,-47654,90693,8924,38563,24229,-131255,54105,25688,-21825,-39406,82110,65815,
    -27152,-29442,-53522,74785,1746,57154,18407,-32218,-32045,-18997,-11208,-35712,-43963,-26605,10870,103735,29977,-9623,5141,-57705,-22299,-107456,49123,-45824,-5330,-58017,-9266,37402,-38285,-21483,-26045,-4496,3775,-1958,2845,32426,-5850,-27095,-48316,2682,-18818,49028,-24934,-16478,10397,35669,-44841,16576,7209,-40661,5492,-22502,-62478,-47580,-109280,19959,26304,31362,101616,6107,-27235,25177,1109,110507,-12935,-16358,-26676,42442,-3741,-11557,59330,-34325,-2406,28360,16580,-101069,-2382,24072,2531,-6827,-10826,39752,10832,86526,7301,13907,30479,2441,47287,-42931,53391,38460,-29608,-24451,46717,-37078,-37169,-6739,-40343,-40037,-33980,-33317,73023,47058,-39963,51998,-12145,16342,-315,37650,41191,31643,4769,-73170,49783,15895,120254,-12959,65185,-43806,-59006,60782,-38025,-9211,33627,19157,28945,16652,
    -64611,28334,-19326,16794,-43647,-8248,-28921,-32809,9303,36947,21287,-24271,-4466,-29884,-31875,41544,-1656,-18100,23591,14960,-13107,8783,-53784,-18530,-41507,-34488,52245,-29084,48457,12780,-17952,-17684,59702,54624,45799,-24299,22277,68538,-8810,-10251,-64502,-48006,8602,-2798,-727,576,14388,-792,26178,-7906,-42758,-54409,-9220,32852,-25425,-19634,-29427,38164,8365,89938,-3934,15000,30905,-32586,48936,80220,3519,22387,15524,-22292,10433,-2296,6902,68874,25966,37179,53560,-12783,-38357,71423,970,-24511,-27813,7323,93539,10110,-3774,35580,54400,-74479,-40680,-38947,3172,-24126,27078,8619,849,3016,-66891,-41041,-19388,21932,10442,19304,-27772,42129,-27631,55156,7504,56970,12218,61965,-41980,-74786,16818,938,23283,18725,37717,28339,4870,-17104,39567,-9297,-19471,-16490,40544,-27273,
    18034,-23284,28337,-17560,-67985,66136,-3283,-95160,-5538,-12473,-6337,-2890,-37122,-57750,-15313,15169,-33394,-35341,-108459,3129,40973,24557,-65474,65838,-36482,-88367,79212,29259,74430,21593,-20117,-13556,-70827,8954,17900,-40136,-54560,2104,52971,-57191,-12926,-32144,11834,65499,91402,6070,-8002,12122,-18406,-795,36862,20527,-15571,-18898,-14710,85813,37824,-58238,-774,57640,53766,-42659,80045,-4877,-46037,81652,23118,29680,15650,-28623,60096,-2778,-66942,3927,-50620,-65107,12983,-53553,23894,30164,-19308,45185,3780,-23641,32131,-52095,-14132,-56539,14300,35437,2298,12555,-4069,99382,-116450,69856,13771,-22992,26134,-2670,27504,10124,-19566,-9491,11548,53999,-29435,-3320,36359,48433,-29366,32008,-3899,-1251,13429,69979,20615,-51471,-12835,-6163,32244,-49230,-68977,-19963,3526,-11629,15675,2003,
    -34142,41475,65037,22079,-6949,-49837,10524,31011,-3738,2487,46458,-61004,9552,-34204,15389,68523,-24785,-7435,81777,99949,-29189,667,-1466,-28731,-1999,8171,12049,16780,-78630,-47534,53252,29970,-17937,-74132,-33545,-14221,-25315,51186,-24921,287,-45038,13627,60521,28306,118037,1972,28141,34085,32092,28263,22462,16613,36498,-14070,6761,13333,109535,-52869,1931,28759,-15805,46595,59132,-34904,-30896,35974,-25594,-15764,23748,44789,30522,-6577,1856,-4949,-27126,11525,10865,-19610,-36173,-5861,25099,-29689,2486,21152,-4082,35530,17898,51571,24296,-53669,-36602,-45494,-21613,-5265,25660,34170,-9413,46440,5100,7134,-66774,20297,-31625,-61033,-26355,43491,38778,17332,10384,154,-16249,-49379,-55789,4629,-103910,-40961,-18149,20279,31979,-2952,3195,2229,6322,8423,47843,13529,3757,9345,
    92299,-12702,20600,-12367,9782,-16097,-37343,-3145,-1884,-18487,-86027,20729,19124,41793,-6245,97239,-59079,-27698,7264,-43761,-9790,-22380,11125,-79279,95847,44746,-98187,38113,66245,-1289,-79625,-46417,21028,101363,-30008,-18629,-8465,-75572,-31341,-33793,25110,73680,-55976,-15168,-70028,30534,-42244,26949,26837,31593,8015,4733,20640,-42204,-10164,-32909,23445,108151,22201,52931,39705,33492,-15511,67409,-82745,-36478,-41442,-52505,-35715,-21181,74044,-19455,-6791,-42062,1003,67944,-20810,-138781,-76275,40772,79035,-55264,-21311,39614,92110,75467,35926,-53362,53638,-3306,8365,24123,-25636,6365,17253,33997,-48878,-8057,-6188,7055,80403,-26671,42059,-43724,-23884,24829,45006,-27102,6353,5563,-12638,-13249,72033,19659,7230,-4082,30293,-20583,33040,43797,39272,8589,46726,50196,-6710,-46193,22958,48106,
    33065,-60615,27250,-21215,-1030,-53024,-44522,-21119,-15212,-26399,-57801,-68902,-9113,18729,1224,-26353,22177,64878,-95590,-62039,25822,63680,-11836,4847,52735,-45911,10402,-104755,67758,38946,51924,5125,-76375,49706,52975,-45890,-20434,-46676,-5119,-16911,-41326,23531,-77667,19971,49661,69021,-22943,-14302,-23200,45546,-37999,14475,-13386,-26024,52837,-7808,40717,-8355,4567,95514,7939,35745,67006,-32689,-104045,-78390,-3306,-22206,-83713,-5618,-38677,-41886,3495,285,-15775,-20973,-14051,-1016,9173,-7332,8006,-8556,-27854,4797,76935,-90667,82454,-12437,30207,59351,-6996,78894,-43481,-44441,-53716,38138,-37046,-21340,-48976,-83751,51854,8017,-2870,-55902,-17981,123909,92015,42547,21005,-16869,-32776,35606,-6812,-12402,54020,24761,8100,-28844,53662,31608,29260,-44883,28507,-23711,6373,-67135,-50405,40016,
    -6144,-21476,-13474,-30641,4093,-69627,-31874,-14246,-60270,-52849,-44219,-18861,-13560,-69951,-9321,16519,73072,24668,-40330,-20475,13399,-974,-4357,-78439,39955,400,-5764,-32063,21715,-23429,-29071,-3079,-26264,-31398,-71895,11830,24146,-26999,-3181,-19965,-17070,11902,-63500,18562,7828,5025,-18095,6884,29053,75976,-45513,-45410,10485,9520,-4319,-5380,-21266,-35950,-13394,9192,48576,-25443,-39784,-68007,-14023,13643,35331,11737,7817,-42924,49600,-6220,45935,27477,32941,-493,-5032,10151,39208,19540,-5584,-50903,28336,-34648,-34300,-23224,-8136,-31885,81636,-2543,-8878,78011,23867,49596,43267,-40356,14844,-27908,15335,26333,35345,-75588,40583,-19782,-47392,49095,23606,75909,1259,-25505,-22307,55074,10893,22591,70267,-4996,11595,15051,21767,43423,-28371,23692,-7133,35216,15019,64446,52346,65873,
    -21237,-34133,-52203,2006,-2392,-23646,-16159,19086,17257,-48160,31335,23410,-11952,-22975,44095,-42066,-87482,26146,6241,-3894,-74890,-25594,64901,52608,-16294,398,17773,77704,-28285,-7483,-37339,48006,-22162,-48811,58157,-16216,-21889,-6698,-14842,-74860,-79282,-20038,4096,-36507,635,13650,46631,12523,-20154,-9859,-20703,-47178,-42449,4525,19093,-13492,24591,-3686,-15433,16478,-14880,113014,11136,106966,1306,-54044,37022,68607,44114,-7149,37678,-51330,-89589,27193,-10351,12824,9891,-15998,70036,106289,32719,-7866,-24308,-37766,21917,-101735,-17202,-36530,1267,-31423,49523,89575,20597,29479,-34120,-3086,1574,-17562,69667,-69529,176,53654,29648,2682,-14332,20757,37204,2793,71103,-10926,38884,20472,28183,-58563,-10586,20563,2249,6282,-19069,-28140,-47200,39745,30945,45012,-11865,6681,14375,11735,
    -5035,-30335,-41281,-10437,9208,-57386,7599,-26182,1747,-30822,-100628,-64830,18919,34265,-37356,-55026,-29455,31584,72825,-60497,-20859,-22183,65153,9178,-45642,2089,-14964,24483,-21229,-3062,-20789,12091,-74293,-35301,30,-38888,2837,-29168,-17875,6179,33534,38659,-1320,-10160,-30885,11021,23610,-15348,5966,42468,3626,-53163,-18075,37991,-13643,31447,95805,-41936,36894,49559,41806,5696,39874,9110,9457,-41967,-67866,-11386,38728,-19681,21133,15902,8423,-70692,19932,-49819,-34543,63160,10458,74930,21833,3749,588,-53188,-11396,35050,7857,-60101,65976,51360,29473,125075,-28968,-58309,-34547,-3405,-40002,-17973,37968,-33897,96105,4103,78710,12323,-256,27421,-22660,-34510,12896,-47157,3774,-48894,-38343,24435,38352,-12782,-43033,-5977,106860,-58467,13013,73270,81778,29262,29879,4793,26761,66402,
    -16309,-79141,17767,91692,-31999,38023,6594,15295,-40173,-27121,89325,56196,-25520,-14503,-44093,10122,43449,7446,-21302,20761,6904,1133,55832,87820,-17143,644,25695,17003,-47406,-42364,8602,30291,7609,18653,-27607,14053,-28591,46048,11181,-17502,19396,43767,37981,530,9082,-24933,-5512,37613,78769,3926,39770,34116,35649,-27546,-8656,26055,-663,64156,6402,-3528,-59951,-5247,16494,-25340,89077,-67593,41591,11904,-24147,-12155,-70499,-9400,-1035,-78017,498,15749,8143,10894,-34178,-34344,-20779,25743,30185,2469,-22547,10491,-16781,18627,16181,-31883,2309,-47805,-15871,-22402,29345,17160,-52536,35709,-22736,19846,32114,10185,-12637,-878,-67259,-20220,-25749,46377,-25077,1632,-22873,-46910,-30623,4574,-32831,-35246,-48000,-8422,18546,-50308,39473,-42869,-67447,16479,-72820,-2919,-18725,49471,
    -10218,-14316,-41695,23833,-18418,38570,56407,-9394,28,-38882,25504,-11402,54185,-79620,44490,-19581,-10424,46159,2364,38125,-32273,-13841,26908,49095,-45685,35212,60353,46565,59038,24428,-69666,-22878,-24991,-43931,-35946,-82165,16622,-9086,-41248,-38704,-47270,-12179,-46757,7759,-42029,31512,-45736,-53650,505,-67719,5583,-74033,69051,68407,-8763,58035,10803,-83372,54826,-46441,24097,-11605,-75864,113844,-10438,-1950,-10072,55102,-42652,-21283,-40565,-25594,79087,-6914,-1277,-64813,-16758,20376,68809,42384,14596,-34297,-26030,-32953,33721,29065,807,-19951,-75625,29250,-81219,-62088,-58011,1054,-65450,20318,56690,17727,8387,-12951,-43636,24022,69760,79978,-14179,43393,49835,50205,10779,22841,6651,25079,-47152,65233,-7738,33290,7967,52321,13263,18855,-60574,66953,23075,-78480,37192,49480,-22247,28588,
    29219,11696,-51165,11115,-50977,-57975,10716,20100,-5705,-63305,-5871,-27674,-25240,-1312,-6250,72711,42551,-38701,-3151,47788,-17523,-25665,17109,-36597,-30813,-1743,-1909,20195,-93724,-13389,-29923,-16952,-44816,4571,20898,-78778,60732,5938,10579,6585,-2746,42040,-27735,753,45908,-16210,-33758,14408,-13624,16013,50600,-38914,-14148,27738,32749,-31305,66297,-6087,32597,50576,53786,-32761,36164,49871,16580,61609,-18781,-58605,-999,12755,-39083,36020,6405,-39213,-26644,14978,28827,17311,-27457,117614,37255,41416,-27020,-24547,69287,1227,44328,-59990,45774,-75336,-27927,-45434,48334,-27755,-10725,33807,-26298,30336,22600,7521,59540,38701,54745,29497,27997,-6876,-22018,69232,56170,26188,7540,34373,-26266,-47956,-14977,19700,-4477,-43233,-2989,3115,33904,-66074,20609,60843,-24452,1856,58365,19680,
    33866,-43626,11952,33890,3819,-6465,36514,3778,-6425,-1696,46100,-17085,11513,42168,-46584,-13668,-6399,52931,41327,-82508,-64010,6384,11625,2622,-39260,8437,-31763,-48888,-34476,-38651,-38507,-7434,-19164,-20843,-28350,-40721,-36748,10060,22860,-13171,58143,15939,-98039,8952,21372,-7347,14233,-11104,2335,30695,8300,20921,26090,88771,-12372,-5971,21795,-55045,-2908,-19317,-32427,58058,-19423,-36553,-13945,-24812,52112,17376,40899,-34957,-35739,12684,-3937,-52545,-16417,66235,-105202,-39970,18225,39067,76565,10634,-43326,-11186,-31749,-19926,14390,-44956,43032,47557,78833,240,55823,16997,-25369,-5187,-8401,-4152,61243,50952,45778,-718,37569,-70736,21866,-59128,-35506,45329,14337,-62022,-30372,14571,-30765,-26573,18019,18502,-33006,58560,-10021,-21454,30673,-6585,96143,39109,-12968,43918,-56964,10577,
    -4757,-77824,-22462,-35657,23844,-71862,-76070,16053,-66600,-27168,-42818,-46109,-42337,-493,-25468,-22766,-13817,-23826,-18225,14001,34107,16574,53167,-37573,-18385,-8277,30501,44524,-26864,-70110,-84089,28760,-38155,-39812,7497,-22285,-14501,16766,17426,-24254,-47402,-20415,-7768,549,-21587,-46416,1386,6951,42102,36539,-25029,-32603,-50391,-37173,22678,-6734,2406,28536,-29689,-15671,-78757,-9610,28728,20105,-3099,-32451,51203,51473,18139,-18936,11181,-79020,-18813,41277,29893,-11822,151,500,12480,-13490,-5884,74973,-43286,-32181,29226,-92615,-22402,23743,15704,28021,-10043,-46260,-30197,-37234,-14432,-552,-15502,5132,46289,-20414,44463,43167,-81889,-24770,-14591,-17218,13150,-25285,25998,21072,3242,11830,-24310,8199,50507,32647,17699,604,-54009,3172,67931,4420,24931,-10655,-14423,520,6549,-100448,
    23693,9175,-33710,-20452,7438,10404,4638,-27056,1297,-14707,48138,-30375,58097,8714,-3555,-74709,-34226,-80672,-59999,-34266,61990,-51539,-1199,-20423,-30023,42055,10382,18606,-35785,91808,3864,-29503,12679,39246,-91751,106407,-30646,-605,-8870,3403,54801,-58843,57689,25483,-10832,-8543,35622,26249,-11817,26809,-21548,32813,-11623,-17332,-39845,97738,34886,-53384,3102,-45413,64653,-34692,131321,-8735,4869,19123,24111,-32053,-14123,-50909,-28384,61595,-66143,33031,76810,-24023,-8360,15474,-16872,-17567,-796,127280,18619,-4244,-5674,18001,34970,28338,-76735,-1274,2156,-9279,49319,-22130,-21128,-25504,-86405,-20206,91516,38344,-25091,-825,-8636,25151,106062,27835,-10851,17573,-4445,10924,-6654,-60483,27297,30086,7889,-7649,-4387,-54277,-59430,24807,-16007,36161,12353,-70082,-63054,40025,-25465,-57823,
    -16005,-24962,9278,-56840,-65936,-41647,-37678,-38307,22659,-40941,-84744,57537,-34932,-39705,-50493,12506,-71117,27178,6283,-31474,-40294,-22765,-52578,36389,-10187,-104906,-42805,-42442,-23506,-4224,-29837,13157,-21132,22315,23902,-28395,-20461,10190,-39810,-84955,-22471,39873,-82281,55180,47119,-21821,6629,63210,71148,-2974,34597,81361,-31988,-60328,7039,-37694,1272,52256,-25947,33812,58858,16680,48318,-17350,63691,-33819,-19263,18514,-51727,-14570,35949,-40814,-9439,-32575,46332,32410,7245,-14924,-40204,41443,-5669,27721,22434,83237,94533,-32771,-12801,-64894,49362,6759,86296,64447,2424,-72140,43032,57351,-30694,-15167,55427,-61297,46683,4764,9848,-84317,-44321,5818,96359,-9492,41771,7512,66992,-23305,12596,50652,19050,26026,-42250,-3482,19427,-30165,18707,-5941,67037,23163,-14960,-55729,61922,2510,
    19856,48588,43176,12566,-6570,-60019,-1992,-14311,78077,-47868,55310,11200,46159,39855,3717,610,-13768,6053,-32804,51067,-94475,-40058,-10450,-37725,81103,-31021,-3060,-14833,61425,5680,-18138,17458,-25728,22523,38554,35474,18811,19712,-43268,16635,-104606,-23576,-14991,-25672,26419,-9038,-28863,65031,21705,37472,-4283,-13084,21992,10637,-72761,-32761,-28317,-23330,-20445,74686,-35671,-18609,-28944,-36268,13942,54291,5772,-23476,-66287,31699,-35024,-61102,-10559,22859,-7175,-20407,75046,-18692,19490,27320,4090,-6357,-49888,14154,-19689,768,-71604,9297,-33122,-80284,-36506,48464,32177,-40134,50430,-16505,41060,-678,64466,3893,5489,422,24978,-13475,-15765,-13029,44754,-611,19417,57710,45045,64920,36645,7303,24094,-59949,-5247,38126,25990,-34462,-69473,-39919,20095,17257,75692,25593,26808,15348,
    -7401,11358,6048,26762,10445,-35028,18750,15561,-13643,-21455,91390,-27399,-76838,49591,-17498,-14392,-37292,21847,-7507,29702,-29642,7308,-93445,28726,-30181,54231,-11418,-8377,-52658,16532,19408,-924,-14172,-27412,-35192,-46768,-65142,-42466,35196,70818,-20266,-5790,24236,49667,70836,-27212,23080,-36316,9611,-21847,10520,-13411,6262,-11291,3890,7865,-30811,39213,-13580,-21388,-22644,51602,-22338,9136,14215,36629,30076,-38755,13986,-41937,-31611,-9151,54974,69140,-54477,25455,-30278,50725,11294,31836,-439,-37400,-18457,42252,-100828,-29489,-36978,20289,-22447,-17810,69477,-79659,-37906,-26362,-133569,-28106,19656,21981,-4234,-2858,65566,-26599,22107,74773,-28223,-38430,39832,3695,37592,-25130,-58186,9785,-5674,39736,-21892,21496,-9086,-52883,34457,-10518,-63376,41111,-25476,-27229,-62384,42154,11168,-51021,
    -69087,66397,4073,-97951,-9567,-16030,-74331,171,-2090,48799,56804,25985,-61726,-57247,30422,32064,-5417,-32998,12282,-11116,-42377,4492,24659,-26116,-19466,-21170,-23280,49072,-15317,6767,-24222,-46415,35777,-593,3264,53849,4001,-12557,-48925,9742,53682,10303,-1644,-9343,24198,-36462,-3981,-54289,42803,-32820,2374,9345,11425,5448,-3461,-30734,23324,57933,31502,-86101,-40027,12436,-49113,35692,102553,3411,-66684,19409,-15287,-87331,-171,22864,86767,-8110,11034,3981,-35170,11894,-18201,-4401,-51840,-5493,-81620,11161,63057,22578,-7073,-54638,-15714,-21466,9551,-83869,-31733,16089,-56209,-35764,26097,-59989,-5874,39896,13,-50662,-28894,-3952,-47030,-46192,28836,-9178,-68144,-18635,-46175,-29882,43954,42054,-9243,-24308,-122686,-2986,44766,-32335,8511,-18052,29099,-17437,-31779,31972,22084,-23276,
    -3843,58284,-15119,-21381,-15523,-17915,72890,77863,44800,-26785,-3131,-3490,34968,47981,-33281,-44961,-21631,-90244,43576,-22734,-25770,-78890,-45092,-12986,80329,123806,-26233,-91027,-18402,96568,-19073,-11331,9708,-86384,-76051,39722,34038,-34931,-110982,59905,21958,10656,21088,-2825,-53014,-14631,-7916,26209,9472,71180,-1523,31725,31864,20773,11785,16339,-20036,-37735,-6359,9099,58576,12019,40259,50664,6360,33144,-4227,-107367,-20390,15349,17554,61192,-103434,-3798,17441,-17036,5279,-64414,5633,-18547,-1291,9387,55942,15603,41967,-28083,-18195,6784,7939,-5771,10587,-7435,-8522,-61024,47383,-2019,27874,-9837,85797,-24979,54889,-31947,-29236,-45190,43918,-34027,-5589,-12530,-14442,36108,29199,-46186,40327,-7709,3947,7516,3034,-49958,22896,32565,3926,16402,40383,38413,-50291,14536,2994,-24905,
    26546,13848,-15940,-9725,-6050,-48683,38872,-51674,-36922,-54164,28210,-3574,48457,-8119,9041,-50666,2804,9067,16148,-30766,23878,-29946,11843,531,-26987,2760,20533,11560,-91352,-32628,-54311,-15318,-33397,-25652,-44858,-31097,-26667,80277,38341,-10579,-7156,19943,47074,29092,47902,-60085,-14920,13849,81664,-46903,45,16065,20974,-8539,99033,37072,-10340,-13702,-2908,-3353,-89700,-74484,-39781,-4826,17326,116312,76056,-23953,38447,-23946,-30444,-12054,-8105,66332,-2531,75420,3843,10888,4967,62158,24324,-7887,11391,5768,-26681,-19725,-48220,-84223,34316,2644,-13613,1461,64742,43526,-49972,34689,7505,5799,-61341,43508,86961,-100331,60802,12508,-11732,-16584,41583,51387,19748,-23595,-77402,-30662,54102,20710,-39141,44797,-79706,-13841,5142,-1218,-41158,19224,69110,23698,-32528,38505,23773,22519,
    -38902,37222,20737,16468,43297,15286,-28633,-54708,21883,27984,58799,11066,-43972,-85820,-19638,-350,-33624,-49854,21289,38988,-21016,-10757,57832,45323,-73731,-60149,-48549,42899,-36638,11301,33916,-3543,-387,3546,51250,-24135,-36173,57891,-34662,-98897,14309,-62027,-39696,-24277,-41080,-19957,2004,-2332,9809,-56787,23613,-1443,8821,-46007,-33543,23509,25682,-31047,-64621,11239,45168,2724,-62551,-12930,-39052,-18200,-1366,-29886,-17827,-30779,88601,-44792,34108,-2152,32792,-21039,-38209,-46295,-44397,15615,16194,-4406,-46885,-21644,57638,96146,-10842,12839,-44353,-28486,-22744,-22610,-16285,14086,44003,-10843,-53573,6476,70693,50247,-33808,43136,-18526,-27374,61727,-8139,-48490,10737,12380,89242,38818,44860,13654,-7693,6367,43717,-58718,-29785,-5127,-61714,-26178,71716,1111,14698,58047,-5953,22068,-5947,
    30319,-30245,96345,99135,-10195,13827,17800,28701,52341,-27005,21301,-18966,47419,9316,-18468,-32620,-58097,-52507,-36517,-26824,-31281,1737,-48401,52279,56664,25499,-49692,36174,22157,-8674,39171,64859,-97365,17106,43614,-21287,-7508,1909,-17253,71979,98470,-51692,32638,26004,18439,16628,46832,-27686,-33680,-23121,-45294,-22720,-71300,14308,13690,-6964,69392,-11934,-59586,9433,16377,-22069,58123,16662,42658,-15342,-29848,19647,31069,2794,-41877,-1592,-20969,-17110,97230,-30837,49142,46085,21964,-65271,-36112,40672,-14112,47705,2179,-34798,12212,97997,-72625,-6718,46856,36678,16928,-50397,50196,27194,-38656,-27898,62318,43741,38545,-37707,22789,-5908,47918,44990,-17958,-10637,28759,63363,-27296,1490,30386,8270,45676,-10696,-72823,-38240,68451,36823,-29743,-5755,70825,-28442,-13507,75131,-36069,-79682,
    -19849,44433,26035,-32452,-26888,-60051,-12855,-5489,-47582,-18833,78653,13605,-35314,-64611,22505,35605,9270,-24848,-39830,-12870,-33505,55814,23151,31538,-13994,-4592,-37617,-1933,-7155,-21366,-92518,-2930,-32350,20300,40847,-49088,5309,1601,42703,-14769,1392,-55444,7837,9578,-34994,-46777,-40396,-12489,86020,3292,-1575,70150,165,-27179,37561,24888,127271,24634,-57173,30635,-14001,-7399,-5656,4296,34098,42763,-61093,-84755,10438,-60831,23803,-28548,-34804,-530,-36967,49536,9138,-82140,-13816,31588,23794,-21989,7770,-124644,123344,6414,1012,-36815,14775,24524,14606,-7280,-44289,-35516,-11907,-6891,30138,31076,63317,19004,20562,-37464,33274,-83236,447,11887,-42616,23425,22191,41668,7815,63254,63750,-4083,91780,29741,-59471,48465,4798,40896,-2463,-102884,21738,-60164,-62914,-10721,32707,28513,
    34938,-13697,-1963,3825,2596,-50463,45503,14987,15739,5445,1307,-13215,28087,-29733,-10425,-14338,634,-17069,66849,133310,35162,7939,87,14784,-36033,64548,-16468,25746,3525,-10866,-16838,37109,18367,-21570,40383,-74649,12582,31549,-8105,28118,45593,-27570,22703,-8214,-24348,-6209,29049,-22800,-29339,10915,-13195,-30155,64109,47948,-7903,54411,24523,23979,-839,1407,-15279,-4797,-66224,36722,-30483,45491,-32934,35535,46649,43202,-39463,-6331,58198,67560,-8184,-3567,-58696,-44066,18165,23276,57741,-31918,-19481,27114,-40526,-71637,114917,8860,-48843,29297,-25862,-28265,-671,78535,11351,-11422,-10598,-8715,-34694,4109,-15097,-2925,1723,12271,12655,-50577,-23076,-13561,-7523,-42953,-45998,76685,936,9869,48412,-3184,-12071,-29727,17871,-52916,-13575,27253,-57952,6833,-24234,-72,-16591,-14543,
    55395,-14167,-54427,64589,-38461,-35619,41928,8672,-38911,5387,-26134,28488,-10289,12417,64118,-88334,-4892,22754,69829,27568,43922,69837,31728,5432,67895,-12106,11638,-48075,73643,63090,1269,-13014,45573,-558,-11123,-69231,3847,-59844,-10590,71939,-46597,-2874,30247,-45401,-13215,-5521,28887,33542,3140,-10261,-25024,-100162,16710,22313,27823,-6899,-59210,53890,-17042,34053,4950,68243,-65091,-69848,441,8319,7026,-32488,-23691,56872,-2021,-14340,44308,12994,-19403,43704,56156,-2663,43959,356,20644,-44410,-15098,28530,-28156,5281,-29845,62782,-17391,-2158,-20440,1664,17320,-10034,-30003,33896,82289,-111499,-25576,-64893,24549,7615,61781,-60883,10876,5644,18127,-11781,7982,18066,11637,-75997,9511,-31711,12576,-55895,3740,-70358,-10145,-2098,-24927,9780,-4771,60493,57424,29373,19946,-21657,
    -7626,7118,-18093,57589,-59496,-13423,-35321,-23436,36938,65787,107735,-11209,17332,-25455,66461,-10260,-29026,59908,-12453,60061,-23839,31927,-32139,35726,-43739,-30668,-6718,-2353,-48450,40883,-46747,4538,-30702,-26396,36918,24886,12069,37744,-3777,-11796,-27495,-21644,58181,60582,-81786,21580,2649,-48858,18033,41717,29753,-39160,-85685,65285,-14950,-62147,52402,21473,-95151,-77387,-27168,38456,37123,30447,18525,33809,-30374,18113,39360,42567,-60521,61491,13026,-7243,-24424,72949,-5910,-2142,68272,-13926,2182,-41032,-14098,-49330,-20438,-3357,25407,-27666,-87303,-18970,-38717,-55092,36294,2475,9159,13333,23626,-92852,-20941,45997,11096,43971,18331,4631,13900,17782,35181,21356,-30702,46685,14616,-8639,11185,-69791,-47303,-48397,-36879,-46855,-28960,-6840,15934,-46810,-50937,-2142,-19220,54862,-42740,-35,
    -28514,-33288,73152,-55220,24083,26530,-17732,13276,-12539,-4967,-10793,-15717,24514,76525,-6275,42577,-40560,-25561,34290,-17390,-48704,15071,-27064,8142,38975,-17972,45815,6981,20081,-65197,-4018,-13653,-54905,58733,-7263,14355,-5011,-87909,-6100,5899,6766,7178,-1391,-1714,3552,63540,11995,-22870,-126700,68474,-63661,-36870,-74819,-28300,30727,-25906,618,54484,4053,67994,27823,18845,12064,-24772,-43861,-22430,-38698,-25959,-17707,-78449,-12484,-44614,3364,-24088,46522,10127,-14187,-15823,-62865,30806,5737,492,1316,-24102,66372,-57826,6300,5501,50117,-28847,-11806,23796,-72497,42058,39524,82569,-19291,79177,-18339,-28444,7696,56067,15306,15711,-44507,44864,-932,-16211,44516,26610,6438,30445,-53491,-28213,42778,-43580,16597,9368,-2024,32661,50560,-40996,51522,-6362,-29858,-44463,41736,-704,
    -39117,15247,344,34113,-52591,3375,81221,-14176,-39102,-6649,9456,53803,24956,7093,29243,-8102,11409,-52264,57514,-10421,-48636,-20914,-58338,76595,40006,53660,-51866,14008,-32547,-48457,-38652,-32977,16745,-24835,-29176,4989,-39033,38940,-48049,14037,-22864,-21751,13822,1730,28151,-16397,-27489,74441,31956,-37849,-48017,31841,16964,-19404,-12580,-29534,-49113,-12804,-25179,8656,-61303,-37924,19883,49997,-35271,4027,4428,-18779,22539,23315,-43840,-27621,29592,-10514,1567,37033,-49686,-29410,30236,-36944,-66911,-78385,-35976,53004,-22475,1850,-16636,30741,-50721,30523,24479,-33823,-95199,-33943,-26093,8414,26042,27750,-84347,63652,20492,-7460,31667,28313,-45610,24490,-72155,-25515,17626,32519,-15540,-36025,13069,-35935,9137,-18785,30026,-8430,33057,32747,-53883,30768,24198,-18856,-24215,-34000,38099,-58875,
    -5838,18362,-8580,86800,-3856,40528,-8011,-20888,-29437,-17061,1798,22204,3105,-92850,10126,-7043,74444,86644,-25962,43934,22509,-5089,87211,-48147,-68869,-7240,-15519,33008,-20650,-48699,48297,-11050,-50889,-40595,9113,29178,20956,16976,-26960,-64270,-28976,31636,20500,-19162,60360,-25878,-41589,19787,3985,-94659,374,9803,-12571,-95797,-201,18402,-21973,-65674,-1791,5894,-7703,789,8207,2848,-15423,2571,17835,42075,-67833,3621,-25019,-17329,11503,33137,64703,-64818,32365,51162,-39302,42392,38352,8670,-84057,5672,-60716,40855,1943,4237,11674,-44646,49976,3913,50943,-36660,-5175,35833,-41380,36365,-13176,418,-125930,67432,-20079,-34852,-19213,79076,-85162,28271,-24876,5342,61831,20718,-90238,-61329,-1980,69955,24938,-4078,-7422,4274,-74296,47503,-32166,75280,1249,14762,58210,24697,
    76074,-22602,-20367,9840,46789,44954,12028,-30925,-85370,18669,2529,-24239,-19703,106358,58368,82885,-26934,28406,-2164,-23056,5468,6278,-72770,-60539,7086,-21892,49142,28444,6699,7304,40082,54008,-32298,-67933,75951,51899,49761,-1669,-13323,2651,-24550,-19914,-16791,29956,41800,-22054,-60482,-119364,-86925,4567,102477,20182,-25728,-48217,10038,26040,-19803,3774,-5084,2454,-5003,20036,-29942,37607,-65795,6578,36297,-11719,22757,371,47726,51660,-28917,-60704,-95376,-57501,38876,-26066,8034,-10905,40324,41409,16809,-42467,11002,-5061,30248,18,-13474,-44099,62108,3373,-4358,35739,13314,375,25123,-12959,-29824,-35867,-80089,-46399,-12593,10174,49276,-9518,-27199,-11187,2867,-31951,-6945,51684,20240,-31834,24957,23535,50054,5912,-44336,5629,-50374,11953,-24267,-8825,264,-18435,-16731,15925,
    100239,-80799,-17374,31407,15247,-32571,1962,-11627,-71649,36853,-6957,32822,-31589,67642,36355,-16900,-2151,98717,-50267,-2599,2318,89796,15458,-21236,-17993,-2015,23298,13596,79887,14967,2024,67456,-47818,-21846,-14168,20336,58497,8046,77533,-9045,67709,-74009,57340,36872,-88958,8632,-11269,-13094,-93592,10964,-13423,-88715,46673,-30109,-32607,10910,-22520,6248,-38817,-72703,-10466,15129,28564,-26549,-45660,-6908,100923,-20132,39854,37345,-22579,-8235,-74308,66987,-20393,-15126,-8159,69547,82527,-101631,10769,4821,-54327,-142150,-48530,26442,-21574,-25597,-30248,70607,11902,33929,18665,81372,-38420,-55123,30537,61182,-2620,12741,-65522,8188,-11352,21641,-48318,-39702,-35598,-33076,12069,6830,-20192,-12614,-35574,79711,-20533,41585,-2448,14138,-21353,6904,-29159,-14275,37854,4971,-14149,26371,-69243,1124,
    12212,19693,-13351,-19678,-24718,40323,-9884,-34013,25478,-8658,83441,-16577,-56675,-6650,14929,-54573,-48736,-25536,-94244,16083,19072,-67372,-78832,36544,-21802,70327,-23269,-61079,29776,33481,29432,-27414,6096,-18904,6689,5479,-48277,55032,53398,-35065,28544,-41460,-60539,-13175,-740,85741,17540,-84853,-73231,44935,4775,29238,28009,20705,63374,-224,7418,-69054,-32806,-9453,-108386,-1850,47362,-362,5042,40703,58068,38131,58159,38535,8909,-26381,-17699,-56210,-21519,-55546,-65449,-8507,31641,-23447,-19273,-60690,20349,-31989,-31028,-113595,-24153,-70730,-1777,-40450,1523,-36999,6847,63200,2178,38707,-39043,-40836,79366,-36460,45505,57186,-36672,6257,-24460,-47814,33546,348,-70409,5425,-19873,-30175,51116,91060,17920,-38449,-13149,-35694,-30975,-415,19463,-38672,-5812,-1863,25931,-74667,34831,-57255,
    35889,-35538,15410,-20192,37355,47674,29207,-38657,28479,8637,31625,-32087,-4385,6910,-14287,60441,-34523,-40149,48926,-10467,-18531,5181,9688,-4688,1821,-15989,53639,33971,-28045,-49526,-9018,62572,40750,3046,-30448,29029,-31500,42497,33770,-19814,-28748,-63759,-769,40146,94823,-4861,-12832,-9216,-52394,-53016,55131,5931,83982,-5773,27341,-31691,23569,-21165,63490,-26275,-17627,-16250,55707,68346,23891,-39884,-11954,4968,684,-45274,5226,27178,4054,-66293,-584,47601,90685,-27316,-11639,-9197,30247,43024,16603,10409,8950,1200,-2042,54677,-8075,-12004,-30238,-33011,-97151,-42422,-4345,-56940,1838,8149,-11363,-3741,-8638,-20376,-58775,-30456,-24741,24482,-921,-3582,-25449,-25654,-24476,-35596,27375,-2926,-50435,15655,-39545,-14694,-72207,-39113,-46620,66929,58298,-41082,59291,3160,25243,21085,
    59522,9244,-29875,-18689,14136,-4373,-58777,-37624,-18589,12257,45480,-62,-71425,-32215,66199,-25252,3377,42031,-61159,-27732,4544,-1344,-15899,-62292,-13794,-33155,-3699,35181,-6482,36496,50332,17063,49109,14433,861,110672,-63802,-11338,-76537,-97665,88782,3735,-44102,36505,15454,-20155,-38095,-57632,-14115,-2320,-38334,-27918,-5450,-82844,-87771,7652,-47425,72569,59490,47620,-4943,-3582,-50423,-100030,-43847,12575,54969,31257,-18059,69492,23275,4344,-2133,16844,-41656,-29345,-80881,-7357,-30119,-24208,-22230,47856,-41607,-16394,-28715,45703,-39021,-43150,-15815,-22962,29804,7261,47944,-50419,27631,-1464,-25247,-71290,100971,29810,-100207,-28036,-51411,12161,-18736,39153,9433,61413,24690,-41980,-12401,-24221,3290,34221,880,54391,30657,1911,43952,29122,-27100,62887,-48612,25297,14534,-25933,-19690,-63155,
    31092,22092,-10437,-55807,35652,-34830,-43300,42164,-14067,-80551,-802,61521,-2268,3344,35847,63773,15909,27928,10264,-82059,-80468,52088,1249,85924,32755,-43719,26569,549,35474,5508,-62124,47492,-19874,-15052,70404,-10260,-13225,-5203,12150,-13988,-19268,43897,4043,-23713,29611,8800,85878,23492,-4735,-17780,27556,25280,-26963,13284,-16105,6840,29961,4966,-26851,1245,64077,-2887,26353,16894,60711,-2999,-21539,16282,-22639,-51925,30498,-21262,-6137,-55163,-1527,-19226,7009,41620,8493,-33930,8774,20410,58474,-64491,-25970,-14517,6243,11921,1297,28615,40624,47807,-13684,-40377,-13249,19698,-42387,-17025,66416,-39450,-4496,-10885,-25400,-30840,-13307,-49306,22259,-9855,-29014,43104,-15303,-32381,-424,77781,-5601,25896,-48458,105472,-51029,53570,33592,17098,-48001,8982,46609,5193,24684,19340,
    -9089,-16194,8392,-78282,29997,45825,-6264,-16378,24676,-29098,100134,-12937,9690,-43792,15481,-61480,9865,-51294,-34171,-38920,-34111,13493,25306,45190,5318,-10040,34386,-18837,6343,-15543,37497,59275,-11937,82033,9880,-68211,-73930,41686,2530,-28765,6646,36573,21947,-12999,64460,28286,55234,16123,41709,21816,24309,6700,2859,-162,-19929,29563,53017,26440,-12757,64262,28685,3486,23339,35036,19886,49748,29840,-1641,-18484,19387,42718,-34742,1358,34307,-10018,-694,43007,42391,95180,3028,-17466,-18295,68493,32613,-75089,-2132,22648,-28680,-21343,-25774,52320,60823,52394,22833,-33505,24066,-48170,36016,-61303,-78016,54054,14395,32913,51158,-5463,18763,3559,41078,11023,7166,-20676,-47270,45034,6163,-7750,38150,-68980,13362,-22942,-261,9947,26597,-34348,-47258,-18438,-22522,-22834,-11687,
    -23752,31326,-32222,16984,-13674,61178,-32992,12478,-7800,16121,29006,23926,14526,-10097,19013,-10174,-46500,-19884,-43816,-6284,-4429,60192,23437,7205,-79187,9525,26485,-3524,-73906,-7348,70571,-54695,-5474,-27563,20262,1380,-6624,-52543,-16035,-40398,-65736,-78471,15081,73365,-67146,34129,-33611,8900,-119535,-55243,-20918,-26690,44367,-41570,-29151,41302,-1944,-15712,-33871,24611,23617,58219,-60760,13842,-1551,19958,-1915,-39950,10249,-78365,48445,-270,-25581,73684,47222,384,-75091,29563,35164,-67379,-6991,13213,67926,62824,-20571,29554,37824,14270,-17495,61328,-17717,30136,-58559,61622,-23979,-28545,68833,28531,-23514,-11359,-62323,-62114,77326,70038,61598,47005,49986,36655,38541,6138,29308,-20162,-70010,5749,-8894,-8998,-45204,27304,-52810,-16815,-17001,51542,73468,-31241,28840,25937,60517,37238,
    -15185,2949,54980,153074,-7809,-36471,35907,504,51727,-21622,-9180,-47807,36376,-43336,-23479,-8476,-2087,700,24806,-40021,13001,-71507,40504,4664,-77473,-4200,8203,67589,13002,6016,45161,-32151,-25602,-23380,4969,-41133,12220,-78742,39633,-7486,-81052,-50304,-10418,-11221,3747,62950,-11983,50013,34920,-6962,-79263,6470,-48173,20863,6632,-28980,39598,5807,739,44879,28827,38319,-24667,-32291,-29706,-38321,-27827,25255,21623,-19674,-71287,13793,-50157,14819,43894,41461,-15243,76867,-96645,-10304,-41961,-60945,11573,27628,38213,-15775,-31824,35533,17279,-35479,14632,77834,-12144,16748,-9472,58911,-5510,7935,64516,31349,-33054,86776,81170,59679,49845,43854,56657,-15355,34594,25004,-3316,31180,-53585,15353,32178,16228,28449,6149,14592,70954,-17149,63024,-60719,22707,-27718,39265,1842,3986,
    44825,19104,-23591,38036,-14815,1000,5014,-14265,-29030,45513,26038,-53858,-1620,-14053,16064,7587,2476,-18638,-41389,-3542,16202,7143,37540,-48428,-21161,-70336,8900,22340,27070,-78180,-28300,15064,-2980,-10173,-7538,45817,41710,-6287,35035,17174,-23801,-49281,-44538,15623,-10010,-37543,-54986,1533,-46129,-47127,-2276,-24650,39043,-15444,-5140,-10514,-23881,17928,-14163,80974,-59649,-75415,-22166,-2398,-13445,69337,2909,-50083,-61311,48182,29441,-6842,-36680,90732,-26228,8989,28968,25065,-24664,14314,86492,-3286,-13828,27970,3298,-21297,-2424,-53105,11896,-11673,-59669,-30785,-5233,121201,25198,2029,-50219,-84365,-38136,-14692,-5085,4786,11003,-68823,-15113,11697,-25422,41031,-26490,-76,-15757,-1524,-22461,20889,-57468,-29136,32660,-34594,57408,-10210,-19611,-28512,-29041,9083,12575,-19552,81125,51066,
    -50209,6877,3593,64956,21749,-3351,-67696,-33411,54971,-48664,559,9460,-75204,20262,13734,22919,27987,28293,-20992,-30449,54745,-27897,13206,-9128,-50909,-10840,-24812,-42237,-52730,56953,20960,-31941,40970,-5981,37652,-30878,57405,-51409,-3941,-17943,-21737,24461,18252,-36621,50875,-12363,11253,55873,47481,29296,12830,15704,-80173,-14943,16694,-18704,3969,20987,32265,-58839,13056,-50638,28033,-27162,11316,-2057,-66034,17527,-18551,-70590,-20455,21141,-23675,-4407,14837,10342,18689,14225,-3555,28481,-4154,-19637,4696,-1621,21327,10968,-13638,24895,-59056,-106193,50821,-7084,17418,-20980,40407,62027,-28269,-23721,-3094,-26965,-4292,35798,44194,4039,19203,-3480,36680,13805,74178,25650,61875,-37596,20248,16672,-59461,12913,1644,-80545,-11177,9191,-48911,-30354,-3788,-61288,-5016,6794,87028,20579,
    18850,74857,-69882,-21119,-11904,-5519,-43715,59178,-20128,-62182,-29878,-53368,-52619,-59938,50447,9095,37328,-1877,-14560,-22418,-56578,8284,66505,-5964,-34905,-23194,44179,39539,-13601,-12642,14764,1529,-50578,10732,2412,-8,63058,-38946,30938,-70204,-67512,-54210,30005,28724,1146,-19167,-29742,-403,-53398,-13684,3574,-77779,-23852,18974,-41403,-19417,20908,64524,39406,44160,33873,-28480,-38215,35867,-6983,35736,-21891,12597,-8569,59051,106064,-24094,10802,10706,-23266,-28213,8487,5473,31712,14797,-33720,-731,-19527,-30080,27455,71240,35096,-46900,-48404,9381,-10540,-6528,-75903,25052,-36754,-2987,-18366,-14545,-22852,53706,-1816,9957,-29901,40755,25039,-48922,25750,-36437,4824,13914,6291,5619,14893,38515,681,14528,17462,-21878,-10476,-17001,-31401,30978,-4023,59182,-100441,-53785,37977,27549,
    24298,6573,77823,56215,19031,5124,-13744,-27041,3619,-16026,106575,-23626,-50413,-47567,-6149,8325,-104687,-40667,14159,8515,-35071,32221,-53833,50551,10329,29950,55334,-42129,45470,27023,48074,25821,10150,17231,10609,-20669,-64392,-12748,-11359,77458,-9671,-63605,-22537,-18268,-15290,59740,39138,61466,-26043,-44271,11966,16605,-6863,64120,3043,27777,-26337,33257,-28319,11043,33218,59207,32591,-39116,53762,-9989,-6348,17066,10407,49172,29101,8551,-36239,-37016,63913,-33524,18039,-1913,-22659,-23929,49471,-3519,38361,-18661,-65863,-16289,23665,47915,6383,69231,-31532,-32655,-44100,-72392,24471,98479,-17239,-45572,56488,-87671,-4206,88162,41206,-39174,31275,11463,-53792,14479,12130,62389,39089,-12225,7211,38159,-11305,-41185,11106,50628,-51067,1540,14715,-26397,-43357,7578,31831,-44168,10979,23708,
    4951,44152,10419,21000,-134,16243,-37968,-89900,35815,-13027,2832,53612,42716,24956,-21710,58756,-28850,59456,-64607,-4703,20418,38862,-46008,-28817,-34216,-8282,-46327,34984,-235,17098,21555,-45531,46969,-69542,-1180,2700,50264,-82316,-24058,177,-16747,26750,-52706,19334,23290,2844,-8697,27979,-34865,17398,4801,36294,-36000,-38100,-6030,-8883,-4828,6017,-20176,-35170,51596,26575,34837,-48868,19455,30888,-67537,-30579,-5915,-38239,19417,-59257,56825,488,-65906,39985,-61401,102682,-14651,-12708,-7592,-43792,-11920,-20464,-92372,-10062,-2167,-45298,-6838,-1346,62564,-2912,-48616,56110,-37973,-28145,73236,-55415,12308,74007,17079,-43258,46918,-59649,-34826,-2641,-6394,2516,638,16503,26966,-37770,36654,19829,66806,2885,38295,-31971,5785,-13568,-41491,20980,23895,-26398,-12459,24902,49817,49654,
    2977,15455,-2389,-55210,-10147,-19825,42285,66462,34216,-10295,-20774,-10925,-18255,81764,-10370,-8176,-4246,-89303,76896,17269,-85396,-15340,-20163,-38808,-9484,3233,-17650,40898,-6591,35803,-34954,-47304,42546,-34131,60051,-3301,-32638,26828,-65656,48610,64562,-115636,-6838,71128,-65105,31222,57945,-422,-33629,-16745,2571,-4729,-53162,-104,-18501,-17447,44896,-30476,12987,-6828,54378,-53544,7909,23335,22812,34001,-46495,-24808,695,-24849,-44036,19915,-3448,64579,31079,-13279,-25544,15201,34360,-4224,-57527,43919,-33223,-11957,3016,-7947,7435,44551,-16845,-1766,37020,-6867,4379,1656,37520,37032,45373,39813,15268,49339,-25684,52691,5183,30562,54901,-12441,-40886,-27151,3166,46198,-2788,29907,-14967,9848,29851,-33013,3566,30441,22508,16813,-26894,-97209,44803,19035,5958,-63773,-30918,-29199,
    6438,-48272,-31389,-52117,64126,6776,-58710,25649,-30357,67220,-76830,-49101,-37225,-35224,963,11345,37237,8574,-27271,-79897,39788,96890,60809,-44456,-17783,-57420,83468,-58662,24778,31570,123739,3487,-39113,-35032,-10584,58504,32504,3745,76875,-26957,29071,30071,67129,11326,-3999,-50032,48381,-11962,-16906,-17345,-30147,-91686,-88105,-37344,28521,-2040,11186,43568,18856,-10961,-23202,-11158,33425,-15020,-27150,-70657,-3796,-25009,34456,-30017,58476,3735,2384,52423,6802,47780,12932,-452,-54568,-47400,-23736,-49339,34991,-38796,64935,67764,-71281,20051,-70103,-6889,-20910,3332,-40851,20095,-36028,-15395,-5301,10350,-3386,15890,-8834,-46891,-56071,17969,-49364,-13586,3026,-50791,20744,28796,-60441,-23484,-20991,48589,-56695,43446,-52898,-68091,-1093,-16322,7314,26536,41171,-51999,-9218,-9873,2567,-30903,
    18837,6458,22849,-63692,33066,59615,78548,18300,-4293,-29197,-7476,20244,56071,34208,-37281,69871,-26807,-47005,25832,-14331,72939,11498,-56246,-71583,-41470,-8764,43715,-46688,33438,-14248,-13972,23494,27257,-62486,-32472,20164,-2723,25733,-68434,8391,-45713,62700,50467,-51866,69239,-84875,25201,-26507,-5996,-4394,-23958,37465,-18217,78696,41927,-15113,2134,-13331,42943,-5672,33015,2075,41582,17630,-36860,14378,8799,-44346,77033,22501,28459,81834,-36562,-47846,-20926,-7048,28675,-18447,79828,-27050,21630,54487,75792,-45648,-11275,-18490,-14292,86380,-11284,18,3894,19396,-10719,-40113,13607,40041,41891,12572,53583,-24632,6746,-9890,29462,-53795,10065,-30408,-15400,39934,48719,-82568,8092,-37868,-15625,-63574,-10833,18324,18892,13688,5794,-38191,17637,29336,68165,26228,-25876,45989,67830,-20257,
    -30796,-10585,-39041,21251,19574,-70895,37917,86117,34609,37234,19991,21725,-9318,64196,17824,-34309,-13785,-97928,76816,35928,-38849,-21257,1496,-20768,33517,71207,9047,-30768,-37721,-58396,967,-31985,-14572,19967,-46961,17302,2515,63407,-40746,87599,-72548,-44157,76396,42084,-25428,9357,50731,24464,45112,-40848,23804,-90031,15030,-35533,2548,-24045,-10661,99161,-24389,28713,-1916,6428,-42141,-44431,25474,-7982,39315,-12140,42973,50373,-66423,-24108,-51379,41675,76495,-8978,67782,4031,24635,-32248,-37914,20892,16507,50012,-947,49675,2870,4390,35300,-27800,-59343,4753,35375,4787,64103,-48247,32622,-9743,-35488,31040,35533,-6858,6044,-30204,27014,-79531,89667,6255,25598,-17475,-2952,50522,5156,7519,25062,-120276,-29336,41956,-55760,16611,10065,1081,23588,44204,-8218,-8975,-27097,-35092,
    -42687,-1079,30665,-19072,-2483,-41924,-5687,-5599,12573,-43831,8104,28423,20610,-25726,15128,38073,-31223,31888,4155,-18356,23540,-6667,54081,-10003,27495,-17698,-28825,-60354,-25896,-910,-85229,-17728,30947,-45905,-25964,-57892,-32301,6171,12933,-33896,7465,20651,20406,-50671,3388,-4289,12574,87759,-18399,-16497,12191,19393,21700,13665,38115,39741,51107,-31719,-49786,19106,-27611,51647,-14895,22077,14510,33133,-54727,15789,-41237,-28159,16657,-6066,34641,-14922,-20259,25944,-19637,-34648,-9558,39618,59687,-2035,-20138,-103919,-70161,-11646,26826,-87871,-3276,76403,23063,-33344,12909,-47686,-17369,62030,-9812,30976,-23583,-56019,-32972,12612,50460,-42711,-40915,5800,23679,30251,34901,5538,28343,-81957,28583,17445,-49003,14550,20866,59848,-25778,2819,75400,7174,-23999,7769,7416,-18578,-67455,-28851,
    -2745,-28548,43880,31767,47396,-47525,2821,23687,-70458,-42827,36172,-28152,-73617,-42493,-59740,-3972,-24670,34472,45117,-27124,-27242,35947,-4351,33150,10251,97967,20001,7766,6112,2333,-12806,7575,9316,-52281,21820,-6584,10877,-32642,-57828,-53945,11468,24877,-90786,752,20976,-40336,-23540,14258,-1256,51014,-29021,-48971,2530,13802,24799,29241,-11808,-12140,26785,-8810,-11052,26005,-42035,-2150,294,-18017,-16412,88851,58548,10277,8418,-21609,-41446,-1805,45296,-61497,-48823,-29402,-6378,-7615,-95936,27568,-57923,-20021,30495,-6085,-57442,-46832,37304,60181,-73561,16140,-15023,13576,44802,6831,-44741,27138,91291,-6141,11144,-8762,-2049,-39402,41400,46489,19366,-24677,41533,8314,-69839,-45654,8874,64050,3397,55614,-61198,23528,-4526,-25158,-22102,-5788,59022,6080,-30138,77296,7087,24499,
    19061,27507,-2479,-22738,20277,65024,62177,18324,3916,10917,38159,-6017,-23434,24763,19536,66504,267,-45389,70939,39768,-44431,-57354,1666,22597,60587,4625,-39281,43394,-43465,-47,48242,31573,36750,-13622,-15563,-19171,-93183,6910,40956,5434,40513,-1955,-7725,-6057,43876,41013,-18067,-58202,10184,-52748,85824,48555,52375,-63470,-24724,19316,54312,-48874,33854,-35775,17723,-683,-74938,37077,-10435,-32440,-5739,-49694,-15609,-9708,28016,-32440,28049,-45533,8643,-23137,28745,1144,44676,-67029,10604,-6808,38421,5447,-66644,82680,-46607,8555,26217,-14927,-12723,42584,-13284,-26192,39294,-81744,-6670,117629,55700,59887,11889,11814,-11247,28205,18182,-26477,7852,13592,26057,-36426,-13642,22840,12740,-11147,55413,46502,46577,72468,-17161,-37472,8004,-49295,-24193,45794,119405,12643,26292,13855,
    -23528,46293,-76896,-9667,31549,-28124,-40356,53821,78340,15943,42948,-44873,-21105,-13928,4089,-6206,17208,28566,9050,-38727,20836,-16119,63335,-7684,-66513,43539,-8158,64326,-88627,-4853,-26097,23560,42429,4197,-27565,-23396,-41377,19451,-12884,-24568,-16506,23609,-20738,-17587,5842,-61878,-6356,-37354,96016,-6232,10754,-46380,63266,71837,3383,-50808,-897,-23241,25971,-83583,-33416,-58006,-25977,2909,50103,7718,-30163,-1227,44153,-109846,23897,43127,-12304,15343,-52134,-6267,7666,14996,22076,-78516,-10711,1495,-23988,4587,-940,76080,33655,32669,-43035,27534,-68316,-13940,36471,-4646,29087,-45466,41028,39417,26311,27460,13694,19962,-33456,13921,28503,12613,3073,-48300,23543,-22346,748,-42303,66,-14098,7255,46332,-98644,6198,-135,-73679,-12670,52346,9108,-17065,51636,26236,34120,2004,
    -38745,23938,22094,-19909,-9500,-13648,-44988,-49554,57234,19832,19096,37424,-45949,70894,-74576,32106,-11041,3978,-3377,-85038,-13122,-38340,-47646,18146,28666,22543,-63847,-37735,-50736,-27703,-65302,13006,35589,56943,-13530,15746,39404,25288,-7906,29325,-26422,-389,-19886,-62634,-10558,888,30292,49268,-8516,74620,29139,54393,62897,36249,-28572,-36864,37155,17073,-37270,-66072,29071,60932,26623,114798,-9920,5017,29678,-18399,-9910,-17684,33264,58365,-77049,-3961,3321,11999,8973,65776,24592,-47096,1275,-31352,-21422,-9204,33907,-44036,-22820,-19423,-51217,10508,-20763,-18563,-48559,-16131,-33988,-50591,16122,4070,-28464,-27377,12965,18226,19843,-23976,-62697,-78656,-21582,-26379,-33320,-22873,15397,-46320,12579,-3782,-8390,-75995,-47446,-22171,2836,-3512,585,-28111,-85861,-45800,-50270,-66103,-58040,16159,
    -18494,-72044,-65188,4056,-60605,9267,27010,6604,-32858,20757,54315,18199,36392,1299,27824,-34457,25920,-4834,5550,73456,70772,-15898,25592,-13981,-95022,-528,13596,-19256,-27518,26531,-5208,-25024,38353,-4585,7285,-73984,10520,41009,1916,-14516,-124194,8613,-11760,19418,-90592,56779,28676,-21403,38220,-60587,85008,58270,-22365,-54995,42945,-20581,-48536,-45406,-21520,58310,23549,3715,6568,8515,-8585,10815,45447,19917,16641,-2658,31551,-2381,7022,8615,28835,85754,-11640,-75771,35572,56515,3137,-59818,59218,70092,49335,14626,-21648,-23019,-35579,-19144,72511,-23825,45620,2539,-33992,19546,-36930,-30263,-27425,-42013,-17425,15681,23915,26151,70770,18489,37626,39620,-8069,-5052,30165,108874,6919,-61869,-41450,62971,73289,-18129,-52847,113097,28403,65147,-28219,145,81322,25397,-24545,11751,
    48378,28701,-11259,2168,11172,7737,40960,9051,57539,29444,4056,-25917,56882,21139,32225,-274,-9741,15229,71974,90116,34082,-22930,-36185,-69107,-66403,20859,-28576,-18618,22853,7091,-44900,-61557,16493,8451,-54053,-63199,-1880,-18593,-46710,991,-32261,-40662,43601,8423,8339,12540,-1234,-44185,50263,3318,-49665,-3314,-56875,-11342,29405,-33327,28689,-61549,72803,58166,-22422,12871,20254,14922,-30052,49767,-120631,2519,88356,63843,-40074,-5078,22398,-32456,16095,26746,32434,-3429,-7621,35642,24338,-91895,-50488,-13481,-32438,-14667,3400,28331,17126,-1482,8425,14942,8020,-35729,27520,103206,44865,30179,-6574,13183,25454,58005,81645,33877,6771,41608,20182,-56758,-11043,-14700,61878,34824,37735,-8325,13160,69213,68933,-46255,42969,12777,9576,-159,4409,65306,1506,59311,4257,-20182,
    36375,-28979,18259,-51836,65908,71762,-17618,-11442,-50112,-66607,-8233,42911,-6262,6956,60303,25530,-53054,-20768,42073,-24783,20240,-24833,-26792,-46625,-89591,-18469,-11000,-33471,43579,-11926,18265,-4228,18566,9844,-58079,27729,20969,78185,-21398,-27592,-44365,49317,-21101,23904,-43186,-29856,-16592,-3681,43743,-34154,83845,41334,70537,-42374,-73031,50203,-67091,48084,-55390,-12385,30260,-50338,30329,-33659,-10490,65734,23,-25355,67196,29203,94974,6554,-54690,68140,-16723,-36695,-57706,-14024,5477,45388,-41467,42667,-53549,111934,73212,72524,44720,26780,46938,27480,-35452,-52219,20052,-28685,4341,-13717,119905,-71681,-32071,-50824,1418,-19695,-33972,14443,-8167,28086,33964,9633,77297,4545,-59954,10693,-77397,14198,23113,32054,-23039,-6144,-23567,-12403,-52076,-37175,64833,29488,-16039,-29364,-62699,-49014,
    -32186,1564,29339,-16044,7429,-68113,234,-37849,-4351,-53106,-56966,113839,22177,-5137,12426,-8643,61644,-28378,-21282,7936,27482,-33724,-12609,71271,28703,45745,-73060,2549,68769,-276,-93469,-67688,-55209,570,-12440,-7945,13863,37518,-63514,-57896,15515,70081,-50861,-74467,-4224,-60524,8768,44508,14687,-17404,-233,-41010,-24816,41632,20124,-34107,-43805,-73898,5546,-8505,25855,2972,-3533,-40005,-14433,66628,-6267,-12450,43028,-41626,-70367,-32821,19070,28301,-12766,-81274,-45499,31163,7716,-7832,-49992,-37028,-32690,23708,9620,-35782,-10571,-17760,11419,-21207,39800,37658,-21791,33577,-20513,63703,-73670,15011,3415,-52615,98951,31053,-13291,11906,-28244,7669,-27703,18470,5191,28943,-30941,8903,25010,10517,-43460,31414,-8496,-61755,11087,36550,-9698,-39129,8524,-28800,-20478,-15952,51983,23453,
    -24976,-4509,-26371,21674,-19732,-36026,-42461,33181,53160,5148,-43009,-20304,-24932,-49916,-55290,-56899,-8126,29353,-49796,30927,-1429,-4572,32293,20527,30448,8321,25269,69047,-38934,5078,-3545,-7556,-49416,-80471,-6020,39069,130676,-34028,27862,48246,17844,-31182,21268,60967,-14850,31183,26537,2948,-33415,-8617,-55700,16586,-59600,-50738,-51227,-3504,3562,-45713,-53062,59290,-35096,-46,430,31687,45717,-24485,2547,21228,39289,44025,76029,31463,-15921,3933,-12392,47980,10446,-38933,-4107,-626,1155,-63050,-26149,26155,18219,-3817,-3171,7763,56262,443,-55091,-13044,-58319,-67062,-27646,-62820,19926,5691,-7859,-26364,32510,-6353,-26454,-77573,-35602,37411,-29675,54452,7487,30425,14937,3617,40882,-9466,-35085,-24621,69209,-35607,-50179,23132,-27858,30040,-31827,-17617,-16794,-10900,44319,13356,
    3718,4457,-12280,-18832,-5589,-18317,85507,12882,-47077,-17232,-99805,60444,56390,5486,-63347,80329,33412,-1604,-27834,-24354,-157,3915,-32849,-25673,-7579,-34994,22945,5484,3262,1205,-11021,-53840,-27561,50091,14921,20441,83010,37750,-8060,-64921,25699,3722,-86315,-61252,28153,-41201,43321,-46708,2431,-23807,-48337,29773,31297,1661,-21897,4031,-58720,53992,-14361,-14774,47016,-19557,-56276,49544,7055,43516,897,39642,35887,20968,28041,41987,-9386,-6200,51331,4422,70885,-58725,-25096,107519,29134,22701,931,20915,66058,19591,12304,-32963,-24063,28495,18178,-21124,26066,86172,17127,52268,-34571,-42649,-14396,9737,-38205,-31392,2677,4733,31705,12713,-44023,48976,-26089,-12057,31591,29028,42978,56431,-58101,-63319,17763,-55861,-15839,40100,-15955,8445,30481,112943,-60457,-33326,33514,26588,
    -1640,7927,15452,-37275,36675,3543,10941,-69151,31582,27595,-31549,-3926,26816,37453,9836,44920,6510,12576,48912,42970,-29410,-20123,-49227,-14198,22226,32450,-16524,7362,44653,-40302,53265,-34319,36249,18860,-103396,73054,39139,25869,18277,23666,22793,51659,-21299,40583,56398,28440,24595,-46763,-2476,-13818,16946,30837,56981,-49424,-57217,-40544,-18540,1614,-2478,-92004,-193,1079,50486,23973,-2767,-55777,-74875,-15834,14381,48460,-27671,37199,-54785,-42876,3279,1717,-24883,61238,-50363,20786,-5152,-80812,-25592,-1309,15421,30400,35372,-1817,2710,-72444,18466,58188,-43848,-53936,18815,-16885,-41337,-8451,10224,-38420,59434,11045,6656,-51107,-23133,42837,-36306,17509,-30260,6084,-70639,-4641,-22838,52914,28384,-10250,17349,-36962,-35979,14635,-32527,-330,24900,-6494,51468,-37586,21820,11879,
    11080,-33410,-116726,5148,10384,-8241,-97891,-26418,-33641,44086,-51779,-39402,-2121,-11618,-32524,-35014,-34800,98134,77353,36311,29039,57639,23313,32741,-12446,-16140,5370,66656,-58398,12858,-30600,14123,39558,48013,-27820,-18445,-15966,-15945,-41992,16632,34748,-33764,-12254,28776,-50781,-132854,-23026,-12263,-12233,-37460,-39413,-37533,-466,-20972,18565,-73486,-112923,61874,-56186,-76279,-8566,-17035,-56499,26906,14291,-68345,47040,-15099,-17189,61950,45067,16206,9310,-36741,34518,-46018,-34528,27183,-52099,-23302,19606,5201,-33082,61605,33565,86346,-28642,-33893,2795,37568,-50110,24383,-27648,27230,34023,41995,-2271,-47,-23362,36859,25146,-50004,2264,-67584,-29445,28077,29389,-63424,-52783,90622,-16254,-66030,-31293,16777,15925,-55546,-33038,-32563,20076,-68014,13947,101733,-17683,21797,57427,-30382,35901,-22196,
    56892,71932,-85078,-1456,-23833,-15784,-21016,-26014,-2,-7172,-11176,36306,-69907,74161,77903,-49894,13860,-22685,4300,-42378,-28185,-22866,7403,-55200,-29135,-102255,3507,23194,-58637,-8390,-680,53093,7973,4724,-42057,-1185,-117332,-38570,8833,65150,-32344,39476,53884,7921,29277,-37028,10547,-3277,76123,-19544,59569,67084,-70367,-37152,70200,-26551,37189,9041,19213,18594,-49920,12484,29218,-23715,-15873,25274,20088,-62169,-37610,41890,-2053,-35752,-41870,-14845,-49557,-7687,7610,-6912,17039,11974,-12578,26747,33203,106979,20351,24402,-59264,48072,26167,-53288,38318,-18558,90738,-59875,-96996,50957,17804,-48106,35263,-39724,10615,-19191,46087,-30231,-21725,-15466,51736,47155,10126,-6331,53592,-56865,16494,-42871,-9764,-16631,16613,-25436,-37003,36978,2404,-21300,-59819,58665,27288,-10588,-18750,1325,
    -16862,-17076,36066,-16339,33613,13590,20333,47770,15418,-67516,-56388,27463,-40736,-56104,-81995,-1073,-57908,-15781,-35572,3418,-3054,18602,-7565,12860,24950,-37553,-20603,-41860,38560,35251,-4957,20667,-619,-73789,-51648,29588,74968,15563,-6282,-23530,31683,31929,-7189,-17248,38142,-56730,-32150,4107,49526,-3426,40304,-9157,-41045,-57425,-5254,-42394,22175,-97428,4753,4307,39490,-3911,9394,-33797,43716,37329,-8590,12146,16070,34811,19600,-9087,10595,5799,-35222,42,34317,-24291,24937,-24481,-7733,109067,-15967,-17264,5632,31849,-42133,18801,7746,-28266,1073,-60424,-49273,-89572,10869,-34370,27669,9233,50335,-15746,-26044,-2366,-4916,-29738,17160,-148,-16087,-8432,85387,16868,97908,18600,18844,-22266,-48575,50077,101318,-71577,8516,-24221,-35835,2076,19467,28907,-7506,-233,64922,11746,
    66951,50734,-19839,19615,-48799,-19053,24682,-34916,-4920,20797,7706,-13456,-33874,2697,90608,24310,-2954,-28524,-8971,-62792,-40088,-59494,-26427,-45735,-53216,-91873,4933,-1734,-40388,16893,85091,44944,-28746,-100660,17130,-9046,25872,20762,89441,46520,10607,-27687,73728,60675,-6369,-21819,28747,73772,-50308,-57898,56500,-13897,-22753,-79150,2357,79802,42195,-35488,-16638,-4274,-40419,-63084,38683,-22413,-9435,994,-65942,3712,52583,-41907,-5162,-31706,18828,67937,1139,-27874,-3293,52760,-9446,-30361,-27259,11823,-67983,-27210,-66454,5318,9562,28072,-84983,15775,1269,-29660,-18824,39137,9810,55531,19593,62700,43946,30965,6728,24807,-13633,37190,-86083,-3039,-2433,-81683,49823,78188,47916,60088,1785,60993,28514,2147,32058,-37262,-3353,-69869,-54035,-20060,56297,1856,13473,4993,8919,-97923,
    39125,19805,23719,28437,-51564,-33260,60862,-3271,-20057,14876,-26464,55825,63207,79692,39141,-12091,53506,-30768,-41042,-25481,-59989,26,39573,18902,16141,-27133,5716,-36602,25434,21553,47225,-20453,9683,-5668,-15737,89877,-16925,76077,-7220,39082,17905,-50492,-72414,-14566,-56935,15370,31434,-40561,14764,34097,-10586,-29908,20302,40325,-14717,-18674,-24009,4654,-24690,39400,-59666,-39267,-17077,-74054,11433,-56784,44360,-16134,-24769,73013,-38009,14900,13556,-44867,11277,-8895,53699,-12859,36845,29204,-31317,-19845,-13871,94102,-74034,-2224,4573,-2723,-41884,-43268,-2005,-17128,105838,-10252,35561,38465,-19107,-50580,36921,29674,34593,-33501,-23962,15155,13844,-65140,-41284,64367,13196,-64576,-46323,-68282,49341,36508,-94220,-46125,-13628,42542,-26187,32645,-12069,-24441,11718,43650,4192,65739,28367,-14690,
    -34534,56399,72572,-57439,43074,44138,7757,-4303,30279,5183,36265,-42624,-23800,-21342,-36976,26378,-67305,3824,86351,6390,-50658,48397,-45359,30833,58301,-88066,-39572,-78184,34619,14147,36367,-2032,3502,29621,20232,-32013,-18980,-79526,20084,-67680,9455,25752,-12767,46816,94573,-62481,-20975,-8241,-44024,30109,2796,443,-4802,-16702,-3517,-16870,29835,-49654,41052,-51364,35198,-6077,-9674,20074,-21514,8795,-91002,-5695,853,-37375,-30391,14229,26096,-61365,-19121,30530,-97887,54299,-11423,-1706,21187,24130,-21060,-47774,8264,53066,-10802,-16137,-38885,1111,35102,-24977,-102971,-11036,35039,41138,26018,50204,32609,2802,-6992,-41288,65940,43143,38718,58868,-14178,-44585,14661,49704,-26236,8511,16584,10638,-6451,35961,20174,-52013,-3995,-16300,-26266,26283,63014,-55097,28991,-5612,-10212,-40761,
    8275,-20734,58309,-18781,-50330,-14420,12742,30454,-1847,-28777,103928,-54791,13553,88340,11401,-28042,10511,-20164,7529,-57049,-767,-90465,-21848,-2556,-29262,54055,-22242,-7679,-27698,-5159,-75433,-16124,30431,45646,-107027,19978,-86087,49887,754,43376,-22756,-2070,1565,-26198,63951,-20031,101733,28610,47814,103391,3483,58101,63810,46463,-23558,-12890,-13520,-30101,26527,-49582,-75607,-39330,28718,93999,-34114,22043,-11837,59145,92962,53624,-49793,40231,-91653,-6406,-1135,45609,-11857,1829,54682,52068,39738,-11351,6255,29447,-24435,-41676,60525,-53920,-30363,4685,30686,5640,31100,-8757,30230,32327,-20375,37944,9987,50033,-6509,38260,-4114,49838,31571,-12085,-42231,-69376,44850,-22145,-44701,-12127,-6,27180,28874,3376,-73241,-38381,-59026,24324,-12835,15436,56340,11866,-31310,-1246,-56428,-53972,
    32044,43649,-35360,-15726,-2937,-24370,52070,-10014,-19101,47551,-12254,4777,-36993,-5799,-53,-43275,42537,-10101,1571,-15679,18363,17979,33924,-33074,19111,45963,-46191,-74862,35962,28841,-114990,60142,19900,-32544,-51615,43229,38255,703,34625,-25803,36461,38772,-9695,-74494,-11804,-70478,33609,35937,63550,33211,15675,6076,12548,-13581,7603,25299,-30647,-56523,18359,-12491,26397,-81392,-60608,2712,-56714,25957,-8913,-18628,-27202,-41041,-88453,-28211,77422,73120,-13193,17923,-24101,-36287,41640,84064,34544,-12618,51946,-21537,-38405,-38858,-14503,26050,-36125,-41113,-38466,13780,-1248,12902,30,-31106,26993,23005,-39153,8675,3569,-27879,9909,26498,-17783,-44075,-18543,11420,-17050,-46870,-28824,-35162,21588,-30173,-2316,14362,-37111,50400,-54938,-7772,-29040,-10282,43436,-18869,12881,37971,36719,-14792,
    4050,13881,6555,65776,-49215,58337,-19542,24245,55473,53629,24979,-54663,-27904,-33356,23683,-42294,22099,-15026,45489,-21395,61906,-45254,13166,-52784,-10136,85721,-14052,15932,64390,35305,-254,28160,-5745,7712,5961,-54033,9582,-24615,-31549,11142,52663,-23243,-17495,19553,-50737,20358,-1516,-5596,-1491,2173,19243,30503,-33552,-5366,-39917,24551,-51422,-24529,27861,31843,7134,-56029,-51906,74502,-13692,-13154,-8530,31241,-14974,8563,-3501,-21104,-68959,15456,-1758,-7319,-92918,61270,48995,42449,-15889,-36738,21521,47937,41997,-46382,38584,-28531,-1645,-40244,-12823,8496,31361,-19821,-39093,-63386,-12434,-26289,-63546,-66922,-38101,62316,18509,54674,32139,-7424,-8099,-22997,-7168,78882,18503,45317,-49051,-15361,2335,1964,23496,-55228,-3866,-23538,-14491,-39177,-73510,-68381,9363,-1062,-25279,21366,
    30983,-51758,-21788,-28790,-19692,-11996,-17658,27994,-38947,-39182,60209,6378,9666,32171,60766,80733,9504,-19416,-10121,-26576,39624,13000,32478,-92300,-39560,-14100,45752,67891,-27939,-37188,29738,11165,32776,-26378,-43549,-33030,-84883,90319,-5324,-23418,17528,63993,44693,-30941,1131,16361,23739,-29990,2979,-62781,83832,50144,57309,-64775,-11888,-9494,20051,10544,24608,23309,-50567,-15065,-22757,9534,-24145,-12299,6217,74668,42678,9237,50113,454,-40919,8630,91594,-14801,40763,-81508,10251,-43555,32693,37456,26107,36526,-4600,71736,30532,4660,-34701,-2258,48885,-28982,77376,48870,-3178,24421,-25779,-38219,30656,-4609,-44077,23713,-47659,7426,8323,72385,-26592,-19679,64451,-28977,-40677,-5520,24246,14308,-39532,11560,12484,-11314,1999,-22626,29193,22170,37994,15882,76897,51613,-12690,-22393,
    -47,11681,-51628,27413,484,42712,-55133,-78824,-7561,-10376,87132,-28078,59190,9832,29143,-415,3781,52128,-15528,-38517,-42712,-39507,-34280,8852,27930,32032,-34355,-11556,-30522,-24553,-28077,-14937,27891,-53671,-11708,-24078,-7370,27203,-64940,51513,23450,43728,24077,-90952,30454,66059,32289,12320,-52552,21744,20561,65004,18737,-35853,58944,24933,-67621,-25257,-58676,15965,-2329,73242,-27517,39856,-3961,-33731,18353,-36153,-25585,-36992,-25503,35678,38585,-26426,-18737,-53580,-31005,109283,-39776,4770,63120,-28112,-22364,-39665,34974,-7095,-40246,24393,-34515,-21422,-14649,18291,-48522,-40990,1953,27688,12574,-21031,33997,-83693,-42515,51892,46941,-79572,-127515,11245,46973,28037,2436,27655,24784,-70547,-26226,44713,5714,-43975,-67060,15240,103088,-58906,21970,27760,-2157,-23040,20425,-50813,28919,29860,
    -52533,29873,-9705,42501,-21997,-41685,38115,57087,32392,-12909,24802,58691,52522,16330,33919,-24073,61638,-42698,40344,26961,9830,-15464,-4089,9623,18967,44267,-31409,26004,38124,3592,-54297,-30479,5476,17805,-21614,-20330,-34541,8888,-12594,-41567,-12066,11346,125313,17836,5100,1406,-25035,66456,818,-6472,-85773,-1461,82906,-14006,7190,-7755,-44290,-10986,-39847,57336,12427,-82189,41102,14919,13315,-11083,31887,-39631,20546,-38992,-72781,14780,-22861,46465,53441,21470,-42403,60231,116,-54402,-53739,-39323,-15577,-54741,-37022,28813,-13497,9693,-27682,-15812,35450,42884,-16401,13749,30282,-96832,448,42545,24254,39310,-21129,53160,31984,52621,-26355,-59043,-69517,-24346,57883,24146,-36341,33382,-20562,23010,15210,-26106,-28832,-8698,26395,20670,-48496,-34404,-8167,40703,-17627,12803,51706,-6813,
    -22630,23008,41694,66186,40146,-18959,-45965,-29451,9498,21139,-37671,37959,-30290,73970,7814,3778,-12069,20988,30858,77695,16637,95822,4581,-42828,-49960,18999,-30341,-18887,31646,-17505,44962,29284,14566,-35015,6217,49311,9445,12395,-56048,-10214,24814,-27867,13161,-3652,-61077,19191,58638,-25217,81922,19605,-50687,34357,-15088,-33625,21388,-18637,-55811,14890,-32785,-525,-47370,28153,-61193,-41397,-28500,-13517,27289,5903,46707,1285,-60405,-448,-64261,-7664,-24089,-9899,-30160,52488,64601,-20545,-22198,-24024,22931,20758,-19441,-17756,-7477,78355,-15873,-5750,-9330,-3171,27623,-33752,-5895,-75212,34247,-13519,13304,-28436,-31475,-6805,40641,89327,45975,-21034,36471,28479,34509,-38778,-6368,-10530,-65796,42568,-48826,-32670,-2837,22637,-9714,-64546,27720,-29597,27564,17246,-90621,43002,-33952,52282,
    -7259,-51295,47419,-4949,-17861,41604,-94646,-52754,60185,-16850,47743,-22698,-56739,-51385,-47475,28522,-15442,12204,-67697,-19775,7688,-41378,-83479,-14539,-52810,12830,-24115,38932,82578,-52758,-118492,-18690,17823,40027,-30220,2442,-54549,76245,-65485,-6298,-5897,-62453,-102963,-25632,9890,60200,-3594,-12271,35399,12752,67212,60030,-12462,-30767,-50057,-18902,-14279,25594,-87049,-30094,25526,7803,-13018,3645,53107,-22183,23732,25889,-20735,-31678,-30530,-32867,-9815,-139,-13261,1713,32111,41587,-37938,26443,19859,-72941,-32140,-21546,37043,-70324,-20016,6856,28800,30622,26791,24336,-17000,15778,-87001,-29640,-47412,20032,35,-67872,11688,5136,-10883,-5200,-23047,34542,10401,26909,-7337,-13169,12278,44868,-29623,43958,-14037,46204,-8180,12866,34716,23270,15683,-27702,19021,-81443,-46468,-36207,24145,3526,
    41242,36802,-46556,-2897,11591,11872,15024,-16664,7922,31007,-5367,17703,-35003,-5348,21539,50131,12381,-6958,-5257,-63350,21448,-19935,-19303,-11246,-48579,-33258,-28920,36659,24051,39696,-32392,10059,40693,70452,-34787,69292,-14090,40882,41907,-1370,-29071,15446,-35503,-4041,-68581,-22041,-5687,-20489,57303,-79178,23714,-6179,58908,16337,-34931,-17170,-87300,63974,-43032,-61491,-1104,-25007,51727,7779,36192,-34627,-159,-30715,26614,-31837,-2623,-25077,-29044,-18932,46892,-35365,-68427,16548,-18151,-18784,28085,-3459,1562,23615,11683,-11026,59009,47643,121935,14866,-63479,61706,-37072,52630,3148,5708,-1548,-75772,14863,-10227,64401,27168,45893,-43921,-10337,30231,19665,15469,13512,-56323,15747,51248,18826,2532,42275,-48834,7940,67885,-20643,-14749,-31293,-48887,-21737,56971,28303,-32513,1324,50760,
    -31843,19224,-34176,-39831,22047,-20895,66881,16679,25759,32893,-44417,-33379,-3106,-7892,32058,-45305,-18534,19623,108688,81646,-22724,-39293,-1756,24629,-39272,-1365,-46581,15861,-9350,62379,-21300,-18861,108617,-10196,-12238,-47899,-24507,39538,3691,-22023,59536,-22978,-854,-40173,22938,-55222,5398,-137,103662,-1394,-24540,43783,-47517,5966,5086,3281,-53995,-59280,45648,-45453,37738,-3476,1663,-795,7746,4094,-23497,-2083,97320,76338,16722,-27528,9611,-10411,-15958,-50449,-84774,11712,9948,5480,-73461,-24246,-77436,4977,-22416,24789,2987,39398,-19996,45568,-3049,20015,30575,-1107,-8638,68500,355,-5725,-20686,55358,-836,46559,-8978,16212,76121,-21926,23041,-76403,2237,-20773,-60241,-44223,16704,-20442,-1650,47962,-17111,65233,32706,5715,57283,20854,-2362,54810,-18581,-36000,-35457,-61787,
    -48099,-11506,4270,5268,-19633,-35794,-39792,33455,64438,-8590,-20921,17572,21680,20007,62285,-63231,15033,-28521,27012,14598,29952,55547,-1408,44141,-45552,50971,6188,-33075,89627,-24909,35206,-14030,27952,-50195,-39596,-41731,17093,-1233,-29153,-11854,2808,31697,53012,-8098,-17787,-8736,33555,-47061,89663,4917,-11759,-11653,33192,19021,68395,-8226,59581,-338,13014,32647,-95142,5442,7668,-3103,59097,-25060,-27568,36927,82562,-16805,-56906,38626,-12988,-36508,-11893,44812,33335,-57860,19179,-46057,-42939,-73630,43752,203,57822,-21425,21515,87903,-8133,13026,19540,-23124,-8422,-58841,-4447,-43986,43183,-6229,25836,-55154,45736,8268,13621,87674,-3062,-15479,-55375,-2630,108478,10275,1260,56221,65270,-56310,618,-17057,-18291,-15101,-1775,-50829,30218,-60737,22806,-24421,-43931,45454,-25013,32694,
    83529,37450,-12491,-36292,45014,21484,22573,16902,-60918,9265,-40468,47722,-71003,-28178,33252,5129,78451,47171,-98023,-78737,1240,-18615,33739,-33750,11330,-133992,37339,-963,6696,-73520,3268,24848,98308,-26456,-18565,61636,-16367,80575,72604,-26994,-9746,-29170,8711,-3901,24157,-15327,7422,-45914,25055,-41354,25451,32792,-6206,-14348,19542,-42399,12134,62718,29090,28239,-11284,-18599,-45617,-24977,15133,-13896,39058,-15879,120651,74057,41103,-63323,-65477,10769,-51815,7991,29777,20047,82264,-46104,3117,-58314,28335,-338,-39178,54161,-46123,-6731,857,-41592,-37927,8954,48127,4184,-4482,-25733,31964,-5669,-21481,4835,-70411,-30473,-34670,-11564,19372,-51122,-11225,-19342,27276,-31969,-60782,24885,50685,55089,67894,40810,44584,-7187,-10891,-12469,-34447,6070,-21487,68153,-12402,-12142,33970,26967,
    13755,-11921,-33549,27428,91226,-26841,4859,-13246,-19503,5300,-30355,-39070,-8847,6247,27126,56073,-35721,-70167,86512,-29360,-48072,11468,-16805,20858,22469,-8377,43256,-34393,27126,79376,30326,26665,18654,58534,54083,40665,30250,60143,-62913,-55754,8702,-37541,10307,-7099,12735,17070,6415,41808,22307,-17097,23985,4449,1294,41730,-77724,29691,-22990,11757,-26630,-64696,29744,26070,63381,-32240,21344,-36183,-66942,15156,1273,-38679,26377,57562,-2445,-60300,8398,-21462,59445,51253,-37142,-41246,31275,49580,45248,82184,851,-60146,25557,80294,-8026,-16033,-26920,36850,14199,-50466,85929,25744,17343,-115144,57622,-29376,17658,19129,-27537,19223,45696,-9010,13630,-27883,3034,-20051,23580,-36614,-66113,13135,38916,46485,-20017,-10251,1965,32856,-15512,-24092,60962,2775,-8945,-12902,20095,-17599,
    -10985,-6608,13418,53976,-11714,-6423,43659,7601,48908,-21588,13970,-38360,10020,-88029,-10551,34265,-89295,-50123,49816,94186,21803,-42659,34812,-43252,20215,14559,52636,-25453,16133,50894,-49656,57018,25778,-25901,-29821,-14375,55113,10637,-13362,28274,-35121,15352,-58684,-42174,-13450,-36647,32231,5500,67435,-62338,77966,-17682,10997,-1065,69876,26838,-47813,-13693,-17929,-13621,-21588,45124,-36627,64825,73670,-21364,3437,7099,48556,-52015,-1845,10069,43855,-23232,-10521,-27930,56079,-47391,-44599,24648,-47859,12952,335,27957,-10947,-31695,31235,49010,59714,-20894,-9757,42858,51567,1952,46444,24558,16654,58499,-22618,-59138,66124,-35214,27863,11600,28528,2034,21605,48146,-60031,9305,16745,30191,-41599,-15884,-72341,12464,10132,48856,-20708,-61854,-34521,29155,-27602,35063,52103,104247,41547,13055,
    -40988,-38648,19531,-19577,42982,77808,-43405,6081,3503,-26822,67134,-9488,-78255,5437,-41962,-40019,-79328,18183,28558,-2498,-42415,25170,15336,17345,22254,-20989,44146,-17177,57698,-19463,-2744,-8155,32325,-19831,77113,-6505,21431,38703,-28449,-30639,-30950,-15987,-46610,-35147,-74435,-96654,-20152,-16133,15661,-51599,-44668,-2413,42860,3569,-85377,-28708,-41751,34320,-91160,40505,-25844,-65324,10773,41966,11600,34534,-5752,-83931,-11703,-28495,55274,-23023,-29611,131,-1698,32686,-25721,-53795,-6340,61355,-6835,-32771,48609,-24791,19802,-54689,-1611,-66053,35284,12794,-25072,37516,10034,-10340,12961,20937,-15807,-40225,18338,-40374,-56466,-2159,52862,-58856,-11094,-36092,-45703,18588,-36736,49166,31686,-75256,-49836,-25011,-14239,-14606,-20457,29743,21934,44707,-15106,34008,-32686,-40193,72706,-64047,-3603,-8017,
    42945,-41168,-7129,63984,870,-38710,1053,92470,55981,54132,-26092,-21060,46053,69378,-43339,-97373,30881,38030,63101,34674,81283,28890,2402,-82484,118705,58711,-4691,-55869,-47875,9118,20934,5038,37592,28819,-11228,-52670,133992,63735,3306,61322,81993,-23364,-9407,-55928,-151226,69927,52049,-60246,12218,67606,-16221,-36252,-36894,3697,65518,-32241,-36403,89430,-29056,11809,32885,27557,20396,-41480,23275,-87830,110,6855,120812,119849,-47754,-76702,-47557,-15701,68363,60538,29257,29032,-26904,-13941,-1274,-45105,-5233,-17735,-18413,44158,37569,102216,-6825,-76256,-55842,-255,91562,10828,183735,-27854,-19163,26990,-30682,84820,57394,26968,-10439,-50063,30707,-111546,-61605,-59026,-25608,25472,5174,27590,74706,40705,-4461,-99272,-8762,-38185,-8374,-39488,26205,-105164,24341,4123,37749,13705,-31247,-13600,
    30462,-12295,12172,-13719,41512,-12712,67749,40816,-50336,-27921,-51368,78357,16989,26861,-27076,-17559,18717,-42746,21315,-269,-32616,-71230,32444,-4598,-17771,-944,-4361,-39546,22854,29950,5890,-1991,-31144,-14058,-44543,-6316,-46866,13673,-1141,14386,-32339,43590,-45148,-27497,-24506,1411,-20356,43526,45957,21538,37812,13465,65635,3798,-21836,24987,-88092,-12837,-48002,21289,-31499,-70729,-23464,-57425,-10647,44749,-33423,-59743,-16184,-21908,39809,-22919,-74163,-60775,-31018,-23803,6248,-30908,-28356,60479,-22231,-18537,24748,-5917,9158,-10990,37839,-96014,-333,-11605,-48476,-23609,36140,-5829,72001,-55667,-41328,-8122,60734,-34707,72100,17223,-26007,-38072,50770,-44162,64107,-11550,-10734,32189,-43909,-17789,-4721,1451,15361,48213,-13914,1231,-25445,38666,59287,-232,-563,26177,-32163,-8778,72793,29352,
    -30147,-73802,42486,-2963,49785,-14354,-24305,-8246,33534,66944,12825,23673,-10010,-19181,-66386,-12056,-13894,-57286,-46597,5090,-11718,41319,-56681,31767,-18062,25591,20071,-42210,69596,29173,25499,-41940,-39559,-76391,-27397,-25245,-42510,-46410,2508,5015,54914,-52989,33052,7740,-18116,13090,10753,14888,71160,14474,-53606,11972,-81364,11638,33968,-43536,-61025,27458,9872,-19208,6717,7245,-13227,-82626,1926,-45959,19669,-4169,33863,-39826,-106593,3830,-52367,-16118,-59017,20664,17792,-72202,-27673,-85572,-27077,-15780,-19397,-45263,59084,-65683,-78073,65673,-37152,46395,-3115,-29092,16367,52723,15601,33969,12306,36885,67980,45306,37362,-20226,-74478,30364,12918,-16776,48951,-122442,17827,44052,-20699,41772,-8826,35938,27005,61459,1085,-27032,-25902,-5328,95371,-41836,27617,19673,-42515,-31712,-12737,-67846,
    9276,58772,7702,-28405,-4554,-712,-75914,33138,-38635,-7174,20849,9271,-20867,-35934,41902,9195,-26788,-33484,31249,-15969,7323,-10130,-18328,-75938,35843,33792,-24599,53704,-65260,-71875,-548,-36123,-4235,48434,-75658,27405,27700,-24475,15584,-61137,-15856,5927,-3133,-39881,-68276,82580,-8301,83278,-19113,-3049,3274,18441,5780,-4536,-15818,-81353,-44334,-23259,-45573,-27282,18712,-2239,-10628,-67121,19063,64866,-6356,35936,-44420,-43070,69848,10893,-27122,64267,66887,16216,33610,-55136,-77962,31904,58440,-19087,-50099,26720,1866,81760,11520,-48955,-16218,-24337,-19251,-48026,44538,28742,22674,-21582,-16553,18292,5028,990,5379,87803,26125,27086,67309,26135,6475,5410,-14176,25495,28496,18388,-64800,39593,-16957,-14004,-16953,-16302,-39192,20884,-40542,-14101,3652,22561,41578,6548,30509,-4931,
    -41330,-30771,13384,54191,-58201,-6737,-12537,-18989,90524,-13580,42974,-9529,53317,-34536,-30003,-35805,-71924,-19046,-44155,36576,-16703,14257,5167,-15507,1506,-38273,36499,74952,-107041,17430,-30538,90780,-79015,-6326,3734,-42417,61648,-28793,-26629,-34938,-21040,-52645,3876,41861,-60378,-21509,41112,18105,4518,81726,-25659,-50028,-36751,9431,28377,-41734,69502,-24116,-36310,22633,6874,58695,-41571,68876,32321,-42346,-51001,12860,-27970,-26261,-19602,-12694,-25511,-21939,-22743,58487,-1779,28047,-28525,9321,5119,-34903,-18509,-88637,27336,-42647,29839,-10205,-119135,-32653,50372,6345,-19811,-11187,-3867,11714,-24037,77449,19417,75926,-50267,32027,33484,29789,8900,27507,6873,-4817,57740,39556,64220,53443,-29150,3819,-34073,85146,-32796,-7953,5110,17047,24265,49720,-22401,-17035,-40172,13665,-24556,2171,
    17244,49017,-26983,10270,-56985,-22952,7793,7105,-82343,39916,43189,13284,2553,103023,49340,20228,56944,81701,27802,-93874,-31845,-2539,17897,-30972,13028,-120724,-7476,23819,-53626,-22304,-31436,31657,23144,-83760,-24774,-29383,25581,6651,-10862,17981,40282,-18094,88050,13668,-17791,18582,-2390,31154,17859,-43789,-15428,47705,28374,-76773,54122,-60307,-44325,60454,-29540,33038,9895,-36883,19858,-9150,81313,21512,-36505,-34233,-44751,12425,-47408,-5194,-15589,-17582,-3219,-4146,18877,57926,56230,4067,36902,32953,-25010,2671,-8416,-32953,-2649,-28228,-49881,-33074,33437,-37719,45615,54790,27422,24914,8877,-36259,-10459,-24439,-67189,-8916,50791,8778,-19938,26666,-23224,-13541,-1310,26075,28836,-47456,96745,106003,-40889,-21950,1977,-59658,67915,44,-28509,-3271,14868,5143,18667,32580,10055,37756,
    -7403,-20207,-19754,-2400,-5266,59228,-52657,-25093,-18668,-42174,-14405,27805,-17823,-73241,12312,29574,-11390,-1219,24942,28508,-17912,49231,-34506,55345,-8071,52177,-46208,17816,76190,-62960,35179,-26319,12576,2277,-23405,-96329,-55496,25982,-12964,9925,15194,8182,-4593,14272,1288,43779,50979,-46520,43705,-51109,65102,63608,-55406,-14971,-60107,10630,80983,31925,-60845,3751,26925,-47454,-25814,28197,35687,-84313,5202,19776,56929,92569,69640,-40348,3578,-17523,5440,9110,-50990,-33278,-16369,26592,-55962,-71055,46125,16872,60990,-1430,45482,-18114,20231,-11565,45827,-31522,-14035,2789,-67999,-60143,-32821,-65907,21778,-22165,34376,-12948,-33793,-25346,-43696,-35092,23155,-5175,1120,27363,-59530,-13805,-97431,41775,50334,-51470,-37470,-42841,18404,13495,41871,-55107,-3415,-75866,4576,-87704,-15371,-31627,
    16264,12855,-12518,-38696,28466,-29311,-4845,20487,-55225,-35118,-55168,2825,82294,-4376,26709,17217,-12444,13579,-61723,-32933,-69806,-5035,-6229,14272,45953,14733,-5499,43561,21442,-20269,-53968,60830,5318,-31330,-12756,-39123,73441,-73411,21896,29120,-892,-27514,46343,-92930,-16620,34835,57337,111978,-60343,23961,22534,22928,-25129,45305,47686,15645,40608,-15419,-27091,52624,50735,6005,-4342,4042,27322,7324,-40690,762,8626,-14310,100008,-46202,18885,-18639,-29450,-7278,18629,11150,-19139,31791,61207,-21222,27939,-29906,-26609,-21816,51760,49461,-38797,-91,59121,49745,-78488,42502,33807,8343,1534,18593,-19697,-26101,2432,69307,38703,-28676,7655,-21456,-27345,9521,41537,18158,-12906,-29300,-38614,31291,41841,42452,-42374,51486,-616,-13049,7029,2380,-22087,-1685,1373,-36906,6374,39507,
    -522,8578,-13820,18522,-24705,-15677,-43938,-20053,60382,-45479,16724,-276,87537,-1859,20900,-27918,-13780,-9706,-38968,39264,11786,-25605,42266,8914,13077,61107,3054,-1820,19105,-15860,-29313,-2860,19676,12383,-22252,-21037,-16406,-4507,-42804,36717,36360,8777,-12750,-26759,-56428,-11316,64688,16187,-13012,-25908,32641,-1130,74682,45967,-49957,-8508,41104,-36352,-15475,-5674,35394,-35839,-37921,10444,35367,-24378,850,-31448,-101384,15862,-28006,36620,35415,-10841,21825,13645,70672,-82977,-15943,5531,41354,39407,-18239,15831,24870,-36555,41607,38197,6840,-7272,38748,352,20610,-56904,-86237,10117,-72930,-89788,94006,-12060,18452,2609,-30366,-56606,69186,-10520,-27889,106618,-10663,15055,35177,-31378,-45186,-1570,16988,-16321,-31556,-11118,402,20901,51017,-3696,-41518,-95898,51708,64751,-405,-54236,
    -23293,18432,18914,-28517,8130,34620,-5403,51603,33301,20226,-8830,20644,45321,-24268,41455,4715,-17560,-60427,56154,-16030,6504,-46825,42534,-10326,9866,41430,-56270,18774,-36646,31250,-57484,-34124,74736,-66286,-31180,-44501,-38134,-40927,-82344,281,-17921,-38530,23338,-45498,12341,-12551,-32720,12798,7056,-61008,26258,44171,13239,1465,1969,12901,-5804,-84524,-13833,-21197,-37630,49388,-8632,72532,-6234,69860,-100643,-55854,-9782,-10206,41081,28039,2550,9882,-67514,32674,27722,8827,-19145,13893,-46357,-9235,-11265,-13361,105,-10903,3107,-7764,14354,22607,-15213,-49788,16079,-59485,42487,-41689,28897,-21626,15422,-19384,-13759,14352,-34246,-12807,-16729,1862,45688,-63216,-16465,-9552,22619,-40638,7624,-27532,-7364,-9374,-13551,-9102,53809,17863,5561,-35500,-48475,-26849,13428,-48749,11442,23344,
    -1259,-23716,-29174,15567,-4563,-30108,-91946,44802,-5636,-33706,26499,-23919,44223,24436,-16654,-67116,22484,6000,5600,7307,70792,20304,-13973,-2714,18700,83093,-25246,17588,34989,-9855,-27159,-54658,-2698,9677,-24117,-9237,12163,3556,6062,9358,30197,15986,1126,-15391,-62664,15301,17403,-14004,11697,43087,-6927,5346,27234,26711,13780,-30892,2788,3636,38666,-79211,-56923,-35801,-13705,1896,24419,-19571,10576,3887,35698,-16718,17632,-8656,10885,7630,38480,-32620,-92361,78854,32662,-44049,8863,-87377,-27925,19020,-41824,-19280,23721,-2254,51869,2184,-52178,-27083,-27906,49388,-31589,-5625,1961,15515,-61401,40216,82695,-16249,36413,30786,-52129,-22247,-33733,23028,-7883,-19713,45386,-62886,24439,47976,-17118,-13679,-55528,18330,1623,-1236,9919,-3063,-54971,2258,-28049,25,13714,39303,
    63967,58697,37432,-19205,-69510,51236,43416,41615,56556,107086,31005,-9663,-98765,23194,-9442,72599,-71102,-29673,2800,584,45751,4697,-94847,-47137,3760,-97072,54435,-80085,3234,22680,19580,22377,47581,-61542,66338,46990,-20210,13161,-16927,42331,-42750,-33318,27285,40535,24618,-21676,-36056,-16950,-13125,-14497,-24094,22067,2024,-51968,70378,39477,-78643,7373,-8893,-13404,24392,5869,46050,-43535,3769,26432,6789,-45306,17408,1140,-13370,141353,-43436,-22924,-842,-34668,4144,53953,53524,-7685,-35007,40087,31329,-30588,17943,3504,4020,-49071,53311,-1409,39536,-25078,45086,-99682,24398,27946,96859,-26591,-62146,-50101,-43301,-21699,36728,38149,69043,7710,3746,-857,9798,1681,39004,18658,23962,-20127,-1933,19785,-3538,1460,-35873,-18519,13714,-32176,59106,33332,46456,-15698,-31323,-21829,
    -12133,-27407,-44625,76923,43493,-33879,-19995,-24998,2422,-29988,2812,36021,-26981,92449,57025,-47638,22820,-10164,-5252,-39935,-26734,-13990,351,28962,-68168,30156,-7310,71264,-29209,12387,-11556,63741,77364,-26486,-30681,-29184,-17704,-30814,-37746,48527,-8288,32166,-44791,8693,-13573,-73486,3830,69002,45985,-32901,75002,23424,-39480,-23910,10314,71941,-31105,-64103,-33503,61764,-33000,-310,-11451,-29499,-5707,-3264,10971,-31657,-23261,-43808,-16923,42,-24017,36090,-25745,2235,-55840,14019,-48180,-10090,46891,-22267,11265,20647,-16360,-13430,4901,13355,-2054,30287,-6576,38933,31174,-35737,5909,-38931,-27583,43618,7966,-69050,-50348,12501,30313,11231,-10943,50957,-5433,12113,35464,-18269,34996,-27197,-20923,-76279,1474,60393,27974,39315,-25402,49564,-21085,14421,-23241,-12800,29686,14459,-82945,28077,
    -44595,-32971,34665,-25393,35839,-15833,-30763,-9152,13804,468,-36995,-71978,43193,56093,-25455,-74818,-5398,-35987,67984,-34839,-10196,-26324,-34571,-22338,3479,38012,-71961,-25961,7434,-24619,-39432,10790,45894,16422,8768,-41366,-33031,-10332,-42342,-2791,19449,8485,-32902,-11944,-41748,379,-11816,42922,33028,59783,-42281,8723,-74573,32209,8176,-43193,23952,-3294,-40997,35282,-63516,-92554,12438,-45062,-38272,-25445,-7654,6535,-22270,-40231,-16541,25081,77553,-10579,-1437,107300,-7918,-93236,-42829,80262,-17070,9157,47662,60724,679,-47486,22032,6233,53197,-44718,-4243,-3232,56754,-30423,-13825,-15935,-14428,-31731,-52583,-89525,15809,13644,8527,956,8868,-19562,30260,21692,-6006,-4944,49025,-52183,-21133,-47275,13078,-27242,-31495,-49059,30753,43210,-8397,26248,-34623,-133449,39443,-50336,-80583,-79589,
    26676,38108,15109,-35705,-23113,-11787,28705,-5494,-28235,23229,-38147,30337,-5097,-62871,29453,-52294,-16458,35674,17077,37069,16953,-15543,22812,-20949,-24511,-35080,-20412,-62422,-19121,26126,-1982,-31713,-23581,6093,-35368,92538,15312,37465,-21571,-70267,-9492,14040,-86355,-47631,-5709,416,30296,-13566,42395,2001,-39169,7028,29539,-27535,14501,-41929,-45853,14036,39866,-59425,11019,-6282,-77179,-73360,-33623,-23414,-7197,-55159,31833,58458,4798,-57606,41458,28359,-11821,-24849,-9941,35829,51564,-83391,-89418,-30332,19522,32264,-24962,50314,-1020,12447,-21048,41538,-43647,-1273,7050,4071,24126,16511,46791,-72132,-2027,44046,-12044,-22101,-88060,17018,26799,-32089,34810,-40911,-55877,44874,-19568,-50501,2716,52331,-4618,-30974,-2344,39144,-23331,-13194,12972,41955,16956,75653,4930,-15970,40853,-11176,
    -36854,-57219,41753,4069,68208,3641,-19432,-36224,-38011,25906,37112,66379,-60962,-68609,-47633,-52280,7691,-9443,-3821,-9277,-17930,-20429,34929,47835,-6626,-34442,-52989,26,-48310,-65635,8488,-1538,-62409,52191,62105,-29573,-7009,-5381,72067,-81090,-9028,205,-6838,35515,13660,-21882,5717,636,-23835,-50986,-22592,16571,37818,-12153,51036,-55591,-15822,27004,-41719,27918,-57235,39143,-29665,-26616,-32465,47510,38720,-32357,50984,-93997,54874,-38541,-45006,874,-65165,52516,-32807,-86953,-14138,-4809,-15230,-34701,-7067,-57926,37424,-39377,-2134,-52229,76250,15126,8282,7897,1129,24465,-9887,-24551,-6978,34062,31418,-42788,33992,-48040,-32580,-51942,3603,-8221,18681,59688,2453,7996,-3783,2914,8727,-51432,43883,87245,-61751,17141,-63607,49324,12469,-17554,-12031,609,-17781,-22644,42522,28185,
    -5718,48555,-3475,25573,26483,7857,8997,-20257,8161,66773,8607,31956,-30395,-20716,39958,30683,-34615,4160,34208,33142,22014,22085,116554,10975,-38732,51662,65098,-24083,32968,24112,78170,4201,40487,-26010,4949,-35569,23005,-36252,-9525,-4948,-10741,-44761,51403,13895,-24092,6257,-16552,-1276,13684,-84424,6613,23564,44292,-66292,-43059,19684,41578,21921,59956,3077,-6045,36373,49492,31884,-66199,18320,-95071,3713,43556,42232,-12690,-38518,-50164,-18839,-18645,-7083,-63631,-74281,-13602,14184,-2288,-38914,-58368,-22673,1127,32799,86157,-23553,-1812,-45610,-7786,-17460,-36964,-34330,20913,-1094,13113,48998,30099,4431,-16906,55215,-56715,-8816,33089,-2529,-61696,-43563,2199,12211,43808,38686,-15824,-26140,-19070,14626,-25332,-34931,15901,-19362,-9079,-19101,-32442,47883,67287,15181,-33372,67710,
    -26271,18272,-25066,-15051,71212,51068,-11720,7220,-17882,7605,63595,12694,3278,-63868,41241,47805,23556,56005,-19830,-27299,-64852,8158,33430,-23480,11942,1311,-52830,13602,-22182,21036,15508,3524,62010,22715,-52787,15088,61737,15176,103167,-66508,14530,-8523,52761,-16543,-49142,-16882,35943,-6258,19463,-59634,-22010,20627,8354,22398,11041,-88085,-25234,-536,-14309,18494,-9547,-5158,-46352,-34335,24398,27798,-23846,-62954,75593,26926,75054,-95484,29433,-2258,-68477,55766,-36332,-23964,-41087,-32439,13533,-9563,12588,-67184,168,62337,2014,-340,9493,-17889,29229,51340,-62746,-23314,-11447,29094,51502,-33334,38542,73367,-75649,30870,1306,-59306,22398,-86905,-67186,55655,21771,-4293,-33110,-43789,-32953,10235,101195,36075,44041,77843,-52105,34683,-16141,44411,33081,-120,27048,-38404,26712,26927,
    29718,20773,-10223,-2821,-23536,43564,-59306,-10170,55585,14630,116564,-6314,-2107,-91385,-16787,32937,42886,-53172,-67558,108429,12001,51142,-51477,-25694,-40496,-17005,55909,-2286,73389,-12496,-49687,23702,-1880,-105,4329,23150,-24645,-16260,-48699,-45944,-110106,33609,-4562,79916,68411,-9229,-40499,27049,18463,883,22436,11951,43381,-56242,-38825,16244,-28739,-27656,-22838,87409,-48685,-27241,-60203,-4196,87559,92127,51616,23821,-79901,26271,-50861,5366,3876,68007,66713,-41778,-8289,83091,-19616,-26872,-58851,-24621,12753,87569,9924,-22536,-22297,-63845,-48871,23615,-28170,-55781,-36752,25702,42452,-52236,-30147,43374,14568,-24851,-15572,-37222,28510,53189,-31983,41590,75988,54745,-52868,57738,18240,20446,2014,-41630,-50538,-10482,66163,57842,4798,-28443,50224,-38839,50715,35324,3230,1950,-6915,-4181,
    8100,38451,-12240,54064,2514,19809,17487,-15143,-4024,-60213,-17895,29317,-24165,35938,28212,22062,2656,66576,-22190,24288,33007,38720,-6381,2028,2318,18171,-3413,-36348,5945,81459,4020,-17111,16839,-8953,5149,5254,-31813,-30202,-33150,-42758,-53298,56209,19681,-32453,56029,30960,-59603,-72779,-4220,-3216,-14039,-8200,39075,-32336,12809,10834,-77077,28208,45401,-1686,-21935,-16301,13086,-5971,-6661,-29936,14469,41887,13465,2648,-28454,-38307,2706,-84061,28567,-12071,-15335,-29503,21717,-12861,-48642,7951,69764,-23515,-16788,-37614,-35467,79805,11198,-93335,1971,21799,-32939,-33757,30358,-36015,24249,-83468,22576,-47481,-40742,-38970,-37721,-9705,12282,53799,35006,-48845,36370,-14982,-12303,6369,-25226,-648,-38383,-14103,22464,-39748,-60054,-14780,-62414,-14971,95570,-17038,54546,67188,37089,-8633,
    69108,-30798,48273,-48145,-13477,-35086,35483,-20335,-10845,-39273,-36448,-5475,32984,1643,-19116,-9171,15670,83979,-14726,-54042,53022,37339,11902,1275,13909,9329,15458,-28053,-4047,32619,8475,27508,5969,14522,15393,-30343,40694,-12369,3266,-13246,-24289,102935,-21706,-85881,22226,-22792,28164,60772,2811,56646,46773,-4090,53260,16246,4053,9666,27163,59130,-10811,-72668,72041,53981,85068,-70263,-49253,-40465,-26328,-23101,-37642,-30586,-36880,-5926,-2007,-1947,-3962,46863,10523,13410,-34411,71306,85617,15214,34886,-22947,-41596,30023,46894,-78005,-36052,-14295,-17627,71765,5954,-30932,-54182,67003,-41417,-31742,-15266,-21020,20284,19646,-9097,-74159,7647,-12972,11454,-5700,44356,-62119,-52776,-49387,-4479,-40489,-60528,15732,-62698,-18460,-57261,78461,32902,27883,-7855,33366,75793,-16354,26239,29080,
    18480,10532,28877,47470,-2413,-15155,-8940,-88388,-40186,33991,12063,-20949,1334,-5733,46172,147,-3770,19700,-49413,-45101,35740,-4961,17931,-40289,-41549,-20323,20318,30352,-20713,-41333,15860,-45010,-29180,12170,-36255,55307,70223,-6080,32408,-38390,6884,40573,-27641,96055,-27122,72723,-20879,93969,-40554,-30607,17678,-29834,55023,-20567,-51514,83762,3081,7842,47978,35590,64177,-11589,-15007,10500,-27228,71222,13246,8263,-20195,-7698,3803,-44097,5734,-17388,51631,-18857,2490,42886,-31049,-35565,20759,-26286,26278,10428,13663,40184,-47875,-89914,39964,14651,11541,-2799,-65247,106249,-21013,-68047,10358,5581,-18780,40526,3934,-37466,87232,-23046,-45111,-3201,16894,94899,-35757,52117,-16341,46434,-12779,52555,-30068,-62293,22895,-20723,70256,-13359,-34424,8698,-37054,-58001,62091,19117,-46255,80416,
    19705,27900,-4940,-13334,21469,24618,23068,207,22950,2499,-1489,-6604,25283,-74484,-86978,-39856,-16813,-72264,52299,48539,24949,34608,-22619,-12611,-15986,-14855,-765,-3795,50203,6144,31369,-49661,10548,30887,-70831,24990,-121,53347,8657,21097,-38395,-30430,8968,-18617,39574,-50435,29006,38383,-58445,-40396,-19675,-38597,46490,-27531,-15243,21675,-25433,-47347,71559,10038,3167,-48721,-45041,-30558,-18694,15687,-63602,50399,-40667,-67040,-24215,27884,11929,11060,78940,-14126,33967,8886,-24637,17213,-14908,44380,28307,-16832,19715,-43934,33517,46174,20354,-17517,-113308,-35934,-10490,3774,4904,8328,5641,-34491,-20019,-34252,20542,-24778,534,22091,39442,25417,38296,-21472,-25375,45701,27464,33146,45096,-18301,47102,-30769,53390,-10542,-35005,3334,40105,3156,-340,-33240,9535,-77323,30119,-36460,
    -38294,63476,-20032,-15610,48448,-34182,28492,-21526,-36248,-86156,34953,19017,13395,-50854,-7736,35887,-43276,-34976,-31796,-67669,-25759,-8212,46780,44521,53244,-19587,1666,-6785,-6873,-45239,4438,48264,-11189,4145,7154,-26607,16030,-75767,56490,-2894,4809,-13711,-37194,-79382,-35981,-16583,26720,41695,-21999,29921,60578,30027,-28682,18529,19639,-48088,79317,-16056,7593,9486,20255,17059,-51920,-30269,48104,-17716,19766,-13009,32851,-34005,32085,-10658,-25147,-87025,14153,65726,37623,-81851,-44387,23938,-322,6084,11274,-94577,60567,-10762,-1968,11724,-18590,23989,5540,-32754,-63607,8121,-23908,-22571,-45342,13275,73387,38267,40045,23583,24191,-23558,37686,-5712,-46152,80754,65573,36990,15818,46119,68134,13627,8703,33923,-28139,38592,-68697,43151,21155,-39391,-17347,-1012,-23870,40950,-15577,6731,
    -19381,-9035,46658,-95245,11709,-40494,-42387,-52402,6301,18596,27337,-40805,-38592,12322,-8608,-6034,-4132,78744,6335,37791,-44954,-100957,22065,-33651,39243,-5909,-42399,12246,29470,-27775,-43128,26402,-35683,-15495,32278,11620,22317,9538,-38134,27968,97080,25544,84575,-15838,83974,-30273,-12503,-7372,-14918,11949,-20065,40311,8794,-20106,-36347,67543,23798,-68120,25869,-10336,-1113,-31099,14443,59757,-15626,68725,3773,7026,-60112,48564,-21801,-19159,65524,-7310,-49188,-39348,1905,33007,-17274,-7404,12338,-33460,-37369,-35333,35481,-18988,-48844,-57496,18974,-44645,65094,-44387,-38186,-52546,-30848,2181,-44732,-10206,6641,-29074,-55505,-29595,1861,-43885,-31651,17046,-46290,25305,9099,12312,3060,-11412,47976,42795,31128,8624,-50087,-54327,50879,1144,-10193,82839,31132,-134261,-77869,-35194,2016,-30993,
    -10820,-19987,-1834,-3083,-43779,30177,-91743,-7146,-58241,-87448,7165,-49650,23858,63895,78515,-11489,-5712,-17273,14680,39925,-751,12009,4290,-3807,-37117,-21307,-61143,52167,-3798,-50557,264,-80553,16237,53271,32088,56801,-15907,12408,-13934,-56264,-98762,18500,-6810,6461,13643,11825,38668,1328,-7555,30072,-33633,-9508,8142,-43336,-6147,23281,220,-54278,-63758,52207,-78390,-72885,-3074,-4819,13856,55004,16803,-19318,-40538,-52372,21650,-28138,27358,31824,3093,9172,16301,7830,-51415,62271,-4918,-43367,-8631,36486,44891,-44077,61604,-60888,14844,-5780,36718,9214,-26291,-2952,4588,9835,-46237,-82604,-17949,-46109,-32950,18539,75113,18200,-11529,-6796,-17503,56889,16174,41494,17622,-67720,-74853,10752,-31121,-57510,13846,31282,-24628,41235,2149,-10810,-45697,-17608,-2617,-16666,30966,-8845,
    -20229,70025,40659,-94388,42442,-10403,-21460,24191,10538,35048,49404,-81898,-8332,-26011,57496,213,-8532,-45201,-9522,56847,-73720,-55930,-8331,-34082,48274,-11262,29735,33240,35225,-74723,-59188,-885,4085,-30282,9650,33107,-29186,-22131,-55456,-59199,-5489,-37146,63394,-20285,301,158,-11295,-593,-2333,16227,30092,41419,18683,-18525,36034,-60829,54025,57339,27289,-42899,78998,9037,31969,14273,5048,12400,-63440,4850,-54629,35199,-7665,-2916,8121,-3653,-18574,-29231,20291,17001,-29322,-654,40715,23278,-33859,-108005,15317,86257,67524,-50472,-81406,15936,-16320,18629,-77091,-14515,-18319,26584,-34986,-430,28736,-12950,-58010,36187,-18332,7352,18635,28346,-12556,-10997,-37783,17006,-1646,4080,68732,60701,-39146,-16773,5107,5229,73975,-80251,-12164,100578,-57554,54154,14691,-5186,24642,30932,
    -11141,35478,-240,-3801,-6381,6745,57163,-1630,-7610,-34694,-9413,45489,41379,-28295,-13488,3872,58268,-16248,-105191,41665,-39656,-44862,20134,-6246,-3279,39467,46086,14343,6932,-59507,30288,-51878,-14079,-78711,-94314,69024,-46583,74692,17181,-61281,-52272,12371,-14356,-17784,74763,3268,14523,22148,-20036,-100174,-418,35511,27277,-7504,-51022,1797,9932,28236,48021,86314,1140,16800,-29234,10479,-68224,30989,90815,39242,-47704,79830,-36455,57952,50784,-18592,251,-12015,50520,-108484,33441,17996,-4712,37868,-26839,-16976,-26869,81109,-1036,34621,-12362,-34388,-48257,-29691,30588,-84434,-39974,-45453,24025,4424,47533,-5897,-57818,47943,-120820,11970,2621,-46683,21271,22660,-26800,-36646,53715,23360,34894,-44391,6128,-38489,111283,-1241,-6256,-78809,-24363,41756,-37694,37978,-25622,-18787,-39023,663,
    36336,50467,20807,-1977,68216,-18132,-777,-21787,9859,61819,-35047,2969,-33753,-42053,-61247,-53897,-25237,3998,3751,-61901,42201,43250,-14335,56016,14533,-18989,39303,-41615,4294,22188,20385,-46552,35416,-40662,-35124,63387,-31422,-32701,-35938,24343,5666,13368,-29852,12331,10896,-73548,9731,19118,56446,-31932,54464,-36736,2792,11467,54518,-28623,-24004,5965,18320,-56275,35715,44634,-9052,-75202,-24783,-87656,-58631,-19159,-29960,79794,-8010,12752,-26017,-63121,-47834,16948,-23091,49970,-70821,33475,13314,58345,55599,-43408,15927,16849,-24141,-30718,86650,-1936,-24510,-49408,-51403,-78846,-57137,-9503,12760,-68473,27928,-31724,31936,-58921,29424,-100082,5983,7034,59407,17302,63341,-17394,-8714,-53533,-62588,16188,-82944,4426,-29961,-36392,-41681,7555,24898,-28534,-21863,62505,39080,-40118,7479,18483,
    25835,-1839,11657,97206,18397,-13764,114840,23271,74952,-4459,16977,2874,80196,-8942,-21041,-43857,37161,-16690,36803,-4713,-68962,2193,-10559,2831,53682,-29586,102792,-31147,-61545,-23566,5452,55624,-49961,-79110,18438,-14218,33956,34471,38548,19793,-32341,-43290,4808,61874,-31274,46539,-17597,-5912,-17187,16847,-17427,14371,10661,1184,63891,-2937,15116,49789,-39173,67470,-27784,54487,-45605,54,-26005,8437,-30231,-85966,64312,34397,-54816,30597,-112313,-10213,-21862,65789,32487,37718,-13054,14174,4450,-38962,3891,-3336,-27606,-47485,44020,52334,-15284,-7439,-64523,1577,68929,27539,-21366,28794,20424,-5542,-5209,-650,-27080,5432,38959,8259,7096,-62511,5371,-62336,13415,-35835,40253,60456,15564,-39011,14496,-22149,15435,-34907,44027,-14889,4902,-39615,-21667,59109,-44942,-28104,-6978,73557,
    43210,-24863,88398,31965,30833,10586,-43067,-31286,9641,21419,21831,1157,-9226,14041,3024,22701,24731,-14365,-91881,4973,20945,-13499,-9806,26620,-68959,54261,-28966,43338,9751,-39324,-40190,-43431,22249,13708,-7690,10447,6353,-10,25361,-43600,-8577,-47976,45528,20088,26339,-32581,-32632,40519,-34424,36332,1109,22181,51749,21673,-12816,-37853,57614,1022,-1415,-20031,-1903,34872,-10440,51404,-19257,42264,5312,60099,-25951,-18855,-74954,31587,-38419,-24061,-31302,-15292,7294,-33642,-6080,-41330,81375,-36537,-55503,36397,-11373,32690,60129,-9264,29375,-1281,14242,-1747,-52895,1231,-48355,-3736,-10184,59897,-77038,-11785,-19337,83298,-18821,-29738,7333,-22532,-37056,-33656,6253,9484,-22418,-30823,61284,3064,-57541,-602,36380,-29573,-12214,-24602,-19171,29981,-102450,39247,-81994,-26110,-4079,21037,
    48565,-2365,-45409,41818,-73148,-34860,-1194,-39575,-15218,11841,55137,147,43324,17336,65222,-20039,2840,27605,-37870,18588,22700,14159,-55079,39848,-78306,16675,-2017,90265,-43940,-55108,-73357,6751,-11815,-17380,-40412,14238,-15979,36593,54920,16983,3305,-9393,39825,20346,-8931,-13342,-28207,43260,-1038,-28261,49328,47148,-10288,13025,-3829,48681,32073,-16078,-82778,-58136,15985,-5052,29129,81972,-37766,16296,43016,-43834,20799,-3532,-103568,72525,-53758,52102,-71011,17123,-35585,54334,31065,-37405,9439,-52483,-20695,12662,54536,-14552,-41182,-19127,-17936,31003,-20836,9616,85003,41535,-157802,9518,57124,23014,-33420,12084,36047,43435,48011,13653,-63086,-36772,25779,10701,-45395,-14634,-22855,27456,-34739,21093,-31355,18171,-10751,-24185,13934,-33125,-7990,16288,-8273,-20370,-79596,-21239,-93108,-47252,
    42370,52106,7947,-3575,-1996,54414,13903,-86228,-4442,-49007,54223,4343,59765,-33840,42343,46442,-59788,-335,12819,26457,7035,11740,-24348,-16784,-20668,2486,-2340,41544,34841,756,86935,-32978,2555,33770,-31093,-9562,-69272,-17036,-25359,-18025,18647,-57069,27452,44463,75221,-625,-19982,-41653,-54858,-38009,-86768,21852,31337,18053,32618,-106559,60844,-70616,-1272,40237,-41689,37268,48432,-7713,49194,90776,-21079,-1442,77211,14084,3637,-34726,-29535,34019,47166,24504,-2055,-22585,-2831,-37577,45347,-37141,25229,47188,38236,9780,-24389,5472,-50611,-49670,2875,2395,-28814,6184,23617,19180,12579,-32475,76302,17986,-67437,-71253,21133,-47645,26175,67015,-46451,-41776,29675,39766,29269,-2772,4068,34435,36734,-21719,-2247,-39264,12411,120465,-49345,-1390,107495,30367,-65168,14322,78587,620,
    -31676,16683,60056,-18577,-54747,-16053,17987,39397,13253,4311,16864,43496,-14063,-20737,-12959,-40808,-77088,-46899,17012,-28211,18057,70615,69871,42351,-10451,-827,23596,-36498,1460,59909,24508,-36481,50252,-17187,13524,-63179,-71161,36973,-79823,-8246,-32771,-8582,-96487,-75011,-1947,-21908,68977,-35294,107428,-20710,-20924,-12326,-3530,-7950,24966,-29402,-28529,29685,-23210,-33047,70576,85188,-7043,-27151,14928,-44560,-43809,9781,-15535,-19141,-39571,-2377,21311,1752,341,58204,10595,-22178,33738,61484,-62228,-50220,16620,-47426,-15310,-41944,27833,-21909,-273,7,48054,26464,72530,-48139,-21975,17763,25957,585,21692,-7334,25385,15021,-6100,-22332,-7117,8035,-11572,14895,-4797,-39698,-38078,6931,4986,-15672,-81867,-12076,-69463,38782,-26302,81045,365,-24800,-22771,37662,2536,16695,15382,703,
    -60429,-28888,-32798,36148,-26708,-81134,-56925,54698,75488,-55448,33982,104664,-39315,-70741,-9051,58048,8066,-13350,-16758,17377,-29161,70487,17305,22666,69331,13154,35601,13330,-44057,-31420,43706,14368,-12408,-29050,-11609,-65583,-28430,-64995,5454,-2818,25546,3248,76140,13000,13322,-23500,-13436,-15657,-31163,45353,73752,-13476,-45489,-25172,39292,-45404,21514,54155,-35373,-9130,19135,-14538,-81583,-108653,69900,-41936,15970,41483,-16108,-31134,-39422,-22995,-41687,-38390,-49496,10993,-78263,70766,-8133,-32963,-29467,-28381,81042,-58409,1563,12859,-47464,23187,-8230,6163,14085,-18832,31866,-25444,-10278,-52004,31745,34376,4079,-63945,36227,-77544,9405,58854,14631,15297,37106,24354,97341,31763,-29191,41713,20317,14622,-85535,83149,-42381,8763,-81223,117444,92546,-3415,32741,-760,-61714,-10938,123951,-6538,
    12979,-60706,29004,-18808,-45029,-12909,-58779,40344,-54375,61904,15345,-3774,-29730,-11653,38916,12780,-32561,-59801,14733,32543,42081,-37096,31458,-47811,-20341,-25160,61377,19558,-26301,-11091,-24301,48010,-23731,-70063,67015,39874,61715,50824,-8740,-52921,-8188,7636,-10946,47709,-112320,-57458,-38667,11223,61705,-34845,68352,-13454,10237,-34105,-9271,-12916,15146,95712,-18133,-8505,-44223,-36056,29710,47574,-9204,21324,-2706,28828,-48037,25595,-60232,-50589,-1260,26845,39463,701,2236,5570,19018,-20790,-49223,-48503,24292,8948,51252,-7436,-18099,-74550,-7365,-21796,54850,-14601,-76148,55704,-1895,-16930,-31932,20737,-33234,62498,50931,78906,10886,42994,-7888,-61337,-27166,24008,-22116,-43445,3715,49497,78889,-27236,-50120,9588,78666,16715,-38203,-41725,-22000,27435,-12960,13049,-17636,-6993,-49718,-15321,
    -9468,-18849,6096,43407,7550,25630,77370,33050,17057,-59803,-5055,51446,-18848,33077,-43305,20707,81127,50405,-28815,-26633,-77669,18813,-5728,7811,-16494,4,-44304,-7996,-559,-37261,18585,34860,922,40336,-9555,-59455,12006,-25892,51884,39218,-56329,19610,-62596,-26566,28297,32077,43219,-12839,1519,48174,-41645,9960,74634,31812,17868,-62008,3410,774,-11067,-47212,-5378,39924,-14869,23309,22869,-26278,-56724,-31153,-28932,-30361,46199,-40757,-12595,1256,-18637,-21525,-10005,2544,48834,-26185,-49212,9405,7891,-34947,-23671,-39304,20430,26695,21227,39974,-18853,75421,63884,17101,-5677,-10410,-7680,12022,10328,-80588,-14058,10944,10248,-52818,14439,53289,32927,3090,-21892,20454,50870,8507,-4805,-36152,71288,-12144,40460,-32006,-9723,-47275,-14524,-42361,-17897,10946,42060,24075,22541,50475,
    -55328,-1677,-25432,-13538,-31251,8952,-6502,-5692,29555,7512,3425,26873,20278,-60504,-19999,14725,13551,-22151,34962,81949,-2866,-2734,28582,5326,-25985,109997,-36601,48706,-5567,97960,-36834,-43828,21395,13527,-37490,43187,-31743,62464,-40964,-47834,-14894,-6077,22942,25311,-4721,312,-9571,4217,11280,70977,-91577,-23728,36893,15692,-47026,-36281,-17203,-30280,-15462,1746,11844,35019,-25174,16891,-1599,49748,-11038,29381,34974,20372,4566,23695,10829,51597,21597,-25861,-53075,390,35944,-9483,-78515,-82063,27628,-14446,-24764,41388,28955,-41325,7776,58774,-57273,86598,19743,-15734,28767,-40130,19866,-2271,12570,13447,-24168,-63996,-46116,26156,-53855,-13752,35757,39465,9834,-11490,-54224,-2107,77774,-38043,-68135,53490,39475,104839,-17227,69478,34474,30181,-28043,55567,-75479,-23576,-16831,43168,
    50930,-21482,10713,-362,-27254,-64523,65081,9939,93697,-7542,76948,-27085,-40527,56023,-106459,39337,-97833,-9370,7749,272,13467,62700,-74515,-29471,41964,34390,9663,-715,-70373,96486,-37152,74240,63370,24242,59848,-26845,20608,-27628,-25123,48642,97733,-83016,15908,-66992,12295,77,-26982,4581,9375,-19938,-47709,33859,12011,6572,15884,47264,37712,31666,-38347,79964,65300,37199,81858,4218,39760,39766,-12322,-65804,-19763,-15784,9469,2319,-1226,-3521,47801,58292,6042,-31164,-60386,-14564,55517,11782,70034,-29387,-68879,-32050,-82156,9460,39293,13398,20638,6051,42054,-85222,15685,24304,-32234,77385,43320,21964,9861,9100,43559,-40620,-8967,-31696,7505,30070,24602,-47477,88266,-18159,12126,20876,-51331,22843,18974,64957,45043,85407,22756,7557,24076,6925,15207,-61333,-27393,7707,
    27778,-1628,28012,65813,14695,25808,9815,43888,-39631,7617,5337,11710,-31653,-10123,-1289,31617,-5338,-13657,81253,1388,-5210,23890,53115,12919,-14246,-23235,41343,33802,-49081,-19101,75800,25822,-51280,-56391,43957,-73948,30906,-24655,-15954,-25973,-13140,23844,56641,104177,65390,-3055,-57301,-17156,18787,22110,-25234,6890,-87038,17631,3523,-9710,31504,-12764,59449,-11776,52558,43311,-61404,-48799,43031,-20550,-40142,-9232,24770,-12258,27597,11593,-19865,-61205,-22107,-1946,-17279,1120,43470,-79943,-9906,-376,5960,31383,4369,28711,-6613,33025,-21263,-47903,3695,18545,18518,41317,50782,7222,13244,67358,30937,43597,170,-28078,-23660,28372,42636,-18281,14993,-80341,-37992,-22150,-60694,-20127,22723,7156,36628,54091,-85983,-13823,218,-31394,-17672,-44474,45711,28088,30033,28456,23165,580,
    -42282,-21173,-28232,3538,-2594,14734,-18093,74371,16683,-53523,-74952,-56679,-59077,10587,-71120,-2976,-54070,-42527,65880,-83945,32088,-18213,-36766,20345,46934,-81,24144,43233,12037,23827,-103744,36647,-23672,79570,97910,35595,80567,56224,10558,-54985,-31930,-35000,-29786,-21592,-13117,-6977,-23416,24238,-55666,50637,9174,-74226,-106108,-18692,-115652,33999,-5091,15306,-22783,-251,17145,-44521,-8889,26565,17465,-1305,-52604,87068,29444,-28295,60223,-32192,-43683,18065,92738,16455,-19439,46181,-53955,-8677,5539,9480,-59179,-33197,-50902,-70289,51496,-3939,90507,-19315,-16022,21850,912,52798,41965,62129,-26586,7534,-41003,-15072,17093,68077,-10228,40956,-50950,-8288,-22956,-9231,-14358,-11118,59819,6463,-99360,26828,3859,-13643,-412,-40205,-13091,52731,-28284,-48278,-61412,-27082,3608,-83045,23447,-47495,
    -1169,-35623,49192,-36413,28819,-25390,-59148,-14140,26417,-42006,54413,37311,-61400,-35220,-46777,42070,-86018,-5993,-13601,-7380,22974,-34015,6440,-38885,-16644,-44982,-6092,24509,-88365,-44589,-48638,27384,9638,37036,62220,11707,22330,1373,4379,-76200,-9739,7877,-53550,-31899,36960,17671,-30687,-36351,27679,14956,36338,6284,-14794,10760,-14997,-70501,-43256,15149,21148,-60860,7354,9636,-35698,16092,67206,40748,-7396,56693,31357,-59592,1710,20043,-9823,10371,10421,10,73633,17128,6080,95610,2784,38555,-20393,-25669,7763,-10732,-45311,-95559,61304,-64612,10472,-19348,73690,-49238,45560,-5714,31669,23215,13271,-27097,-898,-12694,-2113,49320,-48903,5297,50746,32970,10952,-42953,27626,67824,-21006,-12788,-25686,33909,-12578,-81,-9949,-3968,2811,27795,39782,4884,19858,-7217,44163,-33131,
    52690,12137,49930,32653,-17171,-28773,-83484,-54572,-4556,-63301,-19835,7584,-54686,-75420,28787,-45874,-86991,7179,-8988,56384,36434,-28235,47809,5205,44193,13740,-11498,44261,-22808,-20426,6420,57267,12964,-31127,-5201,-25988,-62520,-96740,-94069,42052,54954,27884,-31421,-51319,-17000,16814,31449,49533,-75810,79750,12676,17786,-69278,139,-10705,30056,1168,52202,-54540,20543,31144,-44483,-8476,4373,-17895,25945,-4984,12377,-28470,18704,-43740,-16639,-70289,-34407,-85639,-47587,-48813,53645,-10551,42619,-86747,-13374,-32930,-15755,-6612,-87444,-6701,-16512,9514,57933,199,14323,6591,-19455,-49815,-46649,-29939,-59382,30787,-111937,695,-30665,72257,-17892,35550,50050,48207,-37521,13130,114597,-23970,6927,2453,17849,-15140,-58631,32695,-62730,3609,-14985,-37589,-34049,-74926,-60499,-74875,-10417,27457,6789,
    44722,-2837,31352,-18440,-21315,8954,-33534,-20672,21736,75980,16933,46731,21828,-64862,-58520,47954,-34665,-46067,-130182,19781,-21581,41984,-37442,-11550,56524,-30683,-8887,-45575,54575,-28377,15817,-60229,14153,16477,-439,33766,-416,53633,-12721,-12220,-16765,-51438,-35481,30573,11649,-38305,-18905,-23965,-42076,-18448,80899,10761,61780,-87071,-9684,10521,-6102,92449,-68395,22760,-19960,5692,-9152,80618,18101,42347,21433,-5812,-32000,42069,-8000,4412,-48080,22003,-1532,-30027,46471,5159,2133,-54451,-33805,46706,-19320,39689,80185,8279,-46697,-181,-18322,71225,11029,-7864,-58796,32617,8007,21255,10947,7432,-48887,-22890,-47135,-34430,-17104,-112679,-20424,42092,2366,41164,-17284,95704,1019,26527,63253,81917,-37538,-12339,31755,-12488,-19650,18371,1505,-30314,4913,19249,-80053,-68096,-945,61159,
    -61073,-3942,13752,-3618,-2052,6643,-7924,-7082,-25581,16746,15143,-49885,35646,38369,-4689,-27967,-36305,-66213,82569,19479,-41469,-12276,-1324,20227,-31873,-30665,-43994,-4463,-1467,2872,-83619,26250,-85881,59544,60923,6820,-2253,40358,-7778,19579,28808,-43382,97024,34780,-49708,-37541,1541,13813,-40335,58717,-84301,-46945,-9902,127372,-7868,17666,48631,-3477,-15966,-45334,-50045,-8208,60735,41276,-68582,50939,9189,40177,-30728,23781,-2888,55965,10745,1416,-14709,-19684,-587,24530,30763,-4887,66196,56543,34752,-45462,-9474,-10686,5568,3903,15424,-5541,-22227,18294,11026,60380,3335,19112,18036,-71965,-13416,55254,-1199,17649,46498,-9307,11544,-61029,-34595,9739,30406,-25960,6199,-51767,-35278,-31692,45153,-13741,-85069,65259,-2869,-26991,-34620,-20712,29260,-7595,-61227,-16702,-26051,-40760,
    -42493,-21046,-30449,-63355,-890,-23303,-3570,-45047,-62778,-36170,70143,-27510,1428,-14387,-22270,-21207,73975,10633,19488,-49397,-49496,-8421,36175,-1903,34689,-15102,5882,-37784,-11337,-15503,78145,-49234,44595,16462,36548,-34333,47122,-4385,96011,-103387,-144994,16734,-65951,-39330,51087,10313,-14551,17308,-28268,52726,17193,24213,58553,-48609,-4149,5028,-39770,-35142,-399,88309,41242,24775,26571,56236,-62014,82946,20768,-37904,-27573,-48831,-13981,-19143,8737,-14236,-49155,8562,-6155,-5935,-44164,79358,1859,-29793,-43907,-38816,59969,31490,-12295,-36392,-38247,-42658,-24719,-6239,-21122,-9910,10729,5157,14604,15925,-1804,-64855,-9879,45639,70546,-17713,24745,18529,3289,11271,-17006,106713,-4806,73565,31695,-55661,15602,31254,36659,23030,22140,56773,-21249,-11201,-53099,800,-6093,-80009,-23132,45778,
    1537,16315,-43416,-67517,-3906,-33182,-14956,-19397,32698,-29099,-106022,26309,98371,41711,-44828,15591,13307,-84540,-34593,-6640,21351,-3840,224,-68615,3323,-12812,-30752,-8022,35595,14617,51955,-34077,16182,42524,3698,52835,-10689,20481,-59436,69104,-67035,-39523,-55316,-1131,-16245,-7629,-4003,20685,-36829,-70370,-23315,44561,-13996,-20440,-2481,-6509,-80165,9538,-4463,78976,-15205,-74128,73200,-1299,-23997,-19999,45333,-59311,-23203,-48647,18793,3422,-33331,71190,41198,2574,61343,-3960,-32550,33029,-51468,48000,27054,45347,47733,-57953,36757,-3600,58289,-464,-1483,105499,20449,-69373,-6325,-24729,14391,-75987,-54546,-91363,16689,-5047,-58281,-12421,38702,39704,-1105,72160,-19130,-6710,55608,-3454,-45720,-66799,54181,-42677,56560,-110851,-7200,-56712,-28912,2254,56676,-83572,10916,-41557,56957,-6564,
    -12027,-28677,-73329,-9340,43400,-97900,-47341,-8481,16315,5463,9025,-808,25588,17019,28248,-18664,14329,71227,24576,277,-51547,64068,-24034,904,62563,50821,-27416,-85423,20526,16384,52074,54322,-5231,-16456,8982,-25518,66109,-52935,11996,24280,-12588,66954,4950,1724,-74192,-13845,28050,18276,16937,12362,33980,-12331,-61299,-13568,59074,-22104,13704,-9372,3368,27760,-54244,76217,-9267,-81081,48369,-17629,-21108,-14022,12162,22204,-20536,-30945,64408,-8706,-33777,91873,-6453,8734,-38117,26235,14844,-44735,-26902,-33217,-17717,-38444,-37476,20118,-23211,23788,41218,13391,-27892,12035,34515,54764,-35867,-14517,-900,-32256,-5356,5244,35113,-13990,-11998,-11575,-15346,-14215,53447,-51618,36992,26676,94072,-27382,35291,26033,4677,-36342,27036,-29588,36924,-28242,14288,61272,24525,-11492,-6092,63861,
    14479,15690,-2278,35461,-1953,42009,24268,56911,3016,-20828,-4433,-2741,74649,-1865,-33581,11900,-17095,7694,46297,4112,1118,25533,36652,-18828,-77393,27225,1280,44461,-59804,-37117,42180,7966,76546,-41321,-11110,7232,21357,-42830,-3645,20020,-61236,-64435,-44896,13095,-58726,32888,29100,7219,35229,-54566,-12002,113858,-26897,-11213,-2736,-65027,-72787,51742,-24509,-68203,16662,60992,24761,29882,22544,-118827,-51006,-66232,83092,-48469,12080,57403,-83197,-21922,19587,92183,67856,-8704,-46964,13378,-22466,-22519,-3415,18487,-1784,18597,20727,-15648,-13605,57739,14940,29611,42725,-2753,-37963,11737,-8323,-7309,4382,24809,-59963,79927,-50689,-14463,57067,-5033,226,-45572,-105803,32752,47758,-4277,2257,-4347,19207,-4381,33594,-52613,-27015,-66235,37494,73283,7577,29237,6753,30737,27318,-454,
    -10213,-51243,-6131,5378,6445,427,16584,-5302,-26554,23345,1678,21627,-37053,21761,-35800,-13899,-32435,-6438,30413,86553,-5171,67425,-85595,-3165,51066,13661,39398,-9425,28697,17079,28282,-30299,-79093,14956,32503,57861,73883,43484,-26044,38809,42572,2649,-52563,36972,-84607,-25502,-30969,-103450,-46219,-1055,-12938,-95384,-49185,11398,-29528,67947,-118514,-36209,-41024,12873,-5367,-5824,-21270,5433,-27125,3844,46656,36983,48538,-23322,17126,7066,29362,25637,32643,5728,-32511,-62927,-30389,339,27270,-1904,-24982,-115789,9959,-31973,-39400,-24029,29155,41105,-61984,-42981,-6149,85376,-23945,34365,-14038,-23510,-2021,42487,-20596,20566,-21080,-73041,-11197,-2279,-41493,6216,-37842,-2083,21726,71830,-51532,50626,-41284,-2139,32967,-35364,9255,-9262,30402,-37025,-30679,-51084,52786,-9023,15347,-7750,
    -17967,775,17298,4450,10169,-4177,-12605,53229,-5508,1030,-45434,-21639,16604,-9242,30800,-5491,5501,24508,-28013,-13401,38692,-30858,-90901,19442,25863,39244,47080,-94618,73,62540,39140,867,12387,-37152,64916,-17548,26127,-27137,-69198,-41975,-59995,48064,-51596,49900,84614,-61362,-39891,-45171,42857,37911,63364,25676,18384,-21625,91501,72175,-18606,-83109,-536,-45114,71680,41395,-8143,72541,-33731,39826,-7240,-14729,-22421,38053,17397,19107,-16153,10982,-35910,-26778,9857,-25876,19330,-1965,46044,-39165,19988,65679,130332,26705,538,-54206,-15434,-36987,10952,-40192,-70299,-73934,-44401,59921,-22431,5732,-12631,-94177,-69722,27864,9229,-33752,6373,50049,20659,24392,-2763,23607,52517,5245,29187,18584,-680,38982,-2162,-92018,-5750,-1160,-15468,32976,-12096,-3910,66215,-47467,7332,13880,
    -52796,-36208,-30516,63283,-41763,16743,-19115,-58171,15453,43071,6103,-12921,96058,-1856,-30191,-22178,-40192,-7901,16252,163455,17210,-18863,-3448,6925,-1355,-23007,-50906,25067,-22321,25256,-23515,-7210,2955,62449,17732,-36087,-38416,5842,-98880,-33850,-21042,-19139,28144,-28669,-3795,51753,-68284,19296,21829,-5416,-4749,68752,10831,18571,-59849,15335,-31823,-43889,-14013,42505,-53389,23860,11222,24377,-32759,-17089,14200,24039,-68466,5792,-22263,-40144,6018,-14626,-20,-25691,-57966,-761,-6865,-28179,51544,-29840,-17200,17164,-25471,23614,-49269,31661,-43374,-59768,-10516,-18647,9542,28657,7556,-28957,-23484,25860,34268,41895,-53867,-2345,-62030,59628,-33559,12196,62926,-16545,-52194,-26145,63487,30357,-89184,-6295,-11590,-1676,17124,1405,72471,-31700,28377,12462,592,922,21306,-62623,-14672,44395,
    -46924,50324,-42401,4226,44323,-25039,3723,-57835,33929,36570,13097,-28166,-37051,-15776,-60020,-1435,-10522,-34631,26168,29276,-14539,5627,-52452,3558,-24498,43490,-19301,-62917,-32899,54618,20847,-37600,30007,-45064,-13974,-43553,-12189,-22861,97349,24127,50351,-32365,34320,-13043,41969,-38754,-13352,15393,15753,57555,27746,56752,-7636,46695,38955,18286,-46076,-28191,-13403,-61289,442,-44716,56221,-101067,-9807,-13615,30960,39858,19374,-35639,-89377,31185,137,-22666,-60930,-32629,-63275,-37066,-87286,-33606,-34125,61343,-17726,-41288,-13127,1559,2362,9235,39692,67907,8663,42588,4069,68979,-12965,-28330,24854,12830,41345,80591,83492,-42979,-21973,-29558,30879,-42927,213,7571,-11175,-14784,-59679,-26947,32939,41710,-27725,17833,32900,101542,-40104,-36364,-25402,-43587,10880,-38393,-49869,1636,-10185,-32972,
    -2174,4326,3887,-20581,-29866,-39676,29573,42549,-34222,35110,-82007,19092,-1055,48392,31755,-25117,43554,-14957,-72915,-61341,-1398,39459,22219,-33109,-63924,-76049,-21054,-52291,36757,39632,-12254,-108298,-144,35070,-39895,-31931,-39436,28159,3449,-85250,-24766,-3073,-36114,35286,-51352,-17729,-28037,23326,31133,-16525,-35003,-45607,1149,-8102,-405,-17610,-16085,9057,3103,25313,62878,-59739,19297,-16385,-100629,52885,-26367,-59983,8520,-46945,-19672,4699,3954,39047,39641,15917,-3681,24206,27302,23546,-19749,-35306,13667,-15850,45757,78830,11580,-19751,-35638,-55191,8811,-8514,-1408,-14954,7492,4890,100974,4538,-12820,52830,9526,10670,26987,39426,2061,-9723,-4048,13706,65433,19708,-2838,-14008,-54307,-32987,10075,-7657,-17375,-40620,30867,65418,-11871,5741,-2224,48439,-6314,-42657,9518,16115,
    -113545,-2394,-37519,-15713,-51049,-16763,49794,3805,4781,-3510,-9226,-23339,12082,9010,20556,46528,4778,-67831,78438,73159,-37023,-60313,15334,48260,13376,-67454,-16253,-1047,-47232,-44678,-40655,32102,-7698,15941,-32029,-45954,19195,-23686,-14859,22582,-9135,17663,-14665,-444,24452,-11509,4636,-56710,53774,-7227,21917,17914,-24802,32805,52495,-111515,56251,46344,-31847,18192,-5405,42372,-14361,25758,21137,82,-60193,-4110,-11947,49972,6455,38016,102229,-35248,-28320,-31626,-1217,-58484,-21513,-5228,33220,16823,26748,22644,65119,34959,45734,21657,-12605,-24564,-5970,-80989,-9467,7871,50583,17419,-1290,-7363,32193,-26560,82507,3689,-14267,65318,-11232,28493,-8707,47215,38034,-57266,14934,68356,26140,-122771,-7476,-30054,8034,4676,92689,13941,54839,-11853,-54203,89856,-22060,-3258,19886,-27371,
    -698,-86092,30135,41510,-30532,48851,-26507,-10228,53564,-6701,53528,16411,4437,-28373,-68073,-30338,-52930,-23585,-40959,48998,5313,-19458,-32559,-8225,-13062,70462,49251,71094,20156,-37555,-954,-3585,11303,-37147,-38021,-5164,-36625,-57008,-27965,15464,-20107,-7526,-4873,52062,-1551,615,-22243,12161,-74114,-4119,55125,24623,-20650,23578,4016,-27220,-74978,11563,29929,8798,-27425,-37226,-37466,-34098,-14107,-15165,10493,29642,-61081,13540,5496,-16742,-16589,-53001,-28627,-6188,-37030,-64935,-55159,13963,61602,-30809,22911,-22073,-28518,49099,-38089,43776,38583,-47521,-44326,-55427,40541,-21397,-15180,16258,12781,-71690,29588,-60368,-28283,-28056,-65715,21027,68675,10898,-31306,-70318,-14061,25473,7998,46178,-67135,27476,-12643,-17375,-27246,-49500,1112,15873,-19096,63157,-37562,-35784,121871,-28094,31976,-42332,
    -12600,-35111,59088,54484,9466,-50977,16889,20964,-48867,-65724,-93407,47089,-22218,-52202,-50036,76835,871,34874,-22759,-46694,-5234,-33973,43971,25659,36185,322,-2415,46759,15720,2334,-11429,-25495,-23847,-22313,-63560,-72726,27197,21722,37261,-78469,25076,-63257,-66692,-22375,-39235,-34459,-23420,-25935,1090,-25730,24967,-23092,-22631,-28972,41155,-62997,21811,23390,-53198,-83275,38303,-2372,-25561,56152,-77351,-46537,9145,11509,24754,4120,-19937,-62443,-3575,20593,46486,-6723,-12011,88633,-17067,54455,-12664,26435,7658,4018,58270,17485,17808,23799,94504,-45088,5161,13364,-29015,75816,-16560,21428,-38183,39467,-31129,35473,75813,30475,-31728,-12398,-7767,2119,16026,-83771,24415,30875,-68547,40945,17988,49259,29577,39167,40973,-8188,2012,68178,3379,43936,25469,-20641,-34690,-53941,-12644,55615,
    -38384,14608,-30398,55957,-35273,11900,2904,35293,-10281,2882,70424,-83872,-41008,44851,22445,-25461,-24915,4445,-9467,61313,-5432,-12918,-30300,7661,-65580,26731,-33499,-23658,-37352,6631,-15882,-20707,-22495,-48541,24574,18680,9997,-30200,-25068,101693,1826,62957,38577,40741,9704,8578,46542,-27746,-5395,25594,-5028,-62389,-9091,-11275,-29457,130921,110082,-48338,53295,-26611,13217,-32186,-16977,32255,19057,35426,15205,34773,-27186,-15369,-6628,49919,73087,21369,17955,-30922,-10072,-9145,62753,38381,6165,-24903,51990,20096,-55254,-76974,-43446,-9464,-17285,-29613,103809,-30583,-23144,-12083,-23018,-79378,-12287,-22670,13892,-45892,65564,-94403,74536,30600,-14245,-28062,-1454,53198,-27614,31367,45649,18649,16012,6372,-19369,-38135,78622,-27064,57475,-37978,-72891,27590,-1690,-47934,47222,36842,-32973,-27795,
    19831,-26386,-1033,20392,55123,42993,13612,-9678,15614,-37320,-12547,16143,-16264,47873,3035,30469,-2136,-22033,-21062,-37116,-30303,-5884,-22784,-40516,-52177,35822,-54252,12640,-48002,-22174,23335,-35279,-52699,-48814,-69505,13850,-23805,37466,62380,13227,26862,-3379,-49281,30088,-39737,-22054,-21208,11249,-56955,1091,45903,-10851,12383,45587,-39814,-57218,53159,-67541,-41639,-30166,-25995,-12699,-100954,-57099,11979,-15348,-54820,-95221,45914,-10684,95849,-24336,-651,-9262,23609,7372,-53264,-12250,-7351,-67301,-39298,-62435,40116,-35286,-52101,-3524,68257,-3612,8289,83271,37296,48489,30172,49611,-49580,4004,2274,7532,-6174,23053,-18024,-53181,-20999,23211,19616,-47852,-70072,33776,24412,-30714,17988,-7572,-37954,12121,61303,79756,45059,20313,-60672,52507,20127,-63500,39531,-30202,-27616,-42161,-34199,-34898,
    -46553,68646,24006,-46105,-7714,-10386,-29160,-20279,-14275,39898,-9918,-37794,-47356,27998,32138,-63564,-50915,101284,69984,18353,50671,-63147,14150,78244,-20276,1429,-35526,33742,-37348,-2368,34789,-35748,-8065,21529,13539,-13367,-83577,89535,-2159,-20180,38574,6434,-14311,2414,-43147,-9521,57523,-758,24091,-54205,-14224,33544,32321,52589,-63317,48887,-43267,-19108,-2089,-86497,-7695,-3888,13444,-9626,-53196,44775,39829,33771,48026,-28097,17529,22796,-13424,2835,-20969,-53398,7220,9110,96072,-32425,-20460,-35852,-23706,68906,-551,1778,-54996,4244,-46866,1501,362,1468,22938,17624,-13473,-7533,397,48296,-51252,31079,53881,-11963,-16135,-20703,-12734,358,-8588,-37098,-23960,-10188,-66344,-15746,-1149,76948,31797,-23079,-61987,4089,17989,-37693,-52393,38415,-14467,11603,40216,47692,-40954,-28981,
    59049,9848,-23767,62933,13627,14534,6312,-54558,38271,3956,-6497,-26140,-44203,2513,-55249,22979,-45250,-51447,48331,-12837,32229,-24760,-56547,-40719,-23585,-65683,22763,-16400,-19085,44258,-31956,47234,41276,-33414,-45965,-33525,-60852,36,29876,38370,2729,45854,7648,-29012,46966,-87491,44279,-5892,41295,62274,27439,35122,-33016,-35201,7310,27683,37889,-44202,21896,-3583,39789,-16254,-30081,-43359,-76494,45595,-14169,-61311,23710,-40813,87496,92076,-39708,-4823,-12183,12656,20090,-51588,-54691,-16340,33187,41693,12019,-10983,22749,63014,4108,9160,34070,87737,-48373,2943,37341,-4856,-11580,2295,-46497,60765,96615,-20109,8662,-15075,72880,-38213,-12396,47278,-9897,25776,46665,15717,10810,-45710,68084,-102224,32405,54330,77270,58355,-6925,-18286,69397,76158,-61801,36672,69298,41699,4724,-52815,
    26491,49867,13772,53089,-45040,63682,5702,-39546,-23338,-13191,92967,-36809,-28121,-7246,50762,-37327,13646,15555,-44483,43742,60727,16566,2438,708,-82806,-29448,13851,-30332,-27098,1471,17156,-5802,-37314,34953,928,-54485,-29305,-46518,53807,-39686,-101102,-36937,82823,16424,-18191,14751,-54265,84574,5567,-6560,-64126,1169,29165,-70602,-32238,58682,2911,2522,-11059,90695,-19044,6128,27495,-49255,-19256,24511,42886,-15657,-37370,1484,-43771,-85232,-10255,28268,29148,-31923,-31541,-10608,-65351,11335,37089,16107,17796,-1562,-35297,-17465,1527,25169,-15523,-10663,34226,14193,-29235,4522,-7503,-1835,18284,10395,2574,25611,-12059,30935,93045,15855,-4307,-22494,-11882,44123,21894,54863,51086,75990,-20500,-34325,-33157,44234,72531,28344,17634,70989,-60516,-26901,-64047,-7305,14059,52874,-14344,-27358,
    -9117,-62116,9827,344,1791,-17257,-9842,-24554,-32176,-9466,-34302,42825,-27378,-8697,44301,1989,-2668,13566,-16302,-34629,-67806,22825,-22232,-17157,-47311,8459,-16751,-80079,54958,5403,-28618,-12843,-23155,-2910,58829,48169,249,11367,-5095,-53782,-9005,57683,-67624,67716,-21858,-16572,9655,-11918,61581,43170,25904,-49436,-21840,-10344,-23822,-22213,71591,50891,-33588,47240,80068,-18957,-49614,10693,1391,-48965,-1001,-45999,998,23818,-10179,4645,43379,22236,2381,75201,764,-86471,36076,54511,-38835,-16930,-4783,20186,59143,-2184,-36086,-17695,26897,6729,60471,-16497,59466,-30365,-10333,45107,44140,-44509,53282,12685,-13401,-62296,-50369,9980,-127694,-29654,20154,38355,7717,33171,-12925,-3446,-61526,-1602,29233,444,-30260,64668,-4523,54201,45158,8121,43822,66103,-19817,-59065,38295,-13107,
    1534,-36380,83409,58698,-18327,3368,50932,22222,-39814,-32109,40304,25966,-29098,-63386,-24490,71432,4574,-57095,-97003,-16542,67861,-18546,-48268,-15615,-11314,53600,11139,-33027,43852,-24280,29174,-29018,12549,29645,-32660,-38114,-23797,-4667,67503,1638,-76694,-17828,20629,8466,50296,-6269,13769,98290,-10651,59792,44106,46475,54476,44815,-19262,6963,77885,68480,17571,-45808,4595,-44785,28236,21153,-6131,-13215,-63469,41977,50422,-18570,-55175,54679,-101250,15344,42429,15903,-5645,27318,39430,-45649,4710,-34470,41851,-28079,-10479,-41722,98393,-7673,47093,70993,-39240,35205,-16648,15206,15109,24234,7702,27047,20692,20405,-1409,17505,4366,23417,14951,19856,-6465,60399,61134,-13293,-40831,44361,1598,16230,15461,16534,82148,59955,-56592,61183,46615,-50228,-64079,3157,-79810,-27439,-50816,46982,
    -72547,24970,-32749,-38609,47019,37664,769,49401,-37761,41189,-19275,1872,-25337,-21520,-33389,25962,-31884,21511,-26807,-29096,-12250,-89306,22360,-9953,-4922,-16259,-18103,37523,-57885,-56203,-15043,55941,-27640,13938,-49958,45565,38451,-54760,-5434,22162,26251,12377,-29503,4816,29126,-29766,-22744,45610,-3062,-37021,-11331,9739,-12612,-19654,-43607,-5842,-49944,33659,44501,-44179,17216,3654,32418,27840,7821,-75279,28831,995,-2923,2246,96357,-31758,1285,-14506,-34777,-31600,-11708,-20496,29997,-29091,34020,43986,-57716,40328,-7825,34877,-37323,12370,44918,-20360,55355,-17965,-38000,43107,35166,-21878,2813,84478,-31815,22880,2334,27000,-42202,-71169,-74468,7079,28862,12386,-24290,-23858,70045,-15957,-23512,-2539,28698,38834,51449,15958,-9785,-64851,-39048,51777,42678,-8230,12406,-20916,57409,44633,
    -18725,37739,49912,7457,-162,36876,-8130,26430,46646,-49627,-37209,1714,-19514,10932,4871,-66729,-27058,-44229,-49596,-27154,-11056,175,-32623,-13991,231,-2701,42906,38923,14149,11894,19898,6184,-51780,-8653,5686,22884,-58876,-37142,-27831,9135,1945,-8113,-2823,58471,68421,2324,11705,21369,-24457,43161,-54024,-18748,16669,-38133,-18674,26728,56648,-14856,-18603,20816,53656,-54371,13122,-15028,-5084,77269,5368,-5117,-35781,4220,6622,-71634,-58610,3667,101782,-46546,-38845,77179,10104,-22044,-50718,25042,24453,23527,-46424,-7418,74783,-83160,16608,18474,14362,36540,-11587,17256,8921,-92069,13745,26819,13052,37303,-20849,-7229,56016,-18290,-22152,14199,11791,24029,22364,65312,-8817,-26605,-50886,-21406,59027,-36584,9946,-1876,-19143,-16242,-1576,36264,-9548,22329,-28843,11664,10589,-20670,
    28837,1817,-17609,-43054,-33877,1962,5371,18686,-61009,-7739,-56474,2952,-7845,-5505,45090,-36712,-37609,112,-7025,-2046,31929,10410,-70601,26358,23673,-39640,-58443,-31391,51301,18268,9327,-25846,-16492,14334,5512,-54047,-39707,-22334,-12241,11083,-18072,-17724,26964,67559,58625,-31479,60661,28608,-53036,5332,-47171,-17907,-5965,-31263,87328,12654,-5888,-59981,-25614,62966,51657,-34018,3189,-20081,-16119,78950,-14676,3097,-14024,11089,-31276,32324,9444,60767,24651,-48400,31559,71854,33278,-44628,-212,1408,45308,43827,63850,-20963,10624,-68483,-31875,73775,9609,81200,-52431,-7198,45692,21350,9785,-26973,58597,-67564,5735,42404,15280,-26997,2439,9604,86070,-58301,26004,67427,33166,-78389,77819,73864,73724,-1269,43034,-52507,-2284,-11418,36951,71237,3300,-29403,5030,-83036,45511,13522,
    4728,36433,-15185,-14342,-27097,-43716,-45644,94807,-80430,-55997,-46970,-55816,17636,-23463,39852,57539,17782,22268,8215,-37103,30779,-19223,-8561,-23183,-48687,-18692,37944,14581,7413,-13796,-21927,-18563,19090,41129,-60251,-59082,-75484,-16717,-17021,-12238,-32361,29516,-26896,-277,23897,6577,5519,38616,31665,-29017,-44937,73700,1825,24122,-15420,2191,11172,3441,20527,16516,-30769,-9305,70461,36892,-19203,-6883,11704,44561,75836,27402,11377,21109,-44508,-2833,16521,30090,-15410,-32474,-45743,23585,9905,9551,4060,37369,112172,-22279,57550,25537,102257,2006,-1311,-18694,-13131,-63293,3187,14412,50519,57863,37723,-48302,23960,52928,2889,32042,-20170,41905,60964,36251,33238,-22494,6623,7462,-90790,36,94162,4745,19728,-40384,6406,6578,55430,-36401,17833,44097,33202,-80445,-41833,-8939,
    54310,50611,-57013,63399,-50269,41146,34052,-39344,6926,22429,-49725,36248,15799,4989,56275,-102612,38801,-45429,16684,4553,70501,53633,-50192,-46289,21359,71543,-46375,49532,25052,4140,-18527,-64925,6865,-24936,-37744,-5889,-35873,31543,-36534,1761,15677,-105016,59194,18220,2399,-5059,-8723,10823,-24780,-31809,-27165,-41253,-30352,-54649,48550,-6015,-76141,53400,-59851,367,47568,-23412,-12892,-62487,-28799,50659,-51409,-23980,78366,79584,-35484,16153,4605,43591,56802,-14322,-42122,39337,-24511,-53578,-50380,-14382,-89311,-15929,-46392,99724,59819,26860,-72613,109878,-108604,42139,58053,38978,-8525,20954,-13830,-20487,3570,8930,-76192,104859,56829,-10489,-28363,62906,-4397,-36588,81393,16539,-14405,10774,910,13680,5627,-47978,-14146,-84122,22139,-33335,-5105,-31431,-37564,52261,-39866,-4000,6756,12953,
    -11382,19510,-25901,26048,77510,8381,60408,-57816,-23437,-75834,-61098,25015,-14261,7544,2231,154,26806,-9701,-7646,-45984,9112,15895,26132,17289,-16876,375,-32930,19851,14478,-11513,-55560,-10570,22260,19110,-24348,-46162,-38861,419,11389,-77720,33163,-39,16020,-31258,-43186,49487,-17758,19245,15933,9373,-40369,-32702,-5274,12411,-14951,11968,35923,-29463,-10414,25842,45553,29796,49350,-11944,-36038,58089,22715,-55699,20550,-19571,78657,-38611,-348,54645,26346,13839,-32558,-89406,-22356,13327,79687,-41939,69226,-3777,-35891,-10681,66476,-24983,-31333,-62018,-10610,-32292,44528,42154,-64250,-12504,-64375,-21676,31353,-18043,23423,-27482,68101,51307,-11349,1819,1694,22519,34388,-33299,-62054,-37543,-51673,21812,50666,90331,-41284,-33007,-21125,81469,-17277,22706,-65949,-23749,-12429,17948,-17411,-26694,
    -38542,-52457,22935,-37733,79789,-17728,-27225,9913,26774,-61966,57133,-90284,23466,82437,-33853,287,83271,11812,85570,59365,-70298,-59497,-6896,1990,19683,41039,49706,16265,-7834,-50408,23402,-48684,25157,-50755,-5890,-46612,-72209,103290,-62622,49965,64566,-1812,2747,-1129,41495,50752,6727,-24341,23050,31059,49924,54113,-12779,49889,-50895,4230,-32561,-114360,-8807,30157,17136,-41466,-55569,15185,-7032,33955,31566,-6020,56138,15560,2267,64394,24594,-23496,1409,-22843,188,41768,17293,38950,-5041,29963,14491,-5730,-28199,-23212,-22264,-26792,42654,-556,-14007,-21710,86787,-61998,-10526,-37079,-53532,74347,27130,-4076,65432,-17129,-11497,47274,-34145,10104,-12984,-3836,18524,-1619,-32000,32785,-63363,1282,25490,-4730,-7311,-9988,95199,-56422,66923,-6208,26634,-88385,51402,-6540,-58469,30128,
    -33285,-4589,15445,22200,11726,-47874,8396,3302,-6817,25991,-56703,38657,68684,-14201,-11753,25757,15762,-49713,49450,-42967,-17658,57147,35646,-1978,-999,-23103,-14342,-15761,2878,29790,-21484,-21831,-2422,-23288,-50822,-8278,78571,36191,-16339,-29643,56760,69469,15234,4946,-56465,-61243,-19307,-10805,88165,-38338,-28425,27202,14802,7014,-14527,-8775,-19080,-44186,6909,-52216,-12737,35649,-2291,23726,-12785,23719,-49864,48058,-38675,-65526,26622,43028,31406,26704,8783,80456,-14343,-21269,-52844,-2604,24839,54758,19966,-9317,-40704,49876,-28529,-58854,-56641,20883,48439,-73336,49150,-12368,-30889,68717,-34857,6249,-9703,7848,-44576,-17431,28113,60571,21612,46352,-8930,-19284,-2893,26146,39436,-1114,9208,-32784,-62180,12889,-27333,-68796,1417,939,13692,35631,20868,-16585,1262,60275,15810,37924,
    -34386,27118,-42365,-54037,15548,-15788,-54398,8791,-32316,26837,-37436,18272,-26615,15810,-17057,29739,42309,21954,7059,-31427,-9455,-10570,-41165,31325,1126,40720,-53634,-25604,86556,22865,-7026,-24200,-55926,46863,-22759,-79199,27464,-10301,7703,-18804,47164,50229,-59546,-36475,7943,-3511,54,939,31526,-15498,25795,79418,6331,44919,-66939,7849,-82501,-101114,-56775,-33611,11,-5437,-8048,24430,-17014,-59213,11745,72734,12971,-48158,14168,48809,22127,52362,-55024,-84276,-42984,-90487,-2863,99666,60983,17982,-29852,-1961,39828,-26770,30499,-49901,-18376,-5352,26487,-3768,49585,8563,-37492,-12703,-11977,-24266,34457,-47044,70426,-2524,-87547,3904,61253,-7325,-32634,4542,-18221,-4655,-13012,20541,-14029,21809,61030,40745,-539,-29412,-22447,13792,39095,-21675,41003,-86129,19774,-30993,-8878,-70633,
    -6196,-46584,-22361,-15949,-74650,12686,-55376,34928,-40684,9265,-29534,29115,-66300,1902,12772,-3982,54703,-2611,54272,-47306,-3666,-3521,-5894,9013,68738,-23165,19685,-19239,-1182,9954,32415,-4134,23769,-36594,15208,40041,9728,47585,-57504,-68778,-24148,60865,-15572,-42554,-61014,44087,11351,-22475,68539,55613,23287,-5629,-32803,19900,-80627,-34327,-65096,69904,-30326,357,49945,5283,13339,31073,16534,-75579,26976,31908,-33105,-20002,-10276,22061,-10566,-40532,-31505,52000,-1114,-44014,-11882,17764,9173,-8848,-2058,12252,55284,-1781,15191,57251,-8630,-29985,2532,-60924,71123,-24814,80023,33955,-63365,-43282,26748,-8801,-72354,78233,-10190,57945,-16310,-22820,-7871,-57465,33593,-4510,25302,65820,-9855,-25580,-71258,6106,-25717,-41261,-49010,-36540,61262,-37671,-25738,7434,-4024,-8934,-27471,-33704,
    26891,18123,82085,11554,13892,-11641,73786,-9525,5198,46936,25530,39083,-24511,-36069,-37655,46848,-97672,-66612,33155,9150,-14280,21639,-45452,19128,44694,45509,764,-36546,24317,15166,-5230,12610,1089,-64734,-44393,16753,2355,-57715,-12889,36026,-18473,-55509,-43644,59904,-78302,41081,-71119,-44694,-46475,-50696,38702,4058,45816,52874,71940,-43595,52670,47777,4676,-31813,-3549,88601,-47276,-25158,-5989,-5853,34546,1826,13418,70216,-20304,9448,-61152,-21072,47244,11742,4251,-37460,141,-43414,-34300,-9672,47272,59339,24891,-41749,-18307,-43911,18001,27827,-31993,-41138,-64106,22807,27013,-7332,51171,-28705,45660,26145,36782,-31586,-10166,30257,85051,-91175,36223,-11439,9146,-85565,-35623,32299,54308,25583,2208,2503,5210,16361,-50511,42380,4480,-37116,71751,37760,8237,43080,21400,30086,
    -38210,-45055,-22364,-20266,62781,20104,93707,18505,-55006,-2039,-33239,121846,19163,-32193,-4744,73070,-16727,-55792,38201,43875,55801,-54617,65189,18886,42502,33719,29972,22846,-41307,11547,-35007,10761,29494,-30846,-65820,-949,30208,-30421,-34608,-37648,361,-10525,36594,-15238,-37311,-38198,-17890,52300,-64242,-44514,-51320,-41379,39652,-3807,-15936,-8416,-12489,-20372,42693,-25204,-8061,-14146,-40392,5661,-43670,10478,5016,-8529,-7690,-77666,39484,561,2758,-46662,-40681,1353,-40345,-63546,-9624,10942,-49231,65037,-12344,-64579,24872,-1432,33243,16860,35247,-29628,706,11507,-67608,41711,15048,-73656,5564,-19253,-91410,-66922,-58488,-37776,31507,-53566,37940,16259,71153,31690,25808,59073,29542,-14240,-11892,-72279,-75872,-5417,8043,-7889,-7191,43372,-25794,15829,-61910,5307,54071,15301,-19929,8502,
    -21205,-31786,15648,9384,6726,14753,-27419,-14365,-4714,-80750,12986,85599,-61296,-89925,-6130,-9466,6518,36768,-52940,-47894,-1068,26317,-11586,-32999,40116,-31480,-24948,-17889,48820,-146,46246,-10656,18453,-35456,19533,-23412,60561,5539,24614,-36042,-7948,16855,7298,45515,-44148,14337,-28101,20639,-34513,-3023,-45705,-42109,-19872,-67249,-6651,53866,-40983,1663,29670,104991,-6664,-15082,29376,-37837,5827,48890,56468,26728,27215,-31663,51298,-111310,-11839,65037,10428,2587,-78851,28202,32783,20330,-28635,57195,4279,-44770,35170,-108298,22317,7021,20093,7756,-27467,-1279,-43763,-38527,37998,-116621,54483,-65701,22485,-1245,-32183,-8544,19222,-20998,-34904,-380,13942,47212,41566,50432,-28071,42653,42985,-66152,75775,108063,53280,-11119,-39301,34096,-28738,34061,-19038,-104526,15988,49891,-19427,584,
    -2330,29478,-18334,-16875,30351,55881,-32296,-80461,45325,-2957,-66767,53733,32706,34374,9692,30146,91350,23447,-79369,6278,3808,19707,-63074,23789,-58466,89901,3957,19719,56253,21642,438,-99604,11351,2119,-14211,51407,27722,25109,-12298,-25651,34330,46469,-89268,41702,23226,12477,-51938,-56725,-22938,16449,-15404,-4442,31711,17828,-8168,-14098,8974,-74342,60363,-29213,-11988,-65447,10184,36609,12054,-28573,-15305,61342,-29559,8890,-43590,85134,82009,-55861,22054,-30729,-20881,-42233,-39305,42745,-45883,30105,-40898,13402,34642,15271,17420,-70599,51971,-25277,12244,23026,34201,5630,-12591,-87907,-19559,-13364,-42028,-21521,-47104,22719,-24297,20784,14615,26000,-22791,11146,-51379,23597,32169,26575,-34243,4347,30294,-11332,39889,-19654,-9717,-25500,-34693,-50614,48814,-11524,3057,16144,-49978,53900,
    -32350,32204,-9516,-33694,14013,63273,3563,42526,12199,22886,-38243,-8298,56953,-21053,-22920,7527,7103,2271,64466,-14700,-36101,-17296,34506,83637,80593,-11225,2814,1067,16767,67649,-14065,40726,5599,-18330,12628,-40510,-49962,-8153,-54626,-18251,19230,29977,-13136,7127,-18095,50818,15175,19053,-18152,-38412,-6832,25246,32561,-54021,-15487,28064,-12319,-36958,-28048,65229,294,60190,12201,40988,-11761,-14962,-5088,26380,-71847,-45409,36384,-46478,9999,3612,23571,-8513,-66045,18355,-36720,-29402,-13963,53087,-2285,71843,4038,-62215,-8629,-19525,-617,35122,19329,54046,-1358,4506,51104,26283,-25471,-9990,38896,-41681,-6948,-16874,7838,-31713,-9091,51730,68799,-54089,-31523,6202,47022,10567,-32630,22145,32588,33624,14981,37590,84553,2686,30945,-15788,9138,-50161,77559,55347,-39298,46072,
    -35913,-13175,48245,-9977,-9745,-16386,-101518,92745,-94978,-14193,-11648,-42680,8904,45903,52954,-18573,24248,-17447,-24229,22693,-60791,44534,19081,26818,35705,-8632,41229,1271,3655,6798,4265,35824,11215,-1590,29031,-22456,-32096,-51373,-78920,-32846,-51600,18508,35663,66705,31913,102818,45530,24458,11105,74912,5092,32389,-467,43232,-10141,-10383,39836,-56006,24400,6708,29170,15584,51881,9088,16054,-9925,51236,48540,-27305,24244,27539,-18114,-32180,11678,3726,9523,76946,-20099,39603,-35300,17884,8138,6522,55040,1957,-59921,52849,13217,49649,-45094,83064,-13969,-27953,32266,91212,22087,-17496,-36549,-27162,12577,-956,33172,-21012,-10765,16207,33446,-59200,56553,-42577,48183,-5500,22264,-21475,-52913,46388,7320,-3491,29711,-35384,-8044,61673,-43512,-10610,57562,28259,44369,-28625,39795,
    -80451,-61746,39552,-18137,17278,-32509,-81672,-45025,-2403,58600,36085,5856,-68931,39524,-66893,36185,-18996,-22023,-12456,86151,-2573,-24230,3940,42144,13940,-45011,-42093,-11256,-69082,11431,-41278,441,-17965,22464,65779,-160,-13167,14776,-24078,-17649,52031,31351,67714,34837,-17782,26683,5059,6040,54681,15111,-58113,8115,-20820,-47488,-9945,-54669,15267,34938,-59555,-45164,-49403,34638,-29446,-4737,67625,-99190,-10249,43418,15352,-65033,-68283,43345,-6254,-42994,15897,-14658,-7733,17481,-38004,-40469,-74570,50530,-30228,18960,-36246,-77526,51860,-52639,68215,-30004,44875,11632,24314,15074,49435,26248,-67618,117,-33114,6611,12094,-59548,-16933,-4845,-22357,21939,21506,36650,10577,42701,63709,14303,35963,-67194,-43297,-4274,-10185,1109,34020,-25720,75671,-13942,29505,26798,-51074,-49539,-5258,-504,
    64434,-829,14165,68996,20571,-53772,-28547,-48186,75023,61150,-12179,29948,-5125,-32970,48364,-56208,-73676,65949,-40016,10681,19671,12114,5552,30330,-13927,-29382,47210,24862,-9434,-26553,31476,1112,-32473,-478,19443,67411,14330,-2662,2131,-50585,38486,-14808,-53101,51697,-7035,23825,7812,-32149,-32184,79236,-22732,-39285,11157,-25666,30149,-8437,52934,13989,1313,26884,-49263,-34318,49349,-21377,-11028,67563,-9154,45669,-8282,-804,-17632,-9982,-85252,-30188,-17062,-45094,28352,-59490,32451,5350,-2239,-30484,-106810,42379,-55924,-23663,54713,-44935,6829,31320,47528,10989,47036,34597,-14459,34288,-46819,-21027,-46535,-36648,-28678,-14156,45230,-34924,43839,-5407,-24725,-42734,-46557,3697,-5879,-35751,29334,-959,-79594,-51589,-12115,13093,-54694,-16830,18058,84065,-71868,60150,14527,58070,-18781,68827,
    15155,-77514,-18313,27007,-25510,-25898,-85279,-5733,31661,-60328,17695,-318,-40638,45389,17519,16483,-15640,59003,-14116,19022,57289,51518,-69746,-10836,2599,2285,-22010,-12207,-17723,-74016,-41959,40484,-203,-35603,-19942,-39440,27046,-96283,-8168,10682,-51696,61461,-16275,39723,33693,47906,8501,2071,19296,83234,22356,-1307,19797,-8763,36627,-24405,48626,1133,34686,-40810,-53261,14825,11976,-106766,47525,-9990,21207,25426,-34094,-36668,-56858,-38779,77361,-24785,-41483,10452,-17500,43359,1691,-26922,46932,-57755,14056,26345,-69369,-71309,-5452,-28928,30811,50869,101536,6050,-50744,-4462,-17218,-1993,4568,21132,-7764,-43827,94612,-49066,109538,43589,-23920,22969,43253,84455,34619,-20049,17607,-3974,-12176,-16274,-40687,2075,1861,24628,-13848,10036,9730,-30582,95865,7423,51246,106957,23162,-4659,
    -50708,49989,-18306,-38103,20029,-14851,-11963,50700,-18475,-16187,-5572,-31662,33109,-7038,11396,-44879,91550,-37798,-45226,-33027,20991,25469,6008,-33485,-46834,-31545,-48342,-27587,-13240,20702,58417,-7626,2609,18973,-22220,13411,15144,-7018,41534,-18130,-46972,18776,4427,-9657,-8274,56666,47492,19084,39746,18755,-37188,31892,630,-20560,41289,-8359,15543,-36118,-16094,30446,-13938,-36160,46221,-85482,-74106,-20762,16130,6678,-771,-9793,-19777,-14038,22398,-20921,-8985,70666,-5423,-16233,-18398,18973,22441,-50194,-2914,-15867,4073,15198,52199,9026,21076,-54290,29277,35743,-29303,24703,65123,-38213,-33182,3761,-5382,42751,13853,21531,53115,-50985,16470,33800,40157,106496,43464,-5910,16528,4094,3277,-44458,13479,-41102,-4079,2606,-12623,-13285,-19802,-30340,44208,48756,59101,-17403,23420,28584,
    75902,-21169,-43400,-37619,57346,1351,-9235,-45529,23568,-81882,19571,29593,56254,-42280,-1075,-30881,-42528,-30643,-43407,-12614,-12568,-45893,12439,-32297,25379,-74703,-30773,75814,-11127,-29479,-42422,45206,-58476,52840,-26253,64020,-8308,-17248,-23130,2766,59096,36386,-49178,67442,7974,16704,11024,25754,-70475,5181,3517,-34594,1166,44594,-48788,-19243,-23045,-27670,-65738,50147,30143,42061,-37842,72810,-21048,15612,74371,14879,-77254,50889,17631,1222,-52279,17978,-58152,-42403,42615,12847,7466,-69497,52986,89246,-59626,4760,32552,-33079,-52530,-58248,-32045,-4474,-23234,-73361,37055,-4363,-37721,-5497,11680,-84509,56224,-8634,-1084,21870,-51057,-79944,-15850,54338,83592,40841,9252,54696,34273,-22012,-50390,21793,67802,-26483,4589,-85860,15807,42717,-25338,19723,-36492,-1885,-60292,-73296,45643,16638,
    12826,-103336,-20553,61696,24321,10668,-22503,-32827,9586,-42383,-33531,-16519,-3073,-67658,42037,-60239,-6200,55262,-71958,-34420,78510,7064,42491,6932,8656,36904,-24543,-13212,69854,-7297,7498,20267,33991,15681,23709,-39231,30518,-20897,44227,-54393,22473,54348,-45113,-82655,29390,36629,6952,8301,-34737,22946,2090,-25094,-11049,-27350,24401,29300,16129,464,40918,7156,-40139,43722,2026,-6281,11281,-68967,-45556,2089,-9755,-4120,-20879,-31250,4955,23304,-63800,-5845,-7556,26670,-6357,-13879,33076,-77650,-47641,-105645,-18775,-24581,-88646,-15577,-87977,23528,-6408,15324,-39648,-69271,-32445,5133,-11239,7851,65741,-91959,17736,-5335,-40586,-10647,-117229,31146,49113,-14858,-50636,-8630,-12816,358,-9833,16833,-13364,2570,34271,-14463,43658,17420,-5436,51340,-9936,-37903,3151,-49934,8893,1107,
    24509,36961,-56222,4895,21696,-42064,31402,-15264,-21373,-2206,27182,-20565,-2811,-21043,-10227,-66218,-56292,-41216,9802,-5967,-21306,205,60877,2629,-5641,-51859,63488,-61491,26510,60733,19052,8422,-18025,-9360,12359,3784,18044,-17046,59481,41389,-1375,12399,-33464,-15588,-51309,2551,38058,52631,-53206,-50632,45794,-54433,26172,10174,-11166,69083,16546,26099,4911,-13439,-48549,34453,-7970,-5010,52713,6441,-8310,-1094,34442,55854,1984,-46783,-14656,44051,28782,-7576,-43481,-4858,-2296,44967,-74761,354,4229,26423,11963,-54276,7916,-9623,-32656,18958,-48124,-29952,-34842,30446,6747,9010,14750,-75876,18554,-2179,64097,-40286,39239,-47009,14482,-10424,-14447,67639,17761,75651,-18949,527,-797,42138,17151,-52896,-30971,-2978,-85476,9113,-38351,-84339,18717,37140,11233,461,-45394,-19157,
    25953,-42937,-5772,-12932,-10385,11083,-4986,12597,-113,-21359,-109228,-17043,10688,-2046,11493,21471,-2175,41457,-69260,62454,31367,15857,-27745,-20406,-23541,35911,7143,-17977,9523,-16229,19786,51202,1634,51,37840,-32314,-2339,-46368,-85974,-11743,-28662,29590,-22270,-44571,53692,-30324,37817,34454,30306,18005,22004,-9230,-77336,-22238,39338,60086,14108,-76108,-33147,21090,87253,22583,-2283,-22352,-59699,-57081,40218,23469,-40995,-106315,-6792,-16964,63424,35584,-26394,-45178,12465,-52522,19457,30466,52297,31297,20939,13331,38361,988,-6065,21786,-13719,6436,1032,50790,26754,-45679,-21897,34415,-66712,85705,6444,-58230,-73244,50507,-29147,-14259,23261,52486,87275,-7001,-4262,12935,87802,86677,-47011,-25815,-3027,9333,25525,39126,34804,-23358,86051,47460,-19693,-47889,16864,13875,9006,13639,
    -12576,17324,-22500,41510,25789,-63461,-24438,-23569,55491,47473,-38157,21687,-1778,50028,-73703,17046,-8587,-36361,6138,28290,-5492,213,16386,11776,77818,25753,-71068,-14141,-43658,57466,-25967,-1434,-50220,18455,66978,2821,-25917,-44235,-50300,-17447,77245,34999,16620,-40692,12630,42272,-25720,70968,-9315,49090,8135,77581,-29269,8231,-21279,43647,15881,46829,20519,-29358,32714,30155,27773,-63769,-39435,-54104,18365,-18035,-93145,43109,30251,36190,-18457,-34083,-84788,11719,280,22149,5643,8675,39984,8569,12091,8425,-67131,10643,-28299,17679,4429,5174,54162,46181,59930,5884,56843,19483,-8429,-42483,-17728,-2057,-28223,-65099,-62114,-77265,25650,-3771,-57752,11341,-14069,-35548,90424,-76591,-21262,31113,-40787,-12102,-59540,-17214,-33472,-20149,-18873,-8415,34456,4994,-8249,-22280,32309,51771,
    49483,14342,7690,31778,17766,20799,25228,-43149,-56047,-26163,1372,-40548,-18437,-2065,62834,10053,-70002,-24298,54789,-21307,-5427,-30630,7236,-39678,-69271,11055,-15618,56747,-46148,-55182,-41170,44558,-20598,-14963,-78151,44802,-60097,34124,45882,-19527,-33028,-13920,-21486,64263,-37356,-51507,26496,80765,-20983,-19926,40786,-20477,1688,72370,-16515,16394,-1217,-29904,-30549,17646,16619,10857,-17846,5291,-35332,35275,14490,28641,31222,-9405,-17515,2641,5355,7763,79098,36414,39551,-70796,-11926,69621,119965,-54961,26058,-20526,4,-12117,-24548,30138,27464,50205,-8889,19626,-83118,44455,-6144,-54924,-6434,28610,80337,72707,820,39620,-44338,-28408,-393,26330,-34811,12568,35051,-40596,-23715,15693,-2052,-16152,43946,-5115,46314,67497,-28770,24142,-33041,39315,64812,52501,-39002,40189,-24436,-2028,
    -45975,-39534,16631,49857,3975,-5276,-79634,-59474,-32364,56653,-3843,-41617,-48181,38562,31692,19477,-24559,15052,5252,-17486,-27128,-20757,-24563,-11003,-67411,-36864,-13079,-32807,-9060,-12314,-7404,-7228,-46710,-10157,15782,7331,17493,39000,-19158,-44307,-14379,-3316,13686,64367,-61208,-37618,-19345,2938,-18098,3600,-89478,-57501,-66182,-17222,35792,-8230,28992,41504,-41111,-16763,4019,-35333,-27974,27186,-38196,-58247,41522,-103890,14745,-64067,34390,70954,11889,-23789,54244,-13889,10582,-6947,66416,-5612,234,-14329,19505,-30213,62294,29357,18505,-37661,-6123,-10206,-7661,84986,2668,13345,35349,7501,-31549,-56978,-7822,-44028,2088,-40250,60913,-5255,-45710,-66077,17056,-43117,-151,77364,9400,-79158,19582,-22419,41328,-60693,-51042,12203,82537,-59806,-16107,18781,75292,-46735,23111,-43423,995,-21977,
    8888,12702,72882,14729,-675,-23815,-12084,-66534,4819,-32500,-92209,25743,16310,31057,-9274,18196,11815,30441,-76755,13119,21608,21508,21866,28177,-7444,-19840,70419,41891,47337,-18706,-46746,-50078,-39852,27620,22735,23866,8389,46965,-12202,3505,56123,-35038,-22248,-31855,15034,-35154,-13884,-46061,19711,-21914,-13496,-10075,-45286,44319,3741,-36035,72743,55232,-7826,729,4027,-33220,-21790,19176,49364,38698,-18122,3031,21150,64383,20820,47905,-27938,-50467,5525,11648,58919,10062,9809,76143,-22061,31469,40514,-3817,34675,-22093,-2219,-6634,50049,-48333,31158,-16113,30834,21529,-43086,-67206,-52338,27630,-31253,-21513,27656,-46783,27896,45702,-396,10246,-57957,28660,34514,-73400,-23477,-38226,2320,6101,-84841,-13929,-73553,-48147,-14442,7692,9488,-79974,-26671,22649,-23926,-4370,41075,51082,
    31003,-707,43923,31008,-60837,-43589,-43328,-90794,-40160,-32871,68309,-51580,16870,-50481,-3865,8197,37666,81310,9724,93995,19435,37737,10097,-42165,-12284,33620,9921,-9305,1950,-56571,-41866,-33986,-71585,-91018,7800,17495,87285,45367,61141,19426,-99100,55271,-18212,-62974,21406,38137,-60562,11568,19260,16089,24878,27884,30885,-71797,8247,-23109,24545,21320,-24443,-48632,-20084,20387,-29595,5807,29009,-22335,-13832,-7735,-2562,11136,-105082,-40796,26221,22455,3551,-71172,33684,36489,-53548,-25514,-26441,-63536,-118092,52529,16682,-14041,23732,45334,-9683,-3442,-72853,-59690,-44849,54246,-65722,54390,68113,11902,-90107,-14425,-27709,7802,18897,782,918,41458,-38321,17023,-66105,80720,22524,40607,10241,21532,-10369,13232,14754,53968,-848,-13557,-48425,25153,-15853,28403,-2588,15712,10839,12697,
    -40355,13016,-12430,-57316,44963,29714,16614,-28514,-872,-19044,-14671,28948,1413,40074,-1506,48810,9440,16614,107556,51515,-9572,15116,1659,-12836,12452,-58016,32174,-12046,-64437,-125364,-10905,52068,41551,-60534,38087,86106,68437,75010,10187,-19856,-77955,25978,26260,7478,23604,-12748,44881,12787,24565,1002,3575,-9100,29268,-23377,7769,-73432,22954,25341,-12513,-4450,26336,35542,24991,-2957,33182,-44386,-1400,-65625,27715,18427,46965,-36547,3986,-32716,-56529,13300,77924,-1711,31595,-45523,-50590,23237,-49407,-71321,-10490,38670,-26747,60378,-13741,51131,7790,-51409,-32945,33341,-31241,8492,59940,13850,-28261,51037,14537,-43206,-16615,-33341,-59647,-68078,8759,-50696,-59717,-7144,24882,2090,6483,-18578,-31712,-42599,13164,24985,47732,-119480,2063,-3620,45814,41871,-21214,22741,20830,-14471,
    78833,14142,-8807,-35434,-18958,10045,39630,39061,42461,59939,-22966,17191,23548,-59538,-50108,30615,-20180,7296,24504,35681,-14359,62948,-44105,5462,523,-16729,36154,-33568,9079,48653,17760,9306,-6174,-12561,-47808,-26920,-8767,5597,35263,53786,-27202,-17396,50128,37239,-14164,-959,62712,-22295,-31517,-28724,-14557,-55276,12859,-31995,32590,55268,-57438,-56943,3602,-13182,102666,-35370,-25118,-30792,12877,58173,-9784,28863,-43937,-2435,20385,14275,19772,34745,65888,-23155,8001,28148,-6422,34391,2507,103763,71455,-14244,-14432,47726,14697,27014,-68633,14756,-50161,-27604,-2113,-13065,10515,-35959,13360,-64134,15815,-13961,-31599,-23682,45152,43470,53232,18330,49392,-55410,11766,68658,-83791,98764,-26724,13777,21777,-10934,42946,10610,-38677,8836,6880,-19364,-52935,-4448,56424,52451,-3909,-22938,
    47679,-14020,95215,-12158,-46230,-98239,63323,5733,38087,-1914,-59776,45611,-6464,-2880,-8717,36553,24140,-12035,-96082,13367,-3972,-82150,30271,-53342,38672,-9976,-21563,30716,-24332,9147,-75241,-6360,65540,-49285,6628,-12031,28213,-21301,-53534,14830,40468,-20501,-30649,-2693,-13434,-2033,6626,16658,65230,28498,10885,60318,2844,-49360,39020,-11735,44474,-5126,76443,-56186,4267,23107,8610,36902,9873,-52159,-26835,-39905,-12376,16888,-5241,-14021,-95595,14692,-57787,-16286,39219,-45971,14389,17399,21534,11216,-60059,-25792,36081,-11194,69012,-11716,28917,-46376,-7649,52620,50074,-50076,5349,22187,-6089,-53448,10233,-18145,-47954,-5001,-42055,-93780,99003,42566,-25349,-57605,-12501,20475,17407,-18498,55940,4942,-15424,2436,57163,-37320,49158,1710,-12536,-35380,-26520,28489,65203,46017,-3738,-22086,
    -25504,-45234,-84116,-64773,51005,6465,2303,65399,23192,-24285,1104,-23767,-16985,31052,-38304,-42385,-6718,-55009,38438,-4808,-19366,12764,-69273,21016,-18831,-62738,-7045,14460,-54262,44009,-35902,35093,32976,-60368,15637,-17017,18772,19142,47453,-32178,8573,21161,48601,-49557,-52384,-144406,-41459,-3364,52514,13581,23488,8006,-4201,24541,-12257,39945,-17980,-24189,-50423,-22660,13372,-35840,77242,40708,49775,7328,1437,-19894,-8527,-66431,30831,46077,-23150,-15123,44995,-25825,1350,-59653,-28541,12263,52612,79579,50455,-10533,62129,-8203,-21295,28020,-49210,4116,-29769,-10861,-2530,-48761,-9278,-36000,1302,-37784,12035,-12772,17091,-9130,54463,15993,60350,-10528,-4932,-14052,56044,-39349,33369,-75245,14567,-40576,-15637,33694,9708,-31420,-4635,-32512,-18887,43149,37465,-587,3860,-31195,-41915,-135639,
    30912,-18982,-59503,7309,17687,-13090,104979,-31257,-53314,83160,-1710,37368,-17450,1810,74981,39648,5249,6339,14463,10105,-15827,-18219,-33472,-68182,76372,1426,-6961,8718,-29825,-26597,-60605,40286,-32809,27980,10770,17686,70739,-28875,67948,-30493,-34426,-10243,62722,37748,-22307,-25454,-44200,-9907,-39882,30175,15923,-61458,-17354,-82178,66322,5749,-74479,-9960,-12759,-44911,-48059,62474,-69733,47297,-4460,-39648,-39346,-1750,68099,-30673,-6809,-43258,25396,25044,725,4825,-5882,35881,51905,61564,7370,792,-20490,-53689,-34692,-31900,-39205,-31017,-46910,-5618,-11819,41678,19704,21267,-13906,25400,50292,-25757,-37178,17485,-33076,-72758,15022,23474,-61,-22675,27123,-34921,3189,-3782,-5554,59627,64297,34519,3287,48353,48915,-39956,44128,18114,7362,45167,15635,12289,15644,112920,52286,-7569,
    14749,-15788,-64758,57831,-70302,-19826,-108354,34916,23128,15408,17855,-30281,-9270,-17153,-13436,-7723,-22021,46038,8689,39174,9237,55697,9215,-46091,-75657,29891,-42426,17899,-127950,59457,22519,18911,68987,-5272,-6294,-85021,-52764,-4895,-25927,45418,39031,5257,68729,-34810,-51875,-46505,80042,23238,52835,-20551,25176,97558,36081,-32521,20454,2577,-1205,-5195,-92980,-78872,-9158,-35774,-15315,-27963,2442,-53903,51680,-41176,-90187,-39880,3289,50022,-22003,22458,10524,50341,14563,-13777,-7205,27679,-10809,-60858,62167,11253,-23897,56962,-52915,-22309,-44486,35812,114807,51268,74663,8282,-55365,11992,-55605,-36775,21895,17631,-6272,5562,9729,-77911,8875,-36213,-26000,62427,-45020,10505,-11229,-30499,-18984,11342,-32378,-15244,-22122,22925,-63898,25995,47564,-5336,-35795,47902,68435,-61407,4180,-25197,
    -73543,41909,-49516,12421,2966,-27014,5180,-37556,7232,-15380,-30744,16970,50119,-64517,6643,13496,42309,5611,-23137,-17522,-26698,-13058,54613,38220,-66094,-5805,31466,-48750,-104076,25997,97424,-37255,-11733,-24869,56910,7256,48431,30108,-22671,-10471,20944,-56005,2393,67870,-12850,-13845,26487,7842,-2086,-15708,-20446,5256,5494,997,-40680,-1932,5388,-30916,3850,53866,-37052,-2459,40190,18261,-17778,3932,-15303,-29707,66386,-58151,-40638,19362,36168,26538,-1513,4154,-3018,3374,11757,-43204,-56005,9610,-22818,23160,5057,15140,5651,77084,15135,11120,-14955,-10473,7134,-38305,23678,-12943,19273,4705,-26559,67415,-39803,-35758,-46861,-38974,-23061,857,4549,-22266,64665,-2341,49817,78606,-63191,-25405,-12238,-15958,-34309,22136,69308,-16685,432,-35107,82145,32407,-14599,-38843,31828,-584,
    -36159,117518,34690,-52742,-5266,41255,-26992,46812,50149,30253,33280,-3319,85786,47257,32789,30808,25893,-19149,-58923,-48968,-13361,-4979,-28081,70550,-10693,-4509,68423,-35939,-13914,-9122,-64013,37437,35480,-16049,-18970,107800,-78232,21375,-13951,47854,-60223,-60445,48491,-12969,27737,-12848,23276,13193,21284,25096,3144,22331,79199,36477,-19549,-14142,69952,-18065,-48199,-20096,-38730,-39375,60117,10351,46368,-48303,-7980,-6597,-14058,-28877,6945,9680,-16913,-48475,-6529,28323,35939,52499,44253,-52126,-6289,34503,91390,-14407,-28884,-21324,6495,44753,14383,-25595,12233,-13661,-36938,-42418,-8334,-91196,43115,823,-44523,-3686,6386,-39258,15798,-22018,-5634,-31866,11822,37297,4700,7745,47203,8463,8216,1113,-37476,-50956,10662,44903,8950,16055,-16933,-56439,-20529,-5188,1082,47627,23192,4472,
    40167,-41639,-25471,62456,32263,-52457,-272,26077,59953,22601,-34371,30616,24350,-21179,12304,-32921,-5990,-8709,30986,61695,-65742,62839,15613,-19292,9480,-4256,13393,-30969,-55150,76894,-12557,46385,47898,29922,21639,-33255,16174,21162,-58860,-10000,51522,-4136,-11785,-21439,-60983,5944,69176,-37760,12572,-2833,-22257,-54302,28556,-6805,24068,-28036,-13181,20085,23613,18650,-28816,58902,3364,14723,-8942,-69692,-90593,11244,23524,20013,19329,4553,24232,-53799,27794,-18883,-7627,-68594,-58562,11647,-104836,-44528,-60500,10147,-21304,20182,56930,22222,5908,-14716,-13265,18926,33899,-99265,-6225,80094,-40359,-55010,7055,-42235,29015,8127,-33210,-15999,4700,52599,21949,-5116,-83465,42832,-903,-56074,-25300,-14488,-22243,-42563,-36258,11404,47602,-20645,70700,68004,-59978,74525,103122,-1573,-57054,-22448,
    4349,25543,587,17880,22083,-22287,39680,61908,-14043,-32094,61421,-4453,-60842,-26749,11249,57837,7452,-3927,85790,75680,-4,-2122,-59308,-15478,17587,-9627,-18680,-650,-66100,-54311,19092,44621,-38970,55168,38430,-7918,7981,37867,-49155,-18371,-55224,13872,-50592,41112,57052,-835,-5781,60913,48644,-42099,75201,35459,-10664,-12474,-29330,41063,-7414,-21757,-46323,-47269,55062,780,-13671,-53293,29276,-2326,76982,65643,-21682,-3039,-51637,32087,31232,-3050,71849,-40417,3737,6048,54381,-6274,-9266,41158,4670,-4129,-64640,-26335,-6766,-39298,-36272,10089,39090,-4682,65824,-34750,31165,-49554,-26526,-10194,34016,7197,628,61990,1438,16484,102989,-29154,49056,79714,-14732,-52892,74575,36486,-34617,30731,-16050,-14613,24906,-18607,-64897,-36405,-38159,-63182,-6942,-1094,16055,30636,-45842,-23504,
    36280,77805,3317,-35759,44521,6640,-10514,39154,40174,23299,7257,-58207,-4651,-22411,6097,11676,22507,21717,-52821,-7274,-37112,-9690,20377,-6837,-18804,3475,67613,-44653,-33661,2049,-2664,93509,-45409,23361,8457,-4275,17539,-6783,-28934,6502,43233,-53307,23997,1212,596,56847,81798,-13063,-942,38450,25146,38234,55746,12091,23473,-3577,6186,-43366,-5999,46657,45127,-42131,47794,-69638,18648,25727,5731,2959,-4755,-22254,7750,6023,42240,-106687,-25590,-22718,63853,-102889,-78805,-21030,-50676,4669,46132,-16369,-30485,8456,59917,63135,-15541,-37522,39241,-44241,-14268,-5453,5414,5224,-29925,-19422,7741,-7515,5863,22319,-53529,-43849,12274,5420,7902,26265,-21994,44907,-47935,-1362,-23854,69512,-15102,-71366,-91119,-4835,14240,-49856,24313,-40198,-22538,6279,41779,-19346,38425,47933,
    -30904,-845,77356,4609,-32794,1944,-9449,2609,35597,-5891,54180,35272,3454,56074,-11551,-11591,21082,11140,10550,-32544,-11781,-96014,20703,-6725,24893,3632,10049,6983,11453,-23731,-27887,4531,-81296,38280,-64179,54261,-27301,87121,74290,59969,-4585,33444,-18372,-32996,24220,24117,70034,27811,-25927,6570,27903,28556,28707,49621,166,-8144,17733,14417,6323,-34600,-8363,49225,-9064,34807,55431,12516,-341,12116,-28179,46988,-69428,7067,-3229,6956,-19029,-70760,10009,93400,41294,6507,18371,39879,-61330,83403,-31812,20395,-13003,-1312,-9018,-23606,6519,-38165,-4133,-18488,-33721,19926,-419,86921,-56506,-78213,25012,38098,-14218,37749,-38756,-15851,-24119,-21475,-28045,-11203,-14642,-35667,8640,-716,-35163,-55293,-13088,-5471,-58224,-104103,-10632,-26053,44723,-1126,-36208,-36344,-16096,6501,
    7975,50520,13099,11892,46520,-48301,32530,-16023,13406,10797,-35296,13601,-12799,-42870,-88590,34751,-14582,-37258,68133,28022,-50534,2667,18643,3297,-980,-94907,-22578,-23565,-16134,-17938,25948,24760,28334,-14587,-36554,46761,-55919,17759,1409,-39466,-31267,-2430,-78707,-2200,4697,-26614,6265,45079,15418,-64975,73553,91551,22831,-95465,-43874,33906,-34622,5776,-18254,15317,18770,-24781,-33643,-15373,-77694,-16555,55021,-30257,-66420,43213,-43184,-485,-39673,-5011,12557,-27206,-26674,59138,-35138,18476,-75429,-53599,1984,56198,-1811,22051,-7137,-43481,14979,3306,50141,15787,-24133,-1010,-2651,-28804,-20491,83582,-18767,12742,80440,-37071,53760,8620,74651,-5802,65201,-8442,-32949,-35996,-34307,48747,-7525,4786,1718,32759,88044,35387,-5769,-41472,-53397,45616,-5625,1329,-62490,-8743,-41311,46114,
    29884,-49646,-52530,33870,23156,5299,52571,-2379,-54721,11554,-4451,-25085,-22741,34602,-8538,36642,86096,1403,42604,-77349,7577,36799,26273,10208,362,-80512,57458,-22752,-64371,56836,80508,4,-2492,-11804,49663,4541,328,2031,66175,-13177,52202,-29608,36814,94496,29717,45521,-49003,-21677,-48246,22324,-17699,-59658,-25779,24100,-42243,-15286,20486,67028,69473,-21574,5168,16512,56046,-45027,-18366,-57213,-42995,30829,-7524,5939,-12757,13263,6075,-32112,-5808,12713,2190,9880,36678,-23787,-3070,56187,20261,-44734,-48684,-34068,39593,33044,-18490,-51415,16756,-12248,764,20693,14755,53654,-13170,-16292,-27675,52960,17487,-23964,-11574,-4841,-50937,3401,19217,-31634,56110,-33178,13448,4810,22798,-22519,22594,29733,33562,1323,-71277,22588,2438,-48369,39393,23080,-46830,-11742,-15598,-26964,
    75889,41616,18326,-52000,-23004,24302,41575,9696,-48349,4246,-66277,18812,48301,-19302,76815,37827,18485,1048,-14063,-1654,20810,-5014,-19978,-41479,20022,51400,5749,-59534,10619,-40774,-1299,3073,-49542,-24031,-24437,17857,-41769,45972,-37399,-19477,-44914,17454,-29639,-88202,19097,39638,-23613,7277,-43958,22954,-19727,26358,72971,-21680,15973,25817,-12301,25392,-710,-29539,18877,26200,24385,-5652,-69396,-28298,4411,-37631,994,36572,53758,63220,-12642,21814,23982,-32897,9010,7222,-14529,3377,44415,-70397,-13199,43093,5660,2349,110512,-26288,-55320,43333,-46652,16338,-21917,-29038,-17250,3293,13130,-68309,2060,3470,-39277,86179,-91174,-18810,-7524,-9762,-32937,7939,-19868,-39055,23279,13535,-38766,-18082,371,36513,51883,2958,74488,19508,16497,45813,-43304,14673,-44719,-44286,25897,-8941,
    31871,88250,6559,34833,14346,-16272,47006,-26548,-21972,21005,6266,39700,12527,25871,76515,49719,18999,-140458,-239,-46181,-42377,-20693,47366,-53993,-12093,58498,-17598,11189,-11188,-8866,37790,9560,-38006,39456,-27511,40831,-1598,-15467,52147,11860,36011,-59839,16067,26212,-63765,68006,36685,-14927,54257,34432,12935,28157,56972,24816,-40053,6214,82441,52017,44373,-62572,56710,14096,3971,-36510,19907,18338,-17535,-70361,-2515,-16242,27631,43617,-33279,-75591,-35135,14962,65479,-42498,-54142,23575,45739,-12893,35565,28668,-4785,3247,73599,71784,-7611,-18611,12751,23148,24721,74443,-26383,-36475,24451,-29972,25539,64990,13555,-36866,19645,37026,64228,-54899,-32920,31813,-21886,-49664,-43686,-4663,23401,-2480,-66627,-24845,-88984,59169,3897,-4111,-54676,-44680,11604,79861,-13455,56527,32961,26455,
    -32986,22361,-32621,-25917,30401,40119,24368,-58126,-21958,34775,-14548,-46044,16023,33344,34709,-83726,11299,-69142,37673,8312,-56521,-23313,15432,-10002,2884,-53904,-1556,-26026,-7400,49026,35578,11602,76263,58179,-37731,-6405,-111662,44383,-39372,47370,98109,-22409,35218,-46323,-59623,40853,23478,-33131,26908,-43740,59634,33110,96508,-5470,-29012,17712,-27914,10990,17802,19795,-22223,-24269,29474,-7743,-11384,-20600,-7589,-8504,-65908,-854,33387,4574,52926,-41329,34041,-3819,64294,-57962,-41289,4925,83342,50420,45047,11439,-77697,34201,-25783,-1414,24373,-20171,-5451,13188,21751,-53464,31985,-25112,-54949,-17215,11061,-11627,19558,-45162,-25483,-33271,-44415,38549,-16450,47876,-48924,-33087,7083,-45845,-42200,21422,17478,-10641,42026,38931,86860,-14870,15716,59668,1532,22820,49820,-17315,18830,22170,
    80927,30951,-5348,5304,1973,69955,67929,38724,9172,-20690,23524,-2070,64971,235,38021,50048,-39251,29771,-33554,17267,-16888,38019,-75021,53192,7705,61521,35939,36546,-32673,-4541,96834,5127,-23378,-45927,-10189,70494,-18672,-6941,23800,26821,-6441,16920,-7674,26000,-13671,80560,-46297,-44283,-77605,-39912,12424,9460,17294,-33098,26598,20937,18246,-67403,44120,-21562,-18957,21108,-50413,-11743,34637,-22636,-1554,12335,18937,-35979,-36173,19980,-15263,47158,81085,-4876,6040,2660,-11878,-74828,98,4917,43946,118496,2455,51578,-27020,22379,-56565,-33526,-6074,-51191,-70287,-49945,-17027,5623,2738,-5255,-22403,44777,-60652,31192,-76665,52757,65831,1413,-1537,-72068,-18571,22120,-28212,-4275,-32382,23211,-14216,10548,27903,14205,-51982,9182,10796,30190,112032,-8482,8310,13149,15605,-11585,
    31380,-54651,-6208,18523,2126,10593,31029,6798,18836,-42459,-48473,-12323,44514,-17607,-69715,-53018,-45401,8251,9397,28728,51705,28304,3372,37126,-4768,40177,38377,-5385,2276,75741,41661,2988,-35146,51986,36567,-19122,19903,57187,108082,44989,-8191,13648,27597,-14555,-996,4527,-39164,-20110,-67713,76025,-50747,-78810,-27095,-4259,29997,40509,-32081,-61929,31367,18802,31457,28041,1838,-93449,-5968,66713,-21758,59315,38311,47764,2800,-42185,-53408,-12884,75848,-5993,12131,35848,-69749,-20019,-8616,40261,-45792,-29446,-4212,-1562,47114,25308,50759,-26127,-37891,6156,15731,-23187,35201,-15357,15573,10266,27404,-1682,6801,32891,-50524,-5340,-2764,1660,-51588,-16338,23765,67180,28451,87098,-22833,-13117,52422,56348,25222,-26547,18646,31159,-11961,-40610,-29103,2994,-54089,42576,23342,-6990,
    -36537,-11219,-5600,-7330,58195,-58250,-12839,35078,52996,-27285,42653,27708,-29241,-37361,-100814,20796,-13688,35722,-43013,16103,-45508,-5735,29218,70256,1393,65846,60260,-22833,-18106,-27763,21765,29780,29952,-25908,72871,-14182,5292,-34629,-33602,29396,-18257,-43991,-50012,45642,-2439,-7511,-3698,34919,-46230,-9589,50979,34050,1783,55099,-15629,-38658,-7521,-44965,57353,10443,-68366,17060,7630,-29682,6916,-7549,9386,44155,-17833,-12386,-35956,-15183,-47225,-20790,-52025,-23501,-2325,5363,37058,-39356,-53947,75672,-32269,-24047,-9921,-58482,-45335,-1430,41605,47173,29075,-22091,-31559,-46145,23362,62143,-38133,61342,6957,-44392,15667,22094,-82286,-35614,42982,11231,-14075,-72642,-2551,2268,40737,53915,31223,3547,44662,39060,11179,10663,-2347,-85824,52068,-27340,62733,24878,50831,-62087,-15767,22885,
    15188,97504,28925,-19340,-44796,-53202,1866,22548,32435,-327,4715,12474,23891,-8306,-53612,14515,-30898,-1750,-17689,-21815,-14873,16224,-15018,-23840,-17420,-62900,40371,-48871,11727,16868,12430,22102,-27931,-35282,51126,63222,-10617,-39263,-11255,-18843,-11172,10518,-38504,64456,61123,-33230,-12958,89501,-28021,-29755,-28702,38523,-27422,22379,-16281,49949,-38429,-16860,-16191,66320,3596,-73038,48234,-23773,-26560,74048,-6784,913,-17018,-49014,-4395,19353,-14361,47513,22024,-11,-125371,19540,8682,-28681,-31531,5867,-32632,40844,-43014,-23193,-67923,-28761,-9453,12538,34853,-38010,13230,79052,29321,54461,37724,10206,53791,68533,-4639,26980,-9661,-19793,21835,8357,-26023,-99635,-48688,48663,31124,52171,-14459,31223,77091,18446,-11753,-14426,23648,-77138,-57380,-39519,35076,16879,-31750,-42562,585,-6804,
    29748,-29936,-42309,60561,47357,53077,-28964,32383,-10805,18612,-5708,-47569,-22046,-19341,41724,-7606,-2426,12798,26289,-223,-9875,48991,21123,-13686,13278,-22865,59412,7096,80019,56673,55444,34347,27254,25073,-47547,14882,-75585,60403,35669,-14938,7734,-16950,93332,20737,-13049,-17764,45757,-3630,45149,-42528,39544,-47246,32000,-88324,-49634,22404,-106267,49278,-35049,-17903,-82740,-39401,-54342,-27947,28453,-81249,-3494,19522,20105,53803,37460,-44153,-23893,-42368,5127,12359,22478,13695,-8771,-64461,-60912,-43597,27792,22852,9103,28967,-17964,44564,-41067,7448,5239,12669,-91217,-1062,22534,-34858,4897,31975,1158,33427,32108,18303,-17208,-40385,-92538,-25503,30528,12859,38465,-9980,-84483,16538,-9144,72630,9192,8175,21394,6483,-39033,-21084,32260,42501,-53260,19228,15372,-40190,-5589,21231,
    -52897,-8941,-69150,-4452,33190,-503,-65800,16358,-2363,25931,-20191,-23681,-15405,9368,16740,31771,85711,-66028,-2340,22218,-1322,-41987,41588,-10492,-89396,-26374,-64230,-11691,497,-13474,67333,-41513,-13451,43122,20128,-5215,-30961,-24624,-15679,44730,-19059,-15176,38637,44460,-70811,-26667,-14735,52049,-10228,1241,18359,3290,-15819,9983,54795,-77866,-3269,33804,3807,-38641,-45017,-59472,524,3193,-32229,-71779,18440,-42232,-85334,5287,21036,-5613,7346,20,-35616,11647,-57088,3317,13201,-14141,22349,-30637,19232,-70935,94938,-5324,21101,-58498,14844,-9915,14739,1766,212,51925,-5870,33036,2114,-130472,-81939,-24584,-69958,-35936,-54507,-55478,2066,12605,-11000,-30277,40255,14363,74451,-19349,-66288,-28858,25870,-26602,-62198,-83954,-74465,13484,55750,-51432,-1330,-846,5799,-105932,53302,-23120,
    -45190,26318,31044,93805,-43836,5502,-7388,-21440,-4075,-15627,26626,8378,87104,-33485,-8926,-37953,-34014,18483,-28127,68050,34937,92197,-64393,38579,-11634,-25140,22445,-26818,7143,20621,15920,-34315,-2980,32068,45054,-45813,4216,-59430,15409,-86824,-60605,13257,8303,8843,-45368,-12393,2832,16842,-84154,-23777,-44170,-21166,-36402,45574,82045,-5967,-28392,54530,-89313,113127,-59783,21434,10334,-18018,-7353,102550,22113,-8464,-17947,-37761,-117373,-6718,32156,-8623,23845,64784,-34055,-22887,-45606,22595,-16225,-46871,53752,-63505,4653,-11198,-19363,705,-4805,-46579,25229,1412,-37992,27744,-14144,17258,48678,-90798,-20647,31589,19872,6440,108770,69595,115366,-30438,-23270,-5367,-10937,27672,-39810,46179,-11268,-58261,110307,-67342,3046,-43898,-41775,4100,-5452,-35126,9261,-22489,-44790,-14520,9227,-14807,
    -51839,31068,39515,-73015,11903,-30998,47922,-7747,37472,61531,-46582,53661,3041,44738,43772,27584,-4383,-624,17175,10927,-44880,-51748,-74487,34348,-35737,73442,-21053,-46754,86646,-22397,-33574,-25292,6990,-82006,-80547,101323,49666,-11077,27391,77870,-26588,51325,-18907,11314,27837,-95185,23638,8730,66214,555,-7591,-7933,-209,34379,35127,-31864,-11850,-92184,3055,-119291,-6684,-38062,55787,-14551,-2670,-20401,7575,-34908,8,-31265,-58555,95567,16148,64613,-87781,-1832,-37696,5910,8394,30553,40613,34041,13130,17467,2158,-6898,-12312,-24885,-14862,17203,1289,-17518,-6651,12919,-4408,-66262,70486,-19795,15108,-22068,-16445,-12124,-17321,-5257,25689,-44067,44719,-22817,-27352,38419,-3309,-23011,-48412,-34796,-58023,-9284,51321,52084,-26138,26235,-35887,20454,-21397,-36734,31730,31206,18943,-4623,
    -75410,-38879,-21883,-7741,-5362,31390,-18154,70598,23602,-14405,-21613,-56077,-30117,19739,-72180,35941,-30988,52206,43627,-58590,-19433,1200,-13725,29366,-2190,11176,-4181,-81838,2458,29010,-28618,42070,39109,-37314,22525,-22266,15289,-8890,-64851,11179,-34204,-16427,-19478,-84756,29209,-4207,-33447,15128,-16053,-1926,55940,21413,-37885,21130,-65056,7735,-37675,-72116,-57235,-48315,-8593,20089,55120,27111,-7517,-5022,1789,71698,14778,64920,61057,6759,-16740,-40584,-20789,-1666,-21239,43780,-1206,21243,-9215,-17877,-18282,-4767,-36074,-1436,-15642,19642,4628,42636,57801,-47179,-42654,-16117,73985,20622,40449,63508,-33963,-102376,-30645,40150,21575,41413,-52269,35931,-10530,-29629,25158,1109,79277,40305,-56634,-41900,-71330,18255,-37017,-28498,11095,10426,-41882,-13549,7339,12610,51585,-41676,-21580,51357,
    28006,-40919,-15296,27349,13557,-10466,29058,9810,-91377,-12670,64880,-30522,-25849,-47628,-4673,6219,40097,74149,-21603,-33752,-25696,42115,8376,-41895,-59523,-74161,19280,-22918,79688,-63914,-3559,26254,-40340,-8889,-35249,57253,56229,22827,88099,21259,-80853,50881,-51001,47288,-24538,26377,19954,20884,-9021,19034,56780,31222,23526,4087,-61469,28120,-25151,-22778,652,16767,3630,-34467,-24442,-39753,34904,-48015,-52776,39384,22565,93756,13132,1589,-15758,18303,-43202,-14396,-24316,59973,25761,64478,-39329,-11184,-17585,23639,49135,24720,-44254,-65175,12360,45477,-12654,-29561,-46697,-28722,-20090,26519,-8944,720,-13378,-29234,5013,-40438,6765,-39641,-73140,49238,21860,22031,-18656,-10553,-34688,-16885,-36830,63525,10242,-20387,48327,58169,-34639,-24833,-14754,27731,81715,-13794,-3486,-40242,-14247,82111,
    17492,-13483,-34023,-49878,-4050,21772,-19054,-62830,-20078,-17986,27769,23716,-28590,5997,4419,68283,21060,-5036,-21000,62372,-37295,7129,3710,55494,17911,8562,-53772,-3921,13387,-36437,-11642,-56851,-18029,76499,18373,55671,-57728,41907,-17224,-41568,-48917,10059,11040,43177,90922,-2631,-63644,48529,-98626,-20051,-101,-36910,-18280,-47559,-89836,-38620,-37747,11595,-14067,-33777,8321,-11941,-22448,-10353,-61069,1701,108929,-17387,-64100,25371,-4057,-5902,19544,10601,15707,-24000,10897,38967,45811,-99729,7098,21504,-37981,-20445,-1915,24970,2307,-15609,-96394,-22722,3060,-69310,-67452,-23491,3833,-114719,12034,-29306,63268,23527,42439,3067,-79293,-20246,-10690,-73504,-16194,8264,26765,21473,29038,7751,-27690,-33684,42069,-17043,65723,-15328,-35055,9208,-66200,1737,-1570,-84653,-38882,-47205,26568,-48108,
    -3095,-45255,14804,35978,-21409,-2619,-40702,69880,14013,58883,-5770,-24535,-14497,-10564,6915,-49354,23941,-30400,45145,-33150,-599,1908,-24117,36717,-4330,76853,-15398,42796,6917,10246,15725,13628,44466,10583,-34981,-78397,-67633,21802,-24359,12865,8000,3351,28998,-48628,10954,-16010,-4768,-49122,5597,17920,28916,5992,39734,21847,23728,335,-46323,-21524,-62351,-26130,-44086,55656,-8791,34568,9482,19023,-5040,84847,16471,-5251,-21720,2932,-63267,-15677,-4370,-27706,-53383,75816,-55460,33803,-25142,-37328,-29061,44793,35283,-31089,65613,-102726,62801,-14713,-983,52135,41178,-54937,7569,-18902,-53075,29622,-44267,-101893,-26094,80559,20312,8521,3856,19240,6669,53164,28193,7850,-71160,-6887,-47997,-74193,-50976,-6031,-45366,-62218,7739,3767,102129,28173,-146143,38312,8419,16943,-729,11633,
    -31466,11711,73452,-1576,65663,66157,-37668,-42646,38221,16809,-26839,23847,-29497,-19806,57641,-21519,-32194,-19266,-38733,-1919,-77613,-22746,-8558,44691,-10038,7742,-26671,-64193,72311,27006,13577,17018,-13822,-20100,-2275,-34871,6538,19478,-59473,24160,-18208,-8251,80165,-2758,-13885,31273,17264,32791,22722,-4425,-30586,9490,-64586,-2463,57821,5628,23438,-11133,-9427,38766,19936,-24486,-1661,-47732,20465,-14787,-20473,-34082,-22424,5273,69824,-85555,7426,-8914,-82242,-18099,27315,-22466,17002,-58293,-287,-60109,25504,-15034,-5262,3231,-105672,83202,-75624,-27670,29268,19752,-38686,-47443,32092,72854,72211,-65440,33147,-71917,-84698,-48734,-38544,-75346,-25320,-59908,-10248,-45548,22962,92692,22637,7012,-26736,35129,27779,51513,21633,22002,48058,74822,-28574,-7320,53653,-62570,-27891,-51545,87915,2634,
    -16239,9412,-2083,-88929,-495,-10734,25547,-43580,-32592,17277,1405,35515,39668,-34354,-55854,38678,-41490,-48592,-15830,47674,21618,-36445,21622,-11539,91053,-34768,-67104,-113692,51247,920,34553,-30526,49377,-13773,7627,25611,-1329,21723,3749,-9835,39285,-19604,86685,-36153,-21801,21439,-23364,34968,-33665,30176,-13472,47001,14671,-17222,-26766,6402,51381,32354,-43516,-2595,-8225,-8165,62695,-31413,-10426,-30295,16343,-4297,-32649,58342,-19757,27973,-74864,-63745,-63393,-48542,-43847,14329,26012,-9926,-38468,99048,-34110,-1074,63316,-28473,63303,-45002,-26834,-44882,-4417,9970,-8156,-80082,8655,-37043,-9138,-133326,25851,-30171,13569,-41169,2467,-103824,60962,-63389,-16482,47118,30692,1014,241,-41909,108796,11577,-34807,26160,6210,3445,-8384,-15429,21936,646,4665,-32746,-78592,-40093,-44781,-80421,
    54228,-32348,-35409,48538,-33988,-35676,22137,-87396,-15117,-63120,-397,61321,-21235,20133,16975,59154,21715,5848,-37187,47133,-7246,7763,-1464,16747,-7092,42499,46484,97766,-28548,-64733,-49297,29503,11998,-11026,-24079,-31336,-13677,-48170,-43022,57334,-17325,-56794,4582,28247,-41008,21914,-42423,27695,-55524,29794,7189,12876,31875,57601,-46629,1758,-3287,43554,15313,8131,22029,-11458,55859,63092,-658,-55551,95211,36590,-41822,-56790,-35523,36750,-8380,5003,13531,-56749,16197,-26776,876,-12805,9082,-8541,48957,-28499,-29583,-46615,22483,7264,-22819,59886,45937,10627,-12143,60292,-97226,-95373,4716,-6295,-41342,-10319,-5084,-23881,45559,51554,82257,-4964,-5461,-3851,26326,-24132,-27025,38856,-24796,-32973,-21830,100,47379,13059,20892,-13243,-9060,29678,-27303,-69356,-14858,55368,15904,20417,
    28595,45947,13823,-32889,32651,16895,15434,-55048,27116,-19092,-56764,-12837,38854,39210,-50780,109398,-31641,-7130,49828,-41054,-10049,-139,26841,-45095,-4187,-10739,8989,-153,-17173,-23250,-14407,18243,46281,42458,-9915,21662,-85984,36522,1169,66888,1019,85574,-87089,-89363,-46028,-33194,-24765,26745,35779,21819,-27486,17617,-20693,82582,-59865,-28174,6410,-3842,-28505,-106912,-22293,37847,-36141,15494,-17190,-22867,2062,-84291,-20228,-47356,93050,65763,-17975,-1978,-12411,63018,19582,-8839,-5656,-35561,31155,23277,26508,34975,-7262,31509,23056,35543,35394,92820,-74094,-315,73842,-6854,-19077,-14803,-33825,-2185,11764,9942,26799,23857,15619,-105578,38782,-20640,-41215,31197,-44739,-21914,53869,3610,-33534,-50381,33922,-11926,40722,90395,20839,8243,47957,-37148,-50361,-36411,6744,-12024,-27230,29403,
    -52330,5728,-30310,-20856,49627,-7943,8740,-1808,115730,-54676,-49479,90136,10764,51181,-15071,-18738,-26818,-3701,9438,-69066,-59429,62431,-45537,32966,41264,45397,-27974,-12117,59186,-1172,17359,-6240,-11869,48484,-15359,18805,32460,-31181,22924,-5434,-23556,-14817,-9252,10570,2402,-6364,48918,-31802,-5697,52423,-73873,-24051,9075,-18663,28444,-59372,-56978,-25332,-47347,-16231,-12852,-4577,63738,78505,12365,-35788,-37205,-28730,-14740,-52495,35088,-23373,-62100,3908,9835,-2859,-15936,11975,-21196,-15436,-27215,-21357,-4901,-17868,8071,-63844,37766,95849,46201,35945,56256,64513,-56137,11109,12767,-36615,35220,90572,28027,2521,-39971,17873,-21231,54190,-12176,29250,-25441,-87785,42882,48077,-19859,13219,-59791,-35480,52841,35597,-52915,6525,-85517,-5617,-19115,201,-11382,-68372,-29447,-20217,-19078,-31285,
    -1614,25607,3615,14517,-13987,60244,60975,-36696,11849,-36382,56394,-19679,64367,73355,-29103,60107,-130714,-37699,57497,-13245,-61452,-59116,4186,25417,-45512,52839,-45644,86330,-24749,60501,1895,26474,2978,-15796,31427,1197,-22238,-69358,-19132,102659,34020,-16616,-37261,9368,-71279,-19842,-32075,12540,-43691,-65650,34595,33996,-41808,64631,3915,-63113,11628,17278,26534,-21450,13780,16402,38154,100957,-58613,-26765,-26836,-58370,25969,-53351,-35602,52510,-11528,-7554,14964,23720,-20734,-86000,-14620,49411,11892,66868,2788,17718,28114,35463,-25774,9455,64379,-10619,11653,56760,-6036,34870,-31913,44607,-22841,18294,43760,-9399,34508,76926,-20960,-17284,59210,22066,-3708,7803,-7492,20938,68838,18280,-20261,-57608,6066,-28437,4622,-11614,33850,-17721,-11723,15760,60325,-18215,26877,-33236,-31428,15427,
    -5826,-52784,-63805,2601,-80312,21710,6532,-26298,-40749,-6423,4522,8881,32797,-34766,-30878,-60856,-2518,53585,-7831,49971,18941,-23601,19806,16700,24744,-90438,46067,39185,-33287,-25427,-13611,30909,1900,78372,16700,-16289,3468,12268,-48556,4604,24614,-11238,-21467,-1918,52745,-63331,48842,-24588,33695,-23409,-9568,32410,-54079,-20809,-34013,18355,-40817,-3928,3692,29674,41871,-39361,23676,-12237,8694,3806,36406,709,-7219,-27139,14788,-51216,17893,33204,-2951,35820,-10650,24733,42078,54498,637,-29130,27090,-6361,20987,1903,-38285,26982,-24876,-76041,52954,-9921,26844,-4835,-62895,19564,-61204,-64746,-17713,-8447,14936,897,82777,-14340,-36568,51470,-51198,19592,-451,23199,37242,43646,19968,-1504,45616,-74867,-32222,-27939,30666,-9182,-4873,10967,-9847,-58433,20235,792,50840,19967,
    -15120,42535,10040,-25012,6965,36910,-26927,33088,-41024,56675,-18715,-8071,-74505,-50306,-26146,99374,9521,-29221,-93959,30288,37228,20229,-30721,33398,80780,31551,-41406,51321,91988,-56659,10707,5218,53756,82473,11441,1882,-42694,-44244,11372,48554,-358,-19327,22,-15771,79304,-6367,17372,-22994,67562,-26010,-3491,83640,-16476,-10549,-33082,-15982,43777,61364,18339,-17713,651,-14280,-8390,21457,-21388,-45756,43765,64797,11311,87030,67523,-17775,-46814,51240,-50634,-16425,-25166,-36687,21507,-8111,-24727,-67797,10613,19614,47343,26905,12386,-79843,-19152,-25932,48427,-17917,-51126,-16739,-102509,-28332,-40906,-44594,-37367,-614,9433,-26457,-69661,-104740,-30419,-49383,17873,21553,-7059,-15641,-41452,-18434,-10825,36353,-39125,-52083,20618,23401,17662,-2837,57691,-66553,-68634,-9171,-3900,-73776,4235,53321,
    -97266,24880,-19581,16099,-88745,-41682,-16290,24679,24698,83598,-13189,20290,-16891,1102,-29886,-7827,35407,-11866,38520,58731,72566,-65897,25465,12186,37107,10455,-78149,60273,52766,14083,-84492,-30991,8110,23399,-29673,-32410,-10448,-34059,-73689,59909,71482,-4152,54050,-47167,-31178,-32212,24297,30428,9401,-50864,-52592,12260,-36201,-1906,32321,46151,17441,-25649,56918,9200,18964,-33639,45567,-2847,-6690,17600,69826,-50438,-49291,-26413,-30499,21242,54003,64396,-26894,-28058,12789,7229,20719,-50871,14241,-40987,-51367,-91881,18238,11364,-17750,-968,39388,-5858,37253,-21715,17907,43953,-13989,-7920,-26900,60624,10560,17867,60359,2142,-9034,56915,-31956,-23579,-61864,-58173,-48339,-16033,22080,-85332,25647,-30304,74325,-34737,-15977,-6684,57437,-38871,31086,-59178,-19048,-71163,33221,28790,22565,7661,
    -12566,-69239,40468,10816,6851,-6475,34614,5425,-27978,-23760,-32840,15737,49560,43036,-31258,1916,509,93935,39627,-32798,-23343,19717,6480,10931,29276,360,-14735,-54239,28620,46650,9713,-12780,-29680,-31987,45016,4839,84715,61185,53936,32895,26112,24076,-18335,14000,4986,4155,-3598,12567,-7566,-39710,42822,2593,41768,9705,-13492,65418,36419,-21893,-5163,6348,-48445,-22791,16720,-21800,18814,852,-59290,41552,-7613,7145,-62479,-60093,27195,-38898,-15010,7032,-20394,-48382,10912,33502,85914,61758,-35632,80804,-16317,-57173,8688,13786,40931,-33994,-69208,-4796,255,-35200,23044,59613,34653,12415,-72888,-26058,-51588,13732,-65140,-61375,-50539,17891,7531,10583,-3090,-31048,17207,-45595,-60012,21611,-8323,-4909,-12140,-19731,-44442,29557,10462,-28466,58059,1454,91185,-55115,-18907,-4112,
    2506,-72794,-37478,-50194,-3813,2989,-11150,54599,-4520,22914,33732,-25259,-12175,6560,-93179,42344,-96789,-51228,-3176,5491,-63393,95843,-39666,6400,62890,32921,96418,2681,17637,15797,-54248,-21920,30066,-45944,29601,-68234,7566,-26915,-20867,24788,-3968,-6174,47710,-8670,-25607,-22859,-32961,12692,23076,43579,1070,-56615,-17186,48064,-578,-41627,5121,8466,-2021,17896,47432,28325,25762,20136,88715,62059,-78636,17874,9505,-15725,16244,-18190,4453,32444,67325,-27900,10892,-455,1190,70457,-11630,102832,74928,2690,28946,-34136,-62815,-46772,35589,17028,-58810,-40573,-2304,-18445,26290,-5066,35427,16845,17281,-26734,31138,-38443,47401,1033,2449,14462,-23992,-1459,49074,42522,56952,79129,19299,6693,-18140,29333,22587,18933,25313,31974,66699,-54619,3029,52482,25186,-15292,-38616,-15848,
    52388,-15613,-23233,-606,551,24207,-15972,11212,82751,36724,-6823,-59275,49914,27623,19206,-34999,-86532,16889,77078,-13199,-8370,48115,-108380,-16355,26230,40023,-29858,63268,-50429,75295,61037,-10168,21169,57035,13602,7633,45720,-39900,14385,-40647,8445,68040,-21635,57496,47735,50798,-57587,-19879,-29018,18423,-13369,-1678,-17433,-36945,7755,91924,-18050,28261,-16322,10429,101245,61954,65557,26281,-1629,30663,-58124,-23831,10926,5830,37243,-15128,20751,71617,21848,71035,-35245,11323,-80970,-16620,39057,-32739,-12350,78124,-50559,31040,-31262,-84792,-14784,51274,-12282,96254,28587,0,28361,-40156,37194,5606,1060,32747,16678,20404,-13801,-25049,21331,13915,34321,-18641,-42890,-42568,-8213,-45036,-25153,-16296,-21501,52903,19916,-85494,25637,-7871,-34210,89899,74970,-11130,26268,40907,38480,-7581,
    -36715,-16959,9079,-630,22711,-97923,31594,86565,71431,-15240,-15942,752,-55961,37685,-258,-32537,43407,-44226,53446,27171,-46818,-38117,-36986,-52805,45506,79972,-45505,-409,-34298,-41568,8133,24647,-1746,37967,6583,-49186,88763,-611,-44904,56253,25224,86918,-49199,-334,-36975,20658,118641,-5353,26332,125698,-8650,-4694,1169,49329,71990,-55326,-14464,-48953,-16786,-36719,40857,7529,-32277,5235,26764,-43388,-48554,7434,18655,-45479,-34557,70228,3286,42320,-60327,-31552,52293,9971,5890,61991,-4168,-56674,3600,-16249,10949,-31782,63719,-5365,-1873,-36510,41962,24296,58810,-32580,103567,39190,-24147,41642,29712,-92750,100832,32906,-43289,4050,11653,-46011,-2554,-21775,-53760,24982,-19920,-14352,43185,-13586,18901,7438,-28997,-11959,-44797,7976,79538,13307,-9907,18192,-697,-50875,14861,-47365,
    27847,21388,-13276,-65467,62254,-1041,6061,-5992,33284,47139,-15355,-50939,3059,10798,-8950,27699,-3337,7913,60019,-11016,-45168,-16569,-35953,-1003,-5988,-53689,17210,35655,39871,7697,7358,5223,-11218,42944,-26830,74452,-21419,12408,709,4479,45247,70130,-60859,49300,-33432,-41730,-2180,14068,12107,-67224,3318,-27426,-10261,54210,-38485,-34334,-52161,-19411,-11161,-43169,-4938,23457,21291,17733,72171,23872,-17554,-27237,17563,-44053,23395,22419,44828,42263,-41708,-40395,-24693,33342,-14916,-20167,-23229,63588,-1897,8747,42905,-365,-51400,20976,34846,18801,-73068,-3652,8202,9894,-649,24609,-19159,64022,-15638,-59810,22112,37710,3070,-23985,-18181,76334,13730,44822,11099,326,12510,-13405,-73509,11513,42639,-14573,36658,-2797,-15269,-7978,10212,-5022,22924,43722,-4759,-26498,19428,1670,
    7160,-32841,89089,60907,93542,-66818,-21048,-127,-12825,51332,9454,-76743,7681,33984,2651,-17577,-55023,-35900,46899,102606,53223,-49499,43306,-26430,-31798,5764,33998,-53898,23967,-39733,-7674,9173,3985,-6747,23759,64540,-22126,42370,-39846,12089,49405,-82636,37787,52321,-45834,20931,-17860,-12670,-18807,46344,-20225,-21647,-40046,49519,-38547,16818,69167,-22824,-30958,-7172,2897,-6195,-40336,-94244,-40402,-61552,81583,-11501,59859,28882,-69435,102915,38406,-48403,-22254,50017,29096,21919,13654,-20387,24079,21657,-23596,14161,-20598,24727,-54132,40653,26736,12210,-37660,-52480,-18088,-26946,4410,-51131,2367,6639,-4105,43217,4213,1924,-26904,-20631,29723,-6467,54775,63964,4622,-36651,-98540,11528,-64137,-30056,-21764,-36664,-7720,19521,80085,19553,6071,45330,34351,-28863,-69504,23851,-19453,2488,
    -18550,-52942,-3709,13447,26375,-8159,28691,-52648,32220,-49418,-47515,3090,-8293,29952,-30531,-29472,-11015,-72586,-31509,-39825,-25314,9321,-58484,4732,56314,65505,19247,78365,1749,25359,-39981,-32118,-46473,-60159,-32239,-19893,-10037,-7029,-22733,-75230,-1481,3639,-56290,13205,10660,26310,-8808,-48612,38008,140325,-12391,-1960,-50456,-4999,28848,-11098,45816,-69908,6267,-35337,66393,-2344,-16406,8371,-57662,-42082,1604,-32015,46382,-48337,31049,8731,-25296,-42047,-56764,17603,4045,-21992,18040,29613,-8808,-56950,12303,-75532,44372,-16791,-29527,6260,-16007,50823,-61015,81856,-38501,6347,-122497,-32439,1594,30352,63273,-12411,30897,-10995,20728,3850,-45088,7874,15057,-25860,44392,-5669,658,-60525,13074,19586,-81445,77374,24660,-14108,-4817,62047,4265,39279,20477,-8409,-44486,24496,31614,21672,
    -55212,-73053,-29706,56078,61098,34968,22913,13370,-24340,-92221,-12630,38083,-20121,19830,-26906,60738,2365,1566,15598,-8278,-28691,4269,-1530,-19610,-5853,-2511,-59546,-60413,-37324,-53480,5341,51976,71914,-49729,54087,20738,5396,-19037,-45695,-2624,-81272,42554,-50075,43451,-20944,-47812,-50479,-18067,17171,28578,56987,-8653,34679,-5590,-27228,-25242,1000,-51545,-31549,-59288,-48310,21629,-4958,11160,-61,-19894,-4360,-22266,-20678,-59820,63634,9056,-16368,-39891,-12675,-18453,-12685,-38428,18343,-32038,-63268,-38984,2284,1957,18958,-19199,-26740,16611,35363,-19766,49319,-12464,-17188,-44771,-1552,-78989,-9403,-33057,31985,-41622,-4371,-64796,26774,-33069,-16039,-19481,33921,55010,15809,-23884,25950,61569,-36798,-65736,54391,63997,45537,6657,10678,-806,-41840,27366,-10834,24540,1060,29675,-16365,12679,
    -50456,-28840,-75342,-28919,-6548,-38578,-13420,31925,41853,62578,-740,25465,-29348,-12284,-34987,-49339,-56010,-50380,45891,41485,2524,37718,-18899,20823,23334,-37826,-45885,13562,50286,-12647,-58154,33738,-22799,68228,-68196,48059,46380,31039,53374,39821,-36824,11122,57820,31747,-36599,-94508,80683,42667,41508,11941,-63287,-79596,24842,-117,-56740,10317,-9920,73989,-40404,2858,37930,-58169,44284,33634,-24785,2027,-45919,-54803,-22739,-20832,-9631,-8438,30795,112962,29897,39447,-7957,27624,3244,4703,21626,-25671,-4802,15378,27076,-58117,74743,27537,6945,7502,-459,-19190,-48462,18777,-3995,-35453,62850,-8588,-39968,32620,57129,-27184,-15752,16167,-56996,-55362,-69201,-14264,-493,31678,38700,11337,-14815,-72785,111409,-39633,-11835,-23701,-26489,-51431,-41598,36312,17707,-46220,-63638,-87144,-42637,-71665,
    -43719,-16288,15182,-20943,8650,43259,-46596,57101,55426,-43096,11928,-15590,27275,-33411,-68871,32622,-15141,-78900,12826,93940,26912,-52087,26662,-38994,-29979,37611,14983,69126,-28141,-71319,-36126,-2855,-46232,18270,-36071,29589,-11821,42231,-102180,-19419,-70329,2387,34608,45233,44446,7530,-5482,72239,-26004,22198,20332,15589,-4883,-3212,-60034,17086,-24580,-74726,-9680,47285,14142,-54842,-26142,-31884,-32812,39977,25771,52962,-82665,-24267,45773,68553,2197,76805,18545,-30618,69426,11428,-20764,4945,24698,-26585,27302,31707,42150,71731,3833,-34987,20708,22381,-37498,-32847,39573,-43848,56787,-40045,-43685,52268,60745,-8125,14926,9780,-73365,25011,-39943,2205,54667,25803,2013,15842,38650,14016,6709,1141,-68411,21986,71371,78950,301,-12632,71618,100405,34311,-25363,249,18552,-1235,48630,
    -21632,-11278,-17089,19585,-68250,-6479,35902,21179,11678,57540,17295,6102,-28390,-42414,21970,-46714,-48044,47724,42006,16477,-9498,-2240,-9875,51968,7433,6284,97890,987,-17590,19461,62773,6548,4659,-44903,3865,11054,-16940,-2783,-883,-9155,14838,-27817,52952,70593,76738,15364,-36057,-2442,-68653,-28325,-109306,-28666,-29522,-76316,22013,89514,-45120,-34858,26571,44251,19159,-31950,24899,-54115,-29485,59889,-7840,12830,29368,29316,-33111,19597,-71616,30680,-2147,6366,-16526,5183,2315,326,-41398,1829,-67734,17436,-1583,34842,4026,-80496,-4570,2120,-19754,902,37341,849,73112,48327,-23027,-10803,24544,10203,-41206,26622,-9020,-83132,-13854,-10024,-2520,-32194,-27053,48974,-16520,-27841,4002,5838,1407,-16646,-14530,-14047,31580,21000,13090,73117,-9967,-17797,12796,-54672,56477,26460,
    -3017,81224,-33492,20266,24563,36999,13677,68270,-18135,44999,23036,-31126,24539,-24770,28986,65351,71274,-16820,9798,64834,-15207,16129,13979,3644,-8250,29355,-32922,-22086,1922,39650,84934,-34356,-28929,37798,-14519,-39963,-27568,-89139,22064,44866,-19006,20212,40919,83512,60660,66984,12776,3998,-34939,-5441,-12770,-59453,14911,32223,-46789,-24951,-18216,-2523,-26657,50179,61809,16344,-16118,-19760,-1355,-52102,-58162,-48402,8688,72419,49254,34943,10800,-40781,-26266,-9859,-9180,25640,-42217,-72679,-48200,-13167,16674,45391,38527,1643,119420,28286,41252,-50526,20591,-12334,-17244,19391,75434,6306,7588,-9366,22904,21488,69889,-30251,-42639,-24143,10602,10909,77471,-4359,-39362,42660,12786,-33981,-26158,-9832,45197,56898,11051,-13526,-50068,17943,78313,-64372,38171,8384,-11715,-21303,21008,-18655,
    35048,519,12915,30606,45347,-6771,-21125,8820,6543,10994,-77485,-341,18111,34645,13846,-6894,35480,-21005,-10320,-17407,-46808,-33771,20106,-19239,13413,32965,-81792,-59077,-61657,-10405,19624,-13925,-39475,-71013,27265,-75914,-1552,15235,19848,35329,36721,711,7943,-5113,22538,40087,69412,16517,3146,35054,60674,30781,-42601,-91888,65073,60007,72672,-83391,6921,-22534,54143,8308,-58693,-34858,-2809,-99576,9699,-35576,-11656,-40191,25372,-39960,48692,-32681,1839,37873,67109,-22861,-41313,17470,4418,-16107,-11705,642,37079,-64002,-10507,70818,38701,746,58305,7983,-30265,2024,-23674,-47792,-12549,-5734,18160,-13458,45770,-15158,-81057,-73229,-38556,-3521,21937,8710,-33189,38042,-1340,32816,58547,12354,35114,37701,-11027,-49429,-3905,10179,25178,-26427,-14148,-22789,37326,10142,-30171,-81316,
    -10031,40663,28544,-24878,51070,37477,4283,47957,18226,-35211,-53810,-25789,-20646,-1792,33779,-16,-16253,27143,-33815,38808,-71305,-77478,24499,-19334,-30161,44180,-37800,-37951,82316,-23872,-46623,44203,-59431,-68527,-53306,23618,-6141,-48117,19032,-23431,36524,41762,-45649,-80518,27577,60144,15577,320,-3118,-18172,28622,77781,24096,-3887,-31652,19346,27301,-46630,3137,-47645,11306,-74999,-86579,3589,-26559,-60273,8160,35266,7670,6784,-32248,2573,51874,-6413,-31513,-89248,-11431,-76351,19825,43878,-86031,23162,-22361,75992,-20101,3952,-3302,-18565,-40195,34492,26729,78940,40969,1214,7217,-20356,43575,25464,30678,-9809,12081,-42647,-84150,-3339,26073,-80641,47226,31669,-36603,22825,-3992,30609,-74837,51187,-6137,18621,14106,61999,8182,-65467,-46533,53588,8161,29025,-1844,38752,5034,19494,
    17438,22293,51803,-12994,-56128,12205,5285,-26377,-28632,58801,8736,-30368,22802,-41066,-45110,57472,11473,-7441,1909,9851,59250,-21692,25015,-24409,72230,-57144,-16484,-11257,-14517,-31808,26145,28560,79099,81275,-7021,21216,-29143,-16461,43984,31045,21871,-9887,-32661,-17400,31080,-37821,34908,-7762,-1927,740,-39317,45128,20887,-103852,97396,-16146,28499,28783,-27835,13087,-20117,-312,2601,-34911,50256,-44520,-30612,-47551,-4187,-10409,-22070,-5701,46008,-66253,50283,36595,25193,75721,-105074,2169,-66150,-2950,11525,30568,5286,18110,44998,-37792,15958,-4165,-5574,-48005,45211,4750,77164,-30628,-69883,-11329,31373,-2596,10877,-14506,33073,1204,34074,25650,7722,-33535,962,25348,-53085,-2375,66151,6031,37735,-77537,8175,-14307,60761,-6372,73465,-9640,-29780,-4971,-40279,22337,43914,16037,
    -30182,-65061,39675,-58436,-32134,25442,-47707,15796,46009,38481,15540,-36625,74939,-13796,-71550,6392,49393,-22773,-21401,49071,-8560,-68078,55439,-3973,7350,-3097,-43250,36030,11039,-20259,-31746,-54411,22767,24439,48109,12096,-60415,21354,-51884,2849,6276,22182,33567,-79615,13567,-18261,20237,38964,-3031,-31281,-46082,6244,-26927,24583,-34109,-20513,-4574,-24925,-759,-93184,6807,-15372,46280,33764,-52309,-73094,26678,10740,-106174,-26856,1587,-24404,50162,-21526,2111,27380,-33588,20475,-16896,-10109,31245,68988,-9068,-30196,17840,6192,-13860,11448,1429,-48427,-9303,17012,90881,-10441,37568,82198,-37210,-40580,49187,23045,-44111,41636,-28896,-43988,-33359,23750,3032,33749,-37322,31104,82307,-20498,-83410,-28914,13386,24716,-53574,-11315,4395,18133,36824,18558,-26368,-40133,83104,14663,6701,-77308,
    60741,-35617,57595,-33119,19526,106887,-16665,-13890,-16022,-6178,50475,16822,65663,2696,-2490,44549,-78873,-27421,-111625,-11380,-8408,4986,71049,-6251,26943,-50688,14596,15334,58552,-9482,9702,62329,11243,-1857,44856,38457,-18061,-91831,21877,-35795,35522,-18319,-26719,26022,-44523,39428,-141802,12530,-21020,-4208,-3558,18024,24049,63302,13101,-15127,91983,-20621,14856,40665,13064,-73341,-57917,-26801,67968,13073,22068,341,-80361,-51601,-24536,39875,38060,-47655,-31185,34880,42744,-129891,7535,9498,48495,80554,25458,31528,858,-21984,1886,-7228,32359,-1523,-23343,-8774,31139,35777,23025,-31300,8800,48112,-45608,-32424,-146678,-20787,32546,19931,124733,67538,-26116,66231,20000,-9783,96757,22800,26832,-13513,-696,28706,-53295,106717,-23081,41271,-21707,18715,-6243,-32162,-23505,-13680,-74289,55984,
    -33229,-62899,30389,63594,26862,118158,34769,-4158,-14174,-18564,66727,48839,-16380,-127900,-39510,-3002,-27553,11324,-73215,56847,10380,51973,25358,79222,-33200,44531,12048,-40027,19242,-67508,30779,36550,-89493,-4031,37927,4381,5168,-27332,59600,-38471,-57531,-7663,83067,52764,55873,7513,-46927,63340,-48490,17482,-16943,-28934,43299,4028,-38055,-30308,15797,-844,-58346,7720,-47133,-32357,-2640,-66597,23247,-21966,-30409,31264,6438,48299,34168,-899,-8451,-69535,-37088,13280,-55340,-21917,55898,-5909,-59870,24694,42182,-18400,4334,36572,15672,4898,53608,5991,15924,-21902,-58844,-13704,-27540,-85029,52161,-28102,5817,-28784,-47210,-33839,-12560,-21151,-13378,47055,12724,63883,54928,56273,-3237,20537,-44250,-25339,-846,13738,-16791,-1431,-30836,-8307,-7210,16648,-5967,-45619,-36804,-44168,27557,15773,
    -55494,-9584,-6172,59442,-28926,10006,15958,-14536,-23707,28940,29442,1968,-55881,10141,38478,-10517,72143,-123,1688,21368,-14119,7577,-19182,-25944,-22250,-5628,-16048,-63408,45303,14655,-57515,-8554,44508,-29398,-45125,-5602,15584,-26785,-53304,1996,1263,-6578,68225,-10861,-41811,40000,-28580,-60733,-20551,-2725,-20887,-16688,17814,13477,-7708,-2235,-36977,-21096,5529,-5565,2261,-35674,-15277,-1536,-7596,-20297,16571,-4874,39259,61407,21382,58184,-6378,-49638,-34305,1493,7510,909,33740,-10327,1071,-34348,-11816,-31939,-82985,3242,19253,-11031,3961,69982,10725,3350,-23347,9789,28326,25710,-27582,-33958,3929,8000,5324,-26900,75158,39093,-13751,7948,-32458,28489,6585,-20092,20070,-46198,-48524,-55145,-31933,30219,-6693,9703,10102,-8918,-28361,58181,34828,49229,48455,54330,20006,89338,
    14051,5448,7099,-21836,40430,-56226,-38838,14218,-20,-20124,35886,-10941,-39620,-18244,48739,52841,41549,-7811,-9577,-28800,6854,29037,6728,-25092,-32565,-5313,22975,-84513,-14679,13066,-48139,-16506,-8406,18741,69738,-13523,-1216,83632,-36493,-92484,7807,73117,8713,-6013,-40087,-25549,-4162,-40864,56144,-7460,-35671,-60881,7057,54021,6031,-63865,-6101,77500,25168,34543,-12364,13897,-1205,45407,18966,20315,-71937,-9744,32840,-37719,-50265,34681,-3819,24941,19377,-3588,45043,-57195,-19457,22146,6030,27381,33827,-17973,67551,-64365,53767,-37003,28782,-55275,-59125,-3526,32895,-43761,26563,97093,-28946,-9033,21245,-663,68972,12823,-20497,65288,-21631,-25885,71498,81163,31773,-52185,-17186,21581,3185,-39330,2569,23762,-51452,4062,42244,21886,74499,3986,9677,42035,-67128,-3851,28772,-2152,
    -799,4610,54404,-53775,12552,26119,-69752,6730,24849,-42984,-61476,64306,-9301,-52686,-80977,-91,-53527,-2195,-41328,-36158,48190,-462,-40039,44050,40509,-10819,-80857,9312,47141,-33820,35486,-15357,-33757,23558,-23923,34114,24116,-47488,-25491,9360,-20671,24388,-75378,-57745,-4812,-6700,20827,24034,2926,-25745,-23682,70801,-64612,-37489,-27452,7927,-63514,-3877,-651,-4073,-37462,2329,49041,-52933,-32805,-4772,38427,43456,13539,-44109,98410,-36125,-826,110004,-8232,8635,-28614,3778,-7945,-51105,-46925,64600,-30250,16425,11947,-11035,-9464,48643,56268,70472,-58969,-14941,14647,4133,61096,-43493,29151,-39018,-5705,24055,19616,-48001,-50867,10817,745,66117,47234,-22267,86409,-23657,-13224,8485,3073,-25785,22548,-62706,11043,16393,-58531,-56711,11451,-32486,60394,-16536,12649,38500,30174,-25164,
    -47519,5592,-45513,8179,-41133,13370,-29322,-29501,-44353,-45134,-6421,-12515,-79426,9280,48719,-29075,-10296,-1570,40107,-70865,-1957,595,19727,-1562,60279,-86008,-66470,-37757,-79367,46585,1378,-606,10936,40215,61251,5116,-40099,-8971,60334,-16615,-12063,52410,30043,54927,826,16640,-46951,-3758,-21096,65084,-17673,-8609,-92380,-45084,18038,12509,27157,57724,5455,39542,6447,-9753,-41690,-23403,-10128,-2095,-43754,-6884,56502,-17690,61525,-41002,-28378,-33367,20360,22010,-51958,791,80856,-68806,-12928,-33250,40453,34916,6381,-35015,14951,74059,-11314,-15015,-40034,-10683,7675,6705,67227,15165,27858,-66235,-6601,-14965,21599,-71530,52631,-31349,-48827,34575,17884,-61642,27425,31727,47478,6499,85306,-24681,85097,-69187,15319,-17381,-16167,54663,-26230,19052,-33273,8079,-31680,-26316,55310,24087,
    68604,-4244,-29682,-2158,70722,13714,24301,-33855,32030,-40639,-42339,23086,26654,-36336,9981,-61920,-68170,-17874,6058,-252,12728,10127,-59934,-44976,-24535,-100344,-25681,34962,-43639,69398,1982,57805,21382,-82528,22060,-25414,-3167,-24936,-27884,18967,94660,14942,14864,2757,-23477,585,718,7781,-41098,-44699,12604,-28882,-8767,23311,108502,-107,49794,-81830,4626,50340,42755,46399,-20750,-88998,1693,65697,-14620,-36571,-19582,-101846,17599,2326,26388,39465,1395,-2734,53678,-4050,-27225,-30985,89848,11619,6695,-94573,-62036,14578,-64831,-25620,-20941,18974,6204,436,25366,-30344,-6931,-4405,-13665,6272,102485,-34701,5675,-8012,-20662,-12591,18937,19009,59903,-62719,26643,-30185,18636,-73422,75867,-14623,37705,62714,13822,7690,46932,-14388,20805,52815,75312,35820,45528,-55178,35493,-42765,
    26699,102672,71162,780,5671,6258,-8462,-12581,-8744,-32876,-52264,42691,-6410,13006,66075,8688,-69360,25248,52600,-10821,1489,24031,43241,-14813,-34743,-103797,-14466,-21142,-5602,26578,18915,-44488,-15713,-35718,28119,-31024,-59741,-29053,-2719,-50126,40016,13805,-7400,7591,-23327,48803,-34390,-13821,84486,-85887,-51119,40673,82145,45869,12484,-34233,-3462,28955,33387,-39146,-15535,-11128,-66367,-8221,20290,67745,-37851,-5230,3144,-76818,-9009,11602,24587,-5711,44548,24733,-30513,-65345,23139,2843,-11060,42276,54857,38462,-1102,1190,7439,-13186,28393,32267,-15301,-48369,5009,34152,-42113,12630,97723,-54057,1980,47928,-22888,-67820,20360,68888,20382,60538,87569,26620,-37727,-23523,-72935,-57579,26240,37791,-46353,-40804,-81095,29451,28940,-5592,8000,-14684,9933,10529,38722,36286,-26838,12764,
    -9462,-24264,-10423,36723,-6491,-35540,-9700,121348,42477,49947,-72343,9555,14093,8697,-24402,51340,39277,-12725,5299,-47756,-52583,-2797,-7433,-31482,33730,48557,-55582,-50560,1471,50621,-46951,2931,8423,-13176,-44387,-84822,22488,-95749,-91558,-1411,74802,-10085,-22719,-12640,22638,35264,23400,32913,-7670,-405,16413,36545,-40427,-301,35760,-43846,1300,-9062,-21514,-8376,67970,33042,24892,6540,-3512,-90800,-62011,23735,-62121,-12435,-6657,3338,-10551,-85254,29224,26841,-14388,-892,4175,55410,29839,-48601,38326,17490,24614,4987,56556,-8429,42678,-37727,106848,52625,14112,-33561,81977,-49820,-1901,-19027,8162,-22793,-4484,24980,831,-47803,53367,25984,-9637,-30720,14478,-32783,34996,31009,9043,-43383,21185,-28393,24554,-33404,-22327,11141,3381,-39521,91227,-12825,-15202,-494,39166,-14819,
    40741,57980,23455,-17075,19128,80189,-17951,36461,-41552,19749,63139,-28292,-30203,55099,62847,-2672,-2156,56844,9980,-113256,-29323,-76118,46353,-24482,-62997,14884,-6298,30608,-47040,-516,97893,-22221,111747,-19332,10195,16962,-50345,21934,-10561,10929,-7453,8875,37802,5821,15445,9701,4744,54902,44882,-73316,126890,94903,-11283,52766,-99816,-46737,18643,30981,9141,44368,58711,-31113,3672,-1450,2853,47562,-41473,-24523,62500,69980,23050,14917,9811,11140,-20120,100068,46121,-9636,1084,99305,21977,-28788,56451,34131,-21612,124161,-71070,-15935,-30354,-57585,78223,-14756,26645,-84607,5727,-41237,27595,4510,-12659,32557,-120053,52377,-49032,-62981,-48415,-79075,-51770,-5213,23896,-2289,22467,-3915,-5920,13096,-45375,2024,6456,-36486,51333,17500,-68043,6888,-18740,18832,31250,-48468,-40560,37968,
    -310,-2567,-78752,-19124,6431,12755,33501,-87674,-96374,43829,-18155,5172,1245,-27084,36000,13493,20488,-89770,-17325,24415,-20386,-21916,50402,-20091,-73209,-68188,-26585,-44358,-38647,71351,-37882,-10110,17253,-14921,-18120,-13172,-18849,-23665,27405,19513,72666,-58927,67087,1825,-43669,-71058,-33856,-8394,28746,-50522,-40297,23280,586,-54250,-1291,40565,8048,-27422,23614,5966,-11025,-50980,-41645,106851,-29401,42481,-7941,-59920,-23475,9491,2051,-5450,30003,78323,13395,-25115,-88406,-94868,38173,13657,-56135,15097,-5737,3316,88913,-31941,57572,-99139,-19914,21814,16452,13223,-15078,19121,-50071,1444,-61066,-39054,-37488,39131,1526,-92399,5869,-21969,3133,-666,7074,33433,-32153,45871,55184,-6963,29632,2022,16966,70461,13432,-7376,26,-10006,25200,32559,-13407,2043,-28358,-17594,-27700,-6358,
    18262,13450,-14850,-92977,-51638,-17551,-10461,-33120,-49622,-100381,-38922,71940,14120,29658,30376,-63255,26762,-119898,-44418,-12841,8866,55309,22007,-11972,-33314,28503,19331,-50781,-629,49416,-15666,-71704,-77113,49813,-28003,3584,-69459,50967,-74120,-13274,32971,-8875,-10068,-11969,-46049,-12456,48267,-12583,11756,-6636,36832,-4819,18338,32419,-49652,3153,-23543,-407,-46590,26696,22057,-39466,-26014,-15780,35034,39712,51134,1077,-57170,-64943,38971,12222,41526,-14699,84,-61027,-25860,-75653,16528,36957,-79396,46333,33653,-38180,7416,9095,4768,-40878,-17679,22675,38829,-6662,31043,-36115,-163778,4975,22978,-47365,3700,5358,65691,12024,-5467,25843,29515,-53073,-20682,90154,20100,-26287,-47737,60645,-56120,18876,27867,-30144,-43915,12084,-31479,75161,13476,-48981,-47362,74116,-90108,-66979,-20431,-49339,
    -53829,-19545,23318,59471,-57728,97072,-63711,26666,-22144,-3611,16132,7277,-8092,29097,11738,38728,-17274,1875,16643,48238,6269,-38938,32355,19692,14723,15226,-21432,74954,20637,-17,-2153,-50571,43224,-48430,-32274,-33326,3328,-47481,-57751,-24594,-23241,-34538,47081,14081,-6192,29451,-48858,-2858,-55358,-25104,9648,33876,-24560,-6571,-48773,-3348,53165,-40538,60402,47803,-71120,43618,-35101,-32578,-18457,12888,11209,66238,-78955,-63547,-17635,12150,-11845,1526,-85598,308,-33640,-20626,-24814,11409,43591,-25467,-16345,-25154,33778,23243,1420,-8116,-27879,-51492,25802,-27794,-55502,-9242,-26742,4765,-38278,211,-13901,-18324,-84478,34247,-6050,11212,57194,10683,31600,-8225,30777,29380,43584,-77525,-14005,-35206,-116493,16559,25328,48917,-28215,66233,55387,-39658,-20715,-51951,41014,32693,15456,27119,
    12713,-79544,33398,8863,52629,-31696,-1225,69086,-20094,-363,36169,-14445,-89299,42098,60241,74912,15109,-48374,33980,32847,20363,25148,-49926,-79733,33304,-10217,46968,-4371,25611,22479,-23595,-17715,39651,63158,7070,-18702,2572,25984,16432,-36816,13069,14263,-30064,24494,24459,-20320,-16283,-25352,70552,14752,3532,-15412,54856,13570,19567,-62673,14347,33419,27872,-51051,28415,-29362,48903,51203,24083,-32569,-44295,-54911,61356,-22987,25203,-35519,-58373,41583,29890,-27891,21414,22656,-13977,-2561,-4278,-52538,9975,-75846,-26720,3582,65358,-35696,65228,-45732,-1118,50853,-20523,-26664,20032,24204,24047,13603,-35951,5165,84720,4175,-16267,80215,-11407,-4856,-33939,-18459,-83757,21945,-18434,-65014,1748,-20850,-17432,-9720,51212,-17679,-18572,-15086,32930,-48266,-17466,5236,49635,-41247,7212,-10188,
    3596,3035,-2387,64214,52076,39508,6150,1137,16887,-19031,5597,28175,-31212,-15635,-5013,29607,95990,-6190,12770,3743,22274,2700,17860,7121,-12125,27085,-14099,-32750,17206,-41851,18845,-15096,37389,46613,-22501,44595,28842,94478,-22140,18008,-19477,-21071,-8891,-53583,19771,24054,37968,-69068,40317,-5137,5479,-3948,33102,-9224,-53359,25721,26554,4794,2869,5922,-19239,-10992,-2468,-49391,40436,-27283,-53720,-24514,57172,18598,15531,-21480,22152,-59102,11253,-43213,70669,59229,8022,-50396,-46538,-13319,-11722,42091,-11322,28599,-1636,82038,6104,-11957,-51394,1661,-7691,-104799,-5154,26632,-19473,-26609,-20010,5487,4787,-23942,24434,11090,-4206,6109,-39731,12218,-22350,-33724,-63344,-6793,-39464,18455,108146,-29042,-9835,37646,8092,13073,-31560,-31168,29824,22075,-34133,65941,35071,25937,
    -72288,-38897,-34037,60151,17001,-7231,5374,8052,-30949,39339,-27518,-9642,5251,3906,8956,17382,71932,39602,52017,56578,-40610,32278,13019,43811,11270,-23102,-44738,-36429,-10344,35117,-2486,-40918,17682,-14276,49790,-16043,67852,29205,16806,-7643,11576,33858,-22515,20517,-22916,-6881,-44072,-11001,18529,-83959,20490,-18143,-12783,7083,-53202,597,12911,-79326,13767,-59009,7472,-36878,9118,9572,15127,-10460,323,-29778,-701,7554,-7043,-44077,3382,6060,3634,27658,12985,-16867,-1735,-74739,21614,-15981,-28169,13967,9485,33112,16768,11739,29966,-86436,-28671,-15670,39546,12261,53295,-18263,-15407,-48441,-52815,-3335,40704,-44695,27753,-21447,20990,-28694,-32804,-3435,-91121,31183,54081,-994,-94274,-24525,761,-8385,33589,-34803,9876,-25592,-47935,-32221,68525,-13045,104698,60008,32165,-9178,
    14880,-19339,-27063,-34868,-72501,-4980,-38024,-27476,2906,24877,12047,-63186,36044,4252,-47696,32385,-58484,35963,19115,19060,49647,11398,-663,-21050,-46622,14511,5610,-1822,51109,1944,-117376,-11732,-35932,-56618,17994,27637,-29207,12031,-48797,-36939,-55126,72926,-95446,-54978,16152,19153,-74669,-31641,-20615,15962,50702,-7260,26943,36484,7599,19864,-86993,-55867,-12104,-86099,32768,28649,25919,52527,-537,-49576,19753,108096,-31931,-67587,-62278,37139,17856,-54930,876,-87846,-21327,-19118,4921,7836,34164,50502,-29699,-29388,4681,-48925,33202,-33912,14110,12329,-10836,27235,35691,26447,40631,-44960,13406,8541,53232,-30312,34776,-33717,-29503,3116,37812,43085,10142,-8552,-44094,3241,53054,57426,-71176,-28247,-51756,16265,-3835,-2982,-28414,-122790,-10046,32392,27394,-1002,47464,143499,16526,12538,
    26923,-15428,103331,18093,-34646,-34784,114677,-6417,-5307,-88471,-39182,46229,-12494,-17459,-30843,32244,-7316,44894,-11011,-6696,-21652,-6011,-54634,5570,79556,19066,-4667,3940,-435,-26594,-4219,1653,49539,-17408,-5994,-42907,-45766,-87816,-31299,18174,5297,12634,-36814,-37514,134112,50383,35095,-17972,-14417,96000,-44821,33692,-62718,-27869,31164,-28953,72236,91050,93010,16600,51545,49575,6676,-6543,8168,-24710,-27540,3576,37037,10163,-8110,4715,8529,-69043,-31626,7468,16442,-74476,68904,28255,-61256,58147,-69261,39361,-39804,-22439,-1506,80935,384,18038,53757,16300,5238,-84738,-361,27286,-14919,-1334,6724,-46059,-20370,-31923,14633,-18324,54161,66785,-19406,-33058,13772,-15510,-45848,38464,99135,-72462,40297,21175,67001,1954,35271,35438,-13570,26831,11121,18147,6552,41770,-13032,-20146,
    -26271,-39864,12830,12344,-67863,15517,-47334,7515,-24366,-82649,-5935,-39104,42779,55062,18106,-12230,-14145,14455,-49339,29020,-20780,13403,-68158,7109,12256,31680,-12632,21422,-7167,12294,-53135,7644,29908,-17872,58890,33720,8659,53018,-22458,-2437,-49027,49294,-79074,62981,43618,4860,3171,-40697,-1213,80746,64242,68955,4085,-9262,-14484,8346,43448,-89494,-71234,12245,11291,-17511,7256,71802,43742,-10190,36884,9463,-23616,-5404,-5282,-305,19620,34299,-13704,-17200,-5599,-44824,26945,3040,-46269,-88743,-17956,76364,38344,8958,-3878,-22606,13586,-42911,49020,-5566,21687,-44678,-110630,2506,13930,-81020,-22127,-2793,-23122,-22290,-64639,-35925,16984,-13229,-40939,61185,-52631,53898,-24567,2725,-25954,-12542,15755,-47080,60940,-8691,48594,56652,-1895,-15653,-45470,-31968,-4493,-36517,-1449,3976,
    14825,-554,-77197,64768,-63101,-4926,21876,-35963,16531,-75689,-1641,-19460,83148,-14681,1919,-12767,37386,5070,40810,-25654,54523,55098,-5986,70634,-52668,76953,-13051,50999,-26502,47217,69366,-30865,1639,29429,10827,-59891,30012,45701,88062,54749,-34135,29424,40912,15783,44324,20775,6631,-15387,-9035,32702,-80572,20494,-19712,7670,-32510,8413,2231,-54132,30310,13065,-37625,33970,59935,-5347,31223,67486,-56194,43777,55138,37821,-16940,28186,-68583,11833,4363,-1374,-8797,44573,3056,37144,-39221,-24427,-11693,48919,16004,-43793,58275,-7824,-42855,-14923,-32784,-32779,36942,-58449,-102525,-31682,9031,-78818,-6838,-75790,25056,-8686,65509,51657,33013,40745,-23074,56654,53540,-6635,-24155,45607,43027,-71343,24707,-14777,-1113,-93076,-38231,64426,-3978,-29234,-5458,7382,-102876,20645,7705,-57418,
    -8312,12604,-77897,-18775,-26231,-55089,-20886,49750,58017,-13536,-34883,-56888,1952,-20165,21071,-33822,-67255,42933,11476,-16904,-11568,53579,-54800,-8413,65316,-12947,82011,14300,-25110,88701,14084,4895,59759,-49446,-29907,-9154,-59493,663,-7379,-23094,54886,-50887,4036,46663,8723,-2534,-2371,-40171,5289,-7561,-40463,-60197,-42854,-13750,38350,-7958,-33283,37491,-729,110474,8967,-22703,2069,63300,35630,-16193,-67489,50961,41777,34784,12763,25451,19998,7797,80319,29435,38755,-55147,-9518,8630,-32675,2173,-8988,-24142,-4097,-27355,-65495,8872,43313,32207,15343,-11165,-13561,-88583,-1783,10774,31201,15887,12297,29210,5032,16619,-21278,52877,-29363,96467,-15342,-6524,46414,23654,26680,66880,51084,46100,45422,-10945,88481,-36436,79600,33564,-37985,32578,-21921,21241,-55784,-36314,-27596,-9697,
    1364,-5246,27125,-30195,15046,-14248,-50799,47011,-3169,8783,91074,39455,-26523,-72550,-51123,-12906,-13711,-25046,-64113,-30752,7905,19801,-35005,19850,-9528,4778,62310,-23395,429,61347,65198,12338,-14391,12170,1585,-38549,25458,-8120,-1729,-53784,-15522,-23274,59111,15387,20820,-87777,-8471,-9370,-50939,-3511,48644,-46468,-388,3451,18843,-22874,-21521,-9928,11172,-15607,84198,-34013,-29483,4603,47264,41745,16942,40555,-754,-27531,10135,15183,9810,-32773,-325,26141,-23061,38942,-9103,-18773,-10922,21379,20124,-77125,32485,91406,-22304,-1290,33665,-65594,-19220,-18295,-25078,-48990,-46926,3451,10577,42347,12854,-51411,-33929,4175,-12313,69085,23971,-8756,-23250,-1516,54613,70997,-32932,67669,68770,38462,-64319,83284,-44277,-48138,-75675,22298,-28089,80201,-2253,15795,7012,-52871,-4286,-7364,
    27888,2153,-13569,-93896,-31636,70170,1806,-72836,-18949,-21458,-14332,-16796,4473,-37481,-27361,69522,40071,15509,-122774,6062,-26666,-7440,27771,-41033,-33726,-46387,-52542,27598,67966,30247,-1781,58941,-31792,-25404,-18776,-35841,-19934,-111150,22703,14394,14390,66592,8720,-95885,32470,36928,-57648,40467,6529,42143,-59156,26416,57926,21521,-33328,-34448,62047,-47308,-28185,8487,36447,26480,16231,35874,23070,-42016,33702,-35942,-42345,1199,98433,558,41070,-11611,-42888,-73734,-58018,-35801,94259,393,-40820,70892,38461,7782,-24259,30222,-34532,54766,-15276,50041,-11512,49726,-21468,70585,-74513,-2262,21599,-1313,70100,-11199,-47398,8079,15739,-11009,-26430,107008,3970,80533,9534,28754,55190,1716,41465,-18417,15984,8307,52690,56073,-14867,-32586,9907,72099,-38617,11424,-33019,-25830,-10320,50707,
    -7940,27019,26699,15674,-45979,-28315,46258,10808,13729,39764,-32064,47271,42688,18428,12225,37408,-3330,15277,-8346,12836,-46232,45945,26507,32633,48557,-38935,1462,-13447,-22081,18861,69023,-17578,-37047,-54929,22373,-30122,-12437,-19311,6772,10381,58396,68896,61231,-54223,30848,3575,-16576,40538,13693,-7638,-49308,17578,-15830,31200,-32644,-34219,-10065,53771,-38785,-17066,7656,7464,-33662,-80102,40813,-41058,-40520,-15583,-7037,-25589,-74364,85471,-58229,-32363,-65170,2579,42764,-22920,-4699,-28449,15466,46149,-26346,-41604,-59823,4602,60137,-12831,-88975,2327,24613,58941,68748,-15213,24780,-2565,-6498,15165,20692,7867,-38753,-13855,-40523,-14740,38023,-24962,-29540,-25949,31488,-18309,25148,-51484,-55580,-18063,-72739,-9718,-62578,24518,18276,-12362,15759,-35630,19377,27990,-24289,22064,60780,45869,
    25908,9862,-26164,-7643,-36105,2125,-14763,-1594,-22978,-8945,-33665,8871,17492,13010,86845,10719,37616,-9947,-25759,-64698,-39859,22664,-22954,25344,-45773,-29017,-23343,1653,40491,68878,-29504,-31061,-24617,41469,21526,-4975,-42646,2007,12094,15061,-1852,41154,-21140,31898,-327,79431,10579,-51431,26680,-37136,38307,-2193,-35061,-22686,-29137,48935,13909,22044,-81031,16988,84103,15476,23138,79098,27738,-33948,-32438,16123,39420,734,67241,-35246,-22468,-24919,60689,-36442,-48629,-27540,20259,-57540,-18655,14285,55657,69726,19372,51127,40833,-12788,-52427,72335,55437,24565,11461,27772,-2587,-48190,12963,-37519,33236,61585,-27982,-12342,54259,-19127,3657,41432,24580,-27870,-6040,41600,-62211,64405,-10305,13367,70460,4591,32657,19395,26770,46713,-55558,8464,-29807,22535,-47304,41586,-2795,7834,
    7720,17476,6412,18262,6085,7230,15343,8565,-25421,20063,27713,-58135,-36880,7447,-19692,10910,-15449,-12944,83815,-8183,-20376,-5581,54489,-13894,-12914,4392,-17068,2896,-38786,3475,31690,23377,-13589,10822,36746,-63440,-43100,-77889,-23505,34418,16891,-68659,48616,-60845,6952,-15424,3027,16376,47108,23588,-67480,35600,20543,53178,-27149,-37214,61553,-5849,-6366,13860,39401,-26325,-71340,-28555,-47378,-45108,-45187,-22726,9765,-26359,-4538,37483,33665,-35068,-28916,35291,-52975,-25589,-18289,91392,-21958,-8333,66166,1709,29061,28615,7018,31169,-42031,18620,-11339,71683,-4251,-37117,19809,39227,4176,35894,8252,-4617,-59000,-27593,51672,-20919,33410,7518,56411,10003,44023,-5789,28887,16844,-33858,-4104,-63227,6400,-53050,1711,41367,3503,36528,26534,-70359,5440,76584,-83388,-39923,-7909,
    37478,28887,57761,-18184,5735,-50245,1649,-38740,37806,-29496,-5202,44591,-9720,-6379,15638,29851,-46508,-102022,-24990,-15910,-33139,33972,-33715,-15372,-3061,-3337,-40967,-31456,109954,41020,46032,-14487,-22839,-1111,12425,-39634,-51207,-37617,-80660,53557,80934,5821,47886,-29891,-17773,-13521,-13244,28258,48110,-64748,7867,59973,7114,33033,4890,-5275,4750,42469,-16344,62055,25316,-455,10896,-1628,-51268,64049,-3688,-36368,-26038,7675,-19652,44795,-22074,48136,-25313,-5071,-60545,-10670,-36517,-4132,-28228,49742,34202,-22096,46491,-17662,15020,3807,-43212,60356,1428,30275,7874,-24739,-9577,-8093,47434,-1661,102724,-18759,-18835,14212,-51752,41266,44888,7323,45509,-5751,73021,-16160,-47779,-12467,-49676,-11839,-6947,33840,-107706,-39752,26041,-55663,79932,-13900,-7225,-3393,-31592,-52536,-1909,9443,
    16194,-8564,-35311,-33136,45578,5785,-7909,-55524,31304,37648,-62652,-36591,-2330,26305,41545,-26667,23607,-27008,35838,-31389,21974,11037,-95171,-25337,-32310,-5329,67945,-55137,39975,-14822,21248,34630,-10298,8291,-4612,63416,35130,89979,14634,25966,7432,-12398,-97985,20668,35352,-12397,79054,-26765,-23548,-4696,93895,37976,32842,27368,3331,43259,-62871,-1009,11335,-70090,3313,6243,-3471,-4569,-37110,-7430,32177,64296,84048,-48163,-13446,49119,24248,38830,-17081,-30964,12044,52800,5384,-71167,18900,-77016,-867,-21584,951,6271,-14235,44458,-74802,97905,-54753,26029,-24314,14695,-8141,17141,18940,1636,-42362,-5125,40268,-8470,7987,-22706,-37857,-18846,15232,4413,-3021,-45431,15980,91665,26100,15324,-7797,-67855,34689,59737,5875,-70075,-2996,15648,12603,12935,-10919,-14390,-4677,25882,
    36816,-19049,70615,-5176,-39875,-76108,13910,-30656,-34161,-74086,35623,18119,45496,-52453,33638,-115526,-41937,14363,-58391,47997,81315,44529,-756,9619,-44056,60885,64332,-23073,-8933,-12949,6298,-60645,-15004,-75424,-21336,-24769,49926,14408,-21075,16953,-40925,13892,-91598,14574,42249,36020,49661,-59721,-33854,56167,6625,-33045,9577,25604,48307,4938,-10426,-12632,65401,8596,-2124,13247,56832,-13386,-56902,90960,17344,48698,143619,8082,-57382,-4761,-1197,64527,-18432,-7447,-15500,31844,29246,-71545,-102604,-14629,-17299,-16539,-25300,-69391,82544,15483,-33424,58387,-75368,-45357,-14375,15665,-26879,31094,71415,-22314,-16338,17768,83264,34559,14568,36498,-28364,60504,1191,37296,-58255,75589,-81969,7448,18746,7398,17272,-6272,38940,64787,-22115,48465,194,31487,-57580,-3489,399,78251,-49490,8985,
    12812,-9498,48757,20106,67381,14927,-25504,-29970,-87701,-25275,-88320,-62315,-13332,57597,89538,49647,21837,39763,24372,-6341,-80936,33567,-17201,12352,30307,23215,90344,14893,67226,32248,1997,38833,7817,43532,54476,-26880,-9024,11845,-24454,-15723,63626,-32029,-72378,1382,-12371,66737,7774,-82722,-39299,56430,80181,6227,-45637,42,-77346,82350,9419,22226,-67975,-39304,84805,76390,-42035,29754,-36787,-86987,-4772,10078,96286,-9255,6286,-4753,19019,-41784,3939,-37228,5263,33964,-43314,92629,-47755,-67816,-21188,-5587,33663,2000,39109,12847,2824,12595,-2815,48086,46482,17719,-55677,26096,26466,-22746,-74118,-53071,2599,13940,-10145,31311,42738,-1579,-10242,-20371,-2632,23948,-73847,129758,-76220,7083,36115,33184,-12037,-14582,97505,44657,9370,-89697,-3210,18821,-18799,-4466,-65561,73288,
    27459,29397,87201,-48569,37498,67631,23789,-12226,17834,-4127,-106118,30203,29824,4787,-24443,26750,-15109,41207,-30613,22131,26975,11367,39012,-37617,14621,-51146,-16168,34556,50477,23059,44675,-14208,8353,-29026,13470,-30322,-3337,-52900,31982,51091,34203,2352,65836,-58880,37171,-1146,80760,-32657,-6544,-21921,-60114,-429,-24855,-4157,52898,18715,46725,-55941,73430,-38280,-3563,44809,-8053,-24690,-58171,20757,-74,2559,12473,-76119,67361,-15727,44598,76454,-69038,23123,35209,-59559,7095,-753,-24116,-28282,-3101,-53286,26359,57540,-28560,37561,-8466,13097,-15653,55318,18098,-2098,-87206,34993,-657,75504,-16585,-40702,-30330,15383,14256,-9828,10097,30870,39576,-61825,38749,-10564,-25400,-41260,60566,15820,15438,33772,30097,42395,-71491,-40031,13931,29266,17223,3571,35065,26822,-49267,-21304,
    25279,-150470,4193,38874,-47366,8754,15860,43005,-43568,7305,-52217,45451,-4257,-5994,-88279,45319,-39581,-94202,-89795,-16779,4878,19171,4034,25876,-4900,-19480,57413,113139,38178,-41442,-17766,-5586,-32628,19303,15452,34906,8919,53378,97722,-19432,-4164,-31577,24817,-1891,-52152,-15683,-31590,28725,-75417,1618,78397,20146,-30313,-40656,-60592,3636,44610,41860,-91783,-64903,14946,-31524,-20073,25397,54497,-43566,-778,7869,-69954,12535,22964,-20645,-135933,-20122,45446,-19875,-729,-5224,16841,62163,14201,5847,6319,-11979,92714,21338,14047,-43142,-4910,24073,-4905,32682,2563,30007,-129851,-18267,-86427,29375,-20267,-37795,13184,3542,7137,61299,71381,22473,-47099,42714,33865,6039,-47399,31621,-22680,37133,-28264,48299,26823,-23094,-104834,23724,40322,-31456,-15925,-39055,-45380,-41817,-31325,-52523,
    -9825,-2666,27932,12678,44443,62610,5033,26987,30482,29712,57070,-56436,42088,-24973,-100710,16119,-16602,-26095,27344,119904,-32930,-12568,30836,28662,-25737,4647,-24974,42914,-58320,-102235,62,3280,-81568,-37017,40834,-18520,69959,21273,-88686,-12061,-16111,-27914,17934,31150,-40991,59174,-46835,-45886,-5922,-13803,-8925,-48383,-3180,41317,-55873,-16145,-20965,-39022,-83963,1685,105382,52231,-21733,30659,-29385,4079,11289,11427,2184,6607,-34529,40695,2531,-30971,22438,-77296,7980,30391,-11045,-55634,14907,26880,-1931,67480,44355,70545,24102,9320,-26226,-32702,-68803,-65163,10786,37963,12325,43861,-57546,44644,50273,59670,-15211,70020,8244,-4019,4943,-18733,54088,42170,-6686,15656,22614,30720,-52410,45648,-73539,19047,-60301,10638,50877,-20633,50179,43459,20157,15565,-3371,13148,6809,35473,
    -29972,38796,13515,17308,24091,-22938,8698,-36522,-40558,29494,8561,1219,19807,35554,28912,39818,8840,-13301,7304,-1255,-31701,30983,31856,56808,-34173,-67886,46080,18599,-61192,-15110,60771,-35718,35507,38232,34776,53790,-4919,-788,4036,-12991,-71861,5288,-37640,49952,24473,-35187,-10280,37128,-40101,-86720,50534,23013,-74158,21723,-58272,-10199,-33697,14389,-246,60908,18850,48834,-3597,-26035,-122135,87137,-470,24402,-29047,30239,-40606,877,-58464,1932,-8136,55215,-35866,23914,-27980,1818,19192,-21454,61235,55743,-24096,25419,-3052,19370,30607,-14202,56438,30155,-10792,98368,8803,50469,-15383,42498,-76969,41221,-46876,33392,-4676,-55656,59587,41276,-14189,-25416,-27529,-9875,-4766,39488,-46637,-75024,-70656,2995,34713,36106,-58483,-874,-43450,-46993,10732,5446,29248,-18425,-24071,58644,
    5022,11461,56218,59940,-66631,-35334,30080,60822,71929,28357,29862,-45622,-32700,-22218,-40842,-81152,-23057,12040,179,38023,62441,-80348,27236,4827,27541,14261,18096,-49660,-23846,32828,-13773,6794,-22213,36398,-18101,-40869,-5503,11645,63517,47714,-38677,-4004,78643,-7749,-2261,37827,30540,72528,-59288,-9203,2010,-42690,27877,-12475,45157,15906,20579,58,96709,-34219,-10686,14041,19234,-20291,-17346,-6243,5740,19709,-47798,5740,-52246,7983,-26153,-21565,14328,-30896,4845,84999,16596,-23269,2694,33665,20107,13896,-36697,5893,-35003,44939,48022,-98958,-11509,-10594,-21977,-77913,-17903,-24783,-28924,52559,-24875,-32708,32412,8666,-17224,52618,68456,-43149,43192,-51515,21463,-5219,8498,40258,-2311,-12711,-57226,2311,13847,830,-27450,-12016,1095,102621,-44765,-28908,84199,22919,-52948,-82647,
    -50986,2010,-30549,19096,29861,-7068,59437,38138,-6166,54069,6666,8937,-48907,-34954,-26050,27555,-15107,8800,38274,17103,18630,-16144,20915,19741,-17988,-33767,839,-14887,-6748,-54943,-36705,30069,56904,32044,17650,-60269,21146,-23384,-11729,-1255,-42937,-67577,-29160,-27115,-68623,-16196,-42522,21210,-8656,-104563,35367,1008,27507,2030,24865,18557,14027,93875,540,-32137,-10541,1204,28199,58245,46845,895,-4897,246,22730,-39832,39925,-26872,-54849,39538,-25698,-48948,2916,-6080,70419,-29160,-55122,14113,-17394,-45549,15406,37007,-31385,24677,-13918,31931,-14455,9645,-25629,24602,48911,9025,10954,35453,-89856,-32145,-15865,-29832,16127,46889,111066,20209,-22444,-37468,23661,7308,7890,55209,21454,-6370,79779,37709,-13413,15138,31,-14007,-18637,-46581,-21394,27302,-423,-10504,-298,16679,
    62885,-20559,2994,-12784,89575,-11529,-15349,-16639,5519,-12642,-60200,27735,-5570,20296,6417,-18673,66143,14326,-25184,-58613,73228,-48705,25846,-44071,-13744,77394,-44423,32742,147784,-3596,13017,-11400,-36961,-30706,-21901,33234,64191,-1416,33236,18106,54750,65299,4471,-69644,-66778,37855,-6804,32677,67266,-17146,-7067,66308,13508,-38312,-22539,3487,-64813,-10056,9145,-77384,-1992,-61009,-9320,-64535,3918,-85050,17129,6799,30888,61828,-52768,18611,-11767,-3879,16845,-35894,-10544,61551,11405,46032,-19339,57570,-38441,81653,-43390,-6108,-25582,-11701,23396,102355,-73213,18250,23740,-8447,-20089,-43526,-14055,-12189,17565,-41772,37594,7043,-20409,15624,-36770,4087,37404,37853,36515,-6073,-28035,-12183,45429,72238,-54863,-38065,38412,-27873,12478,-37731,-12261,-6865,3558,58961,-35662,-24094,-37556,19683,
    14936,22515,11835,-2711,-2230,-43454,31767,-85059,35570,-10501,83267,55766,17649,-54346,34553,98289,-5907,-6070,-20453,41531,-98881,-110,-4912,-48073,-22797,-63509,16071,36704,-21898,-32868,-25806,45985,30883,-91515,2113,56604,51873,-50910,23899,14480,-29473,-38170,28300,78366,43343,8288,-2193,7134,-26815,11790,44268,29840,32846,6323,29837,-118093,-18802,31397,-28638,2797,-4955,8255,2844,5349,24876,52775,-41672,513,-16051,-644,-49619,58550,1176,-17767,-95672,82046,-2567,37153,-20092,-5833,-52728,-15909,-20515,-19501,-52138,7648,9439,-22776,38623,-36270,98217,10826,22706,70195,-5294,8463,47461,-39650,25959,15178,-9990,-23894,33205,4468,-42936,22199,19563,-19129,-5866,-4822,-16641,67597,33111,1754,-24401,-5777,7975,5566,-51502,-37872,-77461,17892,167,89500,-47252,-10156,21814,109661,
    -20086,-19092,-21337,20379,16378,22290,7059,-101669,-15127,-1906,33706,15741,-13451,77649,21755,-38454,1654,-6201,28029,40196,-22130,54735,-93623,-36456,14835,17236,6728,52865,-27850,-52548,27237,20217,-7658,-19862,10685,42512,42048,24499,20693,-22165,26773,-38434,36127,78335,-18289,-27605,-10316,39776,-35713,-6559,40694,-2427,12227,-27882,-11461,-15407,-35011,82874,5488,-67453,-11833,-17621,21821,-29225,-43340,-26334,43324,-29963,-29511,-10964,-34640,34008,39130,30744,-22229,-12173,2095,46849,48608,-86535,-2429,4966,-96050,-17941,-19972,36655,5366,50943,-95592,40391,-68031,-310,29863,5978,6931,23986,49184,31589,-59242,76609,5179,30842,9746,35124,-16139,-21714,-43612,6786,52268,-14127,11865,15528,-6179,36721,15200,32172,-20818,46347,-11684,-48066,-53373,-1719,16962,59686,-23635,44011,-88886,-34591,
    -26392,-1696,38485,29119,70801,-35242,-64193,145762,42036,-49407,-3991,31512,-28699,70697,-33703,15478,-5383,-25495,54524,7669,-28208,110098,-42773,-41692,25790,-10678,-12281,-42169,35822,-26739,41900,31302,-66486,15829,-5697,-10544,12627,-7957,-85824,-33118,18633,23318,-62226,18769,2069,-23951,25335,-46672,72483,10324,-69270,9664,-45480,-4750,-29326,-33729,33232,-8901,-30665,87155,-43550,348,-24517,-73874,-29128,-1693,-34853,-18890,-16902,-10432,24301,1273,7814,-14729,-3489,46655,29805,-59511,-53403,-37759,36515,-14182,18278,91457,3604,-5508,-31196,54559,-3646,13028,341,-70552,28957,-11301,39253,-31601,3498,59222,39425,7660,42999,39865,26759,7030,4239,11967,74983,-9749,-1182,-28970,28502,-10942,-43510,-9218,6901,-27147,-8165,-24199,49416,-14322,-40436,-39675,45486,-12745,-29862,-15874,-14006,-84,
    -17530,36916,-49479,46771,-27099,38770,12816,9534,107162,40522,50295,15685,43024,-85655,-4540,-14449,-27065,-43880,41956,-30458,-56304,-62112,50711,13137,-6699,85538,-19654,-5659,-29424,9925,21298,-34795,26580,41531,-61433,21928,-53246,9379,-18641,18343,29139,-53019,36211,20244,-14455,28733,-6116,14045,-26926,-22024,7940,-48738,66467,52144,-2627,-19921,13274,38985,24780,-3105,-14979,20500,-31139,-11053,50188,-12093,-47934,48306,-43430,57,-82865,59559,10038,-52784,10164,16649,34651,-5136,-83491,-66402,-16343,47860,-23816,23440,-17095,33259,-20254,44392,25180,14860,-8171,5268,17735,-66148,34361,12114,6199,-11172,91265,-36101,-36416,73458,-63337,5180,-5871,-52165,1261,1757,-23011,25218,62440,-12849,-29752,-3180,-24291,-67453,-28691,4949,18024,-25446,42387,23129,-7561,10367,-13599,-49817,14143,6204,
    -9531,-48957,53623,-38509,31192,7759,63586,55810,38176,18748,-34094,55811,63690,-15405,-53921,-35482,65981,-35389,-64463,11386,90320,-2854,10225,-26097,46455,18013,919,39000,48622,-13348,-20579,-36341,-1855,-46847,-76955,51293,1988,38016,-9644,-13605,-23367,44122,14436,-28687,47443,11794,-70393,15477,67019,-52062,-65150,12545,23120,4702,-12998,-34054,-55686,-7270,-3688,32793,-23562,33510,-48388,-16228,-2704,34393,-35007,-23875,-29840,6362,2188,-11694,-7840,31017,28599,24953,13167,38387,35719,-20511,-8407,-4837,23838,7149,-7740,-26158,-43507,16786,32788,-58349,-16725,-58164,42498,-45717,-30096,-93867,41059,35178,-60915,-52436,3873,3194,-55199,10591,-30263,30424,46329,-1456,-18220,-67923,-7801,33218,41215,-58650,35278,-1915,49270,-70709,-1871,706,48549,63747,8791,41075,-26350,-41917,-38127,-29904,
    -25734,11202,-51626,19242,-12357,39377,-62236,53674,-13809,-20044,46484,-59386,7534,-1296,9484,-32081,16940,-24311,64380,-51295,-7771,-13252,58388,-84922,31562,-52880,-16191,-25690,24651,21320,-7298,3062,14811,-38413,8523,486,26225,30655,-38930,-24008,-44725,-25835,858,10165,-12151,21274,-18664,-69387,28575,9848,-15620,-15849,17023,4026,-19568,6323,43828,-44924,-27654,36123,-14542,45641,-10281,-66350,73759,22272,26891,29460,39693,3802,81375,-47453,24473,-22292,50571,20052,11254,12874,-4133,3105,-3508,-50664,36384,13630,17526,16481,744,-20979,-14573,-69124,23498,-27148,34297,-2426,80392,-45938,44423,-55557,97575,33288,-60910,-69323,54950,-18789,2748,3584,-56196,93996,39880,6647,1572,14684,55990,-3363,74680,-77251,-31531,24194,-5790,25212,-45347,6968,754,29589,48906,63724,-12722,-35859,
    -7753,-38902,13524,-11696,11978,-24141,-91011,10067,26678,-15444,36236,45289,-41626,16725,-27947,-41796,-16801,72164,-50943,-24012,1091,-22069,-60671,52352,65114,-12971,-28167,30424,-29510,-21697,7801,34515,-2153,-4081,8590,33461,21794,-17864,8562,22306,44609,20066,-34888,-12042,-17781,-2625,22762,49569,-14144,-19758,29415,43884,-43786,-45335,-57873,8345,11517,341,-24070,12905,-64731,-17489,49430,-8402,25111,-37074,32489,22446,-48870,17165,10401,-41976,-22182,76424,3518,42693,4157,44310,-18928,46009,-2656,-358,-42816,49390,-42671,-51799,-55205,-15206,-42783,-25074,15028,-13777,37380,-51709,-736,-33321,-47183,8464,2419,-17715,6073,-4273,-26805,2957,2536,-21359,21255,-11679,38327,-23335,25956,-98208,-13113,36308,-67624,-13245,881,-45841,-21934,-40985,-33322,13170,2001,-83716,57518,28511,-39769,-71175,
    7590,-44059,-31030,-45049,7448,22209,-60323,187,-6294,-12080,14792,-19458,-23713,-75267,-66272,-2738,2483,-36262,-7443,75383,18831,4393,51754,3955,51095,33269,-9504,43239,22826,46844,46390,28837,-33423,72376,-419,-11870,-5097,-54426,544,8038,24330,-2104,73892,-29392,70332,7907,-59577,-12015,-45997,71304,-26364,-23685,-21477,-6051,-47104,37329,34766,-31713,79031,-49668,6130,-34695,34203,-81340,8930,-50109,1847,79942,-105434,14067,16488,-29570,-32986,6865,-95822,-29770,-46951,20031,-41102,5778,-43293,3142,-59069,-32924,59069,41809,64603,-30450,-5373,-42060,-36686,22588,17165,-8655,17029,-97472,-88131,-9520,31786,-30784,17780,-3793,-56465,-29729,42984,35926,8366,2986,7307,42216,-43664,59233,47014,33929,-12522,32032,-24497,14568,-76729,-17511,-16157,-28446,-16940,-73743,-29900,-50738,-14169,6236,
    27850,-587,42655,-10328,-26322,55927,-49130,17116,120,43051,-1953,-32243,112,12724,50382,-68038,-17951,44130,-17298,8259,74034,1887,2447,8603,-32356,17286,7362,6442,66738,-68233,145,-80348,57616,11445,27026,-61313,27310,-49510,15149,62333,-111440,19819,-96929,80213,-44728,63204,7552,-24296,-80835,12020,9545,-41534,3943,12447,1126,-23714,-79104,24223,-21075,22557,85577,-23560,-7987,22350,-21216,16743,26387,-5065,98140,20665,-41294,4724,-38778,19989,51432,35756,2773,64510,23274,5727,-17357,-63476,37204,-12685,-19442,-18934,28002,-113560,27002,57163,42859,30990,-25330,70833,53875,-59702,32990,1565,10088,2547,38862,3017,2859,2458,23916,-31373,-4109,-36065,-101498,85082,-91832,55641,-51810,-6256,43097,-40873,11645,-39599,13915,45034,17622,5895,-50176,-69687,113160,41467,52349,25735,
    -26007,-19026,-16685,98952,37643,4256,36804,-63865,30955,65873,36049,6310,36267,47312,-48985,8653,-37550,-18517,-29640,45805,-32368,6936,-41879,25286,-33700,-10478,-73314,-3327,-59759,-9758,30916,38095,-20638,-22391,33053,-3208,32737,-84717,49318,89528,-10362,12777,39960,22219,10173,-36234,22119,-14460,-7858,44753,-12559,-2671,-59153,-16701,82432,13577,15497,-45445,-23687,-20785,12081,37941,-37894,6716,28456,-66114,-8122,-94041,6268,-74396,-26660,69047,30839,-108966,-32421,30354,20086,-2071,-2322,-3443,-14871,-15697,25388,4802,-31357,-35844,-93096,30932,16831,-39940,94182,44985,11756,37967,-31256,-1413,37455,-22824,-10676,10127,21934,-104903,66846,-17979,-34845,15813,3907,18701,-62802,34919,5243,20371,-15827,-42159,55195,-28783,-11983,-64896,45042,-45483,-20951,37383,-35444,13445,-9641,32622,26595,10671,
    -42928,-30456,-3681,82844,-16944,-26245,34324,7130,-28783,4383,-80653,23099,21559,53702,-562,6078,92845,38429,16178,29606,-10858,-15394,81318,6853,-4865,-44235,-4429,-7887,-8049,24308,-14281,-39279,-26575,5687,-37933,-36152,51490,25154,-25937,24724,26884,16324,14697,-5180,-62689,-13101,-72639,14990,-5836,18879,31102,11722,31955,23479,-55360,-32488,-10530,34211,-42519,68531,-33963,-47357,2989,25366,-1884,52190,-47561,-21252,-21099,30392,-87977,-47313,-70472,-84954,15776,-16246,-29165,-24993,32987,25181,-8722,-19594,31948,93341,32745,-30696,46025,14730,2645,-33050,593,55652,-24155,344,4635,-66469,-55188,-1045,-38393,166,22851,-33402,33345,15326,43757,5642,-68627,-28295,35892,-33934,-74725,-34557,3133,-22624,29827,-48610,51551,-51937,-14363,-17045,9574,-63986,15886,-1703,2242,-35280,-28706,37147,
    -39030,43188,1125,-30663,2203,58824,-45057,-78240,-61872,17135,54535,13733,-13942,-50118,19584,59801,-17902,-5836,-67049,-6277,44251,-22884,-31207,38900,-51588,-24894,-18720,28567,-23718,8897,-26592,20239,63007,18571,14672,53602,-16811,-53786,18463,-32717,-54909,-4731,17673,-46182,14258,3907,10816,108609,-64417,-48956,40474,43922,-88449,-29359,-23930,49975,25496,33659,21328,50301,88257,69019,-8375,-49277,-28159,503,30181,11743,3269,45011,16882,5798,7397,6652,17116,4152,5433,-31663,-11033,28130,69113,-17815,6415,-46092,-11522,127423,-75030,-65312,-13007,21209,51659,104089,-41968,21174,29127,63151,-38860,1838,13199,-8815,-46336,-8108,7347,-61556,58318,-3327,28941,39699,-10040,-19968,49493,-13790,46596,8895,-22798,52597,47613,36719,8878,-23594,-29456,99331,40347,-11145,35593,-24559,10849,29415,
    -52758,53645,-23932,-49623,-32263,53776,4363,-13495,14048,45516,41937,17246,1373,8960,-32282,2099,-42910,48770,-1663,2466,-10609,59154,-32437,23949,-6879,-6132,-93479,18624,41498,17590,71083,-31132,-9067,-10650,7991,24708,42468,-32717,51906,14699,-1178,41857,13189,20620,-28106,5304,-24596,-26419,67803,-63976,-32275,20643,70256,43403,-23676,-44617,40009,-29029,8768,-11961,21123,5800,-8841,-76526,68197,33895,42662,-16508,-41766,8654,44489,-41034,40507,38540,-16856,64008,-26079,-39492,17596,-51040,24245,-39402,4997,-78357,-14127,33888,-90974,-28023,-38649,10461,129158,-1518,-57297,74293,17951,4943,28250,41519,30161,42566,-50956,-12267,59463,-36208,28002,-4201,-105039,85888,32644,-51392,36462,52587,12330,-15332,13132,32394,-46527,57776,-90124,33033,-22601,-31718,57771,39649,24074,23313,-2947,21729,
    2014,-38640,8036,-6076,49213,-30283,28402,-22853,9475,9001,-15837,47509,88175,28343,4910,2814,23557,10213,41078,11386,-18900,-1572,33596,3093,1747,-57346,94171,228,16638,-55406,19686,19502,-75797,-75703,-21133,-36519,13984,8805,-63094,-30164,-66074,17668,-15498,23293,-34508,47953,82592,46847,-3099,-83727,67893,23213,45185,10125,-52805,-17571,-45411,6579,1581,36266,-2137,20600,-70747,8846,-31841,-79704,27284,-43840,-112774,-27264,-48590,44948,-37784,30918,-38669,-7234,145682,9237,90481,-6828,44447,83981,57186,-13366,4442,-37838,-26052,-82498,-71096,56762,70339,58710,-7949,88595,10665,29894,-19491,-57622,-32443,20107,15462,-28980,17949,-30357,57686,-18652,52626,-18813,-88349,-44201,18879,806,1380,49655,16827,-68263,45640,29281,-18174,-57440,34550,55925,15290,-34881,100747,26753,-29515,36508,
    -50568,-43907,-22995,-14269,-45294,-28682,46333,-36829,-16401,42506,-6595,-17350,44284,-56389,64071,312,15129,-49389,62366,22103,-26636,-35132,9425,32669,-8305,59023,-2011,23118,-28770,46457,-24110,60579,6294,-64162,2377,127973,-29764,43909,4600,-56210,-5242,52972,11962,21664,20517,-15685,-289,-29342,47758,-38885,29944,-664,-28781,5659,38031,54757,76628,-43153,9489,-77397,22625,-8799,31479,30625,4851,-66322,34534,-4830,11158,-69761,-65712,29777,73132,-77942,-14374,24912,17677,-42636,43106,-20779,-24435,-70664,17144,-60542,-29192,12156,19487,82637,11984,-7903,40303,75844,2679,-73922,-17164,-110879,-26220,-53338,41228,-9074,51148,-57509,-10054,74693,51646,-50107,-34230,10540,-30068,-85984,-10326,39421,77154,-89064,-7445,27372,30421,69343,21932,-4130,-92385,10737,5039,-15848,-23210,60377,-29446,2773,
    39650,-18904,-36089,-8582,-282,20588,-74709,10214,-19558,-47480,-47,9472,-19554,-68827,7880,-12570,15543,6650,-38906,-57204,27246,88916,-9528,-26457,75102,-13928,44507,-32280,64299,40529,-95710,28257,-8383,-3332,16354,-124556,12338,289,-6745,-19839,39271,25843,13066,-66473,-64310,-7085,-4279,-31993,0,34007,-18872,7118,-28334,-20385,66409,-10423,-10298,15585,27737,17483,-6244,10482,9031,40166,-1399,-26954,-18234,27910,13769,-5635,44973,-19398,-27137,-4561,11620,-37353,-26948,60141,53589,-6680,-11639,24428,22255,-34384,-14001,-66523,23157,-1347,23362,32561,-66886,61625,23150,-33534,-36204,-10816,29135,-85344,-59826,-49599,23746,24230,49020,-29350,-14994,44432,-12683,-27333,26784,50470,-8002,-57081,23188,12744,58067,11644,50206,81709,-6224,17782,37676,-5503,-56167,30316,7878,-21931,-13418,24079,
    12967,34405,-27598,28112,76378,-9267,21808,-119411,-10317,-10333,-28747,17792,-32674,60026,8326,19750,-41606,22409,23160,-62664,21660,-37362,20599,-72760,-42768,3592,-32743,-13646,29985,882,17695,49482,44881,-1687,-36432,31782,52615,-13681,58114,18034,-37998,-46903,-37011,-20081,-20976,-21218,-39462,-6997,-17795,69222,-42070,7218,14203,57551,-3240,27237,-17424,14321,13207,-7633,-34849,-54585,15191,-47917,-90054,-37345,39571,-23961,53511,-50054,-31927,36923,-50020,8448,-130525,-37310,33571,-39848,-29851,-85464,-2154,-4261,-72821,2705,6909,-21650,7493,67515,1918,27886,-103011,82466,4923,57682,17588,12002,57162,33457,26916,-22604,15258,-26958,-43969,-38590,-15825,-100452,37216,-52513,-62521,-22529,-8571,-93845,-18170,51960,36873,12596,22456,89532,13181,-27872,-58281,58058,6434,-55825,-29472,-19729,-7589,6794,
    -41012,-44476,46359,-10985,13568,25816,-5171,-5138,-48772,-72804,28078,-42382,-12714,51678,29707,-47812,72523,-32013,485,-58743,-10902,-61770,11374,-32228,-20675,-4306,10866,-12656,-21579,-19903,-15990,-8827,6573,7316,-10333,40831,-34729,41735,31839,69383,-11829,4372,-2957,23115,9930,51825,20687,-38977,-17641,57214,-31834,14655,-10624,18513,-23139,39324,-34907,36495,34042,2063,-27074,-25909,45553,12509,-11785,12267,20733,-47500,58524,34245,30723,-8924,44108,14576,28819,11462,-1013,72669,23349,8802,4215,6874,28778,5211,-69174,23721,-8350,47877,65197,4075,-31772,-19681,-11745,-41881,18106,-130452,15619,-72976,-10024,8949,65724,-5238,6042,-21051,-72513,-69829,-7054,31601,9891,-45053,-79904,-3717,-9824,-39775,46308,-76287,15841,-351,98563,-49257,-83252,55913,24581,-9372,-36496,-45605,24576,45648,
    10421,16064,-57596,-26197,27480,45445,-52986,8608,65113,28722,38520,2316,-25655,4530,-66007,-68135,-68578,-56135,-41621,-30132,-8929,30473,-21618,41074,49027,72386,9621,-5860,-16690,43451,12402,3017,93906,-11932,17718,63865,-41900,128983,-40334,45422,57581,-12151,42824,-6755,-6904,-29903,1675,-15523,9228,14124,58071,-13412,56027,-41900,-19947,91824,-63934,-28954,-39439,-8545,-23800,-25005,-26257,-46256,87858,23809,33261,79150,6523,20683,55361,36529,-21933,-37898,117665,-41981,-6644,19064,-11208,-106216,-48522,-3296,63196,11652,-41907,54359,-68769,-24480,-31222,27600,-49414,-33630,53700,-40087,51256,-69791,-43957,6868,36880,11633,-51497,-56821,-28855,-19741,-12468,23938,-38932,59063,37410,49842,-51597,-36752,103541,105163,268,24344,-69730,79844,-13575,28835,26734,-13267,25676,4147,-36174,10737,-6007,-15622,
    14426,46401,-48626,2257,32363,-52358,6211,-9819,11629,-7336,-61634,99051,23311,34956,45624,23802,33972,48348,3534,-12463,-22085,-35330,31363,19284,16537,-50534,-48002,-69306,-64651,30940,3225,-79265,61095,-37483,-950,22715,-45044,1646,-3711,49310,-39235,32413,34460,-14774,7804,-31812,28861,56621,9215,-25545,7653,26046,-73597,-31295,32193,44484,-32845,14900,46029,7953,-5221,21530,-73223,-69502,-14989,7459,-2453,-59062,4598,8304,14836,-39885,27921,-32378,-27540,21216,-23118,12275,-7955,-33731,39089,-6787,40101,-6821,-37560,4622,-14964,55003,61012,-18997,4796,-132,28220,-45208,69725,8506,70568,-36306,-21427,-26112,3347,-112561,-9681,-78376,1677,-15375,93039,-57095,11589,1558,32340,-32075,37975,615,48147,-10576,9286,14785,-52210,1852,9854,-29890,34011,6051,34330,-6063,-4382,-94497,
    -11094,-5397,62200,-56709,66624,32856,59260,63464,46249,74746,-51641,70119,-82185,-37666,-9956,75739,66447,80742,217,-45908,-64323,-20007,21370,7883,42061,31103,-10548,-62117,35773,-54037,11030,-4344,-10263,3263,-67824,23062,-23853,60580,23712,-27850,67381,36066,-16478,741,33281,15289,24357,-62550,53674,-29148,-38581,8833,-22364,51921,-38907,-34224,13256,-1015,-15927,13711,-16064,-36381,-88336,5559,-2107,-38242,-19229,-14651,25109,25950,36387,1857,55492,-97065,-85721,-12503,52267,-47046,32229,-2516,-68398,26786,45766,7987,-21230,19885,-82197,39144,20837,-6113,29544,-41776,42607,-38710,-16513,-68049,54175,-15586,-16878,4203,-44928,6062,-123648,-32090,-39811,-103172,40847,-34420,-56525,-27249,-93949,-31172,-81397,17111,35564,23325,-29573,15954,52461,34660,10222,-18397,-40300,956,-15066,-39301,3643,42798,
    -13801,29936,-2094,5075,47678,-42111,-21113,3089,-9030,10885,33445,74472,-7228,-13493,-65680,-14470,27187,51616,23910,-40601,3931,-10157,50003,45891,-29114,-46002,13463,37373,-14722,-15815,20493,-49229,-108804,-27090,-58573,30564,-5750,54773,-14895,27057,2228,51220,1406,-4258,-15612,-30683,-829,-16360,8676,-71236,52461,-72727,42350,-16004,-104863,21732,-37154,12750,16463,6058,-28507,41824,34544,-13601,-37362,80775,-5795,56963,25487,46509,-19298,-50423,-11894,-41807,24457,-11437,-86967,48204,-40405,46874,-27585,49739,1894,-14454,15435,27353,42131,-67420,64902,-18081,-5696,-12962,-68850,-20400,-28703,-33234,-2900,19208,-37374,41505,42302,-63195,57835,-60358,-39898,26868,11665,35910,-517,4536,-46580,-56296,7198,25080,-35788,5106,-57429,-55386,-64612,-37886,-6454,-3644,-4885,-55320,62720,27817,-38415,29526,
    27024,26404,27803,-53757,5594,-23757,-3686,-12566,-40550,-40950,-76719,6043,7681,-51338,-33865,10149,25972,50803,-71656,-63261,22000,-8404,13982,31231,-9328,-61759,22154,88474,48903,17788,-24880,-13537,-29141,37080,-4004,32550,789,-17511,121457,-65136,-15920,17126,-47943,-75424,31994,-26301,-60519,866,-44296,25265,-56236,-16394,-31733,186,-9339,-1069,24182,-46810,23371,-5679,-24729,-70179,-29410,-17130,-5888,28110,-8688,56331,30216,-55657,-18442,-41014,-32971,-12373,37393,-63155,-17256,-49570,-35766,28837,-49816,-5962,-108148,-78800,23725,41149,-8297,-59852,14245,-27171,-40442,81804,-22906,21296,-42711,4797,15984,-19612,872,-3916,-11462,2680,-29249,12189,70506,-3102,-43000,-88098,-29737,19139,-47868,10878,18466,43243,52299,8188,-27521,-48250,-3220,458,-49367,9317,-21998,-48321,-5265,6776,74244,-12529,
    16656,16472,24292,26720,-24297,12321,-10302,-42480,-49555,34766,-74240,28487,31514,28651,-1599,-35718,12791,-21479,27232,60750,13157,20833,1643,69648,-3987,27485,-59466,14414,19893,-36900,-5233,-83735,30997,-19125,-18648,10918,3601,17300,-16376,16962,51218,2132,-28295,-32317,-48771,-15281,-55891,21848,-7749,-30477,16258,-8886,5996,6547,35578,-10181,-26251,8600,-44058,-80637,-19575,-35156,55476,-92246,-968,5851,-10473,-6308,-44585,-56651,-51485,-8651,21580,22157,14836,-38691,-62362,35116,-77210,2671,60499,-49142,-30657,-78757,-10430,2792,49627,-25800,48267,-35126,-25373,-4284,-14043,31118,55310,-38201,17121,-47311,-13388,12228,77052,22442,-5237,2013,18061,-8285,-127723,-27850,-36482,-13597,-10372,37866,11185,-52239,41188,-16707,-4571,4183,1602,36242,3075,-58867,-65511,24124,-15952,18340,35455,-7019,
    28886,-68301,-2129,-3379,-23693,79374,-10411,7904,-81380,-69533,-8929,5364,23136,-10986,25339,-110180,-17846,26994,-18966,-29752,54492,-19564,-9689,36201,10265,37747,-30598,-23263,5963,-42658,-64363,-18560,-29037,-9538,9972,-47858,11829,59398,-6413,-15151,-30322,-70531,25468,-48483,-49101,-2951,21175,33267,-68762,45143,54836,-34626,22358,49598,23606,66252,-25316,-28414,-64060,7104,20129,-24224,-15942,-2497,-20710,14302,62612,78507,34510,27148,-7139,-20666,-3541,-29481,38641,-39337,21965,188,40392,29365,30150,-40065,-39466,-9600,6580,-46505,-15004,-6359,-21202,50463,11538,54164,-32636,-20275,-35762,3981,29650,-37978,-38804,17100,30632,23445,-2847,-14052,-17278,-82340,-2137,2028,84520,-19374,-17695,-35158,-13800,44551,-23879,52406,-45735,12633,-9130,28737,-3065,-22322,10691,-14942,-33669,1600,2269,-12608,
    -45569,-53281,19434,-52282,37252,-13590,41830,-39652,-67499,30944,-43678,39004,-80759,-63214,-9452,19206,15765,15124,53142,-11056,-51972,14475,-8410,69534,9157,-44580,26257,-10968,906,-43150,-21434,64560,-54657,-35344,50776,-25916,44870,-1141,39309,-121253,33282,-12032,-12154,50994,-9861,57774,5830,-42902,26316,20518,-9376,-14917,-23873,52606,-18734,-43108,69185,26646,14636,19169,30961,5793,-55519,-66921,32485,42780,-18495,46994,9571,5421,74011,-72439,64042,-18187,-57005,5995,2288,-87716,57648,76223,40311,-16993,9435,-56365,-17384,-3930,-38175,-49134,32299,-16773,88304,24476,-36736,41007,-17378,-15953,15951,-43187,11307,234,8030,-25931,-11477,-312,39726,-49879,-54264,-14900,-3935,-32089,-25114,93145,-25248,-12673,110119,69643,4590,22591,-2746,-9757,-45045,-27204,60972,39007,9762,-5161,9698,18605,
    10596,17385,11763,-44144,-7254,-1010,-31270,-39420,19566,-3394,88435,-18914,-13432,-49072,18566,-6432,-12640,-73772,-13685,-7878,-32102,2179,-29823,-71307,9927,66420,-11016,25836,95363,-10165,-83119,84214,-78997,-4589,-38728,-17898,-51259,-2484,-43160,28695,26158,10767,52934,12635,-339,20740,9087,33875,7143,-25258,8267,-8718,42798,10450,-27671,69064,-26027,-11352,-4869,104268,-84061,39420,-34365,34228,30627,34248,60424,52594,-15995,-45986,11887,-35605,20280,84404,67147,-64648,-50079,3592,19532,-8925,-64464,-31141,40960,-22546,-89694,-98496,-15466,-68127,-72,40274,44949,57738,-18339,39068,-16193,8626,-9084,49884,-664,11097,33403,-22004,-66347,33970,-59515,-23781,13792,30466,19412,-57465,-4410,11912,-34847,9941,7826,26818,-49856,58342,2564,-112159,3236,-26915,18537,-76815,-11382,-28368,-27970,19059,
    -35147,31055,33651,126618,12550,13270,-22470,41053,-11467,72838,37813,-71625,11573,796,65627,-45244,47549,-26547,55037,-27114,21109,-62401,6850,366,-107987,-15760,37737,19448,-97182,-4356,13026,33331,17209,51144,4690,13541,-28692,-18029,-40342,42719,-32839,-56712,35762,59611,2040,-40832,12269,67375,34175,-40526,-12088,35829,-19786,52961,31292,-1966,-33386,-6276,4873,-1142,-23934,-35605,31435,50607,10493,24757,-30487,9309,47034,-79602,-3701,91031,52828,3894,61387,48311,40789,12512,19299,-40359,-19961,-49762,34379,33169,-39321,41161,-27599,64969,34741,14588,17796,36070,17572,-25275,38999,-11228,12404,34129,-52296,47452,-20774,-8869,25575,59421,30142,32633,4699,-56733,-22042,-11016,-12412,15472,-21665,-94270,53286,-24530,-5133,3734,37256,-57301,-1426,27745,-44670,17026,-13510,63710,-22535,11147,
    11234,12791,72141,-762,-28988,-38111,-86433,-2772,9543,-721,-12118,-24963,-16066,39548,39767,-76477,-437,1547,10442,25657,-10544,-79725,-32281,20752,-17325,41950,-31094,18506,9999,-52792,26741,-62991,-101129,-22517,-48326,-192,-31056,66806,-61725,2525,-1223,-9036,1655,-25907,-3417,37224,68769,-29478,66953,-33570,13615,7355,9608,4566,9752,-15020,-12170,17899,-44789,8228,-61818,-19076,29827,45983,-44110,-44567,73511,10553,27254,-36468,-47203,39576,-8349,6322,-47835,-49466,66757,-13053,-38168,11710,-3835,-53566,-69946,6915,95224,-30692,28427,74576,76508,-52882,2427,23920,-68884,-9336,-34015,19214,32296,77430,-39570,-3378,92440,80728,6764,17965,-2209,-1595,15418,3823,-7451,32524,-13836,33873,-25049,-29955,-14735,-28845,-52348,-14722,32572,-50794,-14917,-32754,1887,-22048,-19926,17786,5168,-17703,
    18318,6022,-22071,-16888,-107326,-108181,-19495,35496,26985,-7827,28224,-4562,12900,104895,14456,54396,61074,26813,14446,-52022,-56190,-26286,19115,-72288,8279,12651,-15325,29484,-17729,-30593,-26878,8238,41097,-33378,-8747,-71200,-106960,-81436,-41371,27635,35431,-63180,55206,19897,23822,28167,58623,-35847,-37144,34337,3,64969,67054,22007,36213,-38826,80916,-4586,80973,4402,-66832,60946,9559,40310,22806,-95055,54356,7967,-52326,-69561,-61825,77344,-18789,-44985,-45762,42027,34371,44049,-42,1570,57624,-2270,-3008,640,-30014,-60899,-10257,-5792,25485,8430,108974,71983,31910,-31533,-49558,13385,-96014,25809,10701,-51567,-17325,65815,28378,28810,4678,36039,18500,31829,27329,-61158,43860,-57415,-2999,-3834,-55399,-30676,-5290,-27823,8881,-12025,51380,-3938,-1658,44953,26504,7600,-13510,55415,
    51249,1079,-56744,-33706,-19308,-33436,-12899,-74742,-7335,19703,34748,39569,1956,22541,30959,31215,44269,-38087,-49451,-51791,-42444,-32396,-20831,40002,9123,28420,-5521,73923,-43755,25683,-25291,-27011,-46719,37142,-95311,47231,-100437,95264,29370,25931,119070,35608,12219,92147,-1160,28789,-16785,-22943,6035,4934,401,-1922,-2923,-50554,-15970,4415,45745,657,42639,-38069,-3631,-7183,11020,51234,-34952,-2854,53970,15845,-9284,105325,37091,47921,-5576,37739,6359,-15654,-71903,33838,9002,-28803,54539,77774,-33036,70083,-37955,51373,51256,-41875,39286,-2414,8655,-55716,48385,71230,-50281,-52952,-45246,-35150,15078,51186,-3566,-33289,-33909,-13821,-34677,30438,-15833,13641,52417,-3914,-16118,-2559,35163,-8225,-27115,-2675,35515,-3201,43333,19812,-9801,-2309,50068,-7209,-26317,-26319,-58560,5677,
    6046,-13578,-24407,16685,27720,10467,-39983,-17415,-42780,-36297,-29197,-17705,44909,35621,-600,28147,-12252,29302,-5822,11020,3966,31761,821,-48400,28546,35153,-14965,-7741,30261,-19699,9869,-47620,31613,11892,-11623,-21644,-33352,-8053,-41969,-60484,3237,16460,1282,-41823,-9027,61958,-9183,2744,-48309,-65742,13907,71281,32918,-26387,6388,-39267,-62928,32537,-61329,25995,-32667,78052,-10949,-22479,-38253,12387,69185,27269,22010,-25672,35504,-14704,168,-37224,11097,38894,16016,-46048,-82476,-13909,81509,-43087,-40251,-1962,31170,20337,-6452,53646,-7014,66135,32059,-35370,-65740,19192,34328,14683,41965,-7743,41395,2819,42754,54138,17523,-47378,-51761,-2202,10445,-157,-13711,-35582,-7233,3362,-6809,3087,36275,-11918,28927,-26462,53426,2719,46301,48318,9458,86930,-34930,5724,36200,21118,
    -12053,61342,9957,28282,-28698,-45188,57832,-23612,66449,-20907,-316,-17983,7831,-39761,56318,52118,-3725,-29605,25841,32984,-22291,-26330,-12962,-28506,-18520,-1219,17638,46905,26324,-34197,-89665,43806,-83174,31516,-54733,82186,-16077,36219,36115,23263,27930,-30323,-1674,53657,8537,-78360,-23387,-78204,1737,-6622,-16663,-51463,4440,10872,43587,-29236,60362,64283,-91974,-43777,-64647,-65515,-55949,84603,43045,35285,-44748,-38774,15980,-38089,-51635,46079,-9919,-26703,26064,-48911,-20134,16868,17377,-2918,-77961,-15910,27317,22107,39288,-51156,52809,25311,14877,-1764,-44237,41388,12022,37027,-72104,-40798,44430,-27781,-5722,10464,94032,-94344,21010,60837,45253,-28524,45750,-41670,-17689,1244,-70954,-47000,44280,-30318,27132,-39108,-82343,9474,4731,-48673,-45210,-32686,5177,-7409,-63538,28931,59669,-1865,
    17010,-65704,20160,-29683,-76041,8601,-70140,55845,30318,-43930,66850,17130,-24697,7751,-27982,-1364,-3633,63341,-63449,-22100,-13808,-16643,-13972,-3092,51551,35444,39960,23147,-39576,-14798,-51726,-1097,-21762,-68927,-36182,5308,35773,2328,-56615,24851,18399,80234,-79944,-8390,52770,-14324,29765,-71839,-5247,84383,-9928,-1966,10828,14694,-3165,19990,2060,-34696,9590,-22317,-20017,-61768,-14408,76396,23092,-3865,49017,5631,-22517,2177,-46269,8774,-31467,-18354,-79212,825,-41036,-29108,53046,115266,-6961,44152,-23912,17394,73843,-58216,-14969,-66614,14300,-32030,-10773,-99136,98967,19036,-57650,13303,4796,-65959,4146,-36726,-35425,-68501,-12431,-10921,-39758,-19229,46841,41578,-12055,3728,-24942,-16828,517,18944,-90733,-31261,-40685,-69373,-55081,24819,32231,-11560,3982,-61049,46625,-45621,-52736,-20524,
    -5926,59448,-13263,46111,14989,-9327,12420,76318,-2979,-53320,-71879,67467,-4622,17727,-54774,-12733,10020,-117559,54473,-27082,-42868,141904,-20145,55399,90486,-54637,931,-32686,-2642,-2436,58413,-10882,-50216,18480,-6957,-44007,32651,24691,-3847,78266,22904,19570,-19581,-33570,-11078,-35834,37699,-30408,-35514,69911,35917,-32808,4456,-2450,48667,-35832,27612,105291,-14751,10307,30869,42338,-69356,-35777,-20566,-12104,-37932,-6153,-50542,17517,940,-57808,-62696,-35556,9509,-38949,-45972,12860,-22441,9713,-18355,37870,57231,433,86823,17917,26419,17124,-426,-21101,-23228,-31757,2960,12221,-91799,-12547,44080,-56911,-38498,-70511,40003,-46157,45909,17475,94119,-14066,-10867,-3408,75739,68448,-30946,-35504,38176,10319,-12164,-56430,-17194,-100071,-88808,-54675,34416,-67681,31455,-52353,-31367,-26152,51325,-66819,
    11420,-60129,-623,1525,25384,1105,9398,21581,43196,16522,3130,-26214,39785,-19504,-28315,4908,30780,10666,-95420,40712,35186,-104312,41728,25756,-44616,-17547,-19882,8265,61638,-14788,-12899,-57007,27872,-10475,-43117,70113,-25262,39003,55762,79099,-40869,21874,-77790,10880,-5838,-4496,5244,-26580,-3628,31568,-30251,-5497,-12564,8743,18870,-67577,-50802,-44024,3860,-71951,-54179,-5425,22001,6431,-8769,-12926,49256,-9942,45010,57599,-39913,24540,-4921,29435,-2532,-2904,-43365,40398,12745,-66933,51649,-37168,3513,69237,-39017,-13838,50963,-2619,46520,41133,-16863,-39360,57679,58693,44108,6765,27879,-14440,28736,111,-42866,-64548,-92844,22781,-19922,17111,38172,-797,-1106,-8722,-38856,22030,-46070,34471,9105,16642,18485,5909,-11343,32527,-9597,55796,-39948,19463,-11181,-29523,2272,-14924,
    4633,-12364,-33076,-11375,-269,-6553,-4702,-48767,-47291,14763,-7518,-31157,-69114,29468,-30562,46505,30793,5389,-97777,-23468,-3487,21869,19898,-41764,-21583,-105433,83846,-32055,-31091,11952,27921,39425,37458,3355,88618,71858,-408,81238,64106,-39110,-40618,-41725,8335,24740,50934,-32836,-55652,-18906,-19111,-1523,32597,51814,-16601,-131577,-100027,8250,-22060,-45627,-12793,25052,-44054,-37401,56879,-57873,-36965,11796,56388,-26752,3563,18783,-4770,-16215,-23560,26537,56674,23787,40292,-23891,-27993,-42784,-2977,12308,-13861,65762,14039,31632,-20066,-1875,13633,-7973,-31089,-38992,20762,15136,-14603,23807,-220,47981,23551,35269,-15561,11802,-78815,-35964,-21763,54896,13177,24282,-27727,5960,-15984,52837,-12642,13200,39005,29910,69555,6177,37252,29016,-19894,-5525,91605,-8614,34935,-89694,-23700,-23890,
    29032,-20537,-28894,-51953,-91984,-14522,-23812,-13244,-9630,16022,-33172,-28041,45699,24722,-39663,-40444,-5004,464,3860,-31062,624,-67378,44764,-50273,23193,31087,-32120,-108441,-4098,25035,1334,-36493,38160,10460,-9562,-11731,18608,36428,-88085,60599,-33599,63116,-42921,-28754,55677,24119,31102,1439,-2948,119258,13720,31363,6966,-15468,-55196,-22491,-24709,-8458,13172,-16867,21146,-13448,-26839,-41619,-27071,-75822,64708,14671,-13232,-25685,-17092,61397,10528,30471,-62134,-12307,-18477,124031,85040,-16647,-74858,-12711,8510,50990,-44619,51297,-31294,-75728,-58963,-26789,26246,23701,72970,-43843,9956,-6772,63803,-80929,13656,-28361,52611,-22386,-58711,-28061,-63645,-79488,37544,7910,-42552,39677,-4990,-12962,10558,-4689,-12156,-67999,57917,15566,64357,58076,31616,36208,-58724,-33528,-6721,-55709,79267,-3674,
    8181,-60905,-32611,-44453,14370,47260,8845,-13538,-5203,30694,-6549,46451,6173,-40211,33532,-64768,68294,-15565,28797,89053,42908,-4670,-16045,-1610,2573,29628,-30575,-30711,83556,2085,37613,-47851,10367,-44821,14696,-12686,-27537,31627,-59370,57933,41763,94340,83172,-29451,-4556,61290,157,-14920,46860,-31550,2681,22678,54948,72145,-43584,12478,-36620,-128575,14050,9984,-23534,-8503,-9977,-46525,2354,35925,-36499,95108,-12608,19264,-46780,20451,36461,13943,-58475,-54584,-44994,-8039,22726,-64873,-102711,-25620,29857,-5548,-31716,12700,-44864,-19219,10635,44017,27586,1462,87018,-66381,-36119,-13150,-62643,-41638,16968,-63516,-10034,14538,2849,19207,-10935,25121,60502,13333,-1231,-20773,-19292,65704,2275,-38653,-54471,37571,11586,15185,-21203,-7648,84186,-38525,-3528,20628,37536,-22716,-55271,33147,
    -19313,-15365,584,-4158,2103,-20238,-22408,-15368,22809,30840,70538,-13876,67821,-24402,21844,28623,-15906,3088,59291,39661,-14484,-10676,-12702,9404,31990,59546,-1506,-80117,-9577,28639,-27425,-44660,5309,-49853,-18606,45177,21702,95968,-9329,30763,-32637,24117,1906,-15813,43201,-60735,26088,16215,30437,-30649,31571,22761,-49219,-64871,-41048,27948,-45624,-57499,27872,-7322,-97884,31770,-21521,34346,-52786,34915,-49954,23587,9798,18583,-16018,27503,4548,-40767,-3193,-7393,-1638,-39044,21102,19112,-10105,24565,-49454,78051,6855,-28474,19322,-11827,-14713,-42382,-21689,-8567,8075,-37704,6824,-17375,7147,15207,-34695,864,59312,-8486,-45218,-19593,12564,187,36518,-48377,-74880,9434,31722,-1341,37184,-24074,8305,-30604,39161,2897,35804,-54947,-39793,47591,-4484,-66284,50447,39135,-46524,-58791,
    -43826,-38943,-32068,-2009,-15587,45414,-80597,38079,-22769,49202,-7213,20951,11165,58333,-6077,-13965,-12455,33202,42567,-4500,8977,-3333,32942,19198,11430,50478,-40509,18186,8866,-4818,362,-13821,-36011,-10381,13037,6025,54832,-46060,32316,-41838,-42175,5789,-2799,26333,-61705,30719,-7242,-18125,-80533,7819,3563,-67829,-27400,19314,-63636,-39796,34512,-73723,3672,-23425,-6154,12897,-9809,28755,31368,42666,-7325,-22520,19657,58897,80877,-3614,25790,15975,7717,62642,1398,-26197,-17918,87803,57761,-30531,-46789,1727,50258,13721,36449,-20369,21651,-7047,-65350,-39216,-30805,52434,67502,-34850,-9961,46278,-12110,-10253,-31156,-28968,37860,29028,-69405,-4663,-49514,40150,-37712,14567,39015,15650,-2880,-68181,-27022,-11537,-28072,-7314,-26217,38167,40108,9259,35188,47743,49971,43992,-42182,-30129,
    55883,-7096,-85028,-10059,11339,9133,-59430,-88013,-10325,-49145,-31609,-17061,-49472,71723,59747,-42406,40054,35625,-30776,-89554,70887,-68683,13387,-77111,-10836,-47507,-78427,37496,4464,-47891,31588,-26952,-42914,15726,-40136,-32033,30677,-76502,116119,3892,18715,-41836,-7891,69449,-41149,-3774,26300,-17081,-47621,-18574,-12250,-744,-11150,6391,-24460,-47852,-10626,-6404,30019,-4302,11156,-69173,43239,48649,-115937,-13853,46013,-77474,36306,-2438,41841,38358,1906,79726,-29460,-14427,-6507,66594,29694,67218,12711,67471,-26138,-7744,33649,-9185,72295,-13599,7748,65078,16228,51821,982,82518,-150736,6482,2086,-30263,-63128,43968,6112,-22830,54370,28072,-49640,8809,25086,33134,86541,-8457,27317,-121459,-14276,-12036,17791,-23916,6064,-13592,-9622,-41022,-15846,8885,22071,-60755,11,-16639,-59187,-58739,
    -60444,9900,-24812,-85856,27638,-1060,8943,-32923,-28214,91696,-8832,-5437,-19413,8061,-82583,53918,-24121,61610,70195,14827,-54791,24132,-4675,26483,25844,-28217,-26080,-45800,91312,12406,371,-27169,103,84425,-11195,25975,67297,-29044,75687,1304,-32494,-4051,34677,-526,21644,-33926,30707,28262,-41770,-82356,-52926,8667,50578,-19628,-90017,-6507,-53396,50149,70020,-44263,-44648,69592,60674,61773,-60205,28430,13942,31394,73926,92035,31311,13274,-9220,3248,-22442,-36648,-41024,-38291,33529,10639,-17734,38676,-31960,-21082,14757,42523,31144,-25779,-40039,89783,-30160,-6391,-133947,-10837,3842,49382,-14285,59196,-28803,23622,37103,-27343,15548,-28445,12419,14,-10529,-22722,-22873,-6030,21003,-17596,30241,18171,28851,-20042,34348,55095,-48325,-8725,-2992,37728,39088,-49172,-17044,-38419,-68689,50170,
    39686,23882,-41482,48325,37592,-35145,-29321,44039,47084,-80754,-27540,-53508,-13060,46969,-1727,-14216,1978,45354,-55007,64318,-9090,-15151,-36229,-76917,-46005,-8673,-62985,59473,-39757,8164,25169,-44692,-28921,66417,2202,-28129,11369,-6531,-42589,68310,-16531,20269,-63030,59552,-54431,27684,60113,-1214,-37201,-55639,65151,-12085,-10744,55576,-76022,23905,27862,-120,-3751,40230,42970,20705,-59630,-43844,16212,27770,64849,-25060,1971,-59083,93693,-45160,18516,69050,60646,-36170,-28312,63979,2614,10626,21980,-18361,29518,40722,-42164,-33948,-22470,-11969,31009,20346,30207,23972,36205,77343,-84659,70242,21266,-47786,-18358,25355,61436,-41914,-4594,-25756,56476,13108,34097,50068,-52067,-9941,98270,9160,-31835,36045,56876,-37049,57546,-35938,73958,-21426,-43594,-25909,14077,22463,34856,16747,-4612,15606,
    1930,-11521,-65534,62422,43779,-24800,57611,-4057,-19467,12853,6705,-19202,-2278,-4153,-4725,-24837,58823,46151,72965,-68698,22778,-61896,64032,34129,-69949,21691,72227,-57996,-18091,8023,11931,-9184,65380,-61410,41144,-19368,57364,24157,19741,53561,-56082,-27473,-4238,30325,-53780,-487,19712,-6479,-32961,1345,1785,-59049,48936,13898,-59911,50923,-14895,8543,29784,-65192,-2222,40636,52214,29530,-13815,29563,37114,-24761,78364,-23739,34128,43290,-50733,38506,-5116,-59277,-19417,30694,42099,-57832,-12540,-13123,29102,25544,17827,4534,31921,15418,-16904,40096,-123724,25121,-12722,-24990,-2818,-20381,40419,-17906,-39132,-44010,-32001,-30972,76246,-95812,6807,-28687,25244,-28302,17039,32781,-13564,-32634,-15969,-29203,28839,-17355,19541,56693,9717,67185,-60508,41642,-59640,-1756,71083,46092,-11473,-44766,
    19775,57241,49867,3442,-2804,-39,-19934,-28642,23509,-30766,30099,-1153,41468,60432,58538,-48577,-19203,48968,4613,4565,11056,-3855,-23599,30234,36782,-32594,-34198,-72478,79447,-24794,-47745,-20197,16945,10030,22246,-70491,-7480,95970,20746,-288,-17404,-52750,31047,-32781,-32340,-25978,12462,17286,-17770,-75569,8923,29677,4988,-10076,1092,15792,31524,37035,-36623,48855,-102192,-3200,32080,11152,-31298,61126,-1258,-43112,92266,1655,-78066,-25478,-51970,-3851,54642,16394,-11901,30219,34616,-24991,-8939,-73866,-42250,-1206,7598,-23710,35486,26280,-49505,-39664,14453,49293,-13012,1708,5271,12573,15745,63244,-59397,9992,38821,37911,77228,-10926,45223,-99839,-27951,-40386,56244,-22348,13215,16249,36228,4604,61754,1793,-65642,-980,35092,-13306,-60571,-66205,21267,-11896,6415,3483,-72484,21321,
    69779,-6372,22703,6569,-29010,-28868,18418,9201,-39914,-28388,33941,-14933,-86119,44414,17720,66131,28258,36059,-83387,-69719,-63131,56187,-71768,-29726,29940,-35703,22170,38186,46494,-44580,-3079,91261,28254,46666,73607,-66815,-12553,-14925,60487,-3811,87073,-26005,-41552,14391,-26072,74123,17399,-45530,10980,-12332,-24110,37370,-22364,12751,22449,-24746,-31059,101358,-83840,-8772,34932,-14363,10035,18113,56624,-115505,23960,-19652,-29902,-42424,42108,-2579,-25740,-49756,-32639,5605,-3814,-27665,41382,24640,-31740,-17078,-27929,-31789,18398,-89197,66608,-7141,-29103,-4301,70993,-9060,42935,-5944,-70456,29844,-76846,-56658,10245,31785,47572,95443,-8835,-44876,10986,-35618,-91314,50636,-1308,-3925,-67139,43787,27143,23726,49560,38014,-62502,29365,44552,19525,38215,-93361,-50343,41861,-61455,-25681,-21616,-3752,
    1387,-24492,-56768,18136,53228,32793,35892,-47495,-31080,-41603,61400,15494,-43817,-32281,-50643,7491,-60007,-23029,-3754,50097,30781,59300,37244,-15196,16531,-42849,87203,12073,-43741,26008,-6076,24865,16024,-20815,-30936,-8257,-49211,-30192,-38514,-46765,1052,7061,41278,9897,-22357,-84006,-72611,-28208,39733,-81866,83994,3200,31827,-19936,2729,7928,-44828,-4288,64770,-24944,66997,6833,-22356,-76828,-13685,24676,29210,8426,-42285,-60289,13108,-1561,12837,-116893,71275,-57029,-44,-75652,-14060,-17228,13152,50927,114463,-25011,-8158,88173,-32913,11073,-10104,-23569,-43602,-51882,-35398,7755,-52782,-33522,-49381,-16334,13919,44010,63378,-60024,38931,11246,54555,59369,21463,61028,34344,-2769,-10239,1269,55461,-11757,-56554,53284,-9169,48327,-30439,32678,17965,84131,-11289,100230,49571,60987,30722,115642,
    19468,15365,13769,-20429,-1886,12216,53049,-8695,-26002,52497,-26365,-27895,32041,-27635,59760,32686,-31384,-44882,33113,-30730,31568,-68048,22499,-20903,37261,-75742,47751,-72201,-17804,-45850,20384,24146,-100,-2814,-396,525,48450,39893,23567,-51268,-39728,20048,-18891,-11800,40,-533,-33206,48433,-26199,-117982,41592,29555,-4730,30614,52257,-30130,-31600,52764,15858,-19604,2175,4814,62627,55113,-70696,30285,19704,-35717,66369,11175,4723,13011,9891,40237,-11870,44603,48384,-32297,15905,-35316,644,-12356,44203,52756,53364,17894,-35921,-17980,-77215,1584,-18410,-32096,21773,3805,9946,2720,59352,-36707,-42861,3650,17147,24851,-48018,31834,79038,-53586,-27212,-24924,23871,-5630,3021,-454,-30962,-27316,13570,56070,-17510,-45312,-7572,3439,-16057,38681,105286,-28367,20360,38572,9430,-41635,
    13959,70767,-319,-4322,-80661,32662,-10997,24838,57546,21879,-7813,-71131,-28525,12846,-36657,38953,-56698,5196,67739,19655,-64958,18319,-78899,-44949,18236,-20175,-63095,28621,-47090,9650,-18935,11378,-38889,-57836,74787,10667,7879,37201,-86371,-42670,33940,24283,-60863,-93242,-597,3444,-53168,-19363,16864,-54319,2137,14164,-37489,22969,-53337,-35794,-12144,-27168,-26348,27051,-8271,18023,-28225,3739,59956,52711,-34076,13126,-15584,-71169,13989,3016,122082,35482,85131,10010,824,-65969,-27903,20862,20943,-36546,45706,32741,-45479,31202,-62927,40270,-42000,12085,28851,46315,41033,5553,93172,79293,-35741,59492,3648,27599,-43189,25914,22055,37767,35471,27348,-28506,9582,-59146,-31108,119834,33781,-17334,18036,-6227,10074,59919,-10063,62258,-46766,-45465,-17412,122921,-34100,-42908,-25229,43236,-8412,
    63785,-8026,-32442,16686,18991,8849,-10899,65735,24767,-50676,-1504,-8761,-23828,67472,-649,-43279,-9682,-16584,51889,-60400,6694,58405,44160,1415,-65060,67223,13686,62554,10117,-8892,-17865,-2382,-19821,72491,-17932,-32724,-34734,68342,-9365,12399,33975,-90040,60048,26935,-74461,1619,17051,-45240,26187,-12813,-74101,-31572,12694,78502,-103249,36215,-67520,5254,-77805,49984,17748,-87097,-77695,8770,39709,7910,-8366,6084,66909,16527,52273,52673,-31473,2889,25913,39648,43085,-42127,17248,-1928,31811,-5341,78924,36817,-16590,34029,-1582,42644,-42767,56686,-39137,-49984,71653,15407,-827,-26832,84059,-41957,28307,69856,-25204,9973,-63840,66225,33382,-36186,-29164,-30507,18862,10057,-87277,135,-57741,94737,61847,861,-138080,-40049,-36010,89361,-44359,-11132,25829,-22426,-91579,-72261,10876,-16690,
    10647,-19931,52617,-31202,12750,-42560,4131,-6258,69909,36393,22073,-11188,-25297,12082,-8666,-43131,-57227,26564,9299,20273,-4759,-34765,11679,35161,-3800,-55756,69114,-27423,24155,-29250,-58652,12049,-7662,-73460,61516,66346,32417,92982,10797,1410,-6258,55000,3718,-15175,768,-24964,-13440,46622,51167,-10043,70142,-46990,-3370,-27696,-69830,128003,5221,51327,-6811,-48352,-22960,29592,17179,-26896,7407,26906,-5768,8728,30903,12193,-17534,6345,30415,81671,-34161,-39907,-42872,63661,26228,-19159,11305,-7522,11059,-49581,-74212,5638,-88275,-69334,817,45857,-36565,-11853,4440,-33046,-449,60285,99133,51210,6568,46398,71519,-74577,-5860,-10326,-56714,-52403,-5916,39188,-22908,-96460,-20207,19782,-17479,31612,-58015,37307,3296,63185,105501,-6258,-29778,44981,-16525,-67731,-5414,64033,-45085,-31195,
    21428,2102,61792,22070,-20596,-49875,53413,20932,57476,14943,-27128,12305,-60032,-18281,10869,14463,-66770,59138,7987,31741,-27053,-11161,-12840,25082,27510,-45565,49983,-69092,-1449,60857,39554,-38594,23834,-92563,52198,-20619,75855,30649,44436,22255,-26049,2290,-48698,-33892,71674,-41295,59778,-9133,56832,1772,-35594,-17471,3609,-24344,52303,16926,39888,15685,31685,15126,16180,-30940,42686,4219,26092,64568,-71632,-43449,115476,-29773,-80745,-37344,-37160,45891,-530,46452,69937,-33873,-36832,94233,-97192,-307,-22405,54937,63374,-102068,-23039,-7670,4440,-37473,-8440,61981,63625,-5421,51709,77214,40253,-10217,-985,-45551,-43207,-44808,32332,-27705,53647,-31572,11086,-26167,15969,55101,-51276,6340,25044,31795,-31842,21359,-23383,-12822,-40491,-11408,19241,16467,34331,5655,52959,89700,25076,-34972,
    -5638,13774,70291,-48568,-2406,40383,-38024,48853,-16785,4200,16282,-61296,4573,-7850,-15997,30309,3484,-43976,-43216,27348,-20383,5980,-43253,-40062,10164,-12377,2940,25450,10412,-74447,-55781,5925,60303,24203,31429,-27517,-61543,-15149,-31790,-30533,-48287,-51867,-25654,24226,44821,-11077,-23091,25419,49055,38212,29955,67345,-2556,39457,4102,-93124,7526,51135,-67189,-5149,43936,-32593,-85709,-24688,48770,-53884,75285,-28049,-79019,-71989,32534,-14889,42582,38099,-7879,45249,-7304,-71083,-15458,-37795,3765,2199,25774,-13069,40750,11269,3626,-13725,-2888,17939,24792,17585,33958,57492,-7820,21741,3626,20857,-6720,20085,-41684,9064,-20534,-69735,2595,-43144,55189,42124,-35780,-20654,-7062,40599,-24932,-10166,2453,-15863,22170,75802,31846,7516,56286,-67781,-96344,-16800,-10068,-48288,-11981,-43109,
    18656,67609,3514,24651,27204,-17121,-33712,2757,15369,44404,22186,-48969,-75754,-8357,102588,7413,14406,63900,-293,-81544,-65852,-38265,31257,3512,26345,-95127,-9214,-56588,-50472,445,-5427,112779,-17828,-6102,-5339,-13566,-79029,-62311,56572,77356,37904,25349,83984,4416,90649,-30001,-26787,8626,14157,5275,-12511,33836,27611,-405,91979,13656,595,-15338,-20791,-17438,-16240,21181,45094,20263,12349,21908,22102,-27566,-3716,39033,-38083,-70211,3685,-92343,19433,-55034,-35224,35317,-23977,-43157,17692,-49467,59270,34092,-19265,13154,-34745,63107,26658,26844,11482,49545,-46671,-20788,32444,13114,45963,23317,46375,-19024,-20067,-46598,88916,-51555,-43014,39246,49955,25212,59873,9648,-76659,-40839,44932,-4823,23779,-65126,-65794,21889,7789,-31463,-43520,15034,29766,28716,30412,10532,30870,25584,
    58438,11317,3382,-21205,3300,8251,-67674,15819,5391,-6324,3540,-63455,15671,48201,36092,-35168,57508,31539,-41637,7963,18838,60646,-38747,-51903,20157,53070,-7307,-11399,8393,-9861,13180,-48039,-20809,-45202,22648,28210,29807,37102,-32973,-10785,12399,11355,-19546,-33219,-24301,35928,-25746,-33109,-13501,90166,-21767,-43443,13384,42688,-17145,-7849,-20261,-49587,-94516,32524,28219,-9498,-25247,-16963,-26389,1875,32176,28087,34589,95538,-19240,50905,48944,-621,13134,11455,-8317,-16430,-32123,37123,24553,-5942,33073,1105,7486,-44892,29716,33639,-36268,-15601,-37614,-34238,37495,-14742,52079,-23819,24623,-12752,21123,-12855,-57761,40702,19297,6315,-60818,-15599,-33277,40505,44785,-459,24067,27495,-58100,11976,-76272,-25743,-7260,-33576,13278,44277,18349,-46554,-45620,35140,-33648,-3379,-15769,-33244,
    -46582,-52452,-103712,-15529,-5814,23585,-5484,-50892,-20114,77135,6152,-66933,-38898,32740,-41928,-16063,-11817,-23250,72960,-10087,-8268,8964,-33349,-25190,-40854,-86630,-24785,3171,5368,7712,10619,-16666,43370,23413,26644,-4765,-59664,-32714,-27170,-46399,24188,16311,17744,20033,29895,35970,-39596,13616,-2572,-48440,-45480,-15970,3716,-14753,-74242,41542,-43751,-29068,45811,14656,750,-39319,40786,-89039,-16179,23666,31384,70549,-43821,-94262,28139,15289,39065,32692,43878,-47943,-45400,-31149,-6513,-17677,-28406,15306,32360,75456,-29691,-11989,18161,-34231,-40276,8605,23402,11292,28432,18251,4125,18801,-44523,-48104,-33145,-4703,30803,-22581,69527,-5531,-39,43487,5507,66330,-23940,4520,62770,-66555,-10971,-14261,52707,28704,22248,26534,13043,34791,-2555,64136,10559,-7298,30339,-17542,18855,-25168,
    -5893,-11483,-92073,-43178,31681,-51095,-22274,-18367,-19859,22724,-25998,19423,33669,16325,43923,4146,73545,-61391,-43740,-45983,-66235,33822,68129,-10372,-22149,-29295,-2094,38344,23028,23616,-42081,5538,35150,-39545,9334,-10776,54100,9125,-6553,-30254,-71147,-11126,52099,25346,-31288,11692,20743,-33058,66202,-29079,33540,32876,-10139,-40875,-1072,-597,8510,-53305,-23916,44984,-36597,-1221,-26217,16666,762,65601,18215,36920,-55144,30965,87785,-30005,-336,27536,-14952,36796,48107,-85750,43578,17996,-20878,2079,51762,40406,29774,-1908,28640,-30714,-31890,32841,43367,17496,23409,77094,11230,47560,-159,-86349,27879,-6218,-31059,-23930,162,-76387,30856,7513,20007,54387,-25278,27468,40798,4749,106829,-14273,-113,2725,33662,1409,-46233,32798,15182,-40705,-19533,9733,46354,5672,-21219,38673,
    -5652,-53788,62540,8687,4702,-12472,-1535,-732,-18320,-57150,-20519,60766,-32803,52831,-11325,332,-17544,23851,-7152,35522,23409,-35188,42873,63831,-29500,-46408,21248,37656,-7597,-49194,-47839,-8206,35976,-42447,9752,925,81543,11299,-43683,-2986,-51679,-1785,-52248,-4070,-97998,6727,-32900,-1226,37601,58577,-43319,-14735,-27674,81108,-6408,-28064,39599,-71129,-4193,-8498,7463,-5828,-14771,854,36917,13003,23138,-76514,-6078,8738,41357,27307,-43429,-37021,-43536,-11356,51152,-38286,44377,14876,2790,31502,-29816,-26752,67416,-9847,1856,-21605,8976,22214,-28195,-19138,-50896,6929,20306,-13511,1059,1410,23285,2284,-36724,30279,84441,-37316,28899,-47780,-31791,11934,35637,-5351,2053,-68488,-21286,-101385,-4864,-7695,-26084,66572,-36307,-36900,32858,-29686,-12366,-39219,51199,25916,-11170,71835,
    -45582,38260,5606,42713,9641,59072,-1252,28877,-26121,12974,11578,22684,50249,21579,17095,19341,14061,-14669,30085,-28261,32259,-30907,39330,70222,-3588,64089,-30637,-11090,32375,113,-13173,-46580,-36534,51155,-69725,15201,25726,-75655,520,80353,-32793,10934,83985,-61779,-15135,18642,-41708,101949,-17292,-13685,-59468,-29898,8756,42748,227,10943,-199,-3786,38668,16661,79512,59945,-92,-8723,1551,-16570,-12898,-41708,-28430,15796,26411,-1496,31762,-16001,32718,-7011,-9456,28425,-46027,-84783,24455,10275,72016,25809,60,65446,-66606,60162,52084,-2857,6670,-16872,-59404,50481,16550,2298,97858,22953,-15931,24844,-24203,6760,52631,7564,-31847,-10847,-18,21753,29055,-11786,-22604,-16019,-54882,-38097,11366,-75221,-23949,2551,-21941,9504,-15401,55950,-56616,-37130,15514,-12621,10421,15789,
    -34181,-35393,23419,-31033,53441,-37414,-48426,-88004,-63746,-40293,-9340,-2012,-9663,42285,21257,-19284,18833,28385,-32236,-99489,-56962,-10623,-5731,5583,64392,-30144,-5044,-26281,34673,4570,-19821,2110,-58977,1470,20905,53252,2172,26446,44192,4905,13313,-47441,19922,6115,45669,-45858,-49638,-41026,-123845,90970,50497,29365,-69492,59047,-23119,-27478,44936,13102,17710,-44328,-64494,44132,-21469,-13979,-73285,-12571,23299,50286,36801,51964,-10869,-8686,-29370,-66949,-59062,-15150,-10306,49812,18049,-23990,-16518,25409,-29080,-67586,33899,-40469,-32963,27683,-56799,38155,53299,-31235,-60610,24485,-71584,1141,22400,-30010,-6003,-20543,-44400,-31798,-12507,-6689,-6969,5859,-54212,-11234,79810,-11881,-35831,22760,54298,-17200,28010,12917,10411,-40928,-22483,-5388,-74240,-28970,58301,-121136,-108853,-32564,-20577,-32106,
    -4286,-36263,47744,-11781,7915,24925,-23711,-12794,-21293,-9676,64347,-63599,41582,15973,32951,-23779,24462,38501,-40531,21132,-14799,-57549,52428,-28773,-20027,24303,-30224,-20640,-17760,-9177,37982,-7544,62825,-30558,7954,1047,55013,2448,-10887,-51939,-26203,54975,-7599,-20764,-37175,19689,-15103,9765,5062,102356,-27258,-42362,61439,8031,-19385,61397,79840,-64693,-9465,48473,-53786,-17569,-9395,-29271,20842,-26813,-50781,-15764,-15305,-83534,-43888,-11526,75818,-5273,-60068,36061,-8428,867,-9857,43406,-31146,-1749,-38385,-20088,-50130,-18895,18531,-53174,47359,-81432,21313,49307,15533,-30926,23832,-46872,-81596,-18978,-41840,-21896,-9636,-19707,43584,-37466,-50503,-61336,-26103,46104,40358,-20870,-1365,-11,49467,-33989,-29333,15014,-26609,84134,14799,3185,19825,220,13825,29876,10733,10613,-48423,14816,
    18884,-47063,-1385,30697,3801,8852,-29308,39695,-9511,13685,26658,34872,11871,-32971,-18818,-38203,-108131,57271,-61888,14287,64054,24643,-36112,27756,46006,82908,14430,304,23497,-10111,-11016,3074,47689,-23834,10395,9026,19368,-10234,1329,-6157,-67828,-40201,12493,31841,9736,-53898,-12879,9345,-75178,-39150,78007,-18483,-39351,10140,15639,28305,-25753,-8480,-9366,-29478,-4109,-31268,-43230,15489,21900,1408,49160,19293,56996,5397,12027,-2813,3906,23765,55080,36586,-2130,39052,95396,-38175,-1724,2062,-20183,-9655,-103450,-32220,-15813,-60613,-6237,9490,15554,-16716,-18831,11227,20952,-27166,37651,-65784,-33797,-1168,-11976,-21368,4324,16086,-14435,-68783,-16423,-20216,-2550,260,-3159,-40158,-8625,12692,3777,-37271,-27785,-10301,-6283,-75584,-1350,-20278,-9366,-5463,45526,39093,5722,-21632,
    46507,-43132,57931,14507,14842,4487,45309,-47777,16838,40991,-35886,24668,-46468,-18200,20972,103100,-26550,-64253,-33220,48797,-49832,34504,-1858,-49502,39850,-27917,23912,106517,19148,-11867,53805,-4221,-4670,-2477,-24538,67024,53233,-22295,57564,-31541,111853,-33805,-23087,38664,-29564,-14632,-16357,-54433,49672,-56317,-25023,14626,-58,-55084,-11323,-23942,35670,41846,12608,-21707,17369,22962,-98519,61142,35993,17893,-48023,-31441,25148,-12096,8240,25310,-28723,-23140,34705,16401,45632,-80665,-42390,-19547,31390,1220,17502,5536,47845,42287,-16507,-701,21731,19719,-11712,44602,35503,63130,7981,-28541,16748,34150,56680,47985,-38157,-26475,2695,30884,71083,43658,-48804,44982,20263,-49658,-80028,40298,27758,10135,-55400,19980,27917,7852,-2806,-22625,24690,9580,2099,82678,-15937,49839,-16546,26243,
    -30786,14135,-62541,11818,14119,-7875,48553,51559,26928,56860,9570,-16837,-31196,103742,-4281,38138,10664,-808,29587,-64957,-18439,52635,-72294,-977,43872,21913,-26193,-56569,10268,-4753,15856,10441,-10008,-41610,-62842,28365,55841,1174,51878,71994,17681,59021,109205,55227,-22237,-35614,22017,34104,-36358,-19072,3606,-11274,-18302,-63532,65925,-33998,25469,7678,-53271,-33963,27616,61148,18062,48636,-21986,4737,-56774,-45807,-1694,-2634,-37422,97670,-15927,25301,69292,17151,-47758,35246,34496,-65308,71250,1362,41301,-37515,-47067,69453,21293,4815,-50281,33981,5538,-11059,-24699,-12393,13079,17112,-61296,94368,-16547,-10794,-55650,-9302,44542,-31998,-74372,-7372,-26258,-58096,29055,2916,64467,5967,51081,-1711,-39917,-55513,33523,-255,-34734,-31724,22650,11289,6664,5568,-31307,-18618,-79002,19850,
    36844,-16218,56456,-28577,13833,-26445,18589,-72730,50978,-119072,54680,39566,11233,28025,-7117,-81849,-129408,27850,-15505,-788,35796,-66307,-46382,-36936,46731,36650,-49015,10601,-14156,-31447,23265,-26262,7821,3801,-1039,11615,-19335,-34528,-30133,14351,17368,-26787,-53145,20957,25777,28461,-17905,28239,-38141,119301,-28943,-44160,4146,72958,49620,-63850,-73513,-41632,16161,-25701,-64991,-22363,27294,-54484,19094,19493,20182,24139,32638,-6739,-54432,-11012,28223,-52921,20602,-12918,9435,-17941,43182,45120,67511,32101,-101738,-32387,-104016,-73433,-35712,7143,82661,-18015,-646,-51847,1288,372,40304,-18769,30439,-28683,41343,16375,35710,-6476,48850,-16931,-9605,95913,-48025,23919,73025,4638,68150,37984,40872,26410,24718,-44374,-17096,-3732,-23156,16708,-95483,15155,58576,-37710,-27282,59796,-4484,-62051,
    -20363,-21723,19048,-52300,-19940,3182,38503,-35886,37245,-52742,36438,-16269,22301,-4582,-25759,21546,-75589,-31452,55589,11232,45138,-30386,24574,55446,-23532,47399,-6852,23470,-56841,2058,-10415,-1274,-56279,54127,5395,517,-9866,14506,-33317,-40836,-23207,-1077,-33515,7486,29397,-21696,-6067,33677,13111,35682,-4221,31951,-35622,-42006,-26133,8164,21225,-50068,-80426,25712,27640,1221,-5364,43128,12293,67397,9301,-62107,61198,-31293,-54314,17879,-35578,59163,-1954,16612,-31161,24965,-19117,-18605,-35772,15293,33324,-11848,25631,-13942,-6994,-58581,-41,46740,-1708,43619,-32564,2916,56627,6939,21006,129874,27836,-4008,-6767,20070,-19784,-6469,-12836,10960,27490,-27541,21688,47305,33807,39401,-98265,-37997,-11317,18233,-33533,-45422,-7596,34469,39265,-57735,10161,-32723,-62572,-67106,28063,4130,
    -50710,-67823,-56917,28361,-30613,-4529,-34449,18514,36801,18925,-45339,75663,25984,-18003,-75389,14112,31354,-35452,47111,66845,-11826,-24373,-14042,18205,49145,21574,-15474,32253,33531,-9231,-116809,-37306,-53032,-673,-29087,-72753,-30751,29484,-86126,73860,13149,44177,40513,-50765,9133,33826,7788,-18469,81842,-6312,14105,32035,-41545,49058,-24787,-56357,-44492,55027,42649,53560,-20753,-40901,-47575,21385,32604,-21324,-2981,51358,-72347,97382,-41819,-32225,25930,-7045,12377,24820,36803,-67848,-25998,84255,6172,-2164,13137,444,59608,-15046,-32032,24369,38841,23700,-19990,901,22343,17917,7284,18993,-35343,-38031,-13177,-22844,-14638,-57914,-6844,-309,-54779,31823,43056,83951,42239,-43375,-4237,7878,-4229,-18228,-65497,-61569,19957,5687,-479,-48392,79152,-69001,28882,-23368,-12596,-33474,11613,-13928,
    -59822,3416,-21515,34472,-25078,1946,56483,26300,8994,40664,59459,-24180,52385,-63303,34197,73128,-37646,37824,9000,6777,-21990,-115255,30643,12510,-87099,70235,10488,16679,-24922,70848,-58960,52161,31583,-15230,24998,-9655,13131,-59431,-23813,-26067,-36251,-17358,30531,-28767,-9258,-19829,11838,-2054,34389,-34879,4851,-15777,3938,18164,-1168,50482,69218,-29212,27846,-16685,-44633,65826,17116,96450,30130,-101749,-25913,19560,43127,-13535,-11309,10135,57084,46604,-7717,49767,21109,-28494,2694,47551,-21234,-18923,-15077,-45112,-11195,23528,6392,21021,13837,-72194,91798,41786,-43813,-1551,8536,14625,5674,-47690,44218,4916,-97530,55068,-102636,51336,-19316,-35567,-4947,-44759,-26987,-55978,16725,13090,-51570,-30288,-6362,25920,3730,-13485,30503,-15158,-30758,32057,9439,-35102,-2294,49049,-10622,-11228,
    -38642,71566,37289,-13092,17065,-15276,77974,46500,-8904,-49904,30489,-42799,5987,9444,25790,20163,22573,28669,16881,-34082,-20721,-63325,-11651,32462,-16537,65402,-34959,-39834,19478,30348,2972,3897,-94780,16260,11484,-70645,40556,-159370,-38454,9492,44526,-47926,-52795,-8548,38543,48727,-7563,-44461,-19216,65520,-41673,7375,3140,90748,21722,13766,48664,-18056,37073,31034,90203,34002,40069,6179,3242,-35090,6180,-15113,11935,37191,-37833,33934,1621,-54299,-80981,-63481,-8901,21444,6453,-44797,51022,70773,-44886,7553,63527,-14742,43468,1629,17849,10406,72216,69212,-94102,-11268,30658,21705,23740,7679,59979,35029,18543,63237,35152,-43833,89025,-24957,-37342,-40307,1171,-62728,-13547,-50334,-24485,41633,15472,2480,-4670,42839,60088,-15712,-53890,4074,82980,16168,2538,64679,58059,-5320,
    17880,-53004,-3050,31072,32489,3663,-40274,-32388,51150,-44089,27633,2855,-36005,32439,6671,-53511,-42737,21047,-55851,-33949,52123,-13367,55665,28428,33908,-20179,10411,-7784,16944,13651,-16100,40081,-21675,70523,-7405,-44630,-280,-73287,28417,14622,26619,16129,45691,-17159,74152,-19929,2542,52440,-15484,117483,-56594,-34287,23058,18928,-27447,-41083,17969,13771,4764,1079,6621,2847,-6246,-7780,44087,-13252,-22862,-4328,-2214,-10743,-46488,-43953,41156,3245,1820,-1096,10451,52729,32011,-40829,38620,-39877,-37793,-29425,-65659,-45647,-59245,45131,-59602,4590,36236,48150,-49609,36234,60343,-15134,60039,44198,58969,-21440,17439,-40312,57161,-42303,-88935,42919,-41719,-35666,76871,36950,3477,2058,35784,22003,29512,-11536,-31278,-25827,29526,-11600,-50408,18846,-9779,-1342,-46372,69227,50306,-18988,
    1467,-19363,5498,17920,-58376,-43569,39532,13117,28693,-38196,-48,-4057,32316,2346,-6459,-36056,73856,-52297,13992,993,5782,14079,1896,-27912,-20830,-10277,81436,-5792,-3111,-22637,4728,-30987,3793,-45392,24072,16634,-40657,29939,-22272,-32940,-33764,-75767,19646,25464,10078,-36075,23900,-1011,24405,31103,-36855,6705,-543,72470,68351,-52786,42315,23432,34641,31444,-35329,-56543,-26340,-47354,11081,-18517,8114,-19846,50870,-56776,-30070,60980,8309,-40334,-60484,47158,70739,-27912,39719,-71497,-8209,-40842,50822,-40500,-66660,-30529,5705,92087,64338,47165,-10436,49399,52412,-6560,15043,-8736,19456,61311,-10746,39155,28910,-27146,-9110,63556,-5778,-7924,-97017,-11405,11054,-19930,58450,5861,15926,-83346,-20380,24656,-11894,62617,16992,5263,11265,-10647,-24379,89160,-38546,33755,57441,31221,
    -13570,48118,1287,75349,-36643,-35327,9790,18614,41540,-9848,-41236,-6434,-33360,26812,-2087,16685,8466,-17915,70145,-28797,-12112,-24241,-40962,82509,-42512,7395,41303,49354,-65800,44893,20509,50869,-26474,1032,-14516,-62158,-18172,47048,18089,-8685,43467,-44954,-29312,21540,-24333,17861,10824,-17871,9649,58109,1084,-24755,-9255,27397,-22634,-35777,59319,33999,-40059,28651,30006,-12568,-34188,42454,34703,28795,-67892,-68075,-23294,17511,36716,55137,-64392,-64849,16138,-37293,34513,11179,-6438,-41595,-58730,-36510,-37027,44363,73755,26083,23886,32906,37053,-25988,-15247,-20792,-8693,7981,-68128,9942,-6572,44610,-23616,6017,-88,44199,26000,40192,-22962,-27541,-16709,3249,70806,7798,-19961,-21154,1750,-54937,-19082,-14980,-27414,-17401,20849,-10903,40805,-88841,-70647,41280,-32665,-70236,10207,-3682,
    19366,-34012,-16496,-53888,5119,8719,-44067,-23631,-45895,-27259,-17828,-31757,28498,40688,22694,14209,25398,-42472,29336,-35955,-16044,51456,107412,-54354,-62296,-75029,29621,66206,-51655,-68237,-26826,-1463,75514,22998,-22193,-42113,-68887,58173,-26583,-17451,25875,-69852,2989,-66948,-65658,35036,-5128,16332,-5175,-53205,9647,25865,-15761,-19732,-53809,-24530,57958,47516,-3112,13786,-42829,-41722,33916,-4827,-39675,29824,-48250,-6496,-54627,54848,4743,-65029,-16908,17414,36709,19051,48718,-52206,-45047,29144,28356,-16525,-8942,16067,27970,33012,39507,-26824,-8996,53504,-22601,-4491,36021,28085,-29470,9191,16388,-21021,-41572,26924,-41680,80050,4455,-78455,3618,781,-51764,64492,-14373,-11787,14213,17886,-57416,13116,25563,34511,11061,24339,24297,82682,-10400,-64983,-17283,3227,14701,-82788,-64532,-24168,
    -10300,-457,29044,-19107,21944,-31207,-101571,18867,36867,41489,-73246,-27831,-35210,47890,49482,14872,44794,21043,9587,-46378,-1000,48764,15209,65,16065,35201,19045,13771,55890,3823,25182,-43787,19419,-18602,70050,65811,-7686,18795,-31237,3308,10505,51112,-26552,-34686,76651,5298,3770,-32065,1291,-40723,18106,-18317,-51458,-27688,-32913,40878,-10598,-495,-27069,-12241,-48404,41,6023,25943,2065,-63882,-22285,51156,24459,-34412,-14372,73987,-21685,-19070,95531,-45706,-9163,10749,-7922,-23730,-33025,-23563,-32342,-3952,-50021,2823,31183,-53892,-10950,51273,26828,114315,41695,-47298,31091,-40716,-72285,48199,12135,-11216,12217,15181,-31126,-21188,30215,-9352,-30649,-27917,-46055,-28367,-43735,5025,-59325,-9649,-18338,-24426,-11441,-17908,38035,-57176,-1353,-74937,-46605,43930,17268,35172,-38563,-7609,
    -15733,-42385,-32435,-27123,-11924,56547,-4403,-12031,2644,61505,-41857,20980,29461,29605,-78244,-86290,-11062,830,-21516,-20588,18516,27536,-49091,54842,15549,-38676,8857,6172,27299,58085,-32556,5312,56006,-48374,-23993,29304,-12190,26413,49931,29880,42705,16415,35470,14875,-2377,32699,51593,-19120,-5728,20998,-21087,-33843,34664,-29976,-9545,89470,20995,-82773,55665,23458,9658,-2729,11925,26890,-6825,-462,-5541,31891,-21592,25169,-7553,11146,8704,-41183,-31979,-31760,21995,-45450,13060,-14140,2711,13436,-4345,-45069,60745,12663,13615,45848,19516,-3021,-113656,-47866,-36172,-62769,-7582,-66734,12888,4328,4105,-58898,-32847,-14386,52049,-28931,-29502,33006,21570,61788,32675,43674,46667,-9934,80446,-7117,43342,-8524,49122,20532,-2624,-7700,36285,7094,-34018,-65838,52664,-49059,-46500,-15120,
    -24045,-26677,-20252,-1744,21363,8110,42078,-46027,7349,-1614,-49249,-3833,3325,-41430,-28332,-37624,21794,462,65983,-3022,-10730,-12871,99246,-29679,-25294,-32926,26424,-32576,-19535,-13506,34625,-75663,-90786,-12836,2534,3867,65759,86217,25605,-21877,-26241,69097,34836,-30631,-51648,52452,19340,-35482,-15572,-1692,-50524,-39618,-18389,-29470,12035,39416,16824,17977,59447,-29913,40200,-16234,43140,-1731,-27498,24431,-26285,-1388,-18751,45946,21515,24143,43505,7243,61584,-52851,6961,71188,-31549,62775,-17763,61334,311,52662,7844,-6987,24885,28110,12563,21407,-63598,46976,4912,-13316,41038,31682,10271,-56932,31506,3742,20511,-16806,17323,13479,-43010,-39206,-14719,10550,-35027,-980,-18495,29277,-18147,-31076,-26435,-15309,33457,44180,35999,15786,-8215,94736,-28560,23618,14907,39219,-22777,17885};
}